(* ::Package:: *)

If[Not@ValueQ[$Copyright],$Copyright=False];
If[$Copyright,Print["stylepackage.m: Copyright by D.Volin. Free to use and modify, but should be cited if used"]];


(* ::Section:: *)
(*Style Assignments*)


(* ::Subsection::Closed:: *)
(*Explanations*)


(* ::Text:: *)
(*On the lowest level I will use notation:  function$specification[indices][shift]. This will allow to make unique Makeboxes and Makeexpression command. Then I'm making number of assignments for equivalent usage.*)


(* ::Text:: *)
(*Meaning of notatons:*)


(* ::Text:: *)
(*T$u - upper gauge*)
(*T$r - right gauge*)
(*T$w - white gauge*)
(*T$g - physical gauge*)
(**)
(*gauges for Mathematica really mean different writing: upper gauge - ghotic, right gauge - script, white gauge - double struck,  physical gauge - new predefined symbols.*)
(**)
(*T$h - hatted function*)
(*T$b - barred function (complex conjugation)*)
(**)
(*q$f - exterior form*)
(**)
(*Below some not implemented but potentially possible  usages:*)
(**)
(*T$c - continuation of T-function using Hirota equations in the strip. Automatically assumed for hatted functions, but is sometimes useful for T-funcitons in the mirror strip*)
(**)
(*Some reserved notations that will be rarely or not used:*)
(*f$a - slightly shifted above the cut (reserved, though we agreed on different notation, in the argument of shift)*)
(*f$z - slightly shifted below the cut (reserved, though we agreed on different notation, in the argument of shift)*)
(**)
(*r$i - density with support on the short cut, inside (normally not needed, in good set of quantities this is automatic)*)
(*r$o - density with support on the long cut, outside (normally not needed, in good set of quantities this is automatic)*)
(**)
(*T$l - left gauge*)
(*T$wl - left white*)
(*T$gr - right-based physical*)
(*T$gl - left-based physical*)
(*T$ur - right-based computational*)
(*T$ul - left-based computational*)


(* ::Subsection:: *)
(*Internal definitions *)


(* ::Subsubsection::Closed:: *)
(*Ordering of specifications*)


specorder={"g","w","u","r","f","b","h","t"}//Reverse;


(* ::Subsubsection::Closed:: *)
(*Interpretation of overscripts*)


OverscriptToSpecs={"_"->"b","^"->"h","~"->"t"};


(* ::Subsubsection::Closed:: *)
(*Mapping between alphabets*)


With[{
Acharacter=ToCharacterCode["A"][[1]],
Zcharacter=ToCharacterCode["Z"][[1]],
acharacter=ToCharacterCode["a"][[1]],
zcharacter=ToCharacterCode["z"][[1]]},
With[{
ShiftToDSCapital=ToCharacterCode["\[DoubleStruckCapitalA]"][[1]]-Acharacter,
ShiftToDS=ToCharacterCode["\[DoubleStruckA]"][[1]]-acharacter,
ShiftToGoCapital=ToCharacterCode["\[GothicCapitalA]"][[1]]-Acharacter,
ShiftToGo=ToCharacterCode["\[GothicA]"][[1]]-acharacter,
ShiftToScCapital=ToCharacterCode["\[ScriptCapitalA]"][[1]]-Acharacter,
ShiftToSc=ToCharacterCode["\[ScriptA]"][[1]]-acharacter,
ShiftToPhCapital=ToCharacterCode["\:ff21"][[1]]-Acharacter,
ShiftToPh=ToCharacterCode["\:ff41"][[1]]-acharacter
},
	ToDoubleStruck[r_String]:=Block[{str=ToCharacterCode[r]},FromCharacterCode[
								Which[Acharacter<=#<=Zcharacter,#+ShiftToDSCapital,
									acharacter<=#<=zcharacter,#+ShiftToDS,
									True,#]&/@str
								]];
	ToGothic[r_String]:=Block[{str=ToCharacterCode[r]},FromCharacterCode[
								Which[Acharacter<=#<=Zcharacter,#+ShiftToGoCapital,
									acharacter<=#<=zcharacter,#+ShiftToGo,
									True,#]&/@str
							]];
	ToScript[r_String]:=Block[{str=ToCharacterCode[r]},FromCharacterCode[
								Which[Acharacter<=#<=Zcharacter,#+ShiftToScCapital,
								acharacter<=#<=zcharacter,#+ShiftToSc,
								True,#]&/@str
							]];
	ToPhysical[r_String]:=Block[{str=ToCharacterCode[r]},FromCharacterCode[
								Which[Acharacter<=#<=Zcharacter,#+ShiftToPhCapital,
								acharacter<=#<=zcharacter,#+ShiftToPh,
								True,#]&/@str
							]];
]]


(* ::Subsubsection:: *)
(*Predefined colors, USER flexibility*)


mBlue=RGBColor[0,0,.8];
mRed=RGBColor[.8,0,0];
mGreen=RGBColor[0,.5,0];
mGray=GrayLevel[0.4];
upColor=RGBColor[.54,.35,.15];
physicalgaugeColor=RGBColor[0,7/16,0];
rightgaugeColor=mRed;
whitegaugeColor=Black;
uppergaugeColor=upColor;
GenericColor=mBlue;(*Hue[5/12,1,1/3];*)
ColorOfGaugeTransformations=RGBColor[0,.6,0];
zetaColor=mGray;
etaColor=mRed;
rootColor=mGreen;
wLineColor=RGBColor[{159,120,71}/255];
formColor=mRed;


$ListOfMeanings={"Th"->"T$gh","\[ScriptCapitalF]h"->"\[ScriptCapitalF]$h","hh"->"h$h","xh"->"x$h","yh"->"y$h","Uh"->"U$h","U0h"->"U0$h","Ub"->"U$b","Qt"->"Q$t"}; (* add here to make custom shortcuts *)
$StyleTable={"\[ScriptCapitalT]"->StyleBox["\[ScriptCapitalT]",{FontColor->Red}],"q"->StyleBox["q",{FontColor->mBlue}],"p"->StyleBox["p",{FontColor->mRed}]}; (* add here to make custom appearance *)
$DefaultArgs={"\[DoubleStruckCapitalT]"->{1,1}}; (* add here to make custom default arguments. This example shows that \[DoubleStruckCapitalT] will be interpreted as Subscript[\[DoubleStruckCapitalT], 1,1] *)


(* ::Subsection:: *)
(*Predefined Interpretations (Assignments to $FunctionsWithArguments, $ExteriorForms, $FunctionsWithoutArguments can be modified, to adjust functionality. )*)


(* ::Text:: *)
(* Legend:   {name of functions, if gauge-dependent, style,  default args,}. *)
(*name of function:  String expression. Here I write only pure version, no hatting, complex conjugation, different gauge etc*)
(*if gauge-dependent:  True, False*)
(*style - list of rules to substitute to the StyleBox of this funcitons.  It can be list of 4 lists - for four different gauges. "Default" can be an option*)
(*order of gauges:  black-white-upper-right*)
(*default args:  "None" - means absent. Otherwise - argument in a {} wrapper (for the case when there are many)*)


If[Not@ValueQ["$Qfunction"],$Qfunction=True];
$FunctionsWithArguments=DeleteCases[{
(*{"a",False,{FontColor->mRed},"None"} (* this is for defining oscillator algebra *),
{"b",False,{FontColor->mRed},"None"} (* this is for defining oscillator algebra *),
{"\[Alpha]",False,{FontColor->mRed},"None"} (* this is for defining oscillator algebra *),
{"\[Beta]",False,{FontColor->mRed},"None"} (* this is for defining oscillator algebra *),
{"\[CapitalDelta]",False,{FontColor->mRed},"None"} (* this is for defining oscillator algebra *),*)
{"T",True,"Default",{1,1}},
{"W",False,{FontColor->upColor},{0}}(* = \!\(
\*SubsuperscriptBox[\(q\), \(3\), \([\(+a\)]\)] 
\*SubsuperscriptBox[
OverscriptBox[\(q\), \(_\)], \(4\), \([\(-a\)]\)]\)+\!\(
\*SubsuperscriptBox[\(q\), \(4\), \([\(+a\)]\)] 
\*SubsuperscriptBox[
OverscriptBox[\(q\), \(_\)], \(3\), \([\(-a\)]\)]\) *),
{"Y",False,"Default","None"},
{"Yt",False,"Default","None"},
{"Yo",False,"Default","None"},
{"q",True,{"Default",{FontColor->RGBColor[0,0,0]},"Default","Default"},"None"},
{"p",True,"Default","None"},
(*{"\[CapitalPsi]",False,{FontColor->zetaColor,FontWeight->Bold},"None"},*)
{"\[Eta]",False,{FontColor->etaColor,FontWeight->Bold},"None"},
{"\[Eta]0",False,{FontColor->rootColor,FontWeight->Bold},"None"},
{"\[Zeta]",False,{FontColor->zetaColor,FontWeight->Bold},"None"},
If[$Qfunction,{"Q",False,{FontColor->formColor},"None"},{}],
{"\[ScriptU]",False,{FontColor->RGBColor[0,0,.4]},{0}}
},{}];
(* all contents of this cell below this line is automatic creation of interpretation rules. Do not change *)
(* generating replacement rules from data above *)
$ListOfMeanings=Join[$ListOfMeanings,Select[(#[[1]]->#[[1]]<>"$")&/@$FunctionsWithArguments,(Not[  MemberQ[ $ListOfMeanings[[All,1]] , #[[1]] ] ])&  ]];
$ListOfMeanings=Join[$ListOfMeanings,Select[Flatten@DeleteCases[If[#[[2]],
		{ToDoubleStruck[#[[1]]]->#[[1]]<>"$w",
		ToGothic[#[[1]]]->#[[1]]<>"$u",
		ToScript[#[[1]]]->#[[1]]<>"$r",
		ToPhysical[#[[1]]]->#[[1]]<>"$g"}]&/@$FunctionsWithArguments,Null],(Not[  MemberQ[ $ListOfMeanings[[All,1]] , #[[1]] ] ])&  ]];

$StyleTable=Join[$StyleTable,
Select[If[#[[2]],
	If[ #[[3]]==="Default",
		{ToPhysical[#[[1]]]->StyleBox[ToPhysical[#[[1]]],{FontColor->physicalgaugeColor,FontWeight->Bold}],
		 ToDoubleStruck[#[[1]]]->StyleBox[ToDoubleStruck[#[[1]]],{FontColor->whitegaugeColor}],
		 ToGothic[#[[1]]]->StyleBox[ToGothic[#[[1]]],{FontColor->uppergaugeColor}],
		 ToScript[#[[1]]]->StyleBox[ToScript[#[[1]]],{FontColor->rightgaugeColor}]
		},
		{ToPhysical[#[[1]]]->StyleBox[ToPhysical[#[[1]]],#[[3,1]]/."Default"->{FontColor->physicalgaugeColor,FontWeight->Bold}],
		 ToDoubleStruck[#[[1]]]->StyleBox[ToDoubleStruck[#[[1]]],#[[3,2]]/."Default"->{FontColor->whitegaugeColor}],
		 ToGothic[#[[1]]]->StyleBox[ToGothic[#[[1]]],#[[3,3]]/."Default"->{FontColor->uppergaugeColor}],
		 ToScript[#[[1]]]->StyleBox[ToScript[#[[1]]],#[[3,4]]/."Default"->{FontColor->rightgaugeColor}]
		}
	]
,
	#[[1]]->StyleBox[#[[1]],#[[3]]/."Default"->{FontColor->GenericColor}]
]&/@$FunctionsWithArguments//Flatten
,
(Not[  MemberQ[ $StyleTable[[All,1]] , #[[1]] ] ])&]
];

$DefaultArgs=Join[$DefaultArgs,Select[If[#[[2]],
	{ToPhysical[#[[1]]]->#[[4]],
	 ToDoubleStruck[#[[1]]]->#[[4]],
	 ToGothic[#[[1]]]->#[[4]],
	 ToScript[#[[1]]]->#[[4]]}
	,
	#[[1]]->#[[4]]
]&/@DeleteCases[$FunctionsWithArguments,{__,"None"}]//Flatten,(Not[  MemberQ[ $DefaultArgs[[All,1]] , #[[1]] ] ])&]
];


(* ::Text:: *)
(* Exterior form. Legend :   {name, if gauge - dependent,  style, default dimension}. *)
(*name:  String expression. Here I write only pure version, no hatting, complex conjugation, different gauge etc*)
(*if gauge - dependent :  True, False*)
(*default dimension:  "None" - means should be always written explicitly. Otherwise - a number.*)
(*        style - list of rules to substitute to the StyleBox of this funcitons.  It can be list of 4 lists - for four different gauges. "Default" can be an option*)
(*       order of gauges :  black - white - upper - right*)


$ExteriorForms={
{"q",True,"Default",1},
{"p",True,"Default",1},
{"\[Omega]",False,"Default",2}(*,
{"\[Chi]",False,"Default",2}*)
};
(* all contents of this cell below this line is automatic creation of interpretation rules. Do not change *)
(* generating replacement rules from data above *)
$ListOfMeanings=Join[$ListOfMeanings,Select[(#[[1]]->#[[1]]<>"$f")&/@$ExteriorForms,(Not[  MemberQ[ $ListOfMeanings[[All,1]] , #[[1]] ] ])&  ]];
$ListOfMeanings=Join[$ListOfMeanings,Select[Flatten@DeleteCases[If[#[[2]],
		{ToDoubleStruck[#[[1]]]->#[[1]]<>"$wf",
		ToGothic[#[[1]]]->#[[1]]<>"$uf",
		ToScript[#[[1]]]->#[[1]]<>"$rf",
		ToPhysical[#[[1]]]->#[[1]]<>"$gf"}]&/@$ExteriorForms,Null],(Not[  MemberQ[ $ListOfMeanings[[All,1]] , #[[1]] ] ])&  ]];

$StyleTable=Join[$StyleTable,
Select[If[#[[2]],
	If[ #[[3]]==="Default",
		{ToPhysical[#[[1]]]->StyleBox[ToPhysical[#[[1]]],{FontColor->physicalgaugeColor,FontWeight->Bold}],
		 ToDoubleStruck[#[[1]]]->StyleBox[ToDoubleStruck[#[[1]]],{FontColor->whitegaugeColor}],
		 ToGothic[#[[1]]]->StyleBox[ToGothic[#[[1]]],{FontColor->uppergaugeColor}],
		 ToScript[#[[1]]]->StyleBox[ToScript[#[[1]]],{FontColor->rightgaugeColor}]
		},
		{ToPhysical[#[[1]]]->StyleBox[ToPhysical[#[[1]]],#[[3,1]]/."Default"->{FontColor->physicalgaugeColor,FontWeight->Bold}],
		 ToDoubleStruck[#[[1]]]->StyleBox[ToDoubleStruck[#[[1]]],#[[3,2]]/."Default"->{FontColor->whitegaugeColor}],
		 ToGothic[#[[1]]]->StyleBox[ToGothic[#[[1]]],#[[3,3]]/."Default"->{FontColor->uppergaugeColor}],
		 ToScript[#[[1]]]->StyleBox[ToScript[#[[1]]],#[[3,4]]/."Default"->{FontColor->rightgaugeColor}]
		}
	]
,
	#[[1]]->StyleBox[#[[1]],#[[3]]/."Default"->{FontColor->GenericColor}]
]&/@$ExteriorForms//Flatten
,
(Not[  MemberQ[ $StyleTable[[All,1]] , #[[1]] ] ])&]
];

$DefaultFormArgs=Select[If[#[[2]],
	{#[[1]]->{#[[4]]},
	 ToPhysical[#[[1]]]->{#[[4]]},
	 ToDoubleStruck[#[[1]]]->{#[[4]]},
	 ToGothic[#[[1]]]->{#[[4]]},
	 ToScript[#[[1]]]->{#[[4]]}}
	,
	#[[1]]->{#[[4]]}
]&/@DeleteCases[$ExteriorForms,{__,"None"}]//Flatten,(Not[  MemberQ[ $DefaultArgs[[All,1]] , #[[1]] ] ])&];
$DefaultArgs=Join[$DefaultArgs,$DefaultFormArgs];


(* ::Text:: *)
(* Functions without arguments. Legend :   {name, if gauge - dependent, style}. *)


If[Not@ValueQ["$QBaxter"],$QBaxter=True];
$FunctionsWithoutArguments=DeleteCases[{
(*{"\[Alpha]",False,{FontColor->wLineColor}}
(* \[Alpha] is introduced for particular notion of coupling constant, I just wanted to make it in color *),
*)
{"\[Phi]",False,{FontColor->wLineColor}}
(* Angle parameters of Wilson lines *),
{"\[Theta]",False,{FontColor->wLineColor}}
(* Angle parameters of Wilson lines *),
{"H",False,{FontColor->upColor}}(* = Log[Subscript[T, 1,0]/Subsuperscript[T, 0,0, +]]-asymptotic *),
{"\[ScriptCapitalF]",False,"Default"},
{"\[ScriptCapitalC]",False,{FontColor->mBlue}},
{"Bp",False,"Default"},
{"Bm",False,"Default"},
{"Rp",False,"Default"},
{"Rm",False,"Default"},
{"Qb",False,"Default"},
{"\[ScriptCapitalX]",False,"Default"},
{"\[ScriptCapitalY]",False,"Default"},
{"h",False,"Default"},
{"f",False,"Default"},
(*{"B",False,{FontWeight->Bold}},*) 
{"x",False,"Default"},
{"y",False,"Default"},
{"U",False,"Default"},
{"V",False,"Default"},
If[$QBaxter,{"Q",False,{FontColor->mRed}},{}],
{"\[Delta]Q",False,{FontColor->mRed}},
{"Q0",False,{FontColor->RGBColor[{1/2,1/2,0}]}}
},{}];
(* all contents of this cell below this line is automatic creation of interpretation rules. Do not change *)

$ListOfMeanings=Join[$ListOfMeanings,Select[(#[[1]]->#[[1]]<>"$")&/@$FunctionsWithoutArguments,(Not[  MemberQ[ $ListOfMeanings[[All,1]] , #[[1]] ] ])&  ]];
$ListOfMeanings=Join[$ListOfMeanings,Select[Flatten@DeleteCases[If[#[[2]],
		{ToDoubleStruck[#[[1]]]->#[[1]]<>"$w",
		ToGothic[#[[1]]]->#[[1]]<>"$u",
		ToScript[#[[1]]]->#[[1]]<>"$r",
		ToPhysical[#[[1]]]->#[[1]]<>"$g"}]&/@$FunctionsWithoutArguments,Null],(Not[  MemberQ[ $ListOfMeanings[[All,1]] , #[[1]] ] ])&  ]];

(* generating replacement rules from data above *)
$StyleTable=Join[$StyleTable,
Select[If[#[[2]],
	If[ #[[3]]==="Default",
		{ToPhysical[#[[1]]]->StyleBox[ToPhysical[#[[1]]],{FontColor->physicalgaugeColor,FontWeight->Bold}],
		 ToDoubleStruck[#[[1]]]->StyleBox[ToDoubleStruck[#[[1]]],{FontColor->whitegaugeColor}],
		 ToGothic[#[[1]]]->StyleBox[ToGothic[#[[1]]],{FontColor->uppergaugeColor}],
		 ToScript[#[[1]]]->StyleBox[ToScript[#[[1]]],{FontColor->rightgaugeColor}]
		},
		{ToPhysical[#[[1]]]->StyleBox[ToPhysical[#[[1]]],#[[3,1]]/."Default"->{FontColor->physicalgaugeColor,FontWeight->Bold}],
		 ToDoubleStruck[#[[1]]]->StyleBox[ToDoubleStruck[#[[1]]],#[[3,2]]/."Default"->{FontColor->whitegaugeColor}],
		 ToGothic[#[[1]]]->StyleBox[ToGothic[#[[1]]],#[[3,3]]/."Default"->{FontColor->uppergaugeColor}],
		 ToScript[#[[1]]]->StyleBox[ToScript[#[[1]]],#[[3,4]]/."Default"->{FontColor->rightgaugeColor}]
		}
	]
,
	#[[1]]->StyleBox[#[[1]],#[[3]]/."Default"->{FontColor->ColorOfGaugeTransformations}]
]&/@$FunctionsWithoutArguments//Flatten
,
(Not[  MemberQ[ $StyleTable[[All,1]] , #[[1]] ] ])&]
];


(* sorting specs in a good order *)
FindSpecs[xxx_String]:=If[Length[#]==0,{},Characters@First[#]]&@StringCases[xxx,"$"~~str__:>str,1];
FindSpecs[xxx_]:=FindSpecs[ToString[xxx]];
FindFun[xxx_String]:=StringCases[xxx,Shortest[str__~~"$"]:>str,1][[1]];
FindFun[xxx_]:=FindFun[ToString[xxx]];
SortSpecs[xxx_List]:=Reverse@SortBy[ xxx , Position[specorder,#]& ];
$ListOfMeanings=Rule[#[[1]],StringJoin[ FindFun[ #[[2]] ],"$",Reverse@SortBy[ FindSpecs[ #[[2]] ] , Position[specorder,#]& ] ]]&/@$ListOfMeanings;


FunctionsOfSpectralParameter=Join[
								$FunctionsWithArguments[[All,1]],
								$ExteriorForms[[All,1]],
								$FunctionsWithoutArguments[[All,1]],
								$ListOfMeanings[[All,1]]];
(* Obsolete, but is used in the present version of program. If everything works propertly, it should be just equal to $ListOfMeanings[[All,1]] *)


(* ::Subsection::Closed:: *)
(*MakeExpression definitions*)


(* ::Text:: *)
(*Here I teach mathematica to properly parse typed input. *)


(* ::Text:: *)
(*Strict rules of interpretation are the following*)
(*- Expression containing $ should be exact, therefore no MakeExpression for them*)
(*- r in Expression of type  f[r_] always means shift!!! Therefore mathematica will attempt to find default arguments of f. In particular q[r] will be interpreted as q$f[1][r].  Expressions like f[a_,b__] are interpreted as f[a,b][0] *)


(* T[a__][r] type of expression, i.e. quasicomplete, things like gauge might be omitted *)
MakeExpression[RowBox[{RowBox[{f_/;MemberQ[FunctionsOfSpectralParameter,f],"[",a__,"]"}],"[",b_,"]"}],StandardForm]:=MakeExpression[
	RowBox[{RowBox[{f/.$ListOfMeanings,"[",a,"]"}],"[",b,"]"}],StandardForm];
(* Overscript[T, _][a__][r] type of expression, i.e. quasicomplete, things like gauge might be omitted *)
MakeExpression[RowBox[{RowBox[{f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],"[",a__,"]"}],"[",b_,"]"}],StandardForm]:=Block[{fun,specs},
	fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]];
	specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1];
	specs=SortBy[Union[specs,List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs],Position[specorder,#]&]//Reverse;
	MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",a,"]"}],"[",b,"]"}],StandardForm]
];
(* T[a,b] type of expression, it is essential that there are two arguments a,b. Interpreted as T[a,b][0]*)
MakeExpression[RowBox[{f_/;MemberQ[FunctionsOfSpectralParameter,f],"[",a_,",",b__,"]"}],StandardForm]:=MakeExpression[
	RowBox[{RowBox[{f/.$ListOfMeanings,"[",a,",",b,"]"}],"[",0,"]"}],StandardForm];
MakeExpression[RowBox[{f_/;MemberQ[FunctionsOfSpectralParameter,f],"[",RowBox[{a_,",",b__}],"]"}],StandardForm]:=MakeExpression[
	RowBox[{RowBox[{f/.$ListOfMeanings,"[",a,",",b,"]"}],"[",0,"]"}],StandardForm];

(* Overscript[T, _][a,b] type of expression *)
MakeExpression[RowBox[{f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],"[",a_,",",b__,"]"}],StandardForm]:=
		MakeExpression[RowBox[{RowBox[{f,"[",RowBox[{a,",",b}],"]"}],"[","0","]"}],StandardForm]
MakeExpression[RowBox[{f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],"[",RowBox[{a_,",",b__}],"]"}],StandardForm]:=
		MakeExpression[RowBox[{RowBox[{f,"[",RowBox[{a,",",b}],"]"}],"[","0","]"}],StandardForm]


(* f[a], Overscript[f, _][a] type of expression, a always means shift. *)
MakeExpression[RowBox[{f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],"[",a_,"]"}],StandardForm]:=Block[{fun,specs},
	fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]];
	specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1];
	If[MemberQ[$DefaultFormArgs[[All,1]],fun],specs=Union[specs,{"f"}]];
	specs=SortBy[Union[specs,List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs],Position[specorder,#]&]//Reverse;
	Which[MemberQ[$DefaultArgs[[All,1]],fun],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fun/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		MemberQ[$DefaultArgs[[All,1]],Flatten[f][[1]]],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[Flatten[f][[1]]/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		True,
		MakeExpression[  RowBox[{StringJoin[fun,"$",specs],"[",a,"]"}],StandardForm  ]
	]
	];

MakeExpression[RowBox[{f_/;MemberQ[FunctionsOfSpectralParameter,f],"[",a_,"]"}],StandardForm]:=Block[{fun,specs},
	fun=StringCases[f/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]];
	specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[f/.$ListOfMeanings,"$"~~str__:>str,1];
	If[MemberQ[$DefaultFormArgs[[All,1]],fun],specs=Union[specs,{"f"}]];
	specs=SortBy[specs,Position[specorder,#]&]//Reverse;
	Which[MemberQ[$DefaultArgs[[All,1]],fun],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fun/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		MemberQ[$DefaultArgs[[All,1]],f],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[f/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		True,
		MakeExpression[  RowBox[{StringJoin[fun,"$",specs],"[",a,"]"}],StandardForm  ]
	]
	];


(*MakeExpression for input like Subscript[r, a,b][shift]*)
MakeExpression[RowBox[{SubscriptBox[f_/;MemberQ[FunctionsOfSpectralParameter,f],sub_],"[",a_,"]"}],StandardForm]:=MakeExpression[
	Block[{fun=StringCases[f/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	   specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[f/.$ListOfMeanings,"$"~~str__:>str,1],
	   args=If[MatchQ[sub,RowBox[{"(",__,")"}]],sub[[1,2]],sub]},
	If[Not[{args}==={sub}],AppendTo[specs,"f"]];
	specs=SortBy[Union@specs,Position[specorder,#]&]//Reverse;
	RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",args,"]"}],"[",a,"]"}]
	]
	,StandardForm];
MakeExpression[RowBox[{SubscriptBox[f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],sub_],"[",a_,"]"}],StandardForm]:=MakeExpression[
	Block[{fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	   specs=(If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1])~Join~(List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs),
	   args=If[MatchQ[sub,RowBox[{"(",__,")"}]],sub[[1,2]],sub]},
	If[Not[{args}==={sub}],AppendTo[specs,"f"]];
	specs=SortBy[Union@specs,Position[specorder,#]&]//Reverse;
	RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",args,"]"}],"[",a,"]"}]
	]
	,StandardForm];


(* ::Text:: *)
(*MakeExpression for customized input like *)
(*\!\(\*SuperscriptBox[*)
(*SubscriptBox[\(r\), \(a, b\)], \([shift]\)]\),*)
(*\!\(\*SuperscriptBox[\(r\), \([shift]\)]\),SuperPlus[r],SuperMinus[r] etc.*)


Subscript[Q, \[Placeholder]]


(* Subsuperscript *)
MakeExpression[SubsuperscriptBox[f_/;MemberQ[FunctionsOfSpectralParameter,f],sub_,RowBox[{"[",a_,"]"}]],StandardForm]:=MakeExpression[
	Block[{fun=StringCases[f/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	   specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[f/.$ListOfMeanings,"$"~~str__:>str,1],
	   args=If[MatchQ[sub,RowBox[{"(",__,")"}]],sub[[1,2]],sub]},
	If[Not[{args}==={sub}],AppendTo[specs,"f"]];
	specs=SortBy[Union@specs,Position[specorder,#]&]//Reverse;
	RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",args/."|":>Sequence[",","\"\<|\>\"",","]//.RowBox[{aa__}]:>aa,"]"}],"[",a,"]"}]
	]
	,StandardForm];
MakeExpression[SubsuperscriptBox[f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],sub_,RowBox[{"[",a_,"]"}]],StandardForm]:=MakeExpression[
	Block[{fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	   specs=(If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1])~Join~(List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs),
	   
		args=If[MatchQ[sub,RowBox[{"(",__,")"}]],sub[[1,2]],sub]},
	If[Not[{args}==={sub}],AppendTo[specs,"f"]];
	specs=SortBy[Union@specs,Position[specorder,#]&]//Reverse;
	RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",args/."|":>Sequence[",","\"\<|\>\"",","]//.RowBox[{aa__}]:>aa,"]"}],"[",a,"]"}]
	]
	,StandardForm];

MakeExpression[SubsuperscriptBox[f_,sub_,"+"],StandardForm]:=MakeExpression[SubsuperscriptBox[f,sub,RowBox[{"[","1","]"}]],StandardForm];
MakeExpression[SubsuperscriptBox[f_,sub_,"-"],StandardForm]:=MakeExpression[SubsuperscriptBox[f,sub,RowBox[{"[",RowBox[{"-", "1"}],"]"}]],StandardForm];

(* Superscript *)
MakeExpression[SuperscriptBox[f_/;MemberQ[FunctionsOfSpectralParameter,f],RowBox[{"[",a_,"]"}]],StandardForm]:=Block[
	{fun=StringCases[f/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	   specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[f/.$ListOfMeanings,"$"~~str__:>str,1]
	},
	If[MemberQ[$DefaultFormArgs[[All,1]],fun],specs=Union[specs,{"f"}]];
	specs=SortBy[specs,Position[specorder,#]&]//Reverse;
	Which[MemberQ[$DefaultArgs[[All,1]],fun],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fun/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		MemberQ[$DefaultArgs[[All,1]],f],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[f/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		True,
		MakeExpression[  RowBox[{StringJoin[fun,"$",specs],"[",a,"]"}],StandardForm  ]
	]
	];

MakeExpression[SuperscriptBox[f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],RowBox[{"[",a_,"]"}]],StandardForm]:=Block[
	{fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	 specs=(If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1])~Join~(List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs)
	   },
	If[MemberQ[$DefaultFormArgs[[All,1]],fun],specs=Union[specs,{"f"}]];
	specs=SortBy[specs,Position[specorder,#]&]//Reverse;
	Which[MemberQ[$DefaultArgs[[All,1]],fun],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fun/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		MemberQ[$DefaultArgs[[All,1]],Flatten[f][[1]]],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[Flatten[f][[1]]/.$DefaultArgs][[1,2]],"]"}],"[",a,"]"}],StandardForm],
		True,
		MakeExpression[  RowBox[{StringJoin[fun,"$",specs],"[",a,"]"}],StandardForm  ]
	]
	];

MakeExpression[SuperscriptBox[f_,"+"],StandardForm]:=MakeExpression[SuperscriptBox[f,RowBox[{"[","1","]"}]],StandardForm];
MakeExpression[SuperscriptBox[f_,"-"],StandardForm]:=MakeExpression[SuperscriptBox[f,RowBox[{"[",RowBox[{"-", "1"}],"]"}]],StandardForm];

(* Subscript *)
MakeExpression[SubscriptBox[f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],sub_],StandardForm]:=MakeExpression[
	Block[{fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
		 specs=(If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1])~Join~(List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs),
	     args=If[MatchQ[sub,RowBox[{"(",__,")"}]],sub[[1,2]],sub]},
	If[Not[{args}==={sub}],AppendTo[specs,"f"]];
	specs=SortBy[specs,Position[specorder,#]&]//Reverse;
	RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",args/."|":>Sequence[",","\"\<|\>\"",","]//.RowBox[{aa__}]:>aa,"]"}],"[",0,"]"}]
	]
	,StandardForm];
MakeExpression[SubscriptBox[f_/;MemberQ[FunctionsOfSpectralParameter,f],sub_],StandardForm]:=MakeExpression[
	Block[{fun=StringCases[f/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
	   specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[f/.$ListOfMeanings,"$"~~str__:>str,1],
	   args=If[MatchQ[sub,RowBox[{"(",__,")"}]],sub[[1,2]],sub]},
	If[Not[{args}==={sub}],AppendTo[specs,"f"]];
	specs=SortBy[specs,Position[specorder,#]&]//Reverse;
	RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",args/."|":>Sequence[",","\"\<|\>\"",","]//.RowBox[{aa__}]:>aa,"]"}],"[",0,"]"}]
	]
	,StandardForm];
(* Only overscripted *)
MakeExpression[f_OverscriptBox/;MemberQ[FunctionsOfSpectralParameter,Flatten[f][[1]]],StandardForm]:=Block[
		{fun=StringCases[Flatten[f][[1]]/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
		 specs=(If[Length[#]==0,{},Characters@First[#]]&@StringCases[Flatten[f][[1]]/.$ListOfMeanings,"$"~~str__:>str,1])~Join~(List@@Flatten[f][[2;;-1]]/.OverscriptToSpecs)
		},
	If[MemberQ[$DefaultFormArgs[[All,1]],fun],specs=Union[specs,{"f"}]];
	specs=SortBy[specs,Position[specorder,#]&]//Reverse;
	Which[MemberQ[$DefaultArgs[[All,1]],fun],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fun/.$DefaultArgs][[1,2]],"]"}],"[",0,"]"}],StandardForm],
		MemberQ[$DefaultArgs[[All,1]],Flatten[f][[1]]],
		MakeExpression[
		RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[Flatten[f][[1]]/.$DefaultArgs][[1,2]],"]"}],"[",0,"]"}],StandardForm],
		True,
		MakeExpression[  RowBox[{StringJoin[fun,"$",specs],"[",0,"]"}],StandardForm  ]
	]
	];


(* ::Text:: *)
(*Finally, interpretation of  Fun as Fun$[0] !  (If Fun belongs to predefined above class "FuncitonsOfSpectralParameter")*)


FinalAssignment[fn_]:=Block[
{fun=StringCases[fn/.$ListOfMeanings,Shortest[str__~~"$"]:>str,1][[1]],
specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[fn/.$ListOfMeanings,"$"~~str__:>str,1]}
,
	If[MemberQ[$DefaultFormArgs[[All,1]],fun],AppendTo[specs,"f"]];
	specs=SortBy[Union@specs,Position[specorder,fn]&]//Reverse;
	Which[MemberQ[$DefaultArgs[[All,1]],fun],
		ToExpression[
		RowBox[{fn,"=",RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fun/.$DefaultArgs][[1,2]],"]"}],"[",0,"]"}]}]],
		MemberQ[$DefaultArgs[[All,1]],fn],
		ToExpression[
		RowBox[{fn,"=",RowBox[{RowBox[{StringJoin[fun,"$",specs],"[",ToBoxes[fn/.$DefaultArgs][[1,2]],"]"}],"[",0,"]"}]}]],
		MemberQ[$FunctionsWithoutArguments[[All,1]],fun],
		ToExpression[RowBox[{fn,"=", RowBox[{StringJoin[fun,"$",specs],"[",0,"]"}]}]]
	]
];
FinalAssignment/@FunctionsOfSpectralParameter;


(* ::Subsection::Closed:: *)
(*MakeBoxes definitions*)


uDependentQ[x_]:=Not[StringFreeQ[StringTake[ToString[x],{2,-1}],"$"]]&&(Head[x]===Symbol)&&Not[StringFreeQ[If[Length[#]==0,"e",First[#]]&@StringCases[ToString[x],"$"~~str__:>str,1],LetterCharacter]];


MakeBoxes[{1,2,3}]


Position[{a,b},c]


MakeBoxes[(f_/;uDependentQ[f])[ainternal__:0][rinternal_:0],StandardForm]:=
Block[{tm1,tm2,fun=StringCases[ToString[f],Shortest[str__~~"$"]:>str,1][[1]],
	specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[ToString[f],"$"~~str__:>str,1],
	tpset,tmpset,subbox},
	tpset=Which[MemberQ[specs,"w"],ToDoubleStruck[fun],
				MemberQ[specs,"g"],ToPhysical[fun],
				MemberQ[specs,"r"],ToScript[fun],
				MemberQ[specs,"u"],ToGothic[fun],
				True,fun]/.$StyleTable;
	If[Not[Head[tpset]===StyleBox],tpset=StyleBox[tpset]];
	tmpset=tpset;
	If[MemberQ[specs,"t"],tpset=OverscriptBox[tpset,tmpset/.StyleBox[rr_,xx___]:>StyleBox["~",xx]]];
	If[MemberQ[specs,"b"],tpset=OverscriptBox[tpset,tmpset/.StyleBox[rr_,xx___]:>StyleBox["_",xx]]];
	If[MemberQ[specs,"h"],tpset=OverscriptBox[tpset,tmpset/.StyleBox[rr_,xx___]:>StyleBox["^",xx]]];
	If[((tmpset[[1]]/.$DefaultArgs)==={ainternal})  && (Not@MemberQ[$ExteriorForms[[All,1]],fun]||MemberQ[specs,"f"]),
			Which[rinternal===0,
					RowBox[{tpset}],
				rinternal===1,
					SuperscriptBox[tpset,RowBox[{"+"}]],
				rinternal===-1,
					SuperscriptBox[tpset,RowBox[{"-"}]],
				True,
					SuperscriptBox[tpset,RowBox[{"[",MakeBoxes[rinternal],"]"}]]
					]
		,
		subbox=If[MemberQ[specs,"f"],RowBox[{"(",MakeBoxes[ainternal],")"}],
If[Not[(tm1=Position[{ainternal},"|"])==={}],
RowBox[{ToBoxes[{ainternal}[[1;;tm1[[1,1]]-1]]][[1,2]],"|",ToBoxes[{ainternal}[[tm1[[1,1]]+1;;-1]]][[1,2]]}]
,
MakeBoxes[{ainternal}][[1,2]]
]
];
		Which[rinternal===0,
					SubscriptBox[tpset,subbox],
			rinternal===1,
					SubsuperscriptBox[tpset,subbox,RowBox[{"+"}]],
			rinternal===-1,
					SubsuperscriptBox[tpset,subbox,RowBox[{"-"}]],
			True,
					SubsuperscriptBox[tpset,subbox,RowBox[{"[",MakeBoxes[rinternal],"]"}]]
				]
	]
	];
MakeBoxes[(f_/;uDependentQ[f])[rinternal_:0],StandardForm]:=
Block[{fun=StringCases[ToString[f],Shortest[str__~~"$"]:>str,1][[1]],
	specs=If[Length[#]==0,{},Characters@First[#]]&@StringCases[ToString[f],"$"~~str__:>str,1],
	tpset,tmpset},
	tpset=Which[MemberQ[specs,"w"],ToDoubleStruck[fun],
				MemberQ[specs,"g"],ToPhysical[fun],
				MemberQ[specs,"r"],ToScript[fun],
				MemberQ[specs,"u"],ToGothic[fun],
				True,fun]/.$StyleTable;
	If[Not[Head[tpset]===StyleBox],tpset=StyleBox[tpset]];
	tmpset=tpset;
	If[MemberQ[specs,"t"],tpset=OverscriptBox[tpset,tmpset/.StyleBox[rr_,xx___]:>StyleBox["~",xx]]];
	If[MemberQ[specs,"b"],tpset=OverscriptBox[tpset,tmpset/.StyleBox[rr_,xx___]:>StyleBox["_",xx]]];
	If[MemberQ[specs,"h"],tpset=OverscriptBox[tpset,tmpset/.StyleBox[rr_,xx___]:>StyleBox["^",xx]]];
	Which[
		rinternal===0,
			If[MemberQ[$FunctionsWithoutArguments[[All,1]],fun],RowBox[{tpset}],SuperscriptBox[tpset,RowBox[{"[","0","]"}]]],
		rinternal===1,
			SuperscriptBox[tpset,RowBox[{"+"}]],
		rinternal===-1,
			SuperscriptBox[tpset,RowBox[{"-"}]],
		True,
			SuperscriptBox[tpset,RowBox[{"[",MakeBoxes[rinternal],"]"}]]
	]];



(* ::Section:: *)
(*Working with objects*)


(* ::Subsection:: *)
(*Complex conjugation (mconj)*)


mconj={Complex[a_,b_]:>Complex[a,-b],
\[Alpha]-> 




\!\(\*OverscriptBox[\(\[Alpha]\), \(_\)]\),




\!\(\*OverscriptBox[\(\[Alpha]\), \(_\)]\)->\[Alpha],
\[Eta]$[a__][r_:0]:>\[Eta]$b[a][-Conjugate[r]],\[Eta]$b[a__][r_:0]:>\[Eta]$[a][-Conjugate[r]],
\[Eta]$h[a__][r_:0]:>\[Eta]$bh[a][-Conjugate[r]],\[Eta]$bh[a__][r_:0]:>\[Eta]$h[a][-Conjugate[r]],
\[Eta]0$[a__][r_:0]:>\[Eta]0$b[a][-Conjugate[r]],\[Eta]0$b[a__][r_:0]:>\[Eta]0$[a][-Conjugate[r]],
H$[r_:0]:>H$b[-Conjugate[r]],H$b[r_:0]:>H$[-Conjugate[r]],
\[Zeta]$[a___][0]:>\[Zeta]$[a][0],
q$[a__][r_:0]:>q$b[a][-Conjugate[r]],q$b[a__][r_:0]:>q$[a][-Conjugate[r]],Q0$[r_:0]:>Q0$[-Conjugate[r]],Q$[r_:0]:>Q$[-Conjugate[r]],
\[Delta]Q$[r_:0]:>\[Delta]Q$b[-Conjugate[r]],\[Delta]Q$b[r_:0]:>\[Delta]Q$[-Conjugate[r]],
U$[a_?StringQ][r_:0]:>U$b[a][-Conjugate[r]],
U$b[a_?StringQ][r_:0]:>U$[a][-Conjugate[r]],
U$[r_:0]:>U$b[-Conjugate[r]],U$b[r_:0]:>U$[-Conjugate[r]],
V$[r_:0]:>V$b[-Conjugate[r]],V$b[r_:0]:>V$[-Conjugate[r]],
x$h[r_:0]:>x$h[-Conjugate[r]],x$[r_:0]:>1/x$[-Conjugate[r]],y$h[r_:0]:>y$h[-Conjugate[r]],
W$[a__][r_:0]:>W$[a][-Conjugate[r]],W$h[a__][r_:0]:>W$h[a][-Conjugate[r]],
\[ScriptU]$[a_][r_:0]:>\[ScriptU]$[Conjugate[a]/.Conjugate[v0]->v0][-Conjugate[r]],
T$[a_,b_][r_:0]:>T$[a,b][-Conjugate[r]],Y$[a_,b_][r_:0]:>Y$[a,b][-Conjugate[r]],
xxx_?NumericQ:>Conjugate[xxx]};


(* ::Subtitle::Closed:: *)
(*Generic*)


(* ::Subsection::Closed:: *)
(*Check that input if of proper form (i.e. of the type xx$yyy), basic operations with symbols*)


dolexpressionQ[x_]:=MatchQ[x,_?uDependentQ[__][_]]||MatchQ[x,_?uDependentQ[_]];
hextFormQ[x_]:=Not[(StringFreeQ[#,"$"]||StringFreeQ[#,"f"])&[StringTake[ToString[x],{2,-1}]]]&&(Head[x]===Symbol);
extFormQ[y_]:=MatchQ[y,_?hextFormQ[_][_]]


RemoveSpec[x_,spec_String]:=Symbol[FindFun[ToString[x]]<>"$"<>DeleteCases[FindSpecs[ToString[x]],spec]]


(* ::Subsection:: *)
(*Shift operation*)


(* Shift replacemnt *)
Shift[m_]:={(xxx:Except[\[Zeta]$|\[Eta]0$|\[Eta]0$b|\[Zeta]$b|y$|y$h,_?uDependentQ])[a__][r_:0]:>xxx[a][r+m],(xxx:Except[\[Zeta]$|\[Eta]0$|\[Eta]0$b|\[Zeta]$b|y$|y$h,_?uDependentQ])[r_:0]:>xxx[r+m]};
Shift\[Eta]0[m_]:={\[Eta]0$[a__][r_:0]:>\[Eta]0$[a][r+m],\[Eta]0$b[a__][r_:0]:>\[Eta]0$b[a][r+m]};


(* ::Subsection:: *)
(*Definition of Wedge*)


$dim=4;
ClearAll[Wedge];
Wedge[]=1;
Wedge[x_]:=x;
Wedge[x___,z1_+z2_,y___]:=Wedge[x,z1,y]+Wedge[x,z2,y];
Wedge[x___,(a_/;FreeQ[a,_?extFormQ]) b_,y___]:=a Wedge[x,b,y];
Wedge[x___,(a_/;FreeQ[a,_?extFormQ]) ,y___]:=a Wedge[x,y];
Wedge[x___,y_[a_/;OddQ[a]][b_:0],y_[a_/;OddQ[a]][b_:0],z___]:=0;
Wedge[x___,a1_,a2_,y___]:=(-1)^(Head[a1][[1]] Head[a2][[1]]) Wedge[x,a2,a1,y]/; OrderedQ[{a1,a2}]&&Not[a1===a2];
Wedge[x___,(a_?hextFormQ)[0][r_:0],y___]:=Symbol[FindFun[a]<>"$"<>DeleteCases[FindSpecs[a],"f"]][0][r] Wedge[x,y];
SetAttributes[Wedge,Flat];


512/657 528//N
