(* ::Package:: *)

If[Not@ValueQ[$Copyright],$Copyright=False];
If[$Copyright,Print["zetafunctions.m: Copyright by S.Leurent adn D.Volin. Free to use and modify, but should be cited if used"]];


(* ::Subsection::Closed:: *)
(*Basic transforms and groupping together.*)


(* ::Text:: *)
(*transformation between different objects*)


subZetas={\[Zeta]$[1][0]->EulerGamma,\[Zeta]$[a_][0]:>Zeta[a]};
sub\[Eta]0\[Eta]={\[Eta]0$->\[Eta]$,\[Eta]0$b->\[Eta]$b};
sub\[Eta]\[Eta]0={\[Eta]$->\[Eta]0$,\[Eta]$b->\[Eta]0$b};


(* ::Text:: *)
(*Simplifying Sqrt[3]*)


densqrt3={((a_.) + Sqrt[3]*(c_.))^(r_ /; r < 0) :> ((a^2 - 3*c^2)/(a - c*Sqrt[3]))^r,
((a_.) + (c_.)/Sqrt[3])^(r_ /; r < 0) :> ((a^2 - c^2/3)/(a - c/Sqrt[3]))^r};


Attributes[sqrt3Simp]={Listable};


sqrt3Simp[Ee_]:=Collect[Ee/.densqrt3,Sqrt[3]];
sqrt3Simp/:eEe_/.sqrt3Simp:=sqrt3Simp[eEe];
sqrt3Simp/:eEe_//.sqrt3Simp:=FixedPoint[sqrt3Simp,eEe];


(* ::Text:: *)
(*Lists to use in Collect (merely for older code, ExpaCol defined below is more efficient)*)


\[Eta]list={Times[
Power[\[Eta]$[__][_],_.],
Power[\[Eta]$b[__][_],_.]],\[Eta]$[__][_],\[Eta]$b[__][_]};
\[Eta]0list={Times[
Power[\[Eta]0$[__][_],_.],
Power[\[Eta]0$b[__][_],_.]],\[Eta]0$[__][_],\[Eta]0$b[__][_]};
Q0list={HoldPattern@Times[Power[Q0$[_],_.],Power[Q0$[_],_.]],Q0$[_]};
Module[{mPattern,maxLength=4},
\[Zeta]list=Join[(Times@@Table[Power[\[Zeta]$[mPattern[ToExpression["dummy"<>ToString[i]],BlankSequence[]]][0],_.],{i,#}])&/@Reverse@Range[2,maxLength],{\[Zeta]$[__][0]}]/.mPattern->Pattern];


(* ::Text:: *)
(*ExpaCol below collects with respect to given patterns. *)


ExpaCol[expr_,var_]:=Expand[Collect[expr,var],var];
ExpaCol[expr_,var_,f_]:=Expand[Collect[expr,var,f],var];


(* ::Text:: *)
(*Patterns for ExpaCol*)


\[Eta]patt=(\[Eta]$|\[Eta]$b)[__][_];
\[Eta]0patt=(\[Eta]0$|\[Eta]0$b)[__][_];
\[Zeta]patt=\[Zeta]$[__][_];
Q0patt=Q0$[_];


(* ::Subsection::Closed:: *)
(*Working with \[Eta]- and \[Zeta]-functions, analytics*)


shiftp\[Eta][a_]:={\[Eta]$[aa__][a]:>If[Length[{aa}]==1,1/(u+(I a)/2)^aa,(u+(I a)/2)^-{aa}[[1]] \[Eta]$[aa][[2;;-1]][a+2]]+\[Eta]$[aa][a+2]};
shiftm\[Eta][a_]:={\[Eta]$[aa__][a]:>-If[Length[{aa}]==1,1/(u+(I a)/2-I)^aa,(u+(I a)/2-I)^-{aa}[[1]] \[Eta]$[aa][[2;;-1]][a]]+\[Eta]$[aa][a-2]};
shiftp\[Eta]b[a_]:={\[Eta]$b[aa__][a]:>(\[Eta]$[aa][-Conjugate[a]]/.shiftm\[Eta][-Conjugate[a]]/.mconj)};
shiftm\[Eta]b[a_]:={\[Eta]$b[aa__][a]:>(\[Eta]$[aa][-Conjugate[a]]/.shiftp\[Eta][-Conjugate[a]]/.mconj)};


sub\[Zeta]\[Eta]={\[Zeta]$[a__][0]:>I^Total[{a}]\[Eta]$[a][2]};
sub\[Eta]\[Zeta]={\[Eta]$[a__][2]:>I^-Total[{a}] \[Zeta]$[a][0],\[Eta]$b[a__][-2]:>I^+Total[{a}] \[Zeta]$[a][0]};


\[Eta]D[\[Eta]$[a__][0],0]:=\[Eta]$[a][0];
\[Eta]D[\[Eta]$[a__][0],r_:1]:=-Sum[{a}[[k]]\[Eta]D[\[Eta]$[Sequence@@ReplacePart[{a},k->{a}[[k]]+1]][0],r-1],{k,Length[{a}]}]//Expand
subtoPolyGammas={(xxx:(\[Eta]$[1,1]|\[Eta]$b[1,1]|\[Eta]$[1,1,1]|\[Eta]$b[1,1,1]))[r_:0]:>(xxx[r]/.subspecial\[Eta]/.subtoPolyGammas),
\[Eta]$[a_][r_:0]:>(I^a PolyGamma[a-1,-I (u+(I r)/2)])/Gamma[a],
\[Eta]$b[a_][r_:0]:>((-I)^a PolyGamma[a-1,I (u+(I r)/2)])/Gamma[a]};
subspecial\[Eta]={\[Eta]$[1,1][r_:0]:>1/2 (\[Eta]$[1][r]^2-\[Eta]$[2][r]),\[Eta]$[1,1,1][r_:0]:>1/6 (\[Eta]$[1][r]^3-3\[Eta]$[1][r]\[Eta]$[2][r]+2\[Eta]$[3][r]),\[Eta]$b[1,1][r_:0]:>1/2 (\[Eta]$b[1][r]^2-\[Eta]$b[2][r]),\[Eta]$b[1,1,1][r_:0]:>1/6 (\[Eta]$b[1][r]^3-3\[Eta]$b[1][r]\[Eta]$b[2][r]+2\[Eta]$b[3][r])};
Compute\[Eta][inp_,point_,prec_:MachinePrecision]:=Block[{},
(inp/.subspecial\[Eta]/.subtoPolyGammas/.u->point)/.{\[Eta]$[a_,b_][r_:0]:>NSum[(1/(u+(I r)/2)^a \[Eta]$[b][r+2]/.subtoPolyGammas/.u->point+I n),{n,0,\[Infinity]},WorkingPrecision->prec],\[Eta]$[a_,1,1][r_:0]:>NSum[(1/(u+(I r)/2)^a \[Eta]$[1,1][r+2]/.subspecial\[Eta]/.subtoPolyGammas/.u->point+I n),{n,0,\[Infinity]},WorkingPrecision->prec]}
/.{\[Eta]$b[a_,b_][r_:0]:>NSum[(1/(u+(I r)/2)^a \[Eta]$b[b][r+2]/.subtoPolyGammas/.u->point-I n),{n,0,\[Infinity]},WorkingPrecision->prec],\[Eta]$b[a_,1,1][r_:0]:>NSum[(1/(u+(I r)/2)^a \[Eta]$b[1,1][r+2]/.subspecial\[Eta]/.subtoPolyGammas/.u->point-I n),{n,0,\[Infinity]},WorkingPrecision->prec]}//N[#,prec]&
]
(* temporary replacement to track accuracy of intermediate computations *)
sub\[CapitalPsi]p={\[CapitalPsi]p[6/(Sqrt[3]+6 u)]->I PolyGamma[0,-I (I/2+1/(2 Sqrt[3])+u)],\[CapitalPsi]p[-(6/(Sqrt[3]-6 u))]->I PolyGamma[0,-I (I/2-1/(2 Sqrt[3])+u)],
\[CapitalPsi]p[36/(Sqrt[3]+6 u)^2]->-PolyGamma[1,-I (I/2+1/(2 Sqrt[3])+u)],
\[CapitalPsi]p[36/(Sqrt[3]-6 u)^2]->-PolyGamma[1,-I (I/2-1/(2 Sqrt[3])+u)]
};


subspecial = {\[Eta]$[a__ /; FreeQ[{a}, 1], b__ /; Union[{b}] == {1}][r_:0] :> Block[{tmp = {a, Sequence @@ {b}[[1 ;; -2]]}}, 
      (1/Length[{b}])*(\[Eta]$[1][r]*(\[Eta]$ @@ tmp)[r] - Total[(\[Eta]$[Sequence @@ #1][r] & ) /@ Transpose[DiagonalMatrix[ConstantArray[1, Length[tmp]]] + tmp]] - 
        Total[(\[Eta]$[Sequence @@ #1][r] & ) /@ (Insert[tmp, 1, #1] & ) /@ Range[Length[{a}]]])], 
    \[Eta]$[a__, b__][r_:0] /; Length[Union[{b}]] == 1 && {b}[[1]] < {a}[[-1]] :> Block[{tmp = {a, Sequence @@ {b}[[1 ;; -2]]}}, 
      (1/Length[{b}])*(\[Eta]$[{b}[[1]]][r]*(\[Eta]$ @@ tmp)[r] - Total[(\[Eta]$[Sequence @@ #1][r] & ) /@ 
          Transpose[DiagonalMatrix[ConstantArray[{b}[[1]], Length[tmp]]] + tmp]] - Total[(\[Eta]$[Sequence @@ #1][r] & ) /@ 
          (Insert[tmp, {b}[[1]], #1] & ) /@ Range[Length[{a}]]])]}; 
subtopowerbasis = Join[{\[Eta]$[a_, b_][r_:0] /; b < a :> \[Eta]$[a][r]*\[Eta]$[b][r] - \[Eta]$[a + b][r] - \[Eta]$[b, a][r], 
     \[Eta]$[1, b__][rr_:0] /; Union[{b}] == {1} :> 
      (Expand[With[{len = Length[{1, b}]}, (\[Eta]$[1][0]*(\[Eta]$[b][0] /. subtopowerbasis) + (D[\[Eta]$[b][0] /. subtopowerbasis /. \[Eta]$[r__][0] :> \[Eta]$[r][\[Alpha]], 
              \[Alpha]] /. Derivative[1][\[Eta]$[r__]][\[Alpha]] :> \[Eta]D[\[Eta]$[r][0]]) /. \[Alpha] -> 0)/len]] /. Shift[rr]), 
     \[Eta]$[a_ /; IntegerQ[a], b__][rr_:0] /; Length[Union[{a, b}]] == 1 :> With[{len = Length[{a, b}]}, 
       \[Eta]$[Sequence @@ ConstantArray[1, len]][0] /. subtopowerbasis /. \[Eta]$[r_][0] :> \[Eta]$[a*r][rr]], 
     \[Eta]$b[a__][r_:0] :> (\[Eta]$[a][-r] /. subtopowerbasis /. mconj)}, subspecial]; 
GenerateTuples[{a__}, {b__}, r_] := Total[((\[Eta]$ @@ #1)[r] & ) /@ Block[{tmp1 = {a}, l1 = Length[{a}], tmp2 = {b}, l2 = Length[{b}], tmp, tp, tup}, 
      tup = Select[Tuples[Range[0, l2], l1], OrderedQ]; Flatten[Table[tmp = (tmp1[[Flatten[Position[tup[[k]], #1]]]] & ) /@ Range[0, l2]; 
          tp = Transpose[{tmp2, tmp[[2 ;; -1]]}]; tp = (Which[Length[#1[[2]]] === 0, m[{#1[[1]]}, "AA"], Length[#1[[2]]] === 1, 
              m[{#1[[1]] + #1[[2,1]]}, {#1[[1]], #1[[2,1]]}], True, m[Join[{#1[[1]] + #1[[2,1]]}, #1[[2,2 ;; -1]]], Join[{#1[[1]]}, #1[[2]]]]] & ) /@ tp; 
          (Join[tmp[[1]], #1] & ) /@ Flatten /@ Select[Tuples[tp /. m -> List], FreeQ[#1, "AA"] & ], {k, Length[tup]}], 1]]]; 
subtoMHZbasis = {xxx:\[Eta]$b[a__][r_:0]*\[Eta]$b[b__][r_:0] :> (xxx /. mconj /. subtoMHZbasis /. mconj),
 xxx: \[Eta]$b[a__][r_:0]^(k_ /; IntegerQ[k] && k > 1) :> (xxx /. mconj /. subtoMHZbasis /. mconj),
    \[Eta]$[a_][r_:0]^2 :> 2*\[Eta]$[a, a][r] + \[Eta]$[2*a][r], \[Eta]$[a_][r_:0]*\[Eta]$[b_][r_:0] :> \[Eta]$[a, b][r] + \[Eta]$[b, a][r] + \[Eta]$[a + b][r], 
    \[Eta]$[a_][r_:0]*\[Eta]$[b__][r_:0] :> Block[{tmp = {b}}, Total[(\[Eta]$[Sequence @@ #1][r] & ) /@ Transpose[DiagonalMatrix[ConstantArray[a, Length[tmp]]] + tmp]] + 
       Total[(\[Eta]$[Sequence @@ #1][r] & ) /@ (Insert[tmp, a, #1] & ) /@ Range[Length[{b}] + 1]]], \[Eta]$[a__][r_:0]*\[Eta]$[b__][r_:0] :> GenerateTuples[{a}, {b}, r], 
    \[Eta]$[a__][r_:0]^(k_ /; IntegerQ[k] && k > 1) :> \[Eta]$[a][r]^(k - 2)*GenerateTuples[{a}, {a}, r]}; 
FuntoMHZbasis[expr_] := FixedPoint[Expand[Expand[#1] /. subtoMHZbasis] & , expr]; 


PossibleZetas[k_,r_:\[Infinity]]:=\[Zeta]$[Sequence@@#][0]&/@DeleteCases[Flatten[Permutations/@IntegerPartitions[k,r],1],{__,1}];


listdif[aa_List]:=aa-Join[{0},aa[[1;;-2]]];
listresum[aa_List]:=Total[aa[[1;;#]]]&/@Range[Length[aa]];
ClearAll[ZetaDuality];
ZetaDuality[aa_List]:=Block[{pos=Position[aa,Except[1],{1},Heads->False]//Flatten,ps,qs},{qs=Reverse@listdif[pos],ps=Reverse@Part[aa,pos]-1};
ReplacePart[ConstantArray[1,Total[ps]],Thread[Rule[listresum[ps],qs+1]]]
]/;Not[aa[[-1]]===1];
subZetaDuality={\[Zeta]$[a__][0]/;Not[{a}[[-1]]===1]:>(\[Zeta]$@@ZetaDuality[{a}])[0]};


(* ::Text:: *)
(*UniversalRelations*)


subEvenZetas = {\[Zeta]$[a_ /; EvenQ[a] && a > 2][0] :> 
     (Zeta[a] /. Pi^(r_)?EvenQ :> (6*\[Zeta]$[2][0])^(r/2)), 
    Pi^(r_)?EvenQ :> (6*\[Zeta]$[2][0])^(r/2)}; 
genericPatterns = {\[Zeta]$[1, a_ /; a > 1][0] :> 
     Expand[(1/2)*a*\[Zeta]$[1 + a][0] - (1/2)*Sum[\[Zeta]$[a - k][0]*\[Zeta]$[1 + k][0], 
         {k, a - 2}]], \[Zeta]$[a__, 2][0] /; Union[{a}] == {1} :> 
     \[Zeta]$[2 + Length[{a}]][0], \[Zeta]$[a__, 3][0] /; Union[{a}] == {1} :> 
     \[Zeta]$[1, 2 + Length[{a}]][0], \[Zeta]$[t_ /; OddQ[t] && t > 1, (s_)?EvenQ][0] :> 
     \[Zeta]$[s][0]*\[Zeta]$[t][0] + (1/2)*(Binomial[s + t, s] - 1)*\[Zeta]$[s + t][0] - 
      Sum[(Binomial[2*r, s - 1] + Binomial[2*r, t - 1])*(\[Zeta]$[1 + 2*r][0]*
         \[Zeta]$[-1 - 2*r + s + t][0]), {r, (1/2)*(s + t - 1) - 1}], 
    \[Zeta]$[(s_)?EvenQ, t_ /; OddQ[t] && t > 1][0] :> 
     (\[Zeta]$[s][0]*\[Zeta]$[t][0] - \[Zeta]$[s + t][0] - \[Zeta]$[t, s][0] /. subZetaReduce)}; 
symmetricZetas = {xxx:(\[Zeta]$[(a_)?IntegerQ, b__][0] /; Union[{a, b}] == {a}) :> 
     Expand[xxx /. sub\[Zeta]\[Eta] /. subtopowerbasis /. sub\[Eta]\[Zeta]], 
    \[Zeta]$[a__][0] /; (Union[Partition[{a}, 2]] && EvenQ[Length[{a}]]) === {{1, 3}} :> 
     (2*(6*\[Zeta]$[2][0])^((2/2)*Length[{a}]))/(2*Length[{a}] + 2)!}; 
dualityAboveEquator = {\[Zeta]$[a__][0] /; Length[{a}] > (Plus[a])/2 && Length[{a}] > 1 :> 
     (\[Zeta]$[a][0] /. subZetaDuality)}; 
subRemovingOnes = {\[Zeta]$[a__ /;  !{a}[[-1]] === 1, b__ /; Union[{b}] === {1}][0] :> 
     Expand[\[Zeta]$[a, b][0] /. sub\[Zeta]\[Eta] //. subtopowerbasis /. sub\[Eta]\[Zeta]]}; 
orderingZetas = {xxx:(\[Zeta]$[aa_, bb_][0] /; aa > bb) :> 
     Expand[xxx /. sub\[Zeta]\[Eta] /. subtopowerbasis /. sub\[Eta]\[Zeta]]}; 
(*orderingZetas is not in UniversalRelations because it interferes with FindGuys defined below*)


UniversalRelations=Join[subEvenZetas,subRemovingOnes,
genericPatterns,symmetricZetas,dualityAboveEquator];


FixedRelations = << "ZetaUpTo13.data"; 
Depth2Zetas=<<"Depth2ZetasUpTo36.data";


nonreducibleZetas = {\[Zeta]$[2, 6][0], \[Zeta]$[2, 8][0], \[Zeta]$[1, 2, 8][0], \[Zeta]$[2, 10][0], 
    \[Zeta]$[1, 1, 2, 8][0], \[Zeta]$[1, 2, 10][0], \[Zeta]$[1, 3, 9][0],\[Zeta]$[2,12][0],\[Zeta]$[4,10][0]};
nonreducibleZetaPatterns ={
\[Zeta]$[2,PatternTest[_,(EvenQ[#]&&(#>= 6))&]][0],
\[Zeta]$[4,PatternTest[_,(EvenQ[#]&&(#>= 10))&]][0],
\[Zeta]$[6,PatternTest[_,(EvenQ[#]&&(#>= 14))&]][0],
\[Zeta]$[8,PatternTest[_,(EvenQ[#]&&(#>= 18))&]][0],
\[Zeta]$[10,PatternTest[_,(EvenQ[#]&&(#>= 22))&]][0],
\[Zeta]$[12,PatternTest[_,(EvenQ[#]&&(#>= 26))&]][0],
\[Zeta]$[1,2,PatternTest[_,(EvenQ[#]&&(#>= 8))&]][0],
\[Zeta]$[1,3,PatternTest[_,(OddQ[#]&&(#>= 9))&]][0],
\[Zeta]$[1,4,PatternTest[_,(EvenQ[#]&&(#>= 12))&]][0],
\[Zeta]$[2,4,PatternTest[_,(OddQ[#]&&(#>= 11))&]][0],
\[Zeta]$[1,1,2,PatternTest[_,(EvenQ[#]&&(#>= 8))&]][0]};


Nnnreducibles = Apply[Rule, Transpose[{{\[Zeta]$[2, 6][0], \[Zeta]$[2, 8][0], \[Zeta]$[1, 2, 8][0], \[Zeta]$[2, 10][0], \[Zeta]$[1, 1, 2, 8][0], 
 \[Zeta]$[1, 2, 10][0], \[Zeta]$[1, 3, 9][0]}, 
      SetPrecision[{0.0178197404168359883626595302487246121687131371102911884188213619\
17613480276416046371828621019205879420000000000000000000000000000000001`99.25090137330\
598, 0.0041224696783998322240469568386942088558126273584685692852455179287171112774063\
88331275945345243417388`99.61515747000436, 0.00004638308072501997415281994157608276321\
75582978541529471355717992369229319062597060880745638843351497`98.6663595907324, 0.000\
99920678720969184043380148821583760914101923281940968488222068567218532718591256303503\
6540127775898599999999999999999999999999999999`99.99965537536379, 1.325925456599534061\
912182636672112259858120335602205094853620810513756069541252368120831486047305821`99.1\
2251910876144*^-6, 4.69913803744366544719901171902359883853189896561268094160742444302\
9775185866233937741108502629661955000000000000000000000000000001`99.67201820263512*^-6\
, 7.1777264214071801976649108949075447821800480586139462103654216628128238101324713739\
25337911513644451`99.85598690120571*^-6}, 99]}], {1}]; 


subZetaReduce=Dispatch@Join[FixedRelations,Depth2Zetas,UniversalRelations];


(* ::Text:: *)
(*Below is a code which allow to generate additional relations between Zeta-functions if needed.*)


(* ::Text:: *)
(*Fixed Relations (precomputed all burden up to  \[Zeta]-s of weight 13)*)


(* ::Text:: *)
(*Harmonic and Shuffle relations*)


subShuffle = {\[Zeta]$[a_][0]*\[Zeta]$[b_][0] :> 
     Sum[Binomial[b + a - k - 1, b - 1]*\[Zeta]$[k, b + a - k][0], 
       {k, a}] + Sum[Binomial[b + a - k - 1, a - 1]*
        \[Zeta]$[k, b + a - k][0], {k, b}], \[Zeta]$[a_ /; a > 1][0]^2 :> 
     Sum[Binomial[2*a - k - 1, a - 1]*\[Zeta]$[k, 2*a - k][0], 
       {k, a}] + Sum[Binomial[2*a - k - 1, a - 1]*
        \[Zeta]$[k, 2*a - k][0], {k, a}], 
    \[Zeta]$[b__ /;  !{b}[[-1]] === 1][0]*\[Zeta]$[a_][0] :> 
     Expand[Block[{aa, tmp}, tmp = aa /@ Range[Length[{b}] + 1]; 
         Collect[Expand[Total[Times @@ tmp*(Times @@ (#1^({a, b} - 
                   1)) & ) /@ Join[{ReplacePart[tmp, 1 -> 
                   Total[tmp]]}, (Join[{Total[tmp[[#1 ;; -1]]]}, 
                   tmp[[1 ;; #1 - 2]], {tmp[[#1 - 1]] + tmp[[#1]]}, 
                   tmp[[#1 + 1 ;; -1]]] & ) /@ Range[2, 
                  Length[{b}] + 1]]]], Times @@ 
            (tmp /. aa[i_] :> aa[i]^(_.))] /. 
          xxx:Product[aa[i]^(_.), {i, Length[tmp]}] :> 
           (Times @@ Gamma /@ #1*\[Zeta]$[Sequence @@ #1][0] & )[
            List @@ xxx /. aa[_]^(bb_.) :> bb]]/
       (Gamma[a]*Times @@ Gamma /@ {b})]}; 


subHarmonic = {\[Zeta]$[a_][0]^2 :> 2*\[Zeta]$[a, a][0] + \[Zeta]$[2*a][0], 
    \[Zeta]$[a_][0]*\[Zeta]$[b_][0] :> \[Zeta]$[a, b][0] + \[Zeta]$[b, a][0] + \[Zeta]$[a + b][0], 
    \[Zeta]$[a_][0]*\[Zeta]$[b__][0] :> Block[{tmp = {b}}, 
      Total[(\[Zeta]$[Sequence @@ #1][0] & ) /@ Transpose[
          DiagonalMatrix[ConstantArray[a, Length[tmp]]] + tmp]] + 
       Total[(\[Zeta]$[Sequence @@ #1][0] & ) /@ (Insert[tmp, a, #1] & ) /@ 
          Range[Length[{b}] + 1]]], \[Zeta]$[a__][0]*\[Zeta]$[b__][0] :> 
     GenerateTuples[{a}, {b}], \[Zeta]$[a__][0]^(k_ /; IntegerQ[k] && k > 1) :> 
     \[Zeta]$[a][0]^(k - 2)*GenerateTuples[{a}, {a}]}; 
FunHarmonic[expr_] := FixedPoint[Expand[Expand[#1] /. subHarmonic] & , expr];
subStuffle=subHarmonic;
FunStuffle=FunHarmonic; 


GenerateTuples[{a__}, {b__}] := Total[((\[Zeta]$ @@ #1)[0] & ) /@ 
     Block[{m, tmp1 = {a}, l1 = Length[{a}], tmp2 = {b}, 
       l2 = Length[{b}], tmp, tp, tup}, 
      tup = Select[Tuples[Range[0, l2], l1], OrderedQ]; 
       Flatten[Table[tmp = (tmp1[[Flatten[Position[tup[[k]], 
                #1]]]] & ) /@ Range[0, l2]; 
          tp = Transpose[{tmp2, tmp[[2 ;; -1]]}]; 
          tp = (Which[Length[#1[[2]]] === 0, m[{#1[[1]]}, "AA"], 
              Length[#1[[2]]] === 1, m[{#1[[1]] + #1[[2,1]]}, {
                #1[[1]], #1[[2,1]]}], True, m[Join[{#1[[1]] + 
                  #1[[2,1]]}, #1[[2,2 ;; -1]]], Join[{#1[[1]]}, 
                #1[[2]]]]] & ) /@ tp; (Join[tmp[[1]], #1] & ) /@ 
           Flatten /@ Select[Tuples[tp /. m -> List], 
             FreeQ[#1, "AA"] & ], {k, Length[tup]}], 1]]]; 


(* ::Text:: *)
(*Getting new relations*)


ClearAll[FindGuys];
FindGuys[guys_List, addrules_:{}] := Block[{Zetas,tmp,tmp2, \[Zeta]1s, vars, eqn1, eqn2, eqns, iii, sol, soltmp, mrules,mvars,ii,ee,tmp3},
Off[Solve::svars]; 
   NotebookDelete[tmp]; tmp = PrintTemporary["solving \!\(\*SubscriptBox[\(\[Zeta]\), \(1, __\)]\)"];
	Zetas=Cases[guys[[Sequence@@#]]&/@Position[guys-(guys/.addrules/.subZetaReduce)//Expand,0,{1}],Except[Alternatives @@ nonreducibleZetaPatterns]];
    \[Zeta]1s = Solve[(Cases[Zetas,  \[Zeta]$[1, __][0]] /. \[Zeta]$[1, a__][0] :> (\[Zeta]$[1][0]*\[Zeta]$[a][0] /. subHarmonic) - 
           (\[Zeta]$[1][0]*\[Zeta]$[a][0] /. subShuffle)) == 0, Cases[Zetas,  \[Zeta]$[1, __][0]]][[1]]; 
    vars = Cases[Zetas, Except[\[Zeta]$[1, __][0]]]; NotebookDelete[tmp]; 
    tmp = PrintTemporary["Generating equations. iii will run twice from 1 to "<>ToString[Length[vars]]]; 
	eqn1 = Monitor[Table[vars[[iii]] /. \[Zeta]$[a_, b__][0] :> (\[Zeta]$[a][0]*\[Zeta]$[b][0] /. subHarmonic) - \[Zeta]$[a][0]*\[Zeta]$[b][0],{iii,Length[vars]}],iii]; 
    eqn2 = Monitor[Table[vars[[iii]] /. \[Zeta]$[a_, b__][0] :> (\[Zeta]$[a][0]*\[Zeta]$[b][0] /. subShuffle) - \[Zeta]$[a][0]*\[Zeta]$[b][0],{iii,Length[vars]}],iii]; 
    mrules = Dispatch[Flatten[{\[Zeta]1s, subZetaReduce[[1]], addrules}]];
ii=1;
	NotebookDelete[tmp];tmp = PrintTemporary["Reducing known zeta-functions in equations. iii will run from 1 to "<>ToString[2 Length[vars]]]; 
    eqns = Monitor[Map[(ii++;#//.mrules//Expand)&,Join[eqn1,eqn2]],ii];
NotebookDelete[tmp];
	NotebookDelete[tmp]; 
    tmp = PrintTemporary["solving equations"]; sol = {}; tmp2=PrintTemporary["iii will change from 1"*StringJoin[" to ", ToString[Length[eqns]]]]; 
    Monitor[Do[mvars=Cases[vars, Except[Alternatives @@ sol[[All,1]]]];(*Print[mvars];*)
soltmp = Select[Solve[Expand[eqns[[iii]] /. sol //. mrules] == 0, mvars][[1]],MemberQ[mvars,#[[1]]]&]; 
       sol = Join[Expand[sol /. soltmp], soltmp], {iii, Length[eqns]}], {"iii", iii}]; 
On[Solve::svars];
NotebookDelete[tmp];NotebookDelete[tmp2];
 Join[\[Zeta]1s/.sol/.addrules//.subZetaReduce//Expand, sol]]


(* ::Subsection::Closed:: *)
(*Taylor expansion*)


(* ::Text:: *)
(*Both TaylorExpand and TaylorOptions are substitutions which compute Taylor expensions.*)
(*Their Options (order of the developpement, name of the varioable, position arround which Taylo expension is done, etc) are given by Options[Taylor]. The Substitution TaylorOptions can additionaly take options as its parameters*)


PositiveIntegerQ[aA_]:=Block[{a=sqrt3Simp[aA]},((Im@a == 0)&&(a==Round[a])&&(Re[a]>=0))];

Options[Taylor]={Order->3,Point->0,Variable->u,\[Eta]Factor->1,Series->Keep};

TaylorExpand:=With[{X=OptionValue[Taylor,Variable]},{e_:>(If[OptionValue[Taylor,Series]===Keep,Identity,Normal]@Series[
e/.\[Eta]$[a__][b_]:>\[Eta]$[a][X OptionValue[Taylor,\[Eta]Factor],b]/.\[Eta]$b[a__][b_]:>\[Eta]$b[a][X OptionValue[Taylor,\[Eta]Factor],b]//.{
\[Eta]$[a__][b_,c_]/;PositiveIntegerQ[((b/.X->OptionValue[Taylor,Point])+I c/2)/-I]:>(\[Eta]$[a][c]/.shiftp\[Eta][c]/.u->(b)/.\[Eta]$[x__][y_]->\[Eta]$[x][b,y]),
\[Eta]$b[a__][b_,c_]/;PositiveIntegerQ[((b/.X->OptionValue[Taylor,Point])+I c/2)/I]:>(\[Eta]$[a][-c]/.shiftp\[Eta][-c]/.Complex[x_,y_]:>Complex[x,-y]/.u->(b)/.\[Eta]$[AA__][BB_]:>\[Eta]$b[AA][b,-BB])
},{X,OptionValue[Taylor,Point],OptionValue[Taylor,Order]}]//.{
PP_[a__][OptionValue[Taylor,Point] OptionValue[Taylor,\[Eta]Factor],b_]/;MemberQ[{\[Eta]$,\[Eta]$b},PP]:>PP[a][(OptionValue[Taylor,Point]OptionValue[Taylor,\[Eta]Factor])/(I/2)+b],
Derivative[r_,0][\[Eta]$[a__]][A_,b_]:>(\[Eta]D[\[Eta]$[a][0],r]/.\[Eta]$[x__][0]:>\[Eta]$[x][A,b]),
Derivative[r_,0][\[Eta]$b[a__]][A_,b_]:>(\[Eta]D[\[Eta]$[a][0],r]/.\[Eta]$[x__][0]:>\[Eta]$b[x][A,b])}/.sub\[Eta]\[Zeta])}]

TaylorOptions[options__]:={e_:>Block[{MTaylor},Options[MTaylor]=Options[Taylor];SetOptions[MTaylor,options];
Block[{Taylor=MTaylor},e/.TaylorExpand]]};


(*below tries to generalize to computing residues .. *)
(*The only options which affect Residu are "Variable" and "Point", and that can be given under the form "/.ResiduPt[Var,Pt]"*)


Residu:=With[{X=OptionValue[Taylor,Variable]},{e_:>(Residue[
e/.\[Eta]$[a__][b_]:>\[Eta]$[a][X,b]/.\[Eta]$b[a__][b_]:>\[Eta]$b[a][X,b]//.{
\[Eta]$[a__][b_,c_]/;PositiveIntegerQ[((b/.X->OptionValue[Taylor,Point])+I c/2)/-I]:>(\[Eta]$[a][c]/.shiftp\[Eta][c]/.u->(b)/.\[Eta]$[x__][y_]->\[Eta]$[x][b,y]),
\[Eta]$b[a__][b_,c_]/;PositiveIntegerQ[((b/.X->OptionValue[Taylor,Point])+I c/2)/I]:>(\[Eta]$[a][-c]/.shiftp\[Eta][-c]/.Complex[x_,y_]:>Complex[x,-y]/.u->(b)/.\[Eta]$[AA__][BB_]:>\[Eta]$b[AA][b,-BB])
},{X,OptionValue[Taylor,Point]}]//.{
PP_[a__][OptionValue[Taylor,Point],b_]/;MemberQ[{\[Eta]$,\[Eta]$b},PP]:>PP[a][(OptionValue[Taylor,Point])/(I/2)+b],
Derivative[r_,0][\[Eta]$[a__]][A_,b_]:>(\[Eta]D[\[Eta]$[a][0],r]/.\[Eta]$[x__][0]:>\[Eta]$[x][A,b]),
Derivative[r_,0][\[Eta]$b[a__]][A_,b_]:>(\[Eta]D[\[Eta]$[a][0],r]/.\[Eta]$[x__][0]:>\[Eta]$b[x][A,b])})}];

ResiduePt[Var_,Pt_]:={e_:>Block[{MTaylor},Options[MTaylor]=Options[Taylor];SetOptions[MTaylor,
Variable->Var,Point->Pt];
Block[{Taylor=MTaylor},e/.Residu]]};



(* ::Subsubsection::Closed:: *)
(*examples*)


(* ::Text:: *)
(*Expand u+\[Eta]$[3][0] at u=Subscript[v, 0], to the order g^4*)


(* ::Input:: *)
(*u+\[Eta]$[3][0]/.Taylorv0[4]*)


(* ::Text:: *)
(*Expand u+\[Eta]$[3][0] at u=0, to the order u^2*)


(* ::Input:: *)
(*u+\[Eta]$[3][0]/.TaylorOptions[Order->2]*)


(* ::Text:: *)
(*Expand u+\[Eta]$[3][0] at u=g z, to the order g^2*)


(* ::Input:: *)
(*g z +\[Eta]$[3][0]/.TaylorOptions[Variable->g,\[Eta]Factor->z,Order->2]*)


(* ::Subsection::Closed:: *)
(*Replacing \[Eta]bar with \[Eta]-s (for two-indexed only)*)


(* ::Text:: *)
(*Replacing \[Eta]bar with \[Eta]-s and Subscript[\[Eta]bar, 1] using periodicity properties*)


rep\[Eta]b = {\[Eta]$b[a_, b_][r_] :> With[{}, \[Eta]$[a, b][1] + (\[Eta]$[b][1] + \[Eta]$b[b][-1])*\[Eta]$b[a][-1] - \[Eta]$b[a + b][-1] - 
        (AA[0]+Sum[AA[i]*(\[Eta]$[i][1] + \[Eta]$b[i][-1]), {i, 1, a + b}]) /. 
       Simplify[Solve[(List @@ Collect[(\[Eta]$[a, b][1] + (\[Eta]$[b][1] + \[Eta]$b[b][-1])*\[Eta]$b[a][-1] - \[Eta]$b[a + b][-1] - \[Eta]$b[a, b][-1] - 
                   (AA[0]+Sum[AA[i]*(\[Eta]$[i][1] + \[Eta]$b[i][-1]), {i, 1, a + b}]) /. Shift[-1])/u /. TaylorOptions[Order -> -1] //. subZetaReduce, u] /. u -> 1) == 0, 
          AA /@ Range[0, a + b]][[1]]] /. Shift[r + 1]]};
rep\[Eta]b\[Eta]1={\[Eta]$b[a_][r_] :> With[{}, -\[Eta]$[a][1] + Sum[AA[i]*(\[Eta]$[1][1] + \[Eta]$b[1][-1])^i, {i, 0, a}] /. 
       Simplify[Solve[(List @@ Collect[(\[Eta]$[a][1] + \[Eta]$b[a][-1] - Sum[AA[i]*(\[Eta]$[1][1] + \[Eta]$b[1][-1])^i, {i, 0, a}] /. Shift[-1])/u /. 
                TaylorOptions[Order -> -1] //. subZetaReduce, u] /. u -> 1) == 0, AA /@ Range[0, a]][[1]]] /. Shift[r + 1]]};


Module[{tmp,per,maxpower=10}, tmp = Table[\[Eta]$[a][1] + \[Eta]$b[a][-1], {a, maxpower}]; 
  subdeperiod = Solve[tmp == (tmp /. rep\[Eta]b\[Eta]1 /. (\[Eta]$[1][1] + \[Eta]$b[1][-1])^(k_.) -> 
    per[k]), per /@ Range[maxpower]][[1]] /. per[k_] -> (\[Eta]$[1][1] + \[Eta]$b[1][-1])^k]; 
fundeperiod[expr_]:=Module[{per},Collect[Collect[expr/.\[Eta]0$b[1][-1]->(per-\[Eta]0$[1][1]),per]/.per^a_.:>((\[Eta]$[1][1] + \[Eta]$b[1][-1])^a/.subdeperiod/.sub\[Eta]\[Eta]0),canlist]];


(* ::Subsection::Closed:: *)
(*Working with \[ScriptU]-function*)


v00=1/Sqrt[12](* for Konishi*);


subusu={\[ScriptU]$[var_][sh_:0]:>(u+var+I sh/2/.v0->v00)};
subtousu={u^(a_/;a<0):>\[ScriptU]$[0][0]^a,(ee_. u+b_.)^(a_/;a<0):>ee^a  \[ScriptU]$[Re[b/ee]/.{v00->v0,-v00->-v0}][2 Im[b/ee]]^a,(Q0$[r_:0])^(a_/;a<0):>(\[ScriptU]$[v0][r]\[ScriptU]$[-v0][r])^a};


subapartusu = {\[ScriptU]$[var1_][sh1_]^(pow1_ /; pow1 < 0)*\[ScriptU]$[var2_][sh2_]^
       (pow2_ /; pow2 < 0) :> 
     With[{odif = Collect[1/(var2 + I*(sh2/2) - var1 - I*(sh1/2) /. v0 -> v00) /. 
          densqrt3, Sqrt[3]]}, 
      Plus @@ (\[ScriptU]$[var1][sh1]^(pow1 + #1)*Collect[(-1)^#1*odif^(#1 - pow2)*
             (Pochhammer[-pow2, #1]/#1!), Sqrt[3]] & ) /@ Range[0, -pow1 - 1] + 
       Plus @@ (\[ScriptU]$[var2][sh2]^(pow2 + #1)*Collect[(-1)^pow1*odif^(#1 - pow1)*
             (Pochhammer[-pow1, #1]/#1!), Sqrt[3]] & ) /@ Range[0, -pow2 - 1]]}; 
funapartusu[expr_]:= FixedPoint[(ExpaCol[#,\[ScriptU]$[_][_]]/.subapartusu)&,expr];


subdepu=\[ScriptU]$[var1_][sh1_]:>\[ScriptU]$[var1][u,sh1];
subindepu=\[ScriptU]$[var1_][u,sh1_]:>\[ScriptU]$[var1][sh1];
\[ScriptU]$[var1_][a_?NumericQ,sh1_]:=a+var1+I/2 sh1/.v0->v00;
Derivative[1,0][\[ScriptU]$[_]]=1&;
Derivative[a_/;a>1,0][\[ScriptU]$[_]]=0&;


subreduceusu = {u^(a_ /; a > 0)*\[ScriptU]$[var_][sh_]^(b_ /; b < 0) :> 
     Block[{blah}, Collect[Collect[(blah - (var /. v0 -> v00) - (I*sh)/2)^a*blah^b, 
          blah] /. blah^(pow_ /; pow < 0) :> \[ScriptU]$[var][sh]^pow /. 
        blah :> u + (var /. v0 -> v00) + (I*sh)/2, {u, \[ScriptU]$[var][sh], Sqrt[3]}]],
u*\[ScriptU]$[var_][sh_]^(b_ /; b < 0) :>Block[{blah,a=1}, Collect[Collect[(blah - (var /. v0 -> v00) - (I*sh)/2)^a*blah^b, 
          blah] /. blah^(pow_ /; pow < 0) :> \[ScriptU]$[var][sh]^pow /. 
        blah :> u + (var /. v0 -> v00) + (I*sh)/2, {\[ScriptU]$[_][_], Sqrt[3]}]]}; 
funreduceusu[expr_]:= FixedPoint[(Expand[#]/.subreduceusu)&,expr];


canlist={g,\[Eta]list,\[ScriptU]$[_][_],\[Eta]0list,Sqrt[3],\[Zeta]list}//Flatten;
zcanlist={zlist,\[Eta]0list,\[Zeta]list}//Flatten;


(* ::Subsection::Closed:: *)
(*Bold compute : compute \[Eta]-functions at a specific point*)


EMreplint=Function[\[CapitalLambda],t^aa_:>(Sum[-((I^n BernoulliB[n+1])/Gamma[n+2]) (-1)^n Gamma[n-aa]/Gamma[-aa] 1/t^(-aa+n),{n,-1,\[CapitalLambda]+aa/.\[Epsilon]->0}]-If[(aa/.\[Epsilon]->0)==-1,(I \[Mu]^(1+aa))/(1+aa),0]-If[(\[CapitalLambda]+aa/.\[Epsilon]->0)<0,0,1/t^-aa])];
EMreplout=Function[\[CapitalLambda],t^aa_:>(Sum[-((I^n BernoulliB[n+1])/Gamma[n+2]) (-1)^n Gamma[n-aa]/Gamma[-aa] 1/t^(-aa+n),{n,-1,\[CapitalLambda]+aa/.\[Epsilon]->0}]-If[(aa/.\[Epsilon]->0)==-1,(I \[Mu]^(1+aa))/(1+aa),0])];


FindPrecu=Compile[{{prec,_Real},{apb,_Integer}},Block[{tmp=1/(2 \[Pi]) prec/Log[10,E]+1/(2 \[Pi]) ((apb-1/2) Log[(2 \[Pi])/apb]-Log[2 \[Pi]]+(apb))},tmp-1/(4 \[Pi]) Log[tmp]]];
MyEM[uu_,a_,b_,prec_]:=Block[{uneed,nn},If[Abs[uu]<(uneed=FindPrecu[prec,a+b]),(*Print[Floor[\[Pi] uneed+(3/4-(a+b)/2)]];*)nn=Ceiling[Sqrt[uneed^2-Re[uu]^2]-Im[uu]],nn=0];
Sum[1/(uu+I k)^a \[Eta]$[b][2+2*k]/.subtoPolyGammas/.u->uu//Evaluate,{k,0,nn-1}]+Mycompute[a,b,uu+I nn,Floor[\[Pi] uneed+(3/4-(a+b)/2)]]//N[#,{\[Infinity],prec}]&]
Mycompute[a_, b_, u_, \[CapitalLambda]_] := 
 Block[{}, -(1/4) 1/u^(a + b) + 
   Sum[(-1)^r/u^(a + b + 2 r) Gamma[a + b + 2 r]/
     Gamma[b] Sum[
       BernoulliB[2 r - 2 k + 2]/Gamma[2 r - 2 k + 3] BernoulliB[2 k]/
       Gamma[2 k + 1] Gamma[b + 2 k - 1]/Gamma[a + b + 2 k - 1], {k, 
       0, r + 1}], {r, -1, \[CapitalLambda]}] +
   +Sum[1/u^(a + b + 2 r - 1) 1/(2 I) ((-1)^r BernoulliB[2 r])/
      Gamma[2 r + 1] (Gamma[b + 2 r - 1]/Gamma[b] - 
        Gamma[a + b + 2 r - 1]/Gamma[a + b]), {r, 
      0, \[CapitalLambda]}]]


ClearAll[Generic\[Eta],Generic\[Eta]List];
Generic\[Eta][aa_List,\[CapitalLambda]_]:=Generic\[Eta][aa,\[CapitalLambda]]=Block[{tmp=1,\[Epsilon]depen=(aa[[-1]]==1)},Do[If[\[Epsilon]depen,tmp=(1/t^(1+\[Epsilon]) tmp//Expand)/.EMreplint[\[CapitalLambda]]//FunctionExpand,tmp=(1/t^aa[[-k]] tmp//Expand)/.EMreplint[\[CapitalLambda]]//FunctionExpand];\[Epsilon]depen=\[Epsilon]depen&&(aa[[-k-1]]==1),{k,1,Length[aa]-1}];
If[\[Epsilon]depen,tmp=(1/t^(1+\[Epsilon]) tmp//Expand)/.EMreplout[\[CapitalLambda]]//FunctionExpand,tmp=(1/t^aa[[1]] tmp//Expand)/.EMreplout[\[CapitalLambda]]//FunctionExpand];
Collect[tmp,t,(Series[#,{\[Epsilon],0,0}]//Normal)/.Log[\[Mu]]->0/.Log[t]->Log[-I t]&]]


Generic\[Eta][{a_,b_},n_]/;b>=2:=Normal@Series[TwoIndPartSum[a,b,t,Ceiling[n/2-2]],{t,\[Infinity],n}]


TwoIndPartSum[a_, b_, u_, \[CapitalLambda]_] (*Partialsum for two indices*):= Block[{}, -(1/4) 1/u^(a + b) + 
  Sum[(-1)^r/u^(a + b + 2 r) Gamma[a + b + 2 r]/
    Gamma[b] Sum[
      BernoulliB[2 r - 2 k + 2]/Gamma[2 r - 2 k + 3] BernoulliB[2 k]/
      Gamma[2 k + 1] Gamma[b + 2 k - 1]/Gamma[a + b + 2 k - 1], {k, 0,
       r + 1}], {r, -1, \[CapitalLambda]}] +
  +Sum[1/u^(a + b + 2 r - 1) 1/(2 I) ((-1)^r BernoulliB[2 r])/
     Gamma[2 r + 1] (Gamma[b + 2 r - 1]/Gamma[b] - 
       Gamma[a + b + 2 r - 1]/Gamma[a + b]), {r, 
     0, \[CapitalLambda]}]]


Generic\[Eta]List[II_List,m_,shift_:False]:=Block[{n=m+Boole[shift] Total[II-1]},Reverse@((CoefficientList[t^n Generic\[Eta][II,n]/.Log[a_]:>Log[a/.t->u],t]//#*t^Range[0,Length@#-1]&)/t^n)/.u->t];


EtaShift[III_List,prec_,n_:10]:=EtaShift[III,prec,n]=Last@Last@FindRoot[Log[10,Abs[Last@Generic\[Eta]List[III,n+Total[III-1]]]]==(-prec-2),{t,1}];


TryEta[III_List,uu_,prec_,n_:10]/;Length@III<=2  && Last@III>=2:=BoldCompute[\[Eta]$[Sequence@@III][0], uu, prec];


TryEta[III_List,uu_,prec_,n_:10]:=((*Print[{III,uu,prec,n}];*)
TryEta[III,uu,prec,n]=If[Abs[uu]>EtaShift[III,prec,n],
Total[N[Generic\[Eta]List[III,n+Total[III-1]]/.t->uu,{\[Infinity],prec}]],
\[Eta]$[Sequence@@III][0]/.shiftp\[Eta][0]/.u->uu/.\[Eta]$[J___][2]:>TryEta[{J},uu+I,prec,n]])


Module[{f\[Alpha]02,p\[Alpha]02,r\[Alpha]02},
BoldCompute[e_, uu_: 0, prec_: 20] := 
 e /. {A : \[Eta]$[n_Integer][sh_/;Im@sh==0] :> 
    N[(A /. subtoPolyGammas /. u -> uu), {\[Infinity], prec}],
\[Eta]$[0,2][n_?NumericQ]:>BoldCompute[I (u+ I n/2) \[Eta]$[2][n]-I \[Eta]$[1][n]/.u->uu,uu,prec],
   A : \[Eta]$[a_Integer, b_Integer][sh_]/;b>=2 :> MyEM[uu + I/2 sh, a, b, prec],
A: \[Eta]$[III__][sh_] :> TryEta[{III},uu+I/2 sh,prec],
A : \[Eta]$[n_Integer][sh_] :> BoldCompute[\[Eta]$[n][Re@sh],uu+-1/2 Im@sh,prec],
A : \[Eta]$b[n__][sh_]:>Conjugate[BoldCompute[\[Eta]$[n][-Conjugate@sh],Conjugate@uu,prec]],
A : \[Eta]0$[a__][sh_]:>BoldCompute[\[Eta]$[a][sh],v00,prec],
A : \[Eta]0$b[a__][sh_]:>BoldCompute[\[Eta]$b[a][sh],v00,prec],
A : \[Zeta]$[a__][sh_]:>BoldCompute[A/.sub\[Zeta]\[Eta],0,prec],
A : Zeta[n_]:>N[Zeta[n],{\[Infinity],prec}],
A : \[Eta]$[a___][uuu_,sh_]:>BoldCompute[\[Eta]$[a][sh],uuu,prec],
A : \[Eta]$b[a___][uuu_,sh_]:>BoldCompute[\[Eta]$b[a][sh],uuu,prec],
\[Alpha]02:>f\[Alpha]02[prec]
}]


(* ::Subsection::Closed:: *)
(*EMFun : function version of the \[Eta]-functions*)


(* ::Text:: *)
(*VirtualPrec (below) is a stupidity to set the precision of numbersction version of Bold compute : returns an analytic function, suitable for integrals*)


Module[{A},
VirtualPrec[str_String][e_]:=e/.{a_Complex:>VirtualPrec[str][Evaluate@Re@a]+I VirtualPrec[str][Evaluate@Im@a],
a_Real:>(A=StringSplit[ToString[a,InputForm],"`"];ToExpression[A[[1]]<>"`"<>ToString[ToExpression[Last@A<>str]]])}];


VirtualPrec[n_Integer]:=VirtualPrec["+"<>ToString[n]]


EMparams[II_List,prec_?NumericQ]:=EMparams[II,prec]=Block[{GfD(*GuessFromDima*),Devpmt,U,n,k=2,t},
GfD[apb_,pr_]:=Block[{a},{a=FindPrecu[pr,apb],2+2Floor[\[Pi] a + (3/4 - (apb)/2)]}];
While[{U,n}=GfD[Total@II,prec+k];(Abs@Last@Generic\[Eta]List[II,n]/.t->U)>10^(-prec-2),k=k+1];
{U,n}];


Options[EMFun]={Prec->20,
PolyGamma->"Approx",(*This option means that for single-indexed \[Eta]-functions, I should not return polygamma but an approximation of it*)
Shift->0,
MaxForThree->10,
Sequence[]};


(* ::Text:: *)
(*EMFun[{I},Shift->s,Prec->p] returns a function which approximates *)
(*\!\(\*SubsuperscriptBox[\(\[Eta]\), \(I\), \(\([\)\(+s\)\(]\)\)]\) to a precision p*)


EMFun[{0,2},o:OptionsPattern[]]:=With[{sh=OptionValue@Shift},(-I (u+I/2 sh) EMFun[{2},o][#]-I EMFun[{1},o][#])&];


EMFun[II_List/;(1<=Length[II](*<=2*)),o:OptionsPattern[]]:=EMFun[II,o]=Block[{S1,S2,U,n,G\[Eta]f,G\[Eta]L,MN,prec,shift,s,PG,t,u},
shift=OptionValue@Shift;PG=OptionValue@PolyGamma;
Which[Length@II==1 && PG=!="Approx",
S1=Function@@{(\[Eta]$@@II)[shift]/.subtoPolyGammas/.u->#};
S1 (* I've done the trivial case *)
,True,(*let's do the harder one (when a sum has to be performed)*)
prec=OptionValue@Prec;
If[Length@II<=2 && (Last@II)>=2,{U,n}=EMparams[II,prec],U=EtaShift[II,prec,n=OptionValue@MaxForThree]];
U=Max[Ceiling[U-shift/2],0];
Switch[II,{_},S1=Sum[1/(t+I k)^(II[[1]]),{k,0,U-1}]/.t->u+I (shift/2),
{_,__},S1=Sum[
1/(t+I k)^(II[[1]])EMFun[II[[2;;]],Shift->2k+2,
   Prec->If[k==0,prec,(prec+N@Log[10,1/(k)^(II[[1]])])],PolyGamma->PG,MaxForThree->n][t]
,{k,shift/2,U-1+shift/2}]/.t->u
];
G\[Eta]f[0]=0;
MN[p_][{XX_?NumericQ}]:=N[XX,{\[Infinity],p}]//If[#==0 || Precision@#<1,0,#]&;
MN[p_][XX_,YY_]:=MN[p]/@{{XX},{YY}};
MN[p_][XX_?NumericQ]:=Total[{1,I} MN[p][Re[XX],Im[XX]]];
G\[Eta]f[e_]:=Block[{x,y},
x=Select[e,FreeQ[#,t]&];y=e/x;
(MN[prec+Re@N@Log[10,y/.t->U+shift/2]+3][x])y
];
S2=(*Sum from generic\[Eta]*)Total[(G\[Eta]f/@Generic\[Eta]List[II,n])/.t->u+I (U+shift/2)];
(Function@@{Collect[S1+S2/.u->#,(#+_)^_,VirtualPrec["//Max[#,3]&"]]})]];


Generic\[Eta][{a_,b_},n_]/;a>=2 && b>=2:=Normal@Series[TwoIndPartSum[a,b,t,Ceiling[n/2-2]],{t,\[Infinity],n}]


TwoIndPartSum[a_, b_, u_, \[CapitalLambda]_] (*Partialsum for two indices*):= Block[{}, -(1/4) 1/u^(a + b) + 
  Sum[(-1)^r/u^(a + b + 2 r) Gamma[a + b + 2 r]/
    Gamma[b] Sum[
      BernoulliB[2 r - 2 k + 2]/Gamma[2 r - 2 k + 3] BernoulliB[2 k]/
      Gamma[2 k + 1] Gamma[b + 2 k - 1]/Gamma[a + b + 2 k - 1], {k, 0,
       r + 1}], {r, -1, \[CapitalLambda]}] +
  +Sum[1/u^(a + b + 2 r - 1) 1/(2 I) ((-1)^r BernoulliB[2 r])/
     Gamma[2 r + 1] (Gamma[b + 2 r - 1]/Gamma[b] - 
       Gamma[a + b + 2 r - 1]/Gamma[a + b]), {r, 
     0, \[CapitalLambda]}]]


IntPrec[n_]:=Sequence[PrecisionGoal->\[Infinity],AccuracyGoal->n,WorkingPrecision->(n+2)]


(* ::Text:: *)
(*EMFunb does the same for \bar\eta*)


Options@EMFunb=Options@EMFun;EMFunb[II_List/;(1<=Length[II](*<=2*)),o:OptionsPattern[]]:=Function@@{EMFun[II,o][[1]]/.Complex[a_,b_]:>Complex[a,-b]};


\[Section]Prec=12;
specint[inp_,sh_:-I/4]:=Block[{$RecursionLimit=\[Infinity],tmpfun},tmpfun=(-(1/(2\[Pi] I))inp/.subZetas/.{\[Eta]$[a__][___,b_]:>EMFun[{a},Shift->b,Prec->\[Section]Prec][u],
\[Eta]$b[a__][___,b_]:>EMFunb[{a},Shift->-b,Prec->\[Section]Prec][u]});Quiet[tmpfun/.u->aaa+sh//NIntegrate[#,{aaa,-\[Infinity],\[Infinity]},Evaluate@IntPrec@\[Section]Prec]&]];


(* ::Subsection::Closed:: *)
(*PSLQ*)


$PSLQMagnitude=10^20;
$PSLQPrecision=10;
$MaxPSLQSteps=100;
$PSLQ\[Gamma]=1+1/Sqrt[3];
$PSLQPrintMessages=True;
HermiteReduce[mm_]:=Block[{dd},
dd[i_,j_]:=dd[i,j]=Which[i<j,0,i==j,1,j==Length[mm],0,i>j,-(1/mm[[j,j]])Sum[dd[i,k]mm[[k,j]],{k,j+1,i}]];
Table[ dd[i,j],{i,Length[mm]},{j,Length[mm]}]
];
PSLQ[xxx_]:=
Block[{Hx,A,Ainit,Ainvinit,CC,Cinit,L,k,r,hred,n,s,\[Gamma]=$PSLQ\[Gamma],WeNotFound,xx},
$MaxPSLQMagnitude=Min[$PSLQMagnitude,10^(Precision[xxx]-5)];
xx=SetPrecision[xxx,Precision[xxx]+2$PSLQPrecision+5];
n=Length[xx];
s=Table[Sqrt[Sum[ xx[[j]]^2,{j,k,n}] ],{k,n}];
$PSLQLog={};$AinvLog={};
Hx=Table[Which[i<j, 0 , i==j , s[[i+1]]/s[[i]] , i>j , -((xx[[i]] xx[[j]])/(s[[j]]s[[j+1]])) ],{i,n},{j,n-1}];
A=Ainit=Round@HermiteReduce[Hx];
Ainv=Ainvinit=Inverse[Ainit];
CC=Cinit=IdentityMatrix[n-1];
L=A.Hx.CC;k=1;
AppendTo[$PSLQLog,Log[10,Max[Abs[xx.Ainv]]/Min[Abs[xx.Ainv]]]//N];
WeNotFound=True;
While[And[(1/Max[Abs@Diagonal[L]]<$MaxPSLQMagnitude),k<$MaxPSLQSteps,WeNotFound],
r=\[Gamma]^Range[n-1] Abs@Diagonal[L];
r=Position[r,Max[r]][[1,1]];
If[True,A=ReplacePart[A,{r->A[[r+1]],r+1->A[[r]]}]
];
L=QRDecomposition[(A.Hx)\[Transpose]][[-1]]\[Transpose];
hred=Round@HermiteReduce[L];
L=hred.L;
A=hred.A;
Ainv=Inverse[A];
k++;
AppendTo[$PSLQLog,Log[10,Max[Abs[xx.Ainv]]/Min[Abs[xx.Ainv]]]//N];
AppendTo[$AinvLog,Min[Abs[Ainv]//Flatten]//N];
If[($PSLQLog[[-1]]-$PSLQLog[[-2]]>$PSLQPrecision),WeNotFound=False]
];
If[WeNotFound,If[k>= $MaxPSLQSteps,
Print["Expression is not found after " <> ToString[k] <> " iterations"],
Print["Expression is not found for coefficients with less than " <> ToString[Round[Log[10,$MaxPSLQMagnitude]]]<>" digits"]
];,
If[$PSLQPrintMessages,
Print["Expression is found with jump of "<> ToString[($PSLQLog[[-1]]-$PSLQLog[[-2]])]<> " digits"<>
" in "<>ToString[k]<>" steps"];
Print[" Precision of obtaiined zero is at least "<> ToString[ 
Round[-Log[10,Abs@(xx.(Transpose[Ainv][[  Position[Abs[xx.Ainv],Min[Abs[xx.Ainv]]][[1,1]] ]]))]]]<>" digits"]];
Transpose[Ainv][[  Position[Abs[xx.Ainv],Min[Abs[xx.Ainv]]][[1,1]] ]]
]

]


(* ::Subsection::Closed:: *)
(*some integrals of the form (14)*)


(* ::Text:: *)
(*precintanswer[b,a,c] gives the value of the integral (14) if v=1/(2 Sqrt[3])  and J has one single element b, and a has one single element c.*)


Module[{Sargs},
Sargs={(AA:_?NumericQ|v0|u)[rr_]:>(AA+I/2 rr/.v0->v00),u->0};
subSn = {S[u^(a_ /; a < 0), v0[r_]^(b_ /; b < 0)][sh_:0] :> NSum[((I/2)*sh + I*n)^a*(\[Eta]$[-b][sh + 2 + 2*n] /. subtoPolyGammas /. u -> v00 + (I*r)/2), 
      {n, 0, Infinity}], S[v0[r_]^(b_ /; b < 0), u^(a_ /; a < 0)][sh_:0] :> 
     NSum[Evaluate[(v00 + (I*r)/2 + (I/2)*sh + I*n)^b*(\[Eta]$[-a][sh + 2 + 2*n] /. subtoPolyGammas /. u -> 0)], {n, 0, Infinity}],
S[(A_)^(a_/;a<0),(B_)^(b_/;b<0)][sh_:0]:>NSum[Evaluate[
((A/.Sargs)+I/2 sh+I n)^a * (\[Eta]$[-b][sh+2+2n]/. subtoPolyGammas /. u -> (B/.Sargs))],
{n,0,\[Infinity]}]}; ]
onerelation[a_, b_, c_] := If[b === 1 && c === 1, Print["Bad usage"], 
    {(1/Gamma[b])*(D[1/((u - \[Alpha])^a*u^c), {u, b - 1}] /. {(u - \[Alpha])^(aa_)*u^(cc_) :> (-1)^aa*\[Eta]0$[-aa][1]*\[Eta]$[-cc][2]}) + 
      (1/Gamma[c])*(D[1/((u - \[Alpha])^a*u^b), {u, c - 1}] /. {(u - \[Alpha])^(aa_)*u^(cc_) :> (-1)^(aa + cc)*S[u^cc, v0[-1]^aa][2]}) + 
      (SeriesCoefficient[\[Eta]$[b, c][1]/u^a /. TaylorOptions[Order -> -1], {u, 0, -1}] /. \[Eta]$ -> \[Eta]0$)}]; 
secondrelation[a_, b_] := (-\[Eta]$[a][2])*\[Eta]0$[b][1] + S[1/u^a, 1/v0[-1]^b][2] + S[1/v0[-1]^b, 1/u^a][2] + 
    Block[{n, \[Alpha]}, Apart[1/((\[Alpha] + n)^b*n^a), n] /. {n^(aa_)*\[Alpha]^(bb_) :> v0[-1]^bb*\[Eta]$[-aa][2], (n + \[Alpha])^(aa_)*\[Alpha]^(bb_) :> v0[-1]^bb*\[Eta]0$[-aa][1]}]; 
preciousintegral[a_, b_, c_] := (1/Gamma[b])*(Apart[D[1/((u - \[Alpha])^a*u^c), {u, b - 1}], u] /. {(u - \[Alpha])^(aa_) :> (-1)^aa*\[Eta]0$[-aa][1], 
        u^(cc_) :> (-1)^cc*\[Eta]$[-cc][2]} /. \[Alpha] -> v0[-1]) + (1/Gamma[b])*(D[1/((u - \[Alpha])^a*u^c), {u, b - 1}] /. 
      {(u - \[Alpha])^(aa_)*u^(cc_) :> (-1)^(aa + cc)*S[v0[-1]^aa, u^cc][2]}); 
precintanswer[aaa_, bbb_, ccc_] := 
  Expand[preciousintegral[aaa, bbb, ccc] /. xxx:S[v0[-1]^(a_), u^(b_)][2] :> (xxx /. Solve[secondrelation[-b, -a] == 0, xxx][[1]]) //. 
          xxx:S[u^(-1), v0[-1]^(a_)][2] :> (xxx /. Solve[onerelation[-a - 1, 1, 2] == 0, xxx][[1]]) //. xxx:S[u^(b_), v0[-1]^(c_)][2] :> 
          (xxx /. Solve[onerelation[-c, -b, 1] == 0, xxx][[1]]) /. sub\[Eta]\[Zeta] /. subZetaReduce /. sub\[Eta]0\[Eta] //. subtopowerbasis /. sub\[Eta]\[Eta]0]


subtoS={(\[Eta]$[II__][v___,shf_List]/;MemberQ[{{},{0}},{v}]:>((((Re[#][2Im[#]]&/@(I/2 shf))/.Re[u]->u/.Im[u]->0)^(-{II})//ExpandAll)//((S@@#)[0]&)))};


subSto\[Eta]={
S[II___][sh_]:>(\[Eta]$@@({II}/._^c_:>-c))[({II}/.{
u^_:>0,(AA:_?NumericQ|v0|u)[rr_]^_:>((AA+I/2 rr)/(I/2)/.v0->v00)
})+sh//ExpandAll]
};


(* ::Subsection::Closed:: *)
(*Simplifying input*)


zeta[a__]:=\[Zeta]$[a][0];
eta=\[Eta]$;etab=\[Eta]$b;
shiftmeta=shiftm\[Eta];
shiftpeta=shiftp\[Eta];
shiftmetab=shiftm\[Eta]b;
shiftpetab=shiftp\[Eta]b;

epsilon=\[Epsilon];


GeneralizedEta[a__][b_List][Pt___]:=\[Eta]$[a][Pt,-b/(I/2)]
