(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   1630408,      50177]
NotebookOptionsPosition[   1580243,      48707]
NotebookOutlinePosition[   1580965,      48734]
CellTagsIndexPosition[   1580876,      48729]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["\<\
In addition to other packages one should also:\
\>", "Text",
 CellChangeTimes->{{3.569019173628127*^9, 3.569019180321438*^9}}],

Cell[BoxData[
 RowBox[{"Get", "[", "\"\<transcendentality.m\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.568969422936851*^9, 3.5689694295190363`*^9}}],

Cell[CellGroupData[{

Cell["Higher transcendental coefficients", "Section",
 CellChangeTimes->{{3.5413970581295958`*^9, 3.541397061343525*^9}, {
   3.54988220878127*^9, 3.549882221850399*^9}, {3.552193238013246*^9, 
   3.5521932539051723`*^9}, 3.5537720339020433`*^9, {3.556010238345666*^9, 
   3.556010269789706*^9}, {3.56094011444121*^9, 3.560940116095921*^9}, {
   3.562235254765493*^9, 3.562235256652944*^9}, {3.562910524942194*^9, 
   3.56291052508632*^9}, {3.563513460250736*^9, 3.563513461711478*^9}, {
   3.563623150492649*^9, 3.5636231576416407`*^9}, 3.563734726823443*^9}],

Cell[CellGroupData[{

Cell["Init", "Subsection",
 CellChangeTimes->{{3.563730863002884*^9, 3.563730863712579*^9}}],

Cell[TextData[{
 "Here we have to understand how higher transcendental coefficients appear in \
computationt of \[Delta]E8loops. The goal is to find formulas for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["tran", "13"], ",", 
    SuperscriptBox["tran", "12"], ",", 
    SuperscriptBox["tran", "11"]}], TraditionalForm]]],
 ", with special attention to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", "11"], TraditionalForm]]],
 " terms."
}], "Text",
 CellChangeTimes->{{3.563623161539673*^9, 3.56362321558355*^9}, {
  3.56896937598155*^9, 3.568969376323831*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", "11"], TraditionalForm]]],
 " appear also in the asymptotic guys, but there they seem to be trivial (i.e \
only Zeta[_], not Zeta[__]). We will come probably later to their \
computation. Now we consider wrapping correction only. For that we have an \
explicit formula"
}], "Text",
 CellChangeTimes->{{3.563623228262903*^9, 3.563623273292955*^9}, {
  3.567147802444132*^9, 3.567147815069138*^9}, {3.568969390227297*^9, 
  3.568969391164213*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["E", "wrap"], "==", 
   RowBox[{"2", "\[ImaginaryI]", " ", 
    RowBox[{"\[DoubleStruckCapitalC]", "[", 
     RowBox[{
      FractionBox["1", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]], "H"}], "]"}]}]}], 
  TraditionalForm]], "Input",
 CellChangeTimes->{{3.56362327510137*^9, 3.563623308315324*^9}}],

Cell["\<\
All objects upon inspection have the following transcendental structure\
\>", "Text",
 CellChangeTimes->{{3.563730933808198*^9, 3.5637309697968817`*^9}}],

Cell[BoxData[
 RowBox[{"obj", "=", 
  RowBox[{
   SuperscriptBox["tran", 
    RowBox[{"-", "pow"}]], 
   RowBox[{
    SubscriptBox["\[Sum]", "k"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"g", " ", "tran"}], ")"}], "k"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["obj", 
        RowBox[{"k", ",", "1"}]], "+", 
       RowBox[{
        SubscriptBox["obj", 
         RowBox[{"k", ",", "2"}]], 
        SuperscriptBox["tran", 
         RowBox[{"-", "2"}]]}], "+", 
       RowBox[{
        SubscriptBox["obj", 
         RowBox[{"k", ",", "4"}]], 
        SuperscriptBox["tran", 
         RowBox[{"-", "4"}]]}]}], ")"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5637309721953917`*^9, 3.563731025817535*^9}}],

Cell["\<\
pow is essential information: It shows how far the object from maximal \
transcendentality (pow>0 deproaches us from maximal transcendentality with \
each multiplication by obj). We hence expand in tran using knowledge of \
pow-s. pow-s are summarized in this table:\
\>", "Text",
 CellChangeTimes->{{3.5637310286342077`*^9, 3.563731120118786*^9}, 
   3.5689694491862087`*^9}],

Cell[CellGroupData[{

Cell[BoxData["tranpowers"], "Input",
 CellChangeTimes->{{3.5689694415535097`*^9, 3.568969442657048*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"xxx", ":", 
     RowBox[{"comb1", "|", "comb2", "|", 
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"3", ",", "4"}], 
       RowBox[{"[", "_", "]"}]], "|", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], 
       RowBox[{"3", ",", "4"}], 
       RowBox[{"[", "_", "]"}]], "|", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["U",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0.6, 0]]], "2"], "|", 
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"1", ",", "3"}], 
       RowBox[{"[", "_", "]"}]], "|", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], 
       RowBox[{"1", ",", "3"}], 
       RowBox[{"[", "_", "]"}]]}]}], "\[RuleDelayed]", 
    FractionBox["xxx", 
     SuperscriptBox["tran", "2"]]}], ",", 
   RowBox[{
    RowBox[{"xxx", ":", 
     RowBox[{
      OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}]}], "\[RuleDelayed]", 
    FractionBox["xxx", 
     SuperscriptBox["tran", "4"]]}], ",", 
   RowBox[{
    RowBox[{"xxx", ":", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"1", ",", "4"}], 
       RowBox[{"[", "_", "]"}]], "|", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], 
       RowBox[{"1", ",", "4"}], 
       RowBox[{"[", "_", "]"}]]}]}], "\[RuleDelayed]", 
    FractionBox["xxx", "tran"]}], ",", 
   RowBox[{
    RowBox[{"xxx", ":", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], "2", 
       RowBox[{"[", "_", "]"}]], "|", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], "2", 
       RowBox[{"[", "_", "]"}]]}]}], "\[RuleDelayed]", 
    RowBox[{"tran", " ", "xxx"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.568969443300668*^9}]
}, Open  ]],

Cell["I expand now H keeping only first three powers of tran:", "Text",
 CellChangeTimes->{{3.5637311700624323`*^9, 3.563731188828187*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Log", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             FractionBox[
              SubscriptBox["T", 
               RowBox[{"1", ",", "0"}]], 
              RowBox[{
               SuperscriptBox["Q", "+"], 
               SuperscriptBox["Q", "-"]}]], "/.", "subTQ"}], "/.", "subpqb"}],
            "/.", "substandardq"}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{"Collect", "[", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 FractionBox[
                  SubsuperscriptBox["T", 
                   RowBox[{"0", ",", "0"}], "+"], 
                  RowBox[{
                   SuperscriptBox["Q", "+"], 
                   SuperscriptBox["Q", "+"]}]], "/.", "subTQ"}], "/.", 
                "subpqb"}], "/.", "substandardq"}], "//.", 
              "subtoanalyticitystrips"}], ")"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"U$b", "[", "0", "]"}], ",", 
              RowBox[{
               RowBox[{"q$b", "[", 
                RowBox[{"1", ",", "2"}], "]"}], "[", 
               RowBox[{"-", "1"}], "]"}], ",", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["q",
                 FontColor->RGBColor[0, 0, 0.8]], 
                StyleBox["_",
                 FontColor->RGBColor[0, 0, 0.8]]], 
               RowBox[{"1", ",", "2"}], "+"]}], "}"}]}], "]"}], ")"}]}], 
        "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              SubsuperscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               RowBox[{"2", ",", "4"}], "+"]}], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               StyleBox["_",
                FontColor->RGBColor[0, 0, 0.8]]], 
              RowBox[{"1", ",", "3"}], "-"]}], "-", 
            RowBox[{"2", " ", 
             SubsuperscriptBox[
              StyleBox["q",
               FontColor->RGBColor[0, 0, 0.8]], 
              RowBox[{"1", ",", "4"}], "+"], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               StyleBox["_",
                FontColor->RGBColor[0, 0, 0.8]]], 
              RowBox[{"1", ",", "4"}], "-"]}], "-", 
            RowBox[{
             SubsuperscriptBox[
              StyleBox["q",
               FontColor->RGBColor[0, 0, 0.8]], 
              RowBox[{"1", ",", "3"}], "+"], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               StyleBox["_",
                FontColor->RGBColor[0, 0, 0.8]]], 
              RowBox[{"2", ",", "4"}], "-"]}]}], ")"}], "\[Rule]", 
          RowBox[{"-", "comb1"}]}], ",", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            OverscriptBox[
             StyleBox["U",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["_",
              FontColor->RGBColor[0, 0.6, 0]]], "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              SubsuperscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               RowBox[{"2", ",", "4"}], "+"]}], "-", 
             RowBox[{"2", " ", 
              SubsuperscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               RowBox[{"1", ",", "4"}], "+"], " ", 
              SubscriptBox[
               OverscriptBox[
                StyleBox["q",
                 FontColor->RGBColor[0, 0, 0.8]], 
                StyleBox["_",
                 FontColor->RGBColor[0, 0, 0.8]]], "2"]}], "-", 
             RowBox[{
              SubsuperscriptBox[
               StyleBox["q",
                FontColor->RGBColor[0, 0, 0.8]], 
               RowBox[{"1", ",", "3"}], "+"], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["q",
                 FontColor->RGBColor[0, 0, 0.8]], 
                StyleBox["_",
                 FontColor->RGBColor[0, 0, 0.8]]], "2", "2"]}]}], ")"}]}], 
          "\[Rule]", 
          RowBox[{"-", "comb2"}]}]}], "}"}]}], "//.", "substandardq"}], "//.",
      "subtoanalyticitystrips"}], "//.", "substandardq"}], "/.", 
   "tranpowers"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{"%", ",", 
    RowBox[{"{", 
     RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Normal", "[", "%", "]"}], "/.", 
       RowBox[{"tran", "\[Rule]", "1"}]}], "/.", "subq34"}], "//.", 
     "subtoanalyticitystrips"}], "//.", "substandardq"}], "/.", 
   "tranpowers"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"ExpaCol", "[", 
      RowBox[{
       RowBox[{"Normal", "@", 
        RowBox[{"Series", "[", 
         RowBox[{"%", ",", 
          RowBox[{"{", 
           RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}]}], 
       ",", 
       RowBox[{
        RowBox[{"Q", "[", "1", "]"}], "|", 
        RowBox[{"Q", "[", 
         RowBox[{"-", "1"}], "]"}]}]}], "]"}], ",", 
     RowBox[{
      RowBox[{"Not", "[", 
       RowBox[{"FreeQ", "[", 
        RowBox[{"#", ",", 
         RowBox[{"comb1", "|", "comb2", "|", "q$", "|", "q$b", "|", "U$b"}]}],
         "]"}], "]"}], "&"}]}], "]"}], ",", "tran", ",", "Simplify"}], 
  "]"}]}], "Input",
 CellChangeTimes->{
  3.563626695386093*^9, {3.5636267284386263`*^9, 3.563626936934523*^9}, {
   3.56362698150345*^9, 3.563626999254616*^9}, {3.5636270324687977`*^9, 
   3.5636272465711527`*^9}, {3.563639073289575*^9, 3.563639080651922*^9}, {
   3.564824871940507*^9, 3.5648248726637783`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["comb2", 
   RowBox[{
    SuperscriptBox["tran", "2"], " ", 
    SuperscriptBox[
     StyleBox["Q",
      FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       StyleBox["Q",
        FontColor->RGBColor[0.8, 0, 0]], "+"], ")"}], "2"]}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["comb2", "2"], "+", 
    RowBox[{"2", " ", "comb1", " ", "comb2", " ", 
     SuperscriptBox[
      StyleBox["Q",
       FontColor->RGBColor[0.8, 0, 0]], "+"]}], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      StyleBox["Q",
       FontColor->RGBColor[0.8, 0, 0]], "+"], " ", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["U",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["_",
        FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         SubsuperscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          RowBox[{"1", ",", "4"}], "+"], ")"}], "2"], "-", 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         RowBox[{"1", ",", "3"}], "+"], " ", 
        SubsuperscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         RowBox[{"2", ",", "4"}], "+"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0, 0.8]]], "2", "2"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0, 0.8]]], 
         RowBox[{"1", ",", "3"}], "-"]}], "-", 
       RowBox[{"2", " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0, 0.8]]], "2"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0, 0.8]]], 
         RowBox[{"1", ",", "4"}], "-"]}], "+", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         StyleBox["_",
          FontColor->RGBColor[0, 0, 0.8]]], 
        RowBox[{"2", ",", "4"}], "-"]}], ")"}]}]}], 
   RowBox[{"2", " ", 
    SuperscriptBox["tran", "4"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       StyleBox["Q",
        FontColor->RGBColor[0.8, 0, 0]], "-"], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       StyleBox["Q",
        FontColor->RGBColor[0.8, 0, 0]], "+"], ")"}], "4"]}]]}]], "Output",
 CellChangeTimes->{{3.563627157879141*^9, 3.563627189649786*^9}, {
   3.563627220875924*^9, 3.563627246993634*^9}, 3.563639081451953*^9, 
   3.56373119225723*^9, 3.5671478622701178`*^9, 3.568969468292591*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["subcomb"], "Input",
 CellChangeTimes->{{3.568969464799255*^9, 3.568969466641534*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"comb1", "\[Rule]", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"2", ",", "4"}], "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], 
       RowBox[{"1", ",", "3"}], "-"]}], "+", 
     RowBox[{"2", " ", 
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"1", ",", "4"}], "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], 
       RowBox[{"1", ",", "4"}], "-"]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"1", ",", "3"}], "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], 
       RowBox[{"2", ",", "4"}], "-"]}]}]}], ",", 
   RowBox[{"comb2", "\[Rule]", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["U",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SubsuperscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         RowBox[{"2", ",", "4"}], "+"]}], "-", 
       RowBox[{"2", " ", 
        SubsuperscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         RowBox[{"1", ",", "4"}], "+"], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0, 0.8]]], "2"]}], "-", 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         RowBox[{"1", ",", "3"}], "+"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0, 0.8]]], "2", "2"]}]}], ")"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.568969468402687*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Leading tran terms", "Subsection",
 CellChangeTimes->{{3.563731253673662*^9, 3.5637312599051867`*^9}}],

Cell["Leading transcendentality terms are given by:", "Text",
 CellChangeTimes->{{3.563640939383493*^9, 3.563640974243816*^9}, {
  3.5637312669773493`*^9, 3.563731274840811*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox[
    RowBox[{"comb2", " ", 
     SuperscriptBox["g", "8"]}], 
    RowBox[{
     SuperscriptBox[
      StyleBox["Q",
       FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "+"], ")"}], "2"]}]], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"comb1", "\[Rule]", 
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           SubsuperscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            RowBox[{"2", ",", "4"}], "+"]}], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"1", ",", "3"}], "-"]}], "-", 
         RowBox[{"2", " ", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "4"}], "+"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"1", ",", "4"}], "-"]}], "-", 
         RowBox[{
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "3"}], "+"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"2", ",", "4"}], "-"]}]}], ")"}]}]}], ",", 
     RowBox[{"comb2", "\[Rule]", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["U",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["_",
           FontColor->RGBColor[0, 0.6, 0]]], "2"]}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"2", ",", "4"}], "+"]}], "-", 
         RowBox[{"2", " ", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "4"}], "+"], " ", 
          SubscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], "2"]}], "-", 
         RowBox[{
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "3"}], "+"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], "2", "2"]}]}], ")"}]}]}]}], 
    "}"}]}], "//", "Factor"}]], "Input",
 CellChangeTimes->{{3.563640996927593*^9, 3.563641003630746*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["U",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["_",
      FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox[
      StyleBox["q",
       FontColor->RGBColor[0, 0, 0.8]], 
      RowBox[{"2", ",", "4"}], "+"], "+", 
     RowBox[{"2", " ", 
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"1", ",", "4"}], "+"], " ", 
      SubscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], "2"]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"1", ",", "3"}], "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        StyleBox["_",
         FontColor->RGBColor[0, 0, 0.8]]], "2", "2"]}]}], ")"}]}], 
  RowBox[{
   SuperscriptBox[
    StyleBox["Q",
     FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SuperscriptBox[
      StyleBox["Q",
       FontColor->RGBColor[0.8, 0, 0]], "+"], ")"}], "2"]}]]], "Output",
 CellChangeTimes->{{3.5636409985722733`*^9, 3.5636410039734707`*^9}, {
   3.563731262586891*^9, 3.5637312775269747`*^9}, 3.5671478715541697`*^9, 
   3.568969476658502*^9}]
}, Open  ]],

Cell["\<\
Let us discuss in detail how transcendentality is coming to different terms. \
For U for instance we have asymptotic and normal expressions\
\>", "Text",
 CellChangeTimes->{{3.563641010337945*^9, 3.563641034817206*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Exponent", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubsuperscriptBox["U", "\"\<as\>\"", "2"], "-", 
        RowBox[{"\[Alpha]", "  ", 
         SuperscriptBox["U", "2"]}]}], "/.", 
       RowBox[{"nsweak", "[", "5", "]"}]}], ",", "g"}], "]"}], "/.", 
    "subtotran"}], ",", "tran"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Exponent", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubsuperscriptBox["U", "\"\<as\>\"", "2"], "-", " ", 
        SuperscriptBox["U", "2"]}], "/.", 
       RowBox[{"nsweak", "[", "5", "]"}]}], ",", "g"}], "]"}], "/.", 
    "subtotran"}], ",", "tran"}], "]"}]}], "Input",
 CellChangeTimes->{{3.563641036393435*^9, 3.563641101763896*^9}, {
  3.563641195010697*^9, 3.56364120245072*^9}, {3.56714795484623*^9, 
  3.56714796082899*^9}}],

Cell[BoxData[
 RowBox[{"-", "2"}]], "Output",
 CellChangeTimes->{{3.563641046421873*^9, 3.563641102291831*^9}, {
   3.5636411966660013`*^9, 3.563641202817782*^9}, 3.5671479628713493`*^9, 
   3.5689694847168818`*^9}],

Cell[BoxData[
 RowBox[{"-", "3"}]], "Output",
 CellChangeTimes->{{3.563641046421873*^9, 3.563641102291831*^9}, {
   3.5636411966660013`*^9, 3.563641202817782*^9}, 3.5671479628713493`*^9, 
   3.568969484798593*^9}]
}, Open  ]],

Cell["\<\
Only the leading transcendentality is purely asymptotic effect. We hence can \
trace at least it. Let us assume that it holds at any order. Question is also \
about normalization. Well, let us study derivation of U once more\
\>", "Text",
 CellChangeTimes->{{3.563641103782694*^9, 3.563641152381156*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     SuperscriptBox["V", "+"], 
     SuperscriptBox["V", "-"]], "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["Bp", "+"], 
      SuperscriptBox["Bm", "+"]], 
     SubscriptBox["\[Sigma]", "BES"]}]}], ",", 
   RowBox[{"U", "\[Equal]", 
    RowBox[{
     FractionBox["1", 
      SqrtBox[
       SubscriptBox["C", "\[GothicCapitalT]"]]], "Bm", " ", 
     FractionBox["V", 
      SuperscriptBox["x", 
       RowBox[{"L", "/", "2"}]]]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.5409777203347883`*^9, 3.54097774075317*^9}, {
  3.550230733426744*^9, 3.55023078107977*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drphase", "/.", 
  RowBox[{"leadweak", "[", "5", "]"}]}]], "Input",
 CellChangeTimes->{{3.563731337561056*^9, 3.563731343072632*^9}, {
  3.563731378487565*^9, 3.563731379207012*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"12", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "48"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{"120", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]], "-", 
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "6"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"48", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"252", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"336", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"480", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"1260", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "2820"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{"288", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
       RowBox[{"3576", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "-", 
       RowBox[{"5040", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"14112", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "336"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "-", 
       RowBox[{"1008", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"4032", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "48"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"432", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "6"]], "-", 
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "8"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "11"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "10"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"48", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"660", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "8"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"336", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{"1728", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"9900", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "6"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"2820", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"288", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"7596", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{"16128", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"59400", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4320", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
        RowBox[{"31440", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "+", 
        RowBox[{
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"], " ", 
         RowBox[{"(", 
          RowBox[{"26508", "+", 
           FractionBox[
            RowBox[{"4", " ", "Im1"}], 
            SqrtBox["3"]], "+", 
           RowBox[{"5760", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}], "+", 
        RowBox[{"38736", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}], "+", 
        RowBox[{"56448", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "9"]}], "+", 
        RowBox[{"166320", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{{3.56373137626842*^9, 3.563731381684628*^9}}]
}, Open  ]],

Cell["\<\
One can note that drphase has deficit 1, so we put it to zero.\
\>", "Text",
 CellChangeTimes->{{3.563731402571632*^9, 3.563731422010931*^9}}],

Cell["Now the Bethe root", "Text",
 CellChangeTimes->{{3.5637314239389143`*^9, 3.563731426162599*^9}, {
  3.568969500686429*^9, 3.568969501030221*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["v", "0"], "/.", 
  RowBox[{"leadweak", "[", "5", "]"}]}]], "Input",
 CellChangeTimes->{{3.563731433152617*^9, 3.563731440909089*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"2", " ", 
    SqrtBox["3"]}]], "+", 
  FractionBox[
   RowBox[{"4", " ", 
    SuperscriptBox["g", "2"]}], 
   SqrtBox["3"]], "-", 
  FractionBox[
   RowBox[{"10", " ", 
    SuperscriptBox["g", "4"]}], 
   SqrtBox["3"]], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["56", 
      SqrtBox["3"]], "+", 
     RowBox[{"8", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["461", 
       SqrtBox["3"]]}], "-", 
     FractionBox["Im1", "9"], "-", 
     RowBox[{"48", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"80", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.563731437439465*^9, 3.563731441070809*^9}}]
}, Open  ]],

Cell["\<\
It has deficit 3. So we keep only leading term.  This allows us to evaluate \
Bp and Bm\
\>", "Text",
 CellChangeTimes->{{3.5637314453941803`*^9, 3.563731469721139*^9}, {
  3.563731815013789*^9, 3.563731821531809*^9}}],

Cell[BoxData[
 RowBox[{"U", "\[Equal]", 
  RowBox[{
   FractionBox["1", 
    SqrtBox[
     SubscriptBox["C", "\[GothicCapitalT]"]]], "Bm", " ", 
   FractionBox["V", 
    SuperscriptBox["x", 
     RowBox[{"L", "/", "2"}]]]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Bp", ",", 
          RowBox[{"-", 
           FractionBox[
            SuperscriptBox["Bp", "+"], 
            SuperscriptBox["Bm", "+"]]}]}], "}"}], "/.", "subFunsBethe"}], "/.", 
       RowBox[{
        RowBox[{"y$h", "[", "r_", "]"}], "\[RuleDelayed]", 
        FractionBox[
         RowBox[{"v00", "+", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "r"}], "2"]}], "g"]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "2"}], "}"}]}], "]"}], "//", 
    "FullSimplify"}], ",", 
   RowBox[{"{", 
    RowBox[{"g", ",", "x"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.56373147079665*^9, 3.563731533612145*^9}, {
  3.5637316881442547`*^9, 3.563731733993346*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", 
      SqrtBox["3"]]}], "+", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SuperscriptBox["g", "2"]}], 
     SuperscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], "2"]], "-", 
    FractionBox[
     RowBox[{
      SqrtBox["3"], " ", "g"}], 
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]]]}], ",", 
   RowBox[{"1", "-", 
    FractionBox[
     RowBox[{"18", " ", 
      SuperscriptBox["g", "2"]}], 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], "+"], ")"}], "2"]], "-", 
    FractionBox[
     RowBox[{"6", " ", "\[ImaginaryI]", " ", "g"}], 
     SuperscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], "+"]]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.563731474137157*^9, 3.563731534062756*^9}, {
  3.563731688554079*^9, 3.563731734463936*^9}}]
}, Open  ]],

Cell[TextData[{
 "We see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Bp", "\[TildeEqual]", 
    RowBox[{
     RowBox[{"-", "I"}], "/", 
     SqrtBox["3"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  RowBox[{"-", 
   FractionBox[
    SuperscriptBox["Bp", "+"], 
    SuperscriptBox["Bm", "+"]]}]],
  CellChangeTimes->{{3.56373147079665*^9, 3.563731533612145*^9}, {
   3.5637316881442547`*^9, 3.563731733993346*^9}}],
 "\[TildeEqual]1"
}], "Text",
 CellChangeTimes->{{3.5637317365909233`*^9, 3.563731754022016*^9}}],

Cell["For constant prefcator, we have", "Text",
 CellChangeTimes->{{3.563731824747299*^9, 3.5637318299709187`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     FractionBox["1", 
      SqrtBox[
       SubscriptBox["C", "\[GothicCapitalT]"]]], "/.", "weak"}], "/.", 
    RowBox[{"leadweak", "[", "5", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"g", ",", "0", ",", "7"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.563731833847574*^9, 3.563731866111081*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{
    SqrtBox["6"], " ", "g"}], "+", 
   RowBox[{
    SqrtBox["6"], " ", 
    SuperscriptBox["g", "3"]}], "-", 
   RowBox[{"21", " ", 
    SqrtBox[
     FractionBox["3", "2"]], " ", 
    SuperscriptBox["g", "5"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"109", " ", 
       SqrtBox["6"]}], "-", 
      RowBox[{"12", " ", 
       SqrtBox["6"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "7"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "8"],
    SeriesData[$CellContext`g, 0, {}, 1, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   6^Rational[1, 2], 0, 6^Rational[1, 2], 0, (-21) 
    Rational[3, 2]^Rational[1, 2], 0, 
    109 6^Rational[1, 2] - (12 6^Rational[1, 2]) $CellContext`\[Zeta]$[3][0]},
    1, 8, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.563731834777302*^9, 3.56373186634169*^9}}]
}, Open  ]],

Cell["Again, only leading term should be kept", "Text",
 CellChangeTimes->{{3.563731869753849*^9, 3.563731876569109*^9}}],

Cell["\<\
Finally, there is also a normalization factor. Its deviation from 1 has also \
deficit 1:\
\>", "Text",
 CellChangeTimes->{{3.563731763751534*^9, 3.563731793292494*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nfactor", "/.", "weak"}]], "Input",
 CellChangeTimes->{{3.563731776221602*^9, 3.5637317834473953`*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "-", 
   RowBox[{"6", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "1"], " ", 
    SuperscriptBox["g", "2"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"24", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1"]}], "+", 
      RowBox[{"18", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
      RowBox[{"18", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "4"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "168"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1"]}], "-", 
      RowBox[{"144", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
      RowBox[{"36", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1", "3"]}], "-", 
      RowBox[{"72", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "-", 
      RowBox[{"108", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "-", 
      RowBox[{"120", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}]}], ")"}], " ", 
    SuperscriptBox["g", "6"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "8"],
    SeriesData[$CellContext`g, 0, {}, 0, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   1, 0, (-6) $CellContext`\[Zeta]$[1][0], 0, 24 $CellContext`\[Zeta]$[1][0] + 
    18 $CellContext`\[Zeta]$[1][0]^2 + 18 $CellContext`\[Zeta]$[3][0], 
    0, (-168) $CellContext`\[Zeta]$[1][0] - 144 $CellContext`\[Zeta]$[1][0]^2 - 
    36 $CellContext`\[Zeta]$[1][0]^3 - 
    72 $CellContext`\[Zeta]$[3][0] - (
     108 $CellContext`\[Zeta]$[1][0]) $CellContext`\[Zeta]$[3][0] - 
    120 $CellContext`\[Zeta]$[5][0]}, 0, 8, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.56373178095126*^9, 3.563731783747463*^9}}]
}, Open  ]],

Cell["The conclusion is that", "Text",
 CellChangeTimes->{{3.563731795588489*^9, 3.563731797980021*^9}}],

Cell[BoxData[
 RowBox[{"U", "\[Equal]", 
  RowBox[{
   FractionBox["1", 
    SqrtBox[
     SubscriptBox["C", "\[GothicCapitalT]"]]], "Bm", " ", 
   FractionBox["V", 
    SuperscriptBox["x", 
     RowBox[{"L", "/", "2"}]]]}], "\[TildeEqual]", 
  RowBox[{
   SqrtBox["6"], " ", "g", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "I"}], "/", 
     SqrtBox["3"]}], ")"}], 
   FractionBox["1", 
    OverscriptBox["x", "^"]]}], "==", 
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SqrtBox["2"], " ", "g"}], 
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]]]}]}]], "Input",
 CellChangeTimes->{{3.563731882686146*^9, 3.563731912843212*^9}}],

Cell["So U has deficit 1.", "Text",
 CellChangeTimes->{{3.563731920361753*^9, 3.563731927119267*^9}}],

Cell[TextData[{
 "Now we have to evaluate ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "2"], TraditionalForm]]],
 ". This gives us"
}], "Text",
 CellChangeTimes->{{3.56373193557474*^9, 3.563731945110751*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["q", "2"], "/.", "largevolume"}], "/.", 
   RowBox[{"leadweak", "[", "1", "]"}]}], "/.", 
  RowBox[{"u", "\[Rule]", "0"}]}]], "Input",
 CellChangeTimes->{{3.5637319457464437`*^9, 3.563731964064466*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["\[ImaginaryI]", 
   RowBox[{"3", " ", "g", " ", 
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]]}]]}]], "Output",
 CellChangeTimes->{{3.563731948954648*^9, 3.5637319644319677`*^9}, 
   3.5671480467794867`*^9}]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "2"], TraditionalForm]]],
 " has proficit 1."
}], "Text",
 CellChangeTimes->{{3.5637319704139643`*^9, 3.563731975221304*^9}}],

Cell[TextData[{
 "When we meet ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "Q"], TraditionalForm]]],
 " in the denominator, we replace it by ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "Q0"], TraditionalForm]]],
 " and its value at 0 (because aparting and keeping only the most singular \
terms means this. In particular"
}], "Text",
 CellChangeTimes->{{3.563732098200543*^9, 3.563732145342894*^9}, 
   3.563732178593026*^9, 3.568969522038578*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["Q", "+"], 
    SuperscriptBox["Q", "-"]}]], "\[TildeEqual]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     FractionBox["1", 
      RowBox[{
       SuperscriptBox["Q0", "+"], 
       SuperscriptBox["Q0", "-"]}]], "/.", "subQ0"}], "/.", 
    RowBox[{"u", "\[Rule]", "0"}]}], ")"}], "\[Equal]", "9"}]], "Input",
 CellChangeTimes->{{3.563732146246071*^9, 3.5637321708626757`*^9}}],

Cell[TextData[{
 "Using this approximation we can compute ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", 
    RowBox[{"i", ",", "j"}]], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.563732181701222*^9, 3.563732190327139*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    SubsuperscriptBox["q", 
     RowBox[{"1", ",", "3"}], "+"], ",", 
    SubsuperscriptBox["q", 
     RowBox[{"1", ",", "4"}], "+"], ",", 
    SubsuperscriptBox["q", 
     RowBox[{"2", ",", "4"}], "+"]}], "}"}], "==", 
  RowBox[{
   SubsuperscriptBox["q", 
    RowBox[{"1", ",", "2"}], "+"], 
   FractionBox[
    SuperscriptBox["D", "2"], 
    RowBox[{"1", "-", 
     SuperscriptBox["D", "2"]}]], 
   FractionBox[
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       SubscriptBox["q", "2"], ",", 
       SubsuperscriptBox["q", "2", "2"]}], "}"}], 
     SuperscriptBox["U", "2"]}], 
    RowBox[{
     SubsuperscriptBox["q", 
      RowBox[{"1", ",", "2"}], "+"], 
     SubsuperscriptBox["q", 
      RowBox[{"1", ",", "2"}], "-"]}]]}], "\[TildeEqual]", 
  RowBox[{
   RowBox[{"-", "18"}], 
   SuperscriptBox["g", "2"], 
   SuperscriptBox["Q", "+"], 
   FractionBox[
    SuperscriptBox["D", "2"], 
    RowBox[{"1", "-", 
     SuperscriptBox["D", "2"]}]], 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"-", 
      FractionBox["\[ImaginaryI]", 
       RowBox[{"3", " ", "g", " ", 
        StyleBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]],
         FontColor->RGBColor[0, 0.6, 0]]}]]}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"3", " ", "g", " ", 
          StyleBox[
           OverscriptBox[
            StyleBox["x",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]],
           FontColor->RGBColor[0, 0.6, 0]]}]]}], ")"}], "2"]}], "}"}], 
   RowBox[{"(", 
    FractionBox["1", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.563732191590548*^9, 3.56373220345675*^9}, {
  3.5637322379765997`*^9, 3.56373237579834*^9}}],

Cell[TextData[{
 "So we finally get for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["H", "leadtran"], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.5637319869409657`*^9, 3.5637320135805817`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["H", "leadtran"], "==", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     OverscriptBox["U", "_"], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], 
       RowBox[{"2", ",", "4"}], "+"], "+", 
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        RowBox[{"1", ",", "4"}], "+"], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         StyleBox["_",
          FontColor->RGBColor[0, 0, 0.8]]], "2"]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        RowBox[{"1", ",", "3"}], "+"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         StyleBox["_",
          FontColor->RGBColor[0, 0, 0.8]]], "2", "2"]}]}], ")"}]}], 
   RowBox[{
    SuperscriptBox[
     StyleBox["Q",
      FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
    SuperscriptBox[
     StyleBox["Q",
      FontColor->RGBColor[0.8, 0, 0]], "+"]}]], "\[TildeEqual]", 
  RowBox[{
   RowBox[{"-", "36"}], 
   SuperscriptBox["g", "2"], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]], 
      FractionBox[
       SuperscriptBox["D", "2"], 
       RowBox[{"1", "-", 
        SuperscriptBox["D", "2"]}]], 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "-", 
     RowBox[{"2", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "3"]], 
      FractionBox[
       SuperscriptBox["D", "2"], 
       RowBox[{"1", "-", 
        SuperscriptBox["D", "2"]}]], 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "3"]]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]], 
      FractionBox[
       SuperscriptBox["D", "2"], 
       RowBox[{"1", "-", 
        SuperscriptBox["D", "2"]}]], 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.563732016490437*^9, 3.563732090145887*^9}, {
  3.563732354513885*^9, 3.5637324622253113`*^9}, {3.563732665949129*^9, 
  3.5637326666571283`*^9}}],

Cell["To compute energy we use this formula", "Text",
 CellChangeTimes->{{3.56373257689603*^9, 3.563732582653537*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["E", "wrap"], "==", 
   RowBox[{"2", "\[ImaginaryI]", " ", 
    RowBox[{"\[DoubleStruckCapitalC]", "[", 
     RowBox[{
      FractionBox["1", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]], "H"}], "]"}]}]}], 
  TraditionalForm]], "Input"],

Cell["Recall that", "Text",
 CellChangeTimes->{{3.563732585517421*^9, 3.56373258737331*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   FractionBox[
    RowBox[{"x", "+", 
     FractionBox["1", "x"]}], 
    RowBox[{"x", "-", 
     FractionBox["1", "x"]}]], ")"}], "==", 
  FractionBox["1", 
   SqrtBox[
    RowBox[{"1", "-", 
     FractionBox[
      RowBox[{"4", 
       SuperscriptBox["g", "2"]}], 
      SuperscriptBox["u", "2"]]}]]]}]], "Input",
 CellChangeTimes->{{3.5637325918751698`*^9, 3.563732597772482*^9}}],

Cell[TextData[{
 "Integration is clear how to use. At each order of g we will have simple \
integrands ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["u", 
     RowBox[{"-", "a"}]], 
    SubsuperscriptBox["\[Eta]", "b", 
     RowBox[{"[", "2", "]"}]]}], TraditionalForm]]],
 " which is known how to evaluate"
}], "Text",
 CellChangeTimes->{{3.563732613172237*^9, 3.563732643851306*^9}}],

Cell["So we can now compute answer:", "Text",
 CellChangeTimes->{{3.563732602394248*^9, 3.563732609116693*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lhs", "=", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          FractionBox[
           RowBox[{"x", "+", 
            FractionBox["1", "x"]}], 
           RowBox[{"x", "-", 
            FractionBox["1", "x"]}]], ")"}], 
         RowBox[{"{", 
          RowBox[{
           FractionBox["1", 
            SuperscriptBox["x", "4"]], ",", 
           RowBox[{
            RowBox[{"-", "2"}], " ", 
            FractionBox["1", 
             SuperscriptBox["x", "3"]]}], ",", 
           FractionBox["1", 
            SuperscriptBox["x", "2"]]}], "}"}]}], "/.", 
        RowBox[{"x$", "\[Rule]", "x$h"}]}], "/.", "subJuk"}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "50"}], "}"}]}], "]"}], "//", "Normal"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rhs", "=", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Reverse", "@", 
         RowBox[{"{", 
          RowBox[{
           FractionBox["1", 
            SuperscriptBox["x", "4"]], ",", " ", 
           FractionBox["1", 
            SuperscriptBox["x", "3"]], ",", 
           FractionBox["1", 
            SuperscriptBox["x", "2"]]}], "}"}]}], "/.", 
        RowBox[{"x$", "\[Rule]", "x$h"}]}], "/.", "subJuk"}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "50"}], "}"}]}], "]"}], "//", "Normal"}]}],
   ";"}]}], "Input",
 CellChangeTimes->{{3.563651129527851*^9, 3.5636512398746347`*^9}, {
   3.5636515163528757`*^9, 3.563651526648258*^9}, 3.563651756382914*^9, {
   3.5636518308458223`*^9, 3.563651833507731*^9}, {3.563651945142181*^9, 
   3.5636519548950663`*^9}, {3.56365209598636*^9, 3.563652128120563*^9}, {
   3.563693885906275*^9, 3.563693893318754*^9}, {3.5636941708135843`*^9, 
   3.563694173018052*^9}, {3.563711643190213*^9, 3.563711645084939*^9}, {
   3.563715148827971*^9, 3.563715150907089*^9}, {3.563716308222208*^9, 
   3.5637163445384207`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"-", "36"}], ")"}], 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"lhs", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "/.", 
        RowBox[{
         SuperscriptBox["u", 
          RowBox[{"a_", "/;", 
           RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
         FractionBox[
          RowBox[{"D", "[", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"rhs", "[", 
              RowBox[{"[", "i", "]"}], "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"u", ",", 
              RowBox[{
               RowBox[{"-", "a"}], "-", "1"}]}], "}"}]}], "]"}], 
          RowBox[{"Gamma", "[", 
           RowBox[{"-", "a"}], "]"}]]}]}], "/.", 
       RowBox[{
        SuperscriptBox["u", "a_"], "\[Rule]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Eta]$", "[", 
            RowBox[{"-", "a"}], "]"}], "[", "2", "]"}], "/.", 
          "sub\[Eta]\[Zeta]"}], ")"}]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "3"}], "}"}]}], "]"}]}], "//", "Total"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{"%", ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "30"}], "}"}]}], "]"}], ",", 
   "\[Zeta]list"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"dimensions", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SortBy", "[", 
     RowBox[{
      RowBox[{"List", "@@", 
       RowBox[{"Collect", "[", 
        RowBox[{"%", ",", "g"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{"Exponent", "[", 
        RowBox[{"#", ",", "g"}], "]"}], "&"}]}], "]"}], "/.", 
    RowBox[{"g", "\[Rule]", "1"}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Zeta]", "_"], "\[Rule]", "1"}]}]}]}], "Input",
 CellChangeTimes->{{3.563651263581368*^9, 3.563651388799541*^9}, {
   3.563651790124093*^9, 3.5636518067547607`*^9}, {3.563651880624851*^9, 
   3.563651922340316*^9}, {3.563651992160901*^9, 3.563652006529108*^9}, {
   3.563652060575886*^9, 3.563652080475025*^9}, {3.56365211420543*^9, 
   3.563652159581338*^9}, {3.5636893309795513`*^9, 3.563689331428866*^9}, {
   3.563711631838662*^9, 3.563711632353413*^9}, {3.563715103686124*^9, 
   3.56371515475392*^9}, {3.5637154931713257`*^9, 3.563715495425647*^9}, {
   3.563716734100176*^9, 3.56371674035557*^9}, 3.563716879118513*^9, 
   3.56371710294169*^9, {3.5637225505262747`*^9, 3.5637225853482656`*^9}, {
   3.5637230884809303`*^9, 3.563723108224119*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "1440"}], " ", 
   SuperscriptBox["g", "6"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"30240", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"489888", " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"7318080", " ", 
   SuperscriptBox["g", "12"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"106007616", " ", 
   SuperscriptBox["g", "14"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"1517836320", " ", 
   SuperscriptBox["g", "16"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "15"]}], "-", 
  RowBox[{"21661960320", " ", 
   SuperscriptBox["g", "18"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "17"]}], "+", 
  RowBox[{"309361358592", " ", 
   SuperscriptBox["g", "20"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "19"]}], "-", 
  RowBox[{"4429492179840", " ", 
   SuperscriptBox["g", "22"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "21"]}], "+", 
  RowBox[{"63642506159232", " ", 
   SuperscriptBox["g", "24"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "23"]}], "-", 
  RowBox[{"917920761912000", " ", 
   SuperscriptBox["g", "26"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "25"]}], "+", 
  RowBox[{"13291268236272000", " ", 
   SuperscriptBox["g", "28"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "27"]}], "-", 
  RowBox[{"193199249646748800", " ", 
   SuperscriptBox["g", "30"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "29"]}]}]], "Output",
 CellChangeTimes->{{3.563651323493936*^9, 3.5636513894020653`*^9}, 
   3.563651521949863*^9, 3.563651764244966*^9, {3.56365179991206*^9, 
   3.563651807235351*^9}, {3.563651875058694*^9, 3.563651886781352*^9}, {
   3.563651917917268*^9, 3.563651923849807*^9}, {3.563651989247589*^9, 
   3.563652007267198*^9}, {3.563652063492601*^9, 3.563652081244364*^9}, {
   3.563652117614832*^9, 3.5636521609023523`*^9}, {3.563689321953567*^9, 
   3.5636893318898163`*^9}, 3.56369082016411*^9, 3.5636930548554*^9, {
   3.5637116299460707`*^9, 3.563711647072454*^9}, {3.5637151054508133`*^9, 
   3.5637151554022284`*^9}, 3.563715496663999*^9, {3.5637163190300493`*^9, 
   3.563716347132201*^9}, {3.563716731462322*^9, 3.5637167410291233`*^9}, {
   3.563716874782981*^9, 3.563716879791705*^9}, 3.5637171079273987`*^9, {
   3.5637225512393293`*^9, 3.563722585868188*^9}, {3.5637230898420553`*^9, 
   3.563723109012621*^9}, 3.563732676976307*^9, 3.567148083107842*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "1440"}], ",", "30240", ",", 
   RowBox[{"-", "489888"}], ",", "7318080", ",", 
   RowBox[{"-", "106007616"}], ",", "1517836320", ",", 
   RowBox[{"-", "21661960320"}], ",", "309361358592", ",", 
   RowBox[{"-", "4429492179840"}], ",", "63642506159232", ",", 
   RowBox[{"-", "917920761912000"}], ",", "13291268236272000", ",", 
   RowBox[{"-", "193199249646748800"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.563651323493936*^9, 3.5636513894020653`*^9}, 
   3.563651521949863*^9, 3.563651764244966*^9, {3.56365179991206*^9, 
   3.563651807235351*^9}, {3.563651875058694*^9, 3.563651886781352*^9}, {
   3.563651917917268*^9, 3.563651923849807*^9}, {3.563651989247589*^9, 
   3.563652007267198*^9}, {3.563652063492601*^9, 3.563652081244364*^9}, {
   3.563652117614832*^9, 3.5636521609023523`*^9}, {3.563689321953567*^9, 
   3.5636893318898163`*^9}, 3.56369082016411*^9, 3.5636930548554*^9, {
   3.5637116299460707`*^9, 3.563711647072454*^9}, {3.5637151054508133`*^9, 
   3.5637151554022284`*^9}, 3.563715496663999*^9, {3.5637163190300493`*^9, 
   3.563716347132201*^9}, {3.563716731462322*^9, 3.5637167410291233`*^9}, {
   3.563716874782981*^9, 3.563716879791705*^9}, 3.5637171079273987`*^9, {
   3.5637225512393293`*^9, 3.563722585868188*^9}, {3.5637230898420553`*^9, 
   3.563723109012621*^9}, 3.563732676976307*^9, 3.567148083115719*^9}]
}, Open  ]],

Cell["\<\
This indeed reproduces leading transcendentality from explicitly computed \
expressions\
\>", "Text",
 CellChangeTimes->{{3.563732680673922*^9, 3.563732699201231*^9}}],

Cell["\<\
Although this method of computation is very quick, we can make even better \
and find explicit formula for the coefficients. Let us take a look how \
coefficients are computed in practice\
\>", "Text",
 CellChangeTimes->{{3.5637117467876263`*^9, 3.563711762513299*^9}, {
  3.563732834118766*^9, 3.563732854203066*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      FractionBox[
       RowBox[{"x", "+", 
        FractionBox["1", "x"]}], 
       RowBox[{"x", "-", 
        FractionBox["1", "x"]}]], ")"}], 
     FractionBox["1", 
      SuperscriptBox["x", "a"]], 
     FractionBox[
      SuperscriptBox["D", "2"], 
      RowBox[{"1", "-", 
       SuperscriptBox["D", "2"]}]], 
     FractionBox["1", 
      SuperscriptBox["x", 
       RowBox[{"4", "-", "a"}]]]}], "\[Equal]", 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"k", "=", "0"}], "\[Infinity]"], 
     RowBox[{
      RowBox[{
       SubscriptBox["c", "lhs"], "[", "k", "]"}], 
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", "g"}], ")"}], 
        RowBox[{"k", "+", "a"}]], 
       SuperscriptBox["u", 
        RowBox[{"k", "+", "a"}]]], 
      FractionBox[
       SuperscriptBox["D", "2"], 
       RowBox[{"1", "-", 
        SuperscriptBox["D", "2"]}]], 
      RowBox[{
       SubsuperscriptBox["\[Sum]", 
        RowBox[{"r", "=", "0"}], "\[Infinity]"], 
       RowBox[{
        RowBox[{
         SubscriptBox["c", "lhs"], "[", "r", "]"}], 
        FractionBox[
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"2", "g"}], ")"}], 
          RowBox[{"4", "-", "a", "+", "r"}]], 
         SuperscriptBox["u", 
          RowBox[{"4", "-", "a", "+", "r"}]]]}]}]}]}]}], "\[Rule]", 
   "\[IndentingNewLine]", 
   RowBox[{"\[Rule]", " ", 
    RowBox[{
     RowBox[{"-", "2"}], "g", 
     RowBox[{
      SubsuperscriptBox["\[Sum]", 
       RowBox[{"k", "=", "0"}], "\[Infinity]"], 
      RowBox[{
       RowBox[{
        SubscriptBox["c", "lhs"], "[", "k", "]"}], 
       RowBox[{
        SubsuperscriptBox["\[Sum]", 
         RowBox[{"r", "=", "0"}], "\[Infinity]"], 
        RowBox[{
         RowBox[{
          SubscriptBox["c", "rhs"], "[", "r", "]"}], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"2", "g"}], ")"}], 
          RowBox[{"k", "+", "a", "-", "1"}]], 
         RowBox[{
          FractionBox[
           SubscriptBox["\[PartialD]", 
            RowBox[{"k", "+", "a", "-", "1"}]], 
           RowBox[{"Gamma", "[", 
            RowBox[{"k", "+", "a"}], "]"}]], 
          FractionBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"2", "g"}], ")"}], 
            RowBox[{"4", "-", "a", "+", "r"}]], 
           SuperscriptBox["u", 
            RowBox[{"4", "-", "a", "+", "r"}]]]}]}]}]}]}]}]}]}], "/.", 
  RowBox[{
   SuperscriptBox["u", 
    RowBox[{"-", "a"}]], "\[Rule]", 
   RowBox[{
    SuperscriptBox["\[ImaginaryI]", 
     RowBox[{"-", "a"}]], 
    SubscriptBox["\[Zeta]", "a"]}]}]}]], "Input",
 CellChangeTimes->{{3.563711836364029*^9, 3.56371208867737*^9}, {
  3.563712163857523*^9, 3.563712173901123*^9}, {3.5637122396996403`*^9, 
  3.563712315267954*^9}, {3.563732871786923*^9, 3.563732886821911*^9}}],

Cell[TextData[{
 "So lhs (with respect to ",
 Cell[BoxData[
  FractionBox[
   SuperscriptBox["D", "2"], 
   RowBox[{"1", "-", 
    SuperscriptBox["D", "2"]}]]],
  CellChangeTimes->{{3.563711836364029*^9, 3.56371208867737*^9}, {
   3.563712163857523*^9, 3.563712173901123*^9}, {3.5637122396996403`*^9, 
   3.563712315267954*^9}, {3.563732871786923*^9, 3.563732875743306*^9}}],
 ") essentially acts as derivative, except that we have to also divide by ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    RowBox[{"Gamma", "[", "k", "]"}]], TraditionalForm]]],
 ". This division can be modeled by contour integral:"
}], "Text",
 CellChangeTimes->{{3.563712059589521*^9, 3.5637121390574713`*^9}, {
  3.563732867598832*^9, 3.563732893468985*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"z", " ", "u"}]], 
     SuperscriptBox["u", "b"]], 
    FractionBox[
     RowBox[{"\[DifferentialD]", "u"}], 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"k", "=", "0"}], "\[Infinity]"], 
   RowBox[{
    FractionBox["1", 
     RowBox[{"Gamma", "[", 
      RowBox[{"k", "+", "1"}], "]"}]], 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"z", " ", "u"}], ")"}], "k"], 
     SuperscriptBox["u", "b"]], 
    FractionBox[
     RowBox[{"\[DifferentialD]", "u"}], 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"Gamma", "[", "b", "]"}]], 
   SuperscriptBox["z", 
    RowBox[{"b", "-", "1"}]]}]}]], "Input",
 CellChangeTimes->{{3.563690078236952*^9, 3.563690144110783*^9}, {
  3.563690174209565*^9, 3.5636902163507833`*^9}, {3.563712464595559*^9, 
  3.563712486346107*^9}}],

Cell[TextData[{
 "So let us ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "lhs"], TraditionalForm]]],
 "[z]==",
 Cell[BoxData[
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    RowBox[{"lhs", "[", "u", "]"}], 
    FractionBox[
     RowBox[{"\[DifferentialD]", "u"}], 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]],
  CellChangeTimes->{{3.563690078236952*^9, 3.563690144110783*^9}, {
   3.563690174209565*^9, 3.5636902163507833`*^9}}]
}], "Text",
 CellChangeTimes->{{3.56371219816019*^9, 3.563712209024152*^9}, {
  3.563712337118019*^9, 3.5637123393415937`*^9}, {3.563714224916423*^9, 
  3.563714226843382*^9}}],

Cell["Then what we have to compute is", "Text",
 CellChangeTimes->{{3.563712298635767*^9, 3.5637123032682543`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["f", "lhs"], "[", 
     SubscriptBox["\[PartialD]", "u"], "]"}]}], 
   RowBox[{"rhs", "[", 
    FractionBox[
     RowBox[{"2", "g"}], "u"], "]"}]}], "/.", 
  RowBox[{
   SuperscriptBox["u", 
    RowBox[{"-", "a"}]], "\[Rule]", 
   RowBox[{
    SuperscriptBox["\[ImaginaryI]", 
     RowBox[{"-", "a"}]], 
    SubscriptBox["\[Zeta]", "a"]}]}]}]], "Input",
 CellChangeTimes->{{3.563712304264333*^9, 3.563712364662693*^9}, 
   3.563712494430194*^9, {3.563714228696233*^9, 3.563714229387043*^9}, 
   3.5637145702078753`*^9, 3.563732942282906*^9}],

Cell["\<\
To not make derivatives, we will actually make Fourier once more:\
\>", "Text",
 CellChangeTimes->{{3.5637141520912848`*^9, 3.563714186849498*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"z", " ", "u"}]], 
    FractionBox[
     RowBox[{"\[DifferentialD]", "u"}], 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "2"}], "g", " ", 
      RowBox[{
       SubscriptBox["f", "lhs"], "[", 
       SubscriptBox["\[PartialD]", "u"], "]"}], 
      RowBox[{"rhs", "[", 
       FractionBox[
        RowBox[{"2", "g"}], "u"], "]"}]}], ")"}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{
   RowBox[{"-", "2"}], "g", " ", 
   RowBox[{
    SubscriptBox["f", "lhs"], "[", 
    RowBox[{"-", "z"}], "]"}], 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"z", " ", "u"}]], 
     FractionBox[
      RowBox[{"\[DifferentialD]", "u"}], 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"rhs", "[", 
      FractionBox[
       RowBox[{"2", "g"}], "u"], "]"}]}]}]}], "==", 
  RowBox[{
   RowBox[{"-", "2"}], "g", " ", 
   RowBox[{
    SubscriptBox["f", "lhs"], "[", 
    RowBox[{"-", "z"}], "]"}], 
   RowBox[{
    SubscriptBox["f", "rhs"], "[", "z", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.563714217109358*^9, 3.563714280629611*^9}, {
  3.563714596069653*^9, 3.563714625141367*^9}, {3.563716628423236*^9, 
  3.563716629558865*^9}}],

Cell["Let us find f now for the function in question", "Text",
 CellChangeTimes->{{3.563712511062118*^9, 3.563712529403771*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    RowBox[{"(", 
     FractionBox[
      RowBox[{"x", "+", 
       FractionBox["1", "x"]}], 
      RowBox[{"x", "-", 
       FractionBox["1", "x"]}]], ")"}], 
    FractionBox["1", 
     SuperscriptBox["x", "a"]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"z", " ", "u"}]], 
    FractionBox["du", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      FractionBox[
       RowBox[{"x", "+", 
        FractionBox["1", "x"]}], 
       RowBox[{"x", "-", 
        FractionBox["1", "x"]}]], ")"}], 
     FractionBox["1", 
      SuperscriptBox["x", "a"]], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"g", " ", "z", 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         FractionBox["1", "x"]}], ")"}]}]], 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SuperscriptBox["x", "2"]]}], ")"}], 
     FractionBox["dx", 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{
         RowBox[{"-", "2"}], "-", "a"}]], "+", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        RowBox[{"-", "a"}]]}], ")"}], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", "g", " ", "z", " ", 
       RowBox[{"Cos", "[", "\[Phi]", "]"}]}]], 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]}], "\[Equal]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"a", "+", "1"}], ")"}]}], "\[Phi]"}]], "+", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"a", "-", "1"}], ")"}]}], "\[Phi]"}]]}], ")"}], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", "g", " ", "z", " ", 
       RowBox[{"Cos", "[", "\[Phi]", "]"}]}]], 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "+", "1"}], ")"}], 
         RowBox[{"(", 
          RowBox[{"\[Phi]", "+", 
           FractionBox["\[Pi]", "2"]}], ")"}]}]], "+", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "-", "1"}], ")"}], 
         RowBox[{"(", 
          RowBox[{"\[Phi]", "+", 
           FractionBox["\[Pi]", "2"]}], ")"}]}]]}], ")"}], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "2"}], "g", " ", "z", " ", 
       RowBox[{"Sin", "[", "\[Phi]", "]"}]}]], 
     FractionBox[
      RowBox[{" ", "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]"}]]}]}]}], "\[Equal]", "\[IndentingNewLine]", 
  "\[Equal]", 
  RowBox[{"g", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "1"}], ")"}]], 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", 
       RowBox[{"(", 
        RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z"}], ")"}], " ", 
       RowBox[{"Sin", "[", "\[Phi]", "]"}]}]], 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "+", "1"}], ")"}], "\[Phi]"}]], "-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "-", "1"}], ")"}], "\[Phi]"}]]}], ")"}], 
     FractionBox[
      RowBox[{" ", "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]"}]]}]}]}], "\[Equal]", "\[IndentingNewLine]", 
  "\[Equal]", 
  RowBox[{"g", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"a", "+", "1"}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", 
       RowBox[{"a", "+", "1"}]], "[", 
      RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["J", 
       RowBox[{"a", "-", "1"}]], "[", 
      RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.563712539895719*^9, 3.5637125971508493`*^9}, {
  3.563712629434134*^9, 3.563712718731204*^9}, {3.563712837533313*^9, 
  3.563713114247881*^9}, {3.563713292111334*^9, 3.563713306928577*^9}, {
  3.5637140309665194`*^9, 3.563714034363871*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["J", "n"], "[", "z", "]"}], "\[Equal]", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"2", "\[Pi]"}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "\[Pi]"}], "\[Pi]"], 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", "n\[Tau]"}], "+", 
       RowBox[{"\[ImaginaryI]", " ", "z", " ", 
        RowBox[{"Sin", "[", "\[Tau]", "]"}]}]}]], 
     RowBox[{"\[DifferentialD]", "\[Tau]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5637140678997726`*^9, 3.563714068316369*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"g", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
            RowBox[{"a", "+", "1"}]], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SubscriptBox["J", 
               RowBox[{"a", "+", "1"}]], "[", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}], 
             "-", 
             RowBox[{
              SubscriptBox["J", 
               RowBox[{"a", "-", "1"}]], "[", 
              RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}]}], 
            ")"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "r_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"r", ",", "#"}], "]"}], "&"}], ")"}]}]}], "/.", 
         RowBox[{"a", "\[Rule]", "#"}]}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{"1", "/", 
          SqrtBox["3"]}]}]}], "/.", 
       RowBox[{"z", "\[Rule]", 
        RowBox[{"1", "/", "3"}]}]}], "//", "N"}], ")"}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]}], "//", 
  "Chop"}]], "Input",
 CellChangeTimes->{{3.5637131186406107`*^9, 3.563713240284205*^9}, {
  3.563713309409254*^9, 3.563713332331717*^9}, {3.563713448177586*^9, 
  3.563713591661232*^9}, {3.5637138231954403`*^9, 3.5637138497462606`*^9}, {
  3.5637140377791157`*^9, 3.56371404487436*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.6097567032926187`", ",", "0.1138737101535551`", ",", 
   "0.010857524380805469`", ",", "0.0006935232762636682`", ",", 
   "0.000033285134826036556`"}], "}"}]], "Output",
 CellChangeTimes->{{3.56371312095527*^9, 3.563713240857807*^9}, {
   3.5637133198370743`*^9, 3.563713332802545*^9}, {3.5637134489424553`*^9, 
   3.563713532307581*^9}, {3.563713587496565*^9, 3.5637135920203133`*^9}, {
   3.563713823757523*^9, 3.5637138529260283`*^9}, 3.5637140454567537`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             FractionBox[
              RowBox[{"x", "+", 
               FractionBox["1", "x"]}], 
              RowBox[{"x", "-", 
               FractionBox["1", "x"]}]], ")"}], 
            FractionBox["1", 
             SuperscriptBox["x", "a"]], 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{"z", " ", "u"}]], 
            FractionBox["1", 
             RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], "/.", 
           RowBox[{"x$", "\[Rule]", "x$h"}]}], "/.", "subJuk"}], "/.", 
         RowBox[{"z", "\[Rule]", 
          RowBox[{"1", "/", "3"}]}]}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{"1", "/", 
          SqrtBox["3"]}]}]}], "/.", 
       RowBox[{"a", "\[Rule]", "#"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"u", ",", "2", ",", "\[ImaginaryI]", ",", 
        RowBox[{"-", "2"}], ",", 
        RowBox[{"-", "\[ImaginaryI]"}], ",", "2"}], "}"}]}], "]"}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]}], "//", 
  "Chop"}]], "Input",
 CellChangeTimes->{{3.563713336548839*^9, 3.563713370884944*^9}, {
  3.5637136090118217`*^9, 3.5637136280603447`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.6097567032923957`", ",", "0.11387371015356275`", ",", 
   "0.010857524380805082`", ",", "0.0006935232762636761`", ",", 
   "0.00003328513482603844`"}], "}"}]], "Output",
 CellChangeTimes->{3.563713628504252*^9, 3.563714052679265*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox["x", "a"]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"z", " ", "u"}]], 
    FractionBox["du", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox["x", "a"]], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"g", " ", "z", 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         FractionBox["1", "x"]}], ")"}]}]], 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SuperscriptBox["x", "2"]]}], ")"}], 
     FractionBox["dx", 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "a"}]], "-", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        RowBox[{
         RowBox[{"-", "a"}], "-", "2"}]]}], ")"}], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", "g", " ", "z", " ", 
       RowBox[{"Cos", "[", "\[Phi]", "]"}]}]], 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]}], "\[Equal]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"a", "-", "1"}], ")"}]}], "\[Phi]"}]], "-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"a", "+", "1"}], ")"}]}], "\[Phi]"}]]}], ")"}], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", "g", " ", "z", " ", 
       RowBox[{"Cos", "[", "\[Phi]", "]"}]}]], 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", 
  RowBox[{"g", 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "-", "1"}], ")"}], 
         RowBox[{"(", 
          RowBox[{"\[Phi]", "+", 
           FractionBox["\[Pi]", "2"]}], ")"}]}]], "-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "+", "1"}], ")"}], 
         RowBox[{"(", 
          RowBox[{"\[Phi]", "+", 
           FractionBox["\[Pi]", "2"]}], ")"}]}]]}], ")"}], 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "2"}], "g", " ", "z", " ", 
       RowBox[{"Sin", "[", "\[Phi]", "]"}]}]], 
     FractionBox[
      RowBox[{" ", "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]"}]]}]}]}], "\[Equal]", "\[IndentingNewLine]", 
  "\[Equal]", 
  RowBox[{
   RowBox[{"-", "g"}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "1"}], ")"}]], 
   RowBox[{"\[ContourIntegral]", 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", 
       RowBox[{"(", 
        RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z"}], ")"}], " ", 
       RowBox[{"Sin", "[", "\[Phi]", "]"}]}]], 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "+", "1"}], ")"}], "\[Phi]"}]], "+", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], 
         RowBox[{"(", 
          RowBox[{"a", "-", "1"}], ")"}], "\[Phi]"}]]}], ")"}], 
     FractionBox[
      RowBox[{" ", "d\[Phi]"}], 
      RowBox[{"2", "\[Pi]"}]]}]}]}], "\[Equal]", "\[IndentingNewLine]", 
  "\[Equal]", 
  RowBox[{
   RowBox[{"-", "g"}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"a", "+", "1"}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", 
       RowBox[{"a", "+", "1"}]], "[", 
      RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}], "+", 
     RowBox[{
      SubscriptBox["J", 
       RowBox[{"a", "-", "1"}]], "[", 
      RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.563712539895719*^9, 3.5637125971508493`*^9}, {
  3.563712629434134*^9, 3.563712718731204*^9}, {3.563712837533313*^9, 
  3.563713114247881*^9}, {3.563713292111334*^9, 3.563713306928577*^9}, {
  3.5637140309665194`*^9, 3.563714034363871*^9}, {3.563714685313017*^9, 
  3.563714759071412*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"-", "g"}], 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
            RowBox[{"a", "+", "1"}]], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SubscriptBox["J", 
               RowBox[{"a", "+", "1"}]], "[", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}], 
             "+", 
             RowBox[{
              SubscriptBox["J", 
               RowBox[{"a", "-", "1"}]], "[", 
              RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}]}], 
            ")"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "r_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"r", ",", "#"}], "]"}], "&"}], ")"}]}]}], "/.", 
         RowBox[{"a", "\[Rule]", "#"}]}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{"1", "/", 
          SqrtBox["3"]}]}]}], "/.", 
       RowBox[{"z", "\[Rule]", 
        RowBox[{"1", "/", "3"}]}]}], "//", "N"}], ")"}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]}], "//", 
  "Chop"}]], "Input",
 CellChangeTimes->{{3.5637131186406107`*^9, 3.563713240284205*^9}, {
  3.563713309409254*^9, 3.563713332331717*^9}, {3.563713448177586*^9, 
  3.563713591661232*^9}, {3.5637138231954403`*^9, 3.5637138497462606`*^9}, {
  3.5637140377791157`*^9, 3.56371404487436*^9}, {3.563714767565997*^9, 
  3.563714770645612*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.5881081428895755`", ",", "0.11248921958638315`", ",", 
   "0.010791036022237613`", ",", "0.0006909672909082945`", ",", 
   "0.000033203223741818516`"}], "}"}]], "Output",
 CellChangeTimes->{{3.56371312095527*^9, 3.563713240857807*^9}, {
   3.5637133198370743`*^9, 3.563713332802545*^9}, {3.5637134489424553`*^9, 
   3.563713532307581*^9}, {3.563713587496565*^9, 3.5637135920203133`*^9}, {
   3.563713823757523*^9, 3.5637138529260283`*^9}, 3.5637140454567537`*^9, 
   3.563714770954484*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            FractionBox["1", 
             SuperscriptBox["x", "a"]], 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{"z", " ", "u"}]], 
            FractionBox["1", 
             RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], "/.", 
           RowBox[{"x$", "\[Rule]", "x$h"}]}], "/.", "subJuk"}], "/.", 
         RowBox[{"z", "\[Rule]", 
          RowBox[{"1", "/", "3"}]}]}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{"1", "/", 
          SqrtBox["3"]}]}]}], "/.", 
       RowBox[{"a", "\[Rule]", "#"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"u", ",", "2", ",", "\[ImaginaryI]", ",", 
        RowBox[{"-", "2"}], ",", 
        RowBox[{"-", "\[ImaginaryI]"}], ",", "2"}], "}"}]}], "]"}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]}], "//", 
  "Chop"}]], "Input",
 CellChangeTimes->{{3.563713336548839*^9, 3.563713370884944*^9}, {
   3.5637136090118217`*^9, 3.5637136280603447`*^9}, 3.563714785110373*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.5881081428894197`", ",", "0.1124892195864847`", ",", 
   "0.010791036022239214`", ",", "0.0006909672909082799`", ",", 
   "0.00003320322374182009`"}], "}"}]], "Output",
 CellChangeTimes->{3.563713628504252*^9, 3.563714052679265*^9, 
  3.563714787178491*^9}]
}, Open  ]],

Cell["\<\
So, we have for function that produces anomalous dimensions (one should \
multiply by factorial to get them\
\>", "Text",
 CellChangeTimes->{{3.563714798649897*^9, 3.5637148198807983`*^9}, {
  3.5637148808644333`*^9, 3.563714890477597*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"2", "\[ImaginaryI]"}], ")"}], 
        RowBox[{"(", 
         RowBox[{"-", "36"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"newlhs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "/.", 
          RowBox[{"z", "\[Rule]", 
           RowBox[{"-", "z"}]}]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"newrhs", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "0", ",", "10"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"%", "/.", 
     RowBox[{
      SuperscriptBox["z", "a_"], "\[RuleDelayed]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Eta]$", "[", 
           RowBox[{"a", "+", "1"}], "]"}], "[", "2", "]"}], "/.", 
         "sub\[Eta]\[Zeta]"}], ")"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{"a", "+", "1"}], "]"}]}]}]}], "//", "Expand"}], ",", 
   SubscriptBox["\[Zeta]", "_"]}], "]"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"SortBy", "[", 
    RowBox[{
     RowBox[{"List", "@@", 
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{"2", "\[ImaginaryI]"}], ")"}]}], 
            RowBox[{"(", 
             RowBox[{"-", "36"}], ")"}], 
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"a", "\[Equal]", "3"}], ",", 
                 RowBox[{"-", "2"}], ",", "1"}], "]"}], 
               RowBox[{"(", 
                RowBox[{"g", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
                  RowBox[{"a", "+", "1"}]], 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"a", "+", "1"}]], "[", 
                    RowBox[{
                    RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", 
                    "z"}], "]"}], "-", 
                   RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"a", "-", "1"}]], "[", 
                    RowBox[{
                    RowBox[{"-", "2"}], "\[ImaginaryI]", " ", "g", " ", "z"}],
                     "]"}]}], ")"}]}], ")"}], 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", "g"}], 
                  SuperscriptBox[
                   RowBox[{"(", 
                    RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
                   RowBox[{"aa", "+", "1"}]], 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"aa", "+", "1"}]], "[", 
                    RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], "]"}],
                     "+", 
                    RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"aa", "-", "1"}]], "[", 
                    RowBox[{"2", "\[ImaginaryI]", " ", "g", " ", "z"}], 
                    "]"}]}], ")"}]}], "/.", 
                 RowBox[{"aa", "\[Rule]", 
                  RowBox[{"6", "-", "a"}]}]}], ")"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"a", ",", "2", ",", "4"}], "}"}]}], "]"}]}], "/.", 
           RowBox[{
            SubscriptBox["J", "r_"], "\[RuleDelayed]", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"BesselJ", "[", 
               RowBox[{"r", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
          RowBox[{"{", 
           RowBox[{"g", ",", "0", ",", "28"}], "}"}]}], "]"}], ",", "z"}], 
       "]"}]}], ",", 
     RowBox[{
      RowBox[{"Exponent", "[", 
       RowBox[{"#", ",", "z"}], "]"}], "&"}]}], "]"}], "/.", 
   RowBox[{"g", "\[Rule]", "1"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"dimfromgen", "=", 
  RowBox[{"%", "/.", 
   RowBox[{
    SuperscriptBox["z", "a_"], "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ImaginaryI]", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "a"}]], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], 
        RowBox[{"1", "+", "a"}]]}], ")"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{"a", "+", "1"}], "]"}]}]}]}]}]}], "Input",
 CellChangeTimes->{{3.563714862124329*^9, 3.563714976430936*^9}, {
   3.563715009734249*^9, 3.563715033907403*^9}, {3.563715170918679*^9, 
   3.563715192445868*^9}, {3.5637152430985937`*^9, 3.563715282837481*^9}, 
   3.563715360956068*^9, {3.5637154206029263`*^9, 3.563715427819405*^9}, {
   3.563715641797039*^9, 3.563715649815175*^9}, {3.563715680438167*^9, 
   3.56371586864005*^9}, {3.56371592426926*^9, 3.5637159292859793`*^9}, {
   3.56371647738599*^9, 3.563716518310507*^9}, {3.563716578741156*^9, 
   3.563716602752324*^9}, {3.563716678682065*^9, 3.563716721400326*^9}, {
   3.563716755382876*^9, 3.5637167814045753`*^9}, 3.563716834016795*^9, {
   3.563716896352766*^9, 3.5637170158709183`*^9}, {3.5637170472753687`*^9, 
   3.5637170798493557`*^9}, {3.56371718263724*^9, 3.563717182749489*^9}, {
   3.5637172423307533`*^9, 3.5637172715365753`*^9}, {3.563732955469859*^9, 
   3.5637329866896877`*^9}, 3.563733029657304*^9, {3.563733099622262*^9, 
   3.563733114804987*^9}, {3.56373316077316*^9, 3.563733191073697*^9}, 
   3.563733249361669*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "1440"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "5"]}], ",", 
   RowBox[{"30240", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "7"]}], ",", 
   RowBox[{
    RowBox[{"-", "489888"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "9"]}], ",", 
   RowBox[{"7318080", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "11"]}], ",", 
   RowBox[{
    RowBox[{"-", "106007616"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "13"]}], ",", 
   RowBox[{"1517836320", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "15"]}], ",", 
   RowBox[{
    RowBox[{"-", "21661960320"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "17"]}], ",", 
   RowBox[{"309361358592", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "19"]}], ",", 
   RowBox[{
    RowBox[{"-", "4429492179840"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "21"]}], ",", 
   RowBox[{"63642506159232", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "23"]}], ",", 
   RowBox[{
    RowBox[{"-", "917920761912000"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "25"]}], ",", 
   RowBox[{"13291268236272000", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "27"]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.563732972928789*^9, 3.56373303060312*^9}, {
  3.5637331005647078`*^9, 3.563733115420249*^9}, {3.563733163944727*^9, 
  3.5637331921190157`*^9}}]
}, Open  ]],

Cell["\<\
First thing to do is to redefine z, so that we don\[CloseCurlyQuote]t have \
burden with \[ImaginaryI]\
\>", "Text",
 CellChangeTimes->{{3.5637332554756308`*^9, 3.563733274107195*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"SortBy", "[", 
    RowBox[{
     RowBox[{"List", "@@", 
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", "2", ")"}]}], 
            RowBox[{"(", 
             RowBox[{"-", "36"}], ")"}], 
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"a", "\[Equal]", "3"}], ",", 
                 RowBox[{"-", "2"}], ",", "1"}], "]"}], 
               RowBox[{"(", 
                RowBox[{"g", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
                  RowBox[{"a", "+", "1"}]], 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"a", "+", "1"}]], "[", 
                    RowBox[{
                    RowBox[{"-", "2"}], "  ", "g", " ", "z"}], "]"}], "-", 
                   RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"a", "-", "1"}]], "[", 
                    RowBox[{
                    RowBox[{"-", "2"}], " ", "g", " ", "z"}], "]"}]}], 
                  ")"}]}], ")"}], 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", "g"}], 
                  SuperscriptBox[
                   RowBox[{"(", 
                    RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
                   RowBox[{"aa", "+", "1"}]], 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"aa", "+", "1"}]], "[", 
                    RowBox[{"2", " ", "g", " ", "z"}], "]"}], "+", 
                    RowBox[{
                    SubscriptBox["J", 
                    RowBox[{"aa", "-", "1"}]], "[", 
                    RowBox[{"2", " ", "g", " ", "z"}], "]"}]}], ")"}]}], "/.", 
                 RowBox[{"aa", "\[Rule]", 
                  RowBox[{"6", "-", "a"}]}]}], ")"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"a", ",", "2", ",", "4"}], "}"}]}], "]"}]}], "/.", 
           RowBox[{
            SubscriptBox["J", "r_"], "\[RuleDelayed]", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"BesselJ", "[", 
               RowBox[{"r", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
          RowBox[{"{", 
           RowBox[{"g", ",", "0", ",", "28"}], "}"}]}], "]"}], ",", "z"}], 
       "]"}]}], ",", 
     RowBox[{
      RowBox[{"Exponent", "[", 
       RowBox[{"#", ",", "z"}], "]"}], "&"}]}], "]"}], "/.", 
   RowBox[{"g", "\[Rule]", "1"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"dimfromgen", "=", 
  RowBox[{"%", "/.", 
   RowBox[{
    SuperscriptBox["z", "a_"], "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"(", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], 
       RowBox[{"1", "+", "a"}]], ")"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{"a", "+", "1"}], "]"}]}]}]}]}]}], "Input",
 CellChangeTimes->{{3.5637332840163727`*^9, 3.563733293525785*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "1440"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "5"]}], ",", 
   RowBox[{"30240", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "7"]}], ",", 
   RowBox[{
    RowBox[{"-", "489888"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "9"]}], ",", 
   RowBox[{"7318080", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "11"]}], ",", 
   RowBox[{
    RowBox[{"-", "106007616"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "13"]}], ",", 
   RowBox[{"1517836320", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "15"]}], ",", 
   RowBox[{
    RowBox[{"-", "21661960320"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "17"]}], ",", 
   RowBox[{"309361358592", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "19"]}], ",", 
   RowBox[{
    RowBox[{"-", "4429492179840"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "21"]}], ",", 
   RowBox[{"63642506159232", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "23"]}], ",", 
   RowBox[{
    RowBox[{"-", "917920761912000"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "25"]}], ",", 
   RowBox[{"13291268236272000", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "27"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5637332965868797`*^9}]
}, Open  ]],

Cell["We can now explicitly simplify the Bessel functions", "Text",
 CellChangeTimes->{{3.563733217350896*^9, 3.563733230092245*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", "2", ")"}]}], 
    RowBox[{"(", 
     RowBox[{"-", "36"}], ")"}], 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"a", "\[Equal]", "3"}], ",", 
         RowBox[{"-", "2"}], ",", "1"}], "]"}], 
       RowBox[{"(", 
        RowBox[{"g", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
          RowBox[{"a", "+", "1"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["J", 
             RowBox[{"a", "+", "1"}]], "[", 
            RowBox[{
             RowBox[{"-", "2"}], "  ", "g", " ", "z"}], "]"}], "-", 
           RowBox[{
            SubscriptBox["J", 
             RowBox[{"a", "-", "1"}]], "[", 
            RowBox[{
             RowBox[{"-", "2"}], " ", "g", " ", "z"}], "]"}]}], ")"}]}], 
        ")"}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "g"}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
           RowBox[{"aa", "+", "1"}]], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SubscriptBox["J", 
              RowBox[{"aa", "+", "1"}]], "[", 
             RowBox[{"2", " ", "g", " ", "z"}], "]"}], "+", 
            RowBox[{
             SubscriptBox["J", 
              RowBox[{"aa", "-", "1"}]], "[", 
             RowBox[{"2", " ", "g", " ", "z"}], "]"}]}], ")"}]}], "/.", 
         RowBox[{"aa", "\[Rule]", 
          RowBox[{"6", "-", "a"}]}]}], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "2", ",", "4"}], "}"}]}], "]"}]}], "/.", 
   RowBox[{
    SubscriptBox["J", "r_"], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"BesselJ", "[", 
       RowBox[{"r", ",", "#"}], "]"}], "&"}], ")"}]}]}], "//", 
  "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.563733309887944*^9, 3.563733312070055*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "144"}], " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"BesselJ", "[", 
      RowBox[{"2", ",", 
       RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], "-", 
    SuperscriptBox[
     RowBox[{"BesselJ", "[", 
      RowBox[{"4", ",", 
       RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], "+", 
    RowBox[{
     RowBox[{"BesselJ", "[", 
      RowBox[{"3", ",", 
       RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"BesselJ", "[", 
        RowBox[{"1", ",", 
         RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "-", 
       RowBox[{"BesselJ", "[", 
        RowBox[{"5", ",", 
         RowBox[{"2", " ", "g", " ", "z"}]}], "]"}]}], ")"}]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.563733313446453*^9}]
}, Open  ]],

Cell["\<\
This replacement actually means back Laplace transform+ summation to get \
zeta-functions:\
\>", "Text",
 CellChangeTimes->{{3.5637333339444723`*^9, 3.563733353024497*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["z", "a_"], "\[RuleDelayed]", 
  RowBox[{
   RowBox[{"(", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"1", "+", "a"}]], ")"}], 
   RowBox[{"Gamma", "[", 
    RowBox[{"a", "+", "1"}], "]"}]}]}]], "Input"],

Cell["The net result is", "Text",
 CellChangeTimes->{{3.563733357265091*^9, 3.563733359647387*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "144"}], " ", 
   SuperscriptBox["g", "4"], " ", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"BesselJ", "[", 
           RowBox[{"2", ",", 
            RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], "-", 
         SuperscriptBox[
          RowBox[{"BesselJ", "[", 
           RowBox[{"4", ",", 
            RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], "+", 
         RowBox[{
          RowBox[{"BesselJ", "[", 
           RowBox[{"3", ",", 
            RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"BesselJ", "[", 
             RowBox[{"1", ",", 
              RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "-", 
            RowBox[{"BesselJ", "[", 
             RowBox[{"5", ",", 
              RowBox[{"2", " ", "g", " ", "z"}]}], "]"}]}], ")"}]}]}], ")"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], ",", "g", ",", 
     RowBox[{
      RowBox[{"Integrate", "[", 
       RowBox[{
        FractionBox["#", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}]], ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], "&"}]}],
     "]"}]}], "//", "Expand"}]], "Input",
 CellChangeTimes->{{3.563733399834118*^9, 3.563733431078714*^9}, {
  3.563733462152269*^9, 3.563733497301858*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "1440"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"Zeta", "[", "5", "]"}]}], "+", 
  RowBox[{"30240", " ", 
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"Zeta", "[", "7", "]"}]}], "-", 
  RowBox[{"489888", " ", 
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"Zeta", "[", "9", "]"}]}]}]], "Output",
 CellChangeTimes->{{3.563733460060194*^9, 3.563733498828969*^9}}]
}, Open  ]],

Cell[TextData[{
 "Using properties of Bessel functions, one can note that Bessels organize \
into ",
 Cell[BoxData[
  RowBox[{
   FractionBox["3", 
    SuperscriptBox["g", "2"]], " ", 
   RowBox[{"D", "[", 
    RowBox[{
     FractionBox[
      SuperscriptBox[
       RowBox[{"BesselJ", "[", 
        RowBox[{"3", ",", 
         RowBox[{"2", "g", " ", "z"}]}], "]"}], "2"], 
      RowBox[{" ", "z"}]], ",", "z"}], "]"}]}]],
  CellChangeTimes->{{3.563733530849264*^9, 3.5637335632987337`*^9}, {
   3.563733651612928*^9, 3.5637336673516893`*^9}}],
 ":"
}], "Text",
 CellChangeTimes->{{3.56373350963395*^9, 3.56373353001676*^9}, {
  3.563733671531118*^9, 3.563733672638423*^9}, {3.5637337131587276`*^9, 
  3.56373371651012*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["3", 
      SuperscriptBox["g", "2"]], " ", 
     RowBox[{"D", "[", 
      RowBox[{
       FractionBox[
        SuperscriptBox[
         RowBox[{"BesselJ", "[", 
          RowBox[{"3", ",", 
           RowBox[{"2", "g", " ", "z"}]}], "]"}], "2"], 
        RowBox[{" ", "z"}]], ",", "z"}], "]"}]}], ")"}], "-", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"BesselJ", "[", 
       RowBox[{"2", ",", 
        RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], "-", 
     SuperscriptBox[
      RowBox[{"BesselJ", "[", 
       RowBox[{"4", ",", 
        RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], "+", 
     RowBox[{
      RowBox[{"BesselJ", "[", 
       RowBox[{"3", ",", 
        RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"BesselJ", "[", 
         RowBox[{"1", ",", 
          RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "-", 
        RowBox[{"BesselJ", "[", 
         RowBox[{"5", ",", 
          RowBox[{"2", " ", "g", " ", "z"}]}], "]"}]}], ")"}]}]}], ")"}]}], "//",
   "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.563733530849264*^9, 3.5637335632987337`*^9}, {
  3.563733651612928*^9, 3.5637336673516893`*^9}, {3.5637337071819363`*^9, 
  3.56373371040926*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.563733564716586*^9, 3.563733653136044*^9, {3.5637337108399677`*^9, 
   3.563733718095574*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "432"}], " ", 
   SuperscriptBox["g", "2"], " ", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         FractionBox[
          SuperscriptBox[
           RowBox[{"BesselJ", "[", 
            RowBox[{"3", ",", 
             RowBox[{"2", "g", " ", "z"}]}], "]"}], "2"], 
          RowBox[{" ", "z"}]], ",", "z"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}], ",", "g", ",", 
     RowBox[{
      RowBox[{"Integrate", "[", 
       RowBox[{
        FractionBox["#", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}]], ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], "&"}]}],
     "]"}]}], "//", "Expand"}]], "Input",
 CellChangeTimes->{{3.5637336989350157`*^9, 3.563733738135227*^9}, {
  3.56373399328271*^9, 3.563733993426487*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "1440"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"Zeta", "[", "5", "]"}]}], "+", 
  RowBox[{"30240", " ", 
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"Zeta", "[", "7", "]"}]}], "-", 
  RowBox[{"489888", " ", 
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"Zeta", "[", "9", "]"}]}]}]], "Output",
 CellChangeTimes->{{3.5637337321761713`*^9, 3.563733739788289*^9}, 
   3.563733996576199*^9, 3.5671481845539513`*^9}]
}, Open  ]],

Cell["\<\
Derivative can be flipped and the ultimate expression to use is\
\>", "Text",
 CellChangeTimes->{{3.563733748304881*^9, 3.563733760663611*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["Energy", "leadtran"], "=", 
   RowBox[{
    RowBox[{"-", "432"}], " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["\[ExponentialE]", "z"], " ", 
        SuperscriptBox[
         TemplateBox[{"3",RowBox[{"2", " ", "g", " ", "z"}]},
          "BesselJ"], "2"]}], 
       RowBox[{"z", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}], ")"}], "2"]}]], 
      RowBox[{"\[DifferentialD]", "z"}]}]}]}]}], TraditionalForm]], "Input",
 CellChangeTimes->{{3.563733767327158*^9, 3.563733807273047*^9}, {
  3.563733997746306*^9, 3.563733997865631*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "432"}], " ", 
   SuperscriptBox["g", "2"], " ", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"BesselJ", "[", 
         RowBox[{"3", ",", 
          RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}], ",", "g", ",", 
     RowBox[{
      RowBox[{"Integrate", "[", 
       RowBox[{
        RowBox[{
         FractionBox[
          SuperscriptBox["\[ExponentialE]", "z"], 
          RowBox[{"z", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}], ")"}], 
            "2"]}]], "#"}], ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], "&"}]}],
     "]"}]}], "//", "Expand"}]], "Input",
 CellChangeTimes->{{3.5637338224241123`*^9, 3.563733892014867*^9}, {
  3.563734000026215*^9, 3.563734000176593*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "1440"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"Zeta", "[", "5", "]"}]}], "+", 
  RowBox[{"30240", " ", 
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"Zeta", "[", "7", "]"}]}], "-", 
  RowBox[{"489888", " ", 
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"Zeta", "[", "9", "]"}]}]}]], "Output",
 CellChangeTimes->{3.563733896622963*^9, 3.563734004636208*^9, 
  3.564583606027693*^9, 3.567148189454194*^9}]
}, Open  ]],

Cell["\<\
An ultrafast way to proceed is to use this code:\
\>", "Text",
 CellChangeTimes->{{3.563733968177578*^9, 3.563734010213921*^9}, {
  3.568969561116085*^9, 3.568969561276079*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "432"}], 
        SuperscriptBox["g", "2"]}], ")"}], " ", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"BesselJ", "[", 
          RowBox[{"3", ",", 
           RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], " "}], 
       RowBox[{"z", " "}]]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "30"}], "}"}]}], "]"}], ",", ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"#", "/.", 
      RowBox[{
       SuperscriptBox["z", "a_"], "\[RuleDelayed]", 
       RowBox[{
        SubscriptBox["\[Zeta]", "a"], 
        RowBox[{"Gamma", "[", 
         RowBox[{"a", "+", "1"}], "]"}]}]}]}], ")"}], "&"}]}], "]"}]], "Input",\

 CellChangeTimes->{{3.5637303552263117`*^9, 3.563730517781824*^9}, {
  3.563730650837453*^9, 3.563730669766507*^9}, {3.563730704760165*^9, 
  3.56373076977747*^9}, {3.5637308137272167`*^9, 3.563730821704618*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"1440", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"], " ", 
     SuperscriptBox["g", "8"]}]}], "+", 
   RowBox[{"30240", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "7"], " ", 
    SuperscriptBox["g", "10"]}], "-", 
   RowBox[{"489888", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "9"], " ", 
    SuperscriptBox["g", "12"]}], "+", 
   RowBox[{"7318080", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "11"], " ", 
    SuperscriptBox["g", "14"]}], "-", 
   RowBox[{"106007616", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "13"], " ", 
    SuperscriptBox["g", "16"]}], "+", 
   RowBox[{"1517836320", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "15"], " ", 
    SuperscriptBox["g", "18"]}], "-", 
   RowBox[{"21661960320", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "17"], " ", 
    SuperscriptBox["g", "20"]}], "+", 
   RowBox[{"309361358592", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "19"], " ", 
    SuperscriptBox["g", "22"]}], "-", 
   RowBox[{"4429492179840", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "21"], " ", 
    SuperscriptBox["g", "24"]}], "+", 
   RowBox[{"63642506159232", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "23"], " ", 
    SuperscriptBox["g", "26"]}], "-", 
   RowBox[{"917920761912000", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "25"], " ", 
    SuperscriptBox["g", "28"]}], "+", 
   RowBox[{"13291268236272000", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "27"], " ", 
    SuperscriptBox["g", "30"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "31"],
    SeriesData[$CellContext`g, 0, {}, 8, 31, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 
   0, {(-1440) $CellContext`\[Zeta]$[5][0], 0, 
    30240 $CellContext`\[Zeta]$[7][0], 
    0, (-489888) $CellContext`\[Zeta]$[9][0], 0, 
    7318080 $CellContext`\[Zeta]$[11][0], 
    0, (-106007616) $CellContext`\[Zeta]$[13][0], 0, 
    1517836320 $CellContext`\[Zeta]$[15][0], 
    0, (-21661960320) $CellContext`\[Zeta]$[17][0], 0, 
    309361358592 $CellContext`\[Zeta]$[19][0], 
    0, (-4429492179840) $CellContext`\[Zeta]$[21][0], 0, 
    63642506159232 $CellContext`\[Zeta]$[23][0], 
    0, (-917920761912000) $CellContext`\[Zeta]$[25][0], 0, 
    13291268236272000 $CellContext`\[Zeta]$[27][0]}, 8, 31, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.563730440774036*^9, 3.563730518201446*^9}, 
   3.563730558814371*^9, {3.5637306516611853`*^9, 3.563730770332302*^9}, {
   3.5637308186200237`*^9, 3.563730825511694*^9}, 3.563734011688632*^9, 
   3.567148199917238*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"verymanyterms", "=", 
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "432"}], 
           SuperscriptBox["g", "2"]}], ")"}], " ", 
         FractionBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"BesselJ", "[", 
             RowBox[{"3", ",", 
              RowBox[{"2", " ", "g", " ", "z"}]}], "]"}], "2"], " "}], 
          RowBox[{"z", " "}]]}], ",", 
        RowBox[{"{", 
         RowBox[{"g", ",", "0", ",", "1000"}], "}"}]}], "]"}], ",", "g", ",", 
      
      RowBox[{
       RowBox[{"(", 
        RowBox[{"#", "/.", 
         RowBox[{
          SuperscriptBox["z", "a_"], "\[RuleDelayed]", 
          RowBox[{
           SubscriptBox["\[Zeta]", "a"], 
           RowBox[{"Gamma", "[", 
            RowBox[{"a", "+", "1"}], "]"}]}]}]}], ")"}], "&"}]}], "]"}]}], 
   ";"}], "//", "Timing"}]], "Input",
 CellChangeTimes->{{3.5637340209250193`*^9, 3.563734023104556*^9}, {
  3.5637342241661797`*^9, 3.563734231861517*^9}, {3.563734515460684*^9, 
  3.563734575192902*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4.657455999999996`", ",", "Null"}], "}"}]], "Output",
 CellChangeTimes->{
  3.563734023711464*^9, 3.5637342333294687`*^9, {3.5637345214657087`*^9, 
   3.5637345273984528`*^9}, {3.56373455904851*^9, 3.5637345755464993`*^9}, 
   3.567148209479629*^9}]
}, Open  ]],

Cell["\<\
Let us see the radious of convergency of these sums. It should be, as usual \
g==1/4. Let us see\
\>", "Text",
 CellChangeTimes->{{3.563734202424491*^9, 3.563734220957706*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"SortBy", "[", 
         RowBox[{
          RowBox[{"List", "@@", 
           RowBox[{"Collect", "[", 
            RowBox[{"verymanyterms", ",", "g"}], "]"}]}], ",", 
          RowBox[{
           RowBox[{"Exponent", "[", 
            RowBox[{"#", ",", "g"}], "]"}], "&"}]}], "]"}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", "4"}], "+", "#"}]}]}], "/.", "subZetas"}], "//", 
      RowBox[{
       RowBox[{"N", "[", 
        RowBox[{"#", ",", "50"}], "]"}], "&"}]}], "//", "ListPlot"}], ")"}], 
   "&"}], "/@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "/", "300"}], ",", "0", ",", 
    RowBox[{"1", "/", "300"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.563734235001149*^9, 3.5637344802872953`*^9}, {
  3.563734537508484*^9, 3.5637345386051397`*^9}, {3.5637345808583603`*^9, 
  3.5637346256973248`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Hue[0.67, 0.6, 0.6], PointBox[CompressedData["
1:eJw113c41Y3/x3EZIS0jqYiU6tagVMh6kZWRTRllb47jOJxzzDRRGZVkREui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      "]]},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->True,
    AxesOrigin->{0, Automatic},
    PlotRange->Automatic,
    PlotRangeClipping->True], ",", 
   GraphicsBox[
    {Hue[0.67, 0.6, 0.6], PointBox[CompressedData["
1:eJwt1nlYjPsDBfAhstyQLVtliJRSSZZETqWSdVqVUiPt6zTT3syICJEbLkqW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      "]]},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->True,
    AxesOrigin->{0, Automatic},
    PlotRange->Automatic,
    PlotRangeClipping->True], ",", 
   GraphicsBox[
    {Hue[0.67, 0.6, 0.6], PointBox[CompressedData["
1:eJw113c4lf8fx3EkIy2llJbqG03JSDS8REsqkaxwyMje5xyHoyNCykwpUaiU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      "]]},
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->True,
    AxesOrigin->{0, Automatic},
    PlotRange->Automatic,
    PlotRangeClipping->True]}], "}"}]], "Output",
 CellChangeTimes->{
  3.563734256121554*^9, {3.563734286609185*^9, 3.563734481905944*^9}, {
   3.5637345307033052`*^9, 3.563734539339636*^9}, {3.5637345774947643`*^9, 
   3.5637346261867847`*^9}, 3.567148217427497*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Fourier transforms and Bessel functions", "Subsection",
 CellChangeTimes->{{3.5641189814196672`*^9, 3.564118986731862*^9}}],

Cell["We derived two relations:", "Text",
 CellChangeTimes->{{3.564118993029881*^9, 3.564119001479443*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"BesselJ", "[", 
    RowBox[{"n", ",", "z"}], "]"}], ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.56412123529428*^9, 3.564121239793373*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"BesselJ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "1"}], "+", "n"}], ",", "z"}], "]"}], "-", 
    RowBox[{"BesselJ", "[", 
     RowBox[{
      RowBox[{"1", "+", "n"}], ",", "z"}], "]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.5671594726361628`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    RowBox[{"(", 
     FractionBox[
      RowBox[{"x", "+", 
       FractionBox["1", "x"]}], 
      RowBox[{"x", "-", 
       FractionBox["1", "x"]}]], ")"}], 
    FractionBox["1", 
     SuperscriptBox["x", "a"]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    FractionBox["du", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
     RowBox[{"a", "+", "1"}]]}], 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "a"], "[", 
      RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.564119004994749*^9, 3.56411904222265*^9}, {
   3.564121247821135*^9, 3.564121270162437*^9}, 3.5641213952033587`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox["x", "a"]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    FractionBox["du", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", "a"}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"a", "+", "1"}]], 
   FractionBox[
    RowBox[{
     SubscriptBox["J", "a"], "[", 
     RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}]}]], "Input",
 CellChangeTimes->{{3.563712539895719*^9, 3.5637125971508493`*^9}, {
  3.563712629434134*^9, 3.563712718731204*^9}, {3.563712837533313*^9, 
  3.563713114247881*^9}, {3.563713292111334*^9, 3.563713306928577*^9}, {
  3.5637140309665194`*^9, 3.563714034363871*^9}, {3.563714685313017*^9, 
  3.563714759071412*^9}, {3.564119021730042*^9, 3.56411905621176*^9}, {
  3.5641197372966146`*^9, 3.564119763932993*^9}}],

Cell["Numerical check", "Text",
 CellChangeTimes->{{3.564119063623002*^9, 3.564119065421653*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"NIntegrate", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            FractionBox["1", 
             SuperscriptBox[
              RowBox[{"x$h", "[", "0", "]"}], "a"]], 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
            FractionBox["1", 
             RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], "/.", 
           "subJuk"}], "/.", 
          RowBox[{"g", "\[Rule]", 
           RowBox[{"1", "/", "3"}]}]}], "/.", 
         RowBox[{"z", "\[Rule]", 
          RowBox[{"\[ImaginaryI]", "/", "5"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"u", ",", "1", ",", "\[ImaginaryI]", ",", 
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "\[ImaginaryI]"}], ",", "1"}], "}"}]}], "]"}], "-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"-", "a"}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
           RowBox[{"a", "+", "1"}]], 
          FractionBox[
           RowBox[{
            SubscriptBox["J", "a"], "[", 
            RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], "/.", 
         RowBox[{"g", "\[Rule]", 
          RowBox[{"1", "/", "3"}]}]}], "/.", 
        RowBox[{"z", "\[Rule]", 
         RowBox[{"\[ImaginaryI]", "/", "5"}]}]}], ")"}]}], "/.", 
     RowBox[{
      RowBox[{
       SubscriptBox["J", "a_"], "[", "r_", "]"}], "\[RuleDelayed]", 
      RowBox[{"BesselJ", "[", 
       RowBox[{"a", ",", "r"}], "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "5"}], "}"}]}], "]"}], "//", "Chop"}]], "Input",
 CellChangeTimes->{{3.564119067121175*^9, 3.564119174263913*^9}, {
   3.564119215933838*^9, 3.564119264939885*^9}, 3.564119770395506*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.564119213561064*^9, 3.564119266609706*^9}, 
   3.564119772318926*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["1", "4"], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
       RowBox[{"a", "+", "1"}]], 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{
         SubscriptBox["J", "a"], "[", 
         RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}], "/.", 
     RowBox[{"g", "\[Rule]", 
      RowBox[{"1", "/", "3"}]}]}], "/.", 
    RowBox[{"z", "\[Rule]", 
     RowBox[{"\[ImaginaryI]", "/", "5"}]}]}], ")"}], "/.", 
  RowBox[{
   SubscriptBox["J", "a_"], "\[RuleDelayed]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"BesselJ", "[", 
      RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.564121291811599*^9, 3.564121300396579*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "12"], " ", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
   RowBox[{"1", "+", "a"}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"BesselJ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "1"}], "+", "a"}], ",", 
      FractionBox[
       RowBox[{"2", " ", "\[ImaginaryI]"}], "15"]}], "]"}], "-", 
    RowBox[{"BesselJ", "[", 
     RowBox[{
      RowBox[{"1", "+", "a"}], ",", 
      FractionBox[
       RowBox[{"2", " ", "\[ImaginaryI]"}], "15"]}], "]"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{{3.564121286221816*^9, 3.564121300716856*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"NIntegrate", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             FractionBox["1", 
              SuperscriptBox[
               RowBox[{"x$h", "[", "0", "]"}], "a"]], 
             FractionBox[
              RowBox[{"(", 
               RowBox[{"x", "+", 
                FractionBox["1", "x"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{"x", "-", 
                FractionBox["1", "x"]}], ")"}]], 
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
             FractionBox["1", 
              RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], "/.", 
            RowBox[{"x$", "\[Rule]", "x$h"}]}], "/.", "subJuk"}], "/.", 
          RowBox[{"g", "\[Rule]", 
           RowBox[{"1", "/", "3"}]}]}], "/.", 
         RowBox[{"z", "\[Rule]", 
          RowBox[{"\[ImaginaryI]", "/", "5"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"u", ",", "1", ",", "\[ImaginaryI]", ",", 
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "\[ImaginaryI]"}], ",", "1"}], "}"}]}], "]"}], "-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
            RowBox[{"a", "+", "1"}]]}], 
          RowBox[{"D", "[", 
           RowBox[{
            RowBox[{
             SubscriptBox["J", "a"], "[", 
             RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}], "/.", 
         RowBox[{"g", "\[Rule]", 
          RowBox[{"1", "/", "3"}]}]}], "/.", 
        RowBox[{"z", "\[Rule]", 
         RowBox[{"\[ImaginaryI]", "/", "5"}]}]}], ")"}]}], "/.", 
     RowBox[{
      SubscriptBox["J", "a_"], "\[RuleDelayed]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"BesselJ", "[", 
         RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "5"}], "}"}]}], "]"}], "//", "Chop"}]], "Input",
 CellChangeTimes->{{3.5641192273921757`*^9, 3.564119280299808*^9}, {
  3.564119643338069*^9, 3.56411965334569*^9}, {3.56412128100851*^9, 
  3.5641213883278112`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.564119268181491*^9, 3.564119281380596*^9}, {
  3.564119649752408*^9, 3.5641196541706657`*^9}, {3.5641213161079483`*^9, 
  3.564121405135497*^9}}]
}, Open  ]],

Cell["An Inverse of this reperesentation is equally useful to us", "Text",
 CellChangeTimes->{{3.564119673735099*^9, 3.5641196871660347`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["subtoBes1"], "Input",
 CellChangeTimes->{{3.5673170493508673`*^9, 3.567317051133113*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", "2", "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     SubscriptBox["z", "1"]]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox["\[ImaginaryI]", "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}]}], 
     SubscriptBox["z", "3"]]}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{"a_", "/;", 
      RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["z", "2"]], 
      RowBox[{
       SubscriptBox["J", 
        RowBox[{"-", "a"}]], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}]}], 
     SuperscriptBox["\[ImaginaryI]", "a"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5673170515650177`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox["x", "a"]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    FractionBox["du", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", "a"}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
    RowBox[{"a", "+", "1"}]], 
   FractionBox[
    RowBox[{
     SubscriptBox["J", "a"], "[", 
     RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   SuperscriptBox["x", "a"]], "\[Equal]", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564121433632308*^9, 3.56412148410196*^9}, {
  3.564121698101907*^9, 3.564121698789381*^9}, {3.5641616561372137`*^9, 
  3.564161662000765*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"NIntegrate", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
           RowBox[{"(", 
            RowBox[{"a", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "a"], "[", 
               RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "a_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}], "/.", 
         RowBox[{"g", "\[Rule]", 
          RowBox[{"1", "/", "3"}]}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", "5"}], "+", 
          RowBox[{"\[ImaginaryI]", "/", "10"}]}]}]}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], "-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"x$h", "[", "0", "]"}], "a"]], "/.", "subJuk"}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{"1", "/", "3"}]}]}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{
         RowBox[{"1", "/", "5"}], "+", 
         RowBox[{"\[ImaginaryI]", "/", "10"}]}]}]}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "5"}], "}"}]}], "]"}], "//", "Chop"}]], "Input",
 CellChangeTimes->{{3.5641214867415*^9, 3.564121589839327*^9}, {
  3.5641216322288523`*^9, 3.5641216892317677`*^9}, {3.5641616639437923`*^9, 
  3.564161666020763*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.564121523324546*^9, 3.5641215902654657`*^9}, 
   3.5641216326582413`*^9, {3.564121679105885*^9, 3.5641216901759644`*^9}, 
   3.564161554381543*^9, 3.564161667663516*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ContourIntegral]", 
   RowBox[{
    RowBox[{"(", 
     FractionBox[
      RowBox[{"x", "+", 
       FractionBox["1", "x"]}], 
      RowBox[{"x", "-", 
       FractionBox["1", "x"]}]], ")"}], 
    FractionBox["1", 
     SuperscriptBox["x", "a"]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    FractionBox["du", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}]}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "\[ImaginaryI]"}], ")"}], 
     RowBox[{"a", "+", "1"}]]}], 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "a"], "[", 
      RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.564119004994749*^9, 3.56411904222265*^9}, {
   3.564121247821135*^9, 3.564121270162437*^9}, 3.5641213952033587`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{"x", "+", 
      FractionBox["1", "x"]}], 
     RowBox[{"x", "-", 
      FractionBox["1", "x"]}]], ")"}], 
   FractionBox["1", 
    SuperscriptBox["x", "a"]]}], "\[Equal]", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{
         SubscriptBox["J", "a"], "[", 
         RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}], 
     ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564161701406085*^9, 3.5641617040789757`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"NIntegrate", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
             RowBox[{"D", "[", 
              RowBox[{
               RowBox[{
                SubscriptBox["J", "a"], "[", 
                RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}], 
            ")"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "a_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}], "/.", 
         RowBox[{"g", "\[Rule]", 
          RowBox[{"1", "/", "3"}]}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", "5"}], "+", 
          RowBox[{"\[ImaginaryI]", "/", "10"}]}]}]}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], "-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            FractionBox[
             RowBox[{"x", "+", 
              FractionBox["1", "x"]}], 
             RowBox[{"x", "-", 
              FractionBox["1", "x"]}]], ")"}], 
           FractionBox["1", 
            SuperscriptBox[
             RowBox[{"x$h", "[", "0", "]"}], "a"]]}], "/.", 
          RowBox[{"x$", "\[Rule]", "x$h"}]}], "/.", "subJuk"}], "/.", 
        RowBox[{"g", "\[Rule]", 
         RowBox[{"1", "/", "3"}]}]}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{
         RowBox[{"1", "/", "5"}], "+", 
         RowBox[{"\[ImaginaryI]", "/", "10"}]}]}]}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "5"}], "}"}]}], "]"}], "//", "Chop"}]], "Input",
 CellChangeTimes->{{3.564161582778141*^9, 3.5641616371752768`*^9}, {
  3.5641617051991587`*^9, 3.5641617077729483`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.564161608740406*^9, 3.564161646448283*^9}, 
   3.564161711590045*^9}]
}, Open  ]],

Cell["Hence we can compute the generic UpIntegrate integral", "Text",
 CellChangeTimes->{{3.564161714105199*^9, 3.5641617249362392`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"UpIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", 
           RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], 
         RowBox[{
          RowBox[{"\[Eta]$b", "[", "3", "]"}], "[", 
          RowBox[{"-", "2"}], "]"}], 
         FractionBox["1", 
          SuperscriptBox["u", "3"]], 
         RowBox[{
          RowBox[{"\[Eta]$", "[", "4", "]"}], "[", "2", "]"}]}], "/.", 
        RowBox[{"Shift", "[", 
         RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"u", "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          FractionBox["\[Epsilon]", "2"]}]}]}]}], ",", "u"}], "]"}], "//.", 
    "subZetaReduce"}], ",", "\[Zeta]list"}], "]"}]], "Input",
 CellChangeTimes->{{3.564161726441988*^9, 3.564161785517086*^9}, {
  3.564161852423786*^9, 3.564161885553*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    FractionBox["16", "7"]}], " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{
   FractionBox["36", "5"], " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"28", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}]}]], "Output",
 CellChangeTimes->{{3.564161767227129*^9, 3.56416178274549*^9}, {
  3.564161849275948*^9, 3.564161885965239*^9}}]
}, Open  ]],

Cell["I will now compute this integral using for x-generalizations", "Text",
 CellChangeTimes->{{3.564161880858594*^9, 3.564161903577281*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   OverscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    StyleBox["^",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]]], "a", 
   RowBox[{"[", "2", "]"}]], "\[Equal]", 
  RowBox[{
   FractionBox[
    SuperscriptBox["D", "2"], 
    RowBox[{"1", "-", 
     SuperscriptBox["D", "2"]}]], 
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "a"]]}], "==", 
  RowBox[{
   FractionBox[
    SuperscriptBox["D", "2"], 
    RowBox[{"1", "-", 
     SuperscriptBox["D", "2"]}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{"dz", " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
     RowBox[{"(", 
      RowBox[{"a", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", "a"], "[", 
         RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{
    FractionBox["dz", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}]], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564161951263296*^9, 3.564162043566044*^9}, {
  3.564162090625043*^9, 3.564162096924502*^9}}],

Cell["Behaviour of Bessel function at infinity is", "Text",
 CellChangeTimes->{{3.5641625866864977`*^9, 3.56416260700733*^9}, 
   3.5641627968736753`*^9}],

Cell[BoxData[
 RowBox[{"(", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"1", " "}], 
     SqrtBox[
      RowBox[{"\[Pi]", " ", "g", " ", "z"}]]], 
    RowBox[{"Cos", "[", 
     RowBox[{
      RowBox[{"2", "g", " ", "z"}], "-", 
      RowBox[{
       FractionBox["\[Pi]", "2"], 
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", "2"], "+", "a"}], ")"}]}]}], "]"}]}], "+", 
   RowBox[{
    FractionBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"4", " ", 
        SuperscriptBox["a", "2"]}]}], ")"}], 
     RowBox[{"16", 
      SqrtBox["\[Pi]"], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"g", " ", "z"}], ")"}], 
       RowBox[{"3", "/", "2"}]]}]], 
    RowBox[{"Sin", "[", 
     RowBox[{
      RowBox[{"2", "g", " ", "z"}], "-", 
      RowBox[{
       FractionBox["\[Pi]", "2"], 
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", "2"], "+", "a"}], ")"}]}]}], "]"}]}]}], 
  ")"}]], "Input",
 CellChangeTimes->{{3.564162596686325*^9, 3.564162616774575*^9}, {
   3.5641627914945507`*^9, 3.5641628082519217`*^9}, {3.564162851545311*^9, 
   3.564162890904978*^9}, 3.564163132507824*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Series", "[", 
           RowBox[{
            RowBox[{"BesselJ", "[", 
             RowBox[{"#", ",", 
              RowBox[{"2", "g", " ", "z"}]}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"z", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], "//",
           "Normal"}], ")"}], "-", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FractionBox[
            RowBox[{"1", " "}], 
            SqrtBox[
             RowBox[{"\[Pi]", " ", "g", " ", "z"}]]], 
           RowBox[{"Cos", "[", 
            RowBox[{
             RowBox[{"2", "g", " ", "z"}], "-", 
             RowBox[{
              FractionBox["\[Pi]", "2"], 
              RowBox[{"(", 
               RowBox[{
                FractionBox["1", "2"], "+", "a"}], ")"}]}]}], "]"}]}], "+", 
          RowBox[{
           FractionBox[
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"4", " ", 
               SuperscriptBox["a", "2"]}]}], ")"}], 
            RowBox[{"16", 
             SqrtBox["\[Pi]"], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"g", " ", "z"}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]], 
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"2", "g", " ", "z"}], "-", 
             RowBox[{
              FractionBox["\[Pi]", "2"], 
              RowBox[{"(", 
               RowBox[{
                FractionBox["1", "2"], "+", "a"}], ")"}]}]}], "]"}]}]}], 
         ")"}]}], "/.", 
       RowBox[{"a", "\[Rule]", "#"}]}], ")"}], "&"}], "/@", 
    RowBox[{"Range", "[", "5", "]"}]}], "/.", 
   RowBox[{"z", "\[Rule]", 
    RowBox[{"Zeta", "[", "3", "]"}]}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.564162704418086*^9, 3.564162749908782*^9}, {
  3.564162810739469*^9, 3.564162846859131*^9}, {3.564162893944253*^9, 
  3.564163019255534*^9}, {3.564163078801373*^9, 3.564163123770876*^9}}],

Cell["\<\
As long as we are integrating over ral z, it is fine because it is \
osciallating. If decline from the real axis, it is not fine. The \
representation works for Im[u]>-1.\
\>", "Text",
 CellChangeTimes->{{3.5641631432458878`*^9, 3.564163189057555*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   OverscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]]], 
    StyleBox["^",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]]], "a", 
   RowBox[{"[", 
    RowBox[{"-", "2"}], "]"}]], "\[Equal]", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{
    FractionBox["dz", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}]], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564163198141157*^9, 3.5641632255855303`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "a"]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}], " ", 
    RowBox[{"(*", " ", 
     RowBox[{"upper", " ", "half", " ", "plane"}], "*)"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564163243809647*^9, 3.5641633070701237`*^9}}],

Cell["\<\
This reperesentation seems to work up to real axis only. Hence I would \
preferably use complex conjugate, to integrate slightly below real axis\
\>", "Text",
 CellChangeTimes->{{3.5641632635680304`*^9, 3.564163288901905*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "a"]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"+", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}], " ", 
    RowBox[{"(*", " ", 
     RowBox[{"lower", " ", "half", " ", "plane"}], "*)"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564163312113018*^9, 3.564163316644053*^9}}],

Cell["\<\
So we have an integration over u, slightly below real axis of\
\>", "Text",
 CellChangeTimes->{{3.564163326717595*^9, 3.564163338355842*^9}}],

Cell[BoxData[
 SuperscriptBox["\[ExponentialE]", 
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["z", "1"], "+", 
     SubscriptBox["z", "2"], "-", 
     SubscriptBox["z", "3"]}], ")"}], " ", "u"}]]], "Input",
 CellChangeTimes->{{3.564163343778982*^9, 3.564163354013242*^9}}],

Cell["\<\
Integration on the real axis means \[Delta]-function. \
\>", "Text",
 CellChangeTimes->{{3.564163357451573*^9, 3.564163417081308*^9}, {
   3.5641636607278757`*^9, 3.564163665961796*^9}, 3.567148341857155*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox[
     RowBox[{"du", " "}], 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["z", "1"], "+", 
        SubscriptBox["z", "2"], "-", 
        SubscriptBox["z", "3"]}], ")"}], " ", "u"}]]}]}], "=", 
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"\[Delta]", "[", 
    RowBox[{
     SubscriptBox["z", "1"], "+", 
     SubscriptBox["z", "2"], "-", 
     SubscriptBox["z", "3"]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5641637201682*^9, 3.564163753925136*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "a", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{"-", "b"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "c", 
      RowBox[{"[", "2", "]"}]]}], " ", "]"}], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dz", "1"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       SubscriptBox["dz", "2"], " ", 
       RowBox[{
        SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
        RowBox[{
         FractionBox[
          SubscriptBox["dz", "3"], 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "3"]], "-", "1"}]], 
         RowBox[{"(", 
          RowBox[{"\[ImaginaryI]", " ", 
           RowBox[{"\[Delta]", "[", 
            RowBox[{
             SubscriptBox["z", "1"], "+", 
             SubscriptBox["z", "2"], "-", 
             SubscriptBox["z", "3"]}], "]"}]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"a", 
           SuperscriptBox[
            RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
           FractionBox[
            RowBox[{
             SubscriptBox["J", "b"], "[", 
             RowBox[{"2", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], 
            SubscriptBox["z", "1"]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"b", 
           SuperscriptBox[
            RowBox[{"(", "\[ImaginaryI]", ")"}], "b"], 
           FractionBox[
            RowBox[{
             SubscriptBox["J", "b"], "[", 
             RowBox[{"2", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            SubscriptBox["z", "2"]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"c", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "\[ImaginaryI]"}], ")"}], "c"], 
           FractionBox[
            RowBox[{
             SubscriptBox["J", "c"], "[", 
             RowBox[{"2", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}], 
            SubscriptBox["z", "3"]]}], ")"}]}]}]}]}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{"\[ImaginaryI]", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dz", "1"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       FractionBox[
        SubscriptBox["dz", "2"], 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           SubscriptBox["z", "2"], "+", 
           SubscriptBox["z", "1"]}]], "-", "1"}]], 
       RowBox[{"(", 
        RowBox[{"a", 
         SuperscriptBox[
          RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
         FractionBox[
          RowBox[{
           SubscriptBox["J", "b"], "[", 
           RowBox[{"2", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], 
          SubscriptBox["z", "1"]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{"b", 
         SuperscriptBox[
          RowBox[{"(", "\[ImaginaryI]", ")"}], "b"], 
         FractionBox[
          RowBox[{
           SubscriptBox["J", "b"], "[", 
           RowBox[{"2", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}], 
          SubscriptBox["z", "2"]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{"c", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "\[ImaginaryI]"}], ")"}], "c"], 
         FractionBox[
          RowBox[{
           SubscriptBox["J", "c"], "[", 
           RowBox[{"2", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "2"], "+", 
              SubscriptBox["z", "1"]}], ")"}]}], "]"}], 
          RowBox[{
           SubscriptBox["z", "2"], "+", 
           SubscriptBox["z", "1"]}]]}], ")"}]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.564163763148437*^9, 3.564163959510992*^9}, {
  3.564382663532579*^9, 3.564382664014756*^9}}],

Cell["Let us compare now with representation of zeta functions", "Text",
 CellChangeTimes->{{3.564381432459674*^9, 3.564381438672083*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Zeta]", 
   RowBox[{"a", ",", "b"}]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", "1"}], "\[Infinity]"], 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox["n", "a"]], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"m", "=", "1"}], "\[Infinity]"], 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "n"}], ")"}], "b"]]}]}]}], "==", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"Gamma", "[", "a", "]"}]], 
   FractionBox["1", 
    RowBox[{"Gamma", "[", "b", "]"}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dt", "1"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         SubscriptBox["t", "1"], "+", 
         SubscriptBox["t", "2"]}]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       FractionBox[
        SubscriptBox["dt", "2"], 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["t", "2"]], "-", "1"}]], "  ", 
       SubsuperscriptBox["t", "1", 
        RowBox[{"a", "-", "1"}]], 
       SubsuperscriptBox["t", "2", 
        RowBox[{"b", "-", "1"}]]}]}]}]}]}], "==", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"Gamma", "[", "a", "]"}]], 
   FractionBox["1", 
    RowBox[{"Gamma", "[", "b", "]"}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dt", "2"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["t", "2"]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", 
       SubscriptBox["t", "2"], "\[Infinity]"], 
      RowBox[{
       FractionBox[
        SubscriptBox["dt", "1"], 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["t", "1"]], "-", "1"}]], "  ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["t", "1"], "-", 
          SubscriptBox["t", "2"]}], ")"}], 
        RowBox[{"a", "-", "1"}]], 
       SubsuperscriptBox["t", "2", 
        RowBox[{"b", "-", "1"}]]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.564381439180575*^9, 3.5643815754918327`*^9}, 
   3.5643819784713173`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"expectedexpansion", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_"}], "}"}], ",", "order_"}], "]"}], ":=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"\[ImaginaryI]", 
           RowBox[{"(", 
            RowBox[{"a", 
             SuperscriptBox[
              RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "a"], "[", 
               RowBox[{"2", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}], 
              SubscriptBox["z", "1"]]}], ")"}], 
           RowBox[{"(", 
            RowBox[{"b", 
             SuperscriptBox[
              RowBox[{"(", "\[ImaginaryI]", ")"}], "b"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "b"], "[", 
               RowBox[{"2", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              SubscriptBox["z", "2"]]}], ")"}], 
           RowBox[{"(", 
            RowBox[{"c", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "\[ImaginaryI]"}], ")"}], "c"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "c"], "[", 
               RowBox[{"2", "g", " ", 
                RowBox[{"(", 
                 RowBox[{
                  SubscriptBox["z", "2"], "+", 
                  SubscriptBox["z", "1"]}], ")"}]}], "]"}], 
              RowBox[{
               SubscriptBox["z", "2"], "+", 
               SubscriptBox["z", "1"]}]]}], ")"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "aa_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"aa", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "order"}], "}"}]}], "]"}], ",", "g", 
       ",", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Expand", "[", 
           RowBox[{
            RowBox[{
             SubsuperscriptBox["z", "1", "50"], 
             SubsuperscriptBox["z", "2", "50"], "#"}], ",", 
            RowBox[{
             SubscriptBox["z", "1"], "|", 
             SubscriptBox["z", "2"]}]}], "]"}], "/.", 
          RowBox[{
           RowBox[{
            SubsuperscriptBox["z", "1", "pow1_"], 
            SubsuperscriptBox["z", "2", "pow2_"]}], "\[RuleDelayed]", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Zeta]$", "[", 
              RowBox[{
               RowBox[{"pow2", "+", "1", "-", "50"}], ",", 
               RowBox[{"pow1", "+", "1", "-", "50"}]}], "]"}], "[", "0", 
             "]"}], 
            RowBox[{"Gamma", "[", 
             RowBox[{"pow1", "-", "50", "+", "1"}], "]"}], 
            RowBox[{"Gamma", "[", 
             RowBox[{"pow2", "+", "1", "-", "50"}], "]"}]}]}]}], ")"}], 
        "&"}]}], "]"}], "//.", "subZetaReduce"}], ",", "canlist"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.564382015848928*^9, 3.564382242257066*^9}, {
  3.564382668131509*^9, 3.564382668569624*^9}, {3.564382781697082*^9, 
  3.564382781823127*^9}, {3.564382887253467*^9, 3.564382908812154*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"expectedexpansionup", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_"}], "}"}], ",", "order_"}], "]"}], ":=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"\[ImaginaryI]", 
           RowBox[{"(", 
            RowBox[{"a", 
             SuperscriptBox[
              RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "a"], "[", 
               RowBox[{"2", "g", " ", 
                RowBox[{"(", 
                 RowBox[{
                  SubscriptBox["z", "1"], "+", 
                  SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "+", 
                SubscriptBox["z", "2"]}], ")"}]]}], ")"}], 
           RowBox[{"(", 
            RowBox[{"b", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "\[ImaginaryI]"}], ")"}], "b"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "b"], "[", 
               RowBox[{"2", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              SubscriptBox["z", "2"]]}], ")"}], 
           RowBox[{"(", 
            RowBox[{"c", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "\[ImaginaryI]"}], ")"}], "c"], 
             FractionBox[
              RowBox[{
               SubscriptBox["J", "c"], "[", 
               RowBox[{"2", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}], 
              SubscriptBox["z", "1"]]}], ")"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "aa_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"aa", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "order"}], "}"}]}], "]"}], ",", "g", 
       ",", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Expand", "[", 
           RowBox[{
            RowBox[{
             SubsuperscriptBox["z", "1", "50"], 
             SubsuperscriptBox["z", "2", "50"], "#"}], ",", 
            RowBox[{
             SubscriptBox["z", "1"], "|", 
             SubscriptBox["z", "2"]}]}], "]"}], "/.", 
          RowBox[{
           RowBox[{
            SubsuperscriptBox["z", "1", "pow1_"], 
            SubsuperscriptBox["z", "2", "pow2_"]}], "\[RuleDelayed]", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Zeta]$", "[", 
              RowBox[{
               RowBox[{"pow2", "+", "1", "-", "50"}], ",", 
               RowBox[{"pow1", "+", "1", "-", "50"}]}], "]"}], "[", "0", 
             "]"}], 
            RowBox[{"Gamma", "[", 
             RowBox[{"pow1", "-", "50", "+", "1"}], "]"}], 
            RowBox[{"Gamma", "[", 
             RowBox[{"pow2", "+", "1", "-", "50"}], "]"}]}]}]}], ")"}], 
        "&"}]}], "]"}], "//.", "subZetaReduce"}], ",", "canlist"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.56440453573903*^9, 3.564404549024707*^9}, {
   3.5644045946382627`*^9, 3.564404599352675*^9}, 3.564404718151916*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"whatintegralgives", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_"}], "}"}], ",", "order_"}], "]"}], ":=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Times", "@@", 
          RowBox[{"Normal", "@", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Collect", "[", 
              RowBox[{
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"x$h", "[", 
                    RowBox[{"-", "2"}], "]"}], "a"]], ",", 
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"x$h", "[", "0", "]"}], "b"]], ",", 
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"x$h", "[", 
                    RowBox[{"+", "2"}], "]"}], "c"]]}], "}"}], "/.", 
                  "subJuk"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "order"}], "}"}]}], "]"}], ",", 
               "g", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"#", "/.", "subtousu"}], "//", "funapartusu"}], "//",
                   "funreduceusu"}], ")"}], "&"}]}], "]"}], "/.", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  RowBox[{"\[ScriptU]$", "[", "0", "]"}], "[", "2", "]"}], 
                 "pow_"], "\[RuleDelayed]", 
                RowBox[{
                 RowBox[{"\[Eta]$", "[", 
                  RowBox[{"-", "pow"}], "]"}], "[", "2", "]"}]}], ",", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  RowBox[{"\[ScriptU]$", "[", "0", "]"}], "[", 
                  RowBox[{"-", "2"}], "]"}], "pow_"], "\[RuleDelayed]", 
                RowBox[{
                 RowBox[{"\[Eta]$b", "[", 
                  RowBox[{"-", "pow"}], "]"}], "[", 
                 RowBox[{"-", "2"}], "]"}]}]}], "}"}]}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "order"}], "}"}]}], "]"}], ",", "g", 
       ",", 
       RowBox[{
        RowBox[{"UpIntegrate", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             FractionBox["#", 
              RowBox[{
               RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
             RowBox[{"Shift", "[", 
              RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
            RowBox[{"u", "\[Rule]", 
             RowBox[{"u", "-", 
              FractionBox[
               RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}], "2"]}]}]}], "/.",
            "subusu"}], ",", "u"}], "]"}], "&"}]}], "]"}], "//.", 
     "subZetaReduce"}], ",", "canlist"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.564382264474785*^9, 3.5643823426875677`*^9}, {
  3.564382384339113*^9, 3.564382607977057*^9}, {3.564382786837241*^9, 
  3.564382787112805*^9}, {3.564382912325555*^9, 3.5643829153448153`*^9}, {
  3.564387108122184*^9, 3.564387112159421*^9}, {3.56438720581516*^9, 
  3.564387336012033*^9}, {3.5644045381928377`*^9, 3.564404539778638*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"whatintegralgivesup", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_"}], "}"}], ",", "order_"}], "]"}], ":=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Times", "@@", 
          RowBox[{"Normal", "@", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Collect", "[", 
              RowBox[{
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"x$h", "[", 
                    RowBox[{"-", "2"}], "]"}], "a"]], ",", 
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"x$h", "[", "0", "]"}], "b"]], ",", 
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"x$h", "[", 
                    RowBox[{"+", "2"}], "]"}], "c"]]}], "}"}], "/.", 
                  "subJuk"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "order"}], "}"}]}], "]"}], ",", 
               "g", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"#", "/.", "subtousu"}], "//", "funapartusu"}], "//",
                   "funreduceusu"}], ")"}], "&"}]}], "]"}], "/.", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  RowBox[{"\[ScriptU]$", "[", "0", "]"}], "[", "2", "]"}], 
                 "pow_"], "\[RuleDelayed]", 
                RowBox[{
                 RowBox[{"\[Eta]$", "[", 
                  RowBox[{"-", "pow"}], "]"}], "[", "2", "]"}]}], ",", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{
                  RowBox[{"\[ScriptU]$", "[", "0", "]"}], "[", 
                  RowBox[{"-", "2"}], "]"}], "pow_"], "\[RuleDelayed]", 
                RowBox[{
                 RowBox[{"\[Eta]$b", "[", 
                  RowBox[{"-", "pow"}], "]"}], "[", 
                 RowBox[{"-", "2"}], "]"}]}]}], "}"}]}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "order"}], "}"}]}], "]"}], ",", "g", 
       ",", 
       RowBox[{
        RowBox[{"UpIntegrate", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             FractionBox["#", 
              RowBox[{
               RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
             RowBox[{"Shift", "[", 
              RowBox[{"+", "\[Epsilon]"}], "]"}]}], "/.", 
            RowBox[{"u", "\[Rule]", 
             RowBox[{"u", "+", 
              FractionBox[
               RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}], "2"]}]}]}], "/.",
            "subusu"}], ",", "u"}], "]"}], "&"}]}], "]"}], "//.", 
     "subZetaReduce"}], ",", "canlist"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.564382264474785*^9, 3.5643823426875677`*^9}, {
  3.564382384339113*^9, 3.564382607977057*^9}, {3.564382786837241*^9, 
  3.564382787112805*^9}, {3.564382912325555*^9, 3.5643829153448153`*^9}, {
  3.564387108122184*^9, 3.564387112159421*^9}, {3.56438720581516*^9, 
  3.564387336012033*^9}, {3.5644045381928377`*^9, 3.564404539778638*^9}, {
  3.564404609978306*^9, 3.564404614148962*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"mlist", "=", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1", ",", "1"}], "}"}]}], ",", 
     RowBox[{"order", "=", "12"}]}], "}"}], ",", 
   RowBox[{
    RowBox[{"whatintegralgives", "[", 
     RowBox[{"mlist", ",", "order"}], "]"}], "-", 
    RowBox[{"expectedexpansion", "[", 
     RowBox[{"mlist", ",", "order"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.564382345059544*^9, 3.56438236103118*^9}, {
  3.5643825754875517`*^9, 3.564382634796753*^9}, {3.5643826806614428`*^9, 
  3.5643827218041887`*^9}, {3.564382920188653*^9, 3.564382920692212*^9}, {
  3.5643873516278553`*^9, 3.564387351803522*^9}, {3.5643873942981586`*^9, 
  3.56438741136314*^9}, {3.564387467439404*^9, 3.564387478390698*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.564382715620119*^9, 3.564382722681988*^9}, {
   3.564382897721616*^9, 3.564382933638441*^9}, 3.564387056048449*^9, {
   3.564387349641946*^9, 3.564387354020018*^9}, {3.564387400741641*^9, 
   3.564387414040804*^9}, {3.564387470130424*^9, 3.564387484697177*^9}, 
   3.567148368366885*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"mlist", "=", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "1"}], "}"}]}], ",", 
     RowBox[{"order", "=", "10"}]}], "}"}], ",", 
   RowBox[{
    RowBox[{"whatintegralgivesup", "[", 
     RowBox[{"mlist", ",", "order"}], "]"}], "-", " ", 
    RowBox[{"expectedexpansionup", "[", 
     RowBox[{"mlist", ",", "order"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5644046228025084`*^9, 3.564404684789832*^9}, {
  3.564404728338744*^9, 3.56440474205057*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.564404630794999*^9, 3.5644046851746407`*^9}, {
   3.56440472609965*^9, 3.56440474516325*^9}, 3.567148371229273*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Tran", 
    RowBox[{"-", "4"}]], TraditionalForm]]],
 " terms from pure double wrapping term in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "wrap"], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Subsection",
 CellChangeTimes->{{3.5637312125991297`*^9, 3.563731223083002*^9}, {
  3.563734657660665*^9, 3.56373466065263*^9}, {3.56440653773948*^9, 
  3.564406543603314*^9}, {3.568969613332157*^9, 3.5689696421931143`*^9}}],

Cell[TextData[{
 "The next easy thing is to deal with  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "4"}]], TraditionalForm]]],
 " terms appearing at double wrapping. Because all things can be expanded at \
leading order."
}], "Text",
 CellChangeTimes->{{3.563627273211237*^9, 3.5636273378412933`*^9}, {
  3.563734675802908*^9, 3.563734707362774*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"trannew", "=", 
  RowBox[{"2592", " ", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           FractionBox["1", 
            RowBox[{"4", " ", "648", " ", "2", "  ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox[
                StyleBox["Q",
                 FontColor->RGBColor[0.8, 0, 0]], "-"], ")"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox[
                StyleBox["Q",
                 FontColor->RGBColor[0.8, 0, 0]], "+"], ")"}], "4"]}]], 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["comb2", "2"], "+", 
             RowBox[{"2", " ", "comb1", " ", "comb2", " ", 
              SuperscriptBox[
               StyleBox["Q",
                FontColor->RGBColor[0.8, 0, 0]], "+"]}], "+", 
             RowBox[{"2", " ", 
              SuperscriptBox[
               StyleBox["Q",
                FontColor->RGBColor[0.8, 0, 0]], "+"], " ", 
              SuperscriptBox[
               OverscriptBox[
                StyleBox["U",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["_",
                 FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  SubsuperscriptBox[
                   StyleBox["q",
                    FontColor->RGBColor[0, 0, 0.8]], 
                   RowBox[{"1", ",", "4"}], "+"], ")"}], "2"], "-", 
                RowBox[{
                 SubsuperscriptBox[
                  StyleBox["q",
                   FontColor->RGBColor[0, 0, 0.8]], 
                  RowBox[{"1", ",", "3"}], "+"], " ", 
                 SubsuperscriptBox[
                  StyleBox["q",
                   FontColor->RGBColor[0, 0, 0.8]], 
                  RowBox[{"2", ",", "4"}], "+"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 SubsuperscriptBox[
                  OverscriptBox[
                   StyleBox["q",
                    FontColor->RGBColor[0, 0, 0.8]], 
                   StyleBox["_",
                    FontColor->RGBColor[0, 0, 0.8]]], "2", "2"], " ", 
                 SubsuperscriptBox[
                  OverscriptBox[
                   StyleBox["q",
                    FontColor->RGBColor[0, 0, 0.8]], 
                   StyleBox["_",
                    FontColor->RGBColor[0, 0, 0.8]]], 
                  RowBox[{"1", ",", "3"}], "-"]}], "-", 
                RowBox[{"2", " ", 
                 SubscriptBox[
                  OverscriptBox[
                   StyleBox["q",
                    FontColor->RGBColor[0, 0, 0.8]], 
                   StyleBox["_",
                    FontColor->RGBColor[0, 0, 0.8]]], "2"], " ", 
                 SubsuperscriptBox[
                  OverscriptBox[
                   StyleBox["q",
                    FontColor->RGBColor[0, 0, 0.8]], 
                   StyleBox["_",
                    FontColor->RGBColor[0, 0, 0.8]]], 
                  RowBox[{"1", ",", "4"}], "-"]}], "+", 
                SubsuperscriptBox[
                 OverscriptBox[
                  StyleBox["q",
                   FontColor->RGBColor[0, 0, 0.8]], 
                  StyleBox["_",
                   FontColor->RGBColor[0, 0, 0.8]]], 
                 RowBox[{"2", ",", "4"}], "-"]}], ")"}]}]}], ")"}]}], "/.", 
          "subcomb"}], "/.", "leadingtran"}], ",", 
        RowBox[{"Q$", "[", "_", "]"}]}], "]"}], "/.", 
      RowBox[{
       RowBox[{"Q$", "[", "r_", "]"}], "\[RuleDelayed]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"Q0$", "[", "r", "]"}], "/.", "subQ0"}], "/.", 
         RowBox[{"u", "\[Rule]", "0"}]}], ")"}]}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", 
       RowBox[{"x$h", "[", "_", "]"}], ",", 
       RowBox[{
        RowBox[{"\[Eta]$bh", "[", "_", "]"}], "[", 
        RowBox[{"-", "2"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.56714850415567*^9, 3.567148508418536*^9}}],

Cell[BoxData[
 RowBox[{"2592", " ", 
  SuperscriptBox["g", "4"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]], ")"}], "2"], 
     RowBox[{"4", " ", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "8"]}]], "-", 
    FractionBox[
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "7"]], "-", 
    FractionBox[
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "5"]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"], "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "6"]], "+", 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "2", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
       RowBox[{
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", "2", "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "+", 
       RowBox[{
        FractionBox["1", "4"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}]}], ")"}]}], "+", 
    FractionBox[
     RowBox[{
      RowBox[{
       FractionBox["1", "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", "2", "]"}]], ")"}], "2"]}]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", "2", "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]]}], "-", 
       RowBox[{
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]], 
              StyleBox["^",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], "3", 
             RowBox[{"[", "2", "]"}]], ")"}], "2"], "+", 
          RowBox[{
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "2", 
            RowBox[{"[", "2", "]"}]], " ", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "4", 
            RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.567148508935728*^9, 3.567248946336941*^9}]
}, Open  ]],

Cell["\<\
Now the question is how to write compactly generating functional from this \
expression? For instance, we can keep it in the current form, and rely later \
on some Shuffle relations. We will leave this question to the time when other \
integrals will be considered. Then it will be simpler.\
\>", "Text",
 CellChangeTimes->{{3.564406505381312*^9, 3.564406518659336*^9}, {
  3.5644067669153976`*^9, 3.564406804552514*^9}, {3.568969625849914*^9, 
  3.568969626650097*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Correction to densities", "Subsection",
 CellChangeTimes->{{3.5644644744594717`*^9, 3.5644644784084787`*^9}}],

Cell[CellGroupData[{

Cell["Pre Init", "Subsubsection",
 CellChangeTimes->{{3.5645553838585033`*^9, 3.564555396671167*^9}}],

Cell[TextData[{
 "First we compute correction to densities. One checked \[Rho] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " and \[Delta]\[Rho] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]]],
 ". \[Rho]2 is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", "1"], TraditionalForm]]],
 " and \[Delta]\[Rho]2 is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", "0"], TraditionalForm]]],
 " . RHS of equations of densities is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]]],
 ". Indeed as it follows from computations below:"
}], "Text",
 CellChangeTimes->{3.564555307828187*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"rhsYtYtran", "/.", 
     RowBox[{
      RowBox[{"TC", "[", 
       RowBox[{"a_", ",", "b_", ",", "c1_", ",", "c2_"}], "]"}], 
      "\[RuleDelayed]", 
      RowBox[{"Integrate", "[", 
       RowBox[{
        RowBox[{"Normal", "[", 
         RowBox[{"Series", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"a", " ", "b"}], "/.", 
            RowBox[{
             SubscriptBox["J", "aa_"], "\[RuleDelayed]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"BesselJ", "[", 
                RowBox[{"aa", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
           RowBox[{"{", 
            RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "]"}], ",", 
        "c1", ",", "c2"}], "]"}]}]}], "//.", "subZetaReduce"}], ",", 
   "zlist"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"%", "//", "FindTranOrder"}]}], "Input",
 CellChangeTimes->{{3.5645549945446377`*^9, 3.564555192322179*^9}, {
  3.564555224699485*^9, 3.564555256600275*^9}, {3.5645558319906063`*^9, 
  3.564555834863653*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"576", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "8"], " ", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "3"]}], "z"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"186624", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2", "4"]}], 
      RowBox[{"25", " ", "z"}]], "+", 
     RowBox[{
      FractionBox["82944", "25"], " ", "\[ImaginaryI]", " ", "z", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "4"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.564555043739388*^9, 3.56455507884444*^9}, 
   3.564555111516478*^9, {3.5645551437065287`*^9, 3.564555199229792*^9}, 
   3.564555235392379*^9, 3.564555270898209*^9, 3.564555841637933*^9}],

Cell[BoxData["\<\"tran order = -2\"\>"], "Output",
 CellChangeTimes->{{3.564555043739388*^9, 3.56455507884444*^9}, 
   3.564555111516478*^9, {3.5645551437065287`*^9, 3.564555199229792*^9}, 
   3.564555235392379*^9, 3.564555270898209*^9, 3.564555841654024*^9}]
}, Open  ]],

Cell[TextData[{
 "Hence we should expand all expressions up to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]]],
 " order."
}], "Text",
 CellChangeTimes->{{3.564555221661827*^9, 3.564555238405395*^9}, {
  3.5645552761805277`*^9, 3.564555280284039*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Equations to expand", "Subsubsection",
 CellChangeTimes->{{3.5645553838585033`*^9, 3.56455540767109*^9}}],

Cell["I duplicate equations to expand:", "Text",
 CellChangeTimes->{{3.5645553150923843`*^9, 3.5645553376740522`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", "\[Delta]\[Rho]"}], "+", "\[Delta]\[Rho]2contrtran", "+", 
  "W1contrtran", "+", "\[Delta]W0tran", "+", "T10q12pq12mtran", "-", 
  "rhsYtYtran"}]], "Input",
 CellChangeTimes->{{3.564854612083584*^9, 3.5648546641642103`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "+"]}], 
        RowBox[{
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}]], 
       ")"}], "r"], "]"}], "+", 
    RowBox[{"Log", "[", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], "2"], "-", 
           SubsuperscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
            RowBox[{"[", "2", "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox[
            OverscriptBox[
             StyleBox["\[GothicQ]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], 
             StyleBox["_",
              FontColor->RGBColor[0.54, 0.35, 0.15]]], "2"], "-", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[GothicQ]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], 
             StyleBox["_",
              FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}]]}], ")"}]}], 
        RowBox[{
         SuperscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]}]], ")"}], "r"], 
     "]"}], "+", 
    RowBox[{"Log", "[", 
     FractionBox[
      SubscriptBox[
       StyleBox["\[GothicCapitalT]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"1", ",", "0"}]], 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["\[GothicQ]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         RowBox[{"1", ",", "2"}], "+"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["_",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], 
         RowBox[{"1", ",", "2"}], "-"]}], ")"}]], "]"}]}], "==", 
   RowBox[{
    RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
      RowBox[{
       RowBox[{"+", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
     RowBox[{
      FractionBox["du", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      FractionBox["1", 
       RowBox[{
        RowBox[{"4", 
         SuperscriptBox["g", "2"], 
         SuperscriptBox["z", "2"]}], "-", 
        SuperscriptBox["u", "2"]}]], 
      FractionBox["H", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]]}]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
   3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
   3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
   3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
   3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
   3.5644070733510857`*^9, 3.56440708351433*^9}, 3.564464488007196*^9, 
   3.564504436271771*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", 
     FractionBox[
      SubscriptBox[
       StyleBox["\[GothicCapitalT]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"1", ",", "0"}]], 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["\[GothicQ]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         RowBox[{"1", ",", "2"}], "+"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["_",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], 
         RowBox[{"1", ",", "2"}], "-"]}], ")"}]], "]"}], "+", 
    RowBox[{"2", 
     RowBox[{"Log", "[", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox[
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], 
          RowBox[{"1", ",", "0"}], "c"], 
         RowBox[{
          SubsuperscriptBox[
           StyleBox["\[GothicCapitalT]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           RowBox[{"0", ",", "1"}], "c"], " "}]], ")"}], "r"], "]"}]}], "+", 
    RowBox[{"Log", "[", " ", 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "2"], "-", 
          SubsuperscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
           RowBox[{"[", "2", "]"}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2"], "-", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], ")"}], 
        SuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], "-"], " ", 
        SuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]}], ")"}], "r"], "]"}], 
    "+", 
    RowBox[{"Log", "[", 
     FractionBox["1", 
      SubscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "+"], " ", 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], " ", 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}], ")"}],
        "r"]], "]"}]}], "==", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
      RowBox[{
       RowBox[{"+", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
     RowBox[{
      FractionBox["du", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      RowBox[{"(", 
       FractionBox[
        RowBox[{"2", "u"}], 
        RowBox[{
         RowBox[{"4", 
          SuperscriptBox["g", "2"], 
          SuperscriptBox["z", "2"]}], "-", 
         SuperscriptBox["u", "2"]}]], ")"}], "H"}]}], "+", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
      RowBox[{
       RowBox[{"+", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
     RowBox[{
      FractionBox["du", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      RowBox[{"(", 
       FractionBox[
        RowBox[{"2", "u"}], 
        RowBox[{
         RowBox[{"4", 
          SuperscriptBox["g", "2"], 
          SuperscriptBox["z", "2"]}], "-", 
         SuperscriptBox["u", "2"]}]], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "2"}], 
        RowBox[{"Log", "[", 
         RowBox[{
          RowBox[{"(", 
           FractionBox[
            RowBox[{" ", 
             SuperscriptBox[
              StyleBox["\[ScriptCapitalT]",
               FontColor->RGBColor[1, 0, 0]], "+"], " "}], 
            SuperscriptBox[
             StyleBox["\[GothicCapitalT]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]], ")"}], "/", 
          SubscriptBox[
           RowBox[{"(", 
            FractionBox[
             RowBox[{" ", 
              SuperscriptBox[
               StyleBox["\[ScriptCapitalT]",
                FontColor->RGBColor[1, 0, 0]], "+"], " "}], 
             SuperscriptBox[
              StyleBox["\[GothicCapitalT]",
               FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]], ")"}], 
           "asym"]}], "]"}]}], ")"}]}]}]}]}], "}"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{
      RowBox[{"-", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
     RowBox[{
      RowBox[{"+", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     FractionBox["1", 
      RowBox[{
       RowBox[{"4", 
        SuperscriptBox["g", "2"], 
        SuperscriptBox["z", "2"]}], "-", 
       SuperscriptBox["u", "2"]}]], 
     FractionBox["H", 
      SqrtBox[
       RowBox[{"1", "-", 
        FractionBox[
         RowBox[{"4", 
          SuperscriptBox["g", "2"]}], 
         SuperscriptBox["u", "2"]]}]]]}]}]}]],
  CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
    3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
    3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
    3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
    3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
    3.5644070733510857`*^9, 3.56440708351433*^9}, 3.564464488007196*^9}],
 "==",
 "rhsYtYtran"
}], "Subsubsection",
 CellChangeTimes->{
  3.564489523742169*^9, {3.5644952583860617`*^9, 3.564495258694702*^9}}],

Cell["\<\
We got a preliminary understanding that all terms have a deficit of \
transcendentality 2. There is no mixture in g now with magic cancellations in \
the end. In a sence, this is good. Now we are going to explicitly write down \
things\
\>", "Text",
 CellChangeTimes->{{3.564489526854745*^9, 3.5644895711320953`*^9}, {
  3.567148733744051*^9, 3.567148734136421*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         OverscriptBox["x", "^"], "-", 
         FractionBox["1", 
          OverscriptBox["x", "^"]]}], 
        RowBox[{
         OverscriptBox["x", "^"], "+", 
         FractionBox["1", 
          OverscriptBox["x", "^"]]}]], ")"}], 
      FractionBox["1", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]]}], "/.", "subJuk"}], "/.", 
    RowBox[{"g", "\[Rule]", "1"}]}], "//", "FullSimplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{"u", "\[Rule]", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "/", "3"}], "-", 
     RowBox[{"\[ImaginaryI]", "/", "10"}]}]}]}], "//", "N"}]}], "Input",
 CellChangeTimes->{{3.564489572717237*^9, 3.564489687071961*^9}}],

Cell[BoxData[
 RowBox[{"1.`", "\[VeryThinSpace]", "+", 
  RowBox[{"1.0581813203458523`*^-16", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.5644897516120996`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{
     RowBox[{"4", 
      SuperscriptBox["g", "2"], 
      SuperscriptBox["z", "2"]}], "-", 
     SuperscriptBox["u", "2"]}]], ",", "u"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", 
    RowBox[{"4", " ", "g", " ", "z", " ", 
     RowBox[{"(", 
      RowBox[{"u", "-", 
       RowBox[{"2", " ", "g", " ", "z"}]}], ")"}]}]]}], "+", 
  FractionBox["1", 
   RowBox[{"4", " ", "g", " ", "z", " ", 
    RowBox[{"(", 
     RowBox[{"u", "+", 
      RowBox[{"2", " ", "g", " ", "z"}]}], ")"}]}]]}]], "Output",
 CellChangeTimes->{3.564489752889226*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{
      RowBox[{"-", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
     RowBox[{
      RowBox[{"+", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     FractionBox["1", 
      RowBox[{
       RowBox[{"4", 
        SuperscriptBox["g", "2"], 
        SuperscriptBox["z", "2"]}], "-", 
       SuperscriptBox["u", "2"]}]], 
     FractionBox["H", 
      SqrtBox[
       RowBox[{"1", "-", 
        FractionBox[
         RowBox[{"4", 
          SuperscriptBox["g", "2"]}], 
         SuperscriptBox["u", "2"]]}]]]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "==", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "z"]}], ")"}], 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{
      RowBox[{"-", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
     RowBox[{
      RowBox[{"+", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "+", 
         RowBox[{"2", " ", "g", " ", "z"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "-", 
         RowBox[{"2", " ", "g", " ", "z"}]}]]}], ")"}], 
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        OverscriptBox["x", "^"], "+", 
        FractionBox["1", 
         OverscriptBox["x", "^"]]}], 
       RowBox[{
        OverscriptBox["x", "^"], "-", 
        FractionBox["1", 
         OverscriptBox["x", "^"]]}]], ")"}], "H"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564489693893202*^9, 3.56448971457535*^9}, {
  3.564489760233118*^9, 3.564489782995253*^9}}],

Cell["\<\
Now I will use Bessel representation of objects, and make integration by \
residues afterwards.\
\>", "Text",
 CellChangeTimes->{{3.564490494420313*^9, 3.564490501935977*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                FractionBox["1", "36"], 
                RowBox[{"Log", "[", 
                 RowBox[{
                  FractionBox[
                   SubscriptBox["T", 
                    RowBox[{"1", ",", "0"}]], 
                   SubsuperscriptBox["T", 
                    RowBox[{"0", ",", "0"}], "+"]], 
                  FractionBox[
                   SuperscriptBox["Q", "+"], 
                   SuperscriptBox["Q", "-"]]}], "]"}]}], "/.", "subTQ"}], "/.",
               "subpqb"}], "//.", "substandardq"}], "/.", 
            "subtoanalyticitystrips"}], "//.", "substandardq"}], "/.", 
          "tranpowers"}], ",", 
         RowBox[{"{", 
          RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], ",", 
       RowBox[{"U$b", "[", "_", "]"}]}], "]"}], "/.", 
     RowBox[{"tran", "\[Rule]", "1"}]}], "/.", "leadingtran"}], "//", 
   "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{
     RowBox[{"Q$", "[", "r_", "]"}], "\[Rule]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Q0$", "[", "r", "]"}], "/.", "subQ0"}], "/.", 
       RowBox[{"u", "\[Rule]", "0"}]}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["g", "2"], ",", 
     RowBox[{"x$h", "[", "0", "]"}]}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.564490267308949*^9, 3.564490475416184*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", "2", "]"}]], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "+", 
    FractionBox[
     RowBox[{"2", " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
    FractionBox[
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.56449029351615*^9, 3.564490504936891*^9}, 
   3.564496504493559*^9, 3.567148756454976*^9}]
}, Open  ]],

Cell["So we get leading transcendentality representation ", "Text",
 CellChangeTimes->{{3.564490507815213*^9, 3.5644905152229433`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{
      OverscriptBox["x", "^"], "+", 
      FractionBox["1", 
       OverscriptBox["x", "^"]]}], 
     RowBox[{
      OverscriptBox["x", "^"], "-", 
      FractionBox["1", 
       OverscriptBox["x", "^"]]}]], ")"}], "H"}], "\[Equal]", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{"(", 
    FractionBox[
     RowBox[{
      FractionBox["1", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]]], "+", 
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]]}], 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]]]}], "+", 
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]]}]], ")"}], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
     FractionBox[
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.564490520676508*^9, 3.5644905384931507`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{
      FractionBox["1", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]]], "+", 
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]]}], 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]]]}], "+", 
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]]}]], ")"}], 
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "a"]]}], "\[Equal]", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{
         SubscriptBox["J", "a"], "[", 
         RowBox[{"2", "g", " ", "z"}], "]"}], ",", "z"}], "]"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", ">", "0"}], ",", 
      RowBox[{
       RowBox[{"Im", "[", "u", "]"}], "<", "0"}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.564489864157662*^9, 3.5644898769252367`*^9}, {
  3.564490543555594*^9, 3.564490548883254*^9}, {3.5644905951100883`*^9, 
  3.564490595413952*^9}, {3.564490855508321*^9, 3.564490872990353*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "a"]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   OverscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    StyleBox["^",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]]], "a", 
   RowBox[{"[", "2", "]"}]], "\[Equal]", 
  RowBox[{
   FractionBox[
    SuperscriptBox["D", "2"], 
    RowBox[{"1", "-", 
     SuperscriptBox["D", "2"]}]], 
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "a"]]}], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{
    FractionBox["dz", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}]], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5644906411423197`*^9, 3.5644907019413433`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"subtoBes1", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "2", "]"}], 
      "\[RuleDelayed]", " ", 
      RowBox[{"a", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", "a"], "[", 
         RowBox[{"2", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}], 
        SubscriptBox["z", "1"]]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Eta]$bh", "[", "a_", "]"}], "[", 
       RowBox[{"-", "2"}], "]"}], "\[RuleDelayed]", " ", 
      RowBox[{"a", 
       SuperscriptBox[
        RowBox[{"(", "\[ImaginaryI]", ")"}], "a"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", "a"], "[", 
         RowBox[{"2", "g", " ", 
          SubscriptBox["z", "3"]}], "]"}], 
        SubscriptBox["z", "3"]]}]}], ",", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"x$h", "[", "0", "]"}], 
       RowBox[{"a_", "/;", 
        RowBox[{"a", "<", "0"}]}]], ":>", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", "\[ImaginaryI]", ")"}], 
        RowBox[{"-", "a"}]], 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{
          SubscriptBox["J", 
           RowBox[{"-", "a"}]], "[", 
          RowBox[{"2", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}], ",", 
         SubscriptBox["z", "2"]}], "]"}]}]}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.56449094148407*^9, 3.564491035603531*^9}, {
   3.564495153740864*^9, 3.5644951655349503`*^9}, 3.564495213270697*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{
      OverscriptBox["x", "^"], "+", 
      FractionBox["1", 
       OverscriptBox["x", "^"]]}], 
     RowBox[{
      OverscriptBox["x", "^"], "-", 
      FractionBox["1", 
       OverscriptBox["x", "^"]]}]], ")"}], "H"}], "\[Equal]", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dz", "1"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       SubscriptBox["dz", "2"], 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "u", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"8", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "2"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]], "+", 
         FractionBox[
          RowBox[{"12", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "3"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]], "+", 
         FractionBox[
          RowBox[{"4", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "4"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]]}], ")"}]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.564490713973291*^9, 3.564490714109326*^9}, {
  3.564490813127357*^9, 3.56449083534188*^9}, {3.564490877087302*^9, 
  3.564490882602582*^9}, {3.564490919288001*^9, 3.5644909405783987`*^9}, {
  3.5644910278081627`*^9, 3.56449109886162*^9}, {3.564491170001313*^9, 
  3.564491178122882*^9}}],

Cell["Now we are about to perform integration by residues:", "Text",
 CellChangeTimes->{{3.564491517866761*^9, 3.564491534608285*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", 
   RowBox[{
    RowBox[{"-", "\[Infinity]"}], "-", 
    RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
   RowBox[{
    RowBox[{"+", "\[Infinity]"}], "-", 
    RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
  RowBox[{
   FractionBox["du", 
    RowBox[{
     RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"u", "+", 
       RowBox[{"2", " ", "g", " ", "z"}]}]], "-", 
     FractionBox["1", 
      RowBox[{"u", "-", 
       RowBox[{"2", " ", "g", " ", "z"}]}]]}], ")"}], 
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{"\[ImaginaryI]", " ", "u", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["z", "1"], "-", 
       SubscriptBox["z", "2"]}], ")"}]}]]}]}]], "Input",
 CellChangeTimes->{3.564491558020063*^9}],

Cell[TextData[{
 "z is on the real axis. And u is slightly below. So we have simple \
statement. In the region ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], ">", 
    SubscriptBox["z", "2"]}], TraditionalForm]]],
 " we can close integration contour above. There we will meet the pole u== 2g \
z and u==-2gz, and get the answer"
}], "Text",
 CellChangeTimes->{{3.564491560745576*^9, 3.564491642444343*^9}, {
  3.564492762490341*^9, 3.564492765930209*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["z", "1"], "-", 
      SubscriptBox["z", "2"]}], ")"}]}]], "-", 
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["z", "1"], "-", 
      SubscriptBox["z", "2"]}], ")"}]}]]}]], "Input",
 CellChangeTimes->{{3.564491647448083*^9, 3.564491648518939*^9}, {
  3.564492767652121*^9, 3.56449277186838*^9}, {3.564492846675461*^9, 
  3.564492846865506*^9}}],

Cell[TextData[{
 "If the region ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], "<", 
    SubscriptBox["z", "2"]}], TraditionalForm]]],
 ", we can close integration contour below, then we get nothing. Because we \
are already slightly below real axis. Hence a conclusion is that"
}], "Text",
 CellChangeTimes->{{3.564491658627589*^9, 3.5644917021780033`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rhsYtYtran", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "z"]}], ")"}], "36", 
   SuperscriptBox["g", "2"], 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}]], "-", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"8", " ", "g", " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            SubscriptBox["J", "2"], "\[Prime]",
            MultilineFunction->None], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "1"]], "+", 
        FractionBox[
         RowBox[{"12", " ", "g", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            SubscriptBox["J", "3"], "\[Prime]",
            MultilineFunction->None], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "1"]], "+", 
        FractionBox[
         RowBox[{"4", " ", "g", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            SubscriptBox["J", "4"], "\[Prime]",
            MultilineFunction->None], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "1"]]}], ")"}]}], ",", 
     FractionBox["1", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{
  3.564493279733573*^9, {3.565349906461842*^9, 3.565349906706646*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   FractionBox["1", "z"], 
   RowBox[{"36", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "2"], " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]]}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"8", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "2"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]], "+", 
         FractionBox[
          RowBox[{"12", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "3"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]], "+", 
         FractionBox[
          RowBox[{"4", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "4"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.564493280175452*^9, 3.5648546831630783`*^9, 
  3.565347234594789*^9, 3.5653499072130423`*^9, 3.56714891370469*^9}]
}, Closed]],

Cell["\<\
To check the validity of this guy, we expand it at small g and compare the \
obtained answer with  result computed by old means\
\>", "Text",
 CellChangeTimes->{{3.564491803504045*^9, 3.564491837972851*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]], "-", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"8", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "2"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           SubscriptBox["z", "1"]], "+", 
          FractionBox[
           RowBox[{"12", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "3"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           SubscriptBox["z", "1"]], "+", 
          FractionBox[
           RowBox[{"4", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "4"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           SubscriptBox["z", "1"]]}], ")"}]}], "/.", 
       RowBox[{
        SubscriptBox["J", "a_"], "\[RuleDelayed]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"BesselJ", "[", 
           RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], ",", "g", ",", 
    RowBox[{
     RowBox[{"Integrate", "[", 
      RowBox[{"#", ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}], "&"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", 
         FractionBox["\[ImaginaryI]", "z"]}], ")"}], "36", 
       SuperscriptBox["g", "2"], 
       RowBox[{"Collect", "[", 
        RowBox[{
         RowBox[{"%", " ", 
          SubsuperscriptBox["z", "1", "50"]}], ",", 
         SubscriptBox["z", "1"]}], "]"}]}], "/.", 
      RowBox[{
       SubsuperscriptBox["z", "1", "a_"], "\[RuleDelayed]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]$", "[", 
           RowBox[{"a", "+", "1", "-", "50"}], "]"}], "[", "0", "]"}], 
         RowBox[{"Gamma", "[", 
          RowBox[{"a", "+", "1", "-", "50"}], "]"}]}], ")"}]}]}], "//.", 
     "subZetaReduce"}], "/.", 
    RowBox[{
     RowBox[{"Zeta", "[", "a_", "]"}], "\[RuleDelayed]", 
     RowBox[{
      RowBox[{"\[Zeta]$", "[", "a", "]"}], "[", "0", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"g", ",", "\[Zeta]list", ",", "z"}], "}"}], "//", "Flatten"}], 
   ",", "Simplify"}], "]"}]}], "Input",
 CellChangeTimes->{{3.564491839804974*^9, 3.5644920753883743`*^9}, {
   3.564492203408635*^9, 3.564492236860344*^9}, {3.564492284734827*^9, 
   3.564492377852816*^9}, 3.564492795631463*^9, {3.56449284956282*^9, 
   3.564492865153576*^9}, {3.564493169294969*^9, 3.564493169993339*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"8064", " ", 
   SuperscriptBox["g", "9"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "188352"}], "-", 
     RowBox[{"69120", " ", 
      SuperscriptBox["z", "2"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{"3234528", "+", 
     RowBox[{"2014848", " ", 
      SuperscriptBox["z", "2"]}], "+", 
     RowBox[{"506880", " ", 
      SuperscriptBox["z", "4"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "15"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "50068512"}], "-", 
     RowBox[{"39312000", " ", 
      SuperscriptBox["z", "2"]}], "-", 
     RowBox[{"18869760", " ", 
      SuperscriptBox["z", "4"]}], "-", 
     RowBox[{"3354624", " ", 
      SuperscriptBox["z", "6"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "17"], " ", 
   RowBox[{"(", 
    RowBox[{"742710528", "+", 
     RowBox[{"655958016", " ", 
      SuperscriptBox["z", "2"]}], "+", 
     RowBox[{"433649664", " ", 
      SuperscriptBox["z", "4"]}], "+", 
     RowBox[{"158441472", " ", 
      SuperscriptBox["z", "6"]}], "+", 
     RowBox[{"20643840", " ", 
      SuperscriptBox["z", "8"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "15"]}]}]], "Output",
 CellChangeTimes->{{3.564491866468952*^9, 3.564491871952042*^9}, 
   3.5644919021843433`*^9, {3.564491977905754*^9, 3.564492057851514*^9}, {
   3.564492199806299*^9, 3.5644922418978443`*^9}, {3.564492338781246*^9, 
   3.564492382986326*^9}, 3.564492800592698*^9, {3.564492852448943*^9, 
   3.564492869085723*^9}, 3.5644931751219254`*^9, 3.5644960603268213`*^9, 
   3.564854696695589*^9, 3.567148933332466*^9}]
}, Closed]],

Cell["\<\
This a very fast generation of leading transcendentality terms. I will try to \
reproduce the same result from canonical approach.\
\>", "Text",
 CellChangeTimes->{{3.5644920621967287`*^9, 3.564492096210464*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], 
  RowBox[{
   SubsuperscriptBox["\[Integral]", 
    RowBox[{
     RowBox[{"-", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
    RowBox[{
     RowBox[{"+", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
   RowBox[{
    FractionBox["du", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    FractionBox["1", 
     RowBox[{
      RowBox[{"4", 
       SuperscriptBox["g", "2"], 
       SuperscriptBox["z", "2"]}], "-", 
      SuperscriptBox["u", "2"]}]], 
    FractionBox["H", 
     SqrtBox[
      RowBox[{"1", "-", 
       FractionBox[
        RowBox[{"4", 
         SuperscriptBox["g", "2"]}], 
        SuperscriptBox["u", "2"]]}]]]}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tmp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           FractionBox["1", 
            RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], 
          RowBox[{"Collect", "[", 
           RowBox[{
            RowBox[{"Series", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                FractionBox["1", 
                 RowBox[{
                  RowBox[{"4", 
                   SuperscriptBox["g", "2"], 
                   SuperscriptBox["z", "2"]}], "-", 
                  SuperscriptBox["u", "2"]}]], 
                FractionBox["H", 
                 SqrtBox[
                  RowBox[{"1", "-", 
                   FractionBox[
                    RowBox[{"4", 
                    SuperscriptBox["g", "2"]}], 
                    SuperscriptBox["u", "2"]]}]]]}], "/.", 
               RowBox[{"leadweak", "[", "2", "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}], ",", "g"}],
            "]"}]}], "/.", "subtousu"}], "/.", "subQ0"}], "//", 
       "funapartusu"}], "//", "funreduceusu"}], ",", "canlist"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", "\"\<hey\>\"", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp2", "=", 
   RowBox[{"UpIntegrate", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tmp", "/.", 
        RowBox[{"Shift", "[", 
         RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"u", "-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}], "2"]}]}]}], "/.", 
      "subusu"}], ",", "u"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.56449217402029*^9, 3.564492186253193*^9}, {
  3.564492420352888*^9, 3.564492570235972*^9}}],

Cell[BoxData["\<\"hey\"\>"], "Print",
 CellChangeTimes->{3.564492551568494*^9, 3.567148945948606*^9, 
  3.567149005706691*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Coefficient", "[", 
      RowBox[{"#", ",", "tran", ",", 
       RowBox[{"Exponent", "[", 
        RowBox[{"#", ",", "tran"}], "]"}]}], "]"}], "&"}], "[", 
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", " ", "tmp2"}], "/.", 
         "EndSimplify"}], "//.", "subZetaReduce"}], ",", "canlist"}], "]"}], "/.",
      "subtotran"}], "]"}], ",", "canlist"}], "]"}]], "Input",
 CellChangeTimes->{{3.5644925782077723`*^9, 3.564492684275065*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"8064", " ", 
   SuperscriptBox["g", "9"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "188352"}], "-", 
     RowBox[{"69120", " ", 
      SuperscriptBox["z", "2"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}]}]], "Output",
 CellChangeTimes->{{3.564492609937276*^9, 3.564492624575189*^9}, {
   3.5644926659868517`*^9, 3.564492686603736*^9}, 3.567149015091919*^9}]
}, Open  ]],

Cell["\<\
So both approaches match at first two orders. And I would believe that they \
match at higher orders (to check later with explicit computations).\
\>", "Text",
 CellChangeTimes->{{3.564492896222876*^9, 3.564492905244018*^9}, {
  3.564492960043572*^9, 3.56449298228133*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "-"], " ", 
       SuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "+"]}], 
      RowBox[{
       SubsuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], 
       SubsuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}]], ")"}], 
    "r"], "]"}]],
  CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
    3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
    3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
    3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
    3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
    3.5644070733510857`*^9, 3.56440708351433*^9}, 3.564464488007196*^9}],
 "==2 \[Delta]\[Rho],",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"Log", "[", 
    FractionBox["1", 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         StyleBox["\[ScriptCapitalT]",
          FontColor->RGBColor[1, 0, 0]], "-"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptCapitalT]",
          FontColor->RGBColor[1, 0, 0]], "+"], " ", 
        SubsuperscriptBox[
         StyleBox["\[ScriptCapitalT]",
          FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], " ", 
        SubsuperscriptBox[
         StyleBox["\[ScriptCapitalT]",
          FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}], ")"}], 
      "r"]], "]"}]}]],
  CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
    3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
    3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
    3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
    3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
    3.5644070733510857`*^9, 3.56440708351433*^9}, 3.564464488007196*^9}],
 "==0"
}], "Subsubsection",
 CellChangeTimes->{
  3.5644645018830833`*^9, {3.5644935749175*^9, 3.56449359763474*^9}, {
   3.564493636194393*^9, 3.564493638217267*^9}, {3.564494980032051*^9, 
   3.564494983258565*^9}, {3.564555475353527*^9, 3.56455548268537*^9}}],

Cell["\<\
We analysed leading transcendentality of asymptotic solution. Here is \
conclusiong about their structure:\
\>", "Text",
 CellChangeTimes->{{3.564493364603285*^9, 3.564493382626129*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["\[ScriptCapitalT]",
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox["^",
       FontColor->RGBColor[1, 0, 0]]], "-"], "\[TildeEqual]", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["\[ScriptCapitalT]",
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox["^",
       FontColor->RGBColor[1, 0, 0]]], "+"], "\[TildeEqual]", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[ScriptCapitalT]",
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox["^",
       FontColor->RGBColor[1, 0, 0]]], "\[FivePointedStar]", "+"], 
    "\[TildeEqual]", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[ScriptCapitalT]",
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox["^",
       FontColor->RGBColor[1, 0, 0]]], "\[FivePointedStar]", "-"], 
    "\[TildeEqual]", "1"}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["\[GothicCapitalT]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], 
      StyleBox["^",
       FontColor->RGBColor[0.54, 0.35, 0.15]]], "-"], "\[TildeEqual]", 
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", 
      RowBox[{"3", " ", "g", " ", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}]]}]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[GothicCapitalT]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], 
      StyleBox["^",
       FontColor->RGBColor[0.54, 0.35, 0.15]]], "\[FivePointedStar]", "-"], 
    "\[TildeEqual]", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], 
      RowBox[{"3", " ", "g"}]]}]}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["\[GothicCapitalT]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], 
      StyleBox["^",
       FontColor->RGBColor[0.54, 0.35, 0.15]]], "+"], "\[TildeEqual]", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}], 
     RowBox[{"3", " ", "g", " "}]]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[GothicCapitalT]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], 
      StyleBox["^",
       FontColor->RGBColor[0.54, 0.35, 0.15]]], "\[FivePointedStar]", "+"], 
    "\[TildeEqual]", 
    FractionBox["\[ImaginaryI]", 
     RowBox[{"3", " ", "g", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}]]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.564464994724319*^9, 3.564465125725252*^9}}],

Cell["\<\
Corrections are given, so it is straightforward to compute things\
\>", "Text",
 CellChangeTimes->{{3.5644933855385513`*^9, 3.564493412369186*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Delta]\[ScriptCapitalT]", "-"], "\[Equal]", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], "\[Delta]\[Rho]"}], "+", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}]}], ",", 
   RowBox[{
    SubsuperscriptBox["\[Delta]\[ScriptCapitalT]", "\[FivePointedStar]", "-"],
     "\[Equal]", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], "\[Delta]\[Rho]"}], "+", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}]}], ",", 
   RowBox[{
    SuperscriptBox["\[Delta]\[ScriptCapitalT]", "+"], "\[Equal]", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], "\[Delta]\[Rho]"}], "-", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}]}], ",", 
   RowBox[{
    SubsuperscriptBox["\[Delta]\[ScriptCapitalT]", "\[FivePointedStar]", "+"],
     "\[Equal]", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], "\[Delta]\[Rho]"}], "-", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}]}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "-"], " ", 
       SuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "+"]}], 
      RowBox[{
       SubsuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], 
       SubsuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}]], ")"}], 
    "r"], "]"}], "==", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], "\[Delta]\[Rho]"}], "+", 
   RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}], "+", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], "\[Delta]\[Rho]"}], "-", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], "\[Delta]\[Rho]"}], "+", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], "\[Delta]\[Rho]"}], "-", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}]}], "==", 
  RowBox[{"2", " ", "\[Delta]\[Rho]"}]}]], "Input",
 CellChangeTimes->{{3.564493417233017*^9, 3.564493449832871*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      SubscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "+"], " ", 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], " ", 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}], ")"}],
        "r"]], ")"}], "r"], "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", "2"], "\[Delta]\[Rho]"}], "+", 
      RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}]}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], "\[Delta]\[Rho]"}], "-", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], "\[Delta]\[Rho]"}], "+", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], "\[Delta]\[Rho]"}], "-", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]", "]"}]}], ")"}]}], "\[Equal]", 
  "0"}]], "Input",
 CellChangeTimes->{{3.564493612401104*^9, 3.564493628199295*^9}}],

Cell["The same answer as in the small g expansion.", "Text",
 CellChangeTimes->{{3.564493451991499*^9, 3.5644934668545523`*^9}, {
  3.564555533681877*^9, 3.5645555395777283`*^9}}],

Cell[TextData[{
 "We computed ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]]],
 " orders and we should not compute more"
}], "Text",
 CellChangeTimes->{{3.564555561778266*^9, 3.564555571222686*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{"Log", "[", 
   FractionBox[
    SubscriptBox[
     StyleBox["T",
      FontColor->RGBColor[0.54, 0.35, 0.15]], 
     RowBox[{"1", ",", "0"}]], 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["q", 
       RowBox[{"1", ",", "2"}], "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox["q", 
        StyleBox["_",
         FontColor->RGBColor[0.54, 0.35, 0.15]]], 
       RowBox[{"1", ",", "2"}], "-"]}], ")"}]], "]"}]],
  CellChangeTimes->{{3.564488991952964*^9, 3.564489031439087*^9}}],
 "==",
 "T10q12pq12mtran"
}], "Subsubsection",
 CellChangeTimes->{
  3.564495053353127*^9, {3.564495357294701*^9, 3.5644953576482677`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Log", "[", 
         FractionBox[
          SubscriptBox[
           StyleBox["T",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           RowBox[{"1", ",", "0"}]], 
          RowBox[{"(", 
           RowBox[{
            SubsuperscriptBox["q", 
             RowBox[{"1", ",", "2"}], "+"], " ", 
            SubsuperscriptBox[
             OverscriptBox["q", 
              StyleBox["_",
               FontColor->RGBColor[0.54, 0.35, 0.15]]], 
             RowBox[{"1", ",", "2"}], "-"]}], ")"}]], "]"}], "/.", "subTQ"}], 
       "/.", "subpqb"}], "/.", "substandardq"}], "/.", "tranpowers"}], ",", 
    RowBox[{"{", 
     RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], "//", 
  "Normal"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"tran", "\[Rule]", "1"}]}], "/.", "leadingtran"}], "//", 
  "Simplify"}], "\[IndentingNewLine]", 
 RowBox[{"%", "/.", "subtoBes1"}]}], "Input",
 CellChangeTimes->{{3.564488991952964*^9, 3.564489031439087*^9}, {
   3.56449505896703*^9, 3.5644951028788977`*^9}, {3.564495180422647*^9, 
   3.564495184175963*^9}, 3.564555602769293*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SubsuperscriptBox[
      StyleBox["q",
       FontColor->RGBColor[0, 0, 0.8]], 
      RowBox[{"2", ",", "4"}], "+"]}], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["q",
       FontColor->RGBColor[0, 0, 0.8]], 
      StyleBox["_",
       FontColor->RGBColor[0, 0, 0.8]]], 
     RowBox[{"1", ",", "3"}], "-"]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox[
     StyleBox["q",
      FontColor->RGBColor[0, 0, 0.8]], 
     RowBox[{"1", ",", "4"}], "+"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["q",
       FontColor->RGBColor[0, 0, 0.8]], 
      StyleBox["_",
       FontColor->RGBColor[0, 0, 0.8]]], 
     RowBox[{"1", ",", "4"}], "-"]}], "-", 
   RowBox[{
    SubsuperscriptBox[
     StyleBox["q",
      FontColor->RGBColor[0, 0, 0.8]], 
     RowBox[{"1", ",", "3"}], "+"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["q",
       FontColor->RGBColor[0, 0, 0.8]], 
      StyleBox["_",
       FontColor->RGBColor[0, 0, 0.8]]], 
     RowBox[{"2", ",", "4"}], "-"]}]}], 
  RowBox[{
   SuperscriptBox["tran", "2"], " ", 
   SuperscriptBox[
    StyleBox["Q",
     FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
   SuperscriptBox[
    StyleBox["Q",
     FontColor->RGBColor[0.8, 0, 0]], "+"]}]]], "Output",
 CellChangeTimes->{{3.56448900238015*^9, 3.564489031655635*^9}, 
   3.5644950558427343`*^9, {3.564495090175426*^9, 3.564495103362109*^9}, 
   3.564495185029694*^9, 3.564495218801437*^9, {3.564555595515074*^9, 
   3.564555603128248*^9}, 3.564844574565695*^9, 3.567149506581808*^9}],

Cell[BoxData[
 RowBox[{"36", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", "2", "]"}]]}], "-", 
    RowBox[{"2", " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.56448900238015*^9, 3.564489031655635*^9}, 
   3.5644950558427343`*^9, {3.564495090175426*^9, 3.564495103362109*^9}, 
   3.564495185029694*^9, 3.564495218801437*^9, {3.564555595515074*^9, 
   3.564555603128248*^9}, 3.564844574565695*^9, 3.567149506592153*^9}],

Cell[BoxData[
 RowBox[{"36", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"18", " ", 
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "1"]}], "]"}], " ", 
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "3"]}], "]"}]}], 
      RowBox[{
       SubscriptBox["z", "1"], " ", 
       SubscriptBox["z", "3"]}]]}], "-", 
    FractionBox[
     RowBox[{"8", " ", 
      RowBox[{
       SubscriptBox["J", "2"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}], " ", 
      RowBox[{
       SubscriptBox["J", "4"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     RowBox[{
      SubscriptBox["z", "1"], " ", 
      SubscriptBox["z", "3"]}]], "-", 
    FractionBox[
     RowBox[{"8", " ", 
      RowBox[{
       SubscriptBox["J", "2"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}], " ", 
      RowBox[{
       SubscriptBox["J", "4"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}]}], 
     RowBox[{
      SubscriptBox["z", "1"], " ", 
      SubscriptBox["z", "3"]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.56448900238015*^9, 3.564489031655635*^9}, 
   3.5644950558427343`*^9, {3.564495090175426*^9, 3.564495103362109*^9}, 
   3.564495185029694*^9, 3.564495218801437*^9, {3.564555595515074*^9, 
   3.564555603128248*^9}, 3.564844574565695*^9, 3.567149506598309*^9}]
}, Open  ]],

Cell["subtoBes represent Laplace representation of the quantities.", "Text",
 CellChangeTimes->{{3.5645557607044067`*^9, 3.564555787029809*^9}}],

Cell[BoxData[{
 RowBox[{"subtoBes1", " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "this", " ", "one", " ", "for", " ", "slightly", " ", "below", " ", "real",
     " ", "axis", " ", "and", " ", "measure", " ", 
    FractionBox[
     RowBox[{"x", "+", 
      FractionBox["1", "x"]}], 
     RowBox[{"x", "-", 
      FractionBox["1", "x"]}]], "in", " ", "front", " ", "of"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"subtoBes2", " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "this", " ", "one", " ", "for", " ", "slightly", " ", "below", " ", "real",
     " ", "axis", " ", "and", " ", "no", " ", "special", " ", "measure", " ", 
    "in", " ", "front", " ", "of"}], " ", "*)"}]}]}], "Input",
 CellChangeTimes->{{3.564555666030179*^9, 3.5645557407175837`*^9}, {
  3.567149544830237*^9, 3.567149547508313*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", "2", "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     SubscriptBox["z", "1"]]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox["\[ImaginaryI]", "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}]}], 
     SubscriptBox["z", "3"]]}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{"a_", "/;", 
      RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
    RowBox[{
     SuperscriptBox["\[ImaginaryI]", 
      RowBox[{"-", "a"}]], " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["z", "2"]], 
      RowBox[{
       SubscriptBox["J", 
        RowBox[{"-", "a"}]], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}]}]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.5671495825278473`*^9, 3.567149583487596*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", "2", "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     SubscriptBox["z", "1"]]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox["\[ImaginaryI]", "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}]}], 
     SubscriptBox["z", "3"]]}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{"a_", "/;", 
      RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"a", " ", 
       SuperscriptBox["\[ImaginaryI]", 
        RowBox[{"-", "a"}]], " ", 
       RowBox[{
        SubscriptBox["J", 
         RowBox[{"-", "a"}]], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}]}], 
      SubscriptBox["z", "2"]]}]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.564555670532844*^9, {3.564555712821083*^9, 3.564555719336371*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"T10q12pq12mtran", "=", 
   RowBox[{"36", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "u", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "3"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"18", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "3"]}]]}], "-", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "3"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "3"]}]], "-", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "3"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
      RowBox[{
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "3"]]}]]}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
     "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.56449527202882*^9, 3.56449534328274*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ExpaCol", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["z", "1"], 
      SubscriptBox["z", "3"], 
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"T10q12pq12mtran", "/.", 
           RowBox[{
            RowBox[{"TC", "[", 
             RowBox[{"a_", ",", "b_", ",", "c__"}], "]"}], "\[RuleDelayed]", 
            "a"}]}], "/.", 
          RowBox[{
           SubscriptBox["J", "a_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}], "/.", 
         RowBox[{"u", "\[Rule]", 
          RowBox[{"2", "g", " ", "z"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}]}], ",", 
     RowBox[{
      SubscriptBox["z", "1"], "|", 
      SubscriptBox["z", "3"]}]}], "]"}], "/.", 
   RowBox[{
    SubsuperscriptBox["z", "i_", "a_"], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Zeta]$", "[", 
        RowBox[{"a", "+", "1", "-", "1"}], "]"}], "[", "0", "]"}], 
      RowBox[{"Gamma", "[", 
       RowBox[{"a", "+", "1", "-", "1"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"%", "//.", "subZetaReduce"}], ",", "canlist"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"%", "//", "FindTranOrder"}]}], "Input",
 CellChangeTimes->{{3.564495367038109*^9, 3.564495417127839*^9}, {
  3.564495458816358*^9, 3.5644955334383907`*^9}, {3.56455579559692*^9, 
  3.564555798753684*^9}, {3.564555850466586*^9, 3.564555853103084*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"144", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "3"]}], "5"]}], "-", 
     RowBox[{"72", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["15552", "175"], "+", 
        FractionBox[
         RowBox[{"66816", " ", 
          SuperscriptBox["z", "2"]}], "175"]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "4"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"432", "+", 
        RowBox[{"1152", " ", 
         SuperscriptBox["z", "2"]}]}], ")"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.564495373248806*^9, 3.5644954175628653`*^9}, {
   3.5644954612388268`*^9, 3.564495533846681*^9}, 3.5645556188141623`*^9, 
   3.564555792162919*^9, 3.564555853492373*^9, 3.565347263295089*^9, 
   3.567149654185404*^9}],

Cell[BoxData["\<\"tran order = -2\"\>"], "Output",
 CellChangeTimes->{{3.564495373248806*^9, 3.5644954175628653`*^9}, {
   3.5644954612388268`*^9, 3.564495533846681*^9}, 3.5645556188141623`*^9, 
   3.564555792162919*^9, 3.564555853492373*^9, 3.565347263295089*^9, 
   3.5671496541917543`*^9}]
}, Open  ]],

Cell["\<\
But actually I would prefer to remain in \[Eta]-variables for a while (mainly \
because I can)\
\>", "Text",
 CellChangeTimes->{{3.5653501338853683`*^9, 3.565350160424365*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"T10q12pq12mtran", "=", 
  RowBox[{"36", " ", 
   SuperscriptBox["g", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", "2", "]"}]]}], "-", 
     RowBox[{"2", " ", 
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.5653501743825407`*^9, 3.565350174785221*^9}}],

Cell[BoxData[
 RowBox[{"36", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", "2", "]"}]]}], "-", 
    RowBox[{"2", " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.565350175268202*^9, 3.567149659882927*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["\[Integral]", 
    RowBox[{
     RowBox[{"-", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
    RowBox[{
     RowBox[{"+", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
   RowBox[{
    FractionBox["du", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    RowBox[{"(", 
     FractionBox[
      RowBox[{"2", "u"}], 
      RowBox[{
       RowBox[{"4", 
        SuperscriptBox["g", "2"], 
        SuperscriptBox["z", "2"]}], "-", 
       SuperscriptBox["u", "2"]}]], ")"}], "H"}]}]]],
 "==",
 "rhsYoYtran"
}], "Subsubsection",
 CellChangeTimes->{
  3.56449568501725*^9, {3.5645044088534727`*^9, 3.56450440930469*^9}}],

Cell["We make a similar job as before", "Text",
 CellChangeTimes->{{3.5644957000077667`*^9, 3.564495725356617*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart", "[", 
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{"2", "u"}], 
     RowBox[{
      RowBox[{"4", 
       SuperscriptBox["g", "2"], 
       SuperscriptBox["z", "2"]}], "-", 
      SuperscriptBox["u", "2"]}]], ")"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.564495728680805*^9, 3.564495730369396*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", 
    RowBox[{"u", "-", 
     RowBox[{"2", " ", "g", " ", "z"}]}]]}], "-", 
  FractionBox["1", 
   RowBox[{"u", "+", 
    RowBox[{"2", " ", "g", " ", "z"}]}]]}]], "Output",
 CellChangeTimes->{3.564495730799705*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Select", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"H", "/.", 
         RowBox[{"leadweak", "[", "1", "]"}]}], "//", "Expand"}], ",", 
       RowBox[{
        RowBox[{"Not", "[", 
         RowBox[{"FreeQ", "[", 
          RowBox[{"#", ",", "\[Eta]$"}], "]"}], "]"}], "&"}]}], "]"}], "/.", 
     "subtousu"}], "//", "funapartusu"}], "//", "funreduceusu"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", "subtotran"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Coefficient", "[", 
    RowBox[{"#", ",", "tran", ",", 
     RowBox[{"Exponent", "[", 
      RowBox[{"#", ",", "tran"}], "]"}]}], "]"}], "&"}], "[", "%", 
  "]"}]}], "Input",
 CellChangeTimes->{{3.564496371256446*^9, 3.564496441790091*^9}, {
  3.564496593913813*^9, 3.564496696021875*^9}, {3.5645034510135927`*^9, 
  3.56450349728829*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"36", " ", 
     SuperscriptBox["g", "8"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], "2", 
      RowBox[{"[", "2", "]"}]]}], 
    SuperscriptBox[
     StyleBox["\[ScriptU]",
      FontColor->RGBColor[0, 0, 0.4]], "4"]]}], "+", 
  FractionBox[
   RowBox[{"72", " ", 
    SuperscriptBox["g", "8"], " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "3", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    StyleBox["\[ScriptU]",
     FontColor->RGBColor[0, 0, 0.4]], "3"]], "-", 
  FractionBox[
   RowBox[{"36", " ", 
    SuperscriptBox["g", "8"], " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "4", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    StyleBox["\[ScriptU]",
     FontColor->RGBColor[0, 0, 0.4]], "2"]]}]], "Output",
 CellChangeTimes->{{3.564496377080595*^9, 3.564496442221933*^9}, {
   3.564496595313744*^9, 3.564496697979558*^9}, {3.564503452221302*^9, 
   3.564503458072094*^9}, 3.564503497833384*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["2", "u"]}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"36", " ", 
         SuperscriptBox["g", "8"], " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], "2", 
          RowBox[{"[", "2", "]"}]]}], 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "4"]]}], "+", 
      FractionBox[
       RowBox[{"72", " ", 
        SuperscriptBox["g", "8"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "3"]], "-", 
      FractionBox[
       RowBox[{"36", " ", 
        SuperscriptBox["g", "8"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "4", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}]}], "/.", "subusu"}], 
  "//", "Expand"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"UpIntegrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["%", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
      RowBox[{"Shift", "[", 
       RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
     RowBox[{"u", "\[Rule]", 
      RowBox[{"u", "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}], "2"]}]}]}], ",", "u"}], 
   "]"}], "//.", "subZetaReduce"}]}], "Input",
 CellChangeTimes->{{3.5645035148170757`*^9, 3.564503561461813*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"72", " ", 
    SuperscriptBox["g", "8"], " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "2", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox["u", "5"]], "-", 
  FractionBox[
   RowBox[{"144", " ", 
    SuperscriptBox["g", "8"], " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "3", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox["u", "4"]], "+", 
  FractionBox[
   RowBox[{"72", " ", 
    SuperscriptBox["g", "8"], " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "4", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox["u", "3"]]}]], "Output",
 CellChangeTimes->{{3.564503525858914*^9, 3.564503561755886*^9}}],

Cell[BoxData[
 RowBox[{"576", " ", 
  SuperscriptBox["g", "8"], " ", 
  SubsuperscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], "2", "3"]}]], "Output",
 CellChangeTimes->{{3.564503525858914*^9, 3.564503561821768*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"H", "\[Equal]", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
     FractionBox[
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.564490520676508*^9, 3.5644905384931507`*^9}, {
  3.5644957643967457`*^9, 3.5644957668902884`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "a"]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{"dz", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   OverscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    StyleBox["^",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]]], "a", 
   RowBox[{"[", "2", "]"}]], "\[Equal]", 
  RowBox[{
   FractionBox[
    SuperscriptBox["D", "2"], 
    RowBox[{"1", "-", 
     SuperscriptBox["D", "2"]}]], 
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "a"]]}], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{
    FractionBox["dz", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", "z"], "-", "1"}]], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "z", " ", "u"}]], 
    RowBox[{"(", 
     RowBox[{"a", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", "g", " ", "z"}], "]"}], "z"]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5644906411423197`*^9, 3.5644907019413433`*^9}}],

Cell[BoxData[
 RowBox[{"H", "\[Equal]", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dz", "1"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       SubscriptBox["dz", "2"], 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "u", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"18", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]]}], ")"}]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5645042585299263`*^9, 3.564504265568491*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox["\[Integral]", 
    RowBox[{
     RowBox[{"-", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
    RowBox[{
     RowBox[{"+", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
   RowBox[{
    FractionBox["du", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    RowBox[{"(", 
     FractionBox[
      RowBox[{"2", "u"}], 
      RowBox[{
       RowBox[{"4", 
        SuperscriptBox["g", "2"], 
        SuperscriptBox["z", "2"]}], "-", 
       SuperscriptBox["u", "2"]}]], ")"}], "H"}]}], "==", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", 
    RowBox[{
     RowBox[{"-", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
    RowBox[{
     RowBox[{"+", "\[Infinity]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
   RowBox[{
    FractionBox["du", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"u", "-", 
         RowBox[{"2", " ", "g", " ", "z"}]}]]}], "-", 
      FractionBox["1", 
       RowBox[{"u", "+", 
        RowBox[{"2", " ", "g", " ", "z"}]}]]}], ")"}], "H"}]}], "\[Equal]", 
  "\[IndentingNewLine]", "==", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      SubscriptBox["dz", "1"], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       SubscriptBox["dz", "2"], 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"18", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]]}], ")"}]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5644958922948103`*^9, 3.564495951160033*^9}, 
   3.564504290254179*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rhsYoYtran", "=", 
   RowBox[{"36", 
    SuperscriptBox["g", "2"], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"18", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.564495985516262*^9, 3.564496022439354*^9}, 
   3.564504295749679*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"36", 
        SuperscriptBox["g", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]], "+", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"18", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "2"]}]], "+", 
          FractionBox[
           RowBox[{"8", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "2"]}]], "+", 
          FractionBox[
           RowBox[{"8", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "2"]}]]}], ")"}]}], "/.", 
       RowBox[{
        SubscriptBox["J", "a_"], "\[RuleDelayed]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"BesselJ", "[", 
           RowBox[{"a", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], ",", "g", ",", 
    RowBox[{
     RowBox[{"Integrate", "[", 
      RowBox[{"#", ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}], "&"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{"%", " ", 
         SubsuperscriptBox["z", "1", "50"]}], ",", 
        SubscriptBox["z", "1"]}], "]"}], "/.", 
      RowBox[{
       SubsuperscriptBox["z", "1", "a_"], "\[RuleDelayed]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]$", "[", 
           RowBox[{"a", "+", "1", "-", "50"}], "]"}], "[", "0", "]"}], 
         RowBox[{"Gamma", "[", 
          RowBox[{"a", "+", "1", "-", "50"}], "]"}]}], ")"}]}]}], "//.", 
     "subZetaReduce"}], "/.", 
    RowBox[{
     RowBox[{"Zeta", "[", "a_", "]"}], "\[RuleDelayed]", 
     RowBox[{
      RowBox[{"\[Zeta]$", "[", "a", "]"}], "[", "0", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"g", ",", "\[Zeta]list", ",", "z"}], "}"}], "//", "Flatten"}], 
   ",", "Simplify"}], "]"}]}], "Input",
 CellChangeTimes->{{3.564496077997069*^9, 3.5644960933916607`*^9}, 
   3.564504301236412*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"576", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["145152", "25"]}], "-", 
     FractionBox[
      RowBox[{"82944", " ", 
       SuperscriptBox["z", "2"]}], "25"]}], ")"}], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "4"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1631232", "35"], "+", 
     FractionBox[
      RowBox[{"221184", " ", 
       SuperscriptBox["z", "2"]}], "5"], "+", 
     FractionBox[
      RowBox[{"110592", " ", 
       SuperscriptBox["z", "4"]}], "7"]}], ")"}], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "5"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["2171575296", "6125"]}], "-", 
     FractionBox[
      RowBox[{"2509092864", " ", 
       SuperscriptBox["z", "2"]}], "6125"], "-", 
     FractionBox[
      RowBox[{"1732165632", " ", 
       SuperscriptBox["z", "4"]}], "6125"], "-", 
     FractionBox[
      RowBox[{"407568384", " ", 
       SuperscriptBox["z", "6"]}], "6125"]}], ")"}], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "6"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "16"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["5152647168", "1925"], "+", 
     FractionBox[
      RowBox[{"6435127296", " ", 
       SuperscriptBox["z", "2"]}], "1925"], "+", 
     FractionBox[
      RowBox[{"873234432", " ", 
       SuperscriptBox["z", "4"]}], "275"], "+", 
     FractionBox[
      RowBox[{"431751168", " ", 
       SuperscriptBox["z", "6"]}], "275"], "+", 
     FractionBox[
      RowBox[{"14155776", " ", 
       SuperscriptBox["z", "8"]}], "55"]}], ")"}], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "7"]}]}]], "Output",
 CellChangeTimes->{3.5645043915017776`*^9, 3.5671497010172873`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tmp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           FractionBox["1", 
            RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], 
          RowBox[{"Collect", "[", 
           RowBox[{
            RowBox[{"Series", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 FractionBox[
                  RowBox[{"2", "u"}], 
                  RowBox[{
                   RowBox[{"4", 
                    SuperscriptBox["g", "2"], 
                    SuperscriptBox["z", "2"]}], "-", 
                   SuperscriptBox["u", "2"]}]], ")"}], "H"}], "/.", 
               RowBox[{"leadweak", "[", "2", "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}], ",", "g"}],
            "]"}]}], "/.", "subtousu"}], "/.", "subQ0"}], "//", 
       "funapartusu"}], "//", "funreduceusu"}], ",", "canlist"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", "\"\<hey\>\"", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp2", "=", 
   RowBox[{"UpIntegrate", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tmp", "/.", 
        RowBox[{"Shift", "[", 
         RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"u", "-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}], "2"]}]}]}], "/.", 
      "subusu"}], ",", "u"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.56449217402029*^9, 3.564492186253193*^9}, {
   3.564492420352888*^9, 3.564492570235972*^9}, 3.5644961431588087`*^9, {
   3.5644963341158133`*^9, 3.5644963368816223`*^9}, {3.564504315026647*^9, 
   3.564504315329459*^9}, {3.564504356459319*^9, 3.564504356649225*^9}}],

Cell[BoxData["\<\"hey\"\>"], "Print",
 CellChangeTimes->{3.564503419648904*^9, 3.564504316317532*^9, 
  3.5645043627862597`*^9, 3.567149707222723*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Coefficient", "[", 
      RowBox[{"#", ",", "tran", ",", 
       RowBox[{"Exponent", "[", 
        RowBox[{"#", ",", "tran"}], "]"}]}], "]"}], "&"}], "[", 
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"tmp2", "/.", "EndSimplify"}], "//.", "subZetaReduce"}], ",", 
       "canlist"}], "]"}], "/.", "subtotran"}], "]"}], ",", "canlist"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5644925782077723`*^9, 3.564492684275065*^9}, 
   3.564496251440292*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"576", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["145152", "25"]}], "-", 
     FractionBox[
      RowBox[{"82944", " ", 
       SuperscriptBox["z", "2"]}], "25"]}], ")"}], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "4"]}]}]], "Output",
 CellChangeTimes->{{3.56449622732358*^9, 3.5644962541464252`*^9}, 
   3.564496341668158*^9, 3.5645034216205482`*^9, 3.5645043199457817`*^9, 
   3.564504387895524*^9, 3.56714971521275*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rhsYoYtran", "//", "FindTranOrder"}]], "Input",
 CellChangeTimes->{{3.5645728821894093`*^9, 3.564572891508934*^9}}],

Cell[BoxData["\<\"tran order = -2\"\>"], "Output",
 CellChangeTimes->{3.5645728918224697`*^9, 3.5671497153587112`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "2"], "-", 
          SubsuperscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
           RowBox[{"[", "2", "]"}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2"], "-", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], ")"}]}], 
       RowBox[{
        SuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], "-"], " ", 
        SuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]}]], ")"}], "r"], "]"}],
    "==", 
   RowBox[{"\[Delta]\[Rho]2contrtran", "+", "W1contrtran"}]}]],
  CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
    3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
    3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
    3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
    3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
    3.5644070733510857`*^9, 3.56440708351433*^9}, 3.564464488007196*^9, 
    3.564504436271771*^9}],
 "+",
 "\[Delta]W0tran"
}], "Subsubsection",
 CellChangeTimes->{
  3.564509217500421*^9, {3.5645094885017223`*^9, 3.564509590043518*^9}, {
   3.564574522275008*^9, 3.564574526014905*^9}, {3.564575131390028*^9, 
   3.564575131798626*^9}, {3.5648340288017273`*^9, 3.564834033749136*^9}, {
   3.564836103919668*^9, 3.564836104636978*^9}}],

Cell["\<\
Leading tran asymptotic value of this quantity was acutally established\
\>", "Text",
 CellChangeTimes->{{3.564509226733127*^9, 3.5645092563385353`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    OverscriptBox[
     StyleBox["\[GothicCapitalT]",
      FontColor->RGBColor[0.54, 0.35, 0.15]], 
     StyleBox["^",
      FontColor->RGBColor[0.54, 0.35, 0.15]]], "-"], "\[TildeEqual]", 
   RowBox[{"-", 
    FractionBox["\[ImaginaryI]", 
     RowBox[{"3", " ", "g", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}]]}]}], ",", 
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[GothicCapitalT]",
      FontColor->RGBColor[0.54, 0.35, 0.15]], 
     StyleBox["^",
      FontColor->RGBColor[0.54, 0.35, 0.15]]], "\[FivePointedStar]", "-"], 
   "\[TildeEqual]", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}], 
     RowBox[{"3", " ", "g"}]]}]}], ",", 
  RowBox[{
   SuperscriptBox[
    OverscriptBox[
     StyleBox["\[GothicCapitalT]",
      FontColor->RGBColor[0.54, 0.35, 0.15]], 
     StyleBox["^",
      FontColor->RGBColor[0.54, 0.35, 0.15]]], "+"], "\[TildeEqual]", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]]}], 
    RowBox[{"3", " ", "g", " "}]]}], ",", 
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[GothicCapitalT]",
      FontColor->RGBColor[0.54, 0.35, 0.15]], 
     StyleBox["^",
      FontColor->RGBColor[0.54, 0.35, 0.15]]], "\[FivePointedStar]", "+"], 
   "\[TildeEqual]", 
   FractionBox["\[ImaginaryI]", 
    RowBox[{"3", " ", "g", " ", 
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]]}]]}]}]], "Input"],

Cell["\<\
We now should look for corrections. We begin with \[Delta]\[Rho]2 correction\
\>", "Text",
 CellChangeTimes->{{3.564509274233897*^9, 3.5645093043679867`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Delta]q", "2"], "\[Equal]", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", "g"}], 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "1"}], "1"], 
    RowBox[{"dw", " ", 
     FractionBox[
      RowBox[{
       SubscriptBox["\[Delta]\[Rho]", "2"], "[", "w", "]"}], 
      RowBox[{"u", "-", "w"}]]}]}]}], "\[Equal]", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "+", 
   RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], " ", 
   RowBox[{"(*", 
    SuperscriptBox["tran", 
     RowBox[{"-", "1"}]], "*)"}]}]}]], "Input",
 CellChangeTimes->{{3.564509306016109*^9, 3.5645093710517406`*^9}, {
  3.564509439712021*^9, 3.564509444532239*^9}, {3.564573067891477*^9, 
  3.564573071283534*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Delta]q", "2", 
   RowBox[{"[", "2", "]"}]], "==", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", "g"}], 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "1"}], "1"], 
    RowBox[{"dw", " ", 
     FractionBox[
      RowBox[{
       SubscriptBox["\[Delta]\[Rho]", "2"], "[", "w", "]"}], 
      RowBox[{
       RowBox[{"2", "g", " ", 
        RowBox[{"(", 
         RowBox[{"z", "-", "w"}], ")"}]}], "+", "\[ImaginaryI]"}]]}]}]}], 
  "\[Equal]", 
  RowBox[{"g", " ", 
   RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], "  ", 
   RowBox[{"(*", 
    SuperscriptBox["tran", 
     RowBox[{"-", "2"}]], 
    StyleBox["*)", "Text"]}]}]}]], "Input",
 CellChangeTimes->{{3.564509379280222*^9, 3.564509437474392*^9}, {
  3.5645096007294283`*^9, 3.564509632252486*^9}, {3.564573081352612*^9, 
  3.564573084439254*^9}}],

Cell["\<\
Correction for \[Delta]\[Rho]2 is easy to compute then from\
\>", "Text",
 CellChangeTimes->{{3.564509634843781*^9, 3.564509644242784*^9}, {
  3.5645745360506573`*^9, 3.5645745363995247`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
          RowBox[{"[", "2", "]"}]], "-", 
         SubsuperscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
          RowBox[{"[", 
           RowBox[{"+", "o"}], "]"}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
           RowBox[{"[", 
            RowBox[{"-", "o"}], "]"}]]}], "+", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           StyleBox["_",
            FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
          RowBox[{"[", "2", "]"}]], "+", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "_"], "2", 
          RowBox[{"[", 
           RowBox[{"-", "o"}], "]"}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
          RowBox[{"[", 
           RowBox[{"+", "o"}], "]"}]], "+", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "_"], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], ")"}]}]], ")"}], "r"], "]"}], 
  "\[TildeEqual]", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "+", 
        RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}]}], "+", 
     RowBox[{"g", " ", 
      RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " "}]}], 
    FractionBox["\[ImaginaryI]", 
     RowBox[{"3", " ", "g", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}]]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "-", 
        RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}]}], "+", 
     RowBox[{"g", " ", 
      RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " "}]}], 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], 
      RowBox[{"3", " ", "g", " "}]]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "-", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}], "+", 
     RowBox[{"g", " ", 
      RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " "}]}], 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}], 
     RowBox[{"3", " ", "g", " "}]]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "+", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}], "+", 
     RowBox[{"g", " ", 
      RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " "}]}], 
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", 
      RowBox[{"3", " ", "g", " ", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}]]}]]}], "\[Equal]", 
  RowBox[{"-", 
   FractionBox[
    RowBox[{"6", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "2"], " ", 
     RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}]}], 
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]]]}]}]], "Input",
 CellChangeTimes->{{3.564509651337171*^9, 3.564509652380074*^9}, {
  3.564509702851769*^9, 3.564509713281142*^9}, {3.564509768557516*^9, 
  3.5645097800868273`*^9}, {3.5645098109497538`*^9, 3.56450989198659*^9}, {
  3.564510564032597*^9, 3.564510596999476*^9}, {3.564573096864695*^9, 
  3.564573112862549*^9}}],

Cell[TextData[{
 "So, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " order is zero. It is in a sence good. We will try now next tran order.  I \
checked below that leading correction from W-s contribute to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]]],
 " only. So we are free to ignore them in next tran order from \
\[Delta]\[Rho]2."
}], "Text",
 CellChangeTimes->{{3.56457298108475*^9, 3.564573045944244*^9}, {
  3.5645731375188513`*^9, 3.564573137773672*^9}, {3.564573299310608*^9, 
  3.564573327437572*^9}, {3.564573732303043*^9, 3.564573775116962*^9}}],

Cell["\<\
We have to expand asymptotic solution at two leading transcendentalities. \
Asymptotic means q-functions\
\>", "Text",
 CellChangeTimes->{{3.5645738101734123`*^9, 3.564573852217759*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"tmp1", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"ee", " ", 
                  SubsuperscriptBox[
                   StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
                   RowBox[{"[", "2", "]"}]]}], "-", 
                 SubsuperscriptBox[
                  StyleBox["\[GothicQ]",
                   FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
                  RowBox[{"[", 
                   RowBox[{"+", "o"}], "]"}]]}], ")"}], ",", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", 
                  SubsuperscriptBox[
                   OverscriptBox[
                    StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], 
                    StyleBox["_",
                    FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
                   RowBox[{"[", 
                    RowBox[{"-", "o"}], "]"}]]}], "+", 
                 RowBox[{"ee", 
                  SubsuperscriptBox[
                   OverscriptBox[
                    StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], 
                    StyleBox["_",
                    FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
                   RowBox[{"[", 
                    RowBox[{"-", "2"}], "]"}]]}]}], ")"}], ",", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"ee", " ", 
                  SubsuperscriptBox[
                   StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
                   RowBox[{"[", "2", "]"}]]}], "+", 
                 SubsuperscriptBox[
                  OverscriptBox[
                   StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "_"], "2", 
                  RowBox[{"[", 
                   RowBox[{"-", "o"}], "]"}]]}], ")"}], ",", 
               RowBox[{"(", 
                RowBox[{
                 SubsuperscriptBox[
                  StyleBox["\[GothicQ]",
                   FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
                  RowBox[{"[", 
                   RowBox[{"+", "o"}], "]"}]], "+", 
                 RowBox[{"ee", " ", 
                  SubsuperscriptBox[
                   OverscriptBox[
                    StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "_"], "2", 
                   RowBox[{"[", 
                    RowBox[{"-", "2"}], "]"}]]}]}], ")"}]}], "}"}], "/.", 
             RowBox[{"q$u", "\[Rule]", "q$"}]}], "/.", 
            RowBox[{"q$ub", "\[Rule]", "q$b"}]}], "/.", "largevolume"}], "/.", 
          RowBox[{"leadweak", "[", "2", "]"}]}], "/.", 
         RowBox[{
          RowBox[{"x$", "[", "2", "]"}], "\[Rule]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"x$h", "[", "2", "]"}], "/.", "subJuk"}], "/.", 
            RowBox[{"u", "\[Rule]", 
             RowBox[{"2", "g", " ", "z"}]}]}], ")"}]}]}], "/.", 
        RowBox[{
         RowBox[{"x$", "[", 
          RowBox[{"-", "2"}], "]"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"1", "/", 
             RowBox[{"x$h", "[", 
              RowBox[{"-", "2"}], "]"}]}], "/.", "subJuk"}], "/.", 
           RowBox[{"u", "\[Rule]", 
            RowBox[{"2", "g", " ", "z"}]}]}], ")"}]}]}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "0"}], "}"}]}], "]"}], "/.", 
     RowBox[{"o", "\[Rule]", "0"}]}], "/.", 
    RowBox[{"ee", "\[Rule]", "1"}]}], "//", 
   "Normal"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp2", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "+", 
         RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}]}], "+", 
      RowBox[{"g", " ", 
       RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}]}], " ", ",", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "-", 
         RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}]}], "+", 
      RowBox[{"g", " ", 
       RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}]}], " ", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "-", 
        RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}], "+", " ", 
      RowBox[{"g", " ", 
       RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}]}], " ", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "+", 
        RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ")"}], "+", 
      RowBox[{"g", " ", 
       RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}]}]}], " ", "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.564573865744905*^9, 3.564574162512042*^9}, {
  3.564574395477207*^9, 3.564574400149111*^9}, {3.564574469463154*^9, 
  3.564574469839242*^9}, {3.564836992623855*^9, 3.564836997504262*^9}, {
  3.564837031698421*^9, 3.5648370368048487`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["2", "3"], "+", 
    FractionBox["\[ImaginaryI]", 
     RowBox[{"3", " ", "g", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}]]}], ",", 
   RowBox[{
    FractionBox["2", "3"], "-", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}], 
     RowBox[{"3", " ", "g"}]]}], ",", 
   RowBox[{
    FractionBox["2", "3"], "+", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}], 
     RowBox[{"3", " ", "g"}]]}], ",", 
   RowBox[{
    FractionBox["2", "3"], "-", 
    FractionBox["\[ImaginaryI]", 
     RowBox[{"3", " ", "g", " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}]]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.564574395788021*^9, 3.56457440049856*^9}, 
   3.564574470281797*^9, 3.564601905546753*^9, 3.564832625357436*^9, 
   3.564836086268832*^9, 3.5648370373459387`*^9, 3.565347292865417*^9, 
   3.5653521195989237`*^9, 3.567149839135379*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]\[Rho]2contrtran", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp2", "[", 
             RowBox[{"[", "1", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "1", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp2", "[", 
             RowBox[{"[", "2", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "2", "]"}], "]"}]]}], ")"}]}], 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp2", "[", 
             RowBox[{"[", "3", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "3", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp2", "[", 
             RowBox[{"[", "4", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "4", "]"}], "]"}]]}], ")"}]}]], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "2"}], "}"}]}], "]"}], "//", "Simplify"}], 
   "//", "Normal"}]}]], "Input",
 CellChangeTimes->{{3.5645742370444613`*^9, 3.564574260937447*^9}, {
  3.564574295454359*^9, 3.5645743361892767`*^9}, {3.56457445665313*^9, 
  3.5645744631946363`*^9}, {3.564574502039503*^9, 3.5645745101275873`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"6", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[ImaginaryI]", " ", 
       RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " ", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}], "+", 
      RowBox[{"\[Delta]\[Rho]2", " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}]}], ")"}]}], 
   SuperscriptBox[
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]], "2"]]}]], "Output",
 CellChangeTimes->{{3.564574312443399*^9, 3.564574339137362*^9}, 
   3.564574402260476*^9, {3.564574464071541*^9, 3.5645744714681187`*^9}, {
   3.5645745044718523`*^9, 3.564574510732177*^9}, 3.564601910529217*^9, 
   3.564832628079239*^9, 3.564836089410769*^9, 3.564836974721382*^9, 
   3.564837038669722*^9, 3.565347304411216*^9, 3.567149844653442*^9}]
}, Open  ]],

Cell["We can compare this with explicit answer we got previously.", "Text",
 CellChangeTimes->{{3.5645745452499247`*^9, 3.564574553535595*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Delta]\[Rho]2contrtran", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], "\[Rule]", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "7"], 
        RowBox[{"M", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", "9"], 
        RowBox[{"M", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", "11"], 
        RowBox[{"M", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}]}]}]}], ",", 
     RowBox[{
      RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], "\[Rule]", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "7"], 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", "9"], 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", "11"], 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}]}]}]}], ",", 
     RowBox[{"\[Delta]\[Rho]2", "->", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "7"], " ", 
        SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "+", 
       RowBox[{
        SuperscriptBox["g", "9"], " ", 
        SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
       RowBox[{
        SuperscriptBox["g", "11"], 
        SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"mytran", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"%", "/.", "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"2", "g", " ", "z"}]}]}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "13"}], "}"}]}], "]"}], "//.", "denz"}], "//",
     "Simplify"}], "//", "Normal"}]}]}], "Input",
 CellChangeTimes->{{3.56457462427308*^9, 3.564574755700556*^9}, {
  3.564574813939878*^9, 3.564574819946159*^9}, {3.5653473247312613`*^9, 
  3.565347367917945*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"12", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"M", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "-", 
     RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"PV", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["z", "2"]}]}], ")"}], " ", 
      SubscriptBox["\[Delta]\[Rho]2", "lead"]}]}], ")"}]}], "+", 
  RowBox[{"12", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"M", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "-", 
     RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"PV", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["z", "2"]}]}], ")"}], " ", 
      SubscriptBox["\[Delta]\[Rho]2", "sublead"]}]}], ")"}]}], "+", 
  RowBox[{"12", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"M", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}]}], "-", 
     RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"PV", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["z", "2"]}]}], ")"}], " ", 
      SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.5645746551034193`*^9, {3.564574694212682*^9, 3.5645747562114964`*^9}, {
   3.564574815881308*^9, 3.564574820442335*^9}, 3.564601916818069*^9, 
   3.56483263126227*^9, 3.564836091950077*^9, 3.565347313345319*^9, 
   3.565347368466032*^9, 3.567149886134748*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"mytran", "/.", "sub\[Rho]Ansatz"}], "/.", 
    "solut\[Rho]sexplicit"}], "//", "Expand"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{"%", "//", "FindHighTranCoef"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], ",", "g"}], "}"}], ",", "Simplify"}], 
    "]"}], ",", 
   RowBox[{"%", "//", "FindTranOrder"}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.5645748222819433`*^9, 3.5645748612782907`*^9}, {
  3.5653474119056463`*^9, 3.565347468426399*^9}, {3.565347630241747*^9, 
  3.565347661966722*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2016", " ", 
       SuperscriptBox["g", "9"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "-", 
      RowBox[{"1728", " ", 
       SuperscriptBox["g", "11"], " ", 
       RowBox[{"(", 
        RowBox[{"33", "+", 
         RowBox[{"8", " ", 
          SuperscriptBox["z", "2"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], "+", 
      RowBox[{"2112", " ", 
       SuperscriptBox["g", "13"], " ", 
       RowBox[{"(", 
        RowBox[{"529", "+", 
         RowBox[{"240", " ", 
          SuperscriptBox["z", "2"]}], "+", 
         RowBox[{"40", " ", 
          SuperscriptBox["z", "4"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}], 
   ",", "\<\"tran order = -2\"\>"}], "}"}]], "Output",
 CellChangeTimes->{{3.564574827777246*^9, 3.564574862238173*^9}, 
   3.564601919109198*^9, 3.5648326339616137`*^9, 3.5648360945387917`*^9, 
   3.5653473146546288`*^9, 3.565347371931584*^9, {3.5653474212705584`*^9, 
   3.56534742508042*^9}, {3.565347461263864*^9, 3.565347469073978*^9}, 
   3.5653475890353327`*^9, {3.5653476271061783`*^9, 3.5653476624979143`*^9}, 
   3.567149894007854*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"original", " ", "weak", " ", "coupling", " ", "computation"}], 
   " ", "*)"}], 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "9"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"12", " ", 
           SqrtBox[
            RowBox[{"1", "-", 
             SuperscriptBox["z", "2"]}]], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"M", "[", 
              SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}], "-", 
             RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}]}], ")"}]}], 
          "+", 
          RowBox[{"12", " ", 
           RowBox[{"(", 
            RowBox[{"1", "-", 
             RowBox[{"2", " ", 
              SuperscriptBox["z", "2"]}]}], ")"}], " ", 
           SubscriptBox["\[Delta]\[Rho]2", "lead"]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", "11"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"12", " ", "\[ImaginaryI]", " ", 
           SqrtBox[
            RowBox[{"1", "-", 
             SuperscriptBox["z", "2"]}]], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"24", " ", "\[ImaginaryI]", " ", 
              SuperscriptBox["z", "2"], " ", 
              RowBox[{"M", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
             RowBox[{"4", " ", "\[ImaginaryI]", " ", 
              RowBox[{"M", "[", 
               RowBox[{
                SuperscriptBox["w", "2"], " ", 
                SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "]"}]}], "-", 
             RowBox[{"\[ImaginaryI]", " ", 
              RowBox[{"M", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "+", 
             RowBox[{"14", " ", "z", " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
             RowBox[{"24", " ", 
              SuperscriptBox["z", "3"], " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "-", 
             RowBox[{"4", " ", "z", " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}]}], ")"}]}],
           "+", 
          RowBox[{"6", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "11"}], "+", 
                RowBox[{"2", " ", 
                 SuperscriptBox["z", "2"]}], "+", 
                RowBox[{"24", " ", 
                 SuperscriptBox["z", "4"]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "+", 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{"1", "-", 
                RowBox[{"2", " ", 
                 SuperscriptBox["z", "2"]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]\[Rho]2", "sublead"]}]}], ")"}]}]}], 
         ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", "13"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"12", " ", 
           SqrtBox[
            RowBox[{"1", "-", 
             SuperscriptBox["z", "2"]}]], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"4", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "4"}], "+", 
                RowBox[{"39", " ", 
                 SuperscriptBox["z", "2"]}], "+", 
                RowBox[{"36", " ", 
                 SuperscriptBox["z", "4"]}]}], ")"}], " ", 
              RowBox[{"M", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
             RowBox[{"240", " ", 
              SuperscriptBox["z", "2"], " ", 
              RowBox[{"M", "[", 
               RowBox[{
                SuperscriptBox["w", "2"], " ", 
                SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "]"}]}], "+", 
             RowBox[{"16", " ", 
              RowBox[{"M", "[", 
               RowBox[{
                SuperscriptBox["w", "4"], " ", 
                SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "]"}]}], "-", 
             RowBox[{"24", " ", 
              SuperscriptBox["z", "2"], " ", 
              RowBox[{"M", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "-", 
             RowBox[{"4", " ", 
              RowBox[{"M", "[", 
               RowBox[{
                SuperscriptBox["w", "2"], " ", 
                SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "]"}]}], "+", 
             RowBox[{"M", "[", 
              SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}], "+", 
             RowBox[{"64", " ", "\[ImaginaryI]", " ", "z", " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "-", 
             RowBox[{"688", " ", "\[ImaginaryI]", " ", 
              SuperscriptBox["z", "3"], " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
             RowBox[{"288", " ", "\[ImaginaryI]", " ", 
              SuperscriptBox["z", "5"], " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
             RowBox[{"14", " ", "\[ImaginaryI]", " ", "z", " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "+", 
             RowBox[{"24", " ", "\[ImaginaryI]", " ", 
              SuperscriptBox["z", "3"], " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "-", 
             RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
              RowBox[{"PV", "[", 
               SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}]}]}], 
            ")"}]}], "+", 
          RowBox[{"6", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{"50", "+", 
                RowBox[{"376", " ", 
                 SuperscriptBox["z", "2"]}], "-", 
                RowBox[{"832", " ", 
                 SuperscriptBox["z", "4"]}], "+", 
                RowBox[{"288", " ", 
                 SuperscriptBox["z", "6"]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "11"}], "+", 
                RowBox[{"2", " ", 
                 SuperscriptBox["z", "2"]}], "+", 
                RowBox[{"24", " ", 
                 SuperscriptBox["z", "4"]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{"1", "-", 
                RowBox[{"2", " ", 
                 SuperscriptBox["z", "2"]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}], 
         ")"}]}]}], ")"}], "//", "Simplify"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"%", "/.", "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], 
     "//", "Expand"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{"%", "//", "FindHighTranCoef"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SqrtBox[
          RowBox[{"1", "-", 
           SuperscriptBox["z", "2"]}]], ",", "g"}], "}"}], ",", "Simplify"}], 
      "]"}], ",", 
     RowBox[{"%", "//", "FindTranOrder"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.564574760719573*^9, 3.564574765274892*^9}, {
  3.5645748509858217`*^9, 3.564574892668827*^9}, {3.564575566151637*^9, 
  3.56457556778901*^9}, {3.565347736071068*^9, 3.565347736530532*^9}, {
  3.565347773129794*^9, 3.5653477912522173`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2016", " ", 
       SuperscriptBox["g", "9"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "-", 
      RowBox[{"1728", " ", 
       SuperscriptBox["g", "11"], " ", 
       RowBox[{"(", 
        RowBox[{"33", "+", 
         RowBox[{"8", " ", 
          SuperscriptBox["z", "2"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], "+", 
      RowBox[{"2112", " ", 
       SuperscriptBox["g", "13"], " ", 
       RowBox[{"(", 
        RowBox[{"529", "+", 
         RowBox[{"240", " ", 
          SuperscriptBox["z", "2"]}], "+", 
         RowBox[{"40", " ", 
          SuperscriptBox["z", "4"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}], 
   ",", "\<\"tran order = -2\"\>"}], "}"}]], "Output",
 CellChangeTimes->{
  3.564574765817028*^9, {3.564574868757566*^9, 3.564574893001253*^9}, 
   3.564601923287665*^9, 3.5648326369860363`*^9, 3.5648360965208607`*^9, 
   3.565347742960137*^9, {3.565347787316841*^9, 3.5653477919762506`*^9}, 
   3.5671498986192503`*^9}]
}, Open  ]],

Cell["So, it is exactly the same.", "Text",
 CellChangeTimes->{{3.565347804562175*^9, 3.5653478102388906`*^9}}],

Cell["\<\
We see that leading transcendentality was captured correctly (we checked \
leading two orders).\
\>", "Text",
 CellChangeTimes->{{3.564574894906961*^9, 3.5645749137300167`*^9}, {
  3.564601930304411*^9, 3.564601931136634*^9}}],

Cell[TextData[{
 "Let us see ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["W", "1", "+"], TraditionalForm]]],
 ", how it contributes"
}], "Text",
 CellChangeTimes->{{3.564465643892284*^9, 3.5644656521319647`*^9}, {
  3.56460193942479*^9, 3.564601941876545*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     SubsuperscriptBox["W", "1", "+"], "/.", "subWq"}], "/.", "subq34"}], "/.",
    "leadingtran"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}], "\[Rule]", "0"}], 
     "}"}]}], "//", "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "4", "]"}], "\[RuleDelayed]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Eta]$h", "[", "a", "]"}], "[", "2", "]"}], "-", 
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"x$h", "[", "2", "]"}], "a"]]}], ")"}]}]}], "//", 
   "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{"%", "/.", 
      RowBox[{
       RowBox[{"x$h", "[", 
        RowBox[{"r_", "/;", 
         RowBox[{"r", "\[NotEqual]", "0"}]}], "]"}], "\[RuleDelayed]", 
       RowBox[{"(", 
        FractionBox[
         RowBox[{"v00", "+", 
          RowBox[{"I", " ", 
           FractionBox["r", "2"]}]}], "g"], ")"}]}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"W1pcontr", "=", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{
     RowBox[{"x$h", "[", "0", "]"}], "\[Rule]", 
     FractionBox["1", 
      RowBox[{"x$", "[", "0", "]"}]]}]}], "//", "Simplify"}]}]}], "Input",
 CellChangeTimes->{{3.564465652847391*^9, 3.564465763202726*^9}, {
  3.5644843104298153`*^9, 3.5644843482159758`*^9}, {3.56448439748342*^9, 
  3.56448447311189*^9}, {3.5644845079046307`*^9, 3.564484531047647*^9}, {
  3.564573580226115*^9, 3.564573593939554*^9}, {3.5645736331724253`*^9, 
  3.5645736336113853`*^9}}],

Cell[BoxData[
 RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "-", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.5644843021291447`*^9, 3.564484357982725*^9, 3.564484397786927*^9, {
   3.564484439099112*^9, 3.564484473687153*^9}, {3.56448450618578*^9, 
   3.5644845328932056`*^9}, 3.5644845977686663`*^9, {3.564573588292954*^9, 
   3.5645735943681498`*^9}, 3.5645736365182037`*^9, 3.564601950002199*^9, 
   3.564832649469762*^9, 3.565347860976281*^9, 3.5671499109059353`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"W1mcontr", "=", 
  RowBox[{"W1pcontr", "/.", "mconj"}]}]], "Input",
 CellChangeTimes->{{3.5645736374413757`*^9, 3.564573661083602*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    FractionBox[
     RowBox[{
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}]}], 
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]]], "+", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5645736468104563`*^9, 3.564573661446699*^9}, 
   3.564601955284339*^9, 3.564832654136032*^9, 3.565347864412364*^9, 
   3.567149913230157*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"W1contrtran", "=", 
  RowBox[{"36", " ", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      FractionBox["1", "36"], 
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{"Log", "[", 
         RowBox[{
          FractionBox["1", 
           RowBox[{"1", "+", 
            FractionBox["W1pcontr", 
             RowBox[{"tmp1", "[", 
              RowBox[{"[", "3", "]"}], "]"}]]}]], 
          FractionBox["1", 
           RowBox[{"1", "+", 
            FractionBox["W1mcontr", 
             RowBox[{"tmp1", "[", 
              RowBox[{"[", "4", "]"}], "]"}]]}]]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"g", ",", "0", ",", "2"}], "}"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Eta]$h", "[", "_", "]"}], "[", "2", "]"}], 
        RowBox[{
         RowBox[{"\[Eta]$bh", "[", "_", "]"}], "[", 
         RowBox[{"-", "2"}], "]"}]}]}], "}"}], ",", "Simplify"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.564574982766852*^9, 3.564574990923316*^9}, {
  3.564575036132451*^9, 3.564575117299214*^9}}],

Cell[BoxData[
 RowBox[{"36", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{"2", " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", 
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]]]}], "+", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "-", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]]], "-", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564575067338374*^9, 3.564575117958653*^9}, 
   3.564601959240844*^9, 3.564832660362982*^9, 3.564837157771872*^9, 
   3.565347865446801*^9, 3.567149914947618*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"36", " ", 
           SuperscriptBox["g", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", 
               SubsuperscriptBox[
                OverscriptBox[
                 OverscriptBox[
                  StyleBox["\[Eta]",
                   FontWeight->Bold,
                   FontColor->RGBColor[0.8, 0, 0]], 
                  StyleBox["_",
                   FontWeight->Bold,
                   FontColor->RGBColor[0.8, 0, 0]]], 
                 StyleBox["^",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]]], "4", 
                RowBox[{"[", 
                 RowBox[{"-", "2"}], "]"}]]}], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["\[Eta]",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "2", 
               RowBox[{"[", "2", "]"}]]}], "+", 
             RowBox[{"2", " ", 
              SubsuperscriptBox[
               OverscriptBox[
                OverscriptBox[
                 StyleBox["\[Eta]",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]], 
                 StyleBox["_",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "3", 
               RowBox[{"[", 
                RowBox[{"-", "2"}], "]"}]], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["\[Eta]",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "3", 
               RowBox[{"[", "2", "]"}]]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", 
                 FractionBox["1", 
                  StyleBox["x",
                   FontColor->RGBColor[0, 0.6, 0]]]}], "+", 
                StyleBox["x",
                 FontColor->RGBColor[0, 0.6, 0]]}], ")"}], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                OverscriptBox[
                 StyleBox["\[Eta]",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]], 
                 StyleBox["_",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "4", 
               RowBox[{"[", 
                RowBox[{"-", "2"}], "]"}]], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["\[Eta]",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "3", 
               RowBox[{"[", "2", "]"}]]}], "-", 
             RowBox[{
              SubsuperscriptBox[
               OverscriptBox[
                OverscriptBox[
                 StyleBox["\[Eta]",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]], 
                 StyleBox["_",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "2", 
               RowBox[{"[", 
                RowBox[{"-", "2"}], "]"}]], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["\[Eta]",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "4", 
               RowBox[{"[", "2", "]"}]]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                FractionBox["1", 
                 StyleBox["x",
                  FontColor->RGBColor[0, 0.6, 0]]], "-", 
                StyleBox["x",
                 FontColor->RGBColor[0, 0.6, 0]]}], ")"}], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                OverscriptBox[
                 StyleBox["\[Eta]",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]], 
                 StyleBox["_",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "3", 
               RowBox[{"[", 
                RowBox[{"-", "2"}], "]"}]], " ", 
              SubsuperscriptBox[
               OverscriptBox[
                StyleBox["\[Eta]",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "4", 
               RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "/.", 
          RowBox[{
           RowBox[{
            RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "2", "]"}], 
           "\[RuleDelayed]", " ", 
           RowBox[{
            SuperscriptBox["g", "a"], 
            RowBox[{
             RowBox[{"\[Eta]$", "[", "a", "]"}], "[", "2", "]"}]}]}]}], "/.", 
         
         RowBox[{
          RowBox[{
           RowBox[{"\[Eta]$bh", "[", "a_", "]"}], "[", 
           RowBox[{"-", "2"}], "]"}], "\[RuleDelayed]", 
          RowBox[{
           SuperscriptBox["g", 
            RowBox[{"a", " "}]], 
           RowBox[{
            RowBox[{"\[Eta]$b", "[", "a", "]"}], "[", 
            RowBox[{"-", "2"}], "]"}]}]}]}], "/.", "subJuk"}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"2", "g", " ", "z"}]}]}], "/.", 
      RowBox[{"TaylorOptions", "[", 
       RowBox[{
        RowBox[{"Variable", "\[Rule]", "g"}], ",", 
        RowBox[{"\[Eta]Factor", "\[Rule]", 
         RowBox[{"2", " ", "z"}]}], ",", " ", 
        RowBox[{"Order", "\[Rule]", "9"}]}], "]"}]}], "//.", "denz"}], "//.", 
    "subZetaReduce"}], ",", "zlist"}], "]"}]], "Input",
 CellChangeTimes->{{3.564575582587021*^9, 3.5645756948443127`*^9}, {
  3.564575728297538*^9, 3.564575728586454*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    FractionBox["288", "5"]}], " ", 
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"144", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2", "3"]}], "5"], "+", 
     RowBox[{"72", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.5645756081719027`*^9, 3.564575695225868*^9}, 
   3.5645757291133738`*^9, 3.5646019714795837`*^9, 3.56483266336124*^9, 
   3.565347870368248*^9, 3.5671499169876842`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"result", " ", "from", " ", "small", " ", "g", " ", "expansion"}], 
   " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             FractionBox[
              RowBox[{"36", " ", 
               SuperscriptBox["\[Pi]", "4"]}], "5"], "-", 
             FractionBox[
              RowBox[{"2", " ", 
               SuperscriptBox["\[Pi]", "6"]}], "15"], "-", 
             RowBox[{"12", " ", 
              SuperscriptBox["\[Pi]", "2"], " ", 
              RowBox[{"(", 
               RowBox[{"3", "+", 
                RowBox[{"4", " ", 
                 SubscriptBox[
                  StyleBox["\[Zeta]",
                   FontWeight->Bold,
                   FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "-", 
             RowBox[{"36", " ", 
              RowBox[{"(", 
               RowBox[{"9", "+", 
                RowBox[{"2", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "9"}], "+", 
                   SubscriptBox[
                    StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}], ")"}], " ", 
                 SubscriptBox[
                  StyleBox["\[Zeta]",
                   FontWeight->Bold,
                   FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], ")"}], 
           " ", 
           SuperscriptBox["g", "8"]}], "+", 
          RowBox[{
           FractionBox["4", "5"], " ", 
           RowBox[{"(", 
            RowBox[{"945", "-", 
             RowBox[{"105", " ", 
              SuperscriptBox["\[Pi]", "2"]}], "+", 
             SuperscriptBox["\[Pi]", "4"]}], ")"}], " ", 
           SqrtBox[
            RowBox[{"1", "-", 
             SuperscriptBox["z", "2"]}]], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "3"}], "+", 
             RowBox[{"2", " ", 
              SubscriptBox[
               StyleBox["\[Zeta]",
                FontWeight->Bold,
                FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
           SuperscriptBox["g", "9"]}], "+", 
          InterpretationBox[
           SuperscriptBox[
            RowBox[{"O", "[", "g", "]"}], "10"],
           SeriesData[$CellContext`g, 0, {}, 8, 10, 1],
           Editable->False]}],
         SeriesData[$CellContext`g, 0, {
          Rational[36, 5] Pi^4 + 
           Rational[-2, 15] Pi^6 - (12 Pi^2) (3 + 
            4 $CellContext`\[Zeta]$[3][0]) - 
           36 (9 + (
              2 (-9 + $CellContext`\[Zeta]$[3][0])) $CellContext`\[Zeta]$[3][
              0]), ((Rational[4, 5] (945 - 105 Pi^2 + Pi^4)) (
              1 - $CellContext`z^2)^Rational[1, 2]) (-3 + 
            2 $CellContext`\[Zeta]$[3][0])}, 8, 10, 1],
         Editable->False], ")"}], "//.", "subZetaReduce"}], ",", "g"}], "]"}],
     ";"}], "\[IndentingNewLine]", 
   RowBox[{"Coefficient", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "%"}], "/.", "subtotran"}], ",", "tran", ",", 
     RowBox[{"Exponent", "[", 
      RowBox[{
       RowBox[{"%", "/.", "subtotran"}], ",", "tran"}], "]"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.564575700690827*^9, 3.564575808928072*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["144", "5"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"]}], "-", 
  RowBox[{
   FractionBox["288", "5"], " ", 
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{"72", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3", "2"]}]}]], "Output",
 CellChangeTimes->{{3.5645757653787518`*^9, 3.564575809423996*^9}, 
   3.5646019752719927`*^9, 3.564832665354062*^9, 3.5653478721855783`*^9, 
   3.567149920765348*^9}]
}, Open  ]],

Cell["So we got the same at leading order.", "Text",
 CellChangeTimes->{{3.564575815144555*^9, 3.56457581967935*^9}}],

Cell["Now we consider pv correction from W.", "Text",
 CellChangeTimes->{{3.564573363813158*^9, 3.5645733727960052`*^9}, {
  3.56457371129608*^9, 3.564573715479228*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Delta]q", "2"], "[", "W", "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    SubsuperscriptBox["W", "pv", 
     RowBox[{"[", 
      RowBox[{"+", "o"}], "]"}]]}], "-", 
   RowBox[{"C", 
    SubscriptBox["*", "pv"], "W"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubsuperscriptBox["\[Delta]q", "2", 
    RowBox[{"[", "2", "]"}]], "[", "W", "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["C", 
     RowBox[{"[", "2", "]"}]]}], "*", 
   SubscriptBox["W", "pv"]}]}]}], "Input",
 CellChangeTimes->{{3.564573376525957*^9, 3.564573401669739*^9}, {
  3.564576404862105*^9, 3.564576420708066*^9}, {3.5646021187233562`*^9, 
  3.564602131267021*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"W", "/.", "subWq"}], "/.", "subq34"}], "/.", "substandardq"}], 
    "/.", "tranpowers"}], ",", 
   RowBox[{"{", 
    RowBox[{"tran", ",", "\[Infinity]", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5645734157897453`*^9, 3.564573451272725*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], "2"], " ", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "4"}], "+"]}], "-", 
         SubsuperscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          RowBox[{"2", ",", "4"}], "+"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], "2"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"1", ",", "3"}], "-"]}], "-", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           StyleBox["_",
            FontColor->RGBColor[0, 0, 0.8]]], 
          RowBox[{"1", ",", "4"}], "-"]}], ")"}]}], 
      RowBox[{
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "+"]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], "2"], " ", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "3"}], "+"]}], "-", 
         SubsuperscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          RowBox[{"1", ",", "4"}], "+"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], "2"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"1", ",", "4"}], "-"]}], "-", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           StyleBox["_",
            FontColor->RGBColor[0, 0, 0.8]]], 
          RowBox[{"2", ",", "4"}], "-"]}], ")"}]}], 
      RowBox[{
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "+"]}]]}], "tran"], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "2"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 1, 2, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[
   1], {$CellContext`Q$[-1]^(-1) $CellContext`Q$[
       1]^(-1) ($CellContext`q$[2][0] $CellContext`q$[1, 4][
        1] - $CellContext`q$[2, 4][
      1]) ($CellContext`q$b[2][0] $CellContext`q$b[
        1, 3][-1] - $CellContext`q$b[
      1, 4][-1]) + $CellContext`Q$[-1]^(-1) $CellContext`Q$[
       1]^(-1) ($CellContext`q$[2][0] $CellContext`q$[1, 3][
        1] - $CellContext`q$[1, 4][
      1]) ($CellContext`q$b[2][0] $CellContext`q$b[
        1, 4][-1] - $CellContext`q$b[2, 4][-1])}, 1, 2, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.564573427333116*^9, 3.564573451693137*^9}, 
   3.564602134983365*^9, 3.5648326829976273`*^9, 3.565347883791695*^9, 
   3.567149935492634*^9}]
}, Open  ]],

Cell[TextData[{
 "This shows that W is of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 ", and we need only its leading roder. But the order will come both from \
integration and  from principal value piece"
}], "Text",
 CellChangeTimes->{{3.5645763487419558`*^9, 3.564576368755376*^9}, {
  3.5646021387422256`*^9, 3.564602147959332*^9}, {3.5648327052732677`*^9, 
  3.564832710304349*^9}}],

Cell[TextData[{
 "Now we computing ",
 Cell[BoxData[
  SubsuperscriptBox["W", "pv", 
   RowBox[{"[", 
    RowBox[{"+", "o"}], "]"}]]],
  CellChangeTimes->{{3.564573376525957*^9, 3.564573401669739*^9}, {
   3.564576404862105*^9, 3.5645764057554197`*^9}}]
}], "Text",
 CellChangeTimes->{{3.564576373629081*^9, 3.5645764255645437`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"W", "/.", "subWq"}], "/.", "subq34"}], "/.", "leadingtran"}], "/.", 
   RowBox[{"x$h", "\[Rule]", "x$"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}], "\[Rule]", "0"}], 
     "}"}]}], "//", "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Wpvleadtran", "=", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"(", 
      RowBox[{"%", "+", 
       RowBox[{"(", 
        RowBox[{"%", "/.", 
         RowBox[{
          RowBox[{"x$", "[", "0", "]"}], "\[Rule]", 
          FractionBox["1", 
           RowBox[{"x$", "[", "0", "]"}]]}]}], ")"}]}], ")"}]}], "//", 
    "Simplify"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"6", "\[ImaginaryI]", " ", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["1", 
       RowBox[{"6", "\[ImaginaryI]"}]], "%"}], "//", "Expand"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Eta]$h", "[", "_", "]"}], "[", "2", "]"}], 
       RowBox[{
        RowBox[{"\[Eta]$bh", "[", "_", "]"}], "[", 
        RowBox[{"-", "2"}], "]"}]}]}], "}"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.564486179056424*^9, 3.5644862515821*^9}, {
  3.5644867263641777`*^9, 3.564486733174554*^9}, {3.564576728636883*^9, 
  3.56457674980861*^9}}],

Cell[BoxData[
 RowBox[{"6", " ", "\[ImaginaryI]", " ", "g", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], "2"]], "+", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", 
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]]]}], "-", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "2"]]}], "-", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{"2", " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]]], "+", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}], "-", 
    RowBox[{"2", " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564486188024022*^9, 3.5644861986262627`*^9}, 
   3.56448625198724*^9, 3.564486733640711*^9, 3.564576750584318*^9, 
   3.564602152190102*^9, 3.564832711775425*^9, 3.565347886083111*^9, 
   3.567149949176668*^9}]
}, Open  ]],

Cell["Let us verify this expression against weak coupling:", "Text",
 CellChangeTimes->{{3.56448669457798*^9, 3.564486719303176*^9}, {
  3.564576781213562*^9, 3.564576790403318*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"W", "/.", "subWq"}], "/.", "subq34"}], "/.", 
      RowBox[{
       RowBox[{
        OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}], "\[Rule]", "0"}]}],
      "/.", "largevolume"}], "/.", " ", 
    RowBox[{
     FractionBox["1", "x"], "\[Rule]", "x"}]}], "/.", 
   RowBox[{"x", "\[Rule]", 
    FractionBox["1", "x"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], 
    RowBox[{"(", 
     RowBox[{"%", "+", 
      RowBox[{"(", 
       RowBox[{"%", "/.", 
        RowBox[{
         RowBox[{"x$", "[", "0", "]"}], "\[Rule]", 
         FractionBox["1", 
          RowBox[{"x$", "[", "0", "]"}]]}]}], ")"}]}], ")"}]}], "//", 
   "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"%", "/.", 
          RowBox[{"leadweak", "[", "2", "]"}]}], "/.", "subQ0"}], "/.", 
        "subJuk"}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"2", "g", " ", "z"}]}]}], "/.", 
      RowBox[{"TaylorOptions", "[", 
       RowBox[{
        RowBox[{"Variable", "\[Rule]", "g"}], ",", 
        RowBox[{"\[Eta]Factor", "\[Rule]", 
         RowBox[{"2", "z"}]}], ",", " ", 
        RowBox[{"Order", "\[Rule]", "8"}]}], "]"}]}], "//.", "denz"}], "//", 
    "Normal"}], "//", "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"%", "//.", "subZetaReduce"}], ",", "g", ",", "Simplify"}], "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{"tmp", "=", 
  RowBox[{"Coefficient", "[", 
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{"%", "/.", "subtotran"}], ",", "tran"}], "]"}], ",", "tran", 
    ",", 
    RowBox[{"-", "1"}]}], "]"}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.5644863926035433`*^9, 3.564486518633339*^9}, {
  3.56448654958626*^9, 3.564486681066799*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"24", " ", 
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     RowBox[{"2", " ", 
      SuperscriptBox["z", "2"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{
   FractionBox["144", "5"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{
   FractionBox["576", "5"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{
   FractionBox["1152", "5"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox["z", "4"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{"72", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"240", " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"1152", " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox["z", "4"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}]}]], "Output",
 CellChangeTimes->{{3.564486396727399*^9, 3.564486519607449*^9}, {
   3.564486560125296*^9, 3.5644866825759983`*^9}, 3.564576804149988*^9, 
   3.564602173680015*^9, 3.56483272305702*^9, 3.565347897091938*^9, 
   3.5671499593375597`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Wpvleadtran", "/.", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "2", "]"}], 
        "\[RuleDelayed]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Collect", "[", 
           RowBox[{
            RowBox[{"Normal", "@", 
             RowBox[{"Series", "[", 
              RowBox[{
               RowBox[{
                FractionBox["1", 
                 SuperscriptBox[
                  RowBox[{"x$h", "[", "0", "]"}], "a"]], "/.", "subJuk"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{"g", ",", "0", ",", 
                 RowBox[{"a", "+", "2"}]}], "}"}]}], "]"}]}], ",", "u"}], 
           "]"}], "/.", 
          RowBox[{
           SuperscriptBox["u", "aa_"], "\[RuleDelayed]", 
           RowBox[{
            RowBox[{"\[Eta]$", "[", 
             RowBox[{"-", "aa"}], "]"}], "[", "2", "]"}]}]}], ")"}]}]}], "/.", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Eta]$bh", "[", "a_", "]"}], "[", 
        RowBox[{"-", "2"}], "]"}], "\[RuleDelayed]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{"Normal", "@", 
            RowBox[{"Series", "[", 
             RowBox[{
              RowBox[{
               FractionBox["1", 
                SuperscriptBox[
                 RowBox[{"x$h", "[", "0", "]"}], "a"]], "/.", "subJuk"}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", 
                RowBox[{"a", "+", "2"}]}], "}"}]}], "]"}]}], ",", "u"}], 
          "]"}], "/.", 
         RowBox[{
          SuperscriptBox["u", "aa_"], "\[RuleDelayed]", 
          RowBox[{
           RowBox[{"\[Eta]$b", "[", 
            RowBox[{"-", "aa"}], "]"}], "[", 
           RowBox[{"-", "2"}], "]"}]}]}], ")"}]}]}], " ", "/.", 
     RowBox[{"x$h", "\[Rule]", "x$"}]}], "/.", "subJuk"}], "/.", 
   RowBox[{"u", "\[Rule]", 
    RowBox[{"2", "g", " ", "z"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"mrlt", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"%", "/.", 
      RowBox[{"TaylorOptions", "[", 
       RowBox[{
        RowBox[{"Variable", "\[Rule]", "g"}], ",", 
        RowBox[{"\[Eta]Factor", "\[Rule]", 
         RowBox[{"2", "z"}]}], ",", " ", 
        RowBox[{"Order", "\[Rule]", "8"}]}], "]"}]}], "//.", "denz"}], ",", 
    "zlist"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.564486739834738*^9, 3.564486768690473*^9}, {
  3.564486803112505*^9, 3.564486942725855*^9}, {3.5644871422085247`*^9, 
  3.564487144832665*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"24", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"48", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "72"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "4"]}], "-", 
     RowBox[{"72", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{
      SuperscriptBox["z", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"288", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "4"]}], "-", 
        RowBox[{"240", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["z", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "576"}], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "4"]}], "+", 
        RowBox[{"1152", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{{3.56448693268491*^9, 3.564486943169339*^9}, {
   3.564487145710994*^9, 3.5644871571983833`*^9}, 3.564576810790255*^9, 
   3.5646021768550873`*^9, 3.56483272460721*^9, 3.56534789776618*^9, 
   3.567149960006433*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tmp", "-", "mrlt"}], "//.", "subZetaReduce"}], "//", 
  "Simplify"}]], "Input",
 CellChangeTimes->{{3.56448695335235*^9, 3.564486961128192*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.56448696518398*^9, 3.5644871479823093`*^9, 
  3.564576812062381*^9, 3.5646021789550056`*^9, 3.564832726019787*^9, 
  3.5653479001112633`*^9, 3.567149960173839*^9}]
}, Open  ]],

Cell["Now we compute the integral", "Text",
 CellChangeTimes->{{3.5645768419001017`*^9, 3.5645768455374203`*^9}, {
  3.564602184157867*^9, 3.564602187389721*^9}}],

Cell[BoxData[
 RowBox[{"C", 
  SubscriptBox["*", "pv"], "W"}]], "Input",
 CellChangeTimes->{3.564576876995408*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"W", "/.", "subWq"}], "/.", "subq34"}], "/.", "substandardq"}], 
    "/.", "tranpowers"}], ",", 
   RowBox[{"{", 
    RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Wtmp", "=", 
  RowBox[{
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Normal", "[", "%", "]"}], "/.", 
       RowBox[{"tran", "\[Rule]", "1"}]}], "/.", "leadingtran"}], ",", 
     RowBox[{"Q$", "[", "_", "]"}]}], "]"}], "//", "Factor"}]}]}], "Input",
 CellChangeTimes->{{3.564411165703979*^9, 3.564411203481657*^9}, {
  3.564576910221629*^9, 3.564576923268509*^9}, {3.564577422707808*^9, 
  3.56457742344042*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], "2"], " ", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "4"}], "+"]}], "-", 
         SubsuperscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          RowBox[{"2", ",", "4"}], "+"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], "2"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"1", ",", "3"}], "-"]}], "-", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           StyleBox["_",
            FontColor->RGBColor[0, 0, 0.8]]], 
          RowBox[{"1", ",", "4"}], "-"]}], ")"}]}], 
      RowBox[{
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "+"]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], "2"], " ", 
          SubsuperscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           RowBox[{"1", ",", "3"}], "+"]}], "-", 
         SubsuperscriptBox[
          StyleBox["q",
           FontColor->RGBColor[0, 0, 0.8]], 
          RowBox[{"1", ",", "4"}], "+"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], "2"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["q",
             FontColor->RGBColor[0, 0, 0.8]], 
            StyleBox["_",
             FontColor->RGBColor[0, 0, 0.8]]], 
           RowBox[{"1", ",", "4"}], "-"]}], "-", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["q",
            FontColor->RGBColor[0, 0, 0.8]], 
           StyleBox["_",
            FontColor->RGBColor[0, 0, 0.8]]], 
          RowBox[{"2", ",", "4"}], "-"]}], ")"}]}], 
      RowBox[{
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q",
         FontColor->RGBColor[0.8, 0, 0]], "+"]}]]}], "tran"], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "3"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 1, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[
   1], {$CellContext`Q$[-1]^(-1) $CellContext`Q$[
       1]^(-1) ($CellContext`q$[2][0] $CellContext`q$[1, 4][
        1] - $CellContext`q$[2, 4][
      1]) ($CellContext`q$b[2][0] $CellContext`q$b[
        1, 3][-1] - $CellContext`q$b[
      1, 4][-1]) + $CellContext`Q$[-1]^(-1) $CellContext`Q$[
       1]^(-1) ($CellContext`q$[2][0] $CellContext`q$[1, 3][
        1] - $CellContext`q$[1, 4][
      1]) ($CellContext`q$b[2][0] $CellContext`q$b[
        1, 4][-1] - $CellContext`q$b[2, 4][-1])}, 1, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.564411170975143*^9, 3.564411204037298*^9}, {
   3.5645769013609743`*^9, 3.564576923766129*^9}, 3.564577423982298*^9, 
   3.56459895255164*^9, 3.564598992974004*^9, 3.564832740991645*^9, 
   3.565347902516178*^9, 3.567149961625668*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]]}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]]}]], "Output",
 CellChangeTimes->{{3.564411170975143*^9, 3.564411204037298*^9}, {
   3.5645769013609743`*^9, 3.564576923766129*^9}, 3.564577423982298*^9, 
   3.56459895255164*^9, 3.564598992974004*^9, 3.564832740991645*^9, 
   3.565347902516178*^9, 3.56714996164052*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   FractionBox["du", 
    RowBox[{
     RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
   SubscriptBox[
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"v", "-", "u"}]], 
      RowBox[{"W", "[", "u", "]"}]}], ")"}], "pv"]}]}]], "Input",
 CellChangeTimes->{{3.564411265021696*^9, 3.564411295332365*^9}, {
  3.564577048671177*^9, 3.564577048926607*^9}}],

Cell["W is symmetric:", "Text",
 CellChangeTimes->{{3.564577023059819*^9, 3.564577085168489*^9}, {
  3.564577410757155*^9, 3.5645774137640057`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Wtmp", "/.", 
     RowBox[{"\[Eta]$h", "\[Rule]", "\[Eta]$"}]}], "/.", 
    RowBox[{"\[Eta]$bh", "\[Rule]", "\[Eta]$b"}]}], "/.", 
   RowBox[{
    RowBox[{"x$h", "[", "0", "]"}], "\[Rule]", "u"}]}], "//", 
  "asymfun"}]], "Input",
 CellChangeTimes->{{3.56457705939823*^9, 3.5645770767184067`*^9}, {
  3.5648327663076477`*^9, 3.564832766743162*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.564577069919297*^9, 3.564577077168188*^9}, {
   3.564832750213928*^9, 3.56483276714192*^9}, 3.567149966595933*^9}]
}, Open  ]],

Cell["So, we have to compute", "Text",
 CellChangeTimes->{{3.564577088032282*^9, 3.5645770914881783`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox["du", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"v", "-", "u"}]], "+", 
      FractionBox["1", 
       RowBox[{"v", "+", "u"}]]}], ")"}], 
    SubscriptBox[
     RowBox[{"W", "[", "u", "]"}], "pv"]}]}], "==", 
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox["du", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
    FractionBox["1", "2"], 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"W", "[", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}], 
        RowBox[{"v", "-", 
         RowBox[{"(", 
          RowBox[{"u", "+", 
           RowBox[{"\[ImaginaryI]", " ", "o"}]}], ")"}]}]], "+", 
       FractionBox[
        RowBox[{"W", "[", 
         RowBox[{"u", "-", 
          RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}], 
        RowBox[{"v", "-", 
         RowBox[{"(", 
          RowBox[{"u", "-", 
           RowBox[{"\[ImaginaryI]", " ", "o"}]}], ")"}]}]]}], ")"}], 
     "pv"]}]}], "\[Equal]", "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{"\[DoubleStruckCapitalC]", "[", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"u", "-", "v"}]], "-", 
      FractionBox["1", 
       RowBox[{"u", "+", "v"}]]}], ")"}], "W"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.564577103629076*^9, 3.5645771092727613`*^9}, {
  3.564577166939762*^9, 3.564577190607315*^9}, {3.5645772237267237`*^9, 
  3.564577242134116*^9}, {3.564577311808234*^9, 3.564577326614052*^9}, {
  3.564577755733192*^9, 3.56457776787638*^9}}],

Cell["\<\
Integration is slightly below real axis. But  does it matter? It does not, \
because of the parity of W. Anyway we keep it slightly below real axis. Now \
we should produce Bessel function parameterization for W:\
\>", "Text",
 CellChangeTimes->{{3.564577327783667*^9, 3.564577393204803*^9}, 
   3.564832778018435*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{"Wtmp", ",", 
    RowBox[{"x$h", "[", "0", "]"}], ",", 
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"#", "/.", "subtoBes2"}], ")"}], ",", 
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"\[ImaginaryI]", " ", "u", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "3"]}], ")"}]}]], 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "1"]], "-", "1"}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "3"]], "-", "1"}], ")"}]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
      "]"}], "&"}]}], "]"}], "/.", 
  RowBox[{
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{"a_", ",", "b_", ",", "c__"}], "]"}], " ", 
    SuperscriptBox[
     RowBox[{"x$h", "[", "0", "]"}], 
     RowBox[{"pow_", "/;", 
      RowBox[{"pow", "<", "0"}]}]]}], "\[RuleDelayed]", 
   RowBox[{"(", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"a", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"x$h", "[", "0", "]"}], "pow"], "/.", "subtoBes2"}], 
        ")"}]}], ",", 
      RowBox[{
       RowBox[{"b", " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", "u", " ", 
          SubscriptBox["z", "2"]}]]}], "//", "Simplify"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      "c"}], "]"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.564577429160055*^9, 3.5645774527088203`*^9}, {
  3.5645774855015593`*^9, 3.5645777171594152`*^9}, {3.564578080334569*^9, 
  3.564578081059978*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"12", " ", "\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"12", " ", "\[ImaginaryI]", " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "3"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", " ", "u", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "3"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], 
  "+", 
  RowBox[{"TC", "[", 
   RowBox[{
    FractionBox[
     RowBox[{"24", " ", "\[ImaginaryI]", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "2"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"6", " ", "\[ImaginaryI]", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "3"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "3"]}]]}], "-", 
        FractionBox[
         RowBox[{"6", " ", "\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], ")"}]}], 
     SubscriptBox["z", "2"]], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "\[ImaginaryI]"}], " ", "u", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["z", "1"]}], "+", 
         SubscriptBox["z", "2"], "+", 
         SubscriptBox["z", "3"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], 
  "+", 
  RowBox[{"TC", "[", 
   RowBox[{
    FractionBox[
     RowBox[{"12", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "1"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]], "-", 
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], ")"}]}], 
     SubscriptBox["z", "2"]], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "\[ImaginaryI]"}], " ", "u", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["z", "1"]}], "+", 
         SubscriptBox["z", "2"], "+", 
         SubscriptBox["z", "3"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
   "]"}]}]], "Output",
 CellChangeTimes->{{3.564577430348871*^9, 3.564577453134878*^9}, {
   3.564577488638789*^9, 3.5645775209693527`*^9}, {3.5645775510488863`*^9, 
   3.5645775693532887`*^9}, {3.5645776178336153`*^9, 3.564577624451355*^9}, {
   3.564577699430004*^9, 3.564577717700466*^9}, 3.564578082106338*^9, 
   3.5645990148921337`*^9, 3.564602274661619*^9, 3.564832795054281*^9, 
   3.565347905576769*^9, 3.5671499694059067`*^9}]
}, Open  ]],

Cell[TextData[{
 "Now we have to integrate  ",
 Cell[BoxData[
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", "u", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SubscriptBox["z", "1"]}], "+", 
      SubscriptBox["z", "2"], "+", 
      SubscriptBox["z", "3"]}], ")"}]}]]],
  CellChangeTimes->{{3.564577430348871*^9, 3.564577453134878*^9}, {
   3.564577488638789*^9, 3.5645775209693527`*^9}, {3.5645775510488863`*^9, 
   3.5645775693532887`*^9}, {3.5645776178336153`*^9, 3.564577624451355*^9}, {
   3.564577699430004*^9, 3.564577717700466*^9}}],
 " and ",
 Cell[BoxData[
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{"\[ImaginaryI]", " ", "u", " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["z", "1"], "-", 
      SubscriptBox["z", "3"]}], ")"}]}]]],
  CellChangeTimes->{{3.564577430348871*^9, 3.564577453134878*^9}, {
   3.564577488638789*^9, 3.5645775209693527`*^9}, {3.5645775510488863`*^9, 
   3.5645775693532887`*^9}, {3.5645776178336153`*^9, 3.564577624451355*^9}, {
   3.564577699430004*^9, 3.564577717700466*^9}}],
 ", with ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"u", "-", 
       RowBox[{"2", "g", " ", "z"}]}]], "-", 
     FractionBox["1", 
      RowBox[{"u", "+", 
       RowBox[{"2", "g", " ", "z"}]}]]}], ")"}]}]],
  CellChangeTimes->{{3.564577103629076*^9, 3.5645771092727613`*^9}, {
   3.564577166939762*^9, 3.564577190607315*^9}, {3.5645772237267237`*^9, 
   3.564577242134116*^9}, {3.564577311808234*^9, 3.564577326614052*^9}, {
   3.564577755733192*^9, 3.56457776787638*^9}}],
 ", u is slightly below real axis, and z is real. When we close above, we \
pick both poles (overall sign is plus), and if we close below we get zero. \
Condition for closing above is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], ">", 
    SubscriptBox["z", "3"]}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], ">", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["z", "2"], "+", 
      SubscriptBox["z", "3"]}], ")"}]}], TraditionalForm]]],
 ". So let us write down corresponding expressoin"
}], "Text",
 CellChangeTimes->{{3.564577739650069*^9, 3.564577908465494*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Wpvinttran", "=", 
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"12", " ", "\[ImaginaryI]", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "3"]}]]}], "-", 
         FractionBox[
          RowBox[{"12", " ", "\[ImaginaryI]", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "3"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
      FractionBox[
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]], "-", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]]}], ")"}]}], 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "3"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
    RowBox[{"TC", "[", 
     RowBox[{
      FractionBox[
       RowBox[{"24", " ", "\[ImaginaryI]", " ", "g", " ", 
        RowBox[{
         SubscriptBox["J", "2"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{"6", " ", "\[ImaginaryI]", " ", 
             RowBox[{
              SubscriptBox["J", "2"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "3"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "3"]}]]}], "-", 
          FractionBox[
           RowBox[{"6", " ", "\[ImaginaryI]", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "3"]}]]}], ")"}]}], 
       SubscriptBox["z", "2"]], ",", 
      FractionBox[
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "2"], "+", 
                SubscriptBox["z", "3"]}], ")"}]}], ")"}]}]], "-", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "2"], "+", 
                SubscriptBox["z", "3"]}], ")"}]}], ")"}]}]]}], ")"}]}], 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "3"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "3"]}]}], "}"}]}], "]"}], "+", 
    RowBox[{"TC", "[", 
     RowBox[{
      FractionBox[
       RowBox[{"12", " ", "g", " ", 
        RowBox[{
         SubscriptBox["J", "1"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"8", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "3"]}]], "-", 
          FractionBox[
           RowBox[{"8", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "3"]}]]}], ")"}]}], 
       SubscriptBox["z", "2"]], ",", 
      FractionBox[
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "2"], "+", 
                SubscriptBox["z", "3"]}], ")"}]}], ")"}]}]], "-", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", "2", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "2"], "+", 
                SubscriptBox["z", "3"]}], ")"}]}], ")"}]}]]}], ")"}]}], 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "3"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "3"]}]}], "}"}]}], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.564578134434861*^9, 3.5645781949897203`*^9}}],

Cell["\<\
Here I have nested \[Zeta]-function appearing.\
\>", "Text",
 CellChangeTimes->{{3.56457804438033*^9, 3.564578051092003*^9}}],

Cell["\<\
I should check it at least at the leading order. I can even make a numerical \
check for this expression. That is probably I will do\
\>", "Text",
 CellChangeTimes->{{3.5645782091900883`*^9, 3.56457824882064*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Wpvinttran", "/.", 
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{"a_", ",", "b_", ",", "c__"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"pTC", "[", 
     RowBox[{
      RowBox[{"Normal", "@", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"a", " ", "b"}], "/.", 
          RowBox[{
           SubscriptBox["J", "aa_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"aa", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "13"}], "}"}]}], "]"}]}], ",", "c"}], 
     "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp", "=", 
   RowBox[{"%", "/.", 
    RowBox[{
     RowBox[{"pTC", "[", 
      RowBox[{"a_", ",", "b__", ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "3"]}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
     RowBox[{"pTC", "[", 
      RowBox[{
       RowBox[{"Integrate", "[", 
        RowBox[{"a", ",", 
         RowBox[{"{", 
          RowBox[{
           SubscriptBox["z", "2"], ",", "0", ",", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "3"]}]}], "}"}]}], "]"}], ",", "b"}], 
      "]"}]}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5645782411925917`*^9, 3.564578438612217*^9}, {
  3.5645786448854237`*^9, 3.564578698643607*^9}, {3.5645788051149*^9, 
  3.564578828091852*^9}, {3.564579001673624*^9, 3.564579004172229*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tmp", "/.", 
     RowBox[{
      SubscriptBox["z", "a_"], "\[Rule]", 
      SubscriptBox["ee", "a"]}]}], "/.", 
    RowBox[{"z", "\[Rule]", 
     RowBox[{"1", "/", "7"}]}]}], "/.", 
   RowBox[{"g", "\[Rule]", 
    RowBox[{"11", "/", "10"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "/.", 
  RowBox[{"pTC", "\[Rule]", "NIntegrate"}]}]}], "Input",
 CellChangeTimes->{{3.564578440405202*^9, 3.564578468022911*^9}, {
  3.564578736412771*^9, 3.564578755144158*^9}}],

Cell[BoxData[
 RowBox[{"0.`", "\[VeryThinSpace]", "+", 
  RowBox[{"33639.321797546625`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.564578443187188*^9, 3.564578468879726*^9}, 
   3.564578757670961*^9, {3.564578811191916*^9, 3.5645788339105787`*^9}, 
   3.564579017570229*^9, 3.564599040385879*^9, 3.564602305080532*^9, 
   3.564832819717124*^9, 3.565347926606268*^9, 3.565348070804553*^9, 
   3.5671499991818047`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[CapitalLambda]", "=", "13"}], "}"}], ",", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            FractionBox["1", "2"]}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FractionBox["1", 
               RowBox[{"u", "-", "v"}]], "-", 
              FractionBox["1", 
               RowBox[{"u", "+", "v"}]]}], "/.", 
             RowBox[{"v", "\[Rule]", 
              RowBox[{"2", " ", "g", " ", "z"}]}]}], ")"}], "Wtmp"}], "/.", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "2", "]"}], 
             "\[RuleDelayed]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  FractionBox["1", 
                   SuperscriptBox[
                    RowBox[{"x$h", "[", "0", "]"}], "a"]], "/.", "subJuk"}], 
                 ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], 
                "]"}], "/.", 
               RowBox[{
                SuperscriptBox["u", 
                 RowBox[{"pow_", "/;", 
                  RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
                RowBox[{
                 RowBox[{"\[Eta]$", "[", 
                  RowBox[{"-", "pow"}], "]"}], "[", "2", "]"}]}]}], ")"}]}], 
            ",", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Eta]$bh", "[", "a_", "]"}], "[", 
              RowBox[{"-", "2"}], "]"}], "\[RuleDelayed]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  FractionBox["1", 
                   SuperscriptBox[
                    RowBox[{"x$h", "[", "0", "]"}], "a"]], "/.", "subJuk"}], 
                 ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], 
                "]"}], "/.", 
               RowBox[{
                SuperscriptBox["u", 
                 RowBox[{"pow_", "/;", 
                  RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
                RowBox[{
                 RowBox[{"\[Eta]$b", "[", 
                  RowBox[{"-", "pow"}], "]"}], "[", 
                 RowBox[{"-", "2"}], "]"}]}]}], ")"}]}]}], "}"}]}], ")"}], "/.",
         "subJuk"}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], "]"}], "//",
      "Normal"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox["%", 
        RowBox[{
         RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
       RowBox[{"z", "\[Rule]", 
        RowBox[{"1", "/", "7"}]}]}], "/.", 
      RowBox[{"g", "\[Rule]", 
       RowBox[{"11", "/", "10"}]}]}], "/.", "subtoPolyGammas"}], "/.", 
    RowBox[{"u", "\[Rule]", 
     RowBox[{"t", "-", 
      RowBox[{"\[ImaginaryI]", "/", "4"}]}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.564578486952026*^9, 3.564578573411035*^9}, {
  3.564578608581122*^9, 3.5645786303027887`*^9}, {3.564578764097085*^9, 
  3.564578794934279*^9}, {3.564578838150284*^9, 3.564578838469836*^9}, {
  3.564578969997877*^9, 3.5645789856645823`*^9}, {3.564579020846793*^9, 
  3.564579021207189*^9}}],

Cell[BoxData[
 RowBox[{"0.`", "\[VeryThinSpace]", "+", 
  RowBox[{"33639.321839652766`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.564578484516471*^9, 3.56457857396624*^9}, {
   3.5645786094484777`*^9, 3.564578630842327*^9}, {3.564578773229603*^9, 
   3.564578796185793*^9}, 3.564578839869492*^9, 3.564578988067416*^9, 
   3.5645790251498404`*^9, 3.564599044420751*^9, 3.564602312415534*^9, 
   3.5648328238589897`*^9, 3.565347930598317*^9, 3.56715000321355*^9}]
}, Open  ]],

Cell[TextData[{
 "I am not completely sure that only higher transcendental terms were \
captured. Now I am performing analtyical expansion using ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Zeta]", 
    RowBox[{"a", ",", "b"}]], "==", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"Gamma", "[", "a", "]"}]], 
    FractionBox["1", 
     RowBox[{"Gamma", "[", "b", "]"}]], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
     RowBox[{
      FractionBox[
       SubscriptBox["dt", "1"], 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["t", "1"]], "-", "1"}]], 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", 
        SubscriptBox["t", "1"]], 
       RowBox[{
        FractionBox[
         SubscriptBox["dt", "2"], 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["t", "2"]], "-", "1"}]], "  ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["t", "1"], "-", 
           SubscriptBox["t", "2"]}], ")"}], 
         RowBox[{"a", "-", "1"}]], 
        SubsuperscriptBox["t", "2", 
         RowBox[{"b", "-", "1"}]]}]}]}]}]}]}]]]
}], "Text",
 CellChangeTimes->{{3.564832841587296*^9, 3.564832860122726*^9}, {
  3.564834057224621*^9, 3.5648340740957203`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tmp", "=", 
   RowBox[{
    RowBox[{"Wpvinttran", "[", 
     RowBox[{"[", "1", "]"}], "]"}], "/.", 
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{"a_", ",", "b_", ",", "c__"}], "]"}], "\[RuleDelayed]", 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"a", " ", "b", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              SuperscriptBox["\[ExponentialE]", 
               SubscriptBox["z", "1"]]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              SuperscriptBox["\[ExponentialE]", 
               SubscriptBox["z", "3"]]}], ")"}]}], "/.", 
           RowBox[{
            SubscriptBox["J", "aa_"], "\[RuleDelayed]", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"BesselJ", "[", 
               RowBox[{"aa", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
          RowBox[{"{", 
           RowBox[{"g", ",", "0", ",", "13"}], "}"}]}], "]"}], "//", 
        "Normal"}], ",", "c"}], "]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"tmp", "/.", 
  RowBox[{
   RowBox[{"TC", "[", 
    RowBox[{"a__", ",", "c__"}], "]"}], "\[RuleDelayed]", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"ExpaCol", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"var", " ", 
            SubscriptBox["z", "3"], 
            RowBox[{"Together", "[", "a", " ", "]"}]}], "/.", 
           RowBox[{
            SubscriptBox["z", "1"], "\[Rule]", 
            RowBox[{"var", "+", 
             SubscriptBox["z", "3"]}]}]}], ",", 
          RowBox[{"var", "|", 
           SubscriptBox["z", "3"]}]}], "]"}], "/.", 
        RowBox[{
         RowBox[{
          SubsuperscriptBox["z", "3", "bb_."], 
          SuperscriptBox["var", "aa_."]}], "\[RuleDelayed]", 
         RowBox[{
          RowBox[{"Gamma", "[", "aa", "]"}], 
          RowBox[{"Gamma", "[", "bb", "]"}], 
          SubscriptBox["\[Zeta]", 
           RowBox[{"aa", ",", "bb"}]]}]}]}], ")"}], "//.", "subZetaReduce"}], 
     ",", "canlist"}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{"%", "/.", "subtotran"}], ",", "tran"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"%", "/.", 
        RowBox[{"z", "\[Rule]", 
         RowBox[{"1", "/", "3"}]}]}], "/.", 
       RowBox[{"g", "\[Rule]", 
        RowBox[{"11", "/", "10"}]}]}], "/.", "subZetas"}], "/.", 
     "Nnnreducibles"}], "//", "N"}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.56483286079014*^9, 3.5648329385899963`*^9}, {
  3.564833332813792*^9, 3.564833349804419*^9}, {3.564833391165628*^9, 
  3.564833391427698*^9}, {3.564834083696703*^9, 3.5648340843568363`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["58176", "175"], " ", "\[ImaginaryI]", " ", "z", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "4"]}], "-", 
     RowBox[{"2016", " ", "\[ImaginaryI]", " ", "z", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"480", " ", "\[ImaginaryI]", " ", "z", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], 
       RowBox[{"2", ",", "6"}]]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1069056", " ", "\[ImaginaryI]", " ", "z"}], "275"]}], "-", 
        
        FractionBox[
         RowBox[{"178176", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "3"]}], "385"]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "5"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"12816", " ", "\[ImaginaryI]", " ", "z"}], "+", 
        RowBox[{"3840", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox["z", "3"]}]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5", "2"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"27792", " ", "\[ImaginaryI]", " ", "z"}], "+", 
        RowBox[{"1920", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox["z", "3"]}]}], ")"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"4536", " ", "\[ImaginaryI]", " ", "z", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], 
       RowBox[{"2", ",", "8"}]]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"597875328", " ", "\[ImaginaryI]", " ", "z"}], "15925"], "+", 
        FractionBox[
         RowBox[{"5356032", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "3"]}], "637"], "+", 
        FractionBox[
         RowBox[{"168812544", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "5"]}], "79625"]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "6"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "303264"}], " ", "\[ImaginaryI]", " ", "z"}], "-", 
        RowBox[{"147456", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox["z", "3"]}], "-", 
        RowBox[{"23040", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox["z", "5"]}]}], ")"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "357504"}], " ", "\[ImaginaryI]", " ", "z"}], "-", 
        RowBox[{"21504", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox["z", "3"]}], "-", 
        RowBox[{"21504", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox["z", "5"]}]}], ")"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "+", 
     RowBox[{"44352", " ", "\[ImaginaryI]", " ", "z", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], 
       RowBox[{"2", ",", "10"}]]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.564832863955841*^9, {3.564832918229684*^9, 3.564832923617799*^9}, {
   3.564833335336693*^9, 3.5648333502471724`*^9}, 3.564833392237405*^9, 
   3.564834085165907*^9, 3.565347930868792*^9, 3.5671500063927917`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "tran"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["58176", "175"], " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "9"], " ", "z", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2", "4"]}], "+", 
      RowBox[{
       SuperscriptBox["g", "11"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"1069056", " ", "\[ImaginaryI]", " ", "z"}], "275"]}], "-", 
         FractionBox[
          RowBox[{"178176", " ", "\[ImaginaryI]", " ", 
           SuperscriptBox["z", "3"]}], "385"]}], ")"}], " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2", "5"]}], "+", 
      RowBox[{
       SuperscriptBox["g", "13"], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"597875328", " ", "\[ImaginaryI]", " ", "z"}], "15925"], 
         "+", 
         FractionBox[
          RowBox[{"5356032", " ", "\[ImaginaryI]", " ", 
           SuperscriptBox["z", "3"]}], "637"], "+", 
         FractionBox[
          RowBox[{"168812544", " ", "\[ImaginaryI]", " ", 
           SuperscriptBox["z", "5"]}], "79625"]}], ")"}], " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2", "6"]}], "-", 
      RowBox[{"2016", " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "9"], " ", "z", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "+", 
      RowBox[{
       SuperscriptBox["g", "11"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"12816", " ", "\[ImaginaryI]", " ", "z"}], "+", 
         RowBox[{"3840", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "3"]}]}], ")"}], " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5", "2"]}], "+", 
      RowBox[{
       SuperscriptBox["g", "11"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"27792", " ", "\[ImaginaryI]", " ", "z"}], "+", 
         RowBox[{"1920", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "3"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "+", 
      RowBox[{
       SuperscriptBox["g", "13"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "303264"}], " ", "\[ImaginaryI]", " ", "z"}], "-", 
         RowBox[{"147456", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "3"]}], "-", 
         RowBox[{"23040", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "5"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "+", 
      RowBox[{
       SuperscriptBox["g", "13"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "357504"}], " ", "\[ImaginaryI]", " ", "z"}], "-", 
         RowBox[{"21504", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "3"]}], "-", 
         RowBox[{"21504", " ", "\[ImaginaryI]", " ", 
          SuperscriptBox["z", "5"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], "+", 
      RowBox[{"480", " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "9"], " ", "z", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], 
        RowBox[{"2", ",", "6"}]]}], "-", 
      RowBox[{"4536", " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "11"], " ", "z", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], 
        RowBox[{"2", ",", "8"}]]}], "+", 
      RowBox[{"44352", " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "13"], " ", "z", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], 
        RowBox[{"2", ",", "10"}]]}]}], ")"}]}], ",", 
   RowBox[{"0.`", "\[VeryThinSpace]", "-", 
    RowBox[{"5698.093451005621`", " ", "\[ImaginaryI]"}]}]}], "}"}]], "Output",\

 CellChangeTimes->{
  3.564832863955841*^9, {3.564832918229684*^9, 3.564832923617799*^9}, {
   3.564833335336693*^9, 3.5648333502471724`*^9}, 3.564833392237405*^9, 
   3.564834085165907*^9, 3.565347930868792*^9, 3.5671500064501743`*^9}]
}, Open  ]],

Cell["Numerical confirmation", "Text",
 CellChangeTimes->{{3.564834101088052*^9, 3.564834105142425*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tmp", "/.", 
      RowBox[{
       RowBox[{"TC", "[", 
        RowBox[{"a_", ",", "c__"}], "]"}], ":>", 
       RowBox[{"TC", "[", 
        RowBox[{
         FractionBox["a", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", 
             SuperscriptBox["\[ExponentialE]", 
              SubscriptBox["z", "1"]]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", 
             SuperscriptBox["\[ExponentialE]", 
              SubscriptBox["z", "3"]]}], ")"}]}]], ",", "c"}], "]"}]}]}], "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SubscriptBox["z", "1"], "\[Rule]", "ee1"}], ",", 
       RowBox[{
        SubscriptBox["z", "3"], "\[Rule]", "ee2"}]}], "}"}]}], "/.", 
    RowBox[{"z", "\[Rule]", 
     RowBox[{"1", "/", "3"}]}]}], "/.", 
   RowBox[{"g", "\[Rule]", 
    RowBox[{"11", "/", "10"}]}]}], "/.", 
  RowBox[{"TC", "\[Rule]", "NIntegrate"}]}]], "Input",
 CellChangeTimes->{{3.564833222308751*^9, 3.564833255826823*^9}, {
  3.5648334961895113`*^9, 3.56483351293715*^9}}],

Cell[BoxData[
 RowBox[{"0.`", "\[VeryThinSpace]", "-", 
  RowBox[{"5698.093464190064`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.564833231761091*^9, 3.564833256521346*^9}, 
   3.564833515667519*^9, 3.564834106541808*^9, 3.565347938364678*^9, 
   3.56715001378953*^9}]
}, Open  ]],

Cell["\<\
Finally, there is a shifted guy which contributes at leading order\
\>", "Text",
 CellChangeTimes->{{3.564585171232541*^9, 3.564585182149766*^9}, {
  3.564834110507023*^9, 3.564834113036892*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["\[Delta]q", 
    RowBox[{"[", "2", "]"}]], "[", "W", "]"}], "\[Equal]", 
  RowBox[{"-", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["dv", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     FractionBox["1", 
      RowBox[{"\[ImaginaryI]", "+", 
       RowBox[{"2", "g", " ", "z"}], "-", "v"}]], 
     RowBox[{
      SubscriptBox["W", "pv"], "[", "v", "]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.564579073600979*^9, 3.564579120803791*^9}, 
   3.564602495976418*^9}],

Cell["\<\
This seems still being ok. We have to compute now the following expression\
\>", "Text",
 CellChangeTimes->{{3.5645791508510303`*^9, 3.5645791729697523`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ScriptCapitalC]", "*", 
   FractionBox["1", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[ImaginaryI]", "+", 
       RowBox[{"2", "g", " ", "z"}]}], ")"}], "-", "u"}]], "W"}], "==", 
  RowBox[{"\[DoubleStruckCapitalC]", "[", 
   RowBox[{
    FractionBox["1", "2"], 
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[ImaginaryI]", "+", 
          RowBox[{"2", "g", " ", "z"}]}], ")"}], "-", "u"}]], "+", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[ImaginaryI]", "+", 
          RowBox[{"2", "g", " ", "z"}]}], ")"}], "+", "u"}]]}], ")"}], "W"}], 
   "]"}], "==", 
  RowBox[{"\[DoubleStruckCapitalC]", "[", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"u", "-", 
        RowBox[{"(", 
         RowBox[{"\[ImaginaryI]", "+", 
          RowBox[{"2", "g", " ", "z"}]}], ")"}]}]], "-", 
      FractionBox["1", 
       RowBox[{"u", "-", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], "-", 
          RowBox[{"2", "g", " ", "z"}]}], ")"}]}]]}], ")"}], "W"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.564579180469955*^9, 3.564579231552301*^9}, {
  3.564585190629036*^9, 3.564585200845315*^9}, {3.564834271760886*^9, 
  3.564834289381173*^9}}],

Cell[CellGroupData[{

Cell[BoxData["Wtmp"], "Input"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]]}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]]}]], "Output",
 CellChangeTimes->{3.564598133861329*^9, 3.5653535849188547`*^9, 
  3.567150023812683*^9}]
}, Open  ]],

Cell["\<\
As far as I can see, if there is a division by x, such term will be higher \
suppressed in transcendentality. Let us check this assumption.\
\>", "Text",
 CellChangeTimes->{{3.564598140001059*^9, 3.564598165878182*^9}, {
  3.56459909733508*^9, 3.56459909802078*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Select", "[", 
       RowBox[{
        RowBox[{"Collect", "[", 
         RowBox[{"Wtmp", ",", 
          RowBox[{"x$h", "[", "0", "]"}]}], "]"}], ",", 
        RowBox[{
         RowBox[{"Not", "[", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"#", ",", "xh"}], "]"}], "]"}], "&"}]}], "]"}], "/.", 
      RowBox[{
       RowBox[{"x$h", "[", "0", "]"}], "\[Rule]", 
       FractionBox["u", "g"]}]}], "/.", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "2", "]"}], "\[Rule]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Eta]$", "[", "a", "]"}], "[", "2", "]"}], 
       SuperscriptBox["g", "a"]}]}]}], "/.", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]$bh", "[", "a_", "]"}], "[", 
      RowBox[{"-", "2"}], "]"}], "\[Rule]", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Eta]$b", "[", "a", "]"}], "[", 
       RowBox[{"-", "2"}], "]"}], 
      SuperscriptBox["g", "a"]}]}]}], ",", "g"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"%", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["1", "2"]}], 
          RowBox[{"(", 
           RowBox[{
            FractionBox["1", 
             RowBox[{"u", "-", "\[ImaginaryI]"}]], "-", 
            FractionBox["1", 
             RowBox[{"u", "-", 
              RowBox[{"(", 
               RowBox[{"-", "\[ImaginaryI]"}], ")"}]}]]}], ")"}]}], ")"}]}], "/.",
        "subtousu"}], "//", "funapartusu"}], "//", "funreduceusu"}], ",", 
    "canlist"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"UpIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         FractionBox["%", 
          RowBox[{
           RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
         RowBox[{"Shift", "[", 
          RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"u", "-", 
          RowBox[{"I", " ", 
           RowBox[{"\[Epsilon]", "/", "2"}]}]}]}]}], "/.", "subusu"}], ",", 
      "u"}], "]"}], "//.", "subZetaReduce"}], ",", "canlist"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.564598176303832*^9, 3.564598272002108*^9}, {
  3.5645991150056973`*^9, 3.564599181460882*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "3", 
        RowBox[{"[", "2", "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]]}], 
      SuperscriptBox["u", "2"]]}], "+", 
    FractionBox[
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "4", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], "u"], "+", 
    FractionBox[
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "2", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], 
     SuperscriptBox["u", "2"]], "-", 
    FractionBox[
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "2", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], "u"]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5645981818033533`*^9, 3.564598272449501*^9}, {
   3.5645991008409643`*^9, 3.564599125213291*^9}, 3.564599181738925*^9, 
   3.56483413679447*^9, 3.565347950628401*^9, 3.567150031506956*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"96", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2", "2"]}], "5"]}], "-", 
    FractionBox[
     RowBox[{"936", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "3"]}], "35"], "-", 
    RowBox[{"12", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{"12", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{"72", " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
    RowBox[{"120", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5645981818033533`*^9, 3.564598272449501*^9}, {
   3.5645991008409643`*^9, 3.564599125213291*^9}, 3.564599181738925*^9, 
   3.56483413679447*^9, 3.565347950628401*^9, 3.5671500335027227`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FindTranOrder"}]], "Input",
 CellChangeTimes->{{3.564599195285075*^9, 3.5645991991847258`*^9}}],

Cell[BoxData["\<\"tran order = -2\"\>"], "Output",
 CellChangeTimes->{3.564599199436363*^9, 3.564834145823563*^9, 
  3.565347959703321*^9}]
}, Open  ]],

Cell["\<\
As was expected, it lower than it should be. Now we proceed with term where \
x-s are absent.\
\>", "Text",
 CellChangeTimes->{{3.5645992098573723`*^9, 3.564599234733095*^9}, {
  3.5648341478485117`*^9, 3.56483414871881*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Select", "[", 
       RowBox[{
        RowBox[{"Collect", "[", 
         RowBox[{"Wtmp", ",", 
          RowBox[{"x$h", "[", "0", "]"}]}], "]"}], ",", 
        RowBox[{
         RowBox[{"FreeQ", "[", 
          RowBox[{"#", ",", "xh"}], "]"}], "&"}]}], "]"}], "/.", 
      RowBox[{
       RowBox[{"x$h", "[", "0", "]"}], "\[Rule]", 
       FractionBox["u", "g"]}]}], "/.", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Eta]$h", "[", "a_", "]"}], "[", "2", "]"}], "\[Rule]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Eta]$", "[", "a", "]"}], "[", "2", "]"}], 
       SuperscriptBox["g", "a"]}]}]}], "/.", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]$bh", "[", "a_", "]"}], "[", 
      RowBox[{"-", "2"}], "]"}], "\[Rule]", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Eta]$b", "[", "a", "]"}], "[", 
       RowBox[{"-", "2"}], "]"}], 
      SuperscriptBox["g", "a"]}]}]}], ",", "g"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"%", 
       RowBox[{"(", 
        RowBox[{"-", 
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"u", "-", "\[ImaginaryI]"}]], ")"}]}], ")"}]}], "/.", 
      "subtousu"}], "//", "funapartusu"}], "//", "funreduceusu"}], ",", 
   "canlist"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"UpIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         FractionBox["%", 
          RowBox[{
           RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
         RowBox[{"Shift", "[", 
          RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"u", "-", 
          RowBox[{"I", " ", 
           RowBox[{"\[Epsilon]", "/", "2"}]}]}]}]}], "/.", "subusu"}], ",", 
      "u"}], "]"}], "//.", "subZetaReduce"}], ",", "canlist"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.5645992570488157`*^9, 3.5645993138626003`*^9}, {
  3.564599479876686*^9, 3.564599482365089*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", 
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], "4", 
      RowBox[{"[", "2", "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]}], "-", 
    RowBox[{
     SubsuperscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], "3", 
      RowBox[{"[", "2", "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564599259611659*^9, 3.564599314485524*^9}, 
   3.56459948376363*^9, 3.564602520019308*^9, 3.5648341644180613`*^9, 
   3.5653479657846212`*^9, 3.5671500388655987`*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "4", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], 
     SuperscriptBox[
      StyleBox["\[ScriptU]",
       FontColor->RGBColor[0, 0, 0.4]], 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]], "-", 
    FractionBox[
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "3", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], 
     SuperscriptBox[
      StyleBox["\[ScriptU]",
       FontColor->RGBColor[0, 0, 0.4]], 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564599259611659*^9, 3.564599314485524*^9}, 
   3.56459948376363*^9, 3.564602520019308*^9, 3.5648341644180613`*^9, 
   3.5653479657846212`*^9, 3.5671500388739433`*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"96", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "2"]}], "5"], "+", 
    FractionBox[
     RowBox[{"384", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "3"]}], "7"], "+", 
    RowBox[{"12", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "-", 
    RowBox[{
     FractionBox["24", "5"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "2"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{"120", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "-", 
    RowBox[{"420", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564599259611659*^9, 3.564599314485524*^9}, 
   3.56459948376363*^9, 3.564602520019308*^9, 3.5648341644180613`*^9, 
   3.5653479657846212`*^9, 3.5671500395790453`*^9}]
}, Open  ]],

Cell["This was a check that this integral indeed contributes", "Text",
 CellChangeTimes->{{3.564834225286345*^9, 3.5648342363136*^9}}],

Cell["\<\
Now we can make a general statement. We need to compute this guy\
\>", "Text",
 CellChangeTimes->{{3.5645997432592573`*^9, 3.564599750138653*^9}, {
  3.564834303925997*^9, 3.564834308941821*^9}}],

Cell[BoxData[
 RowBox[{"\[ScriptCapitalC]", "*", 
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[ImaginaryI]", "+", 
      RowBox[{"2", "g", " ", "z"}]}], ")"}], "-", "u"}]], "W"}]], "Input"],

Cell["Where for W we take", "Text",
 CellChangeTimes->{{3.564834311079034*^9, 3.564834314048421*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(", 
  RowBox[{"Select", "[", 
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{"Wtmp", ",", 
      RowBox[{"x$h", "[", "0", "]"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"FreeQ", "[", 
      RowBox[{"#", ",", "xh"}], "]"}], "&"}]}], "]"}], ")"}]], "Input",
 CellChangeTimes->{{3.5645997772979183`*^9, 3.564599785623168*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]]}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.5646025321232157`*^9, 3.5648342522639103`*^9, 
  3.564834315154236*^9, 3.5653479769608107`*^9, 3.567150052528693*^9}]
}, Open  ]],

Cell["\<\
So in total we are integrating, on the real axis with measure 1/(-2\[Pi] \
\[ImaginaryI]):\
\>", "Text",
 CellChangeTimes->{{3.564834321796279*^9, 3.56483434410317*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   FractionBox[
    RowBox[{"-", "1"}], 
    RowBox[{"u", "-", "\[ImaginaryI]", " ", "-", 
     RowBox[{"2", "g", " ", "z"}]}]], ")"}], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]]}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.5645998027303123`*^9, 3.5645998111248302`*^9}}],

Cell["It should be reorganized as follows", "Text",
 CellChangeTimes->{{3.564599816824093*^9, 3.5645998249919157`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{"-", "1"}], 
     RowBox[{"u", "-", "\[ImaginaryI]", " ", "-", 
      RowBox[{"2", "g", " ", "z"}]}]], ")"}], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           SubsuperscriptBox[
            OverscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]], 
              StyleBox["_",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "4", 
            RowBox[{"[", 
             RowBox[{"-", "4"}], "]"}]], "+", 
           FractionBox["1", 
            SuperscriptBox[
             RowBox[{"(", 
              SuperscriptBox[
               OverscriptBox[
                StyleBox["x",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]], 
               RowBox[{"[", 
                RowBox[{"-", "2"}], "]"}]], ")"}], "4"]]}], ")"}]}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", 
            RowBox[{"-", "4"}], "]"}]], "+", 
          FractionBox["1", 
           SuperscriptBox[
            RowBox[{"(", 
             SuperscriptBox[
              OverscriptBox[
               StyleBox["x",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], 
              RowBox[{"[", 
               RowBox[{"-", "2"}], "]"}]], ")"}], "3"]]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}], ")"}]}], "\[TildeEqual]", 
  "\[IndentingNewLine]", "\[TildeEqual]", 
  RowBox[{
   RowBox[{"(", 
    FractionBox[
     RowBox[{"-", "1"}], 
     RowBox[{"u", " ", "-", 
      RowBox[{"2", "g", " ", "z"}]}]], ")"}], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           SubsuperscriptBox[
            OverscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]], 
              StyleBox["_",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "4", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}]], "+", 
           FractionBox["1", 
            SuperscriptBox[
             RowBox[{"(", 
              OverscriptBox[
               StyleBox["x",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], ")"}], "4"]]}], ")"}]}], 
        " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], "+", 
          FractionBox["1", 
           SuperscriptBox[
            RowBox[{"(", 
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], ")"}], "3"]]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.564599835212512*^9, 3.564599894208294*^9}, {
  3.5645999501343927`*^9, 3.564599974925384*^9}}],

Cell["\<\
Note that we droped some terms which won\[CloseCurlyQuote]t produce good \
transcendentality.\
\>", "Text",
 CellChangeTimes->{{3.564834425914317*^9, 3.564834438510385*^9}}],

Cell["The integration is to be done slightly below real axis", "Text",
 CellChangeTimes->{{3.564599979057857*^9, 3.564599989952457*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], "+", 
          FractionBox["1", 
           SuperscriptBox[
            RowBox[{"(", 
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], ")"}], "4"]]}], ")"}]}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]], "+", 
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"(", 
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], ")"}], "3"]]}], ")"}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], ",", "xh"}], "]"}]], "Input",
 CellChangeTimes->{{3.564599998019902*^9, 3.564600119228504*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "3", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "4", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
  RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]]}], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.564600023329216*^9, {3.564600058768338*^9, 3.564600068335732*^9}, {
   3.564600109247342*^9, 3.5646001195616913`*^9}, 3.5646025482744513`*^9, 
   3.567150066377267*^9}]
}, Open  ]],

Cell[TextData[{
 "After we make Laplace representation, we take integral with the following \
residue: ",
 Cell[BoxData[
  RowBox[{"(", 
   FractionBox[
    RowBox[{"-", "1"}], 
    RowBox[{"u", " ", "-", 
     RowBox[{"2", "g", " ", "z"}]}]], ")"}]]],
 ". It should be cloed above. Hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], ">", 
    SubscriptBox["z", "2"]}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], ">", 
    SubscriptBox["z", "3"]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.564600217107095*^9, 3.5646002502423573`*^9}, {
  3.56460029867845*^9, 3.564600299135041*^9}, {3.564602579505209*^9, 
  3.5646025927999573`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TC", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]]}], "/.", "subtoBes2"}], ",", 
   
   FractionBox[
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["z", "1"], "-", 
        SubscriptBox["z", "2"]}], ")"}]}]], 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      SubscriptBox["z", "1"]], "-", "1"}]], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "2"], ",", "0", ",", 
     SubscriptBox["z", "1"]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.564834798914661*^9, 3.5648348048227863`*^9}, {
  3.567150076478163*^9, 3.56715007663505*^9}}],

Cell[BoxData[
 RowBox[{"TC", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"144", " ", "g", " ", 
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}], " ", 
       RowBox[{
        SubscriptBox["J", "4"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "1"]}], "]"}]}], 
      RowBox[{
       SubscriptBox["z", "1"], " ", 
       SubscriptBox["z", "2"]}]]}], "-", 
    FractionBox[
     RowBox[{"144", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "3"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}], " ", 
      RowBox[{
       SubscriptBox["J", "4"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}]}], 
     RowBox[{
      SubscriptBox["z", "1"], " ", 
      SubscriptBox["z", "2"]}]]}], ",", 
   FractionBox[
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["z", "1"], "-", 
        SubscriptBox["z", "2"]}], ")"}]}]], 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["\[ExponentialE]", 
      SubscriptBox["z", "1"]]}]], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "2"], ",", "0", ",", 
     SubscriptBox["z", "1"]}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{3.567150077069148*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Wp2inttran", "=", 
  RowBox[{
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", "2", "]"}]]}], 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
       FractionBox[
        RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", "2", "]"}]]}], 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "3"]]}], "/.", "subtoBes2"}], 
     ",", 
     FractionBox[
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}]], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           SubsuperscriptBox[
            OverscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]], 
              StyleBox["_",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "4", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}]]}], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "/.", "subtoBes2"}], ",", 
     FractionBox[
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"\[ImaginaryI]", " ", "2", " ", "g", " ", "z", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "3"]}], ")"}]}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]], "-", "1"}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "3"]], "-", "1"}], ")"}]}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "3"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5646001229410343`*^9, 3.564600211229081*^9}, {
  3.564602604826269*^9, 3.5646026473976192`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"144", " ", "g", " ", 
        RowBox[{
         SubscriptBox["J", "3"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}], " ", 
        RowBox[{
         SubscriptBox["J", "4"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}]}], 
       RowBox[{
        SubscriptBox["z", "1"], " ", 
        SubscriptBox["z", "2"]}]]}], "-", 
     FractionBox[
      RowBox[{"144", " ", "g", " ", 
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "1"]}], "]"}], " ", 
       RowBox[{
        SubscriptBox["J", "4"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}]}], 
      RowBox[{
       SubscriptBox["z", "1"], " ", 
       SubscriptBox["z", "2"]}]]}], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       SubscriptBox["z", "1"]]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"12", " ", "\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"12", " ", "\[ImaginaryI]", " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "3"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "3"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]], "Output",
 CellChangeTimes->{
  3.564600131001*^9, {3.564600203366425*^9, 3.564600211698032*^9}, 
   3.564602563975484*^9, {3.564602622604753*^9, 3.5646026481612864`*^9}, 
   3.564834580775646*^9, 3.565347984131884*^9, 3.565348089165786*^9, 
   3.565353634068326*^9, 3.56715007907476*^9}]
}, Open  ]],

Cell["Numerical check:", "Text",
 CellChangeTimes->{{3.564602656389574*^9, 3.564602661619669*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Wp2inttran", "/.", 
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{"a_", ",", "b_", ",", "c__"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"Normal", "@", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"a", " ", "b"}], "/.", 
          RowBox[{
           SubscriptBox["J", "aa_"], "\[RuleDelayed]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"BesselJ", "[", 
              RowBox[{"aa", ",", "#"}], "]"}], "&"}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}]}], ",", "c"}], 
     "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"g", "\[Rule]", 
     RowBox[{"1", "/", "3"}]}]}], "/.", 
   RowBox[{"z", "\[Rule]", 
    RowBox[{"11", "/", "10"}]}]}], "/.", 
  RowBox[{"TC", "\[Rule]", "NIntegrate"}]}]}], "Input",
 CellChangeTimes->{{3.5646003622777576`*^9, 3.564600450313292*^9}, {
   3.564600669840337*^9, 3.564600670767736*^9}, 3.564600753405168*^9, 
   3.564600841231681*^9, {3.56460112282467*^9, 3.564601122998275*^9}, {
   3.564601433154015*^9, 3.5646014647361517`*^9}, {3.5646015222159777`*^9, 
   3.564601535721219*^9}, 3.564601567377022*^9, {3.56460174098479*^9, 
   3.564601789056464*^9}, {3.564602665351054*^9, 3.564602666684453*^9}, {
   3.564834819243525*^9, 3.56483482007236*^9}, {3.564835036706038*^9, 
   3.5648350369935627`*^9}, {3.5648350739804153`*^9, 3.564835074474072*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "0.41724566265200297`"}], "+", 
  RowBox[{"0.2084825728832492`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{
  3.56460036896428*^9, {3.56460040539797*^9, 3.5646004508104*^9}, 
   3.564600677170952*^9, 3.56460075417673*^9, 3.5646008419262943`*^9, {
   3.5646011152047243`*^9, 3.5646011239479237`*^9}, {3.564601442032138*^9, 
   3.564601465257278*^9}, {3.564601522679357*^9, 3.564601536800452*^9}, 
   3.564601567839634*^9, {3.5646017474316387`*^9, 3.564601790993347*^9}, 
   3.564602671446927*^9, 3.564834584675324*^9, 3.5648348206945553`*^9, 
   3.564835037539464*^9, 3.564835075348137*^9, 3.565348092203548*^9, 
   3.5671500819830723`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       FractionBox["1", 
        RowBox[{
         RowBox[{"2", "g", " ", "z"}], " ", "+", "\[ImaginaryI]", "-", "u"}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              SubsuperscriptBox[
               OverscriptBox[
                OverscriptBox[
                 StyleBox["\[Eta]",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]], 
                 StyleBox["_",
                  FontWeight->Bold,
                  FontColor->RGBColor[0.8, 0, 0]]], 
                StyleBox["^",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], "4", 
               RowBox[{"[", 
                RowBox[{"-", "2"}], "]"}]]}], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "3", 
              RowBox[{"[", "2", "]"}]]}], "+", 
            RowBox[{
             SubsuperscriptBox[
              OverscriptBox[
               OverscriptBox[
                StyleBox["\[Eta]",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]], 
                StyleBox["_",
                 FontWeight->Bold,
                 FontColor->RGBColor[0.8, 0, 0]]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "3", 
              RowBox[{"[", 
               RowBox[{"-", "2"}], "]"}]], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "4", 
              RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "/.", 
         RowBox[{"sub\[Eta]h\[Eta]", "[", "12", "]"}]}], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}], ",", "canlist"}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"UpIntegrate", "[", 
          RowBox[{
           FractionBox["%", 
            RowBox[{
             RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], ",", "u"}],
           "]"}], "//.", "subZetaReduce"}], ",", "canlist"}], "]"}], "/.", 
      "subtotran"}], "//", "FindHighTranCoef"}], ",", "canlist"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"%", "/.", "subZetas"}], "/.", 
     RowBox[{"g", "\[Rule]", 
      RowBox[{"1", "/", "3"}]}]}], "/.", 
    RowBox[{"z", "\[Rule]", 
     RowBox[{"11", "/", "10"}]}]}], "/.", "Nnnreducibles"}], "//", 
  "N"}]}], "Input",
 CellChangeTimes->{{3.5646009273293877`*^9, 3.5646010526295347`*^9}, {
   3.564601328766705*^9, 3.5646013289317007`*^9}, {3.564601407945342*^9, 
   3.564601408760915*^9}, {3.564602694624929*^9, 3.564602723413583*^9}, {
   3.564834674552409*^9, 3.564834732061561*^9}, 3.5648348293847923`*^9, {
   3.564834984075241*^9, 3.564835000463235*^9}, {3.5648350489880867`*^9, 
   3.5648350659259577`*^9}, {3.564835103467101*^9, 3.564835134272422*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "0.41724566243949773`"}], "+", 
  RowBox[{"0.2084825724603162`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.5646009470500507`*^9, 3.564601053095381*^9}, 
   3.564601409391082*^9, {3.564602695411759*^9, 3.564602738169289*^9}, 
   3.564834639030472*^9, {3.564834708268792*^9, 3.564834733484323*^9}, 
   3.564834830635426*^9, 3.564834962303722*^9, 3.5648349956991673`*^9, {
   3.564835028377512*^9, 3.564835069376045*^9}, {3.56483512467909*^9, 
   3.5648351488223133`*^9}, 3.565348094786068*^9, 3.565348170807908*^9, 
   3.567150096822147*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Normal", "@", 
          RowBox[{"Series", "[", 
           RowBox[{
            RowBox[{"Wpvleadtran", "/.", 
             RowBox[{"sub\[Eta]h\[Eta]", "[", "12", "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}]}], ")"}], "/.",
         "subJuk"}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"2", "g", " ", "z"}]}]}], "/.", 
      RowBox[{"TaylorOptions", "[", 
       RowBox[{
        RowBox[{"Variable", "\[Rule]", "g"}], ",", 
        RowBox[{"\[Eta]Factor", "\[Rule]", 
         RowBox[{"2", "z"}]}], ",", " ", 
        RowBox[{"Order", "\[Rule]", "8"}]}], "]"}]}], "//.", "denz"}], "//.", 
    "subZetaReduce"}], ",", "canlist"}], "]"}]], "Input",
 CellChangeTimes->{{3.564835169430983*^9, 3.564835271358387*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{"24", "-", 
     RowBox[{"48", " ", 
      SuperscriptBox["z", "2"]}]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["144", "5"]}], "+", 
        FractionBox[
         RowBox[{"576", " ", 
          SuperscriptBox["z", "2"]}], "5"], "-", 
        FractionBox[
         RowBox[{"1152", " ", 
          SuperscriptBox["z", "4"]}], "5"]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "72"}], "-", 
        RowBox[{"240", " ", 
         SuperscriptBox["z", "2"]}], "+", 
        RowBox[{"1152", " ", 
         SuperscriptBox["z", "4"]}]}], ")"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.5648351728212833`*^9, 3.5648352184277077`*^9}, {
   3.564835248533374*^9, 3.5648352718709087`*^9}, 3.564835499334656*^9, 
   3.565348190523427*^9, 3.567150099644185*^9}]
}, Open  ]],

Cell["We are now ready to compute corresponding ratios:", "Text",
 CellChangeTimes->{{3.564602765745448*^9, 3.564602783494939*^9}, {
  3.564603024080203*^9, 3.564603025052637*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox[
      StyleBox["\[GothicQ]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
      RowBox[{"[", "2", "]"}]], "-", 
     SubsuperscriptBox[
      StyleBox["\[GothicQ]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
      RowBox[{"[", 
       RowBox[{"+", "o"}], "]"}]]}], ")"}], ",", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[GothicQ]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], 
        StyleBox["_",
         FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
       RowBox[{"[", 
        RowBox[{"-", "o"}], "]"}]]}], "+", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[GothicQ]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       StyleBox["_",
        FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]}], ")"}], ",", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox[
      StyleBox["\[GothicQ]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
      RowBox[{"[", "2", "]"}]], "+", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[GothicQ]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], "_"], "2", 
      RowBox[{"[", 
       RowBox[{"-", "o"}], "]"}]]}], ")"}], ",", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox[
      StyleBox["\[GothicQ]",
       FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
      RowBox[{"[", 
       RowBox[{"+", "o"}], "]"}]], "+", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[GothicQ]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], "_"], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]}], ")"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.564602900110667*^9, 3.564602916000824*^9}, 
   3.564602949569305*^9, 3.56460307702184*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tmp3", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "\[Delta]q2W"}], "-", 
      RowBox[{
       FractionBox["1", "2"], "Wpv"}], "-", "Wintpv"}], ",", 
     RowBox[{
      RowBox[{"-", "\[Delta]q2Wcc"}], "-", 
      RowBox[{
       FractionBox["1", "2"], "Wpv"}], "+", "Wintpv"}], ",", 
     RowBox[{
      RowBox[{"-", "\[Delta]q2W"}], "+", 
      RowBox[{
       FractionBox["1", "2"], "Wpv"}], "-", "Wintpv"}], ",", 
     RowBox[{
      RowBox[{"-", "\[Delta]q2Wcc"}], "+", 
      RowBox[{
       FractionBox["1", "2"], "Wpv"}], "+", "Wintpv"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.564602955324399*^9, 3.5646029736247463`*^9}, {
  3.564603036159545*^9, 3.5646030500188217`*^9}, {3.564835343999261*^9, 
  3.56483538740868*^9}, {3.564835513662981*^9, 3.5648355338485737`*^9}, {
  3.564835578461561*^9, 3.564835580888012*^9}, {3.564835644920393*^9, 
  3.564835680890079*^9}, {3.56535367964141*^9, 3.565353683909363*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Delta]W0tran", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "1", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "1", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "2", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "2", "]"}], "]"}]]}], ")"}]}], 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "3", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "3", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "4", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "4", "]"}], "]"}]]}], ")"}]}]], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", "Simplify"}], 
   "//", "Normal"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", "subJuk"}], "/.", 
   RowBox[{"u", "\[Rule]", 
    RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.564602825720697*^9, 3.564602826038568*^9}, {
  3.564835685948802*^9, 3.564835727619754*^9}, {3.5648357699807673`*^9, 
  3.5648357717138042`*^9}, {3.565353518918269*^9, 3.565353522939275*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"3", " ", "\[ImaginaryI]", " ", "g", " ", 
   RowBox[{"(", 
    RowBox[{"\[Delta]q2W", "+", "\[Delta]q2Wcc"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}]}], 
  StyleBox["x",
   FontColor->RGBColor[0, 0.6, 0]]]], "Output",
 CellChangeTimes->{{3.564835692476359*^9, 3.5648357279340677`*^9}, 
   3.56483577235666*^9, 3.565348202216694*^9, 3.565352095592411*^9, 
   3.565352141758438*^9, {3.5653535196705837`*^9, 3.5653535234124393`*^9}, 
   3.5653536853834467`*^9, 3.5671500998392773`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "6"}], " ", "g", " ", 
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{"\[Delta]q2W", "+", "\[Delta]q2Wcc"}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564835692476359*^9, 3.5648357279340677`*^9}, 
   3.56483577235666*^9, 3.565348202216694*^9, 3.565352095592411*^9, 
   3.565352141758438*^9, {3.5653535196705837`*^9, 3.5653535234124393`*^9}, 
   3.5653536853834467`*^9, 3.567150100009111*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]W0tran", "=", 
   RowBox[{
    RowBox[{"-", "6"}], " ", "g", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"144", " ", "g", " ", 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], " ", 
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             RowBox[{
              SubscriptBox["z", "1"], " ", 
              SubscriptBox["z", "2"]}]]}], "-", 
           FractionBox[
            RowBox[{"144", " ", "g", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "2"]}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]], "+", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}]}], ",", 
        FractionBox["1", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]]}]], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
        
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "2"], ",", "0", ",", 
          SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"12", " ", "\[ImaginaryI]", " ", 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "3"]}], "]"}], " ", 
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             RowBox[{
              SubscriptBox["z", "1"], " ", 
              SubscriptBox["z", "3"]}]]}], "-", 
           FractionBox[
            RowBox[{"12", " ", "\[ImaginaryI]", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "3"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "3"]}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "3"]}], ")"}]}]], "+", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "3"]}], ")"}]}]]}], ")"}]}], ",", 
        FractionBox["1", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            SuperscriptBox["\[ExponentialE]", 
             SubscriptBox["z", "1"]]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            SuperscriptBox["\[ExponentialE]", 
             SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
        
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
       "]"}]}], ")"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5648359641958723`*^9, 3.564835967305212*^9}, {
   3.564836008414768*^9, 3.5648360483766613`*^9}, {3.565352179019367*^9, 
   3.565352189124148*^9}, {3.5653522200444736`*^9, 3.565352233688507*^9}, 
   3.565353690195653*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   RowBox[{"Log", "[", " ", 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2"], "-", 
         SubsuperscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
          RowBox[{"[", "2", "]"}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox[
          OverscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           StyleBox["_",
            FontColor->RGBColor[0.54, 0.35, 0.15]]], "2"], "-", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           StyleBox["_",
            FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], ")"}], 
       SuperscriptBox[
        StyleBox["\[GothicCapitalT]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], "-"], " ", 
       SuperscriptBox[
        StyleBox["\[GothicCapitalT]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]}], ")"}], "r"], "]"}], "==", 
   RowBox[{"\[Delta]\[Rho]2contrtranPROD", "-", "W1contrtran"}]}]],
  CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
    3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
    3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
    3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
    3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
    3.5644070733510857`*^9, 3.56440708351433*^9}, 3.564464488007196*^9, 
    3.564504436271771*^9}],
 "+",
 "\[Delta]W0tranPROD"
}], "Subsubsection",
 CellChangeTimes->{
  3.564509217500421*^9, {3.5645094885017223`*^9, 3.564509590043518*^9}, {
   3.564574522275008*^9, 3.564574526014905*^9}, {3.564575131390028*^9, 
   3.564575131798626*^9}, {3.5648340288017273`*^9, 3.564834033749136*^9}, {
   3.564836103919668*^9, 3.564836104636978*^9}, {3.5648361568247643`*^9, 
   3.56483616292057*^9}, {3.5648362252325697`*^9, 3.5648362298134947`*^9}, {
   3.564837166564733*^9, 3.564837171185782*^9}}],

Cell["\<\
We benefit from the computation above (which was pretty much generic)\
\>", "Text",
 CellChangeTimes->{{3.564836169576415*^9, 3.564836187086299*^9}}],

Cell[CellGroupData[{

Cell[BoxData["tmp2"], "Input",
 CellChangeTimes->{{3.5648369480102787`*^9, 3.564836951310429*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[Delta]\[Rho]2", "2"]}], "+", 
    RowBox[{"g", " ", 
     RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}], "-", 
    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[Delta]\[Rho]2", "2"]}], "+", 
    RowBox[{"g", " ", 
     RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}], "+", 
    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ",", 
   RowBox[{
    FractionBox["\[Delta]\[Rho]2", "2"], "+", 
    RowBox[{"g", " ", 
     RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}], "-", 
    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}], ",", 
   RowBox[{
    FractionBox["\[Delta]\[Rho]2", "2"], "+", 
    RowBox[{"g", " ", 
     RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}], "+", 
    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.565348293960629*^9, 3.56715010650259*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]\[Rho]2contrtranPROD", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Log", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", 
          FractionBox[
           RowBox[{"tmp2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], 
           RowBox[{"tmp1", "[", 
            RowBox[{"[", "1", "]"}], "]"}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          FractionBox[
           RowBox[{"tmp2", "[", 
            RowBox[{"[", "2", "]"}], "]"}], 
           RowBox[{"tmp1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          FractionBox[
           RowBox[{"tmp2", "[", 
            RowBox[{"[", "3", "]"}], "]"}], 
           RowBox[{"tmp1", "[", 
            RowBox[{"[", "3", "]"}], "]"}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          FractionBox[
           RowBox[{"tmp2", "[", 
            RowBox[{"[", "4", "]"}], "]"}], 
           RowBox[{"tmp1", "[", 
            RowBox[{"[", "4", "]"}], "]"}]]}], ")"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "2"}], "}"}]}], "]"}], "//", "Simplify"}], 
   "//", "Normal"}]}]], "Input",
 CellChangeTimes->{{3.564836232566319*^9, 3.5648362373038807`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"3", " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Delta]\[Rho]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}]}]}], ")"}]}], 
   StyleBox["x",
    FontColor->RGBColor[0, 0.6, 0]]], "+", 
  FractionBox[
   RowBox[{"9", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", "\[Delta]\[Rho]2", " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["\[Delta]\[Rho]2", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}], "+", 
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}]}], ")"}]}], 
   RowBox[{"4", " ", 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "2"]}]]}]], "Output",
 CellChangeTimes->{3.564836244157199*^9, 3.564836908894177*^9, 
  3.564837049597928*^9, 3.565348295879587*^9, 3.567150108221559*^9}]
}, Open  ]],

Cell["\<\
The expression below should contribute at 9 loops only but we have to keep it \
anyway to give finally a consistent ressumation of transcendentality.\
\>", "Text",
 CellChangeTimes->{{3.564836875134068*^9, 3.564836905172607*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["g", "14"], 
                FractionBox[
                 RowBox[{"9", " ", 
                  SuperscriptBox["g", "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"4", " ", "\[Delta]\[Rho]2", " ", 
                    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", "1"}], "+", 
                    SuperscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}], "+", 
                    RowBox[{
                    SuperscriptBox["\[Delta]\[Rho]2", "2"], " ", 
                    RowBox[{"(", 
                    RowBox[{"1", "+", 
                    SuperscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}], "+", 
                    RowBox[{"4", " ", 
                    SuperscriptBox[
                    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], "2"], " ", 
                    RowBox[{"(", 
                    RowBox[{"1", "+", 
                    SuperscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}]}], 
                   ")"}]}], 
                 RowBox[{"4", " ", 
                  SuperscriptBox[
                   StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "2"]}]]}], "/.", 
               RowBox[{"\[Delta]\[Rho]2", "\[Rule]", 
                SubscriptBox["\[Delta]\[Rho]2", "lead"]}]}], "//.", 
              "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "/.", 
            "subJuk"}], "/.", 
           RowBox[{"u", "\[Rule]", 
            RowBox[{"2", "g", " ", "z"}]}]}], ",", 
          RowBox[{"{", 
           RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], "//.", 
        "denz"}], "//.", "subZetaReduce"}], "//", "fundeperiod"}], ",", 
     "zlist"}], "]"}], "/.", "subtotran"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"FindHighTranCoef", "[", "%", "]"}]}], "Input",
 CellChangeTimes->{{3.5648363098431683`*^9, 3.564836356912759*^9}, {
  3.564836394002995*^9, 3.564836406577448*^9}, {3.564836800054245*^9, 
  3.564836842687531*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "16"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "127008"}], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7", "2"]}], "+", 
    RowBox[{"1016064", " ", 
     SuperscriptBox["z", "2"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7", "2"]}], "-", 
    RowBox[{"1016064", " ", 
     SuperscriptBox["z", "4"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7", "2"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564836317770409*^9, 3.564836338193228*^9}, {
   3.564836389366626*^9, 3.564836411665702*^9}, 3.5648368111880693`*^9, 
   3.5648368495898533`*^9, 3.565348329003759*^9, 3.5671501457574368`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Delta]W0tranPROD", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Log", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "1", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "1", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "2", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "2", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "3", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "3", "]"}], "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           FractionBox[
            RowBox[{"tmp3", "[", 
             RowBox[{"[", "4", "]"}], "]"}], 
            RowBox[{"tmp1", "[", 
             RowBox[{"[", "4", "]"}], "]"}]]}], ")"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", "Simplify"}],
     "//", "Normal"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", "subJuk"}], "/.", 
   RowBox[{"u", "\[Rule]", 
    RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}]}], "Input",
 CellChangeTimes->{{3.564837210024721*^9, 3.5648372438166122`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "6"}], " ", "g", " ", "Wpv", " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]]}], "+", 
  RowBox[{"6", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "Wintpv"}], "+", "\[Delta]q2W", "-", "\[Delta]q2Wcc"}],
     ")"}]}]}]], "Output",
 CellChangeTimes->{{3.564837215234377*^9, 3.5648372450281467`*^9}, 
   3.5653483324650183`*^9, 3.567150146559565*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Wpvleadtran", "/.", "subJuk"}], "/.", 
   RowBox[{"u", "\[Rule]", 
    RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.564837253663301*^9, 3.564837362680994*^9}}],

Cell[BoxData[
 RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     SuperscriptBox["z", "2"], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "2", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "+", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]]}], ")"}], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "+", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["z", "2"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", "2", "]"}]]}], "-", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}], ")"}]}], "+", 
    RowBox[{"z", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "2", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564837339062963*^9, 3.5648373630707283`*^9}, 
   3.565348336535687*^9, 3.56715014698834*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]W0tranPROD", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "6"}], " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Wpvleadtran", "/.", "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}], ")"}], " ", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]]}], "+", 
    RowBox[{"6", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "Wpvinttran"}], "-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"TC", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"144", " ", "g", " ", 
               RowBox[{
                SubscriptBox["J", "3"], "[", 
                RowBox[{"2", " ", "g", " ", 
                 SubscriptBox["z", "2"]}], "]"}], " ", 
               RowBox[{
                SubscriptBox["J", "4"], "[", 
                RowBox[{"2", " ", "g", " ", 
                 SubscriptBox["z", "1"]}], "]"}]}], 
              RowBox[{
               SubscriptBox["z", "1"], " ", 
               SubscriptBox["z", "2"]}]]}], "-", 
            FractionBox[
             RowBox[{"144", " ", "g", " ", 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}], " ", 
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}]}], 
             RowBox[{
              SubscriptBox["z", "1"], " ", 
              SubscriptBox["z", "2"]}]]}], ",", 
           FractionBox[
            RowBox[{
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
               RowBox[{"(", 
                RowBox[{
                 SubscriptBox["z", "1"], "-", 
                 SubscriptBox["z", "2"]}], ")"}]}]], "-", 
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
               " ", 
               RowBox[{"(", 
                RowBox[{
                 SubscriptBox["z", "1"], "-", 
                 SubscriptBox["z", "2"]}], ")"}]}]]}], 
            RowBox[{
             RowBox[{"-", "1"}], "+", 
             SuperscriptBox["\[ExponentialE]", 
              SubscriptBox["z", "1"]]}]], ",", 
           RowBox[{"{", 
            RowBox[{
             SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{
             SubscriptBox["z", "2"], ",", "0", ",", 
             SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
         RowBox[{"TC", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", 
               FractionBox[
                RowBox[{"12", " ", "\[ImaginaryI]", " ", 
                 RowBox[{
                  SubscriptBox["J", "3"], "[", 
                  RowBox[{"2", " ", "g", " ", 
                   SubscriptBox["z", "3"]}], "]"}], " ", 
                 RowBox[{
                  SubscriptBox["J", "4"], "[", 
                  RowBox[{"2", " ", "g", " ", 
                   SubscriptBox["z", "1"]}], "]"}]}], 
                RowBox[{
                 SubscriptBox["z", "1"], " ", 
                 SubscriptBox["z", "3"]}]]}], "-", 
              FractionBox[
               RowBox[{"12", " ", "\[ImaginaryI]", " ", 
                RowBox[{
                 SubscriptBox["J", "3"], "[", 
                 RowBox[{"2", " ", "g", " ", 
                  SubscriptBox["z", "1"]}], "]"}], " ", 
                RowBox[{
                 SubscriptBox["J", "4"], "[", 
                 RowBox[{"2", " ", "g", " ", 
                  SubscriptBox["z", "3"]}], "]"}]}], 
               RowBox[{
                SubscriptBox["z", "1"], " ", 
                SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
           FractionBox[
            RowBox[{
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
               RowBox[{"(", 
                RowBox[{
                 SubscriptBox["z", "1"], "-", 
                 SubscriptBox["z", "3"]}], ")"}]}]], "-", 
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
               " ", 
               RowBox[{"(", 
                RowBox[{
                 SubscriptBox["z", "1"], "-", 
                 SubscriptBox["z", "3"]}], ")"}]}]]}], 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox["\[ExponentialE]", 
                SubscriptBox["z", "1"]]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox["\[ExponentialE]", 
                SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
           RowBox[{"{", 
            RowBox[{
             SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{
             SubscriptBox["z", "3"], ",", "0", ",", 
             SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], ")"}]}], ")"}]}]}]}],
   ";"}]], "Input",
 CellChangeTimes->{{3.564837377037599*^9, 3.5648374519840393`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["Wp2inttran"], "Input",
 CellChangeTimes->{{3.5648374053047543`*^9, 3.564837406540698*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"144", " ", "g", " ", 
        RowBox[{
         SubscriptBox["J", "3"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}], " ", 
        RowBox[{
         SubscriptBox["J", "4"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}]}], 
       RowBox[{
        SubscriptBox["z", "1"], " ", 
        SubscriptBox["z", "2"]}]]}], "-", 
     FractionBox[
      RowBox[{"144", " ", "g", " ", 
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "1"]}], "]"}], " ", 
       RowBox[{
        SubscriptBox["J", "4"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}]}], 
      RowBox[{
       SubscriptBox["z", "1"], " ", 
       SubscriptBox["z", "2"]}]]}], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       SubscriptBox["z", "1"]]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"12", " ", "\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"12", " ", "\[ImaginaryI]", " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "3"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
    FractionBox[
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "3"]}], ")"}]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.564837407245016*^9, 3.5653483438453283`*^9, 
  3.567150147089118*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Wpvinttran"], "Input",
 CellChangeTimes->{{3.564837385402603*^9, 3.564837387410285*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "12"}], " ", "\[ImaginaryI]", " ", "g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"12", " ", "\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"12", " ", "\[ImaginaryI]", " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "3"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "3"]}], ")"}]}]]}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "3"]}], ")"}]}]]}], 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
  RowBox[{"TC", "[", 
   RowBox[{
    FractionBox[
     RowBox[{"24", " ", "\[ImaginaryI]", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "2"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"6", " ", "\[ImaginaryI]", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "3"]}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], " ", 
           SubscriptBox["z", "3"]}]]}], "-", 
        FractionBox[
         RowBox[{"6", " ", "\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], ")"}]}], 
     SubscriptBox["z", "2"]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"], "-", 
           SubscriptBox["z", "3"]}], ")"}]}]]}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"], "-", 
          SubscriptBox["z", "3"]}], ")"}]}]]}], 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", 
      RowBox[{
       SubscriptBox["z", "1"], "-", 
       SubscriptBox["z", "3"]}]}], "}"}]}], "]"}], "+", 
  RowBox[{"TC", "[", 
   RowBox[{
    FractionBox[
     RowBox[{"12", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "1"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]], "-", 
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "3"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "3"]}]]}], ")"}]}], 
     SubscriptBox["z", "2"]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"], "-", 
           SubscriptBox["z", "3"]}], ")"}]}]]}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"], "-", 
          SubscriptBox["z", "3"]}], ")"}]}]]}], 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "3"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", 
      RowBox[{
       SubscriptBox["z", "1"], "-", 
       SubscriptBox["z", "3"]}]}], "}"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.5648373880160713`*^9, 3.5653483458883343`*^9, 
  3.5671501471718597`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{"2", 
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       SubsuperscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], 
        RowBox[{"1", ",", "0"}], "c"], 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         RowBox[{"0", ",", "1"}], "c"], " "}]], ")"}], "r"], "]"}]}]]],
 "==",
 "TCcontrtran"
}], "Subsubsection",
 CellChangeTimes->{
  3.564837563206853*^9, {3.564837763555868*^9, 3.564837764176622*^9}}],

Cell["We start from simple relation:", "Text",
 CellChangeTimes->{{3.5648376440232964`*^9, 3.564837647855468*^9}, {
  3.56483780824268*^9, 3.564837809512026*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tmp1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"2", 
         RowBox[{"Log", "[", 
          RowBox[{"1", "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["tran", 
               RowBox[{"-", "2"}]], "\[Delta]\[Rho]"}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"\[Rho]", "/.", "largevolume"}], "/.", "subJuk"}],
                    "/.", 
                   RowBox[{"u", "\[Rule]", 
                    RowBox[{"2", "g", " ", "z"}]}]}], "/.", 
                  RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "6"}], "}"}]}], "]"}], "//", 
               "Simplify"}], "//", "Normal"}], ")"}]}]}], "]"}]}], "/.", 
        "subtotran"}], ",", 
       RowBox[{"{", 
        RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], "//", 
     "Normal"}], ")"}], "/.", 
   RowBox[{"tran", "\[Rule]", "1"}]}]}]], "Input",
 CellChangeTimes->{{3.5648376493526917`*^9, 3.564837669598878*^9}, {
  3.564837710833755*^9, 3.5648377744563503`*^9}, {3.564840384614914*^9, 
  3.564840384955626*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["\[Delta]\[Rho]", 
   RowBox[{"g", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]]}]], "+", 
  FractionBox[
   SuperscriptBox["\[Delta]\[Rho]", "2"], 
   RowBox[{"4", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["z", "2"]}], ")"}]}]]}]], "Output",
 CellChangeTimes->{3.56483773337066*^9, 3.564837774833014*^9, 
  3.5648403855612707`*^9, 3.565348417761812*^9, 3.5671501554479628`*^9}]
}, Open  ]],

Cell["\<\
\[Delta]\[Rho]2 is computed to the sub-leading transcendentality deficit. But \
this is not all\
\>", "Text",
 CellChangeTimes->{{3.564839253355307*^9, 3.564839290812294*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[GothicCapitalT]", 
   RowBox[{"0", ",", "1"}], "c"], "\[Equal]", 
  RowBox[{"\[Delta]\[Rho]2", "+", 
   SubscriptBox["W", "0"], "-", 
   SubscriptBox[
    OverscriptBox["W", "^"], 
    RowBox[{"0", ",", "pv"}]]}]}]], "Input",
 CellChangeTimes->{{3.5648392736771383`*^9, 3.56483933755796*^9}}],

Cell[TextData[{
 Cell[BoxData[
  SubscriptBox[
   OverscriptBox["W", "^"], 
   RowBox[{"0", ",", "pv"}]]],
  CellChangeTimes->{{3.5648392736771383`*^9, 3.56483933755796*^9}}],
 " was computed."
}], "Text",
 CellChangeTimes->{{3.564839339051498*^9, 3.564839384755072*^9}}],

Cell[CellGroupData[{

Cell[BoxData["Wpvleadtran"], "Input",
 CellChangeTimes->{{3.564839385594296*^9, 3.564839387032729*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   SuperscriptBox[
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]], "2"]], 
  RowBox[{"6", " ", "\[ImaginaryI]", " ", "g", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]]}], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], "4"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], "2"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox[
            StyleBox["x",
             FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", "2", "]"}]]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], "2"], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", "2", "]"}]]}], "+", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], "3"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", "2", "]"}]]}], "+", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.5648393874446793`*^9, 3.565348420742063*^9, 
  3.567150160933852*^9}]
}, Open  ]],

Cell[TextData[{
 "Now we have to compute ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["W", "0"], TraditionalForm]]],
 ". It is a bit different exercise"
}], "Text",
 CellChangeTimes->{{3.5648393889406433`*^9, 3.564839403115652*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Wotran", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["W", "0"], "/.", "subWq"}], "/.", "subq34"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}], "\[Rule]", "0"}],
         "}"}]}], "/.", 
      RowBox[{
       RowBox[{
        RowBox[{"q$b", "[", "2", "]"}], "[", "0", "]"}], "\[Rule]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"q$", "[", "2", "]"}], "[", "0", "]"}], "/.", 
           "leadingtran"}], "/.", 
          RowBox[{"x$h", "\[Rule]", "x$"}]}], "/.", "mconj"}], ")"}]}]}], "/.", 
     RowBox[{
      RowBox[{
       RowBox[{"q$", "[", "2", "]"}], "[", "0", "]"}], "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"q$", "[", "2", "]"}], "[", "0", "]"}], "/.", 
         "leadingtran"}], "/.", 
        RowBox[{"x$h", "\[Rule]", "x$"}]}], ")"}]}]}], "/.", "leadingtran"}], 
   "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.5648394038957148`*^9, 3.5648395233825207`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]]}], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "2", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SubsuperscriptBox[
             OverscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["_",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], 
              StyleBox["^",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], "2", 
             RowBox[{"[", 
              RowBox[{"-", "2"}], "]"}]]}], "+", 
           SubsuperscriptBox[
            OverscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]], 
              StyleBox["_",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "4", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}]]}], ")"}], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", "2", "]"}]]}], "+", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]], " ", 
         RowBox[{"(", 
          RowBox[{
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "2", 
            RowBox[{"[", "2", "]"}]], "-", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["^",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], "4", 
            RowBox[{"[", "2", "]"}]]}], ")"}]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", "2", "]"}]]}], "+", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "4", 
          RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}], 
  StyleBox["x",
   FontColor->RGBColor[0, 0.6, 0]]]], "Output",
 CellChangeTimes->{{3.564839406253397*^9, 3.564839432956871*^9}, {
   3.5648394738858643`*^9, 3.5648395238737*^9}, {3.5653484230870533`*^9, 
   3.565348439365588*^9}, 3.5671501671374083`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tmp2", "=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"1", "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"\[Delta]\[Rho]2", "+", 
               RowBox[{"Collect", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Wotran", "-", "Wpvleadtran"}], "//", 
                    "Simplify"}], ")"}], "/.", "subJuk"}], "/.", 
                   RowBox[{"u", "\[Rule]", 
                    RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}], 
                 ",", "zlist"}], "]"}]}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"Collect", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"Series", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"\[Rho]2", "/.", "largevolume"}], "/.", 
                    RowBox[{"u", "\[Rule]", 
                    RowBox[{"2", " ", "g", " ", "z"}]}]}], "/.", "subJuk"}], "/.", 
                    RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
                   RowBox[{"{", 
                    RowBox[{"g", ",", "0", ",", "5"}], "}"}]}], "]"}], "//.", 
                 "densqrt3"}], ",", "zlist"}], "]"}], ")"}]}]}], "/.", 
           "subtotran"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], "//",
       "Normal"}], ")"}], "/.", 
    RowBox[{"tran", "\[Rule]", "1"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.564839528944422*^9, 3.564839636068571*^9}, {
  3.5648396962408667`*^9, 3.564839734103528*^9}, {3.564840309777136*^9, 
  3.564840313938052*^9}, {3.5648404169478416`*^9, 3.564840426010421*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"3", " ", "g", " ", "\[Delta]\[Rho]2"}], 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]]], "+", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}]], 
   RowBox[{"9", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", 
       SuperscriptBox["\[Delta]\[Rho]2", "2"]}], "+", 
      RowBox[{"16", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["z", "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "2", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "2", 
              RowBox[{"[", "2", "]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["z", "2"]}]], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "3", 
              RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "+", 
         RowBox[{
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "2", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "3", 
              RowBox[{"[", "2", "]"}]]}], "+", 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["z", "2"]}]], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "4", 
              RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}]}], 
     ")"}]}]}]}]], "Output",
 CellChangeTimes->{{3.564839536668021*^9, 3.564839548764113*^9}, {
   3.564839587770668*^9, 3.564839606981977*^9}, {3.564839707917181*^9, 
   3.564839735006988*^9}, 3.564840310927046*^9, {3.56484041955027*^9, 
   3.564840427097703*^9}, 3.565348444113019*^9, 3.56715017309589*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TCcontrtran", "=", 
  RowBox[{"tmp1", "+", "tmp2"}]}]], "Input",
 CellChangeTimes->{{3.564840434258473*^9, 3.564840483124086*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["\[Delta]\[Rho]", 
   RowBox[{"g", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]]}]], "+", 
  FractionBox[
   SuperscriptBox["\[Delta]\[Rho]", "2"], 
   RowBox[{"4", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["z", "2"]}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{"3", " ", "g", " ", "\[Delta]\[Rho]2"}], 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]]], "+", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]}]], 
   RowBox[{"9", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", 
       SuperscriptBox["\[Delta]\[Rho]2", "2"]}], "+", 
      RowBox[{"16", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["z", "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "2", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "2", 
              RowBox[{"[", "2", "]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["z", "2"]}]], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "3", 
              RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "+", 
         RowBox[{
          SubsuperscriptBox[
           OverscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "2", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "3", 
              RowBox[{"[", "2", "]"}]]}], "+", 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["z", "2"]}]], " ", 
             SubsuperscriptBox[
              OverscriptBox[
               StyleBox["\[Eta]",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]], 
               StyleBox["^",
                FontWeight->Bold,
                FontColor->RGBColor[0.8, 0, 0]]], "4", 
              RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}]}], 
     ")"}]}]}]}]], "Output",
 CellChangeTimes->{{3.564840442929208*^9, 3.5648404836360598`*^9}, 
   3.565348446011705*^9, 3.56715017513664*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{
      RowBox[{"-", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
     RowBox[{
      RowBox[{"+", "\[Infinity]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      FractionBox[
       RowBox[{"2", "u"}], 
       RowBox[{
        RowBox[{"4", 
         SuperscriptBox["g", "2"], 
         SuperscriptBox["z", "2"]}], "-", 
        SuperscriptBox["u", "2"]}]], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "2"}], 
       RowBox[{"Log", "[", 
        RowBox[{
         RowBox[{"(", 
          FractionBox[
           RowBox[{" ", 
            SuperscriptBox[
             StyleBox["\[ScriptCapitalT]",
              FontColor->RGBColor[1, 0, 0]], "+"], " "}], 
           SuperscriptBox[
            StyleBox["\[GothicCapitalT]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]], ")"}], "/", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{" ", 
             SuperscriptBox[
              StyleBox["\[ScriptCapitalT]",
               FontColor->RGBColor[1, 0, 0]], "+"], " "}], 
            SuperscriptBox[
             StyleBox["\[GothicCapitalT]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]], ")"}], "asym"]}],
         "]"}]}], ")"}]}]}], "}"}]]],
 " (* not computed, only confirmed from weak coupling that it is tran^-2. We \
will eventually need it*)"
}], "Subsubsection",
 CellChangeTimes->{
  3.564840514996903*^9, {3.565348456454473*^9, 3.565348488001443*^9}}],

Cell["\<\
What I will really check at the moment is that it is transcendentality -2. If \
it is so, I should focus on understanding the constrain on tran -1 terms. It \
should be interesting and instructive.\
\>", "Text",
 CellChangeTimes->{{3.5648405188306837`*^9, 3.564840570422078*^9}}],

Cell["\<\
I am lazy, so I will get inspiration from the weak coupling expansion\
\>", "Text",
 CellChangeTimes->{{3.564840738341175*^9, 3.564840750897172*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
     "RHS\[Delta]\[Rho]", "+", "RHS\[Delta]\[Rho]2", "+", "RHSCW", "+", 
      "RHSW1"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", "/.", "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "//",
    "Simplify"}], "//", "CCompute"}]}], "Input",
 CellChangeTimes->{{3.564840755135586*^9, 3.564840782418489*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "324"}], " ", 
   SuperscriptBox["g", "8"]}], "-", 
  RowBox[{"2808", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"]}], "-", 
  RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1944", "+", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"]}], "-", 
  RowBox[{
   FractionBox["23136", "35"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"]}], "+", 
  RowBox[{"2808", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{"288", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{"216", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
  RowBox[{"1440", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "+", 
  RowBox[{"1296", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "+", 
  RowBox[{"648", " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "+", 
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "864"}], "-", 
     RowBox[{"288", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "-", 
  RowBox[{"216", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"108", "+", 
     RowBox[{"36", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"]}], "-", 
  RowBox[{"432", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
  RowBox[{"432", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
  RowBox[{"72", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"1296", " ", "\[ImaginaryI]"}], "+", 
     RowBox[{"1296", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1296", "+", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1728"}], "-", 
     RowBox[{"576", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "648"}], " ", "\[ImaginaryI]"}], "-", 
     RowBox[{"648", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"432", "+", 
     RowBox[{"144", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"648", "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "864"}], "-", 
     RowBox[{"288", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "648"}], " ", "\[ImaginaryI]"}], "-", 
     RowBox[{"648", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"648", "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "216"}], " ", "\[ImaginaryI]"}], "-", 
     RowBox[{"216", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"432", "+", 
     RowBox[{"144", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"108", "+", 
     RowBox[{"36", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], 
    "4"]}]}]], "Output",
 CellChangeTimes->{{3.5648407517429867`*^9, 3.564840777542234*^9}, 
   3.5648408397554083`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindTranOrder", "[", "%363", "]"}]], "Input",
 CellChangeTimes->{{3.564840843275516*^9, 3.5648409236699257`*^9}}],

Cell[BoxData["\<\"tran order = -2\"\>"], "Output",
 CellChangeTimes->{{3.5648408529090652`*^9, 3.564840892507442*^9}, 
   3.5648409243054523`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Coefficient", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["g", "10"], 
       RowBox[{"SeriesCoefficient", "[", 
        RowBox[{
         RowBox[{
         "RHS\[Delta]\[Rho]", "+", "RHS\[Delta]\[Rho]2", "+", "RHSCW", "+", 
          "RHSW1"}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}]}], "/.", 
      "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "/.", 
    "subtotran"}], ",", "tran", ",", 
   RowBox[{"-", "1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5648409376661158`*^9, 3.564840993656013*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"mtmp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"Select", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"SeriesCoefficient", "[", 
                    RowBox[{
                    RowBox[{
                    "RHS\[Delta]\[Rho]", "+", "RHS\[Delta]\[Rho]2", "+", 
                    "RHSCW", "+", "RHSW1"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}], "//", 
                   "Expand"}], ",", 
                  RowBox[{
                   RowBox[{"Not", "[", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "\[DoubleStruckCapitalC]"}], "]"}], 
                    "]"}], "&"}]}], "]"}], "/.", 
                RowBox[{"\[DoubleStruckCapitalC]", "\[Rule]", "Identity"}]}], 
               "/.", "subtousu"}], "/.", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["u", "2"], "+", "1"}], ")"}], 
                RowBox[{"a_", "/;", 
                 RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"\[ScriptU]", "[", "2", "]"}], 
                  RowBox[{"\[ScriptU]", "[", 
                   RowBox[{"-", "2"}], "]"}]}], ")"}], "a"]}]}], "//", 
             "funapartusu"}], "//", "funreduceusu"}], ",", 
           SubsuperscriptBox["W", "__", 
            RowBox[{"[", "_", "]"}]]}], "]"}], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            SubsuperscriptBox["W", 
             RowBox[{"1", ",", "lead"}], "+"], "\[RuleDelayed]", 
            RowBox[{"SeriesCoefficient", "[", 
             RowBox[{
              RowBox[{
               SubsuperscriptBox["W", "1", "+"], "/.", 
               RowBox[{"leadweak", "[", "1", "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            SubsuperscriptBox["W", 
             RowBox[{"1", ",", "sublead"}], "+"], "\[RuleDelayed]", 
            RowBox[{"SeriesCoefficient", "[", 
             RowBox[{
              RowBox[{
               SubsuperscriptBox["W", "1", "+"], "/.", 
               RowBox[{"leadweak", "[", "2", "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            SubscriptBox["W", "lead"], "\[RuleDelayed]", 
            RowBox[{"SeriesCoefficient", "[", 
             RowBox[{
              RowBox[{"W", "/.", 
               RowBox[{"leadweak", "[", "1", "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            SubscriptBox["W", "sublead"], "\[RuleDelayed]", 
            RowBox[{"SeriesCoefficient", "[", 
             RowBox[{
              RowBox[{"W", "/.", 
               RowBox[{"leadweak", "[", "2", "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}]}]}], 
          "}"}]}], "/.", "subtousu"}], "//", "funapartusu"}], "//", 
      "funreduceusu"}], ",", "canlist"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5648410053820257`*^9, 3.564841026926972*^9}, {
  3.564841063131167*^9, 3.564841246686508*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"mtmprlt", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"UpIntegrate", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", 
               RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]]}], "mtmp"}], "/.", 
            RowBox[{"Shift", "[", 
             RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
           RowBox[{"u", "\[Rule]", 
            RowBox[{"u", "-", 
             FractionBox[
              RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}], "2"]}]}]}], "/.", 
          "subusu"}], ",", "u"}], "]"}], "/.", "EndSimplify"}], "//.", 
      "subZetaReduce"}], ",", "canlist"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.564841028605736*^9, 3.564841028877212*^9}, {
  3.564841255684795*^9, 3.564841303307898*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"UpIntegrate", "::", "Doubtful"}], "MessageName"], ":", 
  " ", "\<\"The integral \
UpIntegrate[\[NoBreak]\\!\\(\\(\\(\\(\\(\\(\\*StyleBox[\\\"\[Eta]\\\", \
List[Rule[FontColor, RGBColor[0.8`, 0, 0]], Rule[FontWeight, Bold]]]\\^\\([0]\
\\)\\)\\)[\\(\\(u, \\(\\({}\\)\\)\\)\\)]\\)\\)\\\\ \
\\(\\(\\(\\(\\(\\(\\*StyleBox[\\\"\[Eta]\\\", List[Rule[FontColor, \
RGBColor[0.8`, 0, 0]], Rule[FontWeight, \
Bold]]]\\)\\&\\(\\*StyleBox[\\\"_\\\", List[Rule[FontColor, RGBColor[0.8`, 0, \
0]], Rule[FontWeight, Bold]]]\\)\\)\\^\\(+\\)\\)\\)[\\(\\(u, \
\\(\\({\\(\\(-2\\)\\) - \[Epsilon]}\\)\\)\\)\\)]\\)\\)\\)\\/\\(\\(\\(-\\(\\(1\
\\/\\(2\\\\ \\@3\\)\\)\\)\\)\\) + u + \\(\\(1\\/2\\\\ \[ImaginaryI]\\\\ \
\\(\\((\\(\\(-1\\)\\) - \[Epsilon])\\)\\)\\)\\)\\)\\)\[NoBreak],\[NoBreak]\\!\
\\(u\\)\[NoBreak]] was not computed due to doubts about the covergence in its \
computation\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.564841294666523*^9, 3.564841961731299*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"UpIntegrate", "::", "Doubtful"}], "MessageName"], ":", 
  " ", "\<\"The integral \
UpIntegrate[\[NoBreak]\\!\\(\\(\\(\\(\\(\\(\\*StyleBox[\\\"\[Eta]\\\", \
List[Rule[FontColor, RGBColor[0.8`, 0, 0]], Rule[FontWeight, Bold]]]\\^\\([0]\
\\)\\)\\)[\\(\\(u, \\(\\({}\\)\\)\\)\\)]\\)\\)\\\\ \
\\(\\(\\(\\(\\(\\(\\*StyleBox[\\\"\[Eta]\\\", List[Rule[FontColor, \
RGBColor[0.8`, 0, 0]], Rule[FontWeight, \
Bold]]]\\)\\&\\(\\*StyleBox[\\\"_\\\", List[Rule[FontColor, RGBColor[0.8`, 0, \
0]], Rule[FontWeight, Bold]]]\\)\\)\\^\\(+\\)\\)\\)[\\(\\(u, \
\\(\\({\\(\\(-2\\)\\) - \[Epsilon]}\\)\\)\\)\\)]\\)\\)\\)\\/\\(1\\/\\(2\\\\ \
\\@3\\) + u + \\(\\(1\\/2\\\\ \[ImaginaryI]\\\\ \\(\\((\\(\\(-1\\)\\) - \
\[Epsilon])\\)\\)\\)\\)\\)\\)\[NoBreak],\[NoBreak]\\!\\(u\\)\[NoBreak]] was \
not computed due to doubts about the covergence in its computation\"\>"}]], \
"Message", "MSG",
 CellChangeTimes->{3.564841294666523*^9, 3.56484196287212*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "poly"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\\!\\(\[ImaginaryI]\\/\\(t\\\\ \[Epsilon]\\)\\)\
\[NoBreak] is not a polynomial. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/poly\\\", ButtonNote -> \
\\\"General::poly\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.564841294666523*^9, 3.564842035105343*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "poly"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\\!\\(\[ImaginaryI]\\/\\(t\\\\ \[Epsilon]\\)\\)\
\[NoBreak] is not a polynomial. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/poly\\\", ButtonNote -> \
\\\"General::poly\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.564841294666523*^9, 3.564842041986703*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "poly"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\\!\\(-\\(\\(1\\/\\(t\\\\ \[Epsilon]\\^2\\)\\)\\)\\)\
\[NoBreak] is not a polynomial. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/poly\\\", ButtonNote -> \
\\\"General::poly\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.564841294666523*^9, 3.564842071806601*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], ":", 
  " ", "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(General :: \
poly\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.564841294666523*^9, 3.5648420718191223`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"mtmprlt", "/.", "subtotran"}], "//", "FindHighTranCoef"}]], "Input",\

 CellChangeTimes->{{3.564842265083918*^9, 3.564842281603464*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"67968", " ", 
    SubsuperscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "2", "4"]}], "175"], "+", 
  RowBox[{
   FractionBox["1315584", "175"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "4"]}], "-", 
  RowBox[{"14256", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"12672", " ", 
   SuperscriptBox["z", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"2304", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "6"}]]}], "+", 
  RowBox[{"5760", " ", 
   SuperscriptBox["z", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "6"}]]}]}]], "Output",
 CellChangeTimes->{{3.564842268105228*^9, 3.564842281953146*^9}}]
}, Open  ]],

Cell["The conclusion is that deficit is two for this integral.", "Text",
 CellChangeTimes->{{3.564854231172686*^9, 3.56485424112385*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Constraint from ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["T", 
      RowBox[{"1", ",", "1"}]], "[", 
     SubscriptBox["v", "0"], "]"}], "\[Equal]", "0"}], TraditionalForm]]]
}], "Subsubsection",
 CellChangeTimes->{{3.564856085876954*^9, 3.564856101942617*^9}}],

Cell[BoxData[
 RowBox[{"\[Delta]\[GothicCapitalT]", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ScriptCapitalC]", 
       RowBox[{"[", 
        RowBox[{"+", "1"}], "]"}]], "-", 
      SuperscriptBox["\[ScriptCapitalC]", 
       RowBox[{"[", 
        RowBox[{"-", "1"}], "]"}]]}], ")"}], 
    OverscriptBox["*", "^"], 
    SubscriptBox["\[Delta]\[Rho]", "2"]}], "+", 
   SubscriptBox["W", "1"], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[ScriptCapitalC]", 
       RowBox[{"[", 
        RowBox[{"+", "1"}], "]"}]], "-", 
      SuperscriptBox["\[ScriptCapitalC]", 
       RowBox[{"[", 
        RowBox[{"-", "1"}], "]"}]]}], ")"}], "*", "W"}]}]}]], "Input"],

Cell[TextData[{
 "Let us see what we get for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["W", "1"], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.564856116475913*^9, 3.564856123455117*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["W", "1"], "/.", "subWq"}], "/.", "subq34"}], "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}], "\[Rule]", "0"}], 
      "}"}]}], "/.", "leadingtran"}], "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"%", "/.", "subJuk"}], "/.", 
       RowBox[{"u", "\[Rule]", 
        SubscriptBox["v", "0"]}]}], "/.", 
      RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "2"}], "}"}]}], "]"}], "//.", "densqrt3"}], 
  "//", "Simplify"}]}], "Input",
 CellChangeTimes->{{3.564856174756403*^9, 3.5648562379032097`*^9}, {
  3.564856313529332*^9, 3.56485632210464*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"12", " ", "\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "3", "]"}]]}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "3", "]"}]]}]}], ")"}], " ", "g"}], "+", 
   RowBox[{"6", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "3"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          SqrtBox["3"]}]}], ")"}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "3", "]"}]]}], "+", 
      RowBox[{"6", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "3", "]"}]]}], "+", 
      RowBox[{"\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "\[ImaginaryI]"}], "+", 
         SqrtBox["3"]}], ")"}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "3", "]"}]]}]}], ")"}], " ", 
    SuperscriptBox["g", "2"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "3"],
    SeriesData[$CellContext`g, 0, {}, 1, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   Complex[0, 
      12] ($CellContext`\[Eta]$bh[4][-3] $CellContext`\[Eta]$h[3][
       3] - $CellContext`\[Eta]$bh[3][-3] $CellContext`\[Eta]$h[4][3]), 
    6 ((-3 + Complex[0, -1] 3^Rational[1, 2]) $CellContext`\[Eta]$bh[
       4][-3] $CellContext`\[Eta]$h[2][3] + 
     6 $CellContext`\[Eta]$bh[3][-3] $CellContext`\[Eta]$h[3][3] + 
     Complex[0, 1] (Complex[0, 3] + 3^Rational[1, 2]) $CellContext`\[Eta]$bh[
       2][-3] $CellContext`\[Eta]$h[4][3])}, 1, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.564856178721043*^9, 3.564856238384899*^9}, 
   3.5648563226626883`*^9, 3.565348522190304*^9, 3.567150212399753*^9}]
}, Open  ]],

Cell[TextData[{
 "We see that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["W", "1"], TraditionalForm]]],
 " has transcendentality -1."
}], "Text",
 CellChangeTimes->{{3.5648563288413143`*^9, 3.564856345144772*^9}}],

Cell["Now, for Cauchy integrals", "Text",
 CellChangeTimes->{{3.564856351281147*^9, 3.56485635929576*^9}}],

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   FractionBox[
    RowBox[{"\[DifferentialD]", "u"}], 
    RowBox[{
     RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", 
      RowBox[{
       SubscriptBox["v", "0"], "+", 
       FractionBox["\[ImaginaryI]", "2"], "-", "u"}]], "-", 
     FractionBox["1", 
      RowBox[{
       SubscriptBox["v", "0"], "-", 
       FractionBox["\[ImaginaryI]", "2"], "-", "u"}]]}], ")"}], 
   "W"}]}]], "Input",
 CellChangeTimes->{{3.564856362037651*^9, 3.56485639931736*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"W", "/.", "subWq"}], "/.", "subq34"}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         OverscriptBox["\[Alpha]", "_"], "|", "\[Alpha]"}], "\[Rule]", "0"}], 
       "}"}]}], "/.", "leadingtran"}], "//", "Simplify"}], ",", 
   RowBox[{"x$h", "[", "_", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5648564115823298`*^9, 3.5648564334229803`*^9}, {
  3.564856724818925*^9, 3.564856725014597*^9}, {3.564856756438682*^9, 
  3.564856804492258*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SubsuperscriptBox[
         OverscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "4", 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]]}], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
   OverscriptBox[
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]], 
    StyleBox["^",
     FontColor->RGBColor[0, 0.6, 0]]]], "+", 
  RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", "2", "]"}]]}], "-", 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "3", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.564856418028528*^9, 3.564856433840458*^9}, {
   3.564856725823351*^9, 3.564856773276703*^9}, 3.564856806182056*^9, 
   3.565348525830388*^9, 3.567150216401361*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"12", " ", "\[ImaginaryI]", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "3"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "3", "]"}]]}], "-", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "3"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "3", "]"}]]}]}], ")"}], " ", "g"}]], "Input"],

Cell[BoxData[
 RowBox[{"UpIntegrate", "[", "]"}]], "Input",
 CellChangeTimes->{{3.564856875105001*^9, 3.5648568797090607`*^9}}],

Cell["\<\
Again, at best we will get transcendentality -1. But this leading order \
cancells between terms! So we do not have enough to cover leading \
transcendentality of \[Delta]\[Rho]2\
\>", "Text",
 CellChangeTimes->{{3.564856512562249*^9, 3.5648565405050163`*^9}, {
  3.5648570861558237`*^9, 3.564857117074625*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox["\[ScriptCapitalC]", 
     RowBox[{"[", 
      RowBox[{"+", "1"}], "]"}]], "-", 
    SuperscriptBox["\[ScriptCapitalC]", 
     RowBox[{"[", 
      RowBox[{"-", "1"}], "]"}]]}], ")"}], 
  OverscriptBox["*", "^"], 
  SubscriptBox["\[Delta]\[Rho]", "2"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", "g", " ", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox[
      RowBox[{"\[DifferentialD]", "z"}], 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{
         SubscriptBox["v", "0"], "+", 
         FractionBox["\[ImaginaryI]", "2"], "-", 
         RowBox[{"2", " ", "g", " ", "z"}]}]], "-", 
       FractionBox["1", 
        RowBox[{
         SubscriptBox["v", "0"], "-", 
         FractionBox["\[ImaginaryI]", "2"], "-", 
         RowBox[{"2", " ", "g", " ", "z"}]}]]}], ")"}], 
     "\[Delta]\[Rho]2"}]}]}], "\[Equal]", 
  RowBox[{"3", "g", " ", 
   RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.564856559679772*^9, 3.564856613082876*^9}, 
   3.564856660050027*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox[
       RowBox[{"2", "g", " "}], 
       RowBox[{
        RowBox[{"-", "2"}], "\[ImaginaryI]"}]], 
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         RowBox[{
          SubscriptBox["v", "0"], "+", 
          FractionBox["\[ImaginaryI]", "2"], "-", 
          RowBox[{"2", " ", "g", " ", "z"}]}]], "-", 
        FractionBox["1", 
         RowBox[{
          SubscriptBox["v", "0"], "-", 
          FractionBox["\[ImaginaryI]", "2"], "-", 
          RowBox[{"2", " ", "g", " ", "z"}]}]]}], ")"}]}], "/.", 
     RowBox[{"leadweak", "[", "5", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", "0", ",", "4"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellChangeTimes->{{3.564856626067399*^9, 3.564856657084543*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"3", " ", "g"}], "+", 
   RowBox[{"6", " ", 
    SqrtBox["3"], " ", "z", " ", 
    SuperscriptBox["g", "2"]}], "-", 
   RowBox[{"12", " ", 
    SuperscriptBox["g", "3"]}], "-", 
   RowBox[{"72", " ", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox["3"], " ", 
      SuperscriptBox["z", "3"]}], ")"}], " ", 
    SuperscriptBox["g", "4"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "5"],
    SeriesData[$CellContext`g, 0, {}, 1, 5, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   3, 6 3^Rational[1, 2] $CellContext`z, -12, (-72) 
    3^Rational[1, 2] $CellContext`z^3}, 1, 5, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.564856636274337*^9, 3.5648566574622803`*^9}, 
   3.5671502256769238`*^9}]
}, Open  ]],

Cell["Conclusion is that M[\[Delta]\[Rho]2]", "Text",
 CellChangeTimes->{{3.5648571230737257`*^9, 3.5648571390250263`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["First simplification for the solution", "Subsubsection",
 CellChangeTimes->{{3.5645553838585033`*^9, 3.56455540767109*^9}, {
  3.564854217831373*^9, 3.564854253195051*^9}}],

Cell["\<\
There is one very simple equation for the leading transcendentality\
\>", "Text",
 CellChangeTimes->{{3.564854264851095*^9, 3.564854275490757*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Delta]\[Rho]2contrtranPROD", ",", "TCcontrtran"}], "}"}], "/.", 
      RowBox[{
       SuperscriptBox["g", "2"], "\[Rule]", "0"}]}], "/.", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"-", "2"}]], "\[Rule]", "0"}]}], "/.", "subJuk"}], "/.", 
   RowBox[{"u", "\[Rule]", 
    RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "6"}], " ", "g", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", "\[Delta]\[Rho]2"}], "-", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", "z", " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}], ",", 
   FractionBox[
    RowBox[{"\[Delta]\[Rho]", "+", 
     RowBox[{"3", " ", 
      SuperscriptBox["g", "2"], " ", "\[Delta]\[Rho]2"}]}], 
    RowBox[{"g", " ", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.564854277557044*^9, 3.565348535988583*^9, 
  3.567151565497942*^9}]
}, Open  ]],

Cell["\<\
Indeed, we can check it at the first three orders\
\>", "Text",
 CellChangeTimes->{{3.564854300201573*^9, 3.564854315360474*^9}, {
  3.565348552208447*^9, 3.565348555614644*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"subPVM", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"PV", "[", 
       RowBox[{"a_", "+", "b_"}], "]"}], "\[RuleDelayed]", 
      RowBox[{
       RowBox[{"PV", "[", 
        RowBox[{"Collect", "[", 
         RowBox[{"a", ",", "g"}], "]"}], "]"}], "+", 
       RowBox[{"PV", "[", 
        RowBox[{"Collect", "[", 
         RowBox[{"b", ",", "g"}], "]"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"M", "[", 
       RowBox[{"a_", "+", "b_"}], "]"}], "\[RuleDelayed]", 
      RowBox[{
       RowBox[{"M", "[", 
        RowBox[{"Collect", "[", 
         RowBox[{"a", ",", "g"}], "]"}], "]"}], "+", 
       RowBox[{"M", "[", 
        RowBox[{"Collect", "[", 
         RowBox[{"b", ",", "g"}], "]"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"PV", "[", 
       RowBox[{
        SuperscriptBox["g", "a_."], "b_"}], "]"}], "\[RuleDelayed]", 
      RowBox[{
       SuperscriptBox["g", "a"], 
       RowBox[{"PV", "[", "b", "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"M", "[", 
       RowBox[{
        SuperscriptBox["g", "a_"], "b_"}], "]"}], "\[RuleDelayed]", 
      RowBox[{
       SuperscriptBox["g", "a"], 
       RowBox[{"M", "[", "b", "]"}]}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Delta]\[Rho]2contrtranPROD", ",", "TCcontrtran"}], "}"}], "/.", 
       RowBox[{
        SuperscriptBox["g", "2"], "\[Rule]", "0"}]}], "/.", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"-", "2"}]], "\[Rule]", "0"}]}], "/.", "subJuk"}], "/.", 
    RowBox[{"u", "\[Rule]", 
     RowBox[{"2", "g", " ", "z"}]}]}], "//", "FullSimplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"%", "/.", 
         RowBox[{"\[Delta]\[Rho]", "\[Rule]", 
          RowBox[{
           SuperscriptBox["g", "9"], 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["\[Delta]\[Rho]", "lead"], "+", 
             RowBox[{
              SuperscriptBox["g", "2"], 
              SubscriptBox["\[Delta]\[Rho]", "sublead"]}], "+", 
             RowBox[{
              SuperscriptBox["g", "4"], 
              SubscriptBox["\[Delta]\[Rho]", "sublead2"]}]}], ")"}]}]}]}], "/.", 
        RowBox[{"\[Delta]\[Rho]2", "\[Rule]", 
         RowBox[{
          SuperscriptBox["g", "7"], 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
            RowBox[{
             SuperscriptBox["g", "2"], 
             SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
            RowBox[{
             SuperscriptBox["g", "4"], 
             SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}]}], "//.",
        "subPVM"}], "/.", "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}],
     "//.", "subZetaReduce"}], "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"FindHighTranCoef", "[", 
  RowBox[{"%", "/.", "subtotran"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "//", "Total"}], "//", "Expand"}]}], "Input",
 CellChangeTimes->{{3.564844377938513*^9, 3.564844377940798*^9}, 
   3.564844567246787*^9, 3.564844616655707*^9, {3.564844722224744*^9, 
   3.564844756850348*^9}, {3.564845724327587*^9, 3.564845728840678*^9}, {
   3.5648457713491507`*^9, 3.564845789740425*^9}, {3.564849211246673*^9, 
   3.564849298012796*^9}, {3.564849387105996*^9, 3.564849494343419*^9}, {
   3.564852274957511*^9, 3.564852313929223*^9}, {3.564854288927848*^9, 
   3.564854293735544*^9}, {3.565348565488483*^9, 3.565348582983377*^9}, 
   3.565348650167452*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "1925"], 
     RowBox[{"12", " ", 
      SuperscriptBox["g", "8"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"161700", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}], "-", 
        RowBox[{"323400", " ", 
         SuperscriptBox["z", "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}], "-", 
        RowBox[{"4573800", " ", 
         SuperscriptBox["g", "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "9"]}], "+", 
        RowBox[{"7484400", " ", 
         SuperscriptBox["g", "2"], " ", 
         SuperscriptBox["z", "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "9"]}], "+", 
        RowBox[{"2217600", " ", 
         SuperscriptBox["g", "2"], " ", 
         SuperscriptBox["z", "4"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "9"]}], "+", 
        RowBox[{"89612600", " ", 
         SuperscriptBox["g", "4"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "11"]}], "-", 
        RowBox[{"117394200", " ", 
         SuperscriptBox["g", "4"], " ", 
         SuperscriptBox["z", "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "11"]}], "-", 
        RowBox[{"71148000", " ", 
         SuperscriptBox["g", "4"], " ", 
         SuperscriptBox["z", "4"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "11"]}], "-", 
        RowBox[{"13552000", " ", 
         SuperscriptBox["g", "4"], " ", 
         SuperscriptBox["z", "6"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}]}], ",", 
   RowBox[{
    FractionBox["1", "1925"], 
    RowBox[{"6", " ", 
     SuperscriptBox["g", "8"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"323400", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"646800", " ", 
        SuperscriptBox["z", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"9147600", " ", 
        SuperscriptBox["g", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"14968800", " ", 
        SuperscriptBox["g", "2"], " ", 
        SuperscriptBox["z", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"4435200", " ", 
        SuperscriptBox["g", "2"], " ", 
        SuperscriptBox["z", "4"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"179225200", " ", 
        SuperscriptBox["g", "4"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}], "-", 
       RowBox[{"234788400", " ", 
        SuperscriptBox["g", "4"], " ", 
        SuperscriptBox["z", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}], "-", 
       RowBox[{"142296000", " ", 
        SuperscriptBox["g", "4"], " ", 
        SuperscriptBox["z", "4"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}], "-", 
       RowBox[{"27104000", " ", 
        SuperscriptBox["g", "4"], " ", 
        SuperscriptBox["z", "6"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}]}], "}"}]], "Output",\

 CellChangeTimes->{
  3.564854296357958*^9, 3.5653485622897673`*^9, {3.565348647059565*^9, 
   3.5653486509393177`*^9}, 3.5671516142832527`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.564854296357958*^9, 3.5653485622897673`*^9, {3.565348647059565*^9, 
   3.5653486509393177`*^9}, 3.567151614301342*^9}]
}, Open  ]],

Cell["\<\
It appears that it should be an equation expressing \[Delta]\[Rho] at leading \
order through \[Delta]\[Rho]2:\
\>", "Text",
 CellChangeTimes->{{3.56485432102437*^9, 3.564854351039482*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]\[Rho]tranrepl", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "6"}], " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               SqrtBox[
                RowBox[{"1", "-", 
                 SuperscriptBox["z", "2"]}]], " ", "\[Delta]\[Rho]2"}], "-", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", "z", " ", 
               RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}], ",", 
           
           FractionBox[
            RowBox[{"\[Delta]\[Rho]", "+", 
             RowBox[{"3", " ", 
              SuperscriptBox["g", "2"], " ", "\[Delta]\[Rho]2"}]}], 
            RowBox[{"g", " ", 
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["z", "2"]}]]}]]}], "}"}], "//", "Total"}], 
        ")"}], "\[Equal]", "0"}], ",", "\[Delta]\[Rho]"}], "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.5648543571740723`*^9, 3.56485438073029*^9}, {
  3.5648545564382057`*^9, 3.564854559228414*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[Delta]\[Rho]", "\[Rule]", 
   RowBox[{"3", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"\[Delta]\[Rho]2", "-", 
      RowBox[{"2", " ", 
       SuperscriptBox["z", "2"], " ", "\[Delta]\[Rho]2"}], "-", 
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.5648543642233686`*^9, 3.564854381152748*^9}, 
   3.564854560094316*^9, 3.565348674941532*^9, 3.567151614570006*^9}]
}, Open  ]],

Cell["\<\
Now, using this equation we use the first the equation coming from \
Log[Y11Y22] (which appear to be nontrivial) to find a condition on \[Delta]\
\[Rho]2\
\>", "Text",
 CellChangeTimes->{{3.564854401295847*^9, 3.564854433156094*^9}}],

Cell["Just a check that PV has a maximal tran order", "Text",
 CellChangeTimes->{{3.564854520905128*^9, 3.564854530664266*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"PV", "[", 
       RowBox[{
        SuperscriptBox["g", "7"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
          RowBox[{
           SuperscriptBox["g", "2"], 
           SubscriptBox["\[Delta]\[Rho]2", "sublead"]}]}], ")"}]}], "]"}], "//.",
       "subPVM"}], "/.", "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], 
   "//.", "subZetaReduce"}], "//", "FindTranOrder"}]], "Input",
 CellChangeTimes->{{3.5648544491166487`*^9, 3.564854511484331*^9}, {
  3.5648548436783*^9, 3.564854849751919*^9}}],

Cell[BoxData["\<\"tran order = 0\"\>"], "Output",
 CellChangeTimes->{{3.564854460254681*^9, 3.564854511774465*^9}, 
   3.564854850579782*^9, 3.56534869117444*^9, 3.567151614737052*^9}]
}, Open  ]],

Cell["Because M does not:", "Text",
 CellChangeTimes->{{3.564854534528101*^9, 3.564854545799724*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["g", "7"], 
      RowBox[{"M", "[", 
       SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "/.", 
     "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "//.", 
   "subZetaReduce"}], "//", "FindTranOrder"}]], "Input",
 CellChangeTimes->{3.5648545426704063`*^9}],

Cell[BoxData["\<\"tran order = -1\"\>"], "Output",
 CellChangeTimes->{3.564854543059558*^9, 3.565348694239216*^9, 
  3.567151614793873*^9}]
}, Open  ]],

Cell["So, we proceed with writing down the equation", "Text",
 CellChangeTimes->{{3.564854567577476*^9, 3.564854576062129*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Y11Y22 equation", "Subsubsection",
 CellChangeTimes->{{3.5648547599209423`*^9, 3.5648547655270157`*^9}}],

Cell["\<\
It is crucial to compare this equation with the exact one. I have to check if \
factors and signs of two are correct.\
\>", "Text",
 CellChangeTimes->{{3.5648571745373287`*^9, 3.564857221190023*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Delta]\[Rho]tranrepl", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "/.", 
        RowBox[{
         RowBox[{"Rule", "[", 
          RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
         RowBox[{"a", "-", "b"}]}]}], "/.", 
       RowBox[{"\[Delta]\[Rho]", "\[Rule]", 
        RowBox[{
         SuperscriptBox["g", "9"], 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["\[Delta]\[Rho]", "lead"], "+", 
           RowBox[{
            SuperscriptBox["g", "2"], 
            SubscriptBox["\[Delta]\[Rho]", "sublead"]}], "+", 
           RowBox[{
            SuperscriptBox["g", "4"], 
            SubscriptBox["\[Delta]\[Rho]", "sublead2"]}]}], ")"}]}]}]}], "/.", 
      RowBox[{"\[Delta]\[Rho]2", "\[Rule]", 
       RowBox[{
        SuperscriptBox["g", "7"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
          RowBox[{
           SuperscriptBox["g", "2"], 
           SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
          RowBox[{
           SuperscriptBox["g", "4"], 
           SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}]}], "//.",
      "subPVM"}], "/.", "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"FindHighTranCoef", "[", 
    RowBox[{
     RowBox[{"Expand", "[", "%", "]"}], "/.", "subtotran"}], "]"}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], ",", "canlist"}], "}"}], "//", 
    "Flatten"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"FindTranOrder", "[", "%", "]"}]}], "Input",
 CellChangeTimes->{{3.56485723408994*^9, 3.5648573999046793`*^9}, {
  3.565348914888913*^9, 3.565348967978374*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox["g", "9"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"1968", " ", 
          SubsuperscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "2", "3"]}], "35"]}], "-", 
       RowBox[{"144", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3", "2"]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "11"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["4608", "175"], "-", 
          FractionBox[
           RowBox[{"133632", " ", 
            SuperscriptBox["z", "2"]}], "35"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "4"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"3456", "+", 
          RowBox[{"13824", " ", 
           SuperscriptBox["z", "2"]}]}], ")"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "576"}], "-", 
          RowBox[{"5760", " ", 
           SuperscriptBox["z", "2"]}]}], ")"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], 
         RowBox[{"2", ",", "6"}]]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "13"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["77184", "175"], "+", 
          FractionBox[
           RowBox[{"84939264", " ", 
            SuperscriptBox["z", "2"]}], "1925"], "+", 
          FractionBox[
           RowBox[{"34587648", " ", 
            SuperscriptBox["z", "4"]}], "1925"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "5"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "11952"}], "-", 
          RowBox[{"64512", " ", 
           SuperscriptBox["z", "2"]}], "-", 
          RowBox[{"48384", " ", 
           SuperscriptBox["z", "4"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5", "2"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "25200"}], "-", 
          RowBox[{"144576", " ", 
           SuperscriptBox["z", "2"]}], "-", 
          RowBox[{"76032", " ", 
           SuperscriptBox["z", "4"]}]}], ")"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"3528", "+", 
          RowBox[{"52416", " ", 
           SuperscriptBox["z", "2"]}], "+", 
          RowBox[{"24192", " ", 
           SuperscriptBox["z", "4"]}]}], ")"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], 
         RowBox[{"2", ",", "8"}]]}]}], ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564857232449811*^9, 3.564857238870983*^9}, {
   3.564857292299838*^9, 3.564857364951377*^9}, 3.564857400340891*^9, 
   3.5653488783167562`*^9, {3.5653489306764193`*^9, 3.565348968338853*^9}, 
   3.565350196545413*^9, 3.567151619599478*^9}],

Cell[BoxData["\<\"tran order = -3\"\>"], "Output",
 CellChangeTimes->{{3.564857232449811*^9, 3.564857238870983*^9}, {
   3.564857292299838*^9, 3.564857364951377*^9}, 3.564857400340891*^9, 
   3.5653488783167562`*^9, {3.5653489306764193`*^9, 3.565348968338853*^9}, 
   3.565350196545413*^9, 3.567151619607114*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"2", " ", "\[Delta]\[Rho]"}], ",", 
             RowBox[{"-", 
              FractionBox[
               RowBox[{"6", " ", 
                SuperscriptBox["g", "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"\[ImaginaryI]", " ", 
                   RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], " ", 
                   StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", "1"}], "+", 
                    SuperscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "2"]}], ")"}]}], "+", 
                  RowBox[{"2", " ", 
                   RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", "1"}], "+", 
                    SuperscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}], "+", 
                  RowBox[{"\[Delta]\[Rho]2", " ", 
                   RowBox[{"(", 
                    RowBox[{"1", "+", 
                    SuperscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], "4"]}], ")"}]}]}], 
                 ")"}]}], 
               SuperscriptBox[
                StyleBox["x",
                 FontColor->RGBColor[0, 0.6, 0]], "2"]]}]}], "}"}], "/.", 
           RowBox[{
            RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], "\[Rule]", "0"}]}], "/.",
           "subJuk"}], "/.", 
         RowBox[{"u", "\[Rule]", 
          RowBox[{"2", " ", "g", " ", "z"}]}]}], " ", "//", "FullSimplify"}], 
       ")"}], "/.", 
      RowBox[{"\[Delta]\[Rho]", "\[Rule]", 
       RowBox[{
        SuperscriptBox["g", "9"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Delta]\[Rho]", "lead"], "+", 
          RowBox[{
           SuperscriptBox["g", "2"], 
           SubscriptBox["\[Delta]\[Rho]", "sublead"]}], "+", 
          RowBox[{
           SuperscriptBox["g", "4"], 
           SubscriptBox["\[Delta]\[Rho]", "sublead2"]}]}], ")"}]}]}]}], "/.", 
     
     RowBox[{"\[Delta]\[Rho]2", "\[Rule]", 
      RowBox[{
       SuperscriptBox["g", "7"], 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
         RowBox[{
          SuperscriptBox["g", "2"], 
          SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "4"], 
          SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}]}], "//.",
     "subPVM"}], ",", "g"}], "]"}]], "Input",
 CellChangeTimes->{{3.5648575031289473`*^9, 3.564857592007863*^9}, 
   3.5653490095235786`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["g", "9"], " ", 
     SubscriptBox["\[Delta]\[Rho]", "lead"]}], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["g", "11"], " ", 
     SubscriptBox["\[Delta]\[Rho]", "sublead"]}], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["g", "13"], " ", 
     SubscriptBox["\[Delta]\[Rho]", "sublead2"]}]}], ",", 
   RowBox[{
    RowBox[{"12", " ", 
     SuperscriptBox["g", "9"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", "z", " ", 
        SqrtBox[
         RowBox[{"1", "-", 
          SuperscriptBox["z", "2"]}]], " ", 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "+", 
       SubscriptBox["\[Delta]\[Rho]2", "lead"], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["z", "2"], " ", 
        SubscriptBox["\[Delta]\[Rho]2", "lead"]}]}], ")"}]}], "+", 
    RowBox[{"12", " ", 
     SuperscriptBox["g", "11"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", "z", " ", 
        SqrtBox[
         RowBox[{"1", "-", 
          SuperscriptBox["z", "2"]}]], " ", 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "sublead"], "]"}]}], "+", 
       SubscriptBox["\[Delta]\[Rho]2", "sublead"], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["z", "2"], " ", 
        SubscriptBox["\[Delta]\[Rho]2", "sublead"]}]}], ")"}]}], "+", 
    RowBox[{"12", " ", 
     SuperscriptBox["g", "13"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", "z", " ", 
        SqrtBox[
         RowBox[{"1", "-", 
          SuperscriptBox["z", "2"]}]], " ", 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "]"}]}], "+", 
       SubscriptBox["\[Delta]\[Rho]2", "sublead2"], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["z", "2"], " ", 
        SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.564857525228627*^9, 3.564857593646661*^9}, 
   3.565349014105842*^9, 3.5653501999821167`*^9, 3.567151623240766*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eqFromY11Y22", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.564857486471468*^9, 3.564857487258236*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "9"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Delta]\[Rho]", "lead"], "+", 
       RowBox[{"6", " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["z", "2"]}]}], ")"}], " ", 
        SubscriptBox["\[Delta]\[Rho]2", "lead"]}]}], ")"}]}], "+", 
    RowBox[{
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     RowBox[{"(", 
      RowBox[{"2268", "+", 
       RowBox[{"12", " ", 
        RowBox[{"M", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "-", 
       RowBox[{"48", " ", "\[ImaginaryI]", " ", "z", " ", 
        RowBox[{"PV", "[", 
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "]"}]}], "-", 
       RowBox[{"\[DoubleStruckCapitalC]", "[", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", 
          SubscriptBox[
           StyleBox["H",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "lead"]}], 
         SuperscriptBox["u", "2"]], "]"}], "+", 
       RowBox[{"\[DoubleStruckCapitalC]", "[", 
        FractionBox[
         RowBox[{"12", " ", "\[ImaginaryI]", " ", 
          SubscriptBox[
           StyleBox["W",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "lead"]}], 
         RowBox[{"1", "+", 
          SuperscriptBox["u", "2"]}]], "]"}], "-", 
       RowBox[{"1512", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2"]}], "+", 
       FractionBox[
        RowBox[{"432", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "2", "2"]}], "5"], "-", 
       RowBox[{"1512", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"1008", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{
        FractionBox["288", "5"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], ")"}]}]], "Output",\

 CellChangeTimes->{3.5648574876673727`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"meqn", "=", 
  RowBox[{
   RowBox[{"Hold", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"2", " ", "\[Delta]\[Rho]"}], "/.", "\[Delta]\[Rho]tranrepl"}],
       ",", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"+", "\[Delta]\[Rho]2contrtran"}], "/.", 
          RowBox[{
           RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], "\[Rule]", "0"}]}], "/.",
          "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"2", " ", "g", " ", "z"}]}]}], " ", "//", "FullSimplify"}], 
      ",", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"+", "W1contrtran"}], "/.", 
          RowBox[{
           RowBox[{"M", "[", "\[Delta]\[Rho]2", "]"}], "\[Rule]", "0"}]}], "/.",
          "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"2", " ", "g", " ", "z"}]}]}], " ", "//", "FullSimplify"}], 
      ",", 
      RowBox[{"+", "\[Delta]W0tran"}], ",", 
      RowBox[{"+", "T10q12pq12mtran"}], ",", 
      RowBox[{"-", "rhsYtYtran"}]}], "}"}], "]"}], "//", 
   "ReleaseHold"}]}]], "Input",
 CellChangeTimes->{{3.564854771215823*^9, 3.564854792660245*^9}, {
  3.564857776494656*^9, 3.564857808743099*^9}, {3.564857877886047*^9, 
  3.5648579007035923`*^9}, {3.565349095174528*^9, 3.565349101186109*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"6", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"\[Delta]\[Rho]2", "-", 
      RowBox[{"2", " ", 
       SuperscriptBox["z", "2"], " ", "\[Delta]\[Rho]2"}], "-", 
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}], ",", 
   RowBox[{"12", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"\[Delta]\[Rho]2", "-", 
      RowBox[{"2", " ", 
       SuperscriptBox["z", "2"], " ", "\[Delta]\[Rho]2"}], "-", 
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", 
       RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"-", "36"}], " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "2", 
          RowBox[{"[", "2", "]"}]], "-", 
         RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "3", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], 
           StyleBox["^",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]]], "3", 
          RowBox[{"[", "2", "]"}]], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "4", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"-", "6"}], " ", "g", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]], "+", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"144", " ", "g", " ", 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], " ", 
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             RowBox[{
              SubscriptBox["z", "1"], " ", 
              SubscriptBox["z", "2"]}]]}], "-", 
           FractionBox[
            RowBox[{"144", " ", "g", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
        FractionBox["1", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]]}]], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
        
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "2"], ",", "0", ",", 
          SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "6"}], " ", "\[ImaginaryI]", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "3"]}], ")"}]}]], "+", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "3"]}], ")"}]}]]}], ")"}], " ", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"12", " ", "\[ImaginaryI]", " ", 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "3"]}], "]"}], " ", 
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             RowBox[{
              SubscriptBox["z", "1"], " ", 
              SubscriptBox["z", "3"]}]]}], "-", 
           FractionBox[
            RowBox[{"12", " ", "\[ImaginaryI]", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "3"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
        FractionBox["1", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            SuperscriptBox["\[ExponentialE]", 
             SubscriptBox["z", "1"]]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            SuperscriptBox["\[ExponentialE]", 
             SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
        
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
       "]"}]}], ")"}]}], ",", 
   RowBox[{"36", " ", 
    SuperscriptBox["g", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]]}], "-", 
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], "+", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "4", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], ",", 
   RowBox[{
    FractionBox["1", "z"], 
    RowBox[{"36", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "2"], " ", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", 
             " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]]}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"8", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "2"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           SubscriptBox["z", "1"]], "+", 
          FractionBox[
           RowBox[{"12", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "3"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           SubscriptBox["z", "1"]], "+", 
          FractionBox[
           RowBox[{"4", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "4"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           SubscriptBox["z", "1"]]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.565350288189114*^9, 3.56535225248839*^9, 
  3.565353700225051*^9, 3.5671516506661253`*^9}]
}, Open  ]],

Cell["\<\
I have to expand this to the leading order in g and to see what it gives\
\>", "Text",
 CellChangeTimes->{{3.5653491354277067`*^9, 3.5653491574152107`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"meqn2", "=", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0", ",", "0"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.565349257739168*^9, 3.565349259940959*^9}, {
  3.56535030604137*^9, 3.565350309872613*^9}, {3.565350435538546*^9, 
  3.565350436457901*^9}, {3.565350858848201*^9, 3.565350858991674*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.565350310419037*^9, 3.5653504367795486`*^9, 
  3.56535085927267*^9, 3.5653522606993113`*^9, 3.567151656019223*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"meqn2", "[", 
   RowBox[{"[", "1", "]"}], "]"}], "=", 
  RowBox[{
   RowBox[{"meqn", "[", 
    RowBox[{"[", "1", "]"}], "]"}], "+", " ", 
   RowBox[{"meqn", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.565350311203156*^9, 3.565350338070533*^9}}],

Cell[BoxData[
 RowBox[{"18", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"\[Delta]\[Rho]2", "-", 
    RowBox[{"2", " ", 
     SuperscriptBox["z", "2"], " ", "\[Delta]\[Rho]2"}], "-", 
    RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5653503278959217`*^9, 3.565350338533195*^9}, 
   3.5653504376956167`*^9, 3.565350860223012*^9, {3.565351237278391*^9, 
   3.5653512398108683`*^9}, 3.565352261893866*^9, 3.565353700569105*^9, 
   3.567151658519865*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"meqn", "[", 
     RowBox[{"[", "3", "]"}], "]"}], "+", 
    RowBox[{"meqn", "[", 
     RowBox[{"[", "5", "]"}], "]"}]}], ",", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], ",", "Simplify"}], "]"}]], "Input",
 CellChangeTimes->{{3.565351308370072*^9, 3.565351308787786*^9}}],

Cell[BoxData[
 RowBox[{"72", " ", "\[ImaginaryI]", " ", 
  SuperscriptBox["g", "2"], " ", 
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", "2", "]"}]]}], "-", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "3", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "4", 
      RowBox[{"[", "2", "]"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.565351309129743*^9, 3.565352263005279*^9, 
  3.5653537013394136`*^9, 3.5671516604494534`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"meqn2", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{
        RowBox[{"Collect", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"meqn", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "+", 
           RowBox[{"meqn", "[", 
            RowBox[{"[", "5", "]"}], "]"}]}], ",", 
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], ",", "Simplify"}], "]"}], "/.", 
        RowBox[{"sub\[Eta]h\[Eta]", "[", "13", "]"}]}], "]"}], "/.", 
      RowBox[{"TaylorOptions", "[", 
       RowBox[{
        RowBox[{"Variable", "\[Rule]", "g"}], ",", 
        RowBox[{"\[Eta]Factor", "\[Rule]", 
         RowBox[{"2", " ", "z"}]}], ",", 
        RowBox[{"Order", "\[Rule]", "13"}], ",", 
        RowBox[{"Series", "\[Rule]", 
         RowBox[{"{", "}"}]}]}], "]"}]}], "//.", "subZetaReduce"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SqrtBox["_"], ",", "canlist"}], "}"}], "//", "Flatten"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.565349160015704*^9, 3.5653492123801823`*^9}, {
  3.565349261166614*^9, 3.5653492641602573`*^9}, {3.5653493746900063`*^9, 
  3.565349387613566*^9}, {3.565350299820612*^9, 3.565350302522623*^9}, {
  3.5653503606780643`*^9, 3.565350364669085*^9}, {3.565350402958096*^9, 
  3.565350408409767*^9}, {3.565351296120117*^9, 3.5653513583150873`*^9}, {
  3.565353838296541*^9, 3.56535384206037*^9}, {3.567151702781535*^9, 
  3.5671517057013893`*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["288", "5"]}], " ", 
     SuperscriptBox["g", "9"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "2"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{
     SuperscriptBox["g", "11"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["4608", "35"], "+", 
          FractionBox[
           RowBox[{"9216", " ", 
            SuperscriptBox["z", "2"]}], "7"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["864", "5"], "-", 
          FractionBox[
           RowBox[{"6912", " ", 
            SuperscriptBox["z", "2"]}], "5"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "13"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["6912", "25"]}], "-", 
          FractionBox[
           RowBox[{"165888", " ", 
            SuperscriptBox["z", "2"]}], "25"], "-", 
          FractionBox[
           RowBox[{"55296", " ", 
            SuperscriptBox["z", "4"]}], "5"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "4"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["13824", "35"]}], "+", 
          FractionBox[
           RowBox[{"27648", " ", 
            SuperscriptBox["z", "2"]}], "35"], "-", 
          FractionBox[
           RowBox[{"73728", " ", 
            SuperscriptBox["z", "4"]}], "7"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["2592", "5"]}], "+", 
          FractionBox[
           RowBox[{"31104", " ", 
            SuperscriptBox["z", "2"]}], "5"], "+", 
          RowBox[{"41472", " ", 
           SuperscriptBox["z", "4"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}]}], ")"}]}]], "Output",\

 CellChangeTimes->{{3.565349162147056*^9, 3.565349212793848*^9}, {
   3.565349253425393*^9, 3.5653492644254837`*^9}, {3.565349377235085*^9, 
   3.5653493897554398`*^9}, {3.5653502949187098`*^9, 3.565350303096684*^9}, {
   3.565350361966674*^9, 3.5653503654367332`*^9}, {3.565350403813027*^9, 
   3.565350408602426*^9}, 3.565350441143585*^9, 3.565350869962388*^9, {
   3.565351303492834*^9, 3.565351358553438*^9}, 3.5653522652559958`*^9, 
   3.565353703866683*^9, 3.567151662473117*^9, {3.567151706835202*^9, 
   3.567151715025379*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"meqn", "[", 
   RowBox[{"[", "4", "]"}], "]"}], "+", 
  RowBox[{"meqn", "[", 
   RowBox[{"[", "6", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.565352012593285*^9, 3.565352025010234*^9}, 
   3.565352270509411*^9, 3.565353717292739*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "6"}], " ", "g", " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]], "+", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{"144", " ", "g", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "2"]}]]}], "-", 
          FractionBox[
           RowBox[{"144", " ", "g", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}], "+", 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "6"}], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]], "+", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]]}], ")"}], " ", "g", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{"12", " ", "\[ImaginaryI]", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "3"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            RowBox[{
             SubscriptBox["z", "1"], " ", 
             SubscriptBox["z", "3"]}]]}], "-", 
          FractionBox[
           RowBox[{"12", " ", "\[ImaginaryI]", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "3"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], " ", 
            SubscriptBox["z", "3"]}]]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "1"]]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
      "]"}]}], ")"}]}], "+", 
  RowBox[{
   FractionBox["1", "z"], 
   RowBox[{"36", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "2"], " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]]}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"8", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "2"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]], "+", 
         FractionBox[
          RowBox[{"12", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "3"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]], "+", 
         FractionBox[
          RowBox[{"4", " ", "g", " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}], " ", 
           RowBox[{
            SuperscriptBox[
             SubscriptBox["J", "4"], "\[Prime]",
             MultilineFunction->None], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          SubscriptBox["z", "1"]]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{{3.565352009711006*^9, 3.565352025352776*^9}, 
   3.565352270898738*^9, 3.565353717649328*^9, 3.5671517164838533`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Expand", "[", 
     RowBox[{
      RowBox[{"meqn", "[", 
       RowBox[{"[", "4", "]"}], "]"}], "+", " ", 
      RowBox[{"meqn", "[", 
       RowBox[{"[", "6", "]"}], "]"}]}], "]"}], "//.", 
    RowBox[{
     RowBox[{"aa1_.", " ", 
      RowBox[{"TC", "[", 
       RowBox[{"bb1_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{"aa1", " ", "bb1"}], ",", "ms"}], "]"}]}]}], "//.", 
   RowBox[{
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{"bb1_", ",", "ms__"}], "]"}], "+", 
     RowBox[{"TC", "[", 
      RowBox[{"bb2_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"bb1", "+", "bb2"}], "//", "Simplify"}], ",", "ms"}], 
     "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"%", "/.", 
      RowBox[{
       RowBox[{"TC", "[", 
        RowBox[{"a_", ",", "c__"}], "]"}], ":>", 
       RowBox[{"TC", "[", 
        RowBox[{
         RowBox[{"Normal", "[", 
          RowBox[{"Series", "[", 
           RowBox[{
            RowBox[{"a", " ", "/.", "subJBes"}], ",", 
            RowBox[{"{", 
             RowBox[{"g", ",", "0", ",", "13"}], "}"}]}], "]"}], "]"}], ",", 
         "c"}], "]"}]}]}], "/.", 
     RowBox[{
      RowBox[{"TC", "[", 
       RowBox[{"a_", ",", "b__", ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["z", "2"], ",", "0", ",", 
          SubscriptBox["z", "1"]}], "}"}]}], "]"}], "\[RuleDelayed]", 
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{"Integrate", "[", 
         RowBox[{"a", ",", 
          RowBox[{"{", 
           RowBox[{
            SubscriptBox["z", "2"], ",", "0", ",", 
            SubscriptBox["z", "1"]}], "}"}]}], "]"}], ",", "b"}], "]"}]}]}], "/.", 
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{"a_", ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "1"]]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
      "]"}], "\[RuleDelayed]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"ExpaCol", "[", 
        RowBox[{
         RowBox[{"a", "  ", 
          SubscriptBox["z", "1"], 
          SubscriptBox["z", "3"]}], ",", 
         SubscriptBox["z", "_"]}], "]"}], "/.", 
       RowBox[{
        RowBox[{
         SubsuperscriptBox["z", "1", "aa_."], 
         SubsuperscriptBox["z", "3", "bb_."]}], "\[RuleDelayed]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Zeta]$", "[", "aa", "]"}], "[", "0", "]"}], 
          RowBox[{
           RowBox[{"\[Zeta]$", "[", "bb", "]"}], "[", "0", "]"}], 
          RowBox[{"Gamma", "[", "aa", "]"}], 
          RowBox[{"Gamma", "[", "bb", "]"}]}], ")"}]}]}], ")"}]}]}], "/.", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{"a_", ",", 
      FractionBox["1", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "i_"]]}], ")"}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "i_"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
     "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"ExpaCol", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "  ", 
          SubscriptBox["z", "i"]}], "//", "Expand"}], ",", 
        SubscriptBox["z", "i"]}], "]"}], "/.", 
      RowBox[{
       SubsuperscriptBox["z", "i", "aa_."], "\[RuleDelayed]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]$", "[", "aa", "]"}], "[", "0", "]"}], 
         RowBox[{"Gamma", "[", "aa", "]"}]}], ")"}]}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"meqn2", "[", 
   RowBox[{"[", "3", "]"}], "]"}], "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"%", "//.", "subZetaReduce"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], ",", "canlist"}], "}"}], "//", 
     "Flatten"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", "/.", "subZetas"}], "/.", 
    RowBox[{"z", "\[Rule]", 
     RowBox[{"11", "/", "10"}]}]}], "/.", 
   RowBox[{"g", "\[Rule]", 
    RowBox[{"7", "/", "10"}]}]}], "//", "N"}]}], "Input",
 CellChangeTimes->{{3.565352348207491*^9, 3.565352448410915*^9}, {
   3.565352493376377*^9, 3.565352644174608*^9}, {3.5653526796138697`*^9, 
   3.56535274864561*^9}, 3.565352812342071*^9, {3.5653528775505238`*^9, 
   3.5653531845010366`*^9}, {3.565353272406559*^9, 3.565353275427176*^9}, 
   3.5653537562189713`*^9, {3.565353851952093*^9, 3.5653538777560797`*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox["g", "9"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["288", "5"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{"3024", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "11"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["4608", "35"]}], "-", 
          FractionBox[
           RowBox[{"9216", " ", 
            SuperscriptBox["z", "2"]}], "7"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["864", "5"]}], "+", 
          FractionBox[
           RowBox[{"6912", " ", 
            SuperscriptBox["z", "2"]}], "5"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"85536", "+", 
          RowBox[{"20736", " ", 
           SuperscriptBox["z", "2"]}]}], ")"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "13"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["6912", "25"], "+", 
          FractionBox[
           RowBox[{"165888", " ", 
            SuperscriptBox["z", "2"]}], "25"], "+", 
          FractionBox[
           RowBox[{"55296", " ", 
            SuperscriptBox["z", "4"]}], "5"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "4"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["13824", "35"], "-", 
          FractionBox[
           RowBox[{"27648", " ", 
            SuperscriptBox["z", "2"]}], "35"], "+", 
          FractionBox[
           RowBox[{"73728", " ", 
            SuperscriptBox["z", "4"]}], "7"]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["2592", "5"], "-", 
          FractionBox[
           RowBox[{"31104", " ", 
            SuperscriptBox["z", "2"]}], "5"], "-", 
          RowBox[{"41472", " ", 
           SuperscriptBox["z", "4"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2", "2"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1675872"}], "-", 
          RowBox[{"760320", " ", 
           SuperscriptBox["z", "2"]}], "-", 
          RowBox[{"126720", " ", 
           SuperscriptBox["z", "4"]}]}], ")"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}], ")"}]}]], "Output",\

 CellChangeTimes->{
  3.565353098621133*^9, {3.5653531290748777`*^9, 3.56535318620163*^9}, {
   3.565353260991041*^9, 3.5653532784939613`*^9}, 3.565353760612146*^9, {
   3.565353858180273*^9, 3.565353880139658*^9}, 3.567151722097373*^9}],

Cell[BoxData[
 RowBox[{"0.`", "\[VeryThinSpace]", "-", 
  RowBox[{"11008.205783689713`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{
  3.565353098621133*^9, {3.5653531290748777`*^9, 3.56535318620163*^9}, {
   3.565353260991041*^9, 3.5653532784939613`*^9}, 3.565353760612146*^9, {
   3.565353858180273*^9, 3.565353880139658*^9}, 3.567151722106071*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"numerical", " ", "check"}], " ", "*)"}], 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Expand", "[", 
       RowBox[{
        RowBox[{"meqn", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "+", 
        RowBox[{"meqn", "[", 
         RowBox[{"[", "6", "]"}], "]"}]}], "]"}], "//.", 
      RowBox[{
       RowBox[{"aa1_.", " ", 
        RowBox[{"TC", "[", 
         RowBox[{"bb1_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
       RowBox[{"TC", "[", 
        RowBox[{
         RowBox[{"aa1", " ", "bb1"}], ",", "ms"}], "]"}]}]}], "//.", 
     RowBox[{
      RowBox[{
       RowBox[{"TC", "[", 
        RowBox[{"bb1_", ",", "ms__"}], "]"}], "+", 
       RowBox[{"TC", "[", 
        RowBox[{"bb2_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"bb1", "+", "bb2"}], "//", "Simplify"}], ",", "ms"}], 
       "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"%", "/.", 
         RowBox[{
          RowBox[{"TC", "[", 
           RowBox[{"a_", ",", "c__"}], "]"}], ":>", 
          RowBox[{"TC", "[", 
           RowBox[{
            RowBox[{"Normal", "[", 
             RowBox[{"Series", "[", 
              RowBox[{
               RowBox[{"a", " ", "/.", "subJBes"}], ",", 
               RowBox[{"{", 
                RowBox[{"g", ",", "0", ",", "13"}], "}"}]}], "]"}], "]"}], 
            ",", "c"}], "]"}]}]}], "/.", 
        RowBox[{
         SubscriptBox["z", "i_"], "\[RuleDelayed]", 
         SubscriptBox["r", "i"]}]}], "/.", 
       RowBox[{
        RowBox[{"TC", "[", 
         RowBox[{"a_", ",", "b_", ",", "m__"}], "]"}], "\[RuleDelayed]", 
        RowBox[{"TC", "[", 
         RowBox[{
          RowBox[{"a", " ", "b"}], ",", "m"}], "]"}]}]}], "/.", 
      RowBox[{"z", "\[Rule]", 
       RowBox[{"11", "/", "10"}]}]}], "/.", 
     RowBox[{"g", "\[Rule]", 
      RowBox[{"7", "/", "10"}]}]}], "/.", 
    RowBox[{"TC", "\[Rule]", "NIntegrate"}]}]}]}]], "Input",
 CellChangeTimes->{{3.565353199081335*^9, 3.565353268357216*^9}, 
   3.565353764233356*^9}],

Cell[BoxData[
 RowBox[{"0.`", "\[VeryThinSpace]", "-", 
  RowBox[{"11008.205801238684`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.565353206324349*^9, 3.565353269391375*^9}, 
   3.5653537648747253`*^9, 3.567151724971942*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"meqn2", ".", 
         RowBox[{"{", 
          RowBox[{"ee", ",", "1", ",", "1"}], "}"}]}], "/.", 
        RowBox[{"\[Delta]\[Rho]2", "\[Rule]", 
         RowBox[{
          SuperscriptBox["g", "7"], 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
            RowBox[{
             SuperscriptBox["g", "2"], 
             SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
            RowBox[{
             SuperscriptBox["g", "4"], 
             SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}]}]}], "//.",
        "subPVM"}], "/.", "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}],
     ",", "canlist", ",", "Simplify"}], "]"}], "//", 
  "FindHighTranCoef"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{"ee", "\[Rule]", "1"}]}], "//", "Expand"}]}], "Input",
 CellChangeTimes->{{3.565350932218803*^9, 3.565351072857478*^9}, {
  3.5653511713900547`*^9, 3.565351176365395*^9}, {3.5653533099341307`*^9, 
  3.565353314580174*^9}, {3.565353363117799*^9, 3.5653533836276903`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "3024"}], " ", 
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{"3024", " ", "ee", " ", 
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{"85536", " ", 
   SuperscriptBox["g", "11"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"85536", " ", "ee", " ", 
   SuperscriptBox["g", "11"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"20736", " ", 
   SuperscriptBox["g", "11"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"20736", " ", "ee", " ", 
   SuperscriptBox["g", "11"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"1675872", " ", 
   SuperscriptBox["g", "13"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"1675872", " ", "ee", " ", 
   SuperscriptBox["g", "13"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"760320", " ", 
   SuperscriptBox["g", "13"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"760320", " ", "ee", " ", 
   SuperscriptBox["g", "13"], " ", 
   SuperscriptBox["z", "2"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"126720", " ", 
   SuperscriptBox["g", "13"], " ", 
   SuperscriptBox["z", "4"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"126720", " ", "ee", " ", 
   SuperscriptBox["g", "13"], " ", 
   SuperscriptBox["z", "4"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}]}]], "Output",
 CellChangeTimes->{{3.565350935767001*^9, 3.5653510819536333`*^9}, 
   3.5653511768160477`*^9, {3.565353305005422*^9, 3.5653533148017263`*^9}, 
   3.56535338512572*^9, 3.56535376899506*^9, 3.567151729524868*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.565350935767001*^9, 3.5653510819536333`*^9}, 
   3.5653511768160477`*^9, {3.565353305005422*^9, 3.5653533148017263`*^9}, 
   3.56535338512572*^9, 3.56535376899506*^9, 3.567151729559663*^9}]
}, Open  ]],

Cell["\<\
This checks the transcendentality equation at first three orders.\
\>", "Text",
 CellChangeTimes->{{3.565353397706354*^9, 3.5653534462024317`*^9}, 
   3.56535409297401*^9}],

Cell["\<\
I will now simplify inhomogeneous part. It is clear that it can be simplified\
\
\>", "Text",
 CellChangeTimes->{{3.565354115514956*^9, 3.5653541274255323`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], "_"], 
     StyleBox["^",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]]], "3", 
    RowBox[{"[", 
     RowBox[{"-", "2"}], "]"}]], "/.", "subtoBes2"}]}]], "Input",
 CellChangeTimes->{{3.565354234129177*^9, 3.5653542506853523`*^9}, {
  3.5653542896638327`*^9, 3.565354296480438*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    RowBox[{
     SubscriptBox["J", "3"], "[", 
     RowBox[{"2", " ", "g", " ", 
      SubscriptBox["z", "3"]}], "]"}]}], 
   SubscriptBox["z", "3"]]}]], "Output",
 CellChangeTimes->{{3.565354237803451*^9, 3.5653542507959347`*^9}, {
   3.565354289907827*^9, 3.565354296806356*^9}, 3.567151740103279*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"%", "//.", 
    RowBox[{
     RowBox[{"aa1_.", " ", 
      RowBox[{"TC", "[", 
       RowBox[{"bb1_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{"aa1", " ", "bb1"}], ",", "ms"}], "]"}]}]}], "//.", 
   RowBox[{
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{"bb1_", ",", "ms__"}], "]"}], "+", 
     RowBox[{"TC", "[", 
      RowBox[{"bb2_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"bb1", "+", "bb2"}], "//", "Simplify"}], ",", "ms"}], 
     "]"}]}]}], "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    RowBox[{
     SubscriptBox["J", "3"], "[", 
     RowBox[{"2", " ", "g", " ", 
      SubscriptBox["z", "3"]}], "]"}]}], 
   SubscriptBox["z", "3"]]}]], "Output",
 CellChangeTimes->{3.5671517416066713`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"meqn", "[", 
      RowBox[{"[", 
       RowBox[{"3", ";;", 
        RowBox[{"-", "1"}]}], "]"}], "]"}]}], "//", "Total"}], "//", 
   "Expand"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{
    RowBox[{"xxx", ":", 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "_", 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "_", 
       RowBox[{"[", "2", "]"}]]}]}], "\[RuleDelayed]", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], 
         RowBox[{"(", 
          RowBox[{"#", "+", 
           RowBox[{"(", 
            RowBox[{"#", "/.", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                SubscriptBox["z", "1"], "\[Rule]", 
                SubscriptBox["z", "3"]}], ",", 
               RowBox[{
                SubscriptBox["z", "3"], "\[Rule]", 
                SubscriptBox["z", "1"]}]}], "}"}]}], ")"}]}], ")"}]}], "&"}], 
       "[", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"\[ImaginaryI]", " ", "2", "g", " ", "z", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]], "xxx"}], "/.", 
         "subtoBes1"}], ")"}], "]"}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "3"]]}], ")"}]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
     "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"rhs", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"%", "//.", 
       RowBox[{
        RowBox[{"aa1_.", " ", 
         RowBox[{"TC", "[", 
          RowBox[{"bb1_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
        RowBox[{"TC", "[", 
         RowBox[{
          RowBox[{"aa1", " ", "bb1"}], ",", "ms"}], "]"}]}]}], "//.", 
      RowBox[{
       RowBox[{
        RowBox[{"TC", "[", 
         RowBox[{"bb1_", ",", "ms__"}], "]"}], "+", 
        RowBox[{"TC", "[", 
         RowBox[{"bb2_", ",", "ms__"}], "]"}]}], "\[RuleDelayed]", 
       RowBox[{"TC", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"bb1", "+", "bb2"}], "//", "Simplify"}], ",", "ms"}], 
        "]"}]}]}], "/.", 
     RowBox[{
      RowBox[{"TC", "[", 
       RowBox[{"0", ",", "__"}], "]"}], "\[Rule]", "0"}]}], "/.", 
    RowBox[{
     RowBox[{"TC", "[", 
      RowBox[{"a_", ",", "ms__"}], "]"}], "\[RuleDelayed]", 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{"Collect", "[", 
        RowBox[{"a", ",", "z", ",", "Simplify"}], "]"}], ",", "ms"}], 
      "]"}]}]}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"4", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}]}], "\[Rule]", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"2", " ", "g", " ", "z", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"4", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}]}], "\[Rule]", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"2", " ", "g", " ", "z", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "]"}]}]}]}], 
    "}"}]}]}]}], "Input",
 CellChangeTimes->{{3.565354127983366*^9, 3.5653544012654543`*^9}, {
  3.565354445335847*^9, 3.56535448459623*^9}, {3.565354994537076*^9, 
  3.565355081850911*^9}, {3.565355288848233*^9, 3.5653552899332237`*^9}, {
  3.565355580043603*^9, 3.5653555815584497`*^9}}],

Cell[BoxData[
 RowBox[{"TC", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"1728", " ", 
       SuperscriptBox["g", "2"], " ", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"2", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
      RowBox[{
       SubscriptBox["z", "1"], " ", 
       SubscriptBox["z", "2"]}]]}], "+", 
    FractionBox[
     RowBox[{"288", " ", 
      SuperscriptBox["g", "3"], " ", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"2", " ", "g", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           SubscriptBox["J", "2"], "\[Prime]",
           MultilineFunction->None], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], "+", 
        RowBox[{"3", " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           SubscriptBox["J", "3"], "\[Prime]",
           MultilineFunction->None], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          SubscriptBox["J", "2"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           SubscriptBox["J", "4"], "\[Prime]",
           MultilineFunction->None], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
     RowBox[{"z", " ", 
      SubscriptBox["z", "1"]}]]}], ",", 
   FractionBox["1", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["\[ExponentialE]", 
      SubscriptBox["z", "1"]]}]], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "2"], ",", "0", ",", 
     SubscriptBox["z", "1"]}], "}"}]}], "]"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.5648965262963877`*^9, {3.564896569636342*^9, 3.564896615389667*^9}, 
   3.565354083856348*^9, {3.56535443131495*^9, 3.5653544852389927`*^9}, 
   3.565355012158764*^9, 3.565355048101018*^9, 3.56535508261283*^9, 
   3.565355290639799*^9, 3.5653555823032417`*^9, 3.567151744367792*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"lhs", "=", 
  RowBox[{"Total", "[", 
   RowBox[{"meqn", "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", "2"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5653552948462753`*^9, 3.565355308050582*^9}}],

Cell[BoxData[
 RowBox[{"18", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"\[Delta]\[Rho]2", "-", 
    RowBox[{"2", " ", 
     SuperscriptBox["z", "2"], " ", "\[Delta]\[Rho]2"}], "-", 
    RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.567151753572538*^9}]
}, Open  ]],

Cell["Now we  use a better way to represent the lhs:", "Text",
 CellChangeTimes->{{3.5653553171823187`*^9, 3.5653553349625063`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"lhs", "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], "\[Rule]", 
          RowBox[{
           FractionBox["1", 
            RowBox[{"2", "\[ImaginaryI]", " "}]], 
           RowBox[{"(", 
            RowBox[{"x", "-", 
             FractionBox["1", "x"]}], ")"}]}]}], ",", 
         RowBox[{"z", "\[Rule]", 
          RowBox[{
           FractionBox["1", "2"], 
           RowBox[{"(", 
            RowBox[{"x", "+", 
             FractionBox["1", "x"]}], ")"}]}]}]}], "}"}]}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}], "\[Rule]", 
         RowBox[{
          FractionBox["1", "2"], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"q$", "[", "2", "]"}], "[", 
             RowBox[{"+", "o"}], "]"}], "+", 
            RowBox[{
             RowBox[{"q$", "[", "2", "]"}], "[", 
             RowBox[{"-", "o"}], "]"}]}], ")"}]}]}], ",", 
        RowBox[{"\[Delta]\[Rho]2", "\[Rule]", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"q$", "[", "2", "]"}], "[", 
           RowBox[{"+", "o"}], "]"}], "-", 
          RowBox[{
           RowBox[{"q$", "[", "2", "]"}], "[", 
           RowBox[{"-", "o"}], "]"}]}]}]}], "}"}]}], "//", "Simplify"}], ",", 
    
    RowBox[{
     RowBox[{"q$", "[", "2", "]"}], "[", "_", "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"x$", "[", "0", "]"}], 
      RowBox[{"-", "2"}]], "\[RuleDelayed]", 
     RowBox[{
      RowBox[{"x$h", "[", 
       RowBox[{"-", "o"}], "]"}], "^", "2"}]}], ",", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"x$", "[", "0", "]"}], 
      RowBox[{"+", "2"}]], "\[RuleDelayed]", 
     RowBox[{
      RowBox[{"x$h", "[", 
       RowBox[{"+", "o"}], "]"}], "^", "2"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5653553389573183`*^9, 3.565355536802597*^9}, {
  3.565356148200794*^9, 3.5653561604071827`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"18", " ", 
   SuperscriptBox["g", "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{"[", 
       RowBox[{"-", "o"}], "]"}]], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], "2", 
    RowBox[{"[", 
     RowBox[{"-", "o"}], "]"}]]}], "-", 
  RowBox[{"18", " ", 
   SuperscriptBox["g", "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{"[", "o", "]"}]], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], "2", 
    RowBox[{"[", "o", "]"}]]}]}]], "Output",
 CellChangeTimes->{{3.565355393045079*^9, 3.565355404714456*^9}, {
   3.565355463065194*^9, 3.56535553763173*^9}, {3.565356153252263*^9, 
   3.5653561607381887`*^9}, 3.567151755780321*^9}]
}, Open  ]],

Cell["So the true equation is", "Text",
 CellChangeTimes->{{3.5653555559525414`*^9, 3.565355558903591*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", 
        RowBox[{"-", "o"}], "]"}]], ")"}], "2"], " ", 
    SubsuperscriptBox["q", "2", 
     RowBox[{"[", 
      RowBox[{"-", "o"}], "]"}]]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", "o", "]"}]], ")"}], "2"], " ", 
    SubsuperscriptBox["q", "2", 
     RowBox[{"[", "o", "]"}]]}]}], "\[Equal]", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"18", 
     SuperscriptBox["g", "2"]}]], "rhs"}], "\[Equal]", " ", 
  RowBox[{"(", 
   RowBox[{"normrhs", "=", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"96", " ", 
          SqrtBox[
           RowBox[{"1", "-", 
            SuperscriptBox["z", "2"]}]], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "g", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "2"]}]]}], "+", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"z", " ", 
          SubscriptBox["z", "1"]}]], 
        RowBox[{"16", " ", "g", " ", 
         SqrtBox[
          RowBox[{"1", "-", 
           SuperscriptBox["z", "2"]}]], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"2", " ", "g", " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "2"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], "+", 
           RowBox[{"3", " ", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "3"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}], " ", 
            RowBox[{
             SuperscriptBox[
              SubscriptBox["J", "4"], "\[Prime]",
              MultilineFunction->None], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}]}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.565355561312065*^9, 3.5653555682340727`*^9}, {
  3.5653556208957243`*^9, 3.565355651751359*^9}, {3.565355689055127*^9, 
  3.565355691379291*^9}, {3.5653561641479177`*^9, 3.565356165380088*^9}}],

Cell[TextData[{
 "In this notation it is clear that there is an ambiguity in solution.",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{
    SubscriptBox["\[Delta]q", "2"], "==", 
    RowBox[{
     FractionBox["AA", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]], 
     RowBox[{"(", 
      RowBox[{
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+", 
       FractionBox["1", 
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}]}]],
  CellChangeTimes->{{3.564897031966495*^9, 3.564897116152637*^9}}],
 " is a zero mode of the equation above."
}], "Text",
 CellChangeTimes->{{3.565355668084835*^9, 3.5653557217563353`*^9}}],

Cell["\<\
This can be further simplified. We will flip derivative in Bessel function \
(no boundary contribution is present, so it is very simple\
\>", "Text",
 CellChangeTimes->{{3.565359061394112*^9, 3.565359110082303*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"normrhs", "=", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"2", " ", "g", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"16", " ", "g", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{
              SubscriptBox["J", "2"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SubscriptBox["J", "2"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
         SubscriptBox["z", "1"]], "-", 
        FractionBox[
         RowBox[{"96", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], "+", 
            RowBox[{
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}], " ", 
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]]}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.56535912314351*^9, 3.565359295152152*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"16", " ", "g", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
      SubscriptBox["z", "1"]], "-", 
     FractionBox[
      RowBox[{"96", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], "+", 
         RowBox[{
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
      RowBox[{
       SubscriptBox["z", "1"], " ", 
       SubscriptBox["z", "2"]}]]}], ")"}], "//", "Expand"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "//.", 
   RowBox[{
    FractionBox[
     RowBox[{
      SubscriptBox["J", "a_"], "[", 
      RowBox[{"2", "g", " ", 
       SubscriptBox["z", "i_"]}], "]"}], 
     SubscriptBox["z", "i_"]], "\[RuleDelayed]", 
    RowBox[{
     FractionBox[
      RowBox[{"2", "g"}], 
      RowBox[{"2", " ", "a"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["J", 
         RowBox[{"a", "+", "1"}]], "[", 
        RowBox[{"2", "g", " ", 
         SubscriptBox["z", "i"]}], "]"}], "+", 
       RowBox[{
        SubscriptBox["J", 
         RowBox[{"a", "-", "1"}]], "[", 
        RowBox[{"2", "g", " ", 
         SubscriptBox["z", "i"]}], "]"}]}], ")"}]}]}]}], "//", 
  "Simplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Collect", "[", 
       RowBox[{"%", ",", 
        RowBox[{
         SubscriptBox["J", "_"], "[", 
         RowBox[{"2", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}]}], "]"}], "/.", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}], "-", 
         RowBox[{
          SubscriptBox["J", "5"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], ")"}], "\[Rule]", 
       RowBox[{"2", 
        RowBox[{
         SuperscriptBox[
          SubscriptBox["J", "4"], "\[Prime]",
          MultilineFunction->None], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}]}]}]}], "//", "Simplify"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        SubscriptBox["J", "4"], "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}], ",", 
      RowBox[{
       SubscriptBox["J", "4"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}]}], " ", "}"}]}], "]"}], "/.", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "1"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}], "-", 
     RowBox[{
      SubscriptBox["J", "5"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}]}], ")"}], "\[Rule]", 
   RowBox[{"2", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        SubscriptBox["J", "2"], "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}], "+", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["J", "4"], "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], ")"}]}]}]}]}], "Input",
 CellChangeTimes->{{3.565360382521234*^9, 3.565360507708996*^9}, {
  3.56536054541722*^9, 3.565360562478661*^9}, {3.5653606404978037`*^9, 
  3.5653607083992662`*^9}, {3.565360745679652*^9, 3.565360769598432*^9}, {
  3.565365834945591*^9, 3.565365880770434*^9}}],

Cell[BoxData[
 RowBox[{"8", " ", 
  SuperscriptBox["g", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["J", "3"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "2"]}], "]"}], " ", 
     RowBox[{
      SubscriptBox["J", "4"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "1"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}], " ", 
     RowBox[{
      SubscriptBox["J", "4"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "2"]}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "4"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "2"]}], "]"}], " ", 
     RowBox[{
      SubscriptBox["J", "5"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "2"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}], "-", 
       RowBox[{
        SubscriptBox["J", "5"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "4"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}], " ", 
     RowBox[{
      SubscriptBox["J", "5"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.565360404653804*^9, 3.56536044370144*^9}, {
   3.565360488584725*^9, 3.56536050972122*^9}, {3.5653605559298573`*^9, 
   3.565360562875346*^9}, {3.565360703896036*^9, 3.565360709123931*^9}, {
   3.565360751668408*^9, 3.565360770252748*^9}, 3.565365624303114*^9, 
   3.56536581995123*^9, {3.5653658548904657`*^9, 3.565365881090522*^9}, 
   3.567151779341379*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"16", " ", 
   SuperscriptBox["g", "2"], " ", 
   RowBox[{
    SubscriptBox["J", "4"], "[", 
    RowBox[{"2", " ", "g", " ", 
     SubscriptBox["z", "2"]}], "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       SubscriptBox["J", "2"], "\[Prime]",
       MultilineFunction->None], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}], "+", 
     RowBox[{
      SuperscriptBox[
       SubscriptBox["J", "4"], "\[Prime]",
       MultilineFunction->None], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}]}], ")"}]}], "+", 
  RowBox[{"16", " ", 
   SuperscriptBox["g", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["J", "2"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}], "+", 
     RowBox[{
      SubscriptBox["J", "4"], "[", 
      RowBox[{"2", " ", "g", " ", 
       SubscriptBox["z", "1"]}], "]"}]}], ")"}], " ", 
   RowBox[{
    SuperscriptBox[
     SubscriptBox["J", "4"], "\[Prime]",
     MultilineFunction->None], "[", 
    RowBox[{"2", " ", "g", " ", 
     SubscriptBox["z", "2"]}], "]"}]}]}]], "Output",
 CellChangeTimes->{{3.565360404653804*^9, 3.56536044370144*^9}, {
   3.565360488584725*^9, 3.56536050972122*^9}, {3.5653605559298573`*^9, 
   3.565360562875346*^9}, {3.565360703896036*^9, 3.565360709123931*^9}, {
   3.565360751668408*^9, 3.565360770252748*^9}, 3.565365624303114*^9, 
   3.56536581995123*^9, {3.5653658548904657`*^9, 3.565365881090522*^9}, 
   3.567151779386664*^9}]
}, Open  ]],

Cell["And finally we get", "Text",
 CellChangeTimes->{{3.5653659276289043`*^9, 3.565365932060094*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"24", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"D", "[", " ", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}], 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}]}], 
        SubscriptBox["z", "1"]], ",", 
       SubscriptBox["z", "1"]}], "]"}], "+", 
     RowBox[{"D", "[", " ", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}], 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}]}], 
        SubscriptBox["z", "1"]], ",", 
       SubscriptBox["z", "2"]}], "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.565365938259709*^9, 3.565365974906588*^9}, {
  3.565366020856194*^9, 3.565366059762259*^9}, {3.565366095468053*^9, 
  3.565366168700164*^9}, {3.565366235190789*^9, 3.565366282990973*^9}}],

Cell[BoxData[
 RowBox[{"24", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       RowBox[{
        SubscriptBox["J", "3"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "1"]}], "]"}], " ", 
       RowBox[{
        SubscriptBox["J", "4"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}]}], 
      SubsuperscriptBox["z", "1", "2"]]}], "+", 
    FractionBox[
     RowBox[{"2", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "4"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["J", "3"], "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     SubscriptBox["z", "1"]], "+", 
    FractionBox[
     RowBox[{"2", " ", "g", " ", 
      RowBox[{
       SubscriptBox["J", "3"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["J", "4"], "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}]}], 
     SubscriptBox["z", "1"]]}], ")"}]}]], "Output",
 CellChangeTimes->{3.5671517822283773`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{"BesselJ", "[", 
       RowBox[{"a", ",", "x"}], "]"}], "x"], "-", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"2", "a"}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"BesselJ", "[", 
         RowBox[{
          RowBox[{"a", "+", "1"}], ",", "x"}], "]"}], "+", 
        RowBox[{"BesselJ", "[", 
         RowBox[{
          RowBox[{"a", "-", "1"}], ",", "x"}], "]"}]}], ")"}]}]}], ",", 
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"BesselJ", "[", 
        RowBox[{"a", ",", "x"}], "]"}], ",", "x"}], "]"}], "-", 
     RowBox[{
      FractionBox["1", "2"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"BesselJ", "[", 
         RowBox[{
          RowBox[{"a", "-", "1"}], ",", "x"}], "]"}], "-", 
        RowBox[{"BesselJ", "[", 
         RowBox[{
          RowBox[{"a", "+", "1"}], ",", "x"}], "]"}]}], ")"}]}]}]}], "}"}], "//",
   "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.565360584454961*^9, 3.565360635614538*^9}}],

Cell["So, equation bcomes", "Text",
 CellChangeTimes->{{3.565366291553376*^9, 3.565366294359735*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", 
        RowBox[{"-", "o"}], "]"}]], ")"}], "2"], " ", 
    SubsuperscriptBox["q", "2", 
     RowBox[{"[", 
      RowBox[{"-", "o"}], "]"}]]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", "o", "]"}]], ")"}], "2"], " ", 
    SubsuperscriptBox["q", "2", 
     RowBox[{"[", "o", "]"}]]}]}], "\[Equal]", 
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{"24", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", "[", " ", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          SubscriptBox["z", "1"]], ",", 
         SubscriptBox["z", "1"]}], "]"}], "+", 
       RowBox[{"D", "[", " ", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          SubscriptBox["z", "1"]], ",", 
         SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
    FractionBox["1", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       SubscriptBox["z", "1"]]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.565366308666971*^9, 3.565366339390527*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"normrhs", "=", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{"24", 
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"2", " ", "g", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", "[", " ", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           SubscriptBox["z", "1"]], ",", 
          SubscriptBox["z", "1"]}], "]"}], "+", 
        RowBox[{"D", "[", " ", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           SubscriptBox["z", "1"]], ",", 
          SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]]}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5653663467363586`*^9, 3.565366350026453*^9}}],

Cell["\<\
Since we have  a significantly simplified equation now, let us recheck that \
it is correct one\
\>", "Text",
 CellChangeTimes->{{3.565355744627665*^9, 3.565355770990035*^9}, {
  3.565356347675116*^9, 3.565356348322446*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"normrhs", "/.", 
  RowBox[{
   RowBox[{"TC", "[", 
    RowBox[{"a_", ",", "b_", ",", "mes__"}], "]"}], "\[RuleDelayed]", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"ExpaCol", "[", 
        RowBox[{
         RowBox[{
          SubscriptBox["z", "1"], 
          SubscriptBox["z", "2"], 
          RowBox[{"Normal", "@", 
           RowBox[{"Series", "[", 
            RowBox[{
             RowBox[{"a", "/.", "subJBes"}], ",", 
             RowBox[{"{", 
              RowBox[{"g", ",", "0", ",", "11"}], "}"}]}], "]"}]}]}], ",", 
         SubscriptBox["z", "_"]}], "]"}], "/.", 
       RowBox[{
        RowBox[{
         SubsuperscriptBox["z", "2", "pow2_."], 
         SubsuperscriptBox["z", "1", "pow1_."]}], "\[RuleDelayed]", 
        FractionBox[
         RowBox[{
          RowBox[{"Gamma", "[", 
           RowBox[{"pow1", "+", "pow2"}], "]"}], 
          RowBox[{
           RowBox[{"\[Zeta]$", "[", 
            RowBox[{"pow1", "+", "pow2"}], "]"}], "[", "0", "]"}]}], 
         "pow2"]}]}], "//.", "subZetaReduce"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SqrtBox["_"], ",", "canlist"}], "}"}], "//", "Flatten"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5653557729113903`*^9, 3.5653560667298183`*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"168", " ", 
     SuperscriptBox["g", "7"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}], "+", 
    RowBox[{
     SuperscriptBox["g", "9"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "4752"}], "-", 
       RowBox[{"1152", " ", 
        SuperscriptBox["z", "2"]}]}], ")"}], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "9"]}], "+", 
    RowBox[{
     SuperscriptBox["g", "11"], " ", 
     RowBox[{"(", 
      RowBox[{"93104", "+", 
       RowBox[{"42240", " ", 
        SuperscriptBox["z", "2"]}], "+", 
       RowBox[{"7040", " ", 
        SuperscriptBox["z", "4"]}]}], ")"}], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5653558009707203`*^9, 3.565355813328887*^9}, {
   3.5653558613174*^9, 3.565355877084549*^9}, {3.565355909163638*^9, 
   3.565355911801379*^9}, {3.565356032986746*^9, 3.565356067405788*^9}, {
   3.565359274270664*^9, 3.565359297629387*^9}, {3.5653664009035883`*^9, 
   3.56536640549445*^9}, 3.567151792807102*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    SuperscriptBox[
                    OverscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], 
                    StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]], 
                    RowBox[{"[", 
                    RowBox[{"-", "o"}], "]"}]], ")"}], "2"], " ", 
                    SubsuperscriptBox["q", "2", 
                    RowBox[{"[", 
                    RowBox[{"-", "o"}], "]"}]]}], "-", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    SuperscriptBox[
                    OverscriptBox[
                    StyleBox["x",
                    FontColor->RGBColor[0, 0.6, 0]], 
                    StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]], 
                    RowBox[{"[", "o", "]"}]], ")"}], "2"], " ", 
                    SubsuperscriptBox["q", "2", 
                    RowBox[{"[", "o", "]"}]]}]}], "/.", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"q$", "[", "2", "]"}], "[", "o", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "2"], "\[Delta]\[Rho]2"}], "+", 
                    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"q$", "[", "2", "]"}], "[", 
                    RowBox[{"-", "o"}], "]"}], "\[RuleDelayed]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", 
                    FractionBox["1", "2"]}], "\[Delta]\[Rho]2"}], "+", 
                    RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}]}], 
                    "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"x$h", "[", "o", "]"}], "\[RuleDelayed]", 
                   RowBox[{"x$", "[", "0", "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"x$h", "[", 
                   RowBox[{"-", "o"}], "]"}], "\[RuleDelayed]", 
                  SuperscriptBox[
                   RowBox[{"x$", "[", "0", "]"}], 
                   RowBox[{"-", "1"}]]}]}], "/.", 
                RowBox[{"\[Delta]\[Rho]2", "\[Rule]", 
                 RowBox[{
                  SuperscriptBox["g", "7"], 
                  RowBox[{"(", 
                   RowBox[{
                    SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
                    RowBox[{
                    SuperscriptBox["g", "2"], 
                    SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
                    RowBox[{
                    SuperscriptBox["g", "4"], 
                    SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], 
                   ")"}]}]}]}], "//.", "subPVM"}], "/.", "sub\[Rho]Ansatz"}], 
             "/.", "solut\[Rho]sexplicit"}], "/.", "subJuk"}], "/.", 
           RowBox[{"u", "\[Rule]", 
            RowBox[{"2", "g", " ", "z"}]}]}], "//.", "subZetaReduce"}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "11"}], "}"}]}], "]"}], "//.", "denz"}],
       ",", "zlist"}], "]"}], "//", "FindHighTranCoef"}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      SqrtBox["_"], ",", "canlist"}], "}"}], "//", "Flatten"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.565356084158267*^9, 3.565356133109725*^9}, {
  3.5653561709773407`*^9, 3.565356264710428*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox[
   RowBox[{"1", "-", 
    SuperscriptBox["z", "2"]}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"168", " ", 
     SuperscriptBox["g", "7"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}], "+", 
    RowBox[{
     SuperscriptBox["g", "9"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "4752"}], "-", 
       RowBox[{"1152", " ", 
        SuperscriptBox["z", "2"]}]}], ")"}], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "9"]}], "+", 
    RowBox[{
     SuperscriptBox["g", "11"], " ", 
     RowBox[{"(", 
      RowBox[{"93104", "+", 
       RowBox[{"42240", " ", 
        SuperscriptBox["z", "2"]}], "+", 
       RowBox[{"7040", " ", 
        SuperscriptBox["z", "4"]}]}], ")"}], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.56535617139983*^9, 3.565356176052552*^9}, {
   3.565356223298635*^9, 3.565356266770506*^9}, 3.5671518010315742`*^9}]
}, Open  ]],

Cell["\<\
We confirm that equation is correct, at least at the leading three orders. \
\>", "Text",
 CellChangeTimes->{{3.5653562768768387`*^9, 3.565356335343175*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", 
        RowBox[{"-", "o"}], "]"}]], ")"}], "2"], " ", 
    SubsuperscriptBox["q", "2", 
     RowBox[{"[", 
      RowBox[{"-", "o"}], "]"}]]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", "o", "]"}]], ")"}], "2"], " ", 
    SubsuperscriptBox["q", "2", 
     RowBox[{"[", "o", "]"}]]}]}], "\[Equal]", 
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{"24", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", "[", " ", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          SubscriptBox["z", "1"]], ",", 
         SubscriptBox["z", "1"]}], "]"}], "+", 
       RowBox[{"D", "[", " ", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "2"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", " ", "g", " ", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          SubscriptBox["z", "1"]], ",", 
         SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
    FractionBox["1", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       SubscriptBox["z", "1"]]}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", 
      SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]], "Input"],

Cell["The equation can be solved using the ansatz.", "Text",
 CellChangeTimes->{{3.565356359144743*^9, 3.565356365208012*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["q", "2", 
   RowBox[{"[", "0", "]"}]], "=", 
  RowBox[{
   RowBox[{
    FractionBox["AA", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]], 
    RowBox[{"(", 
     RowBox[{
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "+", 
      FractionBox["1", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"k", "=", "1"}], "\[Infinity]"], 
    FractionBox[
     RowBox[{"BB", "[", "k", "]"}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{
       RowBox[{"2", "k"}], "+", "1"}]]]}]}]}]], "Input",
 CellChangeTimes->{{3.564897031966495*^9, 3.564897116152637*^9}}],

Cell[TextData[{
 "AA will be fixed from condition that \[GothicCapitalT] is zero at Bethe \
root. But for the rest it is easy to guess solution. We just have to keep all \
",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "x"], TraditionalForm]]],
 " powers. Let us see explicitly. We can use the property"
}], "Text",
 CellChangeTimes->{{3.564897152519067*^9, 3.5648971585123262`*^9}, {
  3.564897200071352*^9, 3.564897287648888*^9}, {3.565359410983245*^9, 
  3.56535944405084*^9}, {3.565367100713262*^9, 3.5653671048891993`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{"\[ImaginaryI]", 
    FractionBox["x", "2"], 
    RowBox[{"(", 
     RowBox[{"t", "+", 
      FractionBox["1", "t"]}], ")"}]}]], "\[Equal]", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", 
     RowBox[{"-", "\[Infinity]"}]}], 
    RowBox[{"+", "\[Infinity]"}]], 
   RowBox[{
    SuperscriptBox["\[ImaginaryI]", "n"], 
    RowBox[{
     SubscriptBox["J", "n"], "[", "x", "]"}], 
    SuperscriptBox["t", "n"]}]}]}]], "Input",
 CellChangeTimes->{{3.565367107839986*^9, 3.56536717120652*^9}, {
  3.565367272697462*^9, 3.565367364909902*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BesselJ", "[", 
  RowBox[{
   RowBox[{"-", "4"}], ",", "e"}], "]"}]], "Input",
 CellChangeTimes->{{3.565367257999901*^9, 3.565367267599564*^9}}],

Cell[BoxData[
 RowBox[{"BesselJ", "[", 
  RowBox[{"4", ",", "e"}], "]"}]], "Output",
 CellChangeTimes->{{3.565367265337742*^9, 3.56536726782092*^9}}]
}, Open  ]],

Cell["Therefore", "Text",
 CellChangeTimes->{{3.565367175168558*^9, 3.565367176096383*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cos", "[", 
   RowBox[{"2", " ", "g", " ", "z", " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["z", "1"], "-", 
      SubscriptBox["z", "2"]}], ")"}]}], "]"}], "\[Equal]", 
  RowBox[{"Cos", "[", 
   RowBox[{"g", 
    RowBox[{"(", 
     RowBox[{"x", "+", 
      FractionBox["1", "x"]}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["z", "1"], "-", 
      SubscriptBox["z", "2"]}], ")"}]}], "]"}], "\[Equal]", 
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         FractionBox["1", "x"]}], ")"}], 
       FractionBox[
        RowBox[{"2", "g", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "2"]}]], "+", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "\[ImaginaryI]"}], 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         FractionBox["1", "x"]}], ")"}], 
       FractionBox[
        RowBox[{"2", "g", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "2"]}]]}], ")"}]}], "\[Equal]", 
  
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"n", "=", 
       RowBox[{"-", "\[Infinity]"}]}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      SuperscriptBox["\[ImaginaryI]", "n"], 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], 
      RowBox[{
       SubscriptBox["J", "n"], "[", 
       RowBox[{"2", "g", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
      SuperscriptBox["x", "n"]}]}], ")"}]}], "==", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", 
     RowBox[{"-", "\[Infinity]"}]}], 
    RowBox[{"+", "\[Infinity]"}]], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], 
    RowBox[{
     SubscriptBox["J", 
      RowBox[{"2", "n"}]], "[", 
     RowBox[{"2", "g", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["z", "1"], "-", 
        SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
    SuperscriptBox["x", 
     RowBox[{"2", "n"}]]}]}]}]], "Input",
 CellChangeTimes->{{3.565367181443878*^9, 3.565367251517654*^9}, {
  3.5653673446910553`*^9, 3.5653674492113533`*^9}}],

Cell[TextData[{
 "I have to multiply this by ",
 Cell[BoxData[
  RowBox[{
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], "\[Rule]", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", "\[ImaginaryI]", " "}]], 
    RowBox[{"(", 
     RowBox[{"x", "-", 
      FractionBox["1", "x"]}], ")"}]}]}]],
  CellChangeTimes->{{3.5653594619721193`*^9, 3.5653594641453342`*^9}}],
 " and keep only negative powers in x"
}], "Text",
 CellChangeTimes->{{3.5653674719096823`*^9, 3.565367493738659*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", "\[ImaginaryI]"}]], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"n", "=", 
       RowBox[{"-", "\[Infinity]"}]}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"], 
      RowBox[{
       SubscriptBox["J", 
        RowBox[{"2", "n"}]], "[", 
       RowBox[{"2", "g", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
      SuperscriptBox["x", 
       RowBox[{"2", "n"}]], 
      RowBox[{"(", 
       RowBox[{"x", "-", 
        FractionBox["1", "x"]}], ")"}]}]}]}], "==", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", "\[ImaginaryI]"}]], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"n", "=", 
       RowBox[{"-", "\[Infinity]"}]}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["J", 
          RowBox[{"2", "n"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], "+", 
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "n"}], "+", "2"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}]}], ")"}], 
      SuperscriptBox["x", 
       RowBox[{
        RowBox[{"2", "n"}], "+", "1"}]]}]}]}]}], "\[Rule]", 
  "\[IndentingNewLine]", 
  RowBox[{"\[Rule]", 
   RowBox[{"(*", " ", 
    RowBox[{"keeping", " ", "only", " ", "negative", " ", "powers"}], " ", 
    "*)"}], " ", 
   RowBox[{
    RowBox[{
     FractionBox["1", 
      RowBox[{"2", "\[ImaginaryI]"}]], 
     RowBox[{
      SubsuperscriptBox["\[Sum]", 
       RowBox[{"n", "=", "1"}], 
       RowBox[{"+", "\[Infinity]"}]], 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["J", 
           RowBox[{"2", "n"}]], "[", 
          RowBox[{"2", "g", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}], "]"}], "+", 
         RowBox[{
          SubscriptBox["J", 
           RowBox[{
            RowBox[{"2", "n"}], "-", "2"}]], "[", 
          RowBox[{"2", "g", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}], "]"}]}], ")"}], 
       SuperscriptBox["x", 
        RowBox[{"1", "-", 
         RowBox[{"2", "n"}]}]]}]}]}], "\[Equal]", "\[IndentingNewLine]", 
    "\[Equal]", 
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], 
     RowBox[{
      SubsuperscriptBox["\[Sum]", 
       RowBox[{"n", "=", "1"}], 
       RowBox[{"+", "\[Infinity]"}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "n"}], "-", "1"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
        RowBox[{"2", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       SuperscriptBox["x", 
        RowBox[{"1", "-", 
         RowBox[{"2", "n"}]}]]}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5653675024913054`*^9, 3.565367653732359*^9}, {
  3.56536775919775*^9, 3.565367790965157*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"subzx", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        SuperscriptBox["z", "2"]}]], "\[Rule]", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"2", "\[ImaginaryI]", " "}]], 
       RowBox[{"(", 
        RowBox[{"x", "-", 
         FractionBox["1", "x"]}], ")"}]}]}], ",", 
     RowBox[{"z", "\[Rule]", 
      RowBox[{
       FractionBox["1", "2"], 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         FractionBox["1", "x"]}], ")"}]}]}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5653594619721193`*^9, 3.5653594641453342`*^9}}],

Cell["\<\
Thee point that capturing only 1/x coefficient is captured by the Bessel \
functions:\
\>", "Text",
 CellChangeTimes->{{3.565366635770733*^9, 3.5653666584697037`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"List", "@@", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Collect", "[", 
        RowBox[{
         RowBox[{"Series", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               SqrtBox[
                RowBox[{"1", "-", 
                 SuperscriptBox["z", "2"]}]], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{"2", " ", "g", " ", "z", " ", "\[Delta]z"}], "]"}]}], 
              "/.", "\[InvisibleSpace]", 
              RowBox[{
               SubscriptBox["z", "i_"], "\[RuleDelayed]", 
               SubscriptBox["e", "i"]}]}], "/.", "\[InvisibleSpace]", 
             "subzx"}], "/.", "\[InvisibleSpace]", 
            RowBox[{"e", "\[Rule]", "z"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"g", ",", "0", ",", "10"}], "}"}]}], "]"}], ",", "x"}], 
        "]"}], "/.", "\[InvisibleSpace]", 
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"a_", "/;", 
          RowBox[{"a", "<", "0"}]}]], "\[Rule]", "0"}]}], "/.", 
      RowBox[{"x", "\[Rule]", 
       RowBox[{"-", 
        FractionBox["1", "x"]}]}]}], ",", "x"}], "]"}]}], ";"}], "\n", 
 RowBox[{"SortBy", "[", 
  RowBox[{"%", ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Exponent", "[", 
      RowBox[{"#1", ",", "x"}], "]"}]}], "&"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "@", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], 
           FractionBox["#", "2"]], 
          FractionBox[
           RowBox[{"(", 
            RowBox[{"#", "-", "1"}], ")"}], 
           SuperscriptBox["x", 
            RowBox[{"#", "-", "1"}]]], 
          FractionBox[
           RowBox[{"BesselJ", "[", 
            RowBox[{
             RowBox[{"#", "-", "1"}], ",", 
             RowBox[{"2", " ", "g", " ", "\[Delta]z"}]}], "]"}], 
           RowBox[{"2", "g", " ", "\[Delta]z"}]]}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "11"}], "}"}]}], "]"}]}], "&"}], "/@", 
     RowBox[{"Range", "[", 
      RowBox[{"2", ",", "13", ",", "2"}], "]"}]}], ")"}]}], "//", 
  "Simplify"}]}], "Input",
 CellChangeTimes->{{3.565358384513098*^9, 3.565358424697413*^9}, {
  3.565358580860248*^9, 3.565358581136766*^9}, {3.565358640972824*^9, 
  3.565358693885962*^9}, {3.565358803534875*^9, 3.565358873273354*^9}, {
  3.565359503664083*^9, 3.565359506696773*^9}, {3.565359550236377*^9, 
  3.565359551709166*^9}, {3.565359627467984*^9, 3.565359682730474*^9}, {
  3.565359716789339*^9, 3.565359722267413*^9}, {3.5653599106355762`*^9, 
  3.56535993705044*^9}, {3.565359984160116*^9, 3.565360067159738*^9}, {
  3.5653667146801233`*^9, 3.565366765670603*^9}, {3.565366892650725*^9, 
  3.565366919085968*^9}, {3.5653677800641727`*^9, 3.565367817792301*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox["\[ImaginaryI]", "2"], "-", 
     RowBox[{
      FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "2"], " ", 
      SuperscriptBox["\[Delta]z", "2"]}], "+", 
     RowBox[{
      FractionBox["1", "24"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox["\[Delta]z", "4"]}], "-", 
     RowBox[{
      FractionBox["1", "288"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "6"], " ", 
      SuperscriptBox["\[Delta]z", "6"]}], "+", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "8"], " ", 
       SuperscriptBox["\[Delta]z", "8"]}], "5760"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "10"], " ", 
       SuperscriptBox["\[Delta]z", "10"]}], "172800"]}], 
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "4"]}], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "2"], " ", 
      SuperscriptBox["\[Delta]z", "2"]}], "+", 
     RowBox[{
      FractionBox["1", "16"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox["\[Delta]z", "4"]}], "-", 
     RowBox[{
      FractionBox["1", "160"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "6"], " ", 
      SuperscriptBox["\[Delta]z", "6"]}], "+", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "8"], " ", 
       SuperscriptBox["\[Delta]z", "8"]}], "2880"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "10"], " ", 
       SuperscriptBox["\[Delta]z", "10"]}], "80640"]}], 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "3"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{
      FractionBox["1", "48"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox["\[Delta]z", "4"]}], "-", 
     RowBox[{
      FractionBox["1", "288"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "6"], " ", 
      SuperscriptBox["\[Delta]z", "6"]}], "+", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "8"], " ", 
       SuperscriptBox["\[Delta]z", "8"]}], "4032"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "10"], " ", 
       SuperscriptBox["\[Delta]z", "10"]}], "96768"]}], 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "5"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["g", "6"], " ", 
        SuperscriptBox["\[Delta]z", "6"]}], "1440"]}], "+", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "8"], " ", 
       SuperscriptBox["\[Delta]z", "8"]}], "11520"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "10"], " ", 
       SuperscriptBox["\[Delta]z", "10"]}], "207360"]}], 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "7"]], ",", 
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "8"], " ", 
       SuperscriptBox["\[Delta]z", "8"]}], "80640"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["g", "10"], " ", 
       SuperscriptBox["\[Delta]z", "10"]}], "806400"]}], 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "9"]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", "10"], " ", 
      SuperscriptBox["\[Delta]z", "10"]}], 
     RowBox[{"7257600", " ", 
      SuperscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], "11"]}]]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5653599126787033`*^9, 3.565359937575354*^9}, {
   3.565359992591631*^9, 3.565360067607115*^9}, 3.565366659506879*^9, {
   3.565366744239102*^9, 3.5653667663597317`*^9}, {3.565366889038492*^9, 
   3.565366919764917*^9}, {3.5653678061158323`*^9, 3.565367818294629*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.5653599126787033`*^9, 3.565359937575354*^9}, {
   3.565359992591631*^9, 3.565360067607115*^9}, 3.565366659506879*^9, {
   3.565366744239102*^9, 3.5653667663597317`*^9}, {3.565366889038492*^9, 
   3.565366919764917*^9}, {3.5653678061158323`*^9, 3.565367818304509*^9}}]
}, Open  ]],

Cell["\<\
We got an explicit expression. However I am not completelly happy with it:\
\>", "Text",
 CellChangeTimes->{{3.5653686650871153`*^9, 3.565368677982997*^9}}],

Cell[CellGroupData[{

Cell[BoxData["normrhs"], "Input",
 CellChangeTimes->{{3.565368720349082*^9, 3.565368722588868*^9}, {
  3.565368804091968*^9, 3.565368818178121*^9}}],

Cell[BoxData[
 RowBox[{"TC", "[", 
  RowBox[{
   RowBox[{"24", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"2", " ", "g", " ", "z", " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["z", "1"], "-", 
        SubscriptBox["z", "2"]}], ")"}]}], "]"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], 
        SubsuperscriptBox["z", "1", "2"]]}], "+", 
      FractionBox[
       RowBox[{"2", " ", "g", " ", 
        RowBox[{
         SubscriptBox["J", "4"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          SubscriptBox["J", "3"], "\[Prime]",
          MultilineFunction->None], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}]}], 
       SubscriptBox["z", "1"]], "+", 
      FractionBox[
       RowBox[{"2", " ", "g", " ", 
        RowBox[{
         SubscriptBox["J", "3"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}], " ", 
        RowBox[{
         SuperscriptBox[
          SubscriptBox["J", "4"], "\[Prime]",
          MultilineFunction->None], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}]}], 
       SubscriptBox["z", "1"]]}], ")"}]}], ",", 
   FractionBox["1", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["\[ExponentialE]", 
      SubscriptBox["z", "1"]]}]], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "2"], ",", "0", ",", 
     SubscriptBox["z", "1"]}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5653688184781437`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  SubscriptBox["q", "2"], "\[Equal]", 
  RowBox[{
   RowBox[{
    FractionBox["AA", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]], 
    RowBox[{"(", 
     RowBox[{
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "+", 
      FractionBox["1", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"n", "=", "1"}], 
     RowBox[{"+", "\[Infinity]"}]], 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{
        RowBox[{"2", "n"}], "+", "1"}]]], 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{"24", " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "n"}], "-", "1"}], ")"}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "n"], 
        FractionBox[
         RowBox[{
          SubscriptBox["J", 
           RowBox[{
            RowBox[{"2", "n"}], "-", "1"}]], "[", 
          RowBox[{"2", "g", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
         RowBox[{"2", "g", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "1"]}], "]"}], "+", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.56536872923306*^9, 3.56536879828448*^9}, {
  3.5653688284027433`*^9, 3.565368840735611*^9}, {3.565368875546954*^9, 
  3.5653688828591957`*^9}}],

Cell["\<\
I tried to make some integrals explicitly, but I don\[CloseCurlyQuote]t find \
the result particularly useful. So I continue with as it is\
\>", "Text",
 CellChangeTimes->{{3.5653717587883167`*^9, 3.565371794489212*^9}}],

Cell["\<\
The coefficients of the rhs can be expanded at weak coupling. Orders of Zetas \
follow simply.\
\>", "Text",
 CellChangeTimes->{{3.565368896368884*^9, 3.565368906213977*^9}, {
  3.56537182172784*^9, 3.565371846582901*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"TC", "[", 
        RowBox[{
         RowBox[{"24", " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", "n"}], "-", "1"}], ")"}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "n"], 
          FractionBox[
           RowBox[{
            SubscriptBox["J", 
             RowBox[{
              RowBox[{"2", "n"}], "-", "1"}]], "[", 
            RowBox[{"2", "g", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
           RowBox[{"2", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"D", "[", " ", 
             RowBox[{
              FractionBox[
               RowBox[{
                RowBox[{
                 SubscriptBox["J", "4"], "[", 
                 RowBox[{"2", " ", "g", " ", 
                  SubscriptBox["z", "2"]}], "]"}], 
                RowBox[{
                 SubscriptBox["J", "3"], "[", 
                 RowBox[{"2", " ", "g", " ", 
                  SubscriptBox["z", "1"]}], "]"}]}], 
               SubscriptBox["z", "1"]], ",", 
              SubscriptBox["z", "1"]}], "]"}], "+", 
            RowBox[{"D", "[", " ", 
             RowBox[{
              FractionBox[
               RowBox[{
                RowBox[{
                 SubscriptBox["J", "4"], "[", 
                 RowBox[{"2", " ", "g", " ", 
                  SubscriptBox["z", "2"]}], "]"}], 
                RowBox[{
                 SubscriptBox["J", "3"], "[", 
                 RowBox[{"2", " ", "g", " ", 
                  SubscriptBox["z", "1"]}], "]"}]}], 
               SubscriptBox["z", "1"]], ",", 
              SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
         FractionBox["1", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["z", "1"]]}]], ",", 
         RowBox[{"{", 
          RowBox[{
           SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           SubscriptBox["z", "2"], ",", "0", ",", 
           SubscriptBox["z", "1"]}], "}"}]}], "]"}], "/.", 
       RowBox[{
        RowBox[{"TC", "[", 
         RowBox[{"a_", ",", "b__"}], "]"}], "\[RuleDelayed]", 
        RowBox[{"ExpaCol", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ExpaCol", "[", 
            RowBox[{
             RowBox[{
              SubscriptBox["z", "1"], 
              SubscriptBox["z", "2"], 
              RowBox[{"Normal", "[", 
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"a", "/.", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    SubscriptBox["z", "1"], "-", 
                    SubscriptBox["z", "2"]}], ")"}], "\[Rule]", 
                    "\[Delta]z"}]}], "/.", "subJBes"}], "/.", 
                  RowBox[{"n", "\[Rule]", "#"}]}], ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "15"}], "}"}]}], "]"}], "]"}]}],
              ",", 
             RowBox[{"\[Delta]z", "|", 
              SubscriptBox["z", "_"]}]}], "]"}], "/.", 
           RowBox[{"\[Delta]z", "\[Rule]", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "2"]}], ")"}]}]}], ",", 
          SubscriptBox["z", "_"]}], "]"}]}]}], "/.", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["z", "1", "pow1_."], 
        SubsuperscriptBox["z", "2", "pow2_."]}], "\[RuleDelayed]", 
       RowBox[{
        FractionBox["1", "pow2"], 
        RowBox[{"Gamma", "[", 
         RowBox[{"pow1", "+", "pow2"}], "]"}], 
        RowBox[{
         RowBox[{"\[Zeta]$", "[", 
          RowBox[{"pow1", "+", "pow2"}], "]"}], "[", "0", "]"}]}]}]}], ")"}], 
    "&"}], "/@", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "5", ",", "1"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"coefsexplicit", "=", "%"}]}], "Input",
 CellChangeTimes->{{3.5653689251113043`*^9, 3.565369128304697*^9}, {
   3.5653691787931423`*^9, 3.565369233444036*^9}, {3.565370290816119*^9, 
   3.565370343657102*^9}, 3.565370410105071*^9, {3.565371808291224*^9, 
   3.565371816714569*^9}, {3.565371913671728*^9, 3.565371916815036*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", "84"}], " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "7"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}], "+", 
    RowBox[{"2520", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "9"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "9"]}], "-", 
    RowBox[{"52272", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "11"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "11"]}], "+", 
    RowBox[{"936936", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "13"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "13"]}], "-", 
    RowBox[{"15615600", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "15"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "15"]}]}], ",", 
   RowBox[{
    RowBox[{"144", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "9"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "9"]}], "-", 
    RowBox[{"5940", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "11"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "11"]}], "+", 
    RowBox[{"156156", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "13"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "13"]}], "-", 
    RowBox[{"3363360", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "15"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "15"]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "220"}], " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "11"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "11"]}], "+", 
    RowBox[{"12012", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "13"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "13"]}], "-", 
    RowBox[{"393120", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "15"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "15"]}]}], ",", 
   RowBox[{
    RowBox[{"312", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "13"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "13"]}], "-", 
    RowBox[{"21840", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["g", "15"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "15"]}]}], ",", 
   RowBox[{
    RowBox[{"-", "420"}], " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "15"], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "15"]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.565370412946992*^9, {3.56537180541367*^9, 3.565371816923143*^9}, 
   3.56537191725196*^9, 3.567151868644964*^9}]
}, Open  ]],

Cell["\<\
Now we are about to fix AA. We first have to solve homogeneous equation\
\>", "Text",
 CellChangeTimes->{{3.565371851773801*^9, 3.565371884829322*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubsuperscriptBox["q", "2", "+"], "[", "v0", "]"}], "-", 
   RowBox[{
    SubsuperscriptBox["q", "2", "-"], "[", "v0", "]"}]}], "\[Equal]", 
  "0"}]], "Input",
 CellChangeTimes->{{3.565371892239809*^9, 3.565371903103022*^9}}],

Cell["\<\
Let us first see what this equation means for the sum\
\>", "Text",
 CellChangeTimes->{{3.565371922524598*^9, 3.565371936971373*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{
      RowBox[{"2", 
       RowBox[{"Range", "[", 
        RowBox[{"Length", "[", "coefsexplicit", "]"}], "]"}]}], "+", "1"}]]], 
   ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{
          RowBox[{"2", "#"}], "+", "5"}]], 
        RowBox[{"c", "[", "#", "]"}]}], ")"}], "&"}], "/@", 
     RowBox[{"Range", "[", 
      RowBox[{"Length", "[", "coefsexplicit", "]"}], "]"}]}], ")"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"%", "/.", 
        RowBox[{"Shift", "[", "1", "]"}]}], ")"}], "-", 
      RowBox[{"(", 
       RowBox[{"%", "/.", 
        RowBox[{"Shift", "[", 
         RowBox[{"-", "1"}], "]"}]}], ")"}]}], "/.", "subJuk"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}], "//", 
  "Simplify"}]}], "Input",
 CellChangeTimes->{{3.565371943005643*^9, 3.565372159371599*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"16", " ", "\[ImaginaryI]", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"12", " ", 
         SuperscriptBox["u", "2"]}]}], ")"}], " ", 
      RowBox[{"c", "[", "1", "]"}], " ", 
      SuperscriptBox["g", "10"]}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["u", "2"]}]}], ")"}], "3"]]}], "+", 
   RowBox[{"96", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], "+", 
           RowBox[{"2", " ", "u"}]}], ")"}], "5"]]}], "+", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[ImaginaryI]", "+", 
          RowBox[{"2", " ", "u"}]}], ")"}], "5"]]}], ")"}], " ", 
    RowBox[{"c", "[", "1", "]"}], " ", 
    SuperscriptBox["g", "12"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "13"],
    SeriesData[$CellContext`g, 0, {}, 10, 13, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   Complex[0, -16] (1 + 4 $CellContext`u^2)^(-3) (-1 + 
     12 $CellContext`u^2) $CellContext`c[1], 0, 
    96 ((-1) (Complex[0, -1] + 2 $CellContext`u)^(-5) + (Complex[0, 1] + 
       2 $CellContext`u)^(-5)) $CellContext`c[1]}, 10, 13, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.565371970613965*^9, 3.5653720789083853`*^9}, {
   3.565372132935088*^9, 3.565372159865059*^9}, 3.5671519287087917`*^9}]
}, Open  ]],

Cell["\<\
An amazing thing is that the leading coefficient is zero. Do we want match \
subleading guy also? Let us take a look on transcendentality\
\>", "Text",
 CellChangeTimes->{{3.56537216788482*^9, 3.565372208921102*^9}, {
  3.565372335606389*^9, 3.5653723419165087`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{
      RowBox[{"2", 
       RowBox[{"Range", "[", 
        RowBox[{"Length", "[", "coefsexplicit", "]"}], "]"}]}], "+", "1"}]]], 
   ".", 
   RowBox[{"(", "coefsexplicit", ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"%", "/.", 
         RowBox[{"Shift", "[", "1", "]"}]}], ")"}], "-", 
       RowBox[{"(", 
        RowBox[{"%", "/.", 
         RowBox[{"Shift", "[", 
          RowBox[{"-", "1"}], "]"}]}], ")"}]}], "/.", "subJuk"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}], "//", "Simplify"}], 
  "//", "Normal"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"%", "/.", 
       RowBox[{"u", "\[Rule]", 
        SubscriptBox["v", "0"]}]}], "/.", 
      RowBox[{"leadweak", "[", "7", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}], ",", 
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"%", "/.", 
       RowBox[{"u", "\[Rule]", 
        SubscriptBox["v", "0"]}]}], "/.", 
      RowBox[{"leadweak", "[", "2", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.565372345121312*^9, 3.565372408708333*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"1344", " ", 
     SuperscriptBox["g", "10"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       RowBox[{"12", " ", 
        SuperscriptBox["u", "2"]}]}], ")"}], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["u", "2"]}]}], ")"}], "3"]]}], "+", 
  FractionBox[
   RowBox[{"8064", " ", 
    SuperscriptBox["g", "12"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"40", " ", 
          SuperscriptBox["u", "2"]}], "+", 
         RowBox[{"80", " ", 
          SuperscriptBox["u", "4"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"4", " ", 
           SuperscriptBox["u", "2"]}]}], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"12", " ", 
          SuperscriptBox["u", "2"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"4", " ", 
       SuperscriptBox["u", "2"]}]}], ")"}], "5"]]}]], "Output",
 CellChangeTimes->{{3.5653723482498198`*^9, 3.5653724089048433`*^9}, 
   3.567151932184905*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   InterpretationBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"2268", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"], " ", 
       SuperscriptBox["g", "12"]}]}], "+", 
     InterpretationBox[
      SuperscriptBox[
       RowBox[{"O", "[", "g", "]"}], "13"],
      SeriesData[$CellContext`g, 0, {}, 12, 13, 1],
      Editable->False]}],
    SeriesData[$CellContext`g, 0, {(-2268) $CellContext`\[Zeta]$[7][0]}, 12, 
     13, 1],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"2268", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"], " ", 
       SuperscriptBox["g", "12"]}]}], "+", 
     InterpretationBox[
      SuperscriptBox[
       RowBox[{"O", "[", "g", "]"}], "13"],
      SeriesData[$CellContext`g, 0, {}, 12, 13, 1],
      Editable->False]}],
    SeriesData[$CellContext`g, 0, {(-2268) $CellContext`\[Zeta]$[7][0]}, 12, 
     13, 1],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.5653723482498198`*^9, 3.5653724089048433`*^9}, 
   3.5671519322400913`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   FractionBox["AA", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "2"]], 
   RowBox[{"(", 
    RowBox[{
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "+", 
     FractionBox["1", 
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"%", "/.", 
          RowBox[{"Shift", "[", "1", "]"}]}], ")"}], "-", 
        RowBox[{"(", 
         RowBox[{"%", "/.", 
          RowBox[{"Shift", "[", 
           RowBox[{"-", "1"}], "]"}]}], ")"}]}], "/.", "subJuk"}], "/.", 
      RowBox[{"u", "\[Rule]", 
       SubscriptBox["v", "0"]}]}], "/.", 
     RowBox[{"leadweak", "[", "7", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", "0", ",", "6"}], "}"}]}], "]"}], "//", 
  "Simplify"}]}], "Input",
 CellChangeTimes->{{3.565372231463822*^9, 3.565372258615026*^9}, {
  3.5653724255138702`*^9, 3.565372429417286*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"3", " ", "\[ImaginaryI]", " ", "AA", " ", "g"}]}], "+", 
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "AA", " ", 
    SuperscriptBox["g", "3"]}], "-", 
   RowBox[{"111", " ", "\[ImaginaryI]", " ", "AA", " ", 
    SuperscriptBox["g", "5"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "7"],
    SeriesData[$CellContext`g, 0, {}, 1, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   Complex[0, -3] $CellContext`AA, 0, Complex[0, 12] $CellContext`AA, 0, 
    Complex[0, -111] $CellContext`AA}, 1, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.565372236661545*^9, 3.565372258825478*^9}, {
   3.565372413795245*^9, 3.565372430564817*^9}, 3.567151951586322*^9}]
}, Open  ]],

Cell["\<\
If we want to match subleading guy, then we get an equation:\
\>", "Text",
 CellChangeTimes->{{3.565372308518341*^9, 3.5653723227409678`*^9}, {
  3.565372441984901*^9, 3.565372445512456*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", "2268"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"], " ", 
      SuperscriptBox["g", "12"]}], "==", 
     RowBox[{
      RowBox[{"-", "3"}], " ", "\[ImaginaryI]", " ", "AA", " ", "g"}]}], ",", 
    "AA"}], "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5653724496042547`*^9, 3.565372459130499*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"AA", "\[Rule]", 
   RowBox[{
    RowBox[{"-", "756"}], " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "11"], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "7"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.565372459517529*^9}]
}, Open  ]],

Cell[TextData[{
 "Solution has too high transcendentality deficit. So, at the order we are \
interested in, it is clear that AA==0. However, we should also check with \
normalization coming from T-functions. I know that they of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " deficit. So AA will be ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "1"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". And this is inapropriate."
}], "Text",
 CellChangeTimes->{{3.56537247468118*^9, 3.5653725617399197`*^9}}],

Cell[TextData[{
 "The conclusion is that leading transcendentality correction to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " is"
}], "Text",
 CellChangeTimes->{{3.565372578347043*^9, 3.565372598857444*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["q", "2"], "==", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", "1"}], 
    RowBox[{"+", "\[Infinity]"}]], 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{
       RowBox[{"2", "n"}], "+", "1"}]]], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "n"}], "-", "1"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
        RowBox[{"2", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "1"]}], "]"}], "+", 
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.565372606797667*^9, 3.565372608441592*^9}}],

Cell[TextData[{
 "The leading correction for this expression starts at ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["g", "10"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". It will be sensible only at 9 loops. That is what I get"
}], "Text",
 CellChangeTimes->{{3.565372776806045*^9, 3.565372812498637*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["q", 
     RowBox[{"2", ",", "\"\<corr\>\""}]], "/.", 
    RowBox[{"nsweak", "[", "1", "]"}]}], "/.", "tranpowers"}], "//", 
  "FindTranOrder"}]], "Input",
 CellChangeTimes->{{3.565372816174213*^9, 3.565372832922081*^9}}],

Cell[BoxData["\<\"tran order = -1\"\>"], "Output",
 CellChangeTimes->{{3.5653728142227297`*^9, 3.565372833167323*^9}}]
}, Open  ]],

Cell["\<\
But I can compute correction to the density, which I know explicitly. So \
there are things to check.\
\>", "Text",
 CellChangeTimes->{{3.565372839426977*^9, 3.565372895343256*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{
      RowBox[{"2", 
       RowBox[{"Range", "[", 
        RowBox[{"Length", "[", "coefsexplicit", "]"}], "]"}]}], "+", "1"}]]], 
   ".", 
   RowBox[{"(", "coefsexplicit", ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{"x$h", "\[Rule]", "x$"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"%", "+", 
          RowBox[{"(", 
           RowBox[{"%", "/.", "mconj"}], ")"}]}], "/.", "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"2", " ", "g", " ", "z"}]}]}], ",", "\[Zeta]list", ",", 
       "FullSimplify"}], "]"}], "/.", 
     RowBox[{
      SqrtBox[
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["g", "2"]}], "-", 
        RowBox[{"4", " ", 
         SuperscriptBox["g", "2"], " ", 
         SuperscriptBox["z", "2"]}]}]], "\[Rule]", 
      RowBox[{"2", "g", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]]}]}]}], "//.", "denz"}], ",", "zlist"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.5653729274445972`*^9, 3.565373065606792*^9}, {
  3.565373176389835*^9, 3.565373203174775*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "7"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"168", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"672", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4752"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "+", 
     RowBox[{"16704", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "+", 
     RowBox[{"4608", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"93104", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "-", 
     RowBox[{"286176", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "-", 
     RowBox[{"154880", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "-", 
     RowBox[{"28160", " ", 
      SuperscriptBox["z", "6"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "13"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "1584960"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "+", 
     RowBox[{"4299360", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "+", 
     RowBox[{"3224832", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "+", 
     RowBox[{"1257984", " ", 
      SuperscriptBox["z", "6"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "+", 
     RowBox[{"159744", " ", 
      SuperscriptBox["z", "8"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "15"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"25247880", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "-", 
     RowBox[{"61377120", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "-", 
     RowBox[{"54741120", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "-", 
     RowBox[{"32256000", " ", 
      SuperscriptBox["z", "6"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "-", 
     RowBox[{"9246720", " ", 
      SuperscriptBox["z", "8"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "-", 
     RowBox[{"860160", " ", 
      SuperscriptBox["z", "10"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.565372921774778*^9, {3.56537295852311*^9, 3.5653730660490522`*^9}, {
   3.565373183402257*^9, 3.5653732119769*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["g", "7"], 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
         RowBox[{
          SuperscriptBox["g", "2"], 
          SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "4"], 
          SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}]}], ")"}]}], "/.", 
      "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "//", 
    "FindHighTranCoef"}], ",", "zlist"}], "]"}]], "Input",
 CellChangeTimes->{{3.565373283830283*^9, 3.5653733038786087`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "7"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"168", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"672", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4752"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "+", 
     RowBox[{"16704", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "+", 
     RowBox[{"4608", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"93104", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "-", 
     RowBox[{"286176", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "-", 
     RowBox[{"154880", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "-", 
     RowBox[{"28160", " ", 
      SuperscriptBox["z", "6"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.5653732916706038`*^9, 3.565373304382491*^9}}]
}, Open  ]],

Cell["\<\
Therefore we have a strong confirmaiton that proposed expression is correct.\
\>", "Text",
 CellChangeTimes->{{3.5653733891566963`*^9, 3.565373401900182*^9}}],

Cell["\<\
Let us also write correction to \[Delta]\[Rho]\
\>", "Text",
 CellChangeTimes->{{3.5653734058601017`*^9, 3.56537341646777*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[Delta]\[Rho]", "\[Rule]", 
   RowBox[{
    RowBox[{"3", " ", 
     SuperscriptBox["g", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"\[Delta]\[Rho]2", "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["z", "2"], " ", "\[Delta]\[Rho]2"}], "-", 
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "z", " ", 
        SqrtBox[
         RowBox[{"1", "-", 
          SuperscriptBox["z", "2"]}]], " ", 
        RowBox[{"PV", "[", "\[Delta]\[Rho]2", "]"}]}]}], ")"}]}], "==", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], 
         RowBox[{"[", 
          RowBox[{"-", "o"}], "]"}]], ")"}], "2"], " ", 
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], "2", 
       RowBox[{"[", 
        RowBox[{"-", "o"}], "]"}]]}], "-", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], 
         RowBox[{"[", "o", "]"}]], ")"}], "2"], " ", 
      SubsuperscriptBox[
       StyleBox["q",
        FontColor->RGBColor[0, 0, 0.8]], "2", 
       RowBox[{"[", "o", "]"}]]}]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.5653736911763163`*^9, 3.5653736973474617`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["q", 
   RowBox[{"1", ",", "right"}]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", "1"}], 
    RowBox[{"+", "\[Infinity]"}]], 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"-", "3"}], 
      SuperscriptBox["g", "2"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{
       RowBox[{"2", "n"}], "-", "1"}]]], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "n"}], "-", "1"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
        RowBox[{"2", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "1"]}], "]"}], "+", 
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.565373425149219*^9, 3.5653734454730043`*^9}, {
  3.565373538444022*^9, 3.5653735454709063`*^9}, {3.56537370845957*^9, 
  3.565373710257246*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", "3"}], 
     SuperscriptBox["g", "2"]}], 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{
      RowBox[{"2", 
       RowBox[{"Range", "[", 
        RowBox[{"Length", "[", "coefsexplicit", "]"}], "]"}]}], "-", "1"}]]], 
   ".", 
   RowBox[{"(", "coefsexplicit", ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{"x$h", "\[Rule]", "x$"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Collect", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"%", "+", 
          RowBox[{"(", 
           RowBox[{"%", "/.", "mconj"}], ")"}]}], "/.", "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"2", " ", "g", " ", "z"}]}]}], ",", "\[Zeta]list", ",", 
       "FullSimplify"}], "]"}], "/.", 
     RowBox[{
      SqrtBox[
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["g", "2"]}], "-", 
        RowBox[{"4", " ", 
         SuperscriptBox["g", "2"], " ", 
         SuperscriptBox["z", "2"]}]}]], "\[Rule]", 
      RowBox[{"2", "g", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]]}]}]}], "//.", "denz"}], ",", "zlist"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.565373462085297*^9, 3.565373465733021*^9}, 
   3.565373554701839*^9, 3.565373715879033*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"504", " ", 
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "14256"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "-", 
     RowBox[{"3456", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "13"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"279312", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "+", 
     RowBox[{"126720", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "+", 
     RowBox[{"21120", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "15"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4754880"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "-", 
     RowBox[{"2927808", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "-", 
     RowBox[{"1003392", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}], "-", 
     RowBox[{"119808", " ", 
      SuperscriptBox["z", "6"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "13"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "17"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"75743640", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "+", 
     RowBox[{"55460160", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "+", 
     RowBox[{"27861120", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "+", 
     RowBox[{"7257600", " ", 
      SuperscriptBox["z", "6"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}], "+", 
     RowBox[{"645120", " ", 
      SuperscriptBox["z", "8"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "15"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.565373467432946*^9, 3.565373556192376*^9, 
  3.565373717236259*^9, 3.56715217891138*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["g", "9"], 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Delta]\[Rho]", "lead"], "+", 
         RowBox[{
          SuperscriptBox["g", "2"], 
          SubscriptBox["\[Delta]\[Rho]", "sublead"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "4"], 
          SubscriptBox["\[Delta]\[Rho]", "sublead2"]}]}], ")"}]}], "/.", 
      "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "//", 
    "FindHighTranCoef"}], ",", "zlist"}], "]"}]], "Input",
 CellChangeTimes->{{3.565373576197866*^9, 3.565373583874531*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"504", " ", 
   SuperscriptBox["g", "9"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "11"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "14256"}], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "-", 
     RowBox[{"3456", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "13"], " ", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"279312", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "+", 
     RowBox[{"126720", " ", 
      SuperscriptBox["z", "2"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}], "+", 
     RowBox[{"21120", " ", 
      SuperscriptBox["z", "4"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.565373584381184*^9, 3.565373718682521*^9, 
  3.567152178980022*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Transcendentality structure of BAE", "Subsubsection",
 CellChangeTimes->{{3.56440733273466*^9, 3.5644073397162037`*^9}}],

Cell["Let us see what we expect based on the answer", "Text",
 CellChangeTimes->{{3.564407350581602*^9, 3.564407357323111*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"subtotran", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"xxx", ":", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
          "\[Eta]$", "|", "\[Eta]$b", "|", "\[Eta]0$", "|", "\[Eta]0$b", "|", 
           "\[Zeta]$"}], ")"}], "[", "a__", "]"}], "[", "_", "]"}]}], 
      "\[RuleDelayed]", 
      RowBox[{
       SuperscriptBox["tran", 
        RowBox[{"Total", "[", 
         RowBox[{"{", "a", "}"}], "]"}]], "xxx"}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"\[ScriptU]$", "[", "a_", "]"}], "[", "r_", "]"}], 
      "\[RuleDelayed]", 
      FractionBox[
       RowBox[{
        RowBox[{"\[ScriptU]$", "[", "a", "]"}], "[", "r", "]"}], "tran"]}], 
     ",", 
     RowBox[{"g", "\[Rule]", 
      FractionBox["g", "tran"]}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.56440753934317*^9, 3.5644075587954197`*^9}, {
  3.56446442032307*^9, 3.5644644240424147`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox[
       SubscriptBox["v", "0"], "v00"], "/.", 
      RowBox[{"leadweak", "[", "7", "]"}]}], "/.", "subIm"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", 
  "Normal"}]], "Input",
 CellChangeTimes->{{3.564407768015316*^9, 3.564407801247497*^9}}],

Cell[BoxData[
 RowBox[{"1", "+", 
  RowBox[{"8", " ", 
   SuperscriptBox["g", "2"]}], "-", 
  RowBox[{"20", " ", 
   SuperscriptBox["g", "4"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{"112", "+", 
     RowBox[{"48", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1192"}], "-", 
     RowBox[{"720", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"960", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "-", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
     RowBox[{"36", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "-", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "-", 
     RowBox[{"36", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], ")"}]}]}]], "Output",\

 CellChangeTimes->{
  3.56440772407821*^9, {3.564407769333106*^9, 3.564407801966054*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"1", "/", "Q"}], "/.", 
       RowBox[{"leadweak", "[", "8", "]"}]}], "/.", "subIm"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"SortBy", "[", 
  RowBox[{
   RowBox[{"List", "@@", 
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"%", "/.", "subtotran"}], "/.", 
        RowBox[{"Q0", "\[Rule]", "Q0"}]}], " ", "/.", 
       RowBox[{"g", "\[Rule]", 
        FractionBox["g", "tran"]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "tran"}], "}"}]}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"Exponent", "[", 
     RowBox[{"#", ",", "g"}], "]"}], "&"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Exponent", "[", 
  RowBox[{"%", ",", "tran"}], "]"}]}], "Input",
 CellChangeTimes->{{3.5644078997752743`*^9, 3.564407903432585*^9}, {
   3.564407990517406*^9, 3.564408008045004*^9}, {3.564408087547873*^9, 
   3.564408101587496*^9}, 3.564408142218552*^9, {3.564408309801865*^9, 
   3.564408441267511*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    StyleBox["Q0",
     FontColor->RGBColor[{
        Rational[1, 2], 
        Rational[1, 2], 0}]]], ",", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox["g", "2"]}], 
    RowBox[{"3", " ", 
     SuperscriptBox["tran", "2"], " ", 
     SuperscriptBox[
      StyleBox["Q0",
       FontColor->RGBColor[{
          Rational[1, 2], 
          Rational[1, 2], 0}]], "2"]}]], ",", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox["g", "4"], " ", 
     RowBox[{"(", 
      RowBox[{"8", "+", 
       RowBox[{"9", " ", 
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]]}]}], ")"}]}], 
    RowBox[{"9", " ", 
     SuperscriptBox["tran", "4"], " ", 
     SuperscriptBox[
      StyleBox["Q0",
       FontColor->RGBColor[{
          Rational[1, 2], 
          Rational[1, 2], 0}]], "3"]}]], ",", 
   RowBox[{
    SuperscriptBox["g", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        FractionBox["64", 
         RowBox[{"27", " ", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "4"]}]], "+", 
        FractionBox["16", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "3"]}]], "-", 
        FractionBox["8", 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]]}], 
       SuperscriptBox["tran", "6"]], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], 
       RowBox[{
        SuperscriptBox["tran", "3"], " ", 
        SuperscriptBox[
         StyleBox["Q0",
          FontColor->RGBColor[{
             Rational[1, 2], 
             Rational[1, 2], 0}]], "2"]}]]}], ")"}]}], ",", 
   RowBox[{
    SuperscriptBox["g", "8"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        FractionBox["256", 
         RowBox[{"81", " ", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "5"]}]], "+", 
        FractionBox["32", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "4"]}]], "-", 
        FractionBox["52", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "3"]}]], "-", 
        FractionBox["16", 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]]}], 
       SuperscriptBox["tran", "8"]], "-", 
      FractionBox[
       RowBox[{"160", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], 
       RowBox[{
        SuperscriptBox["tran", "3"], " ", 
        SuperscriptBox[
         StyleBox["Q0",
          FontColor->RGBColor[{
             Rational[1, 2], 
             Rational[1, 2], 0}]], "2"]}]], "+", 
      FractionBox[
       RowBox[{
        FractionBox[
         RowBox[{"36", " ", "\[ImaginaryI]", " ", 
          SqrtBox["3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]], "-", 
        FractionBox[
         RowBox[{"36", " ", "\[ImaginaryI]", " ", 
          SqrtBox["3"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]]}], 
       SuperscriptBox["tran", "6"]], "+", 
      FractionBox[
       RowBox[{
        FractionBox[
         RowBox[{"64", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], 
         RowBox[{"3", " ", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "3"]}]], "-", 
        FractionBox[
         RowBox[{"56", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]], "-", 
        FractionBox[
         RowBox[{"36", " ", "\[ImaginaryI]", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]], "+", 
        FractionBox[
         RowBox[{"36", " ", "\[ImaginaryI]", " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]]}], 
       SuperscriptBox["tran", "5"]], "+", 
      FractionBox[
       RowBox[{
        FractionBox[
         RowBox[{"6", " ", "\[ImaginaryI]", " ", 
          SqrtBox["3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]], "-", 
        FractionBox[
         RowBox[{"6", " ", "\[ImaginaryI]", " ", 
          SqrtBox["3"], " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], 
            StyleBox["_",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}], 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "2"]]}], 
       SuperscriptBox["tran", "4"]]}], ")"}]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.564408342035375*^9, 3.564408407434184*^9}, 
   3.564408441881524*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "4"}], ",", 
   RowBox[{"-", "3"}], ",", 
   RowBox[{"-", "3"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.564408342035375*^9, 3.564408407434184*^9}, 
   3.5644084419036283`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "0", ",", "1", ",", "1"}], "}"}]], "Input",
 CellChangeTimes->{{3.5644082711356087`*^9, 3.564408292553589*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["\[Gamma]", "0"], "/.", 
       RowBox[{"leadweak", "[", "7", "]"}]}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Im1", "|", "Im2", "|", "Im3", "|", "Im4"}], "\[Rule]", "0"}],
        "}"}]}], "/.", "subtotran"}], "/.", 
    RowBox[{"g", "\[Rule]", 
     FractionBox["g", "tran"]}]}], ",", "tran"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Coefficient", "[", 
  RowBox[{"%", ",", "tran", ",", 
   RowBox[{"-", "3"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.564407357919404*^9, 3.564407375092476*^9}, {
   3.564407563061386*^9, 3.5644075727186403`*^9}, {3.564407608301182*^9, 
   3.564407685254972*^9}, 3.5644077267461443`*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"2877240", " ", 
    SuperscriptBox["g", "14"]}], 
   SuperscriptBox["tran", "14"]], "-", 
  FractionBox[
   RowBox[{"269148", " ", 
    SuperscriptBox["g", "12"]}], 
   SuperscriptBox["tran", "12"]], "+", 
  FractionBox[
   RowBox[{"26508", " ", 
    SuperscriptBox["g", "10"]}], 
   SuperscriptBox["tran", "10"]], "+", 
  FractionBox[
   RowBox[{"336", " ", 
    SuperscriptBox["g", "6"]}], 
   SuperscriptBox["tran", "6"]], "-", 
  FractionBox[
   RowBox[{"48", " ", 
    SuperscriptBox["g", "4"]}], 
   SuperscriptBox["tran", "4"]], "+", 
  FractionBox[
   RowBox[{"12", " ", 
    SuperscriptBox["g", "2"]}], 
   SuperscriptBox["tran", "2"]], "+", 
  FractionBox[
   RowBox[{"697536", " ", 
    SuperscriptBox["g", "14"], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox["tran", "11"]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "2820"}], " ", 
     SuperscriptBox["g", "8"]}], "+", 
    RowBox[{"3456", " ", 
     SuperscriptBox["g", "14"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3", "2"]}]}], 
   SuperscriptBox["tran", "8"]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "55296"}], " ", 
     SuperscriptBox["g", "12"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{"579744", " ", 
     SuperscriptBox["g", "14"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}]}], 
   SuperscriptBox["tran", "9"]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"4320", " ", 
     SuperscriptBox["g", "10"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "-", 
    RowBox[{"44064", " ", 
     SuperscriptBox["g", "12"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "+", 
    RowBox[{"471744", " ", 
     SuperscriptBox["g", "14"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}]}], 
   SuperscriptBox["tran", "7"]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "288"}], " ", 
     SuperscriptBox["g", "8"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{"2880", " ", 
     SuperscriptBox["g", "10"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "-", 
    RowBox[{"30240", " ", 
     SuperscriptBox["g", "12"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}], "+", 
    RowBox[{"338688", " ", 
     SuperscriptBox["g", "14"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "9"]}]}], 
   SuperscriptBox["tran", "5"]]}]], "Output",
 CellChangeTimes->{{3.56440736409009*^9, 3.564407366099688*^9}, 
   3.5644075732228317`*^9, {3.5644076105442266`*^9, 3.564407685615106*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.56440736409009*^9, 3.564407366099688*^9}, 
   3.5644075732228317`*^9, {3.5644076105442266`*^9, 3.56440768563514*^9}}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalLambda]", "=", "7"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Sigma]", "[", 
    RowBox[{"r_", ",", "s_", ",", "n_"}], "]"}], "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "2"}], "\[ImaginaryI]"}], ")"}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], 
    RowBox[{"Sin", "[", 
     RowBox[{
      FractionBox["\[Pi]", "2"], 
      RowBox[{"(", 
       RowBox[{"r", "-", "s"}], ")"}]}], "]"}], 
    RowBox[{"Zeta", "[", 
     RowBox[{
      RowBox[{"2", "n"}], "+", "r", "+", "s"}], "]"}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"2", "n"}], "+", "r", "+", "s"}], "]"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"2", "n"}], "+", "r", "+", "s", "+", "1"}], "]"}]}], ")"}], 
     "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", 
        RowBox[{"n", "+", "1"}], "]"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{"n", "+", "r", "+", "1"}], "]"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{"n", "+", "s", "+", "1"}], "]"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{"n", "+", "r", "+", "s", "+", "1"}], "]"}]}], ")"}]}]}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.552351851435109*^9, 3.552351867508997*^9}, {
   3.5523519179094667`*^9, 3.5523519181706333`*^9}, {3.5538566015442667`*^9, 
   3.5538566028794518`*^9}, {3.553857217554524*^9, 3.5538572179112864`*^9}, {
   3.553857279633211*^9, 3.553857297785171*^9}, 3.5559748910158367`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"xh", "[", "1", "]"}], "r"]], "-", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"xh", "[", 
        RowBox[{"-", "1"}], "]"}], "r"]]}], ")"}], 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"xh", "[", "1", "]"}], "s"]], "-", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"xh", "[", 
        RowBox[{"-", "1"}], "]"}], "s"]]}], ")"}]}], "/.", 
  RowBox[{
   RowBox[{"x$h", "[", "a_", "]"}], "\[RuleDelayed]", 
   FractionBox[
    RowBox[{"v00", "+", 
     RowBox[{"I", " ", 
      FractionBox["a", "2"]}]}], "g"]}]}]], "Input",
 CellChangeTimes->{{3.564420254329112*^9, 3.564420280160748*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"-", 
          FractionBox["\[ImaginaryI]", "2"]}], "+", 
         FractionBox["1", 
          RowBox[{"2", " ", 
           SqrtBox["3"]}]]}], "g"], ")"}], 
      RowBox[{"-", "r"}]]}], "+", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        FractionBox["\[ImaginaryI]", "2"], "+", 
        FractionBox["1", 
         RowBox[{"2", " ", 
          SqrtBox["3"]}]]}], "g"], ")"}], 
     RowBox[{"-", "r"}]]}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"-", 
          FractionBox["\[ImaginaryI]", "2"]}], "+", 
         FractionBox["1", 
          RowBox[{"2", " ", 
           SqrtBox["3"]}]]}], "g"], ")"}], 
      RowBox[{"-", "s"}]]}], "+", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        FractionBox["\[ImaginaryI]", "2"], "+", 
        FractionBox["1", 
         RowBox[{"2", " ", 
          SqrtBox["3"]}]]}], "g"], ")"}], 
     RowBox[{"-", "s"}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.564420264384351*^9, 3.564420280657261*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Im1", "/.", "subIm"}]], "Input",
 CellChangeTimes->{{3.564421243157259*^9, 3.564421250055011*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"405", " ", 
   SqrtBox["3"]}], "+", 
  RowBox[{"648", " ", 
   SqrtBox["3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{"720", " ", 
   SqrtBox["3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"972", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
  RowBox[{"324", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
  RowBox[{"162", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
  RowBox[{"972", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "-", 
  RowBox[{"324", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
  RowBox[{"162", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}]], "Output",
 CellChangeTimes->{3.564421250700241*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[CapitalLambda]", "=", "10"}], "}"}], ",", " ", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{"2", " ", 
       SuperscriptBox["g", 
        RowBox[{"r", "+", "s", "+", 
         RowBox[{"2", "n"}]}]], 
       RowBox[{"\[Sigma]", "[", 
        RowBox[{"r", ",", "s", ",", "n"}], "]"}], 
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"xh", "[", "1", "]"}], "r"]], "-", 
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"xh", "[", 
            RowBox[{"-", "1"}], "]"}], "r"]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"xh", "[", "1", "]"}], "s"]], "-", 
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"xh", "[", 
            RowBox[{"-", "1"}], "]"}], "s"]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "s"]}], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"r", ",", "1", ",", "\[CapitalLambda]", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"s", ",", "1", ",", "\[CapitalLambda]", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n", ",", "0", ",", 
        RowBox[{"\[CapitalLambda]", "-", "r", "-", "s"}]}], "}"}]}], "]"}]}], 
   "]"}], "/.", 
  RowBox[{
   RowBox[{"Zeta", "[", "a_", "]"}], "\[RuleDelayed]", 
   SubscriptBox["\[Zeta]", "a"]}]}]], "Input",
 CellChangeTimes->{{3.5644201405283203`*^9, 3.5644202207393713`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "8"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "3"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{"16", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "5"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"8", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "5"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"80", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "5"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"40", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "7"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"24", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "7"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "5"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "5"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"280", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "7"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"8", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "7"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "6"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "6"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"168", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "7"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"840", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "7"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{"112", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "5"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "5"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{
   FractionBox["224", "3"], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "6"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "6"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"1008", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"32", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "7"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "7"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"672", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "5"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "5"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"4032", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"8", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "8"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "8"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"288", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "6"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "6"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"2688", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"9408", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "9"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"3696", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "5"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "5"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"2640", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "6"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "6"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"18480", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"1320", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "7"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "7"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"13200", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "5"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "5"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"55440", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"440", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "8"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "8"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"6600", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "6"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "6"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"39600", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"110880", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "11"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"302016", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"226512", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "5"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "5"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"755040", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"125840", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "6"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "6"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"566280", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"1359072", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "13"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "-", 
  RowBox[{"10306296", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "15"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "3"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "3"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "15"]}], "-", 
  RowBox[{"8016008", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "15"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "4"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "4"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "15"]}], "-", 
  RowBox[{"17177160", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "15"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "15"]}], "+", 
  RowBox[{"222485120", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["g", "17"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-"], ")"}], "2"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "+"], ")"}], "2"]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "-"]]}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "17"]}]}]], "Output",
 CellChangeTimes->{{3.5644201543338213`*^9, 3.5644202215128927`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\[CapitalLambda]", "=", "10"}], "}"}], ",", 
         RowBox[{"Series", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Log", "[", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"(", 
                   FractionBox[
                    RowBox[{"xh", "[", 
                    RowBox[{"+", "1"}], "]"}], 
                    RowBox[{"xh", "[", 
                    RowBox[{"-", "1"}], "]"}]], ")"}], "2"], 
                 FractionBox[
                  RowBox[{"u", "+", 
                   FractionBox["\[ImaginaryI]", "2"]}], 
                  RowBox[{"u", "-", 
                   FractionBox["\[ImaginaryI]", "2"]}]], 
                 SuperscriptBox[
                  RowBox[{"(", 
                   FractionBox[
                    RowBox[{"1", "+", 
                    FractionBox["1", 
                    RowBox[{
                    RowBox[{"xh", "[", 
                    RowBox[{"-", "1"}], "]"}], 
                    RowBox[{"xh", "[", 
                    RowBox[{"-", "1"}], "]"}], " "}]]}], 
                    RowBox[{"1", "+", 
                    FractionBox["1", 
                    RowBox[{
                    RowBox[{"xh", "[", 
                    RowBox[{"+", "1"}], "]"}], 
                    RowBox[{"xh", "[", 
                    RowBox[{"+", "1"}], "]"}]}]]}]], ")"}], "2"]}], "]"}], 
               ",", 
               RowBox[{"ee", " ", 
                RowBox[{"Sum", "[", 
                 RowBox[{
                  RowBox[{"2", " ", 
                   SuperscriptBox["g", 
                    RowBox[{"r", "+", "s", "+", 
                    RowBox[{"2", "n"}]}]], 
                   RowBox[{"\[Sigma]", "[", 
                    RowBox[{"r", ",", "s", ",", "n"}], "]"}], 
                   RowBox[{"(", 
                    RowBox[{
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"xh", "[", "1", "]"}], "r"]], "-", 
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"xh", "[", 
                    RowBox[{"-", "1"}], "]"}], "r"]]}], ")"}], 
                   RowBox[{"(", 
                    RowBox[{
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"xh", "[", "1", "]"}], "s"]], "-", 
                    FractionBox["1", 
                    SuperscriptBox[
                    RowBox[{"xh", "[", 
                    RowBox[{"-", "1"}], "]"}], "s"]]}], ")"}], 
                   RowBox[{"(", 
                    RowBox[{"1", "-", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "s"]}], ")"}]}], ",", 
                  RowBox[{"{", 
                   RowBox[{"r", ",", "1", ",", "\[CapitalLambda]", ",", "1"}],
                    "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"s", ",", "1", ",", "\[CapitalLambda]", ",", "1"}],
                    "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"n", ",", "0", ",", 
                    RowBox[{"\[CapitalLambda]", "-", "r", "-", "s"}]}], 
                   "}"}]}], "]"}]}]}], "}"}], "/.", "subJuk"}], "/.", 
            RowBox[{"u", "\[Rule]", 
             RowBox[{"v00", "+", 
              RowBox[{
               SuperscriptBox["g", "2"], "\[Delta]u"}]}]}]}], ",", 
           RowBox[{"{", 
            RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}]}], "]"}], "//.",
        "densqrt3"}], "/.", 
      RowBox[{
       RowBox[{"Zeta", "[", "a_", "]"}], "\[RuleDelayed]", 
       SubscriptBox["\[Zeta]", "a"]}]}], "//", "Simplify"}], "//", "Normal"}],
    "//", "Total"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"g", ",", "\[Delta]u"}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.564419251392194*^9, 3.564419379181984*^9}, {
  3.564419842546941*^9, 3.564419866809976*^9}, {3.564419972499345*^9, 
  3.56441998051889*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Series", "::", "ztest1"}], "MessageName"], ":", 
  " ", "\<\"Unable to decide whether numeric quantity \[NoBreak]\\!\\(\\(\\(-\
\\(\\(46656\\/\\((\\(\\(\\(\\(-3\\)\\)\\\\ \[ImaginaryI]\\)\\) + \\@3)\\)\\^6\
\\)\\)\\)\\) + 46656\\/\\((\\(\\(3\\\\ \[ImaginaryI]\\)\\) + \\@3)\\)\\^6\\)\
\[NoBreak] is equal to zero. Assuming it is. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Series\\\", ButtonNote -> \
\\\"Series::ztest1\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.564419983045867*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "-", 
     RowBox[{"9", " ", "\[ImaginaryI]", " ", "\[Delta]u"}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "54"}], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "+", 
     RowBox[{
      FractionBox["9", "2"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SuperscriptBox["\[Delta]u", "2"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"648", " ", "\[ImaginaryI]", " ", "\[Delta]u"}], "-", 
     RowBox[{"108", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SuperscriptBox["\[Delta]u", "2"]}], "+", 
     RowBox[{"72", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", "ee", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2835", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "+", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["\[Delta]u", "3"]}], "-", 
     RowBox[{
      FractionBox["27", "4"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      SuperscriptBox["\[Delta]u", "4"]}], "-", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", "ee", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "+", 
     RowBox[{"\[Delta]u", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "6480"}], " ", "\[ImaginaryI]"}], "-", 
        RowBox[{"216", " ", "\[ImaginaryI]", " ", "ee", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "-", 
     RowBox[{"720", " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", "ee", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.5644192561356163`*^9, 3.564419359392581*^9}, 
   3.564419814129098*^9, {3.564419851317857*^9, 3.564419868011381*^9}, {
   3.564419974806307*^9, 3.564419983648601*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             FractionBox[
              RowBox[{"xh", "[", 
               RowBox[{"+", "1"}], "]"}], 
              RowBox[{"xh", "[", 
               RowBox[{"-", "1"}], "]"}]], ")"}], "2"], 
           FractionBox[
            RowBox[{"u", "+", 
             FractionBox["\[ImaginaryI]", "2"]}], 
            RowBox[{"u", "-", 
             FractionBox["\[ImaginaryI]", "2"]}]], 
           SuperscriptBox[
            RowBox[{"(", 
             FractionBox[
              RowBox[{"1", "+", 
               FractionBox["1", 
                RowBox[{
                 RowBox[{"xh", "[", 
                  RowBox[{"-", "1"}], "]"}], 
                 RowBox[{"xh", "[", 
                  RowBox[{"-", "1"}], "]"}], " "}]]}], 
              RowBox[{"1", "+", 
               FractionBox["1", 
                RowBox[{
                 RowBox[{"xh", "[", 
                  RowBox[{"+", "1"}], "]"}], 
                 RowBox[{"xh", "[", 
                  RowBox[{"+", "1"}], "]"}]}]]}]], ")"}], "2"]}], "]"}], ",", 
         
         RowBox[{"Sum", "[", 
          RowBox[{
           RowBox[{"2", " ", 
            SuperscriptBox["g", 
             RowBox[{"r", "+", "s", "+", 
              RowBox[{"2", "n"}]}]], 
            RowBox[{"\[Sigma]", "[", 
             RowBox[{"r", ",", "s", ",", "n"}], "]"}], 
            RowBox[{"(", 
             RowBox[{
              FractionBox["1", 
               SuperscriptBox[
                RowBox[{"xh", "[", "1", "]"}], "r"]], "-", 
              FractionBox["1", 
               SuperscriptBox[
                RowBox[{"xh", "[", 
                 RowBox[{"-", "1"}], "]"}], "r"]]}], ")"}], 
            RowBox[{"(", 
             RowBox[{
              FractionBox["1", 
               SuperscriptBox[
                RowBox[{"xh", "[", "1", "]"}], "s"]], "-", 
              FractionBox["1", 
               SuperscriptBox[
                RowBox[{"xh", "[", 
                 RowBox[{"-", "1"}], "]"}], "s"]]}], ")"}], 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "s"]}], ")"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"r", ",", "1", ",", "\[CapitalLambda]", ",", "1"}], "}"}],
            ",", 
           RowBox[{"{", 
            RowBox[{"s", ",", "1", ",", "\[CapitalLambda]", ",", "1"}], "}"}],
            ",", 
           RowBox[{"{", 
            RowBox[{"n", ",", "0", ",", 
             RowBox[{"\[CapitalLambda]", "-", "r", "-", "s"}]}], "}"}]}], 
          "]"}]}], "}"}], "/.", "subJuk"}], "/.", 
      RowBox[{"u", "\[Rule]", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"2", 
          SqrtBox["3"]}]], "+", 
        RowBox[{
         SuperscriptBox["g", "2"], "\[Delta]u"}]}]}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", 
       RowBox[{"2", " ", "\[CapitalLambda]"}]}], "}"}]}], "]"}], "//", 
   "FullSimplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"%", "//", "Total"}], ")"}], "+", 
   RowBox[{
    SuperscriptBox["g", "8"], "m1"}], "+", 
   RowBox[{
    SuperscriptBox["g", "10"], "m2"}], "+", 
   RowBox[{
    SuperscriptBox["g", "12"], "m3"}], "+", 
   RowBox[{
    SuperscriptBox["g", "14"], "m4"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"condition", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"%", "/.", 
      RowBox[{"\[Delta]u", "->", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Lambda]", 
           RowBox[{"i", "+", "2"}]], 
          SuperscriptBox["g", 
           RowBox[{"2", "i"}]]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], 
        "]"}]}]}], ")"}], "[", 
    RowBox[{"[", "3", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"bethesolut", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"DeleteCases", "[", 
        RowBox[{"condition", ",", "0"}], "]"}], "\[Equal]", "0"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        SubscriptBox["\[Lambda]", "i"], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "2", ",", 
          RowBox[{"\[CapitalLambda]", "+", "1"}]}], "}"}]}], "]"}]}], "]"}], 
    "[", 
    RowBox[{"[", "1", "]"}], "]"}], "//", "Simplify"}]}]}], "Input",
 CellChangeTimes->{{3.549882547962368*^9, 3.549882568268237*^9}, {
   3.5498826146805162`*^9, 3.5498826276815577`*^9}, {3.552351879593645*^9, 
   3.5523518811873083`*^9}, {3.552351933551113*^9, 3.552351946983827*^9}, {
   3.552352065485135*^9, 3.552352066684208*^9}, {3.553856646637452*^9, 
   3.5538566817785273`*^9}, {3.553857222743487*^9, 3.553857226646655*^9}, {
   3.55385727746134*^9, 3.553857292731312*^9}, 3.555974886571327*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Series", "::", "ztest1"}], "MessageName"], ":", 
  " ", "\<\"Unable to decide whether numeric quantity \[NoBreak]\\!\\(\\(\\(-\
\\(\\(46656\\/\\((\\(\\(\\(\\(-3\\)\\)\\\\ \[ImaginaryI]\\)\\) + \\@3)\\)\\^6\
\\)\\)\\)\\) + 46656\\/\\((\\(\\(3\\\\ \[ImaginaryI]\\)\\) + \\@3)\\)\\^6\\)\
\[NoBreak] is equal to zero. Assuming it is. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Series\\\", ButtonNote -> \
\\\"Series::ztest1\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.555974908116775*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Coefficient", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Im4", "/.", "subIm"}], "/.", "subtotran"}], ",", "tran", ",", 
   "10"}], "]"}]], "Input",
 CellChangeTimes->{{3.564421270339777*^9, 3.56442130849325*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.564421275471912*^9, 3.5644213088631277`*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"subtmp", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Lambda]", "2"], "\[Rule]", 
      FractionBox["4", 
       SqrtBox["3"]]}], ",", 
     RowBox[{
      SubscriptBox["\[Lambda]", "3"], "\[Rule]", 
      RowBox[{"-", 
       FractionBox["10", 
        SqrtBox["3"]]}]}], ",", 
     RowBox[{
      SubscriptBox["\[Lambda]", "4"], "\[Rule]", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{"(", 
         RowBox[{"7", "+", 
          RowBox[{"3", " ", 
           RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}], 
       SqrtBox["3"]]}], ",", 
     RowBox[{
      SubscriptBox["\[Lambda]", "5"], "\[Rule]", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
       FractionBox[
        RowBox[{"461", "+", 
         RowBox[{"144", " ", 
          RowBox[{"Zeta", "[", "3", "]"}]}], "+", 
         RowBox[{"240", " ", 
          RowBox[{"Zeta", "[", "5", "]"}]}]}], 
        SqrtBox["3"]]}]}], ",", 
     RowBox[{
      SubscriptBox["\[Lambda]", "6"], "\[Rule]", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "m2"}], "9"], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         RowBox[{"(", 
          RowBox[{"1133", "+", 
           RowBox[{"252", " ", 
            RowBox[{"Zeta", "[", "3", "]"}]}], "+", 
           RowBox[{"378", " ", 
            RowBox[{"Zeta", "[", "5", "]"}]}], "+", 
           RowBox[{"630", " ", 
            RowBox[{"Zeta", "[", "7", "]"}]}]}], ")"}]}], 
        SqrtBox["3"]]}]}], ",", 
     RowBox[{
      SubscriptBox["\[Lambda]", "7"], "\[Rule]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "15890"}], " ", 
        SqrtBox["3"]}], "+", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "m1"}], "-", 
       FractionBox[
        RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], "9"], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "m3"}], "9"], "-", 
       RowBox[{"3112", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "3", "]"}]}], "-", 
       RowBox[{"96", " ", 
        SqrtBox["3"], " ", 
        SuperscriptBox[
         RowBox[{"Zeta", "[", "3", "]"}], "2"]}], "-", 
       RowBox[{"3888", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "5", "]"}]}], "-", 
       RowBox[{"5544", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "7", "]"}]}], "-", 
       RowBox[{"9408", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "9", "]"}]}]}]}], ",", 
     RowBox[{
      SubscriptBox["\[Lambda]", "8"], "\[Rule]", 
      RowBox[{
       RowBox[{"172616", " ", 
        SqrtBox["3"]}], "-", 
       RowBox[{"8", " ", "\[ImaginaryI]", " ", "m1"}], "+", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "m2"}], "-", 
       FractionBox[
        RowBox[{"4", " ", "\[ImaginaryI]", " ", "m3"}], "9"], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "m4"}], "9"], "+", 
       RowBox[{"36576", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "3", "]"}]}], "+", 
       RowBox[{"192", " ", 
        SqrtBox["3"], " ", 
        SuperscriptBox[
         RowBox[{"Zeta", "[", "3", "]"}], "2"]}], "+", 
       RowBox[{"38272", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "5", "]"}]}], "+", 
       RowBox[{"1920", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "3", "]"}], " ", 
        RowBox[{"Zeta", "[", "5", "]"}]}], "+", 
       RowBox[{"45648", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "7", "]"}]}], "+", 
       RowBox[{"64512", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "9", "]"}]}], "+", 
       RowBox[{"110880", " ", 
        SqrtBox["3"], " ", 
        RowBox[{"Zeta", "[", "11", "]"}]}]}]}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.560861680563663*^9, 3.560861685150146*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", 
      SqrtBox["3"]}]], "+", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"2", "i"}]], 
       SubscriptBox["\[Lambda]", 
        RowBox[{"i", "+", "1"}]]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "7"}], "}"}]}], "]"}]}], "/.", "subtmp"}], "/.", 
  RowBox[{
   RowBox[{"Zeta", "[", "r_", "]"}], "\[RuleDelayed]", 
   RowBox[{
    RowBox[{"\[Zeta]$", "[", "r", "]"}], "[", "0", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.560861652740629*^9, 3.560861690198121*^9}, {
  3.560861737044183*^9, 3.560861757499793*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"2", " ", 
    SqrtBox["3"]}]], "+", 
  FractionBox[
   RowBox[{"4", " ", 
    SuperscriptBox["g", "2"]}], 
   SqrtBox["3"]], "-", 
  FractionBox[
   RowBox[{"10", " ", 
    SuperscriptBox["g", "4"]}], 
   SqrtBox["3"]], "+", 
  FractionBox[
   RowBox[{"8", " ", 
    SuperscriptBox["g", "6"], " ", 
    RowBox[{"(", 
     RowBox[{"7", "+", 
      RowBox[{"3", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], 
   SqrtBox["3"]], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
     FractionBox[
      RowBox[{"461", "+", 
       RowBox[{"144", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"240", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SqrtBox["3"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "m2"}], "9"], "+", 
     FractionBox[
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1133", "+", 
         RowBox[{"252", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], "+", 
         RowBox[{"378", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "5"]}], "+", 
         RowBox[{"630", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], 
      SqrtBox["3"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "15890"}], " ", 
      SqrtBox["3"]}], "+", 
     RowBox[{"2", " ", "\[ImaginaryI]", " ", "m1"}], "-", 
     FractionBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], "9"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "m3"}], "9"], "-", 
     RowBox[{"3112", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"96", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
     RowBox[{"3888", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "-", 
     RowBox[{"5544", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"9408", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"172616", " ", 
      SqrtBox["3"]}], "-", 
     RowBox[{"8", " ", "\[ImaginaryI]", " ", "m1"}], "+", 
     RowBox[{"2", " ", "\[ImaginaryI]", " ", "m2"}], "-", 
     FractionBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "m3"}], "9"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "m4"}], "9"], "+", 
     RowBox[{"36576", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "+", 
     RowBox[{"192", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
     RowBox[{"38272", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"1920", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"45648", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "+", 
     RowBox[{"64512", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "+", 
     RowBox[{"110880", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.560861671737558*^9, 3.560861690823572*^9}, {
  3.560861743145677*^9, 3.5608617580167627`*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      SqrtBox["3"]}]], "+", 
    FractionBox[
     RowBox[{"4", " ", 
      SuperscriptBox["g", "2"]}], 
     SqrtBox["3"]], "-", 
    FractionBox[
     RowBox[{"10", " ", 
      SuperscriptBox["g", "4"]}], 
     SqrtBox["3"]], "+", 
    FractionBox[
     RowBox[{"8", " ", 
      SuperscriptBox["g", "6"], " ", 
      RowBox[{"(", 
       RowBox[{"7", "+", 
        RowBox[{"3", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], 
     SqrtBox["3"]], "+", 
    RowBox[{
     SuperscriptBox["g", "8"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
       FractionBox[
        RowBox[{"461", "+", 
         RowBox[{"144", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], "+", 
         RowBox[{"240", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "5"]}]}], 
        SqrtBox["3"]]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "10"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "m2"}], "9"], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         RowBox[{"(", 
          RowBox[{"1133", "+", 
           RowBox[{"252", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"]}], "+", 
           RowBox[{"378", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "5"]}], "+", 
           RowBox[{"630", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], 
        SqrtBox["3"]]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "12"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "15890"}], " ", 
        SqrtBox["3"]}], "+", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "m1"}], "-", 
       FractionBox[
        RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], "9"], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "m3"}], "9"], "-", 
       RowBox[{"3112", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{"96", " ", 
        SqrtBox["3"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
       RowBox[{"3888", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "-", 
       RowBox[{"5544", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"9408", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "14"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"172616", " ", 
        SqrtBox["3"]}], "-", 
       RowBox[{"8", " ", "\[ImaginaryI]", " ", "m1"}], "+", 
       RowBox[{"2", " ", "\[ImaginaryI]", " ", "m2"}], "-", 
       FractionBox[
        RowBox[{"4", " ", "\[ImaginaryI]", " ", "m3"}], "9"], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "m4"}], "9"], "+", 
       RowBox[{"36576", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"192", " ", 
        SqrtBox["3"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
       RowBox[{"38272", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"1920", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"45648", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{"64512", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"110880", " ", 
        SqrtBox["3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}], "/.", 
   "subZetas"}], "//", "N"}]], "Input",
 CellChangeTimes->{{3.5619982839725857`*^9, 3.5619982866939163`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "15890"}], " ", 
      SqrtBox["3"]}], "+", 
     RowBox[{"2", " ", "\[ImaginaryI]", " ", "m1"}], "-", 
     FractionBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], "9"], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "m3"}], "9"], "-", 
     RowBox[{"3112", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"96", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
     RowBox[{"3888", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "-", 
     RowBox[{"5544", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"9408", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}], "/.", "subZetas"}], "//",
   "N"}]], "Input",
 CellChangeTimes->{{3.5619982524564533`*^9, 3.5619982651728067`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "67235.22259594433`"}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"0.`", "\[VeryThinSpace]", "+", 
     RowBox[{"2.`", " ", "\[ImaginaryI]"}]}], ")"}], " ", "m1"}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"0.`", "\[VeryThinSpace]", "+", 
     RowBox[{"0.4444444444444444`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
   "m2"}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"0.`", "\[VeryThinSpace]", "+", 
     RowBox[{"0.1111111111111111`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
   "m3"}]}]], "Output",
 CellChangeTimes->{{3.561998257068622*^9, 3.561998265603629*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["g", "14"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"172616", " ", 
       SqrtBox["3"]}], "-", 
      RowBox[{"8", " ", "\[ImaginaryI]", " ", "m1"}], "+", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", "m2"}], "-", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m3"}], "9"], "-", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", "m4"}], "9"], "+", 
      RowBox[{"36576", " ", 
       SqrtBox["3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "+", 
      RowBox[{"192", " ", 
       SqrtBox["3"], " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
      RowBox[{"38272", " ", 
       SqrtBox["3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "+", 
      RowBox[{"1920", " ", 
       SqrtBox["3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "+", 
      RowBox[{"45648", " ", 
       SqrtBox["3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "+", 
      RowBox[{"64512", " ", 
       SqrtBox["3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], "+", 
      RowBox[{"110880", " ", 
       SqrtBox["3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}], "/.", "subZetas"}], "//",
   "N"}]], "Input",
 CellChangeTimes->{{3.561998225016214*^9, 3.561998230255763*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "14"], " ", 
  RowBox[{"(", 
   RowBox[{"832326.322746783`", "\[VeryThinSpace]", "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"8.`", " ", "\[ImaginaryI]"}]}], ")"}], " ", "m1"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"2.`", " ", "\[ImaginaryI]"}]}], ")"}], " ", "m2"}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.4444444444444444`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
     "m3"}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.1111111111111111`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
     "m4"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.561998230782606*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
     FractionBox[
      RowBox[{"461", "+", 
       RowBox[{"144", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"240", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SqrtBox["3"]]}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", "\[Rule]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"manal", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "/.", 
         RowBox[{"g", "\[Rule]", "1"}]}], ")"}]}], ",", 
      RowBox[{"m2", "->", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"manal", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "/.", 
         RowBox[{"g", "\[Rule]", "1"}]}], ")"}]}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     SqrtBox["3"], ",", 
     SubscriptBox["\[Zeta]", "_"]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.560861901436336*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "216"}], " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"]}], "-", 
  RowBox[{"108", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
  RowBox[{"18", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"]}], "+", 
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
  RowBox[{"36", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "-", 
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"72", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"108", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"108", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"72", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"18", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"648", " ", "\[ImaginaryI]"}], "+", 
     RowBox[{"144", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "+", 
     RowBox[{"288", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"144", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}]}], 
    ")"}]}], "+", 
  RowBox[{
   SqrtBox["3"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["596", "3"]}], "-", 
     RowBox[{"120", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"160", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"36", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "-", 
     RowBox[{"72", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "216"}], " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "-", 
        RowBox[{"216", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}]}], ")"}]}], "+", 
     RowBox[{"108", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"108", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"36", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.5608619019655533`*^9}]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.56086192916151*^9, 3.560861929850512*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m1"}], "9"]}], "-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "m2"}], "9"], "+", 
     FractionBox[
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1133", "+", 
         RowBox[{"252", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], "+", 
         RowBox[{"378", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "5"]}], "+", 
         RowBox[{"630", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], 
      SqrtBox["3"]]}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", "\[Rule]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"manal", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "/.", 
         RowBox[{"g", "\[Rule]", "1"}]}], ")"}]}], ",", 
      RowBox[{"m2", "->", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"manal", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "/.", 
         RowBox[{"g", "\[Rule]", "1"}]}], ")"}]}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     SqrtBox["3"], ",", 
     SubscriptBox["\[Zeta]", "_"]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.560861723965884*^9, 3.560861733656105*^9}, {
  3.560861789508957*^9, 3.5608618045491343`*^9}, {3.5608618406400423`*^9, 
  3.560861877257656*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"10368", " ", "\[ImaginaryI]", " ", 
    SubsuperscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "2", "3"]}], "5"], "+", 
  RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "+", 
  RowBox[{"648", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "-", 
  RowBox[{"504", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"]}], "+", 
  RowBox[{"108", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "5"]}], "-", 
  RowBox[{"72", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "6"]}], "-", 
  RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "-", 
  RowBox[{"1296", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
  RowBox[{"1008", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "-", 
  RowBox[{"216", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
  RowBox[{"144", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"]}], "+", 
  RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"648", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"2016", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"324", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"432", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "5"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"1296", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"216", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"648", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"3024", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{"216", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"1080", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"648", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"2016", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"216", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"1440", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"504", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "-", 
  RowBox[{"324", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "-", 
  RowBox[{"1080", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "-", 
  RowBox[{"108", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "5"]}], "-", 
  RowBox[{"432", " ", "\[ImaginaryI]", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "5"]}], "-", 
  RowBox[{"72", " ", "\[ImaginaryI]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "6"]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}]}], 
    ")"}]}], "+", 
  RowBox[{
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "6048"}], " ", "\[ImaginaryI]"}], "+", 
     RowBox[{"1296", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "-", 
     RowBox[{
      FractionBox["14688", "5"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
     RowBox[{"1296", " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{
      FractionBox["29376", "5"], " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{
      FractionBox["14688", "5"], " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}]}], 
    ")"}]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "7776"}], " ", "\[ImaginaryI]"}], "-", 
     RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"3888", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "+", 
     RowBox[{"4032", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
     RowBox[{"864", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "+", 
     RowBox[{"864", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"]}], "+", 
     RowBox[{"3888", " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"8064", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{"864", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"4032", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"864", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"5184", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"864", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
     RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
     RowBox[{"864", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}]}], 
    ")"}]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "2"]}], "+", 
     RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"]}], "-", 
     RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "-", 
     RowBox[{"432", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
     RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"864", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"864", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "7776"}], " ", "\[ImaginaryI]"}], "-", 
        RowBox[{"1728", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
        RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
        RowBox[{"1728", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}]}], 
       ")"}]}]}], ")"}]}], "+", 
  RowBox[{
   SqrtBox["3"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["9824", "3"], "+", 
     RowBox[{"1200", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"288", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
     RowBox[{"912", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"2352", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "-", 
     RowBox[{"72", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "-", 
     RowBox[{"216", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"]}], "+", 
     RowBox[{"204", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "5"]}], "+", 
     RowBox[{"144", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
     RowBox[{"432", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
     RowBox[{"408", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "5", "+"]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4896", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "+", 
        RowBox[{"4896", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}]}], ")"}]}], "+", 
     RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{"432", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
     RowBox[{"1020", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{"432", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
     RowBox[{"216", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
     RowBox[{"2040", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
     RowBox[{"72", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
     RowBox[{"432", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
     RowBox[{"2040", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
     RowBox[{"216", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
     RowBox[{"1020", " ", "\[ImaginaryI]", " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
     RowBox[{"204", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "5"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "1"], " ", 
      RowBox[{"(", 
       RowBox[{"2592", "-", 
        RowBox[{"432", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "+", 
        RowBox[{"864", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
        RowBox[{
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "+", 
           RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
            SubsuperscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]0",
               FontWeight->Bold,
               FontColor->RGBColor[0, 0.5, 0]], 
              StyleBox["_",
               FontWeight->Bold,
               FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}]}], ")"}]}], "-", 
        
        RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
        RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
        RowBox[{"432", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}]}], 
       ")"}]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"432", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "1", "+"]}], "+", 
        RowBox[{"1296", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"]}], "-", 
        RowBox[{"2040", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"]}], "+", 
        RowBox[{"432", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
        RowBox[{"6120", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            StyleBox["\[Eta]0",
             FontWeight->Bold,
             FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
        RowBox[{"1296", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
        RowBox[{"6120", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
        RowBox[{"2040", " ", "\[ImaginaryI]", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[Eta]0",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]], 
             StyleBox["_",
              FontWeight->Bold,
              FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}]}], 
       ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.5608617955988073`*^9, 3.560861805032276*^9}, {
  3.560861864843979*^9, 3.560861878186927*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["v", "0"], "/.", 
  RowBox[{"leadweak", "[", "5", "]"}]}]], "Input",
 CellChangeTimes->{{3.560861507669065*^9, 3.560861513997072*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"2", " ", 
    SqrtBox["3"]}]], "+", 
  FractionBox[
   RowBox[{"4", " ", 
    SuperscriptBox["g", "2"]}], 
   SqrtBox["3"]], "-", 
  FractionBox[
   RowBox[{"10", " ", 
    SuperscriptBox["g", "4"]}], 
   SqrtBox["3"]], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["56", 
      SqrtBox["3"]], "+", 
     RowBox[{"8", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["461", 
       SqrtBox["3"]]}], "-", 
     RowBox[{"48", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"80", " ", 
      SqrtBox["3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.560861514427602*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"manal", "[", 
     RowBox[{"[", "1", "]"}], "]"}], "/.", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]0$b", "[", "1", "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], "\[RuleDelayed]", 
     RowBox[{"per", "-", 
      RowBox[{
       RowBox[{"\[Eta]0$", "[", "1", "]"}], "[", "1", "]"}]}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"g", ",", "per"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.560861526723691*^9, 3.5608615987316628`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "405"}], " ", "\[ImaginaryI]", " ", 
     SqrtBox["3"]}], "-", 
    RowBox[{"324", " ", 
     SqrtBox["3"], " ", 
     SuperscriptBox["per", "3"]}], "+", 
    RowBox[{"162", " ", 
     SuperscriptBox["per", "4"]}], "+", 
    RowBox[{
     SuperscriptBox["per", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"972", "-", 
       RowBox[{"1296", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "2"]}]}], ")"}]}], "-", 
    RowBox[{"5832", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2"]}], "+", 
    RowBox[{"1944", " ", 
     SqrtBox["3"], " ", "per", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2"]}], "+", 
    RowBox[{"1944", " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "2"]}], "-", 
    RowBox[{"648", " ", "\[ImaginaryI]", " ", 
     SqrtBox["3"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "-", 
    RowBox[{"720", " ", "\[ImaginaryI]", " ", 
     SqrtBox["3"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "-", 
    RowBox[{"1944", " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
    RowBox[{"648", " ", 
     SqrtBox["3"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
    RowBox[{"324", " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.560861539611775*^9, 3.560861599275898*^9}}]
}, Open  ]],

Cell["Let us now see how this modifies expression for the energy", "Text",
 CellChangeTimes->{{3.549882673745883*^9, 3.5498826816959467`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Expand", "/@", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"2", "-", 
            RowBox[{"4", " ", "\[ImaginaryI]", " ", "g", " ", 
             RowBox[{"(", 
              RowBox[{
               FractionBox["1", 
                SuperscriptBox[
                 OverscriptBox[
                  StyleBox["y",
                   FontColor->RGBColor[0, 0.6, 0]], 
                  StyleBox["^",
                   FontColor->RGBColor[0, 0.6, 0]]], "-"]], "-", 
               FractionBox["1", 
                SuperscriptBox[
                 OverscriptBox[
                  StyleBox["y",
                   FontColor->RGBColor[0, 0.6, 0]], 
                  StyleBox["^",
                   FontColor->RGBColor[0, 0.6, 0]]], "+"]]}], ")"}]}]}], "/.",
            "subJuk"}], "/.", 
          RowBox[{
           SubscriptBox["v", "0"], "\[Rule]", 
           RowBox[{
            FractionBox["1", 
             RowBox[{"2", 
              SqrtBox["3"]}]], "+", 
            RowBox[{
             SuperscriptBox["g", "2"], "\[Delta]u"}]}]}]}], "/.", 
         RowBox[{"\[Delta]u", "->", 
          RowBox[{"Sum", "[", 
           RowBox[{
            RowBox[{
             SubscriptBox["\[Lambda]", 
              RowBox[{"i", "+", "2"}]], 
             SuperscriptBox["g", 
              RowBox[{"2", "i"}]]}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], 
           "]"}]}]}], "/.", "bethesolut"}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], "//", 
     "Simplify"}], ")"}]}], "/.", 
  RowBox[{
   RowBox[{"Zeta", "[", "a_", "]"}], "\[RuleDelayed]", " ", 
   SubscriptBox["\[Zeta]", "a"]}]}]], "Input",
 CellChangeTimes->{{3.549882723242701*^9, 3.549882766432375*^9}, {
   3.5523519678036833`*^9, 3.552351969129526*^9}, {3.552352100697398*^9, 
   3.552352114503006*^9}, {3.553856880442696*^9, 3.553856891856968*^9}, {
   3.553857325881415*^9, 3.553857326137095*^9}, 3.555975358676763*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"2", "+", 
   RowBox[{"12", " ", 
    SuperscriptBox["g", "2"]}], "-", 
   RowBox[{"48", " ", 
    SuperscriptBox["g", "4"]}], "+", 
   RowBox[{"336", " ", 
    SuperscriptBox["g", "6"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "2820"}], "-", 
      RowBox[{"288", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "8"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"26508", "+", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m1"}], 
       SqrtBox["3"]], "+", 
      RowBox[{"4320", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "+", 
      RowBox[{"2880", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}]}], ")"}], " ", 
    SuperscriptBox["g", "10"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "269148"}], "-", 
      FractionBox[
       RowBox[{"20", " ", "\[ImaginaryI]", " ", "m1"}], 
       SqrtBox["3"]], "+", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], 
       SqrtBox["3"]], "-", 
      RowBox[{"55296", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "-", 
      RowBox[{"44064", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "-", 
      RowBox[{"30240", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}]}], ")"}], " ", 
    SuperscriptBox["g", "12"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2877240", "+", 
      RowBox[{"56", " ", "\[ImaginaryI]", " ", 
       SqrtBox["3"], " ", "m1"}], "-", 
      FractionBox[
       RowBox[{"20", " ", "\[ImaginaryI]", " ", "m2"}], 
       SqrtBox["3"]], "+", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m3"}], 
       SqrtBox["3"]], "+", 
      RowBox[{"697536", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "+", 
      RowBox[{"3456", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
      RowBox[{"579744", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "+", 
      RowBox[{"471744", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "+", 
      RowBox[{"338688", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}]}], ")"}], " ", 
    SuperscriptBox["g", "14"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "31857132"}], "-", 
      FractionBox[
       RowBox[{"1844", " ", "\[ImaginaryI]", " ", "m1"}], 
       SqrtBox["3"]], "+", 
      RowBox[{"56", " ", "\[ImaginaryI]", " ", 
       SqrtBox["3"], " ", "m2"}], "-", 
      FractionBox[
       RowBox[{"20", " ", "\[ImaginaryI]", " ", "m3"}], 
       SqrtBox["3"]], "+", 
      FractionBox[
       RowBox[{"4", " ", "\[ImaginaryI]", " ", "m4"}], 
       SqrtBox["3"]], "-", 
      RowBox[{"8912160", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "-", 
      RowBox[{"79488", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
      RowBox[{"7486848", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "-", 
      RowBox[{"69120", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], "-", 
      RowBox[{"6342624", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], "-", 
      RowBox[{"5370624", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], "-", 
      RowBox[{"3991680", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "11"]}]}], ")"}], " ", 
    SuperscriptBox["g", "16"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "17"],
    SeriesData[$CellContext`g, 0, {}, 0, 17, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   2, 0, 12, 0, -48, 0, 336, 0, -2820 - 288 $CellContext`\[Zeta]$[3][0], 0, 
    26508 + (Complex[0, 4] 3^Rational[-1, 2]) $CellContext`m1 + 
    4320 $CellContext`\[Zeta]$[3][0] + 2880 $CellContext`\[Zeta]$[5][0], 
    0, -269148 + (Complex[0, -20] 3^Rational[-1, 2]) $CellContext`m1 + (
      Complex[0, 4] 3^Rational[-1, 2]) $CellContext`m2 - 
    55296 $CellContext`\[Zeta]$[3][0] - 44064 $CellContext`\[Zeta]$[5][0] - 
    30240 $CellContext`\[Zeta]$[7][0], 0, 
    2877240 + (Complex[0, 56] 3^Rational[1, 2]) $CellContext`m1 + (
      Complex[0, -20] 3^Rational[-1, 2]) $CellContext`m2 + (Complex[0, 4] 
      3^Rational[-1, 2]) $CellContext`m3 + 697536 $CellContext`\[Zeta]$[3][0] + 
    3456 $CellContext`\[Zeta]$[3][0]^2 + 579744 $CellContext`\[Zeta]$[5][0] + 
    471744 $CellContext`\[Zeta]$[7][0] + 338688 $CellContext`\[Zeta]$[9][0], 
    0, -31857132 + (Complex[0, -1844] 3^Rational[-1, 2]) $CellContext`m1 + (
      Complex[0, 56] 3^Rational[1, 2]) $CellContext`m2 + (Complex[0, -20] 
      3^Rational[-1, 2]) $CellContext`m3 + (Complex[0, 4] 
      3^Rational[-1, 2]) $CellContext`m4 - 
    8912160 $CellContext`\[Zeta]$[3][0] - 79488 $CellContext`\[Zeta]$[3][0]^2 - 
    7486848 $CellContext`\[Zeta]$[5][0] - (
     69120 $CellContext`\[Zeta]$[3][0]) $CellContext`\[Zeta]$[5][0] - 
    6342624 $CellContext`\[Zeta]$[7][0] - 5370624 $CellContext`\[Zeta]$[9][0] - 
    3991680 $CellContext`\[Zeta]$[11][0]}, 0, 17, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.5498827475057573`*^9, 3.5498827673534822`*^9}, {
   3.55235196406947*^9, 3.552351972717911*^9}, {3.5523520862076693`*^9, 
   3.552352116644101*^9}, {3.5538567027349997`*^9, 3.5538567263420763`*^9}, {
   3.553856890221643*^9, 3.5538568941446238`*^9}, {3.553857318719317*^9, 
   3.5538573377253523`*^9}, 3.555975003104319*^9, 3.555975360420142*^9, 
   3.555975404926256*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     FractionBox["1", 
      SqrtBox[
       SubscriptBox["C", "\[GothicCapitalT]"]]], "/.", "weak"}], "/.", 
    RowBox[{"leadweak", "[", "5", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"g", ",", "0", ",", "9"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.564421412385283*^9, 3.564421428315516*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{
    SqrtBox["6"], " ", "g"}], "+", 
   RowBox[{
    SqrtBox["6"], " ", 
    SuperscriptBox["g", "3"]}], "-", 
   RowBox[{"21", " ", 
    SqrtBox[
     FractionBox["3", "2"]], " ", 
    SuperscriptBox["g", "5"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"109", " ", 
       SqrtBox["6"]}], "-", 
      RowBox[{"12", " ", 
       SqrtBox["6"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "7"]}], "+", 
   RowBox[{
    FractionBox["1", "24"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "27399"}], " ", 
       SqrtBox["6"]}], "+", 
      RowBox[{"4", " ", 
       SqrtBox["2"], " ", "Im1"}], "+", 
      RowBox[{"1152", " ", 
       SqrtBox["6"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "+", 
      RowBox[{"2880", " ", 
       SqrtBox["6"], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}]}], ")"}], " ", 
    SuperscriptBox["g", "9"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "10"],
    SeriesData[$CellContext`g, 0, {}, 1, 10, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   6^Rational[1, 2], 0, 6^Rational[1, 2], 0, (-21) 
    Rational[3, 2]^Rational[1, 2], 0, 
    109 6^Rational[1, 2] - (12 6^Rational[1, 2]) $CellContext`\[Zeta]$[3][0], 
    0, Rational[
     1, 24] ((-27399) 
      6^Rational[1, 2] + (4 2^Rational[1, 2]) $CellContext`Im1 + (1152 
       6^Rational[1, 2]) $CellContext`\[Zeta]$[3][0] + (2880 
       6^Rational[1, 2]) $CellContext`\[Zeta]$[5][0])}, 1, 10, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.564421418080616*^9, 3.564421428854488*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   InterpretationBox[
    RowBox[{"2", "+", 
     RowBox[{"12", " ", 
      SuperscriptBox["g", "2"]}], "-", 
     RowBox[{"48", " ", 
      SuperscriptBox["g", "4"]}], "+", 
     RowBox[{"336", " ", 
      SuperscriptBox["g", "6"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "2820"}], "-", 
        RowBox[{"288", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
      SuperscriptBox["g", "8"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"26508", "+", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", "m1"}], 
         SqrtBox["3"]], "+", 
        RowBox[{"4320", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "+", 
        RowBox[{"2880", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}]}], ")"}], " ", 
      SuperscriptBox["g", "10"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "269148"}], "-", 
        FractionBox[
         RowBox[{"20", " ", "\[ImaginaryI]", " ", "m1"}], 
         SqrtBox["3"]], "+", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], 
         SqrtBox["3"]], "-", 
        RowBox[{"55296", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "-", 
        RowBox[{"44064", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "-", 
        RowBox[{"30240", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}]}], ")"}], " ", 
      SuperscriptBox["g", "12"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"2877240", "+", 
        RowBox[{"56", " ", "\[ImaginaryI]", " ", 
         SqrtBox["3"], " ", "m1"}], "-", 
        FractionBox[
         RowBox[{"20", " ", "\[ImaginaryI]", " ", "m2"}], 
         SqrtBox["3"]], "+", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", "m3"}], 
         SqrtBox["3"]], "+", 
        RowBox[{"697536", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "+", 
        RowBox[{"3456", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
        RowBox[{"579744", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "+", 
        RowBox[{"471744", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}], "+", 
        RowBox[{"338688", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "9"]}]}], ")"}], " ", 
      SuperscriptBox["g", "14"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "31857132"}], "-", 
        FractionBox[
         RowBox[{"1844", " ", "\[ImaginaryI]", " ", "m1"}], 
         SqrtBox["3"]], "+", 
        RowBox[{"56", " ", "\[ImaginaryI]", " ", 
         SqrtBox["3"], " ", "m2"}], "-", 
        FractionBox[
         RowBox[{"20", " ", "\[ImaginaryI]", " ", "m3"}], 
         SqrtBox["3"]], "+", 
        FractionBox[
         RowBox[{"4", " ", "\[ImaginaryI]", " ", "m4"}], 
         SqrtBox["3"]], "-", 
        RowBox[{"8912160", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "-", 
        RowBox[{"79488", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
        RowBox[{"7486848", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "-", 
        RowBox[{"69120", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "-", 
        RowBox[{"6342624", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}], "-", 
        RowBox[{"5370624", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "9"]}], "-", 
        RowBox[{"3991680", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "11"]}]}], ")"}], " ", 
      SuperscriptBox["g", "16"]}], "+", 
     InterpretationBox[
      SuperscriptBox[
       RowBox[{"O", "[", "g", "]"}], "17"],
      SeriesData[$CellContext`g, 0, {}, 0, 17, 1],
      Editable->False]}],
    SeriesData[$CellContext`g, 0, {
     2, 0, 12, 0, -48, 0, 336, 0, -2820 - 288 $CellContext`\[Zeta]$[3][0], 0, 
      26508 + (Complex[0, 4] 3^Rational[-1, 2]) $CellContext`m1 + 
      4320 $CellContext`\[Zeta]$[3][0] + 2880 $CellContext`\[Zeta]$[5][0], 
      0, -269148 + (Complex[0, -20] 3^Rational[-1, 2]) $CellContext`m1 + (
        Complex[0, 4] 3^Rational[-1, 2]) $CellContext`m2 - 
      55296 $CellContext`\[Zeta]$[3][0] - 44064 $CellContext`\[Zeta]$[5][0] - 
      30240 $CellContext`\[Zeta]$[7][0], 0, 
      2877240 + (Complex[0, 56] 3^Rational[1, 2]) $CellContext`m1 + (
        Complex[0, -20] 3^Rational[-1, 2]) $CellContext`m2 + (Complex[0, 4] 
        3^Rational[-1, 2]) $CellContext`m3 + 
      697536 $CellContext`\[Zeta]$[3][0] + 3456 $CellContext`\[Zeta]$[3][0]^2 + 
      579744 $CellContext`\[Zeta]$[5][0] + 471744 $CellContext`\[Zeta]$[7][0] + 
      338688 $CellContext`\[Zeta]$[9][0], 
      0, -31857132 + (Complex[0, -1844] 3^Rational[-1, 2]) $CellContext`m1 + (
        Complex[0, 56] 3^Rational[1, 2]) $CellContext`m2 + (Complex[0, -20] 
        3^Rational[-1, 2]) $CellContext`m3 + (Complex[0, 4] 
        3^Rational[-1, 2]) $CellContext`m4 - 
      8912160 $CellContext`\[Zeta]$[3][0] - 
      79488 $CellContext`\[Zeta]$[3][0]^2 - 
      7486848 $CellContext`\[Zeta]$[5][0] - (
       69120 $CellContext`\[Zeta]$[3][0]) $CellContext`\[Zeta]$[5][0] - 
      6342624 $CellContext`\[Zeta]$[7][0] - 
      5370624 $CellContext`\[Zeta]$[9][0] - 
      3991680 $CellContext`\[Zeta]$[11][0]}, 0, 17, 1],
    Editable->False], "/.", "subZetas"}], "//", "N"}]], "Input",
 CellChangeTimes->{{3.5619985414264717`*^9, 3.5619985446741343`*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"2.`", "+", 
   RowBox[{"12.`", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "2"]}], "-", 
   RowBox[{"48.`", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "4"]}], "+", 
   RowBox[{"336.`", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "6"]}], "-", 
   RowBox[{"3166.1923881099633`", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "8"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"34687.23775646235`", "\[VeryThinSpace]", "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"2.3094010767585034`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m1"}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "10"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "411800.6052677797`"}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"11.547005383792516`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m1"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"2.3094010767585034`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m2"}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "12"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"5.136915263938293`*^6", "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"96.99484522385713`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m1"}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"11.547005383792516`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m2"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"2.3094010767585034`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m3"}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "14"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "6.63050290455782`*^7"}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"1064.6338963856701`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m1"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"96.99484522385713`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m2"}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"11.547005383792516`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m3"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"2.3094010767585034`", " ", "\[ImaginaryI]"}]}], ")"}], " ", 
       "m4"}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"g", "+", "0.`"}], ")"}], "16"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      RowBox[{"g", "+", "0.`"}], "]"}], "17"],
    SeriesData[$CellContext`g, 0., {}, 0, 17, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0., {
   2., 0., 12., 0., -48., 0., 336., 0., -3166.1923881099633`, 0., 
    34687.23775646235 + Complex[0., 2.3094010767585034`] $CellContext`m1, 
    0., -411800.6052677797 + 
    Complex[0., -11.547005383792516`] $CellContext`m1 + 
    Complex[0., 2.3094010767585034`] $CellContext`m2, 0., 
    5.136915263938293*^6 + Complex[0., 96.99484522385713] $CellContext`m1 + 
    Complex[0., -11.547005383792516`] $CellContext`m2 + 
    Complex[0., 2.3094010767585034`] $CellContext`m3, 
    0., -6.63050290455782*^7 + 
    Complex[0., -1064.6338963856701`] $CellContext`m1 + 
    Complex[0., 96.99484522385713] $CellContext`m2 + 
    Complex[0., -11.547005383792516`] $CellContext`m3 + 
    Complex[0., 2.3094010767585034`] $CellContext`m4}, 0, 17, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.56199854548342*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "269148"}], "-", 
     RowBox[{"55296", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"44064", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "-", 
     RowBox[{"30240", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}]}], ")"}], " ", 
   SuperscriptBox["g", "12"]}], "//", "Factor"}]], "Input",
 CellChangeTimes->{{3.5547811528381643`*^9, 3.5547811911377583`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "12"}], " ", 
  SuperscriptBox["g", "12"], " ", 
  RowBox[{"(", 
   RowBox[{"22429", "+", 
    RowBox[{"4608", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "+", 
    RowBox[{"3672", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "+", 
    RowBox[{"2520", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.554781156940363*^9, 3.554781191691702*^9}}]
}, Open  ]],

Cell[TextData[{
 "So, modification of the Bethe equation introduces itself as correction ",
 Cell[BoxData[
  FractionBox[
   RowBox[{"4", " ", "\[ImaginaryI]", " ", "m"}], 
   SqrtBox["3"]]],
  CellChangeTimes->{{3.5498827475057573`*^9, 3.5498827673534822`*^9}, {
   3.55235196406947*^9, 3.552351972717911*^9}, {3.5523520862076693`*^9, 
   3.552352116644101*^9}, {3.5538567027349997`*^9, 3.5538567263420763`*^9}, {
   3.553856890221643*^9, 3.5538568941446238`*^9}}],
 " to the expression for the energy. Now I understand how Janik came with the \
factor ",
 Cell[BoxData[
  FormBox[
   FractionBox["4", 
    SqrtBox["3"]], TraditionalForm]]],
 " in his formulae."
}], "Text",
 CellChangeTimes->{{3.54988278587614*^9, 3.549882849185648*^9}, 
   3.553856867529788*^9, 3.553856904390044*^9}],

Cell["So we have expressions for the energy", "Text",
 CellChangeTimes->{{3.552352130681528*^9, 3.552352136320531*^9}, {
  3.553856781762835*^9, 3.553856791478733*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]E", 
     RowBox[{
      RowBox[{"5", " ", "loop"}], ",", " ", 
      RowBox[{"root", " ", "shift"}]}]], "==", 
    FractionBox[
     RowBox[{"4", " ", "\[ImaginaryI]", " ", "m"}], 
     SqrtBox["3"]]}], ",", 
   RowBox[{
    SubscriptBox["\[Delta]E", 
     RowBox[{
      RowBox[{"6", "loop"}], ",", 
      RowBox[{"root", " ", "shift"}]}]], "==", 
    RowBox[{
     FractionBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", "m2"}], 
      SqrtBox["3"]], "-", 
     FractionBox[
      RowBox[{"20", " ", "\[ImaginaryI]", " ", "m"}], 
      SqrtBox["3"]]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.5523521371152077`*^9, 3.5523521511251173`*^9}, {
  3.552356612779237*^9, 3.5523566346913853`*^9}, {3.553856915579796*^9, 
  3.55385691794971*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "31857132"}], "-", 
    FractionBox[
     RowBox[{"1844", " ", "\[ImaginaryI]", " ", "m1"}], 
     SqrtBox["3"]], "+", 
    RowBox[{"56", " ", "\[ImaginaryI]", " ", 
     SqrtBox["3"], " ", "m2"}], "-", 
    FractionBox[
     RowBox[{"20", " ", "\[ImaginaryI]", " ", "m3"}], 
     SqrtBox["3"]], "+", 
    FractionBox[
     RowBox[{"4", " ", "\[ImaginaryI]", " ", "m4"}], 
     SqrtBox["3"]], "-", 
    RowBox[{"8912160", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"]}], "-", 
    RowBox[{"79488", " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
    RowBox[{"7486848", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "-", 
    RowBox[{"69120", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "-", 
    RowBox[{"6342624", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "7"]}], "-", 
    RowBox[{"5370624", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "9"]}], "-", 
    RowBox[{"3991680", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "11"]}]}], ")"}], "//", 
  "InputForm"}]], "Input",
 CellChangeTimes->{{3.561980003511425*^9, 3.561980005196171*^9}}],

Cell["\<\
-31857132 - ((1844*I)*m1)/Sqrt[3] + (56*I)*Sqrt[3]*m2 - 
 ((20*I)*m3)/Sqrt[3] + ((4*I)*m4)/Sqrt[3] - 8912160*\[Zeta]$[3][0] - 
 79488*\[Zeta]$[3][0]^2 - 7486848*\[Zeta]$[5][0] - 
 69120*\[Zeta]$[3][0]*\[Zeta]$[5][0] - 6342624*\[Zeta]$[7][0] - 
 5370624*\[Zeta]$[9][0] - 3991680*\[Zeta]$[11][0]\
\>", "Output",
 CellChangeTimes->{3.5619800110676107`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "+"]}], 
        RowBox[{
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}]], 
       ")"}], "r"], "]"}], "+", 
    RowBox[{"Log", "[", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], "2"], "-", 
           SubsuperscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
            RowBox[{"[", "2", "]"}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox[
            OverscriptBox[
             StyleBox["\[GothicQ]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], 
             StyleBox["_",
              FontColor->RGBColor[0.54, 0.35, 0.15]]], "2"], "-", 
           SubsuperscriptBox[
            OverscriptBox[
             StyleBox["\[GothicQ]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], 
             StyleBox["_",
              FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}]]}], ")"}]}], 
        RowBox[{
         SuperscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]}]], ")"}], "r"], 
     "]"}], "+", 
    RowBox[{"Log", "[", 
     FractionBox[
      SubscriptBox[
       StyleBox["\[GothicCapitalT]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"1", ",", "0"}]], 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["\[GothicQ]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         RowBox[{"1", ",", "2"}], "+"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["_",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], 
         RowBox[{"1", ",", "2"}], "-"]}], ")"}]], "]"}]}], "==", 
   RowBox[{
    RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
      RowBox[{
       RowBox[{"+", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
     RowBox[{
      FractionBox["du", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      FractionBox["1", 
       RowBox[{
        RowBox[{"4", 
         SuperscriptBox["g", "2"], 
         SuperscriptBox["z", "2"]}], "-", 
        SuperscriptBox["u", "2"]}]], 
      FractionBox["H", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]]}]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.546687667733313*^9, 3.546687694430181*^9}, {
  3.546930060725745*^9, 3.546930086044445*^9}, {3.546930226332904*^9, 
  3.546930243522461*^9}, {3.546931545956169*^9, 3.546931547536798*^9}, {
  3.553713007494966*^9, 3.553713008809545*^9}, {3.5537130408085737`*^9, 
  3.5537130471513863`*^9}, {3.555855342790471*^9, 3.555855347237547*^9}, {
  3.5644070733510857`*^9, 3.56440708351433*^9}}],

Cell["Y11/Y22 equation:", "Text",
 CellChangeTimes->{{3.546931548677003*^9, 3.5469315599295673`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", 
     FractionBox[
      SubscriptBox[
       StyleBox["\[GothicCapitalT]",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"1", ",", "0"}]], 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox[
         StyleBox["\[GothicQ]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         RowBox[{"1", ",", "2"}], "+"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["_",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], 
         RowBox[{"1", ",", "2"}], "-"]}], ")"}]], "]"}], "+", 
    RowBox[{"2", 
     RowBox[{"Log", "[", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox[
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], 
          RowBox[{"1", ",", "0"}], "c"], 
         RowBox[{
          SubsuperscriptBox[
           StyleBox["\[GothicCapitalT]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           RowBox[{"0", ",", "1"}], "c"], " "}]], ")"}], "r"], "]"}]}], "+", 
    RowBox[{"Log", "[", " ", 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "2"], "-", 
          SubsuperscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
           RowBox[{"[", "2", "]"}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2"], "-", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[GothicQ]",
             FontColor->RGBColor[0.54, 0.35, 0.15]], 
            StyleBox["_",
             FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], ")"}], 
        SuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], "-"], " ", 
        SuperscriptBox[
         StyleBox["\[GothicCapitalT]",
          FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]}], ")"}], "r"], "]"}], 
    "+", 
    RowBox[{"Log", "[", 
     FractionBox["1", 
      SubscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "-"], " ", 
         SuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "+"], " ", 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "-"], " ", 
         SubsuperscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], "\[FivePointedStar]", "+"]}], ")"}],
        "r"]], "]"}]}], "==", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
      RowBox[{
       RowBox[{"+", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
     RowBox[{
      FractionBox["du", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      RowBox[{"(", 
       FractionBox[
        RowBox[{"2", "u"}], 
        RowBox[{
         RowBox[{"4", 
          SuperscriptBox["g", "2"], 
          SuperscriptBox["z", "2"]}], "-", 
         SuperscriptBox["u", "2"]}]], ")"}], "H"}]}], "+", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}], 
      RowBox[{
       RowBox[{"+", "\[Infinity]"}], "-", 
       RowBox[{"\[ImaginaryI]", " ", "0"}]}]], 
     RowBox[{
      FractionBox["du", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      RowBox[{"(", 
       FractionBox[
        RowBox[{"2", "u"}], 
        RowBox[{
         RowBox[{"4", 
          SuperscriptBox["g", "2"], 
          SuperscriptBox["z", "2"]}], "-", 
         SuperscriptBox["u", "2"]}]], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "2"}], 
        RowBox[{"Log", "[", 
         RowBox[{
          RowBox[{"(", 
           FractionBox[
            RowBox[{" ", 
             SuperscriptBox[
              StyleBox["\[ScriptCapitalT]",
               FontColor->RGBColor[1, 0, 0]], "+"], " "}], 
            SuperscriptBox[
             StyleBox["\[GothicCapitalT]",
              FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]], ")"}], "/", 
          SubscriptBox[
           RowBox[{"(", 
            FractionBox[
             RowBox[{" ", 
              SuperscriptBox[
               StyleBox["\[ScriptCapitalT]",
                FontColor->RGBColor[1, 0, 0]], "+"], " "}], 
             SuperscriptBox[
              StyleBox["\[GothicCapitalT]",
               FontColor->RGBColor[0.54, 0.35, 0.15]], "+"]], ")"}], 
           "asym"]}], "]"}]}], ")"}]}]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.546931310662352*^9, 3.5469313989313602`*^9}, {
  3.546931442809442*^9, 3.5469315410399837`*^9}, {3.553713091967557*^9, 
  3.553713161571763*^9}, {3.555855326912468*^9, 3.555855337580681*^9}, {
  3.564407093795517*^9, 3.56440712310562*^9}}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Displacement of the Bethe roots\
\>", "Section",
 CellChangeTimes->{{3.565373794479515*^9, 3.565373797869855*^9}, {
  3.567158431116664*^9, 3.5671584455935*^9}}],

Cell[TextData[{
 "Above we learned that up to 8 loops ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " subleading tran correction is zero. Therefore subleading correction to \
transcendentality should come solely from U and asymptotic ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", 
    RowBox[{"2", ",", "\"\<as\>\""}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". We will now explore "
}], "Text",
 CellChangeTimes->{{3.565373801341367*^9, 3.5653738730965433`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Coefficient", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["q", 
      RowBox[{"2", ",", "\"\<as\>\""}]], "/.", 
     RowBox[{"nsweak", "[", "5", "]"}]}], "/.", "subtotran"}], ",", "tran", 
   ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.565373848226829*^9, 3.565373915243246*^9}, {
  3.565374108228497*^9, 3.565374125675663*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", "u"}]], "Output",
 CellChangeTimes->{3.5653741261092243`*^9, 3.567155616224558*^9}]
}, Open  ]],

Cell[TextData[{
 "u is normally ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "1"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". But under circumstances it can be ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", "0"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " also. In fact under these circumstances 1/u is also ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", "0"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". In the expression for subleading transcendentality it is clearly of order \
",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "1"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ":"
}], "Text",
 CellChangeTimes->{{3.565373925295555*^9, 3.5653739442873497`*^9}, {
  3.565373978780521*^9, 3.5653739835249367`*^9}, {3.5653740264290733`*^9, 
  3.565374159161332*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"comb2", "/.", "subcomb"}], ",", 
   RowBox[{
    SuperscriptBox["U", "2"], "/.", "leadingtran"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.565374057617753*^9, 3.565374064684965*^9}, {
  3.565374140028267*^9, 3.5653741706090384`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["U",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["_",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SubsuperscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        RowBox[{"2", ",", "4"}], "+"]}], "-", 
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        RowBox[{"1", ",", "4"}], "+"], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         StyleBox["_",
          FontColor->RGBColor[0, 0, 0.8]]], "2"]}], "-", 
      RowBox[{
       SubsuperscriptBox[
        StyleBox["q",
         FontColor->RGBColor[0, 0, 0.8]], 
        RowBox[{"1", ",", "3"}], "+"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["q",
          FontColor->RGBColor[0, 0, 0.8]], 
         StyleBox["_",
          FontColor->RGBColor[0, 0, 0.8]]], "2", "2"]}]}], ")"}]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox["g", "2"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5653740650669518`*^9, {3.5653741601212597`*^9, 3.565374170910728*^9}}]
}, Open  ]],

Cell[TextData[{
 "We proceed with considering corrections to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["U", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". This is a tough venue. First we have to find corrections to h, after that \
only to U. And we will have tough days with normalization of U."
}], "Text",
 CellChangeTimes->{{3.56537417932808*^9, 3.565374232331922*^9}}],

Cell["Equation on h.", "Subsection",
 CellChangeTimes->{{3.5653742364682703`*^9, 3.5653742448980083`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Log", "[", 
  SubscriptBox[
   RowBox[{"(", 
    OverscriptBox[
     StyleBox["h",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], ")"}], "r"], "]"}]], "Subsection",
 CellChangeTimes->{{3.552193297070553*^9, 3.5521933034819736`*^9}, {
   3.552205941187338*^9, 3.5522059414016953`*^9}, {3.5523672032119102`*^9, 
   3.552367210500926*^9}, {3.553772156125369*^9, 3.553772174865464*^9}, {
   3.553775562934312*^9, 3.553775563607533*^9}, {3.555874071749701*^9, 
   3.5558741029420433`*^9}, {3.555957685930942*^9, 3.5559577071023817`*^9}, {
   3.56085298655398*^9, 3.5608530379889*^9}, {3.560856546890081*^9, 
   3.560856549055603*^9}, {3.5614695317855062`*^9, 3.5614695412408857`*^9}, {
   3.562935485611362*^9, 3.562935543585163*^9}, 3.5653743710109587`*^9}],

Cell[CellGroupData[{

Cell["Init", "Subsubsection",
 CellChangeTimes->{{3.562936411626652*^9, 3.562936412043545*^9}}],

Cell[TextData[{
 "Asymptotically one found (up to an overall constant) ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["hh", "as"], "=", 
    RowBox[{
     SubscriptBox["\[Sigma]", "1"], 
     SuperscriptBox["xh", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "L"}], "/", "2"}], "-", "1"}], "}"}]]}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 CellChangeTimes->{{3.552193343869741*^9, 3.552193372875533*^9}, 
   3.5653777018798237`*^9}],

Cell["\<\
Let us examine dressing factor in more detail. Dressing phase to the given \
order in g is:\
\>", "Text",
 CellChangeTimes->{{3.5653743799593554`*^9, 3.5653743883568363`*^9}, {
  3.565374824891761*^9, 3.5653748331332407`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cdres", "[", 
         RowBox[{"r", ",", "s", ",", "n"}], "]"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"r", "+", "s", "+", 
          RowBox[{"2", " ", "n"}]}]]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           SuperscriptBox[
            OverscriptBox[
             StyleBox["y",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "-"]], ")"}], "s"], "-", 
        SuperscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           SuperscriptBox[
            OverscriptBox[
             StyleBox["y",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+"]], ")"}], "s"]}], ")"}]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "r"]], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "2", ",", 
      RowBox[{"2", " ", "order"}], ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "1", ",", 
      RowBox[{"order", "-", 
       FractionBox["r", "2"], "+", "1"}], ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "0", ",", 
      RowBox[{"order", "-", 
       FractionBox["r", "2"], "-", "s", "+", "1"}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5653755563292627`*^9, 3.565375584825611*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "cdres"}]], "Input",
 CellChangeTimes->{{3.565375269506077*^9, 3.5653752709658403`*^9}}],

Cell[CellGroupData[{

Cell["Global`cdres", "Print", "PrintUsage",
 CellChangeTimes->{3.5653752713865423`*^9},
 CellTags->"Info3565378871-5147713"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"cdres", "[", 
          RowBox[{"r_", ",", "s_", ",", "n_"}], "]"}], ":=", 
         FractionBox[
          RowBox[{"2", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "n"], " ", 
           RowBox[{"Sin", "[", 
            RowBox[{
             FractionBox["1", "2"], " ", "\[Pi]", " ", 
             RowBox[{"(", 
              RowBox[{"r", "-", "s"}], ")"}]}], "]"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"2", " ", "n"}], "+", "r", "+", 
                RowBox[{"(", 
                 RowBox[{"s", "-", "1"}], ")"}]}], ")"}], "!"}], " ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"2", " ", "n"}], "+", "r", "+", "s"}], ")"}], "!"}]}],
             ")"}], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], 
            RowBox[{
             RowBox[{"2", " ", "n"}], "+", "r", "+", "s"}]]}], 
          RowBox[{
           RowBox[{"n", "!"}], " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"n", "+", "r"}], ")"}], "!"}], " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"n", "+", "s"}], ")"}], "!"}], " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"n", "+", "r", "+", "s"}], ")"}], "!"}]}]]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`cdres],
  Editable->False]], "Print",
 CellChangeTimes->{3.565375271457047*^9},
 CellTags->"Info3565378871-5147713"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drphase", "/.", 
  RowBox[{"leadweak", "[", "5", "]"}]}]], "Input",
 CellChangeTimes->{{3.56537484174899*^9, 3.56537484922827*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"12", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "48"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{"120", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]], "-", 
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "5"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "7"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "6"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"48", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"252", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"336", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"480", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"1260", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "2820"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "-", 
       RowBox[{"288", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
       RowBox[{"3576", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "-", 
       RowBox[{"5040", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"14112", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "336"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "-", 
       RowBox[{"1008", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"4032", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "48"}], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "-", 
       RowBox[{"432", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "6"]], "-", 
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "9"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "8"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "11"]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "10"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"48", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"660", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "8"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"336", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{"1728", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"9900", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "6"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"2820", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"288", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{"7596", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{"16128", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"59400", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"4320", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
       RowBox[{"31440", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}], "+", 
       RowBox[{
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"], " ", 
        RowBox[{"(", 
         RowBox[{"26508", "+", 
          FractionBox[
           RowBox[{"4", " ", "Im1"}], 
           SqrtBox["3"]], "+", 
          RowBox[{"5760", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}], "+", 
       RowBox[{"38736", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "7"]}], "+", 
       RowBox[{"56448", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "9"]}], "+", 
       RowBox[{"166320", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "11"]}]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.565374849989683*^9}]
}, Open  ]],

Cell["\<\
We will at most need subleading transcendentality. Therefore I will simplify \
the dependence on the Bethe roots:\
\>", "Text",
 CellChangeTimes->{{3.5653748553482513`*^9, 3.565374890131085*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox["1", 
                  RowBox[{"g", 
                   SuperscriptBox[
                    OverscriptBox[
                    StyleBox["y",
                    FontColor->RGBColor[0, 0.6, 0]], 
                    StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]], "-"]}]], ")"}], "s"], 
               "-", 
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox["1", 
                  RowBox[{"g", 
                   SuperscriptBox[
                    OverscriptBox[
                    StyleBox["y",
                    FontColor->RGBColor[0, 0.6, 0]], 
                    StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]], "+"]}]], ")"}], "s"]}], 
              ")"}], "/.", "subJuk"}], "/.", 
            RowBox[{"leadweak", "[", "5", "]"}]}], "/.", 
           RowBox[{"s", "\[Rule]", "#"}]}], "/.", "subIm"}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", 
       "Simplify"}], "//", "Normal"}], "//", "FindHighTranCoef"}], ")"}], 
   "&"}], "/@", 
  RowBox[{"Range", "[", "6", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox["1", 
                  RowBox[{
                   SubscriptBox["v", "0"], "-", 
                   FractionBox["\[ImaginaryI]", "2"]}]], ")"}], "s"], "-", 
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox["1", 
                  RowBox[{
                   SubscriptBox["v", "0"], "+", 
                   FractionBox["\[ImaginaryI]", "2"]}]], ")"}], "s"]}], ")"}],
              "/.", "subJuk"}], "/.", 
            RowBox[{"leadweak", "[", "1", "]"}]}], "/.", 
           RowBox[{"s", "\[Rule]", "#"}]}], "/.", "subIm"}], ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", 
       "Simplify"}], "//", "Normal"}], "//", "FindHighTranCoef"}], ")"}], 
   "&"}], "/@", 
  RowBox[{"Range", "[", "6", "]"}]}]}], "Input",
 CellChangeTimes->{{3.565374897648634*^9, 3.565374954444283*^9}, {
  3.565374985676141*^9, 3.5653750459241953`*^9}, {3.565375180386673*^9, 
  3.565375194466619*^9}, {3.565375286237939*^9, 3.565375301444117*^9}, {
  3.565375350125692*^9, 3.565375356558704*^9}, {3.5653754688143797`*^9, 
  3.565375476133773*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Series", "::", "ztest1"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Unable to decide whether numeric quantity \
\[NoBreak]\\!\\(46656\\/\\((\\(\\(\\(\\(-3\\)\\)\\\\ \[ImaginaryI]\\)\\) + \
\\@3)\\)\\^6 - 46656\\/\\((\\(\\(3\\\\ \[ImaginaryI]\\)\\) + \\@3)\\)\\^6\\)\
\[NoBreak] is equal to zero. Assuming it is. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Series\\\", ButtonNote -> \
\\\"Series::ztest1\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.567155676054913*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"3", " ", "\[ImaginaryI]"}], ",", 
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"]}], ",", 
   RowBox[{"27", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "2"]}], ",", 
   RowBox[{
    RowBox[{"-", "9"}], " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"]}], ",", 
   RowBox[{
    RowBox[{"-", "27"}], " ", "\[ImaginaryI]"}], ",", 
   RowBox[{
    RowBox[{"-", "162"}], " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"], " ", 
    SuperscriptBox["g", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.565374899135212*^9, 3.565374954995872*^9}, {
   3.565374986173276*^9, 3.565375046206359*^9}, {3.565375181144752*^9, 
   3.565375195256864*^9}, 3.565375302066536*^9, {3.5653753508446198`*^9, 
   3.565375358861087*^9}, 3.565375476574992*^9, 3.5671556761016617`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"3", " ", "\[ImaginaryI]"}], ",", 
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"]}], ",", "0", ",", 
   RowBox[{
    RowBox[{"-", "9"}], " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"]}], ",", 
   RowBox[{
    RowBox[{"-", "27"}], " ", "\[ImaginaryI]"}], ",", "0"}], "}"}]], "Output",\

 CellChangeTimes->{{3.565374899135212*^9, 3.565374954995872*^9}, {
   3.565374986173276*^9, 3.565375046206359*^9}, {3.565375181144752*^9, 
   3.565375195256864*^9}, 3.565375302066536*^9, {3.5653753508446198`*^9, 
   3.565375358861087*^9}, 3.565375476574992*^9, 3.5671556761356916`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{"(", 
                   FractionBox["1", 
                    RowBox[{"g", 
                    SuperscriptBox[
                    OverscriptBox[
                    StyleBox["y",
                    FontColor->RGBColor[0, 0.6, 0]], 
                    StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]], "-"]}]], ")"}], "s"], 
                 "-", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   FractionBox["1", 
                    RowBox[{"g", 
                    SuperscriptBox[
                    OverscriptBox[
                    StyleBox["y",
                    FontColor->RGBColor[0, 0.6, 0]], 
                    StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]], "+"]}]], ")"}], "s"]}], 
                ")"}], "/.", "subJuk"}], "//", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"#", "/.", 
                   RowBox[{"leadweak", "[", "1", "]"}]}], ")"}], "-", 
                 RowBox[{"(", 
                  RowBox[{"#", "/.", 
                   RowBox[{"leadweak", "[", "5", "]"}]}], ")"}]}], "&"}], 
               ")"}]}], ")"}], "/.", 
            RowBox[{"s", "\[Rule]", "#"}]}], "/.", "subIm"}], ",", 
          RowBox[{"{", 
           RowBox[{"g", ",", "0", ",", "5"}], "}"}]}], "]"}], "//", 
        "Simplify"}], "//", "Normal"}], "//", "Expand"}], "//", 
     "FindHighTranCoef"}], ")"}], "&"}], "/@", 
  RowBox[{"Range", "[", "6", "]"}]}]], "Input",
 CellChangeTimes->{{3.5653750917782173`*^9, 3.565375149481861*^9}, 
   3.565375306821327*^9, {3.565375370471149*^9, 3.565375386563739*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"12", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "2"]}], ",", 
   RowBox[{
    RowBox[{"-", "144"}], " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"], " ", 
    SuperscriptBox["g", "4"]}], ",", 
   RowBox[{
    RowBox[{"-", "108"}], " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "2"]}], ",", 
   RowBox[{
    RowBox[{"-", "144"}], " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"], " ", 
    SuperscriptBox["g", "2"]}], ",", 
   RowBox[{"4860", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "4"]}], ",", 
   RowBox[{"648", " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"], " ", 
    SuperscriptBox["g", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.565375120511003*^9, 3.565375152579976*^9}, 
   3.565375321867679*^9, {3.565375374327511*^9, 3.56537538704204*^9}, {
   3.5671556841360807`*^9, 3.567155685953034*^9}}]
}, Open  ]],

Cell["\<\
\[LineSeparator]Conclusion is that we can take only leading value of the \
Bethe root AND S=1 to serve two orders of transcendentality (should be enough \
for our goal).\
\>", "Text",
 CellChangeTimes->{{3.5653750552377367`*^9, 3.565375061900251*^9}, {
   3.565375341297347*^9, 3.565375347817425*^9}, {3.565375399391246*^9, 
   3.56537542417478*^9}, {3.565377540776992*^9, 3.565377544398205*^9}, 
   3.56715569034278*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SubscriptBox["v", "0"], "-", 
      FractionBox["\[ImaginaryI]", "2"]}]], "/", 
    RowBox[{"(", 
     RowBox[{
      SqrtBox["3"], 
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "3"]]}], ")"}]}], "/.", 
   RowBox[{"leadweak", "[", "1", "]"}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.565375441812505*^9, 3.565375449863173*^9}, {
  3.565375490537012*^9, 3.565375522318407*^9}, {3.5653755977628193`*^9, 
  3.565375615235087*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{
  3.565375451353855*^9, {3.5653754931000443`*^9, 3.565375522886796*^9}, {
   3.565375598093614*^9, 3.565375615581689*^9}}]
}, Open  ]],

Cell["So we will be doing", "Text",
 CellChangeTimes->{{3.565375619641151*^9, 3.5653756225666122`*^9}, {
  3.5653757208296947`*^9, 3.565375722427732*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cdres", "[", 
         RowBox[{"r", ",", "s", ",", "n"}], "]"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"r", "+", "s", "+", 
          RowBox[{"2", " ", "n"}]}]], 
        SuperscriptBox[
         RowBox[{"(", 
          SqrtBox["3"], ")"}], "s"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox["\[ExponentialE]", 
           FractionBox[
            RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "3"]], ")"}], "s"], "-", 
        
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{"-", 
            FractionBox[
             RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "3"]}]], ")"}], "s"]}], 
       ")"}]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "r"]], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "2", ",", "\[Infinity]", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "1", ",", "\[Infinity]", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.56537562732824*^9, 3.5653756475232887`*^9}, {
  3.56537594189003*^9, 3.5653759717020807`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "[", 
   RowBox[{"r", ",", "s"}], "]"}], "\[Equal]", 
  RowBox[{"2", " ", 
   RowBox[{"Sin", "[", 
    RowBox[{
     FractionBox["1", "2"], " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{"r", "-", "s"}], ")"}]}], "]"}], 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        SubscriptBox["J", "r"], "[", 
        RowBox[{"2", "g", " ", "t"}], "]"}], 
       RowBox[{
        SubscriptBox["J", "s"], "[", 
        RowBox[{"2", "g", " ", "t"}], "]"}]}], 
      RowBox[{"t", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", "t"], "-", "1"}], ")"}]}]], 
     RowBox[{"\[DifferentialD]", "t"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5653761070421333`*^9, 3.56537614977577*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          FractionBox[
           RowBox[{"r", "-", "s", "+", "1"}], "2"]], 
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", "r"], "[", 
            RowBox[{"2", "g", " ", "t"}], "]"}], 
           RowBox[{
            SubscriptBox["J", "s"], "[", 
            RowBox[{"2", "g", " ", "t"}], "]"}]}], "t"]}], "/.", 
        RowBox[{"r", "\[Rule]", "2"}]}], "/.", 
       RowBox[{"s", "\[Rule]", "1"}]}], "/.", "subJBes"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "/.", 
  RowBox[{
   SuperscriptBox["t", "a_"], "\[RuleDelayed]", 
   RowBox[{
    RowBox[{"Gamma", "[", 
     RowBox[{"a", "+", "1"}], "]"}], 
    SubscriptBox["\[Zeta]", 
     RowBox[{"a", "+", "1"}]]}]}]}]}], "Input",
 CellChangeTimes->{{3.565376180268908*^9, 3.565376326877356*^9}, {
  3.5653764154853773`*^9, 3.565376492527256*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   SuperscriptBox["g", "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "-", 
  RowBox[{"20", " ", 
   SuperscriptBox["g", "5"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "+", 
  RowBox[{"210", " ", 
   SuperscriptBox["g", "7"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"2352", " ", 
   SuperscriptBox["g", "9"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"27720", " ", 
   SuperscriptBox["g", "11"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "-", 
  RowBox[{"339768", " ", 
   SuperscriptBox["g", "13"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}], "+", 
  RowBox[{"4294290", " ", 
   SuperscriptBox["g", "15"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "15"]}]}]], "Output",
 CellChangeTimes->{{3.5653764209699593`*^9, 3.5653764927497263`*^9}, 
   3.567155704958014*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cdres", "[", 
    RowBox[{"2", ",", "1", ",", "#"}], "]"}], "&"}], "/@", 
  RowBox[{"Range", "[", 
   RowBox[{"0", ",", "5"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5653764485142937`*^9, 3.565376495160058*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], ",", 
   RowBox[{
    RowBox[{"-", "20"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "5"]}], ",", 
   RowBox[{"210", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "7"]}], ",", 
   RowBox[{
    RowBox[{"-", "2352"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "9"]}], ",", 
   RowBox[{"27720", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "11"]}], ",", 
   RowBox[{
    RowBox[{"-", "339768"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "13"]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.565376435939056*^9, 3.565376495404437*^9}, 
   3.567155706586967*^9}]
}, Open  ]],

Cell["\<\
FINAL expression to use instead of dressing phase:\
\>", "Text",
 CellChangeTimes->{{3.5653775735558558`*^9, 3.565377586115325*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drphasetran", "=", 
  RowBox[{
   RowBox[{"-", "12"}], " ", "g", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     FractionBox[
      RowBox[{"TC", "[", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "r"], 
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", 
             RowBox[{"2", "r"}]], "[", 
            RowBox[{"2", "g", " ", "t"}], "]"}], 
           RowBox[{
            SubscriptBox["J", "1"], "[", 
            RowBox[{"2", "g", " ", "t"}], "]"}]}], "t"]}], ",", 
        FractionBox["1", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", "t"], "-", "1"}], ")"}]], ",", 
        RowBox[{"{", 
         RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"2", "r"}]]], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "1", ",", "\[Infinity]"}], "}"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.565376532294984*^9, 3.565376632182268*^9}, {
   3.565376664759059*^9, 3.5653766749424*^9}, {3.565376888596423*^9, 
   3.565376940157433*^9}, 3.565376996991593*^9, {3.565377590751727*^9, 
   3.565377632902973*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "12"}], " ", "g", " ", 
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"r", "=", "1"}], "\[Infinity]"], 
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "r"], " ", 
        RowBox[{
         SubscriptBox["J", "1"], "[", 
         RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
        RowBox[{
         SubscriptBox["J", 
          RowBox[{"2", " ", "r"}]], "[", 
         RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "t"], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", "t"]}]], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}], " ", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{
      RowBox[{"-", "2"}], " ", "r"}]]}]}]}]], "Output",
 CellChangeTimes->{{3.565376198199368*^9, 3.565376327150425*^9}, 
   3.565376998150125*^9, {3.565377620130287*^9, 3.56537763341473*^9}, 
   3.567155734920643*^9}]
}, Open  ]],

Cell["Check:", "Text",
 CellChangeTimes->{{3.565377657503909*^9, 3.565377658704678*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ExpaCol", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "12"}], " ", "g", " ", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          FractionBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "r"], 
            RowBox[{
             SubscriptBox["J", 
              RowBox[{"2", "r"}]], "[", 
             RowBox[{"2", "g", " ", "t"}], "]"}], 
            RowBox[{
             SubscriptBox["J", "1"], "[", 
             RowBox[{"2", "g", " ", "t"}], "]"}]}], 
           SuperscriptBox[
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], 
            RowBox[{"2", "r"}]]], ",", 
          RowBox[{"{", 
           RowBox[{"r", ",", "1", ",", "10"}], "}"}]}], "]"}], "/.", 
        "subJBes"}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "13"}], "}"}]}], "]"}]}], ",", "t"}], 
   "]"}], "/.", 
  RowBox[{
   SuperscriptBox["t", "a_"], "\[RuleDelayed]", 
   RowBox[{
    SubscriptBox["\[Zeta]", "a"], 
    RowBox[{"Gamma", "[", "a", "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"drphase", "/.", 
        RowBox[{"leadweak", "[", "6", "]"}]}], "/.", "subIm"}], "//", 
      "Expand"}], "//", "FindHighTranCoef"}], ")"}]}], "//", 
  "Expand"}]}], "Input",
 CellChangeTimes->{{3.565377025266589*^9, 3.565377105369574*^9}, {
   3.565377302068634*^9, 3.565377312433817*^9}, {3.5653774171505327`*^9, 
   3.565377464974811*^9}, {3.565377497116205*^9, 3.565377513561055*^9}, {
   3.565439836371786*^9, 3.5654398451741133`*^9}, 3.565440071480275*^9, {
   3.565440730313669*^9, 3.5654407348557367`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"12", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "-", 
  FractionBox[
   RowBox[{"12", " ", 
    SuperscriptBox["g", "6"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"10", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]}], ")"}], " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "5"]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
  RowBox[{"720", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      SuperscriptBox["g", "8"], 
      RowBox[{"60", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "6"]}]], "+", 
     FractionBox[
      RowBox[{"7", " ", 
       SuperscriptBox["g", "8"]}], 
      RowBox[{"20", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]}]], "+", 
     FractionBox[
      RowBox[{"7", " ", 
       SuperscriptBox["g", "8"]}], 
      RowBox[{"4", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{"40320", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox["g", "10"], 
       RowBox[{"3360", " ", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "8"]}]]}], "-", 
     FractionBox[
      RowBox[{"3", " ", 
       SuperscriptBox["g", "10"]}], 
      RowBox[{"280", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "6"]}]], "-", 
     FractionBox[
      SuperscriptBox["g", "10"], 
      RowBox[{"10", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]}]], "-", 
     FractionBox[
      RowBox[{"7", " ", 
       SuperscriptBox["g", "10"]}], 
      RowBox[{"20", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"3628800", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      SuperscriptBox["g", "12"], 
      RowBox[{"302400", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "10"]}]], "+", 
     FractionBox[
      RowBox[{"11", " ", 
       SuperscriptBox["g", "12"]}], 
      RowBox[{"60480", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "8"]}]], "+", 
     FractionBox[
      RowBox[{"11", " ", 
       SuperscriptBox["g", "12"]}], 
      RowBox[{"4032", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "6"]}]], "+", 
     FractionBox[
      RowBox[{"11", " ", 
       SuperscriptBox["g", "12"]}], 
      RowBox[{"672", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]}]], "+", 
     FractionBox[
      RowBox[{"11", " ", 
       SuperscriptBox["g", "12"]}], 
      RowBox[{"240", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"479001600", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox["g", "14"], 
       RowBox[{"39916800", " ", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], "12"]}]]}], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["g", "14"]}], 
      RowBox[{"6652800", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "10"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["g", "14"]}], 
      RowBox[{"302400", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "8"]}]], "-", 
     FractionBox[
      RowBox[{"143", " ", 
       SuperscriptBox["g", "14"]}], 
      RowBox[{"362880", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "6"]}]], "-", 
     FractionBox[
      RowBox[{"143", " ", 
       SuperscriptBox["g", "14"]}], 
      RowBox[{"80640", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]}]], "-", 
     FractionBox[
      RowBox[{"143", " ", 
       SuperscriptBox["g", "14"]}], 
      RowBox[{"33600", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], ")"}], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "13"]}]}]], "Output",
 CellChangeTimes->{{3.5653770294192953`*^9, 3.565377105650662*^9}, {
   3.5653773061640863`*^9, 3.5653773129512997`*^9}, {3.565377419217285*^9, 
   3.565377466309807*^9}, {3.565377498432118*^9, 3.56537751387733*^9}, 
   3.565377642862727*^9, {3.565439824965947*^9, 3.565439845714836*^9}, 
   3.56544007178121*^9, 3.5654407306539507`*^9, 3.567155743871731*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.5653770294192953`*^9, 3.565377105650662*^9}, {
   3.5653773061640863`*^9, 3.5653773129512997`*^9}, {3.565377419217285*^9, 
   3.565377466309807*^9}, {3.565377498432118*^9, 3.56537751387733*^9}, 
   3.565377642862727*^9, {3.565439824965947*^9, 3.565439845714836*^9}, 
   3.56544007178121*^9, 3.5654407306539507`*^9, 3.567155750905542*^9}]
}, Open  ]],

Cell["\<\
If to apply to the Bethe equations, this can be further simplfied (provided \
we seek for its first influence): \
\>", "Text",
 CellChangeTimes->{{3.565440737035563*^9, 3.565440750178466*^9}, {
  3.56544078864153*^9, 3.565440798409112*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"drphasetranforBethe", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"12", " ", 
      SuperscriptBox["g", "3"]}], 
     SuperscriptBox["u", "2"]], " ", 
    RowBox[{"TC", "[", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{
         SubscriptBox["J", "2"], "[", 
         RowBox[{"2", "g", " ", "t"}], "]"}], 
        RowBox[{
         SubscriptBox["J", "1"], "[", 
         RowBox[{"2", "g", " ", "t"}], "]"}]}], "t"], ",", 
      FractionBox["1", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", "t"], "-", "1"}], ")"}]], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.565440769532342*^9, 3.565440817328597*^9}, {
  3.565441380428097*^9, 3.565441384395131*^9}}],

Cell[TextData[{
 "Now we solve Bethe equations (somebody secretly told me that finite size \
correction are of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "3"}]], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.56544086740856*^9, 3.565440872221917*^9}, {
  3.5654409526910353`*^9, 3.5654409769013433`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              FractionBox[
               RowBox[{"xh", "[", 
                RowBox[{"+", "1"}], "]"}], 
               RowBox[{"xh", "[", 
                RowBox[{"-", "1"}], "]"}]], ")"}], "2"], 
            FractionBox[
             RowBox[{"u", "+", 
              FractionBox["\[ImaginaryI]", "2"]}], 
             RowBox[{"u", "-", 
              FractionBox["\[ImaginaryI]", "2"]}]], 
            SuperscriptBox[
             RowBox[{"(", 
              FractionBox[
               RowBox[{"1", "+", 
                FractionBox["1", 
                 RowBox[{
                  RowBox[{"xh", "[", 
                   RowBox[{"-", "1"}], "]"}], 
                  RowBox[{"xh", "[", 
                   RowBox[{"-", "1"}], "]"}], " "}]]}], 
               RowBox[{"1", "+", 
                FractionBox["1", 
                 RowBox[{
                  RowBox[{"xh", "[", 
                   RowBox[{"+", "1"}], "]"}], 
                  RowBox[{"xh", "[", 
                   RowBox[{"+", "1"}], "]"}]}]]}]], ")"}], "2"]}], "]"}], "-", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"drphasetranforBethe", "/.", 
               RowBox[{"u", "\[Rule]", 
                RowBox[{"u", "+", 
                 FractionBox["I", "2"]}]}]}], ")"}], "-", 
             RowBox[{"(", 
              RowBox[{"drphasetranforBethe", "/.", 
               RowBox[{"u", "\[Rule]", 
                RowBox[{"u", "-", 
                 FractionBox["I", "2"]}]}]}], ")"}]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox["tran", 
            RowBox[{"-", "3"}]], "m"}]}], "/.", "subJuk"}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"u0", "+", 
          RowBox[{
           FractionBox["1", "tran"], "\[Delta]u0"}], "+", 
          RowBox[{
           FractionBox["1", 
            SuperscriptBox["tran", "2"]], "\[Delta]u1"}], "+", 
          RowBox[{
           FractionBox["1", 
            SuperscriptBox["tran", "3"]], "\[Delta]u2"}]}]}]}], "/.", 
       RowBox[{"g", "\[Rule]", 
        FractionBox["g", "tran"]}]}], "/.", " ", 
      RowBox[{
       RowBox[{"2", " ", "g", " ", "t"}], "\[Rule]", " ", 
       RowBox[{"2", " ", "g", " ", "t", " ", "tran"}]}]}], ",", 
     RowBox[{"{", 
      RowBox[{"tran", ",", "\[Infinity]", ",", "3"}], "}"}]}], "]"}], "//.", 
   "densqrt3"}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.56543969661298*^9, 3.565439778372295*^9}, {
   3.565439852491858*^9, 3.5654399043833237`*^9}, {3.565439967342391*^9, 
   3.565440003580909*^9}, 3.5654400554594917`*^9, {3.565440830780801*^9, 
   3.5654409219640636`*^9}, {3.565440979988515*^9, 3.565440983509727*^9}, {
   3.5654410414432*^9, 3.565441045762863*^9}, {3.565441861351644*^9, 
   3.565441861868631*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"Log", "[", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"\[ImaginaryI]", "+", 
        RowBox[{"2", " ", "u0"}]}], ")"}], "3"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], "+", 
        RowBox[{"2", " ", "u0"}]}], ")"}], "3"]], "]"}], "-", 
   FractionBox[
    RowBox[{"12", " ", "\[ImaginaryI]", " ", "\[Delta]u0"}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["u0", "2"]}]}], ")"}], " ", "tran"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", "\[ImaginaryI]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"32", " ", 
        SuperscriptBox["g", "2"], " ", "u0"}], "-", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "4"}], " ", "u0", " ", 
           SuperscriptBox["\[Delta]u0", "2"]}], "+", "\[Delta]u1", "+", 
          RowBox[{"4", " ", 
           SuperscriptBox["u0", "2"], " ", "\[Delta]u1"}]}], ")"}]}]}], 
      ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["u0", "2"]}]}], ")"}], "2"], " ", 
     SuperscriptBox["tran", "2"]}]], "+", 
   FractionBox[
    RowBox[{"m", "-", 
     FractionBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", 
          SuperscriptBox["g", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"12", " ", 
             SuperscriptBox["u0", "2"]}]}], ")"}], " ", "\[Delta]u0"}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "4"}], "+", 
            RowBox[{"48", " ", 
             SuperscriptBox["u0", "2"]}]}], ")"}], " ", 
          SuperscriptBox["\[Delta]u0", "3"]}], "-", 
         RowBox[{"24", " ", 
          RowBox[{"(", 
           RowBox[{"u0", "+", 
            RowBox[{"4", " ", 
             SuperscriptBox["u0", "3"]}]}], ")"}], " ", "\[Delta]u0", " ", 
          "\[Delta]u1"}], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"1", "+", 
             RowBox[{"4", " ", 
              SuperscriptBox["u0", "2"]}]}], ")"}], "2"], " ", 
          "\[Delta]u2"}]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["u0", "2"]}]}], ")"}], "3"]], "+", 
     FractionBox[
      RowBox[{"768", " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "3"], " ", "u0", " ", 
       RowBox[{"TC", "[", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{
            SubscriptBox["J", "1"], "[", 
            RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "t"], ",", 
         FractionBox["1", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           SuperscriptBox["\[ExponentialE]", "t"]}]], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["u0", "2"]}]}], ")"}], "2"]]}], 
    SuperscriptBox["tran", "3"]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "4"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 0, 4, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[1], {
    Log[(Complex[0, -1] + 2 $CellContext`u0)^(-3) (Complex[0, 1] + 
       2 $CellContext`u0)^3], 
    Complex[0, -12] (1 + 4 $CellContext`u0^2)^(-1) $CellContext`\[Delta]u0, 
    Complex[0, 4] (1 + 4 $CellContext`u0^2)^(-2) (
     32 $CellContext`g^2 $CellContext`u0 - 
     3 ((-4) $CellContext`u0 $CellContext`\[Delta]u0^2 + \
$CellContext`\[Delta]u1 + 
      4 $CellContext`u0^2 $CellContext`\[Delta]u1)), $CellContext`m + 
    Complex[0, -4] (1 + 4 $CellContext`u0^2)^(-3) (
      32 $CellContext`g^2 (-1 + 
        12 $CellContext`u0^2) $CellContext`\[Delta]u0 + (-4 + 
        48 $CellContext`u0^2) $CellContext`\[Delta]u0^3 - 
      24 ($CellContext`u0 + 
       4 $CellContext`u0^3) $CellContext`\[Delta]u0 $CellContext`\[Delta]u1 + 
      3 (1 + 4 $CellContext`u0^2)^2 $CellContext`\[Delta]u2) + 
    Complex[0, 768] $CellContext`g^3 $CellContext`u0 (1 + 
       4 $CellContext`u0^2)^(-2) $CellContext`TC[$CellContext`t^(-1) 
       Subscript[$CellContext`J, 1][2 $CellContext`g $CellContext`t] 
       Subscript[$CellContext`J, 2][2 $CellContext`g $CellContext`t], (-1 + 
        E^$CellContext`t)^(-1), {$CellContext`t, 0, 
        DirectedInfinity[1]}]}, 0, 4, 1],
  Editable->False]], "Output",
 CellChangeTimes->{
  3.565439697963337*^9, {3.565439768127071*^9, 3.5654397787271557`*^9}, {
   3.565439875750525*^9, 3.565440004498258*^9}, 3.565440055940892*^9, 
   3.565440088063991*^9, 3.5654409249095993`*^9, 3.5654409857780333`*^9, 
   3.565441046772807*^9, 3.565441386673959*^9, 3.565441862667708*^9, 
   3.567155789079711*^9}]
}, Open  ]],

Cell["\<\
Leading transcendentality equation gives us:\
\>", "Text",
 CellChangeTimes->{{3.565440999187346*^9, 3.565441006912977*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"\[ImaginaryI]", "+", 
        RowBox[{"2", " ", "u0"}]}], ")"}], "3"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], "+", 
        RowBox[{"2", " ", "u0"}]}], ")"}], "3"]], "\[Equal]", "1"}], ",", 
   "u0"}], "]"}]], "Input",
 CellChangeTimes->{{3.565441010557221*^9, 3.565441016108404*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u0", "\[Rule]", 
     RowBox[{"-", 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["3"]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u0", "\[Rule]", 
     FractionBox["1", 
      RowBox[{"2", " ", 
       SqrtBox["3"]}]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.565441016478874*^9}]
}, Open  ]],

Cell["We pick a positive solution.", "Text",
 CellChangeTimes->{{3.565441026864665*^9, 3.565441051265081*^9}}],

Cell[BoxData[
 RowBox[{"\[Delta]u0", "=", "0.", " ", 
  RowBox[{"(*", " ", 
   RowBox[{"from", " ", 
    SuperscriptBox["tran", 
     RowBox[{"-", "1"}]], " ", "equation"}], " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.5654410523258543`*^9, 3.565441062681353*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " equation"
}], "Text",
 CellChangeTimes->{{3.565441035247649*^9, 3.5654410382074347`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"4", " ", "\[ImaginaryI]", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"32", " ", 
            SuperscriptBox["g", "2"], " ", "u0"}], "-", 
           RowBox[{"3", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "4"}], " ", "u0", " ", 
               SuperscriptBox["\[Delta]u0", "2"]}], "+", "\[Delta]u1", "+", 
              RowBox[{"4", " ", 
               SuperscriptBox["u0", "2"], " ", "\[Delta]u1"}]}], ")"}]}]}], 
          ")"}]}], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "+", 
            RowBox[{"4", " ", 
             SuperscriptBox["u0", "2"]}]}], ")"}], "2"], " ", 
         SuperscriptBox["tran", "2"]}]], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"u0", "\[Rule]", "v00"}], ",", 
         RowBox[{"\[Delta]u0", "\[Rule]", "0"}]}], "}"}]}], ")"}], "\[Equal]",
      "0"}], ",", "\[Delta]u1"}], "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.565441073315675*^9, 3.565441098402832*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[Delta]u1", "\[Rule]", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox["g", "2"]}], 
    SqrtBox["3"]]}], "}"}]], "Output",
 CellChangeTimes->{{3.565441095135964*^9, 3.565441098781157*^9}, 
   3.567155825306964*^9}]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "3"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " equation"
}], "Text",
 CellChangeTimes->{{3.565441100973724*^9, 3.565441106253319*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          SuperscriptBox["tran", "3"]], 
         RowBox[{"(", 
          RowBox[{"m", "-", 
           FractionBox[
            RowBox[{"4", " ", "\[ImaginaryI]", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"32", " ", 
                SuperscriptBox["g", "2"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  RowBox[{"12", " ", 
                   SuperscriptBox["u0", "2"]}]}], ")"}], " ", "\[Delta]u0"}], 
               "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "4"}], "+", 
                  RowBox[{"48", " ", 
                   SuperscriptBox["u0", "2"]}]}], ")"}], " ", 
                SuperscriptBox["\[Delta]u0", "3"]}], "-", 
               RowBox[{"24", " ", 
                RowBox[{"(", 
                 RowBox[{"u0", "+", 
                  RowBox[{"4", " ", 
                   SuperscriptBox["u0", "3"]}]}], ")"}], " ", "\[Delta]u0", 
                " ", "\[Delta]u1"}], "+", 
               RowBox[{"3", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{"1", "+", 
                   RowBox[{"4", " ", 
                    SuperscriptBox["u0", "2"]}]}], ")"}], "2"], " ", 
                "\[Delta]u2"}]}], ")"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"1", "+", 
               RowBox[{"4", " ", 
                SuperscriptBox["u0", "2"]}]}], ")"}], "3"]], "+", 
           FractionBox[
            RowBox[{"768", " ", "\[ImaginaryI]", " ", 
             SuperscriptBox["g", "3"], " ", "u0", " ", 
             RowBox[{"TC", "[", 
              RowBox[{
               FractionBox[
                RowBox[{
                 RowBox[{
                  SubscriptBox["J", "1"], "[", 
                  RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
                 RowBox[{
                  SubscriptBox["J", "2"], "[", 
                  RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "t"], ",", 
               FractionBox["1", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", 
                 SuperscriptBox["\[ExponentialE]", "t"]}]], ",", 
               RowBox[{"{", 
                RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"1", "+", 
               RowBox[{"4", " ", 
                SuperscriptBox["u0", "2"]}]}], ")"}], "2"]]}], ")"}], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"u0", "\[Rule]", "v00"}], ",", 
           RowBox[{"\[Delta]u0", "\[Rule]", "0"}], ",", 
           RowBox[{"\[Delta]u1", "\[Rule]", 
            FractionBox[
             RowBox[{"4", " ", 
              SuperscriptBox["g", "2"]}], 
             SqrtBox["3"]]}]}], "}"}]}], ")"}], "\[Equal]", "0"}], ",", 
      "\[Delta]u2"}], "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], "//", "Expand"}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.565441121876444*^9, 3.5654411568462343`*^9}, 
   3.565441411093933*^9, {3.565441874861992*^9, 3.5654418785530148`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[Delta]u2", "\[Rule]", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "m"}], "9"]}], "+", 
    RowBox[{"8", " ", 
     SqrtBox["3"], " ", 
     SuperscriptBox["g", "3"], " ", 
     RowBox[{"TC", "[", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{
          SubscriptBox["J", "1"], "[", 
          RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "2"], "[", 
          RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "t"], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", "t"]}]], ",", 
       RowBox[{"{", 
        RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.565441143929578*^9, 3.565441157075653*^9}, 
   3.565441412264351*^9, 3.565441879437046*^9, 3.567155828070586*^9}]
}, Open  ]],

Cell["\<\
Now we paste this for expression for anomalous dimension\
\>", "Text",
 CellChangeTimes->{{3.56544116183702*^9, 3.565441172130631*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"4", "\[ImaginaryI]", " ", "g", 
         RowBox[{"(", 
          RowBox[{
           FractionBox["1", 
            RowBox[{"x$h", "[", "1", "]"}]], "-", 
           FractionBox["1", 
            RowBox[{"x$h", "[", 
             RowBox[{"-", "1"}], "]"}]]}], ")"}]}], "/.", "subJuk"}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"(", 
         RowBox[{"u0", "+", 
          RowBox[{
           FractionBox["1", "tran"], "\[Delta]u0"}], "+", 
          RowBox[{
           FractionBox["1", 
            SuperscriptBox["tran", "2"]], "\[Delta]u1"}], "+", 
          RowBox[{
           FractionBox["1", 
            SuperscriptBox["tran", "3"]], "\[Delta]u2"}]}], ")"}]}]}], "/.", 
      "subtotran"}], ",", 
     RowBox[{"{", 
      RowBox[{"tran", ",", "\[Infinity]", ",", "5"}], "}"}]}], "]"}], "//.", 
   "densqrt3"}], "//", "Simplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u0", "\[Rule]", "v00"}], ",", 
     RowBox[{"\[Delta]u0", "\[Rule]", "0"}], ",", 
     RowBox[{"\[Delta]u1", "\[Rule]", 
      FractionBox[
       RowBox[{"4", " ", 
        SuperscriptBox["g", "2"]}], 
       SqrtBox["3"]]}], ",", 
     RowBox[{"\[Delta]u2", "\[Rule]", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "m"}], "9"]}], "+", 
       RowBox[{"8", " ", 
        SqrtBox["3"], " ", 
        SuperscriptBox["g", "3"], " ", 
        RowBox[{"TC", "[", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "1"], "[", 
             RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "t"], ",", 
          FractionBox["1", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            SuperscriptBox["\[ExponentialE]", "t"]}]], ",", 
          RowBox[{"{", 
           RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], 
         "]"}]}]}]}]}], "}"}]}], "//", "Simplify"}]}], "Input",
 CellChangeTimes->{{3.565441174544612*^9, 3.565441320045335*^9}, {
  3.565441423565589*^9, 3.5654414238918753`*^9}, {3.565441892202702*^9, 
  3.565441892482009*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{"16", " ", 
     SuperscriptBox["g", "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["u0", "2"]}]}], ")"}], " ", 
     SuperscriptBox["tran", "2"]}]], "-", 
   FractionBox[
    RowBox[{"128", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["g", "2"], " ", "u0", " ", "\[Delta]u0"}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["u0", "2"]}]}], ")"}], "2"], " ", 
     SuperscriptBox["tran", "3"]}]], "+", 
   FractionBox[
    RowBox[{"64", " ", 
     SuperscriptBox["g", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          RowBox[{"12", " ", 
           SuperscriptBox["u0", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          RowBox[{"12", " ", 
           SuperscriptBox["u0", "2"]}]}], ")"}], " ", 
        SuperscriptBox["\[Delta]u0", "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{"u0", "+", 
          RowBox[{"4", " ", 
           SuperscriptBox["u0", "3"]}]}], ")"}], " ", "\[Delta]u1"}]}], 
      ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["u0", "2"]}]}], ")"}], "3"], " ", 
     SuperscriptBox["tran", "4"]}]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["u0", "2"]}]}], ")"}], "4"], " ", 
      SuperscriptBox["tran", "5"]}]], 
    RowBox[{"128", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["g", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"24", " ", 
          SuperscriptBox["g", "2"], " ", "u0", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"4", " ", 
             SuperscriptBox["u0", "2"]}]}], ")"}], " ", "\[Delta]u0"}], "+", 
         RowBox[{"\[Delta]u0", " ", "\[Delta]u1"}], "-", 
         RowBox[{"8", " ", 
          SuperscriptBox["u0", "2"], " ", "\[Delta]u0", " ", "\[Delta]u1"}], 
         "-", 
         RowBox[{"48", " ", 
          SuperscriptBox["u0", "4"], " ", "\[Delta]u0", " ", "\[Delta]u1"}], 
         "+", 
         RowBox[{"16", " ", 
          SuperscriptBox["u0", "5"], " ", "\[Delta]u2"}], "+", 
         RowBox[{"u0", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "8"}], " ", 
             SuperscriptBox["\[Delta]u0", "3"]}], "+", "\[Delta]u2"}], 
           ")"}]}], "+", 
         RowBox[{"8", " ", 
          SuperscriptBox["u0", "3"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"4", " ", 
             SuperscriptBox["\[Delta]u0", "3"]}], "+", "\[Delta]u2"}], 
           ")"}]}]}], ")"}]}], ")"}]}]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "6"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 2, 6, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[1], {
   16 $CellContext`g^2/(1 + 
    4 $CellContext`u0^2), (-128) $CellContext`g^2 $CellContext`u0 (1 + 
      4 $CellContext`u0^2)^(-2) $CellContext`\[Delta]u0, 
    64 $CellContext`g^2 (1 + 
      4 $CellContext`u0^2)^(-3) ($CellContext`g^2 (-1 + 
       12 $CellContext`u0^2) + (-1 + 
       12 $CellContext`u0^2) $CellContext`\[Delta]u0^2 - 
     2 ($CellContext`u0 + 
      4 $CellContext`u0^3) $CellContext`\[Delta]u1), (-128) $CellContext`g^2 (
      1 + 4 $CellContext`u0^2)^(-4) (
     24 $CellContext`g^2 $CellContext`u0 (-1 + 
       4 $CellContext`u0^2) $CellContext`\[Delta]u0 + $CellContext`\[Delta]u0 \
$CellContext`\[Delta]u1 - 
     8 $CellContext`u0^2 $CellContext`\[Delta]u0 $CellContext`\[Delta]u1 - 
     48 $CellContext`u0^4 $CellContext`\[Delta]u0 $CellContext`\[Delta]u1 + 
     16 $CellContext`u0^5 $CellContext`\[Delta]u2 + $CellContext`u0 ((-8) \
$CellContext`\[Delta]u0^3 + $CellContext`\[Delta]u2) + 
     8 $CellContext`u0^3 (
       4 $CellContext`\[Delta]u0^3 + $CellContext`\[Delta]u2))}, 2, 6, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.56544126529836*^9, 3.565441321168089*^9}, 
   3.565441424410124*^9, 3.565441892837647*^9, 3.5671558334476633`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox["g", "2"]}], 
    SuperscriptBox["tran", "2"]], "-", 
   FractionBox[
    RowBox[{"48", " ", 
     SuperscriptBox["g", "4"]}], 
    SuperscriptBox["tran", "4"]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", 
       SuperscriptBox["g", "2"], " ", "m"}], 
      SqrtBox["3"]], "-", 
     RowBox[{"288", " ", 
      SuperscriptBox["g", "5"], " ", 
      RowBox[{"TC", "[", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{
           SubscriptBox["J", "1"], "[", 
           RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "t"], ",", 
        FractionBox["1", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["\[ExponentialE]", "t"]}]], ",", 
        RowBox[{"{", 
         RowBox[{"t", ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}]}]}], 
    SuperscriptBox["tran", "5"]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "6"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 2, 6, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[1], {
   12 $CellContext`g^2, 0, (-48) $CellContext`g^4, 
    Complex[0, 4] 3^Rational[-1, 2] $CellContext`g^2 $CellContext`m - 
    288 $CellContext`g^5 $CellContext`TC[$CellContext`t^(-1) 
      Subscript[$CellContext`J, 1][2 $CellContext`g $CellContext`t] 
      Subscript[$CellContext`J, 2][2 $CellContext`g $CellContext`t], (-1 + 
       E^$CellContext`t)^(-1), {$CellContext`t, 0, 
       DirectedInfinity[1]}]}, 2, 6, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.56544126529836*^9, 3.565441321168089*^9}, 
   3.565441424410124*^9, 3.565441892837647*^9, 3.567155833488566*^9}]
}, Open  ]],

Cell["\<\
We can explicitly check this equation\
\>", "Text",
 CellChangeTimes->{{3.565441443834484*^9, 3.565441449472299*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", "288"}], " ", 
      SuperscriptBox["g", "5"], 
      RowBox[{
       SubscriptBox["J", "1"], "[", 
       RowBox[{"2", " ", "g", " ", "t"}], "]"}], " ", 
      RowBox[{
       SubscriptBox["J", "2"], "[", 
       RowBox[{"2", " ", "g", " ", "t"}], "]"}]}], "/.", "subJBes"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", "0", ",", "15"}], "}"}]}], "]"}], "/.", 
  RowBox[{
   SuperscriptBox["t", "a_"], "\[RuleDelayed]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Zeta]$", "[", "a", "]"}], "[", "0", "]"}], 
    RowBox[{"Gamma", "[", "a", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.565441479891883*^9, 3.565441508363441*^9}, 
   3.565441898503175*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"288", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"], " ", 
     SuperscriptBox["g", "8"]}]}], "+", 
   RowBox[{"2880", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "5"], " ", 
    SuperscriptBox["g", "10"]}], "-", 
   RowBox[{"30240", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "7"], " ", 
    SuperscriptBox["g", "12"]}], "+", 
   RowBox[{"338688", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "9"], " ", 
    SuperscriptBox["g", "14"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "16"],
    SeriesData[$CellContext`g, 0, {}, 8, 16, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 
   0, {(-288) $CellContext`\[Zeta]$[3][0], 0, 
    2880 $CellContext`\[Zeta]$[5][0], 0, (-30240) $CellContext`\[Zeta]$[7][0],
     0, 338688 $CellContext`\[Zeta]$[9][0]}, 8, 16, 1],
  Editable->False]], "Output",
 CellChangeTimes->{
  3.5654413317625427`*^9, {3.56544148294528*^9, 3.56544150862846*^9}, 
   3.565441898910779*^9, 3.567155838368312*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Gamma]", "0"], "/.", 
      RowBox[{"leadweak", "[", "7", "]"}]}], ")"}], "-", 
    SubscriptBox["\[Gamma]", "0"]}], "/.", 
   RowBox[{"leadweak", "[", "2", "]"}]}], "//", "FindHighTranCoef"}]], "Input",\

 CellChangeTimes->{{3.5654415117469053`*^9, 3.565441549736001*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "288"}], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{"2880", " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"30240", " ", 
   SuperscriptBox["g", "12"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{"338688", " ", 
   SuperscriptBox["g", "14"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}]}]], "Output",
 CellChangeTimes->{{3.565441523468565*^9, 3.565441549980364*^9}, 
   3.56544189967235*^9, 3.5671558467866707`*^9}]
}, Open  ]],

Cell["\<\
So we have a very good control on the leading correction to asymptotic energy \
at any order. We have to find leading m. This should not be very difficult.\
\>", "Text",
 CellChangeTimes->{{3.565441910206748*^9, 3.565441952853574*^9}, {
  3.567155878782481*^9, 3.567155880047696*^9}}],

Cell["\<\
First, we look for the correction for h.\
\>", "Text",
 CellChangeTimes->{{3.567157222075618*^9, 3.56715722855307*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", "h", ")"}], "r"], "]"}], "==", 
  RowBox[{"Log", "[", " ", 
   FractionBox[
    RowBox[{
     OverscriptBox[
      StyleBox["h",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], " ", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{
       FractionBox["L", "2"], "+", "1"}]]}], 
    SubscriptBox["\[Sigma]", "1"]], "]"}], "==", 
  RowBox[{"\[ScriptCapitalZ]", "*", 
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         StyleBox["\[ScriptCapitalF]",
          FontColor->RGBColor[0, 0.6, 0]], "+"], 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{
           SubscriptBox["Y", 
            RowBox[{"1", ",", "1"}]], 
           SubscriptBox["Y", 
            RowBox[{"2", ",", "2"}]]}]}], ")"}]}], "\[Rho]"], ")"}], "r"], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5479657623344088`*^9, 3.5479658621787853`*^9}, {
   3.547965923675784*^9, 3.5479659385131187`*^9}, 3.5521933809999647`*^9, {
   3.5629356795425262`*^9, 3.562935705007962*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       OverscriptBox[
        StyleBox["h",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", 
        RowBox[{"+", "o"}], "]"}]], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["h",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{"[", 
        RowBox[{"-", "o"}], "]"}]]}], ")"}], "r"], "]"}], "\[Equal]", 
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        StyleBox["\[ScriptCapitalF]",
         FontColor->RGBColor[0, 0.6, 0]], "+"], 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SubscriptBox["Y", 
           RowBox[{"1", ",", "1"}]], 
          SubscriptBox["Y", 
           RowBox[{"2", ",", "2"}]]}]}], ")"}]}], "\[Rho]"], ")"}], "r"], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.567156133820956*^9, 3.5671561553669853`*^9}, {
  3.567156191254209*^9, 3.5671561977981*^9}}],

Cell[TextData[{
 "Instead of computing convolution with Z, we can explicitly use RH problem \
to see that the integrand is symmetric with respect to x->1/x. Then, to \
define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", "h", ")"}], "r"], TraditionalForm]]],
 ", we take only negative powers of x, and 1/2 of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "0"], TraditionalForm]]],
 ".  The latter is normalization."
}], "Text",
 CellChangeTimes->{{3.5653778860978727`*^9, 3.565377962322044*^9}, {
  3.5671560953849487`*^9, 3.567156130181128*^9}, {3.567156164917111*^9, 
  3.56715617514704*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Contribution from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox["1", "\[Rho]"], ")"}], "r"], "]"}], TraditionalForm]]]
}], "Subsubsection",
 CellChangeTimes->{{3.552194573693015*^9, 3.552194586631557*^9}, {
  3.552195423782762*^9, 3.552195423851169*^9}, {3.5558723898227663`*^9, 
  3.555872393032803*^9}, {3.555873196517317*^9, 3.555873196727844*^9}, {
  3.560854916011812*^9, 3.560854917941217*^9}, {3.5629370657237186`*^9, 
  3.5629370658899813`*^9}, {3.565377971957675*^9, 3.565377974235964*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", "\[Rho]"], ")"}], "r"], "]"}], "\[Equal]", 
  RowBox[{"-", 
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", "\[Rho]", ")"}], "r"], "]"}]}], "\[Equal]", 
  RowBox[{"-", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "+", 
     FractionBox["\[Delta]\[Rho]", 
      SubscriptBox["\[Rho]", "as"]]}], "]"}]}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     FractionBox["\[Delta]\[Rho]", 
      SubscriptBox["\[Rho]", "as"]], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["\[Delta]\[Rho]", 
       SubscriptBox["\[Rho]", "as"]], ")"}], "2"]}]}]}]], "Input",
 CellChangeTimes->{{3.565378010596705*^9, 3.5653780663106413`*^9}}],

Cell["\<\
This will capture two leading transcendentalities. For \[Delta]\[Rho] I have \
explicit expression already. I am not ready to compute subleading one, so we \
compute only leading order here.\
\>", "Text",
 CellChangeTimes->{{3.565378069507272*^9, 3.565378089335938*^9}, {
  3.565378124193701*^9, 3.565378138742568*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Rho]", "/.", "largevolume"}], "/.", 
       RowBox[{"u", "\[Rule]", 
        RowBox[{"2", "g", " ", "z"}]}]}], "/.", "subJuk"}], "/.", 
     RowBox[{"leadweak", "[", "5", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", "0", ",", "9"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellChangeTimes->{{3.5653781977645893`*^9, 3.565378237095169*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"2", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", "g"}], "-", 
   RowBox[{"6", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    SuperscriptBox["g", "3"]}], "+", 
   RowBox[{"42", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    SuperscriptBox["g", "5"]}], "-", 
   RowBox[{"366", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    SuperscriptBox["g", "7"]}], "+", 
   RowBox[{"12", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], " ", 
    RowBox[{"(", 
     RowBox[{"293", "+", 
      RowBox[{"12", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "9"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "10"],
    SeriesData[$CellContext`g, 0, {}, 1, 10, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   2 (1 - $CellContext`z^2)^Rational[1, 2], 
    0, (-6) (1 - $CellContext`z^2)^Rational[1, 2], 0, 
    42 (1 - $CellContext`z^2)^Rational[1, 2], 
    0, (-366) (1 - $CellContext`z^2)^Rational[1, 2], 0, 
    12 (1 - $CellContext`z^2)^Rational[1, 2] (293 + 
     12 $CellContext`\[Zeta]$[3][0])}, 1, 10, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.565378203400234*^9, 3.565378238468275*^9}}]
}, Open  ]],

Cell["\<\
So, at the leading order we have\
\>", "Text",
 CellChangeTimes->{{3.5653782607774067`*^9, 3.5653782652563457`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"(", 
    FractionBox["\[Delta]\[Rho]", 
     SubscriptBox["\[Rho]", "as"]], ")"}]}], "==", 
  RowBox[{"-", 
   RowBox[{"(", 
    FractionBox["\[Delta]\[Rho]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", "g", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", 
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]]]}], "+", 
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]]}], ")"}]}]], ")"}]}], "==", 
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Delta]\[Rho]"}], 
    RowBox[{"g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", 
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]]]}], "+", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], ")"}]}]]}]}]], "Input",
 CellChangeTimes->{{3.5653782969945602`*^9, 3.5653783414422626`*^9}, {
  3.567156343922941*^9, 3.567156344993823*^9}}],

Cell["With \[Delta]\[Rho] given by:", "Text",
 CellChangeTimes->{{3.5653783461524*^9, 3.5653783526730013`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "3"}], 
  SuperscriptBox["g", "2"], 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", "1"}], 
    RowBox[{"+", "\[Infinity]"}]], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], 
        RowBox[{
         RowBox[{"2", "n"}], "-", "1"}]]], "-", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], 
       RowBox[{
        RowBox[{"2", "n"}], "-", "1"}]]}], ")"}], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "n"}], "-", "1"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
        RowBox[{"2", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "1"]}], "]"}], "+", 
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5653784300381804`*^9, 3.565378449430183*^9}}],

Cell["\<\
Expression is obviously symmetric in x->x1/x. We can easily resum it properly\
\
\>", "Text",
 CellChangeTimes->{{3.565378460154917*^9, 3.5653785055925837`*^9}, {
  3.565379011268139*^9, 3.565379025179016*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Delta]\[Rho]"}], 
    RowBox[{"g", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", 
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]]]}], "+", 
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]]}], ")"}]}]]}], "==", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "3"}], "g"}], ")"}], 
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"n", "=", "1"}], 
     RowBox[{"+", "\[Infinity]"}]], 
    RowBox[{
     FractionBox[
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         SuperscriptBox[
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]]], "-", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], 
         RowBox[{
          RowBox[{"2", "n"}], "-", "1"}]]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", 
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]]]}], "+", 
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]]}], ")"}]], 
     RowBox[{"TC", "[", 
      RowBox[{
       RowBox[{"24", "  ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "n"}], "-", "1"}], ")"}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "n"], 
        FractionBox[
         RowBox[{
          SubscriptBox["J", 
           RowBox[{
            RowBox[{"2", "n"}], "-", "1"}]], "[", 
          RowBox[{"2", "g", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
         RowBox[{"2", "g", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "1"]}], "]"}], "+", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "72"}], " ", "g"}], ")"}], 
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], 
     RowBox[{"+", "\[Infinity]"}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], 
         RowBox[{"2", "n"}]]], "+", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], 
        RowBox[{"2", "n"}]], "-", 
       SubscriptBox["\[Delta]", 
        RowBox[{"n", ",", "0"}]]}], ")"}], 
     RowBox[{"TC", "[", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["\[Sum]", 
           RowBox[{"m", "=", 
            RowBox[{"n", "+", "1"}]}], "\[Infinity]"], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "m"}], "-", "1"}], ")"}], 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "m"], 
           FractionBox[
            RowBox[{
             SubscriptBox["J", 
              RowBox[{
               RowBox[{"2", "m"}], "-", "1"}]], "[", 
             RowBox[{"2", "g", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "-", 
                SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
            RowBox[{"2", "g", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]]}]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "1"]}], "]"}], "+", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "72"}], " ", "g"}], ")"}], 
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], 
     RowBox[{"+", "\[Infinity]"}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], 
         RowBox[{"2", "n"}]]], "+", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], 
        RowBox[{"2", "n"}]], "-", 
       SubscriptBox["\[Delta]", 
        RowBox[{"n", ",", "0"}]]}], ")"}], 
     RowBox[{"TC", "[", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["\[Sum]", 
           RowBox[{"m", "=", 
            RowBox[{"n", "+", "1"}]}], "\[Infinity]"], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "m"}], "-", "1"}], ")"}], 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "m"], 
           FractionBox[
            RowBox[{
             SubscriptBox["J", 
              RowBox[{
               RowBox[{"2", "m"}], "-", "1"}]], "[", 
             RowBox[{"2", "g", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "-", 
                SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
            RowBox[{"2", "g", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]]}]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "1"]}], "]"}], "+", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}], "==", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"m", "=", 
       RowBox[{"n", "+", "1"}]}], "\[Infinity]"], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", "m"}], "-", "1"}], ")"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", 
         RowBox[{
          RowBox[{"2", "m"}], "-", "1"}]], "[", 
        RowBox[{"2", "g", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
       RowBox[{"2", "g", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}]]}]}], "==", 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"m", "=", 
       RowBox[{"n", "+", "1"}]}], "\[Infinity]"], 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], "2"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "m"}], "-", "2"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], "+", 
        RowBox[{
         SubscriptBox["J", 
          RowBox[{"2", "m"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}]}], ")"}]}]}], "==", 
    RowBox[{
     FractionBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], 
       RowBox[{"n", "+", "1"}]], "2"], 
     RowBox[{
      SubscriptBox["J", 
       RowBox[{"2", "n"}]], "[", 
      RowBox[{"2", "g", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}], "]"}]}]}], "*)"}], "\[Equal]", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"36", " ", "g"}], ")"}], 
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], 
     RowBox[{"+", "\[Infinity]"}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         OverscriptBox[
          StyleBox["x",
           FontColor->RGBColor[0, 0.6, 0]], 
          StyleBox["^",
           FontColor->RGBColor[0, 0.6, 0]]], 
         RowBox[{"2", "n"}]]], "+", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], 
        RowBox[{"2", "n"}]], "-", 
       SubscriptBox["\[Delta]", 
        RowBox[{"n", ",", "0"}]]}], ")"}], 
     RowBox[{"TC", "[", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "n"], 
        RowBox[{
         SubscriptBox["J", 
          RowBox[{"2", "n"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "1"]}], "]"}], "+", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{
               SubscriptBox["J", "4"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], 
              RowBox[{
               SubscriptBox["J", "3"], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "1"]}], "]"}]}], 
             SubscriptBox["z", "1"]], ",", 
            SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]]}]], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["z", "2"], ",", "0", ",", 
         SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.5653790302879963`*^9, 3.565379208140895*^9}, {
  3.565379333477602*^9, 3.5653793584921017`*^9}, {3.565379401524815*^9, 
  3.5653794152258997`*^9}}],

Cell[TextData[{
 "In particular, we confirm that at ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["g", "8"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " there are no corrections, apart constant one, which was of course expected."
}], "Text",
 CellChangeTimes->{{3.5653796494776163`*^9, 3.565379697706354*^9}, {
  3.5653797493322678`*^9, 3.565379752944159*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Contribution from ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{"1", "-", 
       RowBox[{
        SubscriptBox["Y", 
         RowBox[{"1", ",", "1"}]], 
        SubscriptBox["Y", 
         RowBox[{"2", ",", "2"}]]}]}], 
      SubscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SubscriptBox["Y", 
           RowBox[{"1", ",", "1"}]], 
          SubscriptBox["Y", 
           RowBox[{"2", ",", "2"}]]}]}], ")"}], "asym"]], "]"}], "=", 
    "contYtYtranTOh"}], TraditionalForm]]]
}], "Subsubsection",
 CellChangeTimes->{{3.552194573693015*^9, 3.552194586631557*^9}, {
  3.552195423782762*^9, 3.552195451528096*^9}, {3.555874053686809*^9, 
  3.555874057466106*^9}, {3.5608549217622538`*^9, 3.560854921762404*^9}, {
  3.5629360219227753`*^9, 3.562936021922933*^9}, {3.562937063860057*^9, 
  3.5629370644449177`*^9}, {3.5671586940318537`*^9, 3.5671586942246847`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   FractionBox[
    RowBox[{"1", "-", 
     RowBox[{
      SubscriptBox["Y", 
       RowBox[{"1", ",", "1"}]], 
      SubscriptBox["Y", 
       RowBox[{"2", ",", "2"}]]}]}], 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{
        SubscriptBox["Y", 
         RowBox[{"1", ",", "1"}]], 
        SubscriptBox["Y", 
         RowBox[{"2", ",", "2"}]]}]}], ")"}], "asym"]], "]"}], "\[Equal]", 
  RowBox[{"Log", "[", 
   FractionBox[
    RowBox[{"1", "-", 
     RowBox[{"Y11Y22as", 
      RowBox[{"(", 
       RowBox[{"1", "+", "o"}], ")"}]}]}], 
    RowBox[{"1", "-", "Y11Y22as"}]], "]"}], "\[Equal]", 
  RowBox[{"o", 
   FractionBox[
    RowBox[{" ", "Y11Y22as"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "Y11Y22as"}]]}], "==", 
  RowBox[{
   FractionBox[
    RowBox[{" ", "Y11Y22as"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "Y11Y22as"}]], 
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["Y", 
        RowBox[{"1", ",", "1"}]], 
       SubscriptBox["Y", 
        RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}]}], "\[Equal]", 
  "\[IndentingNewLine]", 
  RowBox[{
   FractionBox[
    RowBox[{" ", "Y11Y22as"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "Y11Y22as"}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
     SqrtBox[
      RowBox[{"1", "-", 
       SuperscriptBox["z", "2"]}]], 
     RowBox[{"\[DoubleStruckCapitalC]", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{
         RowBox[{"4", 
          SuperscriptBox["g", "2"], 
          SuperscriptBox["z", "2"]}], "-", 
         SuperscriptBox["u", "2"]}]], 
       FractionBox["H", 
        SqrtBox[
         RowBox[{"1", "-", 
          FractionBox[
           RowBox[{"4", 
            SuperscriptBox["g", "2"]}], 
           SuperscriptBox["u", "2"]]}]]]}], "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.552195453278018*^9, 3.552195497757874*^9}, {
  3.555873272457108*^9, 3.555873298282538*^9}, {3.555873412927331*^9, 
  3.555873441120566*^9}, {3.562936059143304*^9, 3.562936080280541*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          FractionBox[
           RowBox[{" ", "Y11Y22"}], 
           RowBox[{
            RowBox[{"-", "1"}], "+", "Y11Y22"}]], "/.", "largevolume"}], "/.",
          "subFunsBethe"}], "/.", "subJuk"}], "/.", 
       RowBox[{"leadweak", "[", "4", "]"}]}], "/.", 
      RowBox[{"u", "\[Rule]", 
       RowBox[{"2", "g", " ", "z"}]}]}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "3"}], "}"}]}], "]"}], "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"tmp1", "=", 
  RowBox[{
   RowBox[{"%", "//", "FullSimplify"}], "//", "Normal"}]}]}], "Input",
 CellChangeTimes->{{3.5521955031953583`*^9, 3.5521955123162107`*^9}, {
  3.552195599887342*^9, 3.552195650005445*^9}, {3.5558734853231087`*^9, 
  3.555873486278224*^9}, {3.555873579143408*^9, 3.555873592358659*^9}, {
  3.555873859352264*^9, 3.5558738612814083`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], "+", 
  FractionBox["1", 
   RowBox[{"12", " ", "g", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]]}]], "+", 
  FractionBox[
   RowBox[{"g", " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      RowBox[{"3", " ", 
       SuperscriptBox["z", "2"]}]}], ")"}]}], 
   RowBox[{"3", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]]}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["g", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "7"}], "+", 
      RowBox[{"6", " ", 
       SuperscriptBox["z", "2"]}]}], ")"}]}], 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]]]}]], "Output",
 CellChangeTimes->{{3.5521955054096622`*^9, 3.552195513179824*^9}, {
   3.552195597572797*^9, 3.552195652164297*^9}, 3.5558735079787188`*^9, {
   3.55587358349611*^9, 3.555873596019369*^9}, 3.555873881120298*^9, 
   3.560336891400983*^9, 3.562936150242326*^9, 3.5629362157306223`*^9}]
}, Open  ]],

Cell["\<\

As we want only the leading order in transcendentality, we compute only this \
thing.\
\>", "Text",
 CellChangeTimes->{{3.567158289745501*^9, 3.5671583256385*^9}, {
  3.567237935619996*^9, 3.567237935760708*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"12", " ", "g", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]]}]], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
    SqrtBox[
     RowBox[{"1", "-", 
      SuperscriptBox["z", "2"]}]], 
    RowBox[{"\[DoubleStruckCapitalC]", "[", 
     RowBox[{
      FractionBox["1", 
       RowBox[{
        RowBox[{"4", 
         SuperscriptBox["g", "2"], 
         SuperscriptBox["z", "2"]}], "-", 
        SuperscriptBox["u", "2"]}]], 
      FractionBox["H", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]]}], "]"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.56715831531854*^9, 3.567158318679248*^9}}],

Cell["\<\
And integrand was already discussed once:\
\>", "Text",
 CellChangeTimes->{{3.567158329126082*^9, 3.5671583359096317`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "4"}], "\[ImaginaryI]", " ", "g", 
   SqrtBox[
    RowBox[{"1", "-", 
     SuperscriptBox["z", "2"]}]], 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{
       RowBox[{"4", 
        SuperscriptBox["g", "2"], 
        SuperscriptBox["z", "2"]}], "-", 
       SuperscriptBox["u", "2"]}]], 
     FractionBox["H", 
      SqrtBox[
       RowBox[{"1", "-", 
        FractionBox[
         RowBox[{"4", 
          SuperscriptBox["g", "2"]}], 
         SuperscriptBox["u", "2"]]}]]]}], "]"}]}], "==", 
  "rhsYtYtran"}]], "Input",
 CellChangeTimes->{{3.567158467603812*^9, 3.567158476459936*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"precontYtYtranTOh", "=", 
  RowBox[{
   RowBox[{"rhsYtYtran", " ", 
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"12", " ", "g", " ", 
       SqrtBox[
        RowBox[{"1", "-", 
         SuperscriptBox["z", "2"]}]]}]], ")"}]}], "/.", 
   RowBox[{
    RowBox[{"r_", " ", 
     RowBox[{"TC", "[", 
      RowBox[{"a_", ",", "b__"}], "]"}]}], "\[RuleDelayed]", 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{"r", " ", "a"}], ",", "g"}], "]"}], ",", "b"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.567158483398682*^9, 3.567158485482047*^9}, {
  3.5671586156415663`*^9, 3.567158687940754*^9}, {3.567159113031212*^9, 
  3.567159113300427*^9}}],

Cell[BoxData[
 RowBox[{"TC", "[", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", "\[ImaginaryI]", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
           
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"2", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
      SuperscriptBox["g", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            SubscriptBox["J", "2"], "\[Prime]",
            MultilineFunction->None], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "1"]], "+", 
        FractionBox[
         RowBox[{"12", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            SubscriptBox["J", "3"], "\[Prime]",
            MultilineFunction->None], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "1"]], "+", 
        FractionBox[
         RowBox[{"4", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SuperscriptBox[
            SubscriptBox["J", "4"], "\[Prime]",
            MultilineFunction->None], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "1"]]}], ")"}]}], "z"]}], ",", 
   FractionBox["1", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["\[ExponentialE]", 
      SubscriptBox["z", "1"]]}]], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "2"], ",", "0", ",", 
     SubscriptBox["z", "1"]}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{
  3.567158486408741*^9, {3.567158624497115*^9, 3.567158688472951*^9}, 
   3.567159113877761*^9, 3.567236654552421*^9}]
}, Open  ]],

Cell[TextData[{
 "Now we flip derivative over ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ":"
}], "Text",
 CellChangeTimes->{{3.56715886677932*^9, 3.567158874097068*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"contYtYtranTOh", "=", 
  RowBox[{"precontYtYtranTOh", "/.", 
   RowBox[{
    RowBox[{"TC", "[", 
     RowBox[{"a_", ",", "b__"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"TC", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"D", "[", 
           RowBox[{
            RowBox[{"a", "/.", 
             RowBox[{
              RowBox[{
               SuperscriptBox[
                SubscriptBox["J", "e_"], "\[Prime]",
                MultilineFunction->None], "[", 
               RowBox[{"2", " ", "g", " ", 
                SubscriptBox["z", "2"]}], "]"}], "\[RuleDelayed]", 
              FractionBox[
               RowBox[{
                SubscriptBox["JJ", "e"], "[", 
                RowBox[{"2", " ", "g", " ", "zz"}], "]"}], 
               RowBox[{"2", "g"}]]}]}], ",", 
            SubscriptBox["z", "2"]}], "]"}]}], "/.", 
         RowBox[{"JJ", "\[Rule]", "J"}]}], "/.", 
        RowBox[{"zz", "\[Rule]", 
         SubscriptBox["z", "2"]}]}], "//", "Factor"}], ",", "b"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.567158884403352*^9, 3.5671589572478933`*^9}, {
   3.567159031546015*^9, 3.5671591160314407`*^9}, {3.567159536586239*^9, 
   3.567159598673902*^9}, 3.567159707126206*^9, {3.5672366508390827`*^9, 
   3.567236651586141*^9}}],

Cell[BoxData[
 RowBox[{"TC", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}]], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"4", " ", "\[ImaginaryI]", " ", "g", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], " ", 
     SuperscriptBox["g", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", 
        RowBox[{
         SubscriptBox["J", "3"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}], " ", 
        RowBox[{
         SubscriptBox["J", "3"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["J", "2"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}], " ", 
        RowBox[{
         SubscriptBox["J", "4"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         SubscriptBox["J", "2"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "1"]}], "]"}], " ", 
        RowBox[{
         SubscriptBox["J", "4"], "[", 
         RowBox[{"2", " ", "g", " ", 
          SubscriptBox["z", "2"]}], "]"}]}]}], ")"}]}], 
    SubscriptBox["z", "1"]], ",", 
   FractionBox["1", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["\[ExponentialE]", 
      SubscriptBox["z", "1"]]}]], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "2"], ",", "0", ",", 
     SubscriptBox["z", "1"]}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{{3.567158924802001*^9, 3.567158957510023*^9}, {
   3.567159073420554*^9, 3.567159083179871*^9}, 3.567159116394639*^9, {
   3.56715958147685*^9, 3.567159598916404*^9}, 3.5671597076126757`*^9, 
   3.567236656056513*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Contribution from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Log", "[", 
    FractionBox[
     SuperscriptBox[
      StyleBox["\[ScriptCapitalF]",
       FontColor->RGBColor[0, 0.6, 0]], "+"], 
     SubscriptBox[
      RowBox[{"(", 
       SuperscriptBox[
        StyleBox["\[ScriptCapitalF]",
         FontColor->RGBColor[0, 0.6, 0]], "+"], ")"}], "asym"]], "]"}], 
   TraditionalForm]]],
 " - it is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " order, so does not contribute so far (this section may have mistakes, but ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["tran", 
    RowBox[{"-", "2"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " - is correct statment)"
}], "Subsubsection",
 CellChangeTimes->{{3.552194573693015*^9, 3.552194586631557*^9}, {
  3.552195423782762*^9, 3.552195451528096*^9}, {3.5521965050485697`*^9, 
  3.552196518074411*^9}, {3.552196569144054*^9, 3.552196575356393*^9}, {
  3.552205045118082*^9, 3.55220504515469*^9}, {3.555874436885995*^9, 
  3.555874443483198*^9}, {3.555928460336525*^9, 3.555928464641596*^9}, {
  3.5608549240866327`*^9, 3.560854924086772*^9}, {3.562937062259419*^9, 
  3.5629370624502563`*^9}, {3.567237949736704*^9, 3.567237981886539*^9}}],

Cell["\<\
I am changing sign-type regularization from our paper to something more \
physical\
\>", "Text",
 CellChangeTimes->{{3.562936511390167*^9, 3.562936528275776*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[ScriptCapitalF]", "+"], "=", 
  RowBox[{
   SubscriptBox["\[CapitalLambda]", "F"], 
   RowBox[{
    SubscriptBox["\[Product]", "j"], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[ImaginaryI]", " ", 
       RowBox[{"Cosh", "[", 
        RowBox[{"\[Pi]", " ", 
         RowBox[{"(", 
          RowBox[{"u", "-", 
           SubscriptBox["u", "j"]}], ")"}]}], "]"}]}], ")"}], 
     RowBox[{"Exp", "[", 
      RowBox[{
       SubscriptBox["\[Integral]", 
        OverscriptBox["Z", "\[Vee]"]], 
       RowBox[{
        FractionBox["dv", 
         RowBox[{"2", "\[ImaginaryI]"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Coth", "[", 
           RowBox[{"\[Pi]", 
            RowBox[{"(", 
             RowBox[{"u", "-", "v"}], ")"}]}], "]"}], "+", 
          RowBox[{"Sign", "[", "v", "]"}]}], ")"}], 
        RowBox[{"Log", "[", 
         RowBox[{
          SubscriptBox["Y", 
           RowBox[{"1", ",", "1"}]], 
          SubscriptBox["Y", 
           RowBox[{"2", ",", "2"}]]}], "]"}]}]}], "]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.547966072743767*^9, 3.54796607412173*^9}, {
  3.547966236317339*^9, 3.547966294388851*^9}, {3.547966826922048*^9, 
  3.5479668505432377`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox["\[ScriptCapitalF]", 
      SubscriptBox["\[ScriptCapitalF]", "asym"]], ")"}], "+"], "]"}], "=", 
  RowBox[{
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["dv", 
      RowBox[{"2", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Coth", "[", 
        RowBox[{"\[Pi]", 
         RowBox[{"(", 
          RowBox[{"u", "-", "v"}], ")"}]}], "]"}], "-", 
       SubscriptBox[
        RowBox[{"Coth", "[", 
         RowBox[{"\[Pi]", " ", "u"}], "]"}], "pv"]}], ")"}], 
     SubscriptBox[
      RowBox[{"Log", "[", 
       SubscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["Y", 
           RowBox[{"1", ",", "1"}]], 
          SubscriptBox["Y", 
           RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}], "pv"]}]}], "+", 
   "const"}]}]], "Input",
 CellChangeTimes->{{3.547966491674753*^9, 3.547966543809491*^9}, {
   3.547966856640164*^9, 3.5479668725649157`*^9}, {3.5479669237629766`*^9, 
   3.547966932264473*^9}, 3.55219692680199*^9, {3.5521975315545397`*^9, 
   3.552197558495037*^9}, {3.562936535548952*^9, 3.5629365414209833`*^9}, {
   3.562936597281616*^9, 3.562936598105906*^9}, {3.562936673552699*^9, 
   3.5629366757553864`*^9}, {3.562936805288239*^9, 3.562936805613645*^9}}],

Cell["\<\
We consider v far from the real axis for the moment (say, in the UHP). But \
using the residue of Coth we can bring it to the real axis\
\>", "Text",
 CellChangeTimes->{{3.562936601904418*^9, 3.5629366331671352`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{"Coth", "[", 
    RowBox[{"\[Pi]", " ", "u"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"-", "1"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.562936634221208*^9, 3.562936654828315*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{"\[Pi]", " ", "u"}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "u", "]"}], "0"],
    SeriesData[$CellContext`u, 0, {}, -1, 0, 1],
    Editable->False]}],
  SeriesData[$CellContext`u, 0, {Pi^(-1)}, -1, 0, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.562936655230219*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox["dv", 
     RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]], 
    FractionBox["1", 
     RowBox[{"u", "-", "v"}]], 
    SubscriptBox[
     RowBox[{"Log", "[", 
      SubscriptBox[
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["Y", 
          RowBox[{"1", ",", "1"}]], 
         SubscriptBox["Y", 
          RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}], "pv"]}]}], 
  "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    RowBox[{"Log", "[", 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["Y", 
         RowBox[{"1", ",", "1"}]], 
        SubscriptBox["Y", 
         RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}]}], "+", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["dv", 
      RowBox[{"2", "\[Pi]", " ", "\[ImaginaryI]"}]], 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"u", "-", 
          RowBox[{"Re", "[", "v", "]"}]}]], 
        RowBox[{"Log", "[", 
         SubscriptBox[
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["Y", 
             RowBox[{"1", ",", "1"}]], 
            SubscriptBox["Y", 
             RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}]}], ")"}], 
      "pv"]}]}]}]}]], "Input",
 CellChangeTimes->{{3.56293666440204*^9, 3.562936711004149*^9}, {
  3.562936792380061*^9, 3.562936798760304*^9}}],

Cell[TextData[{
 "Now, -1/2 precisely cancells ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["g", "odd"], TraditionalForm]]],
 " terms from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Log", "[", 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "Y11Y22"}], ")"}], "r"], "]"}], TraditionalForm]]],
 ". We hence get for \[ScriptCapitalF] only nontrivial integral:"
}], "Text",
 CellChangeTimes->{{3.562936715023093*^9, 3.562936765113146*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   SubscriptBox[
    RowBox[{"(", 
     SuperscriptBox["\[ScriptCapitalF]", "+"], ")"}], "r"], "]"}], "=", 
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox["dv", 
     RowBox[{"2", "\[ImaginaryI]"}]], 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Coth", "[", 
          RowBox[{"\[Pi]", 
           RowBox[{"(", 
            RowBox[{"u", "-", "v"}], ")"}]}], "]"}], "-", 
         RowBox[{"Coth", "[", 
          RowBox[{"\[Pi]", " ", "v"}], "]"}]}], ")"}], 
       RowBox[{"Log", "[", 
        SubscriptBox[
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["Y", 
            RowBox[{"1", ",", "1"}]], 
           SubscriptBox["Y", 
            RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}]}], ")"}], 
     "pv"]}]}]}]], "Input",
 CellChangeTimes->{{3.562936770220223*^9, 3.562936787533502*^9}, 
   3.562936901721937*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", " ", 
   SubscriptBox[
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["Y", 
       RowBox[{"1", ",", "1"}]], 
      SubscriptBox["Y", 
       RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}], "\[Equal]", 
  RowBox[{
   OverscriptBox["H", "_"], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "-", 
      FractionBox["1", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{"-", "\[Infinity]"}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      FractionBox["dv", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      FractionBox["1", 
       RowBox[{"u", "-", "v"}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", 
          RowBox[{
           RowBox[{
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "[", 
            RowBox[{"v", "+", 
             RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}], "-", 
           FractionBox["1", 
            RowBox[{
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "[", 
             RowBox[{"v", "+", 
              RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]]}]], 
         RowBox[{
          OverscriptBox["H", "_"], "[", 
          RowBox[{"v", "+", 
           RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", 
          RowBox[{
           RowBox[{
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "[", 
            RowBox[{"v", "-", 
             RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}], "-", 
           FractionBox["1", 
            RowBox[{
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "[", 
             RowBox[{"v", "-", 
              RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]]}]], 
         RowBox[{"H", "[", 
          RowBox[{"v", "-", 
           RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]}]}], 
       ")"}]}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.55327677118294*^9, 3.5532767827944307`*^9}, {
   3.553276964695347*^9, 3.553276978010294*^9}, {3.553277069062745*^9, 
   3.5532771608290577`*^9}, {3.5532773407853813`*^9, 3.5532773410890007`*^9}, 
   3.5671608577744637`*^9}],

Cell["And it is useful to use", "Text",
 CellChangeTimes->{{3.553277434862742*^9, 3.553277439668838*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     FractionBox["u", "g"], 
     SqrtBox[
      RowBox[{"1", "-", 
       FractionBox[
        RowBox[{"4", 
         SuperscriptBox["g", "2"]}], 
        SuperscriptBox["u", "2"]]}]]}], "==", 
    RowBox[{"xh", "-", 
     FractionBox["1", "xh"]}]}], ",", 
   RowBox[{
    RowBox[{"2", "\[ImaginaryI]", 
     SqrtBox[
      RowBox[{"1", "-", 
       FractionBox[
        SuperscriptBox["u", "2"], 
        RowBox[{"4", 
         SuperscriptBox["g", "2"]}]]}]]}], "\[Equal]", 
    RowBox[{"x", "-", 
     FractionBox["1", "x"]}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.553277444586698*^9, 3.55327744784196*^9}, {
  3.5532816764752893`*^9, 3.553281732656057*^9}}],

Cell["Let us make some arrangements.", "Text",
 CellChangeTimes->{{3.567168323136883*^9, 3.5671683348163567`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Log", "[", " ", 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["Y", 
        RowBox[{"1", ",", "1"}]], 
       SubscriptBox["Y", 
        RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}], "\[Equal]", 
   RowBox[{
    OverscriptBox["H", "_"], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "-", 
       FractionBox["1", 
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", 
       RowBox[{"-", "\[Infinity]"}], 
       RowBox[{"+", "\[Infinity]"}]], 
      RowBox[{
       FractionBox["dv", 
        RowBox[{
         RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          RowBox[{"u", "-", "v"}]], "-", 
         FractionBox["1", 
          RowBox[{"u", "+", "v"}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          RowBox[{
           RowBox[{
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "[", 
            RowBox[{"v", "-", 
             RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}], "-", 
           FractionBox["1", 
            RowBox[{
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "[", 
             RowBox[{"v", "-", 
              RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]]}]], 
         RowBox[{"H", "[", 
          RowBox[{"v", "-", 
           RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]}], ")"}]}]}]}]}]}], 
  " ", 
  RowBox[{"(*", " ", "UHP", " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Log", "[", " ", 
   SubscriptBox[
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["Y", 
       RowBox[{"1", ",", "1"}]], 
      SubscriptBox["Y", 
       RowBox[{"2", ",", "2"}]]}], ")"}], "r"], "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", "H"}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "-", 
      FractionBox["1", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{"-", "\[Infinity]"}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      FractionBox["dv", 
       RowBox[{
        RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"u", "-", "v"}]], "-", 
        FractionBox["1", 
         RowBox[{"u", "+", "v"}]]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         RowBox[{
          RowBox[{
           OverscriptBox[
            StyleBox["x",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "[", 
           RowBox[{"v", "-", 
            RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}], "-", 
          FractionBox["1", 
           RowBox[{
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "[", 
            RowBox[{"v", "-", 
             RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]]}]], 
        RowBox[{"H", "[", 
         RowBox[{"v", "-", 
          RowBox[{"\[ImaginaryI]", " ", "0"}]}], "]"}]}], ")"}], " ", 
      RowBox[{"(*", " ", 
       RowBox[{"LHP", ",", " ", 
        RowBox[{"continuation", " ", "through", " ", "long", " ", "cut"}], 
        ",", " ", 
        RowBox[{"avoiding", " ", "short"}]}], " ", "*)"}]}]}]}]}]}]}], "Input",\

 CellChangeTimes->{{3.567168348653139*^9, 3.567168424906509*^9}, {
  3.5671688377172813`*^9, 3.5671688434704857`*^9}, {3.567168977666025*^9, 
  3.567168985576364*^9}}],

Cell["\<\
The part with integral - first integral should be taken explicitly:\
\>", "Text",
 CellChangeTimes->{{3.567168571081428*^9, 3.567168586422409*^9}, {
  3.567168623007234*^9, 3.567168626077119*^9}, {3.567169582139612*^9, 
  3.5671695829599123`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox["du", 
     RowBox[{"2", "\[ImaginaryI]"}]], 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Coth", "[", 
        RowBox[{"\[Pi]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"g", 
            RowBox[{"(", 
             RowBox[{
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], "+", 
              FractionBox["1", 
               OverscriptBox[
                StyleBox["w",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], "-", "u"}], 
          ")"}]}], "]"}], 
       OverscriptBox["Yf", "^"]}], ")"}], "pv"]}]}], "\[Rule]", 
  RowBox[{
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[Pi]", " ", 
          RowBox[{"Coth", "[", 
           RowBox[{"\[Pi]", 
            RowBox[{"(", 
             RowBox[{"u", "-", 
              RowBox[{"g", 
               RowBox[{"(", 
                RowBox[{
                 OverscriptBox[
                  StyleBox["w",
                   FontColor->RGBColor[0, 0.6, 0]], 
                  StyleBox["^",
                   FontColor->RGBColor[0, 0.6, 0]]], "+", 
                 FractionBox["1", 
                  OverscriptBox[
                   StyleBox["w",
                    FontColor->RGBColor[0, 0.6, 0]], 
                   StyleBox["^",
                    FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], ")"}]}], 
           "]"}]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "-", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox["1", 
           RowBox[{"u", "-", "v"}]], "-", 
          FractionBox["1", 
           RowBox[{"u", "+", "v"}]]}], ")"}]}], ")"}], "pv"]}]}], "\[Equal]", 
   "\[IndentingNewLine]", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     SubscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["\[Eta]", "1", 
            RowBox[{"[", "2", "]"}]], "[", 
           RowBox[{"u", "-", 
            RowBox[{"g", 
             RowBox[{"(", 
              RowBox[{
               OverscriptBox[
                StyleBox["w",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]], "+", 
               FractionBox["1", 
                OverscriptBox[
                 StyleBox["w",
                  FontColor->RGBColor[0, 0.6, 0]], 
                 StyleBox["^",
                  FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
          
          FractionBox["1", 
           RowBox[{"u", "-", 
            RowBox[{"g", 
             RowBox[{"(", 
              RowBox[{
               OverscriptBox[
                StyleBox["w",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]], "+", 
               FractionBox["1", 
                OverscriptBox[
                 StyleBox["w",
                  FontColor->RGBColor[0, 0.6, 0]], 
                 StyleBox["^",
                  FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}]], "+", 
          RowBox[{
           SubsuperscriptBox[
            OverscriptBox["\[Eta]", "_"], "1", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}]], "[", 
           RowBox[{"u", "-", 
            RowBox[{"g", 
             RowBox[{"(", 
              RowBox[{
               OverscriptBox[
                StyleBox["w",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]], "+", 
               FractionBox["1", 
                OverscriptBox[
                 StyleBox["w",
                  FontColor->RGBColor[0, 0.6, 0]], 
                 StyleBox["^",
                  FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], 
         ")"}], 
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "-", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["x",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox["1", 
           RowBox[{"u", "-", "v"}]], "-", 
          FractionBox["1", 
           RowBox[{"u", "+", "v"}]]}], ")"}]}], ")"}], "pv"]}]}]}]}]], "Input",\

 CellChangeTimes->{{3.567168643277689*^9, 3.5671686965445538`*^9}, {
   3.5671694259790382`*^9, 3.567169473220902*^9}, 3.567169505736031*^9, {
   3.5671698793809547`*^9, 3.567169890070013*^9}}],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  FractionBox["1", 
   RowBox[{"u", "-", 
    RowBox[{"g", 
     RowBox[{"(", 
      RowBox[{
       OverscriptBox[
        StyleBox["w",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "+", 
       FractionBox["1", 
        OverscriptBox[
         StyleBox["w",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}]]],
  CellChangeTimes->{{3.567168643277689*^9, 3.5671686965445538`*^9}, {
    3.5671694259790382`*^9, 3.567169473220902*^9}, 3.567169505736031*^9}],
 " is giving zero anyway (we are doing principal value integration), for the \
other we use u->-u symmetry to get"
}], "Text",
 CellChangeTimes->{{3.567169810888829*^9, 3.567169857285009*^9}, {
  3.567169893214198*^9, 3.5671698932143497`*^9}}],

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   FractionBox["du", 
    RowBox[{
     RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
   SubscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubsuperscriptBox["\[Eta]", "1", 
          RowBox[{"[", "2", "]"}]], "[", 
         RowBox[{"u", "-", 
          RowBox[{"g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "+", 
             FractionBox["1", 
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
        RowBox[{
         SubsuperscriptBox["\[Eta]", "1", 
          RowBox[{"[", "2", "]"}]], "[", 
         RowBox[{"u", "+", 
          RowBox[{"g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "+", 
             FractionBox["1", 
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], 
       ")"}], 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-", 
          FractionBox["1", 
           OverscriptBox[
            StyleBox["x",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"u", "-", "v"}]], "-", 
        FractionBox["1", 
         RowBox[{"u", "+", "v"}]]}], ")"}]}], ")"}], "pv"]}]}]], "Input",
 CellChangeTimes->{{3.567169863904828*^9, 3.56716989766619*^9}}],

Cell["\<\
Expression is explcitly symmetric in the output variable (as it should). \
Integration slightly above real axis will give zero, we hence integrate \
slightly below real axis and we get\
\>", "Text",
 CellChangeTimes->{{3.567169904893716*^9, 3.567169951449389*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "-", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "+", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          OverscriptBox[
           StyleBox["x",
            FontColor->RGBColor[0, 0.6, 0]], 
           StyleBox["^",
            FontColor->RGBColor[0, 0.6, 0]]], "-", 
          FractionBox["1", 
           OverscriptBox[
            StyleBox["x",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}], 
        RowBox[{"[", 
         RowBox[{"-", "o"}], "]"}]]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "-", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "+", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]]}], ")"}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "==", 
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "-", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "+", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FractionBox["u", "g"], "-", 
          FractionBox["2", 
           OverscriptBox[
            StyleBox["x",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}], 
        RowBox[{"[", 
         RowBox[{"-", "o"}], "]"}]]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "-", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "+", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]]}], ")"}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     FractionBox["v", "g"], ")"}], 
    RowBox[{"(", 
     FractionBox["1", "2"], ")"}], 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], 
     ")"}]}], "+", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "-", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "+", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], ")"}], 
     SuperscriptBox[
      RowBox[{"(", 
       FractionBox["1", 
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]]], ")"}], 
      RowBox[{"[", 
       RowBox[{"-", "o"}], "]"}]], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "-", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "+", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]]}], ")"}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     FractionBox["v", "g"], ")"}], 
    RowBox[{"(", 
     FractionBox["1", "2"], ")"}], 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], 
     ")"}]}], "+", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "-", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "+", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], "+", 
            FractionBox["1", 
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
       RowBox[{
        RowBox[{"\[DifferentialD]", 
         SubscriptBox["z", "2"]}], " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", "u", " ", 
          SubscriptBox["z", "2"]}]], 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "1"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "2"]]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "-", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "+", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]]}], ")"}]}]}]}], "\[Equal]", 
  "\[IndentingNewLine]", "==", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     FractionBox["v", "g"], ")"}], 
    RowBox[{"(", 
     FractionBox["1", "2"], ")"}], 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], 
     ")"}]}], "+", 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
       RowBox[{
        FractionBox[
         RowBox[{"\[DifferentialD]", 
          SubscriptBox["z", "1"]}], 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           SubscriptBox["z", "1"]], "-", "1"}]], 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"\[ImaginaryI]", " ", "u", " ", 
          SubscriptBox["z", "1"]}]], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", 
            SubscriptBox["z", "1"], " ", "g", 
            RowBox[{"(", 
             RowBox[{
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], "+", 
              FractionBox["1", 
               OverscriptBox[
                StyleBox["w",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]], "+", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"+", "\[ImaginaryI]"}], " ", 
            SubscriptBox["z", "1"], " ", "g", 
            RowBox[{"(", 
             RowBox[{
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], "+", 
              FractionBox["1", 
               OverscriptBox[
                StyleBox["w",
                 FontColor->RGBColor[0, 0.6, 0]], 
                StyleBox["^",
                 FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"-", "\[ImaginaryI]"}], ")"}]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
       RowBox[{
        RowBox[{"\[DifferentialD]", 
         SubscriptBox["z", "2"]}], " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", "u", " ", 
          SubscriptBox["z", "2"]}]], 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
           SubscriptBox["J", "1"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         SubscriptBox["z", "2"]]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "-", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "+", "v", "-", 
         RowBox[{"\[ImaginaryI]", " ", "0"}]}]]}], ")"}]}]}]}], "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     FractionBox["v", "g"], ")"}], 
    RowBox[{"(", 
     FractionBox["1", "2"], ")"}], 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "+", 
      RowBox[{
       SubsuperscriptBox["\[Eta]", "1", 
        RowBox[{"[", "2", "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "+", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}], "-", 
      RowBox[{
       SubsuperscriptBox[
        OverscriptBox["\[Eta]", "_"], "1", 
        RowBox[{"[", 
         RowBox[{"-", "2"}], "]"}]], "[", 
       RowBox[{"v", "-", 
        RowBox[{"g", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox[
            StyleBox["w",
             FontColor->RGBColor[0, 0.6, 0]], 
            StyleBox["^",
             FontColor->RGBColor[0, 0.6, 0]]], "+", 
           FractionBox["1", 
            OverscriptBox[
             StyleBox["w",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]}], "]"}]}], 
     ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
      RowBox[{
       FractionBox[
        RowBox[{"\[DifferentialD]", 
         SubscriptBox["z", "1"]}], 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]], "-", "1"}]], 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "v", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]], "-", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", "v", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "-", 
             SubscriptBox["z", "2"]}], ")"}]}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", 
           SubscriptBox["z", "1"], " ", "g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "+", 
             FractionBox["1", 
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"+", "\[ImaginaryI]"}], " ", 
           SubscriptBox["z", "1"], " ", "g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox[
              StyleBox["w",
               FontColor->RGBColor[0, 0.6, 0]], 
              StyleBox["^",
               FontColor->RGBColor[0, 0.6, 0]]], "+", 
             FractionBox["1", 
              OverscriptBox[
               StyleBox["w",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]]]}], ")"}]}]]}], ")"}], 
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}]}]}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", 
       SubscriptBox["z", "1"]], 
      RowBox[{
       RowBox[{"\[DifferentialD]", 
        SubscriptBox["z", "2"]}], " ", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{
          SubscriptBox["J", "1"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], 
        SubscriptBox["z", "2"]]}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5671699610127373`*^9, 3.567169984845992*^9}, {
   3.567170975754492*^9, 3.5671709874450197`*^9}, {3.567171019354773*^9, 
   3.5671711384483137`*^9}, {3.567171352849967*^9, 3.5671714402269573`*^9}, {
   3.5671714712271957`*^9, 3.567171489947905*^9}, {3.567171541831773*^9, 
   3.567171547899065*^9}, {3.567171727813278*^9, 3.5671718244044724`*^9}, {
   3.567171871845549*^9, 3.567171891384115*^9}, {3.567172045675672*^9, 
   3.567172054215065*^9}, {3.567223188502069*^9, 3.567223246211441*^9}, 
   3.5672232774669857`*^9, {3.567233168290769*^9, 3.567233173981722*^9}}],

Cell["\<\
Now we have to integrate with H\
\>", "Text",
 CellChangeTimes->{{3.567233035707939*^9, 3.567233043785318*^9}}],

Cell[BoxData[
 RowBox[{"H", "\[Equal]", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
     FractionBox[
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.567233123997607*^9, 3.567233125990552*^9}}],

Cell["\<\
For H we will make bessel representation\
\>", "Text",
 CellChangeTimes->{{3.567233185285795*^9, 3.567233205893532*^9}}],

Cell[BoxData[
 RowBox[{"H", "\[Equal]", 
  RowBox[{"36", 
   SuperscriptBox["g", "2"], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "2", 
        RowBox[{"[", "2", "]"}]], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "3", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
     FractionBox[
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "4", 
       RowBox[{"[", "2", "]"}]], 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], ")"}]}], "\[Equal]", 
  RowBox[{"TC", "[", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", " ", "u", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["z", "1"], "-", 
         SubscriptBox["z", "2"]}], ")"}]}]], "36", " ", 
     SuperscriptBox["g", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"18", " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "3"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "2"]}]], "+", 
       FractionBox[
        RowBox[{"8", " ", 
         RowBox[{
          SubscriptBox["J", "2"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "2"]}]], "+", 
       FractionBox[
        RowBox[{"8", " ", 
         RowBox[{
          SubscriptBox["J", "2"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "1"]}], "]"}], " ", 
         RowBox[{
          SubscriptBox["J", "4"], "[", 
          RowBox[{"2", " ", "g", " ", 
           SubscriptBox["z", "2"]}], "]"}]}], 
        RowBox[{
         SubscriptBox["z", "1"], " ", 
         SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
    FractionBox["1", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       SubscriptBox["z", "1"]], "-", "1"}]], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.567233219520623*^9, 3.5672332750906153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"\[Integral]", 
    RowBox[{
     FractionBox["du", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", "\[ImaginaryI]"}]], 
     FractionBox["dv", 
      RowBox[{
       RowBox[{"-", "2"}], "\[Pi]", "\[ImaginaryI]"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "-", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox["w", "^"], "+", 
            FractionBox["1", 
             OverscriptBox["w", "^"]]}], ")"}]}]}], "]"}], "+", 
       RowBox[{
        SubsuperscriptBox["\[Eta]", "1", 
         RowBox[{"[", "2", "]"}]], "[", 
        RowBox[{"u", "+", 
         RowBox[{"g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox["w", "^"], "+", 
            FractionBox["1", 
             OverscriptBox["w", "^"]]}], ")"}]}]}], "]"}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          OverscriptBox["x", "^"], "-", 
          FractionBox["1", 
           OverscriptBox["x", "^"]]}], ")"}], 
        RowBox[{"[", 
         RowBox[{"-", "o"}], "]"}]]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "-", "v", "-", 
         RowBox[{"\[ImaginaryI]", "0"}]}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "+", "v", "-", 
         RowBox[{"\[ImaginaryI]", "0"}]}]]}], ")"}], 
     RowBox[{"H", "[", "v", "]"}]}]}]}], "\[Equal]", "\[Equal]", 
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox["dv", 
     RowBox[{
      RowBox[{"-", "2"}], "\[Pi]", "\[ImaginaryI]"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        FractionBox["1", "g"], ")"}], 
       RowBox[{"(", 
        FractionBox["1", "2"], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
         RowBox[{
          RowBox[{"\[DifferentialD]", 
           SubscriptBox["zz", "1"]}], 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{"\[ImaginaryI]", " ", "v", " ", 
              SubscriptBox["zz", "1"]}]], "-", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], " ", "v", " ", 
              SubscriptBox["zz", "1"]}]]}], ")"}], 
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{
             SubscriptBox["\[PartialD]", 
              SubscriptBox["zz", "1"]], 
             FractionBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["\[ExponentialE]", 
                 RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], 
                  SubscriptBox["zz", "1"], "g", 
                  RowBox[{"(", 
                   RowBox[{
                    OverscriptBox["w", "^"], "+", 
                    FractionBox["1", 
                    OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
                SuperscriptBox["\[ExponentialE]", 
                 RowBox[{
                  RowBox[{"+", "\[ImaginaryI]"}], 
                  SubscriptBox["zz", "1"], "g", 
                  RowBox[{"(", 
                   RowBox[{
                    OverscriptBox["w", "^"], "+", 
                    FractionBox["1", 
                    OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
              RowBox[{
               SuperscriptBox["\[ExponentialE]", 
                SubscriptBox["zz", "1"]], "-", "1"}]]}]}], ")"}]}]}], ")"}]}],
       "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
         RowBox[{
          FractionBox[
           RowBox[{"\[DifferentialD]", 
            SubscriptBox["zz", "1"]}], 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             SubscriptBox["zz", "1"]], "-", "1"}]], 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{"\[ImaginaryI]", "v", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["zz", "1"], "-", 
                SubscriptBox["zz", "2"]}], ")"}]}]], "-", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], "v", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["zz", "1"], "-", 
                SubscriptBox["zz", "2"]}], ")"}]}]]}], ")"}], 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], 
              SubscriptBox["zz", "1"], "g", 
              RowBox[{"(", 
               RowBox[{
                OverscriptBox["w", "^"], "+", 
                FractionBox["1", 
                 OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"+", "\[ImaginaryI]"}], 
              SubscriptBox["zz", "1"], "g", 
              RowBox[{"(", 
               RowBox[{
                OverscriptBox["w", "^"], "+", 
                FractionBox["1", 
                 OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
          RowBox[{"(", 
           RowBox[{"-", "\[ImaginaryI]"}], ")"}]}]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "0", 
          SubscriptBox["zz", "1"]], 
         RowBox[{
          RowBox[{"\[DifferentialD]", 
           SubscriptBox["zz", "2"]}], 
          FractionBox[
           RowBox[{"\[ImaginaryI]", 
            RowBox[{
             SubscriptBox["J", "1"], "[", 
             RowBox[{"2", "g", 
              SubscriptBox["zz", "2"]}], "]"}]}], 
           SubscriptBox["zz", "2"]]}]}], ")"}]}]}], ")"}], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", "v", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], "36", 
       SuperscriptBox["g", "2"], 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"18", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "1"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "2"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], 
           SubscriptBox["z", "2"]}]], "+", 
         FractionBox[
          RowBox[{"8", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "1"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "2"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], 
           SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]], "-", "1"}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
     "]"}]}]}], "==", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       FractionBox["1", "g"], ")"}], 
      RowBox[{"(", 
       FractionBox["1", "2"], ")"}], 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
        RowBox[{
         RowBox[{"\[DifferentialD]", 
          SubscriptBox["zz", "1"]}], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"\[ImaginaryI]", " ", "v", " ", 
             SubscriptBox["zz", "1"]}]], "-", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "\[ImaginaryI]"}], " ", "v", " ", 
             SubscriptBox["zz", "1"]}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{
            SubscriptBox["\[PartialD]", 
             SubscriptBox["zz", "1"]], 
            FractionBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["\[ExponentialE]", 
                RowBox[{
                 RowBox[{"-", "\[ImaginaryI]"}], 
                 SubscriptBox["zz", "1"], "g", 
                 RowBox[{"(", 
                  RowBox[{
                   OverscriptBox["w", "^"], "+", 
                   FractionBox["1", 
                    OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
               SuperscriptBox["\[ExponentialE]", 
                RowBox[{
                 RowBox[{"+", "\[ImaginaryI]"}], 
                 SubscriptBox["zz", "1"], "g", 
                 RowBox[{"(", 
                  RowBox[{
                   OverscriptBox["w", "^"], "+", 
                   FractionBox["1", 
                    OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
             RowBox[{
              SuperscriptBox["\[ExponentialE]", 
               SubscriptBox["zz", "1"]], "-", "1"}]]}]}], ")"}]}]}], ")"}]}], 
     "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
        RowBox[{
         FractionBox[
          RowBox[{"\[DifferentialD]", 
           SubscriptBox["zz", "1"]}], 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            SubscriptBox["zz", "1"]], "-", "1"}]], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{"\[ImaginaryI]", "v", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["zz", "1"], "-", 
               SubscriptBox["zz", "2"]}], ")"}]}]], "-", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "\[ImaginaryI]"}], "v", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["zz", "1"], "-", 
               SubscriptBox["zz", "2"]}], ")"}]}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"-", "\[ImaginaryI]"}], 
             SubscriptBox["zz", "1"], "g", 
             RowBox[{"(", 
              RowBox[{
               OverscriptBox["w", "^"], "+", 
               FractionBox["1", 
                OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
           SuperscriptBox["\[ExponentialE]", 
            RowBox[{
             RowBox[{"+", "\[ImaginaryI]"}], 
             SubscriptBox["zz", "1"], "g", 
             RowBox[{"(", 
              RowBox[{
               OverscriptBox["w", "^"], "+", 
               FractionBox["1", 
                OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
         RowBox[{"(", 
          RowBox[{"-", "\[ImaginaryI]"}], ")"}]}]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["\[Integral]", "0", 
         SubscriptBox["zz", "1"]], 
        RowBox[{
         RowBox[{"\[DifferentialD]", 
          SubscriptBox["zz", "2"]}], 
         FractionBox[
          RowBox[{"\[ImaginaryI]", 
           RowBox[{
            SubscriptBox["J", "1"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["zz", "2"]}], "]"}]}], 
          SubscriptBox["zz", "2"]]}]}], ")"}]}]}], ")"}], 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"\[ImaginaryI]", "v", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}]], "36", 
      SuperscriptBox["g", "2"], 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"18", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", "g", 
            SubscriptBox["z", "1"]}], "]"}], 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", "g", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], 
          SubscriptBox["z", "2"]}]], "+", 
        FractionBox[
         RowBox[{"8", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", "g", 
            SubscriptBox["z", "2"]}], "]"}], 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", "g", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], 
          SubscriptBox["z", "2"]}]], "+", 
        FractionBox[
         RowBox[{"8", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", "g", 
            SubscriptBox["z", "1"]}], "]"}], 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", "g", 
            SubscriptBox["z", "2"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], 
          SubscriptBox["z", "2"]}]]}], ")"}]}], ",", 
     FractionBox["1", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]], "-", "1"}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", "\[Infinity]"}], "}"}]}], 
    "]"}]}], "\[Equal]", "\[IndentingNewLine]", "==", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        FractionBox["1", "g"], ")"}], 
       RowBox[{"(", 
        FractionBox["1", "2"], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
         RowBox[{
          RowBox[{"\[DifferentialD]", 
           SubscriptBox["zz", "1"]}], 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{"\[ImaginaryI]", " ", "v", " ", 
              SubscriptBox["zz", "1"]}]], "-", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], " ", "v", " ", 
              SubscriptBox["zz", "1"]}]]}], ")"}], 
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{
             SubscriptBox["\[PartialD]", 
              SubscriptBox["zz", "1"]], 
             FractionBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["\[ExponentialE]", 
                 RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], 
                  SubscriptBox["zz", "1"], "g", 
                  RowBox[{"(", 
                   RowBox[{
                    OverscriptBox["w", "^"], "+", 
                    FractionBox["1", 
                    OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
                SuperscriptBox["\[ExponentialE]", 
                 RowBox[{
                  RowBox[{"+", "\[ImaginaryI]"}], 
                  SubscriptBox["zz", "1"], "g", 
                  RowBox[{"(", 
                   RowBox[{
                    OverscriptBox["w", "^"], "+", 
                    FractionBox["1", 
                    OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
              RowBox[{
               SuperscriptBox["\[ExponentialE]", 
                SubscriptBox["zz", "1"]], "-", "1"}]]}]}], ")"}]}]}], ")"}]}],
       "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
         RowBox[{
          FractionBox[
           RowBox[{"\[DifferentialD]", 
            SubscriptBox["zz", "1"]}], 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             SubscriptBox["zz", "1"]], "-", "1"}]], 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{"\[ImaginaryI]", "v", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["zz", "1"], "-", 
                SubscriptBox["zz", "2"]}], ")"}]}]], "-", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], "v", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["zz", "1"], "-", 
                SubscriptBox["zz", "2"]}], ")"}]}]]}], ")"}], 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], 
              SubscriptBox["zz", "1"], "g", 
              RowBox[{"(", 
               RowBox[{
                OverscriptBox["w", "^"], "+", 
                FractionBox["1", 
                 OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"+", "\[ImaginaryI]"}], 
              SubscriptBox["zz", "1"], "g", 
              RowBox[{"(", 
               RowBox[{
                OverscriptBox["w", "^"], "+", 
                FractionBox["1", 
                 OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
          RowBox[{"(", 
           RowBox[{"-", "\[ImaginaryI]"}], ")"}]}]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "0", 
          SubscriptBox["zz", "1"]], 
         RowBox[{
          RowBox[{"\[DifferentialD]", 
           SubscriptBox["zz", "2"]}], 
          FractionBox[
           RowBox[{"\[ImaginaryI]", 
            RowBox[{
             SubscriptBox["J", "1"], "[", 
             RowBox[{"2", "g", 
              SubscriptBox["zz", "2"]}], "]"}]}], 
           SubscriptBox["zz", "2"]]}]}], ")"}]}]}], ")"}], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"-", "\[ImaginaryI]"}], 
        RowBox[{"2", "g"}]], "36", 
       SuperscriptBox["g", "2"], 
       RowBox[{"(", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", 
            SubscriptBox["z", "3"], "g", 
            RowBox[{"(", 
             RowBox[{
              OverscriptBox["w", "^"], "+", 
              FractionBox["1", 
               OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"+", "\[ImaginaryI]"}], " ", 
            SubscriptBox["z", "3"], "g", 
            RowBox[{"(", 
             RowBox[{
              OverscriptBox["w", "^"], "+", 
              FractionBox["1", 
               OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], ")"}], 
       RowBox[{
        SubscriptBox["\[PartialD]", 
         SubscriptBox["z", "3"]], 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"18", 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", "g", 
              SubscriptBox["z", "1"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", "g", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "-", 
                SubscriptBox["z", "3"]}], ")"}]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]], "+", 
          FractionBox[
           RowBox[{"8", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", "g", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "-", 
                SubscriptBox["z", "3"]}], ")"}]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", "g", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]], "+", 
          FractionBox[
           RowBox[{"8", 
            RowBox[{
             SubscriptBox["J", "2"], "[", 
             RowBox[{"2", "g", 
              SubscriptBox["z", "1"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", "g", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "-", 
                SubscriptBox["z", "3"]}], ")"}]}], "]"}]}], 
           RowBox[{
            SubscriptBox["z", "1"], 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"]}], ")"}]}]]}], ")"}]}]}], ",", 
      RowBox[{
       FractionBox["1", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "1"]], "-", "1"}]], 
       FractionBox["1", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          SubscriptBox["z", "3"]], "-", "1"}]]}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "3"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], "+", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      FractionBox["\[ImaginaryI]", 
       RowBox[{"2", "g"}]], "36", 
      SuperscriptBox["g", "2"], 
      RowBox[{"(", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", 
           SubscriptBox["z", "3"], "g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox["w", "^"], "+", 
             FractionBox["1", 
              OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"+", "\[ImaginaryI]"}], " ", 
           SubscriptBox["z", "3"], "g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox["w", "^"], "+", 
             FractionBox["1", 
              OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], ")"}], 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["z", "3"]], 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"18", 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "1"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "3"], "[", 
            RowBox[{"2", "g", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "+", 
               SubscriptBox["z", "3"]}], ")"}]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "+", 
             SubscriptBox["z", "3"]}], ")"}]}]], "+", 
         FractionBox[
          RowBox[{"8", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", "g", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "+", 
               SubscriptBox["z", "3"]}], ")"}]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "1"]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "+", 
             SubscriptBox["z", "3"]}], ")"}]}]], "+", 
         FractionBox[
          RowBox[{"8", 
           RowBox[{
            SubscriptBox["J", "2"], "[", 
            RowBox[{"2", "g", 
             SubscriptBox["z", "1"]}], "]"}], 
           RowBox[{
            SubscriptBox["J", "4"], "[", 
            RowBox[{"2", "g", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "+", 
               SubscriptBox["z", "3"]}], ")"}]}], "]"}]}], 
          RowBox[{
           SubscriptBox["z", "1"], 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["z", "1"], "+", 
             SubscriptBox["z", "3"]}], ")"}]}]]}], ")"}]}]}], ",", 
     RowBox[{
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]], "-", "1"}]], 
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]], "-", "1"}]]}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}]}], "]"}],
    "+", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          SubscriptBox["z", "3"], "g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox["w", "^"], "+", 
            FractionBox["1", 
             OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"+", "\[ImaginaryI]"}], " ", 
          SubscriptBox["z", "3"], "g", 
          RowBox[{"(", 
           RowBox[{
            OverscriptBox["w", "^"], "+", 
            FractionBox["1", 
             OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}], 
      FractionBox[
       RowBox[{"\[ImaginaryI]", 
        RowBox[{
         SubscriptBox["J", "1"], "[", 
         RowBox[{"2", "g", " ", 
          SubscriptBox["z", "4"]}], "]"}]}], 
       SubscriptBox["z", "4"]], 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"18", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "+", 
              SubscriptBox["z", "3"], "-", 
              SubscriptBox["z", "4"]}], ")"}]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "+", 
            SubscriptBox["z", "3"], "-", 
            SubscriptBox["z", "4"]}], ")"}]}]], "+", 
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "+", 
              SubscriptBox["z", "3"], "-", 
              SubscriptBox["z", "4"]}], ")"}]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "+", 
            SubscriptBox["z", "3"], "-", 
            SubscriptBox["z", "4"]}], ")"}]}]], "+", 
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "+", 
              SubscriptBox["z", "3"], "-", 
              SubscriptBox["z", "4"]}], ")"}]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "+", 
            SubscriptBox["z", "3"], "-", 
            SubscriptBox["z", "4"]}], ")"}]}]]}], ")"}]}], ",", 
     RowBox[{
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]], "-", "1"}]], 
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]], "-", "1"}]]}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "4"], ",", "0", ",", 
       SubscriptBox["z", "3"]}], "}"}]}], "]"}], "+", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", 
           SubscriptBox["z", "3"], "g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox["w", "^"], "+", 
             FractionBox["1", 
              OverscriptBox["w", "^"]]}], ")"}]}]], "+", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{
           RowBox[{"+", "\[ImaginaryI]"}], " ", 
           SubscriptBox["z", "3"], "g", 
           RowBox[{"(", 
            RowBox[{
             OverscriptBox["w", "^"], "+", 
             FractionBox["1", 
              OverscriptBox["w", "^"]]}], ")"}]}]]}], ")"}]}], 
      FractionBox[
       RowBox[{"\[ImaginaryI]", 
        RowBox[{
         SubscriptBox["J", "1"], "[", 
         RowBox[{"2", "g", " ", 
          SubscriptBox["z", "4"]}], "]"}]}], 
       SubscriptBox["z", "4"]], 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"18", " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "3"], "[", 
           RowBox[{"2", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"], "+", 
              SubscriptBox["z", "4"]}], ")"}]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "3"], "+", 
            SubscriptBox["z", "4"]}], ")"}]}]], "+", 
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"], "+", 
              SubscriptBox["z", "4"]}], ")"}]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "3"], "+", 
            SubscriptBox["z", "4"]}], ")"}]}]], "+", 
        FractionBox[
         RowBox[{"8", " ", 
          RowBox[{
           SubscriptBox["J", "2"], "[", 
           RowBox[{"2", " ", "g", " ", 
            SubscriptBox["z", "1"]}], "]"}], " ", 
          RowBox[{
           SubscriptBox["J", "4"], "[", 
           RowBox[{"2", " ", "g", " ", 
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["z", "1"], "-", 
              SubscriptBox["z", "3"], "+", 
              SubscriptBox["z", "4"]}], ")"}]}], "]"}]}], 
         RowBox[{
          SubscriptBox["z", "1"], " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "3"], "+", 
            SubscriptBox["z", "4"]}], ")"}]}]]}], ")"}]}], ",", 
     RowBox[{
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]], "-", "1"}]], 
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "3"]], "-", "1"}]]}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", 
       RowBox[{
        SubscriptBox["z", "3"], "-", 
        SubscriptBox["z", "4"]}], ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "3"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "4"], ",", "0", ",", 
       SubscriptBox["z", "3"]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.567233208076127*^9, 3.567233213411525*^9}, {
  3.567233285404158*^9, 3.567233379261589*^9}, {3.567233497302944*^9, 
  3.5672336639410067`*^9}, {3.56723372902283*^9, 3.5672337803687773`*^9}, {
  3.5672339370592957`*^9, 3.567233985992119*^9}, {3.5672342347127647`*^9, 
  3.567234428413756*^9}, {3.567234607709332*^9, 3.567234737298403*^9}, {
  3.567234786205374*^9, 3.5672348406623077`*^9}, {3.567234940497467*^9, 
  3.56723495243994*^9}, {3.567235015159288*^9, 3.567235024435279*^9}}],

Cell[CellGroupData[{

Cell[BoxData["subtoBes2"], "Input",
 CellChangeTimes->{{3.567233405562797*^9, 3.56723340878059*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", "2", "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     SubscriptBox["z", "1"]]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox["\[ImaginaryI]", "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}]}], 
     SubscriptBox["z", "3"]]}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{"a_", "/;", 
      RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"a", " ", 
       RowBox[{
        SubscriptBox["J", 
         RowBox[{"-", "a"}]], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "2"]}], "]"}]}], 
      RowBox[{
       SubscriptBox["z", "2"], " ", 
       SuperscriptBox["\[ImaginaryI]", "a"]}]]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.567233409068289*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{"a", " ", 
       SuperscriptBox["\[ImaginaryI]", "a"], " ", 
       RowBox[{
        SubscriptBox["J", "a"], "[", 
        RowBox[{"2", " ", "g", " ", 
         SubscriptBox["z", "3"]}], "]"}]}], 
      SubscriptBox["z", "3"]], "/.", 
     RowBox[{"a", "\[Rule]", "1"}]}], "/.", "subJBes"}], ",", 
   RowBox[{"{", 
    RowBox[{"g", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5672334428783293`*^9, 3.567233478254938*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", "g"}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "2"],
    SeriesData[$CellContext`g, 0, {}, 1, 2, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
    Complex[0, 1]}, 1, 2, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.567233444601287*^9, 3.567233478688921*^9}}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Other contributions to the Bethe equation\
\>", "Subsection",
 CellChangeTimes->{{3.567238018550961*^9, 3.567238030502146*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Im1", "/.", "subIm"}], "//", "FindHighTranCoef"}]], "Input",
 CellChangeTimes->{{3.567244455761746*^9, 3.567244480009921*^9}}],

Cell[BoxData[
 RowBox[{"720", " ", 
  SqrtBox["3"], " ", 
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], "5"]}]], "Output",
 CellChangeTimes->{{3.567244461140818*^9, 3.5672444802162457`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       OverscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], 
        StyleBox["^",
         FontColor->RGBColor[1, 0, 0]]], 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " "}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["\[ScriptCapitalT]",
        FontColor->RGBColor[1, 0, 0]], 
       StyleBox["^",
        FontColor->RGBColor[1, 0, 0]]], 
      RowBox[{"[", 
       RowBox[{"+", "2"}], "]"}]]], "/", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         OverscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], 
          StyleBox["^",
           FontColor->RGBColor[1, 0, 0]]], 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " "}], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["\[ScriptCapitalT]",
          FontColor->RGBColor[1, 0, 0]], 
         StyleBox["^",
          FontColor->RGBColor[1, 0, 0]]], 
        RowBox[{"[", 
         RowBox[{"+", "2"}], "]"}]]], ")"}], "asym"]}], "]"}], "==", 
  RowBox[{
   FractionBox["6", "7"], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", "g", " ", 
   RowBox[{"M", "[", "\[Delta]\[Rho]", "]"}]}]}]],
 CellChangeTimes->{{3.552205181043242*^9, 3.552205217682931*^9}, {
  3.5522052656677628`*^9, 3.5522053298096657`*^9}}]], "Subsubsection",
 CellChangeTimes->{{3.552205375767812*^9, 3.55220538309063*^9}, {
  3.552205844110306*^9, 3.5522058489016933`*^9}, {3.5537721902462673`*^9, 
  3.5537721973412437`*^9}, {3.5559587001737413`*^9, 3.555958700173893*^9}, {
  3.567238396106297*^9, 3.567238396106442*^9}, {3.567238463351358*^9, 
  3.567238464479309*^9}}],

Cell["We also got that ", "Text",
 CellChangeTimes->{{3.56724188800421*^9, 3.56724189050556*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"M", "[", "\[Delta]\[Rho]", "]"}], "==", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{"72", 
      SuperscriptBox["g", "2"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "1"], "[", 
        RowBox[{"2", " ", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
       RowBox[{"2", " ", "g", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", "[", " ", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           SubscriptBox["z", "1"]], ",", 
          SubscriptBox["z", "1"]}], "]"}], "+", 
        RowBox[{"D", "[", " ", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           SubscriptBox["z", "1"]], ",", 
          SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]]}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.567241892054063*^9, 3.567241896882895*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       OverscriptBox[
        StyleBox["\[ScriptCapitalT]",
         FontColor->RGBColor[1, 0, 0]], 
        StyleBox["^",
         FontColor->RGBColor[1, 0, 0]]], 
       RowBox[{"[", 
        RowBox[{"-", "2"}], "]"}]], " "}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["\[ScriptCapitalT]",
        FontColor->RGBColor[1, 0, 0]], 
       StyleBox["^",
        FontColor->RGBColor[1, 0, 0]]], 
      RowBox[{"[", 
       RowBox[{"+", "2"}], "]"}]]], "/", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         OverscriptBox[
          StyleBox["\[ScriptCapitalT]",
           FontColor->RGBColor[1, 0, 0]], 
          StyleBox["^",
           FontColor->RGBColor[1, 0, 0]]], 
         RowBox[{"[", 
          RowBox[{"-", "2"}], "]"}]], " "}], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["\[ScriptCapitalT]",
          FontColor->RGBColor[1, 0, 0]], 
         StyleBox["^",
          FontColor->RGBColor[1, 0, 0]]], 
        RowBox[{"[", 
         RowBox[{"+", "2"}], "]"}]]], ")"}], "asym"]}], "]"}], "\[Equal]", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["\[ScriptCapitalC]", 
         RowBox[{"[", 
          RowBox[{"-", "1"}], "]"}]], "-", 
        SuperscriptBox["\[ScriptCapitalC]", 
         RowBox[{"[", 
          RowBox[{"-", "3"}], "]"}]]}], ")"}], 
      OverscriptBox["*", "^"], "\[Delta]\[Rho]"}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["\[ScriptCapitalT]",
        FontColor->RGBColor[1, 0, 0]], 
       StyleBox["^",
        FontColor->RGBColor[1, 0, 0]]], 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]]], "-", 
    RowBox[{"c", ".", "c"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.553772235521481*^9, 3.553772279957571*^9}, {
  3.555957725230073*^9, 3.5559577256586657`*^9}, {3.5559577563570547`*^9, 
  3.555957797509214*^9}, {3.555957912392963*^9, 3.555957913472788*^9}, {
  3.5559586931980667`*^9, 3.555958697622611*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tmp1", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           FractionBox["1", 
            SuperscriptBox[
             OverscriptBox[
              StyleBox["\[ScriptCapitalT]",
               FontColor->RGBColor[1, 0, 0]], 
              StyleBox["^",
               FontColor->RGBColor[1, 0, 0]]], 
             RowBox[{"[", 
              RowBox[{"-", "2"}], "]"}]]], "/.", "largevolume"}], "/.", 
          "subJuk"}], "/.", 
         RowBox[{"u", "\[Rule]", 
          SubscriptBox["v", "0"]}]}], "/.", 
        RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//.", 
     "densqrt3"}], "//", "Simplify"}], "//", "Normal"}]}]], "Input",
 CellChangeTimes->{{3.555957822087865*^9, 3.555957895373212*^9}, {
   3.555957927415238*^9, 3.555957927886791*^9}, 3.555958543210257*^9, {
   3.555958590422051*^9, 3.555958591490354*^9}}],

Cell[BoxData[
 RowBox[{"1", "+", 
  RowBox[{
   FractionBox["3", "7"], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "2"]}], "+", 
  RowBox[{
   FractionBox["6", "343"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"445", " ", "\[ImaginaryI]"}], "+", 
     RowBox[{"114", " ", 
      SqrtBox["3"]}]}], ")"}], " ", 
   SuperscriptBox["g", "4"]}], "+", 
  FractionBox[
   RowBox[{"3", " ", 
    RowBox[{"(", 
     RowBox[{"381431", "-", 
      RowBox[{"60183", " ", "\[ImaginaryI]", " ", 
       SqrtBox["3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "6"]}], "16807"], "-", 
  FractionBox[
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "8"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "170315624"}], " ", "\[ImaginaryI]"}], "-", 
      RowBox[{"19976969", " ", 
       SqrtBox["3"]}], "+", 
      RowBox[{"403368", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "23"}], " ", "\[ImaginaryI]"}], "+", 
         RowBox[{"6", " ", 
          SqrtBox["3"]}]}], ")"}], " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "823543"]}]], "Output",\

 CellChangeTimes->{{3.555957831501809*^9, 3.555957851955184*^9}, {
   3.5559578894735403`*^9, 3.555957928320983*^9}, {3.55595853657743*^9, 
   3.5559585438380413`*^9}, 3.555958591720715*^9, 3.5603369583854933`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tmp2", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"2", "g"}], 
             RowBox[{"2", " ", "\[ImaginaryI]"}]]}], 
           RowBox[{"(", 
            RowBox[{
             FractionBox["1", 
              RowBox[{"u", "-", 
               FractionBox["\[ImaginaryI]", "2"], "-", 
               RowBox[{"2", " ", "g", " ", "z"}]}]], "-", 
             FractionBox["1", 
              RowBox[{"u", "-", 
               FractionBox[
                RowBox[{"3", "\[ImaginaryI]"}], "2"], "-", 
               RowBox[{"2", " ", "g", " ", "z"}]}]]}], ")"}], 
           FractionBox["\[Delta]\[Rho]", 
            SuperscriptBox["tran", "2"]]}], "/.", 
          RowBox[{"u", "\[Rule]", " ", 
           SubscriptBox["v", "0"]}]}], "/.", 
         RowBox[{"leadweak", "[", "2", "]"}]}], "/.", "subtotran"}], ",", 
       RowBox[{"{", 
        RowBox[{"tran", ",", "\[Infinity]", ",", "3"}], "}"}]}], "]"}], "//.",
      "sqrt3Simp"}], "//", "Simplify"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"%", "-", 
     RowBox[{"(", 
      RowBox[{"%", "/.", "mconj"}], ")"}]}], ",", "\[Delta]\[Rho]", ",", 
    "Simplify"}], "]"}], "/.", 
  RowBox[{"tran", "\[Rule]", "1"}]}]}], "Input",
 CellChangeTimes->{{3.553772283692554*^9, 3.553772357740438*^9}, {
  3.555957811245809*^9, 3.5559578184685926`*^9}, {3.555957930525948*^9, 
  3.555958046257812*^9}, {3.5559580779333477`*^9, 3.555958199072961*^9}, {
  3.555958414232709*^9, 3.555958518684589*^9}, {3.567238213541705*^9, 
  3.5672382462688017`*^9}, {3.567238305915949*^9, 3.567238387353368*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["6", "7"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", "10"], " ", 
         RowBox[{"M", "[", 
          SubscriptBox["\[Delta]\[Rho]", "lead"], "]"}]}], "+", 
        RowBox[{
         SuperscriptBox["g", "12"], " ", 
         RowBox[{"M", "[", 
          SubscriptBox["\[Delta]\[Rho]", "sublead"], "]"}]}], "+", 
        RowBox[{
         SuperscriptBox["g", "14"], " ", 
         RowBox[{"M", "[", 
          SubscriptBox["\[Delta]\[Rho]", "sublead2"], "]"}]}]}], ")"}]}], "/.",
      "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "//", 
   "FindHighTranCoef"}], "//", "Expand"}]], "Input",
 CellChangeTimes->{{3.567238432216757*^9, 3.567238508765923*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"6480", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "12"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{
   FractionBox["940896", "7"], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "14"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}]}]], "Output",
 CellChangeTimes->{{3.567238437654716*^9, 3.567238508993939*^9}}],

Cell["\<\
Now we will try to compute M[\[Delta]\[Rho]] explicitly\
\>", "Text",
 CellChangeTimes->{{3.56723860917668*^9, 3.567238616542069*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"M", "[", "\[Delta]\[Rho]", "]"}], "=", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "1"}], "1"], 
    RowBox[{
     FractionBox["dz", "\[Pi]"], 
     RowBox[{"\[Delta]\[Rho]", "[", "z", "]"}]}]}], "==", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "1"}], "1"], 
    RowBox[{
     FractionBox["dz", "\[Pi]"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["q", 
        RowBox[{"1", ",", "right"}], 
        RowBox[{"[", 
         RowBox[{"+", "o"}], "]"}]], "-", 
       SubsuperscriptBox["q", 
        RowBox[{"1", ",", "right"}], 
        RowBox[{"[", 
         RowBox[{"-", "o"}], "]"}]]}], ")"}]}]}], "\[Equal]", 
   RowBox[{
    RowBox[{"-", "3"}], 
    SuperscriptBox["g", "2"], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"n", "=", "1"}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      SubscriptBox["c", "n"], 
      RowBox[{
       SubsuperscriptBox["\[Integral]", 
        RowBox[{"-", "1"}], "1"], 
       RowBox[{
        FractionBox["dz", "\[Pi]"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            FractionBox["1", 
             SuperscriptBox[
              OverscriptBox[
               StyleBox["x",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], 
              RowBox[{
               RowBox[{"2", "n"}], "-", "1"}]]], ")"}], 
           RowBox[{"[", 
            RowBox[{"+", "o"}], "]"}]], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            FractionBox["1", 
             SuperscriptBox[
              OverscriptBox[
               StyleBox["x",
                FontColor->RGBColor[0, 0.6, 0]], 
               StyleBox["^",
                FontColor->RGBColor[0, 0.6, 0]]], 
              RowBox[{
               RowBox[{"2", "n"}], "-", "1"}]]], ")"}], 
           RowBox[{"[", 
            RowBox[{"-", "o"}], "]"}]]}], ")"}]}]}]}]}]}], "==", 
   RowBox[{
    RowBox[{"-", "3"}], 
    SuperscriptBox["g", "2"], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{"n", "=", "1"}], 
      RowBox[{"+", "\[Infinity]"}]], 
     RowBox[{
      SubscriptBox["c", "n"], 
      RowBox[{"\[ContourIntegral]", 
       RowBox[{
        FractionBox["dz", "\[Pi]"], 
        RowBox[{"(", 
         RowBox[{"(", 
          FractionBox["1", 
           SuperscriptBox[
            OverscriptBox[
             StyleBox["x",
              FontColor->RGBColor[0, 0.6, 0]], 
             StyleBox["^",
              FontColor->RGBColor[0, 0.6, 0]]], 
            RowBox[{
             RowBox[{"2", "n"}], "-", "1"}]]], ")"}], ")"}]}]}]}]}]}], "==", 
   RowBox[{
    RowBox[{"-", "3"}], 
    SuperscriptBox["g", "2"], 
    SubscriptBox["c", "1"], 
    RowBox[{"\[ContourIntegral]", 
     RowBox[{
      FractionBox["dz", "\[Pi]"], 
      RowBox[{"(", 
       FractionBox["1", 
        RowBox[{"2", " ", "z"}]], ")"}]}]}]}], "\[Equal]", 
   RowBox[{"3", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "2"], " ", 
    SubscriptBox["c", "1"]}], "==", 
   RowBox[{"TC", "[", 
    RowBox[{
     RowBox[{"72", 
      SuperscriptBox["g", "2"], 
      FractionBox[
       RowBox[{
        SubscriptBox["J", "1"], "[", 
        RowBox[{"2", " ", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
       RowBox[{"2", " ", "g", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["z", "1"], "-", 
          SubscriptBox["z", "2"]}], ")"}]}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", "[", " ", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           SubscriptBox["z", "1"]], ",", 
          SubscriptBox["z", "1"]}], "]"}], "+", 
        RowBox[{"D", "[", " ", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{
             SubscriptBox["J", "4"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "2"]}], "]"}], 
            RowBox[{
             SubscriptBox["J", "3"], "[", 
             RowBox[{"2", " ", "g", " ", 
              SubscriptBox["z", "1"]}], "]"}]}], 
           SubscriptBox["z", "1"]], ",", 
          SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox["\[ExponentialE]", 
        SubscriptBox["z", "1"]]}]], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["z", "2"], ",", "0", ",", 
       SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5672386214142313`*^9, 3.567238828845993*^9}, {
  3.5672389802843733`*^9, 3.567238983298985*^9}}],

Cell["\<\
This can be  checked at weak coupling\
\>", "Text",
 CellChangeTimes->{{3.5672388565575933`*^9, 3.567238862340747*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["6", "7"], " ", "\[ImaginaryI]", " ", 
    SqrtBox["3"], " ", "g", " ", 
    RowBox[{"Integrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"72", 
           SuperscriptBox["g", "2"], 
           FractionBox[
            RowBox[{
             SubscriptBox["J", "1"], "[", 
             RowBox[{"2", " ", "g", " ", 
              RowBox[{"(", 
               RowBox[{
                SubscriptBox["z", "1"], "-", 
                SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
            RowBox[{"2", " ", "g", " ", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["z", "1"], "-", 
               SubscriptBox["z", "2"]}], ")"}]}]], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"D", "[", " ", 
              RowBox[{
               FractionBox[
                RowBox[{
                 RowBox[{
                  SubscriptBox["J", "4"], "[", 
                  RowBox[{"2", " ", "g", " ", 
                   SubscriptBox["z", "2"]}], "]"}], 
                 RowBox[{
                  SubscriptBox["J", "3"], "[", 
                  RowBox[{"2", " ", "g", " ", 
                   SubscriptBox["z", "1"]}], "]"}]}], 
                SubscriptBox["z", "1"]], ",", 
               SubscriptBox["z", "1"]}], "]"}], "+", 
             RowBox[{"D", "[", " ", 
              RowBox[{
               FractionBox[
                RowBox[{
                 RowBox[{
                  SubscriptBox["J", "4"], "[", 
                  RowBox[{"2", " ", "g", " ", 
                   SubscriptBox["z", "2"]}], "]"}], 
                 RowBox[{
                  SubscriptBox["J", "3"], "[", 
                  RowBox[{"2", " ", "g", " ", 
                   SubscriptBox["z", "1"]}], "]"}]}], 
                SubscriptBox["z", "1"]], ",", 
               SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], "/.", "subJBes"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{"g", ",", "0", ",", "14"}], "}"}]}], "]"}], "//", 
       "Normal"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}], "/.", 
   RowBox[{
    SubsuperscriptBox["z", "1", "a_"], "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{"a", "+", "1"}], "]"}], 
     SubscriptBox["\[Zeta]", 
      RowBox[{"a", "+", "1"}]]}]}]}], "//", "Expand"}]], "Input",
 CellChangeTimes->{{3.567238872565682*^9, 3.567238919929715*^9}, {
  3.5672389513733873`*^9, 3.5672389548378763`*^9}, {3.567238985891921*^9, 
  3.567239017849475*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"6480", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "12"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{
   FractionBox["940896", "7"], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "14"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}]}]], "Output",
 CellChangeTimes->{{3.567238874928653*^9, 3.5672389215236673`*^9}, 
   3.567238956167568*^9, {3.567238987225059*^9, 3.5672389921682463`*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"6480", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "12"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{
   FractionBox["940896", "7"], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "14"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["q", 
   RowBox[{"1", ",", "right"}]], "==", 
  RowBox[{
   SubsuperscriptBox["\[Sum]", 
    RowBox[{"n", "=", "1"}], 
    RowBox[{"+", "\[Infinity]"}]], 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"-", "3"}], 
      SuperscriptBox["g", "2"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], 
      RowBox[{
       RowBox[{"2", "n"}], "-", "1"}]]], 
    RowBox[{"TC", "[", 
     RowBox[{
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "n"}], "-", "1"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"], 
       FractionBox[
        RowBox[{
         SubscriptBox["J", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]], "[", 
         RowBox[{"2", "g", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["z", "1"], "-", 
            SubscriptBox["z", "2"]}], ")"}]}], "]"}], 
        RowBox[{"2", "g", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["z", "1"], "-", 
           SubscriptBox["z", "2"]}], ")"}]}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "1"]}], "]"}], "+", 
         RowBox[{"D", "[", " ", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{
              SubscriptBox["J", "4"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "2"]}], "]"}], 
             RowBox[{
              SubscriptBox["J", "3"], "[", 
              RowBox[{"2", " ", "g", " ", 
               SubscriptBox["z", "1"]}], "]"}]}], 
            SubscriptBox["z", "1"]], ",", 
           SubscriptBox["z", "2"]}], "]"}]}], ")"}]}], ",", 
      FractionBox["1", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox["\[ExponentialE]", 
         SubscriptBox["z", "1"]]}]], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "1"], ",", "0", ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["z", "2"], ",", "0", ",", 
        SubscriptBox["z", "1"]}], "}"}]}], "]"}]}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"contTh2", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"#", "-", 
            RowBox[{"(", 
             RowBox[{"#", "/.", "mconj"}], ")"}]}], ")"}], "&"}], "[", 
         RowBox[{"tmp1", " ", "tmp2"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"g", ",", "0", ",", "14"}], "}"}]}], "]"}], "//", "Normal"}],
      "//", "Expand"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", 
      RowBox[{
       SubscriptBox["\[Beta]", "_"], "[", "_", "]"}], ",", 
      SqrtBox["3"]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.555958571301457*^9, 3.555958647874802*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["3", "7"], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   RowBox[{
    SubscriptBox["\[Beta]", "lead"], "[", "0", "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["243", "343"]}], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "lead"], "[", "0", "]"}]}], "+", 
     RowBox[{
      FractionBox["3", "7"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead"], "[", "0", "]"}]}], "+", 
     RowBox[{
      FractionBox["3", "28"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead"], "[", "1", "]"}]}]}], ")"}]}], "+", 
  
  RowBox[{
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"6435", " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "lead"], "[", "0", "]"}]}], "16807"]}], "-", 
     
     RowBox[{
      FractionBox["243", "343"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead"], "[", "0", "]"}]}], "-", 
     RowBox[{
      FractionBox["621", "686"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead"], "[", "1", "]"}]}], "+", 
     RowBox[{
      FractionBox["3", "7"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead2"], "[", "0", "]"}]}], "+", 
     RowBox[{
      FractionBox["3", "28"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead2"], "[", "1", "]"}]}], "+", 
     RowBox[{
      FractionBox["3", "56"], " ", "\[ImaginaryI]", " ", 
      SqrtBox["3"], " ", 
      RowBox[{
       SubscriptBox["\[Beta]", "sublead2"], "[", "2", "]"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.555958649406034*^9, 3.56033697859156*^9}]
}, Open  ]],

Cell["Now we computing", "Text",
 CellChangeTimes->{{3.567238599345056*^9, 3.567238601614698*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     FractionBox["3", "7"], " ", "\[ImaginaryI]", " ", 
     SqrtBox["3"], " ", 
     SuperscriptBox["g", "10"], " ", 
     RowBox[{
      SubscriptBox["\[Beta]", "lead"], "[", "0", "]"}]}], "+", 
    RowBox[{
     SuperscriptBox["g", "12"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["243", "343"]}], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "lead"], "[", "0", "]"}]}], "+", 
       RowBox[{
        FractionBox["3", "7"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead"], "[", "0", "]"}]}], "+", 
       RowBox[{
        FractionBox["3", "28"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead"], "[", "1", "]"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     SuperscriptBox["g", "14"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"6435", " ", "\[ImaginaryI]", " ", 
          SqrtBox["3"], " ", 
          RowBox[{
           SubscriptBox["\[Beta]", "lead"], "[", "0", "]"}]}], "16807"]}], 
       "-", 
       RowBox[{
        FractionBox["243", "343"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead"], "[", "0", "]"}]}], "-", 
       RowBox[{
        FractionBox["621", "686"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead"], "[", "1", "]"}]}], "+", 
       RowBox[{
        FractionBox["3", "7"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead2"], "[", "0", "]"}]}], "+", 
       RowBox[{
        FractionBox["3", "28"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead2"], "[", "1", "]"}]}], "+", 
       RowBox[{
        FractionBox["3", "56"], " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        RowBox[{
         SubscriptBox["\[Beta]", "sublead2"], "[", "2", "]"}]}]}], ")"}]}]}], 
   "/.", "solut\[Rho]sexplicit"}], "//", "FindHighTranCoef"}]], "Input",
 CellChangeTimes->{{3.567238084557666*^9, 3.567238097987265*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"216", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"6480", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "12"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{
   FractionBox["940896", "7"], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "14"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}]}]], "Output",
 CellChangeTimes->{{3.567238091114805*^9, 3.567238098315181*^9}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   RowBox[{
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       SubscriptBox[
        StyleBox["\[GothicCapitalT]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], 
        RowBox[{"2", ",", "1"}]], ")"}], "+"], 
     SuperscriptBox[
      RowBox[{"(", 
       SubscriptBox[
        StyleBox["\[GothicCapitalT]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], 
        RowBox[{"2", ",", "1"}]], ")"}], "-"]], "/", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         SubscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "1"}]], ")"}], "+"], 
       SuperscriptBox[
        RowBox[{"(", 
         SubscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "1"}]], ")"}], "-"]], ")"}], "asym"]}], "]"}], 
  "=", 
  RowBox[{"contTu\[Rho]2", "-", "contTuWh", "+", "contTuW2"}]}]],
 CellChangeTimes->{{3.552205181043242*^9, 3.552205217682931*^9}, {
  3.5522052656677628`*^9, 3.5522053298096657`*^9}}]], "Subsubsection",
 CellChangeTimes->{{3.552205375767812*^9, 3.55220538309063*^9}, {
  3.552205844110306*^9, 3.5522058489016933`*^9}, {3.5522059052663593`*^9, 
  3.552205905266498*^9}, {3.552206287185974*^9, 3.55220628860605*^9}, {
  3.5522193723097267`*^9, 3.552219374913515*^9}, {3.553772570067823*^9, 
  3.553772574118737*^9}, {3.5538298935727654`*^9, 3.553829958269196*^9}, {
  3.5538301754719467`*^9, 3.5538301850217543`*^9}, {3.555961294348217*^9, 
  3.555961294348373*^9}, {3.5559617608051662`*^9, 3.555961765997615*^9}, {
  3.555962334604189*^9, 3.5559623346043377`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   RowBox[{
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       SubscriptBox[
        StyleBox["\[GothicCapitalT]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], 
        RowBox[{"2", ",", "1"}]], ")"}], "+"], 
     SuperscriptBox[
      RowBox[{"(", 
       SubscriptBox[
        StyleBox["\[GothicCapitalT]",
         FontColor->RGBColor[0.54, 0.35, 0.15]], 
        RowBox[{"2", ",", "1"}]], ")"}], "-"]], "/", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         SubscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "1"}]], ")"}], "+"], 
       SuperscriptBox[
        RowBox[{"(", 
         SubscriptBox[
          StyleBox["\[GothicCapitalT]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "1"}]], ")"}], "-"]], ")"}], "asym"]}], "]"}], "==", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ScriptCapitalC]", 
           RowBox[{"[", 
            RowBox[{"+", "2"}], "]"}]], "-", 
          SuperscriptBox["\[ScriptCapitalC]", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], ")"}], 
        OverscriptBox["*", "^"], 
        SubscriptBox["\[Delta]\[Rho]", "2"]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[ScriptCapitalC]", 
           RowBox[{"[", 
            RowBox[{"+", "2"}], "]"}]], "-", 
          SuperscriptBox["\[ScriptCapitalC]", 
           RowBox[{"[", 
            RowBox[{"-", "2"}], "]"}]]}], ")"}], "*", 
        OverscriptBox["W", "^"]}], "+", 
       SubscriptBox["W", "2"]}], 
      SubscriptBox[
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox[
          StyleBox["\[GothicQ]",
           FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
          RowBox[{"[", "2", "]"}]], "+", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[GothicQ]",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           StyleBox["_",
            FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
          RowBox[{"[", 
           RowBox[{"-", "2"}], "]"}]]}], ")"}], "asym"]], ")"}], "+"], "-", 
   "cc"}]}]], "Input",
 CellChangeTimes->{{3.5522064398846617`*^9, 3.5522064467358313`*^9}, {
  3.553772623304038*^9, 3.553772657698886*^9}, {3.553774456703649*^9, 
  3.553774461436727*^9}, {3.5559587815305*^9, 3.5559587874539623`*^9}, {
  3.555959389479803*^9, 3.555959389751789*^9}}],

Cell["Computing denominator.", "Text",
 CellChangeTimes->{{3.5559594003821707`*^9, 3.555959404533608*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tmp1", "=", 
  RowBox[{"tmpden", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                FractionBox["1", 
                 RowBox[{
                  SubsuperscriptBox[
                   StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "2", 
                   RowBox[{"[", "3", "]"}]], "+", 
                  SubsuperscriptBox[
                   OverscriptBox[
                    StyleBox["\[GothicQ]",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], 
                    StyleBox["_",
                    FontColor->RGBColor[0.54, 0.35, 0.15]]], "2", 
                   RowBox[{"[", 
                    RowBox[{"-", "1"}], "]"}]]}]], "/.", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"q$u", "\[Rule]", "q$"}], ",", 
                  RowBox[{"q$ub", "\[Rule]", "q$b"}]}], "}"}]}], "/.", 
               "largevolume"}], "/.", 
              RowBox[{
               RowBox[{"x", "[", "a_", "]"}], "\[RuleDelayed]", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"a", ">", "0"}], ",", 
                 RowBox[{"x$h", "[", "a", "]"}], ",", 
                 FractionBox["1", 
                  RowBox[{"x$h", "[", "a", "]"}]]}], "]"}]}]}], "/.", 
             "subJuk"}], "/.", 
            RowBox[{"u", "\[Rule]", 
             SubscriptBox["v", "0"]}]}], "/.", 
           RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", 
        "Simplify"}], ")"}], "//.", "densqrt3"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", 
       SqrtBox["3"]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5559587428038893`*^9, 3.5559589394420424`*^9}, 
   3.555959218881966*^9, {3.5559596406194773`*^9, 3.555959651218796*^9}, {
   3.55596138138239*^9, 3.5559613828465767`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["3", "4"], "-", 
  FractionBox["\[ImaginaryI]", 
   RowBox[{"4", " ", 
    SqrtBox["3"]}]], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["9", "28"], "-", 
     FractionBox[
      RowBox[{"11", " ", "\[ImaginaryI]"}], 
      RowBox[{"28", " ", 
       SqrtBox["3"]}]]}], ")"}], " ", 
   SuperscriptBox["g", "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1737", "1372"]}], "+", 
     FractionBox[
      RowBox[{"3215", " ", "\[ImaginaryI]"}], 
      RowBox[{"1372", " ", 
       SqrtBox["3"]}]]}], ")"}], " ", 
   SuperscriptBox["g", "4"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["537363", "67228"], "+", 
     RowBox[{
      SqrtBox["3"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"878173", " ", "\[ImaginaryI]"}], "201684"]}], "-", 
        RowBox[{"4", " ", "\[ImaginaryI]", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1277356307", "2470629"], "+", 
     RowBox[{
      SqrtBox["3"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"833279974", " ", "\[ImaginaryI]"}], "2470629"]}], "-", 
        FractionBox[
         RowBox[{"1098", " ", "\[ImaginaryI]", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], "49"]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{"3474", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "49"]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.5559592806394587`*^9, 3.5559593915679626`*^9, 3.5559606924073677`*^9, {
   3.5559613780244293`*^9, 3.55596138345255*^9}, 3.560337028682087*^9, 
   3.5603372055505037`*^9}]
}, Open  ]],

Cell["Now computing numerator", "Text",
 CellChangeTimes->{{3.5522065399523487`*^9, 3.552206543247525*^9}}],

Cell[TextData[{
 "Contribution from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Delta]\[Rho]", "2"], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.555961322412122*^9, 3.555961333363645*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{"2", "g"}], 
            RowBox[{"2", " ", "\[ImaginaryI]"}]]}], 
          RowBox[{"(", 
           RowBox[{
            FractionBox["1", 
             RowBox[{"u", "+", 
              FractionBox[
               RowBox[{"3", "\[ImaginaryI]"}], "2"], "-", 
              RowBox[{"2", "g", " ", "w"}]}]], "-", 
            FractionBox["1", 
             RowBox[{"u", "-", 
              FractionBox["\[ImaginaryI]", "2"], "-", 
              RowBox[{"2", "g", " ", "w"}]}]]}], ")"}]}], "/.", 
         RowBox[{"u", "\[Rule]", 
          SubscriptBox["v", "0"]}]}], "/.", 
        RowBox[{"leadweak", "[", "7", "]"}]}], "/.", "subtotran"}], ",", 
      RowBox[{"{", 
       RowBox[{"tran", ",", "\[Infinity]", ",", "3"}], "}"}]}], "]"}], "/.", 
    "densqrt3"}], "//", "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"(", 
  RowBox[{"%", "//", "Normal"}], ")"}]}], "Input",
 CellChangeTimes->{{3.5672421554300756`*^9, 3.567242282524158*^9}, {
  3.567242377642585*^9, 3.567242439887341*^9}, {3.567242505676543*^9, 
  3.567242520604495*^9}, {3.567242629735271*^9, 3.56724262990385*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"3", " ", 
    RowBox[{"(", 
     RowBox[{"5", "-", 
      RowBox[{"\[ImaginaryI]", " ", 
       SqrtBox["3"]}]}], ")"}], " ", "g"}], 
   RowBox[{"7", " ", "tran"}]], "+", 
  FractionBox[
   RowBox[{"12", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"9", " ", "\[ImaginaryI]"}], "+", 
      RowBox[{"13", " ", 
       SqrtBox["3"]}]}], ")"}], " ", 
    SuperscriptBox["g", "2"], " ", "w"}], 
   RowBox[{"49", " ", 
    SuperscriptBox["tran", "2"]}]], "+", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"182", " ", "\[ImaginaryI]"}], "-", 
      RowBox[{"42", " ", 
       SqrtBox["3"]}], "+", 
      RowBox[{"9", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "\[ImaginaryI]"}], "+", 
         RowBox[{"37", " ", 
          SqrtBox["3"]}]}], ")"}], " ", 
       SuperscriptBox["w", "2"]}]}], ")"}]}], 
   RowBox[{"343", " ", 
    SuperscriptBox["tran", "3"]}]]}]], "Output",
 CellChangeTimes->{3.567242440585795*^9, 3.567242506926524*^9, 
  3.567242630284894*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     SqrtBox["3"], " ", 
     SuperscriptBox["g", "8"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}]}]}], ")"}]}], "+", 
    
    RowBox[{
     SqrtBox["3"], " ", 
     SuperscriptBox["g", "10"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["17", "14"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}]}], "+", 
       RowBox[{
        FractionBox["335", "392"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead"], "[", "0", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead"], "[", "1", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "16"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead"], "[", "2", "]"}]}]}], ")"}]}], 
    "+", 
    RowBox[{
     SqrtBox["3"], " ", 
     SuperscriptBox["g", "12"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["4233", "686"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}]}], "-", 
       FractionBox[
        RowBox[{"72789", " ", "\[ImaginaryI]", " ", 
         RowBox[{
          SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}]}], "9604"], "+", 
       RowBox[{
        FractionBox["17", "14"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead"], "[", "0", "]"}]}], "+", 
       RowBox[{
        FractionBox["335", "392"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead"], "[", "1", "]"}]}], "+", 
       RowBox[{
        FractionBox["443", "784"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead"], "[", "2", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead2"], "[", "0", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead2"], "[", "1", "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "16"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead2"], "[", "2", "]"}]}], "-", 
       RowBox[{
        FractionBox["5", "128"], " ", "\[ImaginaryI]", " ", 
        RowBox[{
         SubscriptBox["\[Alpha]", "sublead2"], "[", "3", "]"}]}]}], ")"}]}]}],
    "/.", "solut\[Rho]sexplicit"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp3", "=", "%"}], ";"}]}], "Input",
 CellChangeTimes->{{3.567242710361182*^9, 3.5672427222285147`*^9}, {
  3.567244077936915*^9, 3.567244090409759*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["g", "8"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox["3", "4"]}], "-", 
           FractionBox["\[ImaginaryI]", 
            RowBox[{"4", " ", 
             SqrtBox["3"]}]]}], ")"}], " ", 
         SubsuperscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "v0", ",", "lead"}], "-"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           FractionBox["3", "4"], "-", 
           FractionBox["\[ImaginaryI]", 
            RowBox[{"4", " ", 
             SqrtBox["3"]}]]}], ")"}], " ", 
         SubsuperscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "v0", ",", "lead"}], "+"]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", "10"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox["9", "28"]}], "-", 
           FractionBox[
            RowBox[{"11", " ", "\[ImaginaryI]"}], 
            RowBox[{"28", " ", 
             SqrtBox["3"]}]]}], ")"}], " ", 
         SubsuperscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "v0", ",", "lead"}], "-"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           FractionBox["9", "28"], "-", 
           FractionBox[
            RowBox[{"11", " ", "\[ImaginaryI]"}], 
            RowBox[{"28", " ", 
             SqrtBox["3"]}]]}], ")"}], " ", 
         SubsuperscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "v0", ",", "lead"}], "+"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox["3", "4"]}], "-", 
           FractionBox["\[ImaginaryI]", 
            RowBox[{"4", " ", 
             SqrtBox["3"]}]]}], ")"}], " ", 
         SubsuperscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "v0", ",", "sublead"}], "-"]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           FractionBox["3", "4"], "-", 
           FractionBox["\[ImaginaryI]", 
            RowBox[{"4", " ", 
             SqrtBox["3"]}]]}], ")"}], " ", 
         SubsuperscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          RowBox[{"2", ",", "v0", ",", "sublead"}], "+"]}]}], ")"}]}]}], "//",
     "CCompute"}], "//", "fundeperiod"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp2", "=", "%"}], ";"}]}], "Input",
 CellChangeTimes->{{3.567242791660816*^9, 3.567242793477746*^9}, {
  3.567242928636045*^9, 3.56724296321815*^9}, {3.5672429976253977`*^9, 
  3.5672430134344893`*^9}, {3.5672434481801767`*^9, 3.567243454308505*^9}, {
  3.567244020515812*^9, 3.567244022826894*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Coefficient", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "tmp"}], "+", "tmp2", "+", "tmp3"}], ",", "g", ",", "8"}],
      "]"}], 
    SuperscriptBox["g", "8"]}], "//", "FindHighTranCoef"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", "/.", "sub\[Eta]0\[Eta]"}], "//", "FuntoMHZbasis"}], "//", 
   "Expand"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"%", "-", 
      RowBox[{"(", 
       RowBox[{"%", "/.", "mconj"}], ")"}]}], "/.", "sub\[Eta]\[Eta]0"}], "/.",
     "RidOfBar"}], "//", "Simplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "//", "fundeperiod"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "//.", "subZetaReduce"}], "//", "Simplify"}]}], "Input",
 CellChangeTimes->{{3.567244040799018*^9, 3.567244051334773*^9}, {
  3.567244102001575*^9, 3.567244203716132*^9}, {3.567244256605493*^9, 
  3.567244305940373*^9}, {3.5672443459620123`*^9, 3.567244361669742*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["216", "7"], " ", 
  SuperscriptBox["g", "8"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "20"}], " ", "\[ImaginaryI]", " ", 
     SqrtBox["3"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}], "+", 
    RowBox[{"108", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "3"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
    RowBox[{"18", " ", "\[ImaginaryI]", " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], 
      RowBox[{"2", ",", "3"}], "+"]}], "-", 
    RowBox[{"18", " ", "\[ImaginaryI]", " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], 
      RowBox[{"3", ",", "2"}], "+"]}], "+", 
    RowBox[{"9", " ", "\[ImaginaryI]", " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "+", 
    RowBox[{
     SqrtBox["3"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "-", 
    RowBox[{"9", " ", "\[ImaginaryI]", " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "2", "+"], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "-", 
    RowBox[{
     SqrtBox["3"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "2", "+"], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.5672440516069813`*^9, {3.567244102932374*^9, 3.5672442043966227`*^9}, {
   3.567244258966702*^9, 3.567244306392393*^9}, {3.567244357856121*^9, 
   3.5672443620558233`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FindHighTranCoef"}]], "Input",
 CellChangeTimes->{{3.567242806410921*^9, 3.567242813985552*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"24", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
  RowBox[{
   FractionBox["1152", "5"], " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
  RowBox[{"16", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
  RowBox[{
   FractionBox["1632", "5"], " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
  RowBox[{"2", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
  RowBox[{"96", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
  RowBox[{"8", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "6"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
  RowBox[{"12", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "-", 
  RowBox[{"144", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "-", 
  RowBox[{"2", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
  RowBox[{"60", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "-", 
  RowBox[{"6", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "5"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
  RowBox[{"72", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"]}], "-", 
  RowBox[{"48", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"]}], "+", 
  RowBox[{"6", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"]}], "+", 
  RowBox[{"48", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"]}], "-", 
  RowBox[{"8", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"]}], "-", 
  RowBox[{"32", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{
   FractionBox["3264", "5"], " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"8", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"384", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"48", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "5"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"12", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"144", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"6", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"180", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"30", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"96", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"24", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "+", 
  RowBox[{"48", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"24", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "1", "-"]}], "-", 
  RowBox[{"16", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1632", "5"], " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{"12", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"576", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{"120", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "4"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"6", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{"180", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"60", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"48", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{"36", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "-", 
  RowBox[{"24", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "2"]}], "+", 
  RowBox[{"8", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"384", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
  RowBox[{"160", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"2", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
  RowBox[{"60", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"60", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
  RowBox[{"24", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "-", 
  RowBox[{"8", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "6", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "3"]}], "+", 
  RowBox[{"2", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "-", 
  RowBox[{"96", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
  RowBox[{"120", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]0",
       FontWeight->Bold,
       FontColor->RGBColor[0, 0.5, 0]], "1", "+"], ")"}], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "-", 
  RowBox[{"30", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
  RowBox[{"6", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "4"]}], "+", 
  RowBox[{"48", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "1", "+"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "5"]}], "-", 
  RowBox[{"6", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], "5"]}], "+", 
  RowBox[{"8", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]]], "1", "-"], ")"}], 
    "6"]}]}]], "Output",
 CellChangeTimes->{3.567242849754457*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "fundeperiod"}]], "Input",
 CellChangeTimes->{{3.567242885741467*^9, 3.567242888130309*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
     RowBox[{"2", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "8"}], " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "6", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
     RowBox[{"6", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}], "-", 
     RowBox[{"6", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "5", "-"]}], "+", 
     RowBox[{"8", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "6", "-"]}]}], ")"}]}]}]], "Output",\

 CellChangeTimes->{3.567242891902821*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"49", "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"]}], "-", 
         RowBox[{"36", " ", 
          SuperscriptBox["u", "4"]}], "+", 
         RowBox[{"27", " ", 
          SuperscriptBox["u", "6"]}]}], ")"}], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["W",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         StyleBox["^",
          FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"]}], 
      RowBox[{"27", " ", 
       SqrtBox["3"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "+"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", "3", "]"}]]}]]}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "33271"}], " ", 
           SqrtBox["3"]}], "-", 
          RowBox[{"841008", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "-", 
          RowBox[{"1159749", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "+", 
          RowBox[{"2382939", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "+", 
          RowBox[{"4990491", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "+", 
          RowBox[{"2654775", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "+", 
          RowBox[{"330966", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "-", 
          RowBox[{"24057", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}]}], ")"}], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"45927", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "2"]}], ")"}]}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"49", "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"]}], "-", 
         RowBox[{"36", " ", 
          SuperscriptBox["u", "4"]}], "+", 
         RowBox[{"27", " ", 
          SuperscriptBox["u", "6"]}]}], ")"}], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["W",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         StyleBox["^",
          FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead"]}], 
      RowBox[{"27", " ", 
       SqrtBox["3"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "+"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", "3", "]"}]]}]]}], "]"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["g", "7"], 
      RowBox[{"M", "[", 
       RowBox[{
        SuperscriptBox["w", "2"], 
        SubscriptBox["\[Delta]\[Rho]2", "lead"]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", "9"], 
      RowBox[{"M", "[", 
       RowBox[{
        SuperscriptBox["w", "2"], 
        SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "]"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", "11"], 
      RowBox[{"M", "[", 
       RowBox[{
        SuperscriptBox["w", "2"], 
        SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}], "]"}]}]}], "/.", 
    "sub\[Rho]Ansatz"}], "/.", "solut\[Rho]sexplicit"}], "//", 
  "FindHighTranCoef"}]], "Input",
 CellChangeTimes->{{3.567242526703179*^9, 3.567242615860628*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "21"}], " ", 
   SuperscriptBox["g", "7"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  RowBox[{"630", " ", 
   SuperscriptBox["g", "9"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"13068", " ", 
   SuperscriptBox["g", "11"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}]}]], "Output",
 CellChangeTimes->{{3.567242535760419*^9, 3.567242620732204*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tmpn", "=", 
   RowBox[{
    RowBox[{"Collect", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Series", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"-", 
                    FractionBox[
                    RowBox[{"2", "w", " ", "g"}], 
                    RowBox[{"2", " ", "\[ImaginaryI]"}]]}], 
                    RowBox[{"(", 
                    RowBox[{
                    FractionBox["1", 
                    RowBox[{"u", "+", 
                    FractionBox[
                    RowBox[{"3", "\[ImaginaryI]"}], "2"], "-", 
                    RowBox[{"2", "g", " ", "w"}]}]], "-", 
                    FractionBox["1", 
                    RowBox[{"u", "-", 
                    FractionBox["\[ImaginaryI]", "2"], "-", 
                    RowBox[{"2", "g", " ", "w"}]}]]}], ")"}], 
                    SuperscriptBox["g", "7"], 
                    RowBox[{"(", 
                    RowBox[{
                    SubscriptBox["\[Delta]\[Rho]2", "lead"], "+", 
                    RowBox[{
                    SuperscriptBox["g", "2"], 
                    SubscriptBox["\[Delta]\[Rho]2", "sublead"]}], "+", 
                    RowBox[{
                    SuperscriptBox["g", "4"], 
                    SubscriptBox["\[Delta]\[Rho]2", "sublead2"]}], "+", 
                    RowBox[{
                    SuperscriptBox["g", "6"], "\[Delta]\[Rho]2sb"}]}], 
                    ")"}]}], "/.", 
                   RowBox[{"u", "\[Rule]", " ", 
                    SubscriptBox["v", "0"]}]}], "/.", 
                  RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
                 RowBox[{"{", 
                  RowBox[{"g", ",", "0", ",", "14"}], "}"}]}], "]"}], "//.", 
               "densqrt3"}], "//", "Normal"}], "//", "Expand"}], ")"}], "/.", 
           
           RowBox[{
            SuperscriptBox["w", 
             RowBox[{"a_", "/;", 
              RowBox[{"EvenQ", "[", "a", "]"}]}]], "\[Rule]", "0"}]}], "/.", 
          RowBox[{
           RowBox[{
            SuperscriptBox["w", "a_."], 
            SubscriptBox["\[Delta]\[Rho]2", "b_"]}], "\[RuleDelayed]", 
           RowBox[{"M", "[", 
            RowBox[{
             SuperscriptBox["w", 
              RowBox[{"a", "-", "1"}]], 
             SubscriptBox["\[Delta]\[Rho]2", "b"]}], "]"}]}]}], "/.", 
         "sub\[Rho]Ansatz"}], "/.", 
        RowBox[{"(", 
         RowBox[{"Rule", "@@@", 
          RowBox[{"Cases", "[", 
           RowBox[{
            RowBox[{"solut\[Rho]s", "/.", 
             RowBox[{"Rule", "\[Rule]", "List"}]}], ",", 
            RowBox[{"List", "[", 
             RowBox[{"a_", ",", "0"}], "]"}]}], "]"}]}], ")"}]}], "//", 
       "Expand"}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", 
        RowBox[{
         SubscriptBox["\[Beta]", "_"], "[", "_", "]"}], ",", 
        RowBox[{
         SubscriptBox["\[Alpha]", "_"], "[", "_", "]"}], ",", 
        SqrtBox["3"]}], "}"}], ",", "Simplify"}], "]"}], "/.", 
    RowBox[{
     RowBox[{"w", " ", "\[Delta]\[Rho]2sb"}], "->", "\[Delta]\[Rho]2sb"}]}]}],
   ";"}]], "Input",
 CellChangeTimes->{{3.555959440246872*^9, 3.555959597085898*^9}, {
   3.555959656956286*^9, 3.555959679234445*^9}, {3.5559597284806137`*^9, 
   3.555959762943166*^9}, {3.555959817172202*^9, 3.55595983699745*^9}, 
   3.555960150276371*^9, {3.555960259949939*^9, 3.555960400582122*^9}, {
   3.555960436221867*^9, 3.55596044881701*^9}, {3.555960483882102*^9, 
   3.555960528188892*^9}, 3.555960560102167*^9, {3.555960614052176*^9, 
   3.555960621470096*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{"tmp1", " ", "tmpn"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "14"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"contTu\[Rho]2", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"%", "-", 
        RowBox[{"(", 
         RowBox[{"%", "/.", "mconj"}], ")"}]}], ")"}], "/.", 
      RowBox[{"\[Delta]\[Rho]2sb", "->", "\[Delta]\[Rho]2sb3"}]}], "//", 
     "Expand"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", 
      SqrtBox["3"], ",", 
      RowBox[{
       SubscriptBox["\[Beta]", "_"], "[", "_", "]"}], ",", 
      RowBox[{
       SubscriptBox["\[Alpha]", "_"], "[", "_", "]"}]}], "}"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.555960687874434*^9, 3.5559607913417263`*^9}, {
  3.5559610752133904`*^9, 3.555961078124599*^9}, {3.55596125376224*^9, 
  3.555961263643443*^9}, {3.560337186748975*^9, 3.560337190471943*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["17", "14"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}]}], "+", 
     RowBox[{
      FractionBox["335", "392"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "0", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "1", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "16"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "2", "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["4233", "686"]}], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}]}], "-", 
     FractionBox[
      RowBox[{"72789", " ", "\[ImaginaryI]", " ", 
       RowBox[{
        SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}]}], "9604"], "+", 
     RowBox[{
      FractionBox["17", "14"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "0", "]"}]}], "+", 
     RowBox[{
      FractionBox["335", "392"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "1", "]"}]}], "+", 
     RowBox[{
      FractionBox["443", "784"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "2", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "0", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "1", "]"}]}], "-", 
     RowBox[{
      FractionBox["1", "16"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "2", "]"}]}], "-", 
     RowBox[{
      FractionBox["5", "128"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "3", "]"}]}]}], ")"}]}], 
  "+", 
  RowBox[{
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "3"]}], " ", "\[ImaginaryI]", " ", 
      RowBox[{"\[DoubleStruckCapitalC]", "[", 
       RowBox[{"-", 
        RowBox[{
         FractionBox["1", 
          RowBox[{"2187", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             SuperscriptBox[
              StyleBox["Q0",
               FontColor->RGBColor[{
                  Rational[1, 2], 
                  Rational[1, 2], 0}]], "-"], ")"}], "4"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             SuperscriptBox[
              StyleBox["Q0",
               FontColor->RGBColor[{
                  Rational[1, 2], 
                  Rational[1, 2], 0}]], "+"], ")"}], "4"]}]], 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"9", " ", 
             SuperscriptBox[
              StyleBox["Q0",
               FontColor->RGBColor[{
                  Rational[1, 2], 
                  Rational[1, 2], 0}]], "-"], " ", 
             SuperscriptBox[
              StyleBox["Q0",
               FontColor->RGBColor[{
                  Rational[1, 2], 
                  Rational[1, 2], 0}]], "+"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                RowBox[{"(", 
                 RowBox[{"5", "-", 
                  RowBox[{"144", " ", 
                   SuperscriptBox["u", "2"]}], "-", 
                  RowBox[{"351", " ", 
                   SuperscriptBox["u", "4"]}], "+", 
                  RowBox[{"1404", " ", 
                   SuperscriptBox["u", "6"]}], "+", 
                  RowBox[{"729", " ", 
                   SuperscriptBox["u", "8"]}]}], ")"}], " ", 
                SubscriptBox[
                 StyleBox["W",
                  FontColor->RGBColor[0.54, 0.35, 0.15]], "sublead"]}], "+", 
               RowBox[{"9", " ", 
                SuperscriptBox[
                 StyleBox["Q0",
                  FontColor->RGBColor[{
                    Rational[1, 2], 
                    Rational[1, 2], 0}]], "-"], " ", 
                SuperscriptBox[
                 StyleBox["Q0",
                  FontColor->RGBColor[{
                    Rational[1, 2], 
                    Rational[1, 2], 0}]], "+"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"4", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", "1"}], "+", 
                    RowBox[{"3", " ", 
                    SuperscriptBox["u", "2"]}], "+", 
                    RowBox[{"27", " ", 
                    SuperscriptBox["u", "4"]}]}], ")"}], " ", 
                   SubscriptBox[
                    StyleBox["W",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "sublead2"]}], 
                  "+", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"1", "+", 
                    RowBox[{"6", " ", 
                    SuperscriptBox["u", "2"]}], "+", 
                    RowBox[{"18", " ", 
                    SuperscriptBox["u", "4"]}], "+", 
                    RowBox[{"27", " ", 
                    SuperscriptBox["u", "6"]}]}], ")"}], " ", 
                   SubscriptBox[
                    StyleBox["W",
                    FontColor->RGBColor[0.54, 0.35, 0.15]], "sublead3"]}]}], 
                 ")"}]}]}], ")"}]}], "+", 
            RowBox[{"8", " ", 
             SubscriptBox[
              StyleBox["W",
               FontColor->RGBColor[0.54, 0.35, 0.15]], "lead"], " ", 
             RowBox[{"(", 
              RowBox[{"1", "+", 
               RowBox[{
                SuperscriptBox["u", "2"], " ", 
                RowBox[{"(", 
                 RowBox[{"255", "-", 
                  RowBox[{"9", " ", 
                   SubscriptBox[
                    StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
               RowBox[{"54", " ", 
                SuperscriptBox["u", "4"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "16"}], "+", 
                  SubscriptBox[
                   StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}], ")"}]}], "+", 
               RowBox[{"6561", " ", 
                SuperscriptBox["u", "12"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "1"}], "+", 
                  SubscriptBox[
                   StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}], ")"}]}], "-", 
               RowBox[{"3", " ", 
                SubscriptBox[
                 StyleBox["\[Zeta]",
                  FontWeight->Bold,
                  FontColor->GrayLevel[0.4]], "3"]}], "+", 
               RowBox[{"810", " ", 
                SuperscriptBox["u", "8"], " ", 
                RowBox[{"(", 
                 RowBox[{"2", "+", 
                  RowBox[{"3", " ", 
                   SubscriptBox[
                    StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
               RowBox[{"729", " ", 
                SuperscriptBox["u", "10"], " ", 
                RowBox[{"(", 
                 RowBox[{"3", "+", 
                  RowBox[{"7", " ", 
                   SubscriptBox[
                    StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
               RowBox[{"27", " ", 
                SuperscriptBox["u", "6"], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"-", "271"}], "+", 
                  RowBox[{"21", " ", 
                   SubscriptBox[
                    StyleBox["\[Zeta]",
                    FontWeight->Bold,
                    FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], 
              ")"}]}]}], ")"}]}]}]}], "]"}]}], "+", 
     RowBox[{
      FractionBox["5371", "686"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead"], "[", "0", "]"}]}], "-", 
     FractionBox[
      RowBox[{"56741", " ", "\[ImaginaryI]", " ", 
       RowBox[{
        SubscriptBox["\[Alpha]", "sublead"], "[", "1", "]"}]}], "19208"], "-", 
     FractionBox[
      RowBox[{"17147", " ", "\[ImaginaryI]", " ", 
       RowBox[{
        SubscriptBox["\[Alpha]", "sublead"], "[", "2", "]"}]}], "5488"], "-", 
     
     RowBox[{
      FractionBox["11", "14"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "0", "]"}]}], "+", 
     RowBox[{
      FractionBox["139", "392"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "1", "]"}]}], "+", 
     RowBox[{
      FractionBox["247", "784"], " ", "\[ImaginaryI]", " ", 
      RowBox[{
       SubscriptBox["\[Alpha]", "sublead2"], "[", "2", "]"}]}], "+", 
     FractionBox[
      RowBox[{"1559", " ", "\[ImaginaryI]", " ", 
       RowBox[{
        SubscriptBox["\[Alpha]", "sublead2"], "[", "3", "]"}]}], "6272"], "+", 
     RowBox[{
      FractionBox["1", "3"], " ", "\[ImaginaryI]", " ", 
      SubscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"1", ",", "v0", ",", "sublead3"}]]}], "+", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "1", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"12819977", " ", "\[ImaginaryI]"}], "470596"], "-", 
        RowBox[{"6", " ", "\[ImaginaryI]", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Alpha]", "lead"], "[", "0", "]"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"2247821", " ", "\[ImaginaryI]"}], "33614"]}], "-", 
        RowBox[{"24", " ", "\[ImaginaryI]", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.555960705298842*^9, {3.555960743944848*^9, 3.555960792422225*^9}, 
   3.5559610786992197`*^9, 3.555961266284768*^9, {3.560337126952341*^9, 
   3.560337155414592*^9}, {3.560337187502145*^9, 3.560337217206129*^9}}]
}, Open  ]],

Cell[TextData[{
 "Contribution from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["W", "2"], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{
  3.55383020748068*^9, {3.553830700199875*^9, 3.5538307099398203`*^9}, {
   3.553830802049371*^9, 3.553830810573573*^9}, {3.553831134242115*^9, 
   3.553831137355349*^9}, {3.5538313716829157`*^9, 3.553831483505082*^9}, {
   3.555961735751958*^9, 3.555961748254781*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"tmpden", " ", 
       SuperscriptBox["g", "8"], 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["W", 
          RowBox[{"2", ",", "v0", ",", "lead"}], "+"], "+", 
         RowBox[{
          SuperscriptBox["g", "2"], 
          SubsuperscriptBox["W", 
           RowBox[{"2", ",", "v0", ",", "sublead"}], "+"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "4"], 
          SubsuperscriptBox["W", 
           RowBox[{"2", ",", "v0", ",", "sublead2"}], "+"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "6"], 
          SubsuperscriptBox["W", 
           RowBox[{"2", ",", "v0", ",", "sublead3"}], "+"]}]}], ")"}]}], ",", 
      
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "14"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", 
      SubsuperscriptBox["W", "__", "+"], ",", 
      SqrtBox["3"]}], "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"contTuW2", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"%", "-", 
     RowBox[{"(", 
      RowBox[{"%", "/.", "mconj"}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"g", ",", 
      SubsuperscriptBox["W", "__", "+"], ",", 
      SubsuperscriptBox["W", "__", "-"], ",", 
      SqrtBox["3"]}], "}"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.55596139933344*^9, 3.555961552445332*^9}, {
  3.555961591430263*^9, 3.555961628751438*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["3", "4"]}], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["3", "4"], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "+"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["9", "28"]}], "-", 
        FractionBox[
         RowBox[{"11", " ", "\[ImaginaryI]"}], 
         RowBox[{"28", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["9", "28"], "-", 
        FractionBox[
         RowBox[{"11", " ", "\[ImaginaryI]"}], 
         RowBox[{"28", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "+"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["3", "4"]}], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["3", "4"], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead"}], "+"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["1737", "1372"], "+", 
        FractionBox[
         RowBox[{"3215", " ", "\[ImaginaryI]"}], 
         RowBox[{"1372", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1737", "1372"]}], "+", 
        FractionBox[
         RowBox[{"3215", " ", "\[ImaginaryI]"}], 
         RowBox[{"1372", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "+"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["9", "28"]}], "-", 
        FractionBox[
         RowBox[{"11", " ", "\[ImaginaryI]"}], 
         RowBox[{"28", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["9", "28"], "-", 
        FractionBox[
         RowBox[{"11", " ", "\[ImaginaryI]"}], 
         RowBox[{"28", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead"}], "+"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["3", "4"]}], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead2"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["3", "4"], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead2"}], "+"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["1737", "1372"], "+", 
        FractionBox[
         RowBox[{"3215", " ", "\[ImaginaryI]"}], 
         RowBox[{"1372", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1737", "1372"]}], "+", 
        FractionBox[
         RowBox[{"3215", " ", "\[ImaginaryI]"}], 
         RowBox[{"1372", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead"}], "+"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["9", "28"]}], "-", 
        FractionBox[
         RowBox[{"11", " ", "\[ImaginaryI]"}], 
         RowBox[{"28", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead2"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["9", "28"], "-", 
        FractionBox[
         RowBox[{"11", " ", "\[ImaginaryI]"}], 
         RowBox[{"28", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead2"}], "+"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["3", "4"]}], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead3"}], "-"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["3", "4"], "-", 
        FractionBox["\[ImaginaryI]", 
         RowBox[{"4", " ", 
          SqrtBox["3"]}]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "sublead3"}], "+"]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "-"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["537363", "67228"]}], "+", 
        RowBox[{
         SqrtBox["3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"878173", " ", "\[ImaginaryI]"}], "201684"]}], "-", 
           RowBox[{"4", " ", "\[ImaginaryI]", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], ")"}]}], "+", 
     RowBox[{
      SubsuperscriptBox[
       StyleBox["W",
        FontColor->RGBColor[0.54, 0.35, 0.15]], 
       RowBox[{"2", ",", "v0", ",", "lead"}], "+"], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox["537363", "67228"], "+", 
        RowBox[{
         SqrtBox["3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"878173", " ", "\[ImaginaryI]"}], "201684"]}], "-", 
           RowBox[{"4", " ", "\[ImaginaryI]", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{{3.555961472144862*^9, 3.555961513305944*^9}, {
   3.555961599606495*^9, 3.555961629572899*^9}, 3.560337247361732*^9}]
}, Open  ]],

Cell[TextData[{
 "Contribution from ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[ScriptCapitalC]", "2"], 
    OverscriptBox["W", "^"]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{
  3.55383020748068*^9, {3.553830700199875*^9, 3.5538307099398203`*^9}, {
   3.553830802049371*^9, 3.553830810573573*^9}, {3.553831134242115*^9, 
   3.553831137355349*^9}, {3.5538313716829157`*^9, 3.553831483505082*^9}, {
   3.555961735751958*^9, 3.555961748254781*^9}, {3.5559617787433558`*^9, 
   3.555961792725968*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"49", "-", 
           RowBox[{"30", " ", 
            SuperscriptBox["u", "2"]}], "-", 
           RowBox[{"36", " ", 
            SuperscriptBox["u", "4"]}], "+", 
           RowBox[{"27", " ", 
            SuperscriptBox["u", "6"]}]}], ")"}], " ", 
         SubscriptBox[
          OverscriptBox[
           StyleBox["W",
            FontColor->RGBColor[0.54, 0.35, 0.15]], 
           StyleBox["^",
            FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"]}], 
        RowBox[{"27", " ", 
         SqrtBox["3"], " ", 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], 
          RowBox[{"[", 
           RowBox[{"-", "3"}], "]"}]], " ", 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "-"], " ", 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], "+"], " ", 
         SuperscriptBox[
          StyleBox["Q0",
           FontColor->RGBColor[{
              Rational[1, 2], 
              Rational[1, 2], 0}]], 
          RowBox[{"[", "3", "]"}]]}]]}], ")"}], "/.", "subtousu"}], "//", 
    "funapartusu"}], "//", "funreduceusu"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"%", "/.", 
       RowBox[{
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"], "\[Rule]", 
        "W"}]}], "/.", 
      RowBox[{"leadweak", "[", "1", "]"}]}], "/.", "subtousu"}], "//", 
    "funapartusu"}], "//", "funreduceusu"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"UpIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         FractionBox["%", 
          RowBox[{
           RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
         RowBox[{"Shift", "[", 
          RowBox[{"-", "\[Epsilon]"}], "]"}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"u", "-", 
          RowBox[{"2", "\[ImaginaryI]", " ", "\[Epsilon]"}]}]}]}], "/.", 
       "subusu"}], ",", "u"}], "]"}], "/.", "EndSimplify"}], "//", 
   "fundeperiod"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmp", "=", "%"}], ";"}]}], "Input",
 CellChangeTimes->{{3.567243531509877*^9, 3.567243586652976*^9}, {
  3.567243647586796*^9, 3.567243684343938*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"tmp", "-", 
   RowBox[{
    SuperscriptBox["g", "8"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       SqrtBox["3"], " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
      RowBox[{"2", " ", 
       SqrtBox["3"], " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], 
         StyleBox["_",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], ")"}]}]}], "//", 
  "FindHighTranCoef"}]], "Input",
 CellChangeTimes->{{3.567243892850985*^9, 3.56724391828187*^9}, {
  3.567243976196949*^9, 3.567243993378159*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    FractionBox["1312", "35"]}], " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"]}], "-", 
  RowBox[{"96", " ", "\[ImaginaryI]", " ", 
   SqrtBox["3"], " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
  RowBox[{"30", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "+", 
  RowBox[{"30", " ", 
   SuperscriptBox["g", "8"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], "2", "+"], " ", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[Eta]0",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]], 
     StyleBox["_",
      FontWeight->Bold,
      FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}]], "Output",
 CellChangeTimes->{{3.567243893730936*^9, 3.567243918910557*^9}, {
  3.567243976785633*^9, 3.567243997959812*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
     RowBox[{"2", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "8"}], " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "6", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
     RowBox[{"6", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "5", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}], "-", 
     RowBox[{"6", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "4", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "5", "-"]}], "+", 
     RowBox[{"8", " ", 
      SqrtBox["3"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]0",
        FontWeight->Bold,
        FontColor->RGBColor[0, 0.5, 0]], "3", "+"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]0",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0, 0.5, 0]]], "6", "-"]}]}], ")"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["tmpden"], "Input",
 CellChangeTimes->{{3.560337311173614*^9, 3.560337312124398*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["3", "4"], "-", 
  FractionBox["\[ImaginaryI]", 
   RowBox[{"4", " ", 
    SqrtBox["3"]}]], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["9", "28"], "-", 
     FractionBox[
      RowBox[{"11", " ", "\[ImaginaryI]"}], 
      RowBox[{"28", " ", 
       SqrtBox["3"]}]]}], ")"}], " ", 
   SuperscriptBox["g", "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1737", "1372"]}], "+", 
     FractionBox[
      RowBox[{"3215", " ", "\[ImaginaryI]"}], 
      RowBox[{"1372", " ", 
       SqrtBox["3"]}]]}], ")"}], " ", 
   SuperscriptBox["g", "4"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["537363", "67228"], "+", 
     RowBox[{
      SqrtBox["3"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"878173", " ", "\[ImaginaryI]"}], "201684"]}], "-", 
        RowBox[{"4", " ", "\[ImaginaryI]", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1277356307", "2470629"], "+", 
     RowBox[{
      SqrtBox["3"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"833279974", " ", "\[ImaginaryI]"}], "2470629"]}], "-", 
        FractionBox[
         RowBox[{"1098", " ", "\[ImaginaryI]", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], "49"]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{"3474", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"]}], "49"]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.5603373125183897`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tmpden", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"u", "+", 
         FractionBox[
          RowBox[{"3", "\[ImaginaryI]"}], "2"], "-", "v"}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "-", 
         FractionBox["\[ImaginaryI]", "2"], "-", "v"}]], "+", 
       FractionBox["1", 
        RowBox[{"u", "+", 
         FractionBox[
          RowBox[{"3", "\[ImaginaryI]"}], "2"], "+", "v"}]], "-", 
       FractionBox["1", 
        RowBox[{"u", "-", 
         FractionBox["\[ImaginaryI]", "2"], "+", "v"}]]}], ")"}]}], ")"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", "%", ")"}], "+", 
         RowBox[{"(", 
          RowBox[{"%", "/.", "mconj"}], ")"}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         SubscriptBox["v", "0"]}]}], "/.", 
       RowBox[{"leadweak", "[", "4", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "6"}], "}"}]}], "]"}], "/.", 
    RowBox[{"v", "\[Rule]", "u"}]}], "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"49", "+", 
      RowBox[{"186", " ", 
       SuperscriptBox["u", "2"]}], "+", 
      RowBox[{"567", " ", 
       SuperscriptBox["u", "4"]}], "+", 
      RowBox[{"378", " ", 
       SuperscriptBox["u", "6"]}], "+", 
      RowBox[{"81", " ", 
       SuperscriptBox["u", "8"]}]}], ")"}], "\[Rule]", 
    RowBox[{"(", 
     RowBox[{"81", 
      SuperscriptBox["Q0", "+"], 
      SuperscriptBox["Q0", "-"], 
      SuperscriptBox["Q0", 
       RowBox[{"[", 
        RowBox[{"+", "3"}], "]"}]], 
      SuperscriptBox["Q0", 
       RowBox[{"[", 
        RowBox[{"-", "3"}], "]"}]]}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"contTuWh", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"%", " ", 
       SuperscriptBox["g", "8"], 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["W", "^"], "lead"], "+", 
         RowBox[{
          SuperscriptBox["g", "2"], 
          SubscriptBox[
           OverscriptBox["W", "^"], "sublead"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "4"], 
          SubscriptBox[
           OverscriptBox["W", "^"], "sublead2"]}], "+", 
         RowBox[{
          SuperscriptBox["g", "6"], 
          SubscriptBox[
           OverscriptBox["W", "^"], "sublead3"]}]}], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "14"}], "}"}]}], "]"}], ",", "g", ",", 
    RowBox[{
     RowBox[{"\[DoubleStruckCapitalC]", "[", 
      RowBox[{"Collect", "[", 
       RowBox[{"#", ",", 
        SubscriptBox[
         OverscriptBox["W", "^"], "_"], ",", "Together"}], "]"}], "]"}], 
     "&"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.555961801624694*^9, 3.555961930094448*^9}, {
  3.555962032898609*^9, 3.555962238991325*^9}, {3.555962297110764*^9, 
  3.555962298897871*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"49", "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"]}], "-", 
         RowBox[{"36", " ", 
          SuperscriptBox["u", "4"]}], "+", 
         RowBox[{"27", " ", 
          SuperscriptBox["u", "6"]}]}], ")"}], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["W",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         StyleBox["^",
          FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"]}], 
      RowBox[{"27", " ", 
       SqrtBox["3"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "+"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", "3", "]"}]]}]]}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "33271"}], " ", 
           SqrtBox["3"]}], "-", 
          RowBox[{"841008", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "-", 
          RowBox[{"1159749", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "+", 
          RowBox[{"2382939", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "+", 
          RowBox[{"4990491", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "+", 
          RowBox[{"2654775", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "+", 
          RowBox[{"330966", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "-", 
          RowBox[{"24057", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}]}], ")"}], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"45927", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "2"]}], ")"}]}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"49", "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"]}], "-", 
         RowBox[{"36", " ", 
          SuperscriptBox["u", "4"]}], "+", 
         RowBox[{"27", " ", 
          SuperscriptBox["u", "6"]}]}], ")"}], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["W",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         StyleBox["^",
          FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead"]}], 
      RowBox[{"27", " ", 
       SqrtBox["3"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "+"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", "3", "]"}]]}]]}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"424023803", " ", 
           SqrtBox["3"]}], "+", 
          RowBox[{"3707799816", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "-", 
          RowBox[{"16479193416", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "-", 
          RowBox[{"102416977017", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "-", 
          RowBox[{"166315793319", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "-", 
          RowBox[{"160786675566", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "-", 
          RowBox[{"123265738116", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "-", 
          RowBox[{"60825607263", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}], "-", 
          RowBox[{"16708511601", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "16"]}], "-", 
          RowBox[{"2229887070", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "18"]}], "+", 
          RowBox[{"459991710", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "20"]}], "+", 
          RowBox[{"189842535", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "22"]}]}], ")"}], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"60761421", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "3"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "33271"}], " ", 
           SqrtBox["3"]}], "-", 
          RowBox[{"841008", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "-", 
          RowBox[{"1159749", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "+", 
          RowBox[{"2382939", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "+", 
          RowBox[{"4990491", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "+", 
          RowBox[{"2654775", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "+", 
          RowBox[{"330966", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "-", 
          RowBox[{"24057", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}]}], ")"}], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead"]}], ")"}], "/", 
      
      RowBox[{"(", 
       RowBox[{"45927", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "2"]}], ")"}]}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"49", "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"]}], "-", 
         RowBox[{"36", " ", 
          SuperscriptBox["u", "4"]}], "+", 
         RowBox[{"27", " ", 
          SuperscriptBox["u", "6"]}]}], ")"}], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["W",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         StyleBox["^",
          FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead2"]}], 
      RowBox[{"27", " ", 
       SqrtBox["3"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "+"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", "3", "]"}]]}]]}], "]"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"\[DoubleStruckCapitalC]", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"424023803", " ", 
           SqrtBox["3"]}], "+", 
          RowBox[{"3707799816", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "-", 
          RowBox[{"16479193416", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "-", 
          RowBox[{"102416977017", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "-", 
          RowBox[{"166315793319", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "-", 
          RowBox[{"160786675566", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "-", 
          RowBox[{"123265738116", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "-", 
          RowBox[{"60825607263", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}], "-", 
          RowBox[{"16708511601", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "16"]}], "-", 
          RowBox[{"2229887070", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "18"]}], "+", 
          RowBox[{"459991710", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "20"]}], "+", 
          RowBox[{"189842535", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "22"]}]}], ")"}], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead"]}], ")"}], "/", 
      
      RowBox[{"(", 
       RowBox[{"60761421", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "3"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "33271"}], " ", 
           SqrtBox["3"]}], "-", 
          RowBox[{"841008", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "-", 
          RowBox[{"1159749", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "+", 
          RowBox[{"2382939", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "+", 
          RowBox[{"4990491", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "+", 
          RowBox[{"2654775", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "+", 
          RowBox[{"330966", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "-", 
          RowBox[{"24057", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}]}], ")"}], " ", 
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead2"]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"45927", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "2"]}], ")"}]}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"49", "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"]}], "-", 
         RowBox[{"36", " ", 
          SuperscriptBox["u", "4"]}], "+", 
         RowBox[{"27", " ", 
          SuperscriptBox["u", "6"]}]}], ")"}], " ", 
       SubscriptBox[
        OverscriptBox[
         StyleBox["W",
          FontColor->RGBColor[0.54, 0.35, 0.15]], 
         StyleBox["^",
          FontColor->RGBColor[0.54, 0.35, 0.15]]], "sublead3"]}], 
      RowBox[{"27", " ", 
       SqrtBox["3"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", 
         RowBox[{"-", "3"}], "]"}]], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "-"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], "+"], " ", 
       SuperscriptBox[
        StyleBox["Q0",
         FontColor->RGBColor[{
            Rational[1, 2], 
            Rational[1, 2], 0}]], 
        RowBox[{"[", "3", "]"}]]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox[
         OverscriptBox[
          StyleBox["W",
           FontColor->RGBColor[0.54, 0.35, 0.15]], 
          StyleBox["^",
           FontColor->RGBColor[0.54, 0.35, 0.15]]], "lead"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "21554005399927"}], " ", 
           SqrtBox["3"]}], "-", 
          RowBox[{"3254285593986", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"]}], "+", 
          RowBox[{"2278990349568891", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"]}], "+", 
          RowBox[{"6852069203236551", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"]}], "-", 
          RowBox[{"777176147562174", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"]}], "-", 
          RowBox[{"16004895252581061", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"]}], "-", 
          RowBox[{"6896853930479049", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"]}], "+", 
          RowBox[{"37949279382974916", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"]}], "+", 
          RowBox[{"82090396462384314", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "16"]}], "+", 
          RowBox[{"83341487760916167", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "18"]}], "+", 
          RowBox[{"50535160937103267", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "20"]}], "+", 
          RowBox[{"18368311870526580", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "22"]}], "+", 
          RowBox[{"3364087745954277", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "24"]}], "+", 
          RowBox[{"35239995667629", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "26"]}], "-", 
          RowBox[{"94603989192336", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "28"]}], "-", 
          RowBox[{"12600822706911", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "30"]}], "-", 
          RowBox[{"2325336249768", " ", 
           SqrtBox["3"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"56601260493624", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "2"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"464346419236560", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "4"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"2338519257872280", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "6"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"7632313417883448", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "8"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"16859145795651936", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "10"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"24017860915124328", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "12"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"18979327336860072", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "14"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"5117582398649112", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "16"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "+", 
          RowBox[{"2354059479996000", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "18"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "+", 
          RowBox[{"232920886895928", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "20"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"2606622670950984", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "22"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"2189537319643632", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "24"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"817378676307144", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "26"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"152414434527768", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "28"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "-", 
          RowBox[{"11575779837552", " ", 
           SqrtBox["3"], " ", 
           SuperscriptBox["u", "30"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"723486239847", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", 
            RowBox[{"-", "3"}], "]"}]], ")"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "-"], ")"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], "+"], ")"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox[
           StyleBox["Q0",
            FontColor->RGBColor[{
               Rational[1, 2], 
               Rational[1, 2], 0}]], 
           RowBox[{"[", "3", "]"}]], ")"}], "4"]}], ")"}]}]}], 
    "]"}]}]}]], "Output",
 CellChangeTimes->{3.5559621990005827`*^9, 3.555962239805408*^9, 
  3.555962300503039*^9, 3.5603373641764307`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Tracing down ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Zeta]", 
    RowBox[{"1", ",", "2", ",", "8"}]], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Section",
 CellChangeTimes->{{3.567267468212706*^9, 3.5672674788533077`*^9}}],

Cell[TextData[{
 "We know that simply the asymptotic solution contributes to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Zeta]", 
    RowBox[{"1", ",", "2", ",", "8"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ". I will trace down first how asymptotic solution contributes, and then \
will check once more computation of the wrapping correction"
}], "Text",
 CellChangeTimes->{{3.567267689632647*^9, 3.567267729200017*^9}, {
  3.567267879291217*^9, 3.567267885410988*^9}}],

Cell["\<\
We have to first expand U at first 3 orders of transcendentality. Prefactor \
is not important we keep only the leading order for the prefactor.\
\>", "Text",
 CellChangeTimes->{{3.567267733128632*^9, 3.5672677393762836`*^9}, {
  3.567267904956677*^9, 3.567267936177476*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     SuperscriptBox["V", "+"], 
     SuperscriptBox["V", "-"]], "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["Bp", "+"], 
      SuperscriptBox["Bm", "+"]], 
     SubscriptBox["\[Sigma]", "BES"]}]}], ",", 
   RowBox[{"U", "\[Equal]", 
    RowBox[{
     FractionBox["1", 
      SqrtBox[
       SubscriptBox["C", "\[GothicCapitalT]"]]], "Bm", " ", 
     FractionBox["V", 
      SuperscriptBox["x", 
       RowBox[{"L", "/", "2"}]]]}]}]}], "}"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"prefactor", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  FractionBox["1", 
                   SqrtBox[
                    SubscriptBox["C", "\[GothicCapitalT]"]]], "/.", "weak"}], 
                 ")"}], 
                FractionBox["Bm", 
                 SuperscriptBox["x", 
                  RowBox[{"L", "/", "2"}]]]}], "/.", 
               RowBox[{"L", "\[Rule]", "2"}]}], "/.", "subFunsBethe"}], "/.", 
             
             RowBox[{"x", "\[Rule]", "ee"}]}], "/.", "subJuk"}], "/.", 
           RowBox[{"leadweak", "[", "5", "]"}]}], "/.", 
          RowBox[{"ee", "\[Rule]", "xh"}]}], "/.", "subtotran"}], ",", 
        RowBox[{"{", 
         RowBox[{"tran", ",", "\[Infinity]", ",", "3"}], "}"}]}], "]"}], "//",
       "Simplify"}], "//", "Normal"}], ")"}], "/.", 
   RowBox[{"tran", "\[Rule]", "1"}]}]}]], "Input",
 CellChangeTimes->{{3.56726795914668*^9, 3.567268064020465*^9}, {
  3.567272695415061*^9, 3.56727269710289*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"3", " ", 
     SqrtBox["2"], " ", 
     SuperscriptBox["g", "2"]}], 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], "+", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SqrtBox["2"], " ", "g"}], 
   OverscriptBox[
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]], 
    StyleBox["^",
     FontColor->RGBColor[0, 0.6, 0]]]], "+", 
  FractionBox[
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "2"}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]}], ")"}]}], 
   RowBox[{
    SqrtBox["2"], " ", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "3"]}]]}]], "Output",
 CellChangeTimes->{{3.567267960095049*^9, 3.567268064295315*^9}, 
   3.567272697862411*^9}]
}, Open  ]],

Cell[TextData[{
 "For V, the dressing phase cannot contribute to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Zeta]", 
    RowBox[{"1", ",", "2", ",", "8"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " (because it starts by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Zeta]", "3"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " at least, so it will go to some products of zeta functions. Hence we have \
to deal only with ",
 Cell[BoxData[
  RowBox[{
   FractionBox[
    SuperscriptBox["V", "+"], 
    SuperscriptBox["V", "-"]], "=", 
   FractionBox[
    SuperscriptBox["Bp", "+"], 
    SuperscriptBox["Bm", "+"]]}]]],
 ")"
}], "Text",
 CellChangeTimes->{{3.567268069388494*^9, 3.567268119182373*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["V", 
    RowBox[{"1", "-", 
     SuperscriptBox["D", 
      RowBox[{"-", "2"}]]}]], "==", 
   FractionBox["Bp", "Bm"]}], ";", 
  RowBox[{"V", "==", 
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox["Bp", "Bm"], ")"}], 
    RowBox[{"-", 
     FractionBox[
      SuperscriptBox["D", "2"], 
      RowBox[{"1", "-", 
       SuperscriptBox["D", "2"]}]]}]]}]}]], "Input",
 CellChangeTimes->{{3.567268169143208*^9, 3.567268197843451*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"-", 
               RowBox[{"Log", "[", 
                RowBox[{"-", 
                 FractionBox[
                  SuperscriptBox["Bp", "+"], 
                  SuperscriptBox["Bm", "+"]]}], "]"}]}], "/.", 
              "subFunsBethe"}], "/.", 
             RowBox[{"Shift", "[", 
              RowBox[{"-", "1"}], "]"}]}], "/.", 
            RowBox[{"x", "\[Rule]", "ee"}]}], "/.", "subJuk"}], "/.", 
          RowBox[{"leadweak", "[", "7", "]"}]}], "/.", "subtotran"}], "/.", 
        RowBox[{"ee", "\[Rule]", "x"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], "//.",
      "densqrt3"}], "//", "Simplify"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"a_", "/;", 
      RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"\[Eta]$h", "[", "1", "]"}], "[", "2", "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Vcontr", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Exp", "[", "%", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], "//", 
    "Simplify"}], "//", "Normal"}]}]}], "Input",
 CellChangeTimes->{{3.5672681214811077`*^9, 3.567268142148343*^9}, {
   3.567268203059972*^9, 3.567268214066245*^9}, {3.567268247295109*^9, 
   3.5672683158834763`*^9}, {3.5672683599123087`*^9, 3.567268362273841*^9}, {
   3.5672684428715897`*^9, 3.567268442991494*^9}, {3.5672685148768597`*^9, 
   3.567268515084447*^9}, {3.567268579686269*^9, 3.5672686801320543`*^9}, 
   3.567272500675362*^9}],

Cell[BoxData[
 RowBox[{"1", "+", 
  FractionBox[
   RowBox[{"6", " ", "\[ImaginaryI]", " ", "g", " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", "2", "]"}]]}], "tran"], "-", 
  FractionBox[
   RowBox[{"18", " ", 
    SuperscriptBox["g", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "1", 
       RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
   SuperscriptBox["tran", "2"]]}]], "Output",
 CellChangeTimes->{{3.567268131942795*^9, 3.567268147312153*^9}, 
   3.5672682045384407`*^9, {3.567268258995863*^9, 3.567268262352784*^9}, {
   3.5672682925789747`*^9, 3.567268317197069*^9}, {3.5672683575401993`*^9, 
   3.567268363764721*^9}, 3.567268443745716*^9, {3.5672685067939787`*^9, 
   3.5672685153559723`*^9}, 3.567268568497912*^9, {3.567268605245721*^9, 
   3.567268680638081*^9}, 3.567272503808387*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"1", "+", 
        FractionBox[
         RowBox[{"6", " ", "\[ImaginaryI]", " ", "g", " ", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "1", 
           RowBox[{"[", "2", "]"}]]}], "tran"], "-", 
        FractionBox[
         RowBox[{"18", " ", 
          SuperscriptBox["g", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox[
             OverscriptBox[
              StyleBox["\[Eta]",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]], 
              StyleBox["^",
               FontWeight->Bold,
               FontColor->RGBColor[0.8, 0, 0]]], "1", 
             RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
         SuperscriptBox["tran", "2"]]}], "/.", 
       RowBox[{"sub\[Eta]h\[Eta]", "[", "8", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//", "Normal"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.567272562998403*^9, 3.567272598488166*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   FractionBox[
    RowBox[{"6", " ", "\[ImaginaryI]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "2"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        SuperscriptBox["g", "4"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["g", "6"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "5", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"5", " ", 
        SuperscriptBox["g", "8"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "7", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "tran"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "18"}], " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
     RowBox[{"36", " ", 
      SuperscriptBox["g", "6"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "3", 
       RowBox[{"[", "2", "]"}]]}], "-", 
     RowBox[{"18", " ", 
      SuperscriptBox["g", "8"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
     RowBox[{"72", " ", 
      SuperscriptBox["g", "8"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "5", 
       RowBox[{"[", "2", "]"}]]}]}], 
    SuperscriptBox["tran", "2"]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "3"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 0, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[1], {
   1, Complex[
     0, 6] ($CellContext`g^2 $CellContext`\[Eta]$[1][
       2] + $CellContext`g^4 $CellContext`\[Eta]$[3][2] + 
     2 $CellContext`g^6 $CellContext`\[Eta]$[5][2] + 
     5 $CellContext`g^8 $CellContext`\[Eta]$[7][
       2]), (-18) $CellContext`g^4 $CellContext`\[Eta]$[1][2]^2 - 
    36 $CellContext`g^6 $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[3][2] - 
    18 $CellContext`g^8 $CellContext`\[Eta]$[3][2]^2 - 
    72 $CellContext`g^8 $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[5][
     2]}, 0, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.5672725928505583`*^9, 3.5672725990645933`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"tmpV", "/.", 
    RowBox[{"ee", "\[Rule]", "0"}]}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{"%", "/.", "subtotran"}], ",", 
   RowBox[{"{", 
    RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}]}], "Input",\

 CellChangeTimes->{{3.567271699759807*^9, 3.56727170621943*^9}, {
  3.567272518254252*^9, 3.567272549875689*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   FractionBox[
    RowBox[{"6", " ", "\[ImaginaryI]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", "2"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{
        SuperscriptBox["g", "4"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["g", "6"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "5", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"5", " ", 
        SuperscriptBox["g", "8"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "7", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "tran"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "18"}], " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
     RowBox[{"36", " ", 
      SuperscriptBox["g", "6"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "3", 
       RowBox[{"[", "2", "]"}]]}], "-", 
     RowBox[{"18", " ", 
      SuperscriptBox["g", "8"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
     RowBox[{"72", " ", 
      SuperscriptBox["g", "8"], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "5", 
       RowBox[{"[", "2", "]"}]]}]}], 
    SuperscriptBox["tran", "2"]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "3"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 0, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[1], {
   1, Complex[
     0, 6] ($CellContext`g^2 $CellContext`\[Eta]$[1][
       2] + $CellContext`g^4 $CellContext`\[Eta]$[3][2] + 
     2 $CellContext`g^6 $CellContext`\[Eta]$[5][2] + 
     5 $CellContext`g^8 $CellContext`\[Eta]$[7][
       2]), (-18) $CellContext`g^4 $CellContext`\[Eta]$[1][2]^2 - 
    36 $CellContext`g^6 $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[3][2] - 
    18 $CellContext`g^8 $CellContext`\[Eta]$[3][2]^2 - 
    72 $CellContext`g^8 $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[5][
     2]}, 0, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{
  3.567271707521016*^9, {3.56727252195223*^9, 3.567272551062903*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["prefactor"], "Input",
 CellChangeTimes->{{3.567272655607382*^9, 3.567272656846953*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"3", " ", 
     SqrtBox["2"], " ", 
     SuperscriptBox["g", "2"]}], 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "2"]]}], "+", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SqrtBox["2"], " ", "g"}], 
   StyleBox["x",
    FontColor->RGBColor[0, 0.6, 0]]], "+", 
  FractionBox[
   RowBox[{"3", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "2"}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], "2"]}]}], ")"}]}], 
   RowBox[{
    SqrtBox["2"], " ", 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], "3"]}]]}]], "Output",
 CellChangeTimes->{3.5672726573525543`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Usqtran", "=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"prefactor", " ", "Vcontr"}], ")"}], "2"], "/.", 
         RowBox[{"tran", "\[Rule]", "1"}]}], "/.", "subtotran"}], ",", 
       RowBox[{"{", 
        RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}], "/.", 
     
     RowBox[{"x", "\[Rule]", "xh"}]}], "//", "Normal"}], ")"}]}]], "Input",
 CellChangeTimes->{{3.567268681545453*^9, 3.567268750721637*^9}, {
  3.5672688715414248`*^9, 3.56726887407733*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox["g", "2"]}], 
    RowBox[{
     SuperscriptBox["tran", "2"], " ", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], "-", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "3"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"2", " ", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "1", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
   RowBox[{
    SuperscriptBox["tran", "3"], " ", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "3"]}]], "+", 
  FractionBox[
   RowBox[{
    FractionBox[
     RowBox[{"30", " ", 
      SuperscriptBox["g", "4"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
    FractionBox[
     RowBox[{"18", " ", 
      SuperscriptBox["g", "4"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
    FractionBox[
     RowBox[{"144", " ", 
      SuperscriptBox["g", "4"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "1", 
       RowBox[{"[", "2", "]"}]]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "3"]], "+", 
    FractionBox[
     RowBox[{"144", " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "1", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], 
   SuperscriptBox["tran", "4"]]}]], "Output",
 CellChangeTimes->{
  3.5672686853584146`*^9, {3.5672687225317383`*^9, 3.567268751058648*^9}, 
   3.567268874469428*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tmp1", "=", 
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Series", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Usqtran", "/.", 
              RowBox[{"sub\[Eta]h\[Eta]", "[", "12", "]"}]}], "/.", 
             "subJuk"}], ",", 
            RowBox[{"{", 
             RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}], "//", 
          "Normal"}], ")"}], "/.", "subtousu"}], "//", "funapartusu"}], "//", 
      "funreduceusu"}], ",", 
     RowBox[{"{", 
      RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.567272742668681*^9, 3.5672727945667973`*^9}, {
  3.56727290051042*^9, 3.567272920019305*^9}}],

Cell[CellGroupData[{

Cell[BoxData["tmp1"], "Input",
 CellChangeTimes->{{3.567272847379656*^9, 3.567272847903804*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"84", " ", 
        SuperscriptBox["g", "12"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "10"]]}], "-", 
     FractionBox[
      RowBox[{"28", " ", 
       SuperscriptBox["g", "10"]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "8"]], "-", 
     FractionBox[
      RowBox[{"10", " ", 
       SuperscriptBox["g", "8"]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "6"]], "-", 
     FractionBox[
      RowBox[{"4", " ", 
       SuperscriptBox["g", "6"]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["g", "4"]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]]}], 
    SuperscriptBox["tran", "2"]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "9"], " ", 
      SuperscriptBox["tran", "3"]}]], 
    RowBox[{"12", " ", "\[ImaginaryI]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"28", " ", 
        SuperscriptBox["g", "12"]}], "+", 
       RowBox[{"9", " ", 
        SuperscriptBox["g", "10"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "2"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox["g", "8"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "4"]}], "+", 
       RowBox[{
        SuperscriptBox["g", "6"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "6"]}], "+", 
       RowBox[{"28", " ", 
        SuperscriptBox["g", "12"], " ", 
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"10", " ", 
        SuperscriptBox["g", "10"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "3"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["g", "8"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "5"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["g", "6"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "7"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"10", " ", 
        SuperscriptBox["g", "12"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "3"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["g", "10"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "5"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["g", "8"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "7"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"8", " ", 
        SuperscriptBox["g", "12"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "5"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "5", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["g", "10"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "7"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "5", 
         RowBox[{"[", "2", "]"}]]}], "+", 
       RowBox[{"10", " ", 
        SuperscriptBox["g", "12"], " ", 
        SuperscriptBox[
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]], "7"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "7", 
         RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     SuperscriptBox["tran", "4"]], 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"168", " ", 
        SuperscriptBox["g", "12"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "8"]], "+", 
      FractionBox[
       RowBox[{"30", " ", 
        SuperscriptBox["g", "10"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "6"]], "-", 
      FractionBox[
       RowBox[{"6", " ", 
        SuperscriptBox["g", "8"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
      FractionBox[
       RowBox[{"18", " ", 
        SuperscriptBox["g", "6"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
      FractionBox[
       RowBox[{"1296", " ", 
        SuperscriptBox["g", "12"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "7"]], "+", 
      FractionBox[
       RowBox[{"432", " ", 
        SuperscriptBox["g", "10"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
      FractionBox[
       RowBox[{"144", " ", 
        SuperscriptBox["g", "8"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
      FractionBox[
       RowBox[{"720", " ", 
        SuperscriptBox["g", "12"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "6"]], "+", 
      FractionBox[
       RowBox[{"288", " ", 
        SuperscriptBox["g", "10"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
      FractionBox[
       RowBox[{"144", " ", 
        SuperscriptBox["g", "8"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
      FractionBox[
       RowBox[{"432", " ", 
        SuperscriptBox["g", "12"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
      FractionBox[
       RowBox[{"144", " ", 
        SuperscriptBox["g", "10"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
      FractionBox[
       RowBox[{"576", " ", 
        SuperscriptBox["g", "12"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
      FractionBox[
       RowBox[{"288", " ", 
        SuperscriptBox["g", "10"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "3", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
      FractionBox[
       RowBox[{"144", " ", 
        SuperscriptBox["g", "12"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
      FractionBox[
       RowBox[{"288", " ", 
        SuperscriptBox["g", "12"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "5", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
      FractionBox[
       RowBox[{"576", " ", 
        SuperscriptBox["g", "12"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "1", 
         RowBox[{"[", "2", "]"}]], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], "5", 
         RowBox[{"[", "2", "]"}]]}], 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "5"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 2, 5, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[
   1], {(-84) $CellContext`g^12 $CellContext`\[ScriptU]$[0][0]^(-10) - 
    28 $CellContext`g^10 $CellContext`\[ScriptU]$[0][0]^(-8) - 
    10 $CellContext`g^8 $CellContext`\[ScriptU]$[0][0]^(-6) - 
    4 $CellContext`g^6 $CellContext`\[ScriptU]$[0][0]^(-4) - 
    2 $CellContext`g^4 $CellContext`\[ScriptU]$[0][0]^(-2), 
    Complex[0, -12] $CellContext`\[ScriptU]$[0][0]^(-9) (28 $CellContext`g^12 + 
     9 $CellContext`g^10 $CellContext`\[ScriptU]$[0][0]^2 + 
     3 $CellContext`g^8 $CellContext`\[ScriptU]$[0][
        0]^4 + $CellContext`g^6 $CellContext`\[ScriptU]$[0][0]^6 + 
     28 $CellContext`g^12 $CellContext`\[ScriptU]$[0][0] $CellContext`\[Eta]$[
       1][2] + 10 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
        0]^3 $CellContext`\[Eta]$[1][2] + 
     4 $CellContext`g^8 $CellContext`\[ScriptU]$[0][0]^5 $CellContext`\[Eta]$[
       1][2] + 2 $CellContext`g^6 $CellContext`\[ScriptU]$[0][
        0]^7 $CellContext`\[Eta]$[1][2] + 
     10 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
        0]^3 $CellContext`\[Eta]$[3][2] + 
     4 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
        0]^5 $CellContext`\[Eta]$[3][2] + 
     2 $CellContext`g^8 $CellContext`\[ScriptU]$[0][0]^7 $CellContext`\[Eta]$[
       3][2] + 8 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
        0]^5 $CellContext`\[Eta]$[5][2] + 
     4 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
        0]^7 $CellContext`\[Eta]$[5][2] + 
     10 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
        0]^7 $CellContext`\[Eta]$[7][2]), 
    168 $CellContext`g^12 $CellContext`\[ScriptU]$[0][0]^(-8) + 
    30 $CellContext`g^10 $CellContext`\[ScriptU]$[0][0]^(-6) - 
    6 $CellContext`g^8 $CellContext`\[ScriptU]$[0][0]^(-4) - 
    18 $CellContext`g^6 $CellContext`\[ScriptU]$[0][0]^(-2) + 
    1296 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-7) $CellContext`\[Eta]$[1][2] + 
    432 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
       0]^(-5) $CellContext`\[Eta]$[1][2] + 
    144 $CellContext`g^8 $CellContext`\[ScriptU]$[0][
       0]^(-3) $CellContext`\[Eta]$[1][2] + 
    720 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-6) $CellContext`\[Eta]$[1][2]^2 + 
    288 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
       0]^(-4) $CellContext`\[Eta]$[1][2]^2 + 
    144 $CellContext`g^8 $CellContext`\[ScriptU]$[0][
       0]^(-2) $CellContext`\[Eta]$[1][2]^2 + 
    432 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-5) $CellContext`\[Eta]$[3][2] + 
    144 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
       0]^(-3) $CellContext`\[Eta]$[3][2] + 
    576 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-4) $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[3][2] + 
    288 $CellContext`g^10 $CellContext`\[ScriptU]$[0][
       0]^(-2) $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[3][2] + 
    144 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-2) $CellContext`\[Eta]$[3][2]^2 + 
    288 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-3) $CellContext`\[Eta]$[5][2] + 
    576 $CellContext`g^12 $CellContext`\[ScriptU]$[0][
       0]^(-2) $CellContext`\[Eta]$[1][2] $CellContext`\[Eta]$[5][2]}, 2, 5, 
   1],
  Editable->False]], "Output",
 CellChangeTimes->{3.567272848511397*^9, 3.567272923444673*^9}]
}, Open  ]],

Cell[BoxData["t"], "Input",
 CellChangeTimes->{3.567272893090485*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{"Series", "[", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{"Series", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             SubsuperscriptBox["U", "\"\<as\>\"", "2"], "/.", 
             RowBox[{"nsweak", "[", "5", "]"}]}], "/.", "subtotran"}], ",", 
           RowBox[{"{", 
            RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}], "//",
          "Normal"}], 
        RowBox[{"tmp1", "//", "Normal"}]], ",", 
       RowBox[{"{", 
        RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], "//.", 
   "subtopowerbasis"}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.5672727959893637`*^9, 3.567272880908511*^9}, {
  3.5672729457387857`*^9, 3.567272980135117*^9}, {3.567273018711945*^9, 
  3.567273024160966*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   FractionBox[
    InterpretationBox[
     RowBox[{
      RowBox[{"-", 
       RowBox[{"12", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"], " ", 
        SuperscriptBox["g", "2"]}]}], "+", 
      RowBox[{"36", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "3"], " ", 
       SuperscriptBox["g", "4"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"24", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
         RowBox[{"240", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "5"]}]}], ")"}], " ", 
       SuperscriptBox["g", "6"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"48", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
         FractionBox[
          RowBox[{"240", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "5"]}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
         RowBox[{"2100", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "7"]}]}], ")"}], " ", 
       SuperscriptBox["g", "8"]}], "+", 
      InterpretationBox[
       SuperscriptBox[
        RowBox[{"O", "[", "g", "]"}], "9"],
       SeriesData[$CellContext`g, 0, {}, 2, 9, 1],
       Editable->False]}],
     SeriesData[$CellContext`g, 
      0, {(-12) $CellContext`\[Zeta]$[1][0], 0, 
       36 $CellContext`\[Zeta]$[3][0], 0, 
       24 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[3][0] - 
       240 $CellContext`\[Zeta]$[5][0], 0, 
       48 $CellContext`\[ScriptU]$[0][0]^(-4) $CellContext`\[Zeta]$[3][0] - 
       240 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[5][0] + 
       2100 $CellContext`\[Zeta]$[7][0]}, 2, 9, 1],
     Editable->False], "tran"], "+", 
   FractionBox[
    InterpretationBox[
     RowBox[{
      RowBox[{"72", " ", 
       SubsuperscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1", "2"], " ", 
       SuperscriptBox["g", "4"]}], "-", 
      RowBox[{"432", " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], ")"}], " ", 
       SuperscriptBox["g", "6"]}], "+", 
      RowBox[{"72", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"4", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "1"], " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"]}], 
           SuperscriptBox[
            StyleBox["\[ScriptU]",
             FontColor->RGBColor[0, 0, 0.4]], "2"]]}], "+", 
         RowBox[{"9", " ", 
          SubsuperscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
         RowBox[{"40", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "1"], " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "5"]}]}], ")"}], " ", 
       SuperscriptBox["g", "8"]}], "+", 
      InterpretationBox[
       SuperscriptBox[
        RowBox[{"O", "[", "g", "]"}], "9"],
       SeriesData[$CellContext`g, 0, {}, 4, 9, 1],
       Editable->False]}],
     SeriesData[$CellContext`g, 0, {
      72 $CellContext`\[Zeta]$[1][0]^2, 
       0, (-432) $CellContext`\[Zeta]$[1][0] $CellContext`\[Zeta]$[3][0], 0, 
       72 ((-4) $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[1][
          0] $CellContext`\[Zeta]$[3][0] + 9 $CellContext`\[Zeta]$[3][0]^2 + 
        40 $CellContext`\[Zeta]$[1][0] $CellContext`\[Zeta]$[5][0])}, 4, 9, 1],
     Editable->False], 
    SuperscriptBox["tran", "2"]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", 
      FractionBox["1", "tran"], "]"}], "3"],
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {}, 0, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`tran, 
   DirectedInfinity[1], {1, 
    SeriesData[$CellContext`g, 
     0, {(-12) $CellContext`\[Zeta]$[1][0], 0, 36 $CellContext`\[Zeta]$[3][0],
       0, 24 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[3][0] - 
      240 $CellContext`\[Zeta]$[5][0], 0, 
      48 $CellContext`\[ScriptU]$[0][0]^(-4) $CellContext`\[Zeta]$[3][0] - 
      240 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[5][0] + 
      2100 $CellContext`\[Zeta]$[7][0]}, 2, 9, 1], 
    SeriesData[$CellContext`g, 0, {
     72 $CellContext`\[Zeta]$[1][0]^2, 
      0, (-432) $CellContext`\[Zeta]$[1][0] $CellContext`\[Zeta]$[3][0], 0, 
      72 ((-4) $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[1][
         0] $CellContext`\[Zeta]$[3][0] + 9 $CellContext`\[Zeta]$[3][0]^2 + 
       40 $CellContext`\[Zeta]$[1][0] $CellContext`\[Zeta]$[5][0])}, 4, 9, 
     1]}, 0, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.567272807105015*^9, 3.567272884304659*^9}, {
   3.567272931137475*^9, 3.567272981454439*^9}, 3.5672730251709557`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"m1tmp", "=", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Usqtran", "/.", 
       RowBox[{"sub\[Eta]h\[Eta]", "[", "12", "]"}]}], "/.", "subJuk"}], ",", 
     
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "12"}], "}"}]}], "]"}], "//", 
   "Normal"}]}]], "Input",
 CellChangeTimes->{{3.5672711956403227`*^9, 3.567271232078389*^9}, {
  3.567271311660182*^9, 3.567271324938793*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox["g", "4"]}], 
    RowBox[{
     SuperscriptBox["tran", "2"], " ", 
     SuperscriptBox["u", "2"]}]]}], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["4", 
       RowBox[{
        SuperscriptBox["tran", "2"], " ", 
        SuperscriptBox["u", "4"]}]]}], "-", 
     FractionBox["18", 
      RowBox[{
       SuperscriptBox["tran", "4"], " ", 
       SuperscriptBox["u", "2"]}]], "-", 
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"2", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["tran", "3"], " ", 
       SuperscriptBox["u", "3"]}]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["10", 
       RowBox[{
        SuperscriptBox["tran", "2"], " ", 
        SuperscriptBox["u", "6"]}]]}], "+", 
     FractionBox[
      RowBox[{"6", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"24", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox["u", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "1", 
             RowBox[{"[", "2", "]"}]], ")"}], "2"]}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["tran", "4"], " ", 
       SuperscriptBox["u", "4"]}]], "-", 
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{"3", "+", 
         RowBox[{"4", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["u", "3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["tran", "3"], " ", 
       SuperscriptBox["u", "5"]}]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["28", 
       RowBox[{
        SuperscriptBox["tran", "2"], " ", 
        SuperscriptBox["u", "8"]}]]}], "+", 
     FractionBox[
      RowBox[{"6", " ", 
       RowBox[{"(", 
        RowBox[{"5", "+", 
         RowBox[{"72", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"48", " ", 
          SuperscriptBox["u", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "1", 
             RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox["u", "3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"48", " ", 
          SuperscriptBox["u", "4"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["tran", "4"], " ", 
       SuperscriptBox["u", "6"]}]], "-", 
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{"9", "+", 
         RowBox[{"10", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["u", "3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["u", "5"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "5", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["tran", "3"], " ", 
       SuperscriptBox["u", "7"]}]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["84", 
       RowBox[{
        SuperscriptBox["tran", "2"], " ", 
        SuperscriptBox["u", "10"]}]]}], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{
        SuperscriptBox["tran", "4"], " ", 
        SuperscriptBox["u", "8"]}]], 
      RowBox[{"24", " ", 
       RowBox[{"(", 
        RowBox[{"7", "+", 
         RowBox[{"54", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"30", " ", 
          SuperscriptBox["u", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "1", 
             RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "+", 
         RowBox[{"18", " ", 
          SuperscriptBox["u", "3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox["u", "4"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"6", " ", 
          SuperscriptBox["u", "6"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "3", 
             RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "+", 
         RowBox[{"12", " ", 
          SuperscriptBox["u", "5"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "5", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox["u", "6"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "5", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}]}], "-", 
     FractionBox[
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{"14", "+", 
         RowBox[{"14", " ", "u", " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "1", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["u", "3"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "3", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["u", "5"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "5", 
           RowBox[{"[", "2", "]"}]]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["u", "7"], " ", 
          SubsuperscriptBox[
           StyleBox["\[Eta]",
            FontWeight->Bold,
            FontColor->RGBColor[0.8, 0, 0]], "7", 
           RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["tran", "3"], " ", 
       SuperscriptBox["u", "9"]}]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.567271200278718*^9, 3.56727123258635*^9}, {
  3.567271312852373*^9, 3.5672713254405117`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox["U", "2"], "/.", 
           RowBox[{"nsweak", "[", "5", "]"}]}], "/.", "subtotran"}], ",", 
         RowBox[{"{", 
          RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}], "//",
        "Normal"}], 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"m1tmp", "/.", "subtousu"}], "//", "funapartusu"}], "//", 
        "funreduceusu"}], "//", "FuntoMHZbasis"}]], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "8"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"tran", ",", "\[Infinity]", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.567271162071661*^9, 3.567271165925415*^9}, {
   3.5672712621642447`*^9, 3.5672714016620617`*^9}, {3.56727143276788*^9, 
   3.567271475732876*^9}, 3.567271547924664*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"12", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "1"]}], "tran"]}], "+", 
       InterpretationBox[
        SuperscriptBox[
         RowBox[{"O", "[", 
          FractionBox["1", "tran"], "]"}], "3"],
        SeriesData[$CellContext`tran, 
         DirectedInfinity[1], {}, 1, 3, 1],
        Editable->False]}],
      SeriesData[$CellContext`tran, 
       DirectedInfinity[1], {(-12) $CellContext`\[Zeta]$[1][0]}, 1, 3, 1],
      Editable->False], ")"}], " ", 
    SuperscriptBox["g", "2"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{
       FractionBox[
        RowBox[{"36", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "tran"], "+", 
       FractionBox[
        RowBox[{"72", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1", "2"]}], 
        SuperscriptBox["tran", "2"]], "+", 
       InterpretationBox[
        SuperscriptBox[
         RowBox[{"O", "[", 
          FractionBox["1", "tran"], "]"}], "3"],
        SeriesData[$CellContext`tran, 
         DirectedInfinity[1], {}, 1, 3, 1],
        Editable->False]}],
      SeriesData[$CellContext`tran, 
       DirectedInfinity[1], {
       36 $CellContext`\[Zeta]$[3][0], 72 $CellContext`\[Zeta]$[1][0]^2}, 1, 
       3, 1],
      Editable->False], ")"}], " ", 
    SuperscriptBox["g", "4"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{
       FractionBox[
        RowBox[{
         FractionBox[
          RowBox[{"24", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
         RowBox[{"240", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "5"]}]}], "tran"], "-", 
       FractionBox[
        RowBox[{"432", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], ")"}]}], 
        SuperscriptBox["tran", "2"]], "+", 
       InterpretationBox[
        SuperscriptBox[
         RowBox[{"O", "[", 
          FractionBox["1", "tran"], "]"}], "3"],
        SeriesData[$CellContext`tran, 
         DirectedInfinity[1], {}, 1, 3, 1],
        Editable->False]}],
      SeriesData[$CellContext`tran, 
       DirectedInfinity[1], {
       24 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[3][0] - 
        240 $CellContext`\[Zeta]$[5][0], (-432) $CellContext`\[Zeta]$[1][
         0] $CellContext`\[Zeta]$[3][0]}, 1, 3, 1],
      Editable->False], ")"}], " ", 
    SuperscriptBox["g", "6"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{
       FractionBox[
        RowBox[{
         FractionBox[
          RowBox[{"48", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
         FractionBox[
          RowBox[{"240", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "5"]}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
         RowBox[{"2520", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "7"]}]}], "tran"], "+", 
       RowBox[{
        FractionBox["1", 
         SuperscriptBox["tran", "2"]], 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"288", " ", "\[ImaginaryI]", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"]}], 
           SuperscriptBox[
            StyleBox["\[ScriptU]",
             FontColor->RGBColor[0, 0, 0.4]], "3"]], "-", 
          FractionBox[
           RowBox[{"288", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "1"], " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"]}], 
           SuperscriptBox[
            StyleBox["\[ScriptU]",
             FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
          RowBox[{"864", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
          FractionBox[
           RowBox[{"720", " ", "\[ImaginaryI]", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "5"]}], 
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]]], "+", 
          RowBox[{"4320", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "5"]}], "-", 
          RowBox[{"1440", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "5"], " ", 
           SubsuperscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], "1", 
            RowBox[{"[", "2", "]"}]]}], "+", 
          FractionBox[
           RowBox[{"36", " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "2", 
             RowBox[{"[", "2", "]"}]]}], 
           SuperscriptBox[
            StyleBox["\[ScriptU]",
             FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
          FractionBox[
           RowBox[{"432", " ", "\[ImaginaryI]", " ", 
            SubscriptBox[
             StyleBox["\[Zeta]",
              FontWeight->Bold,
              FontColor->GrayLevel[0.4]], "3"], " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "2", 
             RowBox[{"[", "2", "]"}]]}], 
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]]], "-", 
          FractionBox[
           RowBox[{"72", " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "3", 
             RowBox[{"[", "2", "]"}]]}], 
           SuperscriptBox[
            StyleBox["\[ScriptU]",
             FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
          RowBox[{"144", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"], " ", 
           SubsuperscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], "3", 
            RowBox[{"[", "2", "]"}]]}], "+", 
          FractionBox[
           RowBox[{"36", " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], "4", 
             RowBox[{"[", "2", "]"}]]}], 
           SuperscriptBox[
            StyleBox["\[ScriptU]",
             FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
          FractionBox[
           RowBox[{"72", " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             RowBox[{"2", ",", "3"}], 
             RowBox[{"[", "2", "]"}]]}], 
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]]], "+", 
          RowBox[{"72", " ", 
           SubsuperscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            RowBox[{"2", ",", "4"}], 
            RowBox[{"[", "2", "]"}]]}], "+", 
          FractionBox[
           RowBox[{"72", " ", 
            SubsuperscriptBox[
             StyleBox["\[Eta]",
              FontWeight->Bold,
              FontColor->RGBColor[0.8, 0, 0]], 
             RowBox[{"3", ",", "2"}], 
             RowBox[{"[", "2", "]"}]]}], 
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]]], "-", 
          RowBox[{"72", " ", 
           SubsuperscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            RowBox[{"3", ",", "3"}], 
            RowBox[{"[", "2", "]"}]]}]}], ")"}]}], "+", 
       InterpretationBox[
        SuperscriptBox[
         RowBox[{"O", "[", 
          FractionBox["1", "tran"], "]"}], "3"],
        SeriesData[$CellContext`tran, 
         DirectedInfinity[1], {}, 1, 3, 1],
        Editable->False]}],
      SeriesData[$CellContext`tran, 
       DirectedInfinity[1], {
       48 $CellContext`\[ScriptU]$[0][0]^(-4) $CellContext`\[Zeta]$[3][0] - 
        240 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[5][0] + 
        2520 $CellContext`\[Zeta]$[7][0], 
        Complex[0, 288] $CellContext`\[ScriptU]$[0][
           0]^(-3) $CellContext`\[Zeta]$[3][0] - 
        288 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[1][
         0] $CellContext`\[Zeta]$[3][0] + 864 $CellContext`\[Zeta]$[3][0]^2 + 
        Complex[0, -720] $CellContext`\[ScriptU]$[0][
           0]^(-1) $CellContext`\[Zeta]$[5][0] + 
        4320 $CellContext`\[Zeta]$[1][0] $CellContext`\[Zeta]$[5][0] + 
        Complex[0, -1440] $CellContext`\[Zeta]$[5][0] $CellContext`\[Eta]$[1][
          2] + 36 $CellContext`\[ScriptU]$[0][0]^(-4) $CellContext`\[Eta]$[2][
          2] + Complex[
          0, 432] $CellContext`\[ScriptU]$[0][0]^(-1) $CellContext`\[Zeta]$[
          3][0] $CellContext`\[Eta]$[2][2] - 
        72 $CellContext`\[ScriptU]$[0][0]^(-3) $CellContext`\[Eta]$[3][2] + 
        Complex[0, 144] $CellContext`\[Zeta]$[3][0] $CellContext`\[Eta]$[3][
          2] + 36 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Eta]$[4][
          2] - 72 $CellContext`\[ScriptU]$[0][0]^(-1) $CellContext`\[Eta]$[
         2, 3][2] + 72 $CellContext`\[Eta]$[2, 4][2] + 
        72 $CellContext`\[ScriptU]$[0][0]^(-1) $CellContext`\[Eta]$[3, 2][2] - 
        72 $CellContext`\[Eta]$[3, 3][2]}, 1, 3, 1],
      Editable->False], ")"}], " ", 
    SuperscriptBox["g", "8"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "9"],
    SeriesData[$CellContext`g, 0, {}, 0, 9, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {1, 0, 
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {(-12) $CellContext`\[Zeta]$[1][0]}, 1, 3, 1], 0, 
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {
     36 $CellContext`\[Zeta]$[3][0], 72 $CellContext`\[Zeta]$[1][0]^2}, 1, 3, 
     1], 0, 
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {
     24 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[3][0] - 
      240 $CellContext`\[Zeta]$[5][0], (-432) $CellContext`\[Zeta]$[1][
       0] $CellContext`\[Zeta]$[3][0]}, 1, 3, 1], 0, 
    SeriesData[$CellContext`tran, 
     DirectedInfinity[1], {
     48 $CellContext`\[ScriptU]$[0][0]^(-4) $CellContext`\[Zeta]$[3][0] - 
      240 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[5][0] + 
      2520 $CellContext`\[Zeta]$[7][0], 
      Complex[0, 288] $CellContext`\[ScriptU]$[0][
         0]^(-3) $CellContext`\[Zeta]$[3][0] - 
      288 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Zeta]$[1][
       0] $CellContext`\[Zeta]$[3][0] + 864 $CellContext`\[Zeta]$[3][0]^2 + 
      Complex[0, -720] $CellContext`\[ScriptU]$[0][
         0]^(-1) $CellContext`\[Zeta]$[5][0] + 
      4320 $CellContext`\[Zeta]$[1][0] $CellContext`\[Zeta]$[5][0] + 
      Complex[0, -1440] $CellContext`\[Zeta]$[5][0] $CellContext`\[Eta]$[1][
        2] + 36 $CellContext`\[ScriptU]$[0][0]^(-4) $CellContext`\[Eta]$[2][
        2] + Complex[
        0, 432] $CellContext`\[ScriptU]$[0][0]^(-1) $CellContext`\[Zeta]$[3][
        0] $CellContext`\[Eta]$[2][2] - 
      72 $CellContext`\[ScriptU]$[0][0]^(-3) $CellContext`\[Eta]$[3][2] + 
      Complex[0, 144] $CellContext`\[Zeta]$[3][0] $CellContext`\[Eta]$[3][2] + 
      36 $CellContext`\[ScriptU]$[0][0]^(-2) $CellContext`\[Eta]$[4][2] - 
      72 $CellContext`\[ScriptU]$[0][0]^(-1) $CellContext`\[Eta]$[2, 3][2] + 
      72 $CellContext`\[Eta]$[2, 4][2] + 
      72 $CellContext`\[ScriptU]$[0][0]^(-1) $CellContext`\[Eta]$[3, 2][2] - 
      72 $CellContext`\[Eta]$[3, 3][2]}, 1, 3, 1]}, 0, 9, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.567271374232872*^9, 3.567271404296791*^9}, {
   3.567271437833434*^9, 3.5672714762315397`*^9}, 3.5672715500069513`*^9}]
}, Open  ]],

Cell[TextData[{
 "Only from here we can in principle get ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Zeta]", 
    RowBox[{"1", ",", "2", ",", "8"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " from asymptotic solution."
}], "Text",
 CellChangeTimes->{{3.5672687531077642`*^9, 3.5672687683158493`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["q", "2"], "/.", "largevolume"}]], "Input",
 CellChangeTimes->{{3.567268768997141*^9, 3.56726877325333*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", 
  RowBox[{"(", 
   RowBox[{"u", "+", 
    FractionBox[
     RowBox[{"4", " ", "g"}], 
     RowBox[{
      SubscriptBox["\[Gamma]", "0"], " ", 
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{3.567268773488879*^9}]
}, Open  ]],

Cell[TextData[{
 "From ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " we cannot get enough \[Eta]-functions. Therefore for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " we can take only leading tarnscendentality. Now situation is quite clear."
}], "Text",
 CellChangeTimes->{{3.567268777387218*^9, 3.567268813682571*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      FractionBox[
       SubsuperscriptBox["q", "2", "2"], "tran"], ",", 
      RowBox[{
       SubscriptBox["q", "2"], 
       SqrtBox["2"]}], ",", "tran"}], "}"}], "/.", "leadingtran"}], "/.", 
   "subtotran"}], ")"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["tran", 
     RowBox[{"9", " ", 
      SuperscriptBox["g", "2"], " ", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["2"], " ", "tran"}], 
     RowBox[{"3", " ", "g", " ", 
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]]}]]}], ",", "tran"}], "}"}]], "Output",\

 CellChangeTimes->{3.567270559827045*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Usqtran"], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox["g", "2"]}], 
    RowBox[{
     SuperscriptBox["tran", "2"], " ", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], "-", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", 
    SuperscriptBox["g", "3"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"2", " ", 
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], " ", 
       SubsuperscriptBox[
        OverscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         StyleBox["^",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]]], "1", 
        RowBox[{"[", "2", "]"}]]}]}], ")"}]}], 
   RowBox[{
    SuperscriptBox["tran", "3"], " ", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "3"]}]], "+", 
  FractionBox[
   RowBox[{
    FractionBox[
     RowBox[{"30", " ", 
      SuperscriptBox["g", "4"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
    FractionBox[
     RowBox[{"18", " ", 
      SuperscriptBox["g", "4"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
    FractionBox[
     RowBox[{"144", " ", 
      SuperscriptBox["g", "4"], " ", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "1", 
       RowBox[{"[", "2", "]"}]]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "3"]], "+", 
    FractionBox[
     RowBox[{"144", " ", 
      SuperscriptBox["g", "4"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "1", 
         RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["x",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], 
   SuperscriptBox["tran", "4"]]}]], "Output",
 CellChangeTimes->{3.567273141536957*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["g", "2"]}], 
      RowBox[{
       SuperscriptBox["tran", "2"], " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], "-", 
    FractionBox[
     RowBox[{"12", " ", "\[ImaginaryI]", " ", 
      SuperscriptBox["g", "3"], " ", 
      RowBox[{"(", 
       RowBox[{"2", " ", 
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "1", 
         RowBox[{"[", "2", "]"}]]}], ")"}]}], 
     RowBox[{
      SuperscriptBox["tran", "3"], " ", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "3"]}]], "+", 
    FractionBox[
     RowBox[{"+", 
      FractionBox[
       RowBox[{"144", " ", 
        SuperscriptBox["g", "4"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "1", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]]}], 
     SuperscriptBox["tran", "4"]]}], 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox["g", "2"]}], 
     RowBox[{
      SuperscriptBox["tran", "2"], " ", 
      SuperscriptBox[
       OverscriptBox[
        StyleBox["x",
         FontColor->RGBColor[0, 0.6, 0]], 
        StyleBox["^",
         FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}]], "//", 
  "Expand"}]], "Input",
 CellChangeTimes->{{3.567273363231847*^9, 3.5672733903291893`*^9}}],

Cell[BoxData[
 RowBox[{"1", "+", 
  FractionBox[
   RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", "2", "]"}]]}], "tran"], "-", 
  FractionBox[
   RowBox[{"72", " ", 
    SuperscriptBox["g", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["^",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], "1", 
       RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
   SuperscriptBox["tran", "2"]]}]], "Output",
 CellChangeTimes->{{3.5672733874621563`*^9, 3.567273390809821*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{
      FractionBox[
       RowBox[{"x", "+", 
        FractionBox["1", "x"]}], 
       RowBox[{"x", "-", 
        FractionBox["1", "x"]}]], "/.", 
      RowBox[{"x", "\[Rule]", "xh"}]}], 
     FractionBox["1", 
      SqrtBox[
       RowBox[{"1", "-", 
        FractionBox[
         RowBox[{"4", 
          SuperscriptBox["g", "2"]}], 
         SuperscriptBox["u", "2"]]}]]]], "/.", "subJuk"}], "//", 
   "FullSimplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"u", "\[Rule]", 
     RowBox[{"1", "-", 
      RowBox[{"\[ImaginaryI]", "/", "10"}]}]}]}], "/.", 
   RowBox[{"g", "\[Rule]", 
    RowBox[{"Random", "[", "]"}]}]}], "//", "N"}]}], "Input",
 CellChangeTimes->{{3.567316311319522*^9, 3.5673163463685017`*^9}}],

Cell[BoxData[
 RowBox[{"1.`", "\[VeryThinSpace]", "+", 
  RowBox[{"1.3877787807814457`*^-17", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.567316332732407*^9, 3.567316349794654*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["subtoBes1"], "Input",
 CellChangeTimes->{{3.567316885813673*^9, 3.5673168873163424`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", "2", "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "\[ImaginaryI]"}], ")"}], "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "1"]}], "]"}]}], 
     SubscriptBox["z", "1"]]}], ",", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "a_", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{"a", " ", 
      SuperscriptBox["\[ImaginaryI]", "a"], " ", 
      RowBox[{
       SubscriptBox["J", "a"], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "3"]}], "]"}]}], 
     SubscriptBox["z", "3"]]}], ",", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], 
     RowBox[{"a_", "/;", 
      RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
    FractionBox[
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["z", "2"]], 
      RowBox[{
       SubscriptBox["J", 
        RowBox[{"-", "a"}]], "[", 
       RowBox[{"2", " ", "g", " ", 
        SubscriptBox["z", "2"]}], "]"}]}], 
     SuperscriptBox["\[ImaginaryI]", "a"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5673168878845043`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"mU2", "=", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["g", "2"]}], 
      RowBox[{
       SuperscriptBox["tran", "2"], " ", 
       SuperscriptBox[
        OverscriptBox[
         StyleBox["x",
          FontColor->RGBColor[0, 0.6, 0]], 
         StyleBox["^",
          FontColor->RGBColor[0, 0.6, 0]]], "2"]}]]}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      FractionBox[
       RowBox[{"12", " ", "\[ImaginaryI]", " ", "g", " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], 
          StyleBox["^",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]]], "1", 
         RowBox[{"[", "2", "]"}]]}], "tran"], "-", 
      FractionBox[
       RowBox[{"72", " ", 
        SuperscriptBox["g", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox[
           OverscriptBox[
            StyleBox["\[Eta]",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]], 
            StyleBox["^",
             FontWeight->Bold,
             FontColor->RGBColor[0.8, 0, 0]]], "1", 
           RowBox[{"[", "2", "]"}]], ")"}], "2"]}], 
       SuperscriptBox["tran", "2"]]}], ")"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5673087877109947`*^9, 3.567308794630343*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        SubsuperscriptBox["q", "2", "2"], "tran"], ",", 
       RowBox[{
        SubscriptBox["q", "2"], 
        SqrtBox["2"]}], ",", "tran"}], "}"}], "/.", "leadingtran"}], "/.", 
    "subtotran"}], ")"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"mU2", " ", "%"}], ",", 
      RowBox[{"{", 
       RowBox[{"tran", ",", "\[Infinity]", ",", "3"}], "}"}]}], "]"}], "//", 
    "Normal"}], "//", "Expand"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", "/.", 
     RowBox[{"\[Eta]$h", "\[Rule]", "\[Eta]$"}]}], "//", "FuntoMHZbasis"}], "//",
    "Expand"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"81", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"%", "/.", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"x$h", "[", "0", "]"}], 
           RowBox[{"a_", "/;", 
            RowBox[{"a", "<", "0"}]}]], 
          RowBox[{
           RowBox[{"\[Eta]$", "[", "b__", "]"}], "[", "2", "]"}]}], 
         "\[RuleDelayed]", 
         RowBox[{
          RowBox[{"\[Eta]$h", "[", 
           RowBox[{
            RowBox[{"-", "a"}], ",", "b"}], "]"}], "[", "2", "]"}]}]}], "/.", 
       
       RowBox[{
        SuperscriptBox[
         RowBox[{"x$h", "[", "0", "]"}], 
         RowBox[{"a_", "/;", 
          RowBox[{"a", "<", "0"}]}]], "\[RuleDelayed]", 
        RowBox[{
         RowBox[{"\[Eta]$h", "[", 
          RowBox[{"-", "a"}], "]"}], "[", "2", "]"}]}]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"%", "/.", "mconj"}], "//", "Reverse"}], ")"}]}]}], "/.", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Eta]$b", "[", "b__", "]"}], "[", 
     RowBox[{"-", "2"}], "]"}], "\[RuleDelayed]", " ", 
    RowBox[{
     RowBox[{"\[Eta]$bh", "[", "b", "]"}], "[", 
     RowBox[{"-", "2"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"SeriesCoefficient", "[", 
        RowBox[{"%", ",", 
         RowBox[{"{", 
          RowBox[{"tran", ",", "\[Infinity]", ",", "4"}], "}"}]}], "]"}], "/.", 
       RowBox[{"\[Eta]$h", "\[Rule]", "\[Eta]$"}]}], "/.", 
      RowBox[{"\[Eta]$bh", "\[Rule]", "\[Eta]$b"}]}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"canlist", ",", "xh"}], "}"}], "//", "Flatten"}]}], "]"}], "//",
    "Expand"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"tmp", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Coefficient", "[", 
     RowBox[{
      RowBox[{"%", "/.", 
       RowBox[{
        RowBox[{"xxx", ":", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"\[Eta]$", "|", "\[Eta]$b"}], ")"}], "[", "a__", "]"}], 
           "[", "_", "]"}], ")"}]}], "\[RuleDelayed]", 
        RowBox[{
         SuperscriptBox["AA", 
          RowBox[{"Length", "[", 
           RowBox[{"{", "a", "}"}], "]"}]], "xxx"}]}]}], ",", "AA", ",", 
      "3"}], "]"}], "/.", 
    RowBox[{"\[Eta]$", "\[Rule]", "\[Eta]$h"}]}], "/.", 
   RowBox[{"\[Eta]$b", "\[Rule]", "\[Eta]$bh"}]}]}]}], "Input",
 CellChangeTimes->{{3.5672688168421297`*^9, 3.567269142419071*^9}, {
  3.567269180817019*^9, 3.567269351984768*^9}, {3.56726938520829*^9, 
  3.567269385786972*^9}, {3.567269513404461*^9, 3.5672695202634697`*^9}, {
  3.567270026623042*^9, 3.567270035659072*^9}, {3.567308819721652*^9, 
  3.56730883094657*^9}, {3.567308864756398*^9, 3.567308960492599*^9}, {
  3.56730903321659*^9, 3.567309128889312*^9}, {3.5673095023722477`*^9, 
  3.567309503723892*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"1", ",", "1"}], 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "2", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
  FractionBox[
   RowBox[{"10368", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"1", ",", "1"}], 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "3", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "3"]], "+", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"1", ",", "1"}], 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "4", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "-", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"2", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "4"]], "+", 
  FractionBox[
   RowBox[{"10368", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"3", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"4", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"2", ",", "1", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
  FractionBox[
   RowBox[{"10368", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"3", ",", "1", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "3"]], "+", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"4", ",", "1", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]]}]], "Output",
 CellChangeTimes->{{3.567268838672681*^9, 3.567268962586295*^9}, 
   3.567269000788507*^9, {3.567269035989581*^9, 3.5672691429187803`*^9}, {
   3.5672692555908947`*^9, 3.5672693010438843`*^9}, {3.567269332715391*^9, 
   3.5672693525050583`*^9}, 3.567269386605991*^9, {3.567269513789041*^9, 
   3.567269520689288*^9}, 3.5672700362511578`*^9, {3.567308825811255*^9, 
   3.567308831401917*^9}, 3.567308867802883*^9, {3.56730892068361*^9, 
   3.567308960898478*^9}, {3.567309093839316*^9, 3.56730912942356*^9}, 
   3.567309504265704*^9}]
}, Open  ]],

Cell[TextData[{
 "This finally the expression that in principle can produce ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Zeta]", 
    RowBox[{"1", ",", "2", ",", "8"}]], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.567269390334545*^9, 3.567269400673193*^9}}],

Cell["\<\
Let us check if it gives correct answer (or expected answer)\
\>", "Text",
 CellChangeTimes->{{3.5672694034043093`*^9, 3.567269413524873*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"tmp", "/.", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]$h", "[", "a__", "]"}], "[", "2", "]"}], 
     "\[RuleDelayed]", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[CapitalLambda]", "=", "16"}], "}"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "@", 
             RowBox[{"Normal", "@", 
              RowBox[{"Series", "[", 
               RowBox[{
                RowBox[{"Product", "[", 
                 RowBox[{
                  RowBox[{"Normal", "@", 
                   RowBox[{"Series", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    FractionBox["1", 
                    SuperscriptBox["xh", 
                    RowBox[{
                    RowBox[{"{", "a", "}"}], "[", 
                    RowBox[{"[", "i", "]"}], "]"}]]], "/.", "subJuk"}], "/.", 
                    
                    RowBox[{"u", "\[Rule]", 
                    RowBox[{"uu", "[", "i", "]"}]}]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}],
                     "]"}]}], ",", 
                  RowBox[{"{", 
                   RowBox[{"i", ",", 
                    RowBox[{"Length", "[", 
                    RowBox[{"{", "a", "}"}], "]"}]}], "}"}]}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], 
               "]"}]}]}], "/.", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"uu", "[", "3", "]"}], 
              RowBox[{"pow_", "/;", 
               RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
             RowBox[{
              RowBox[{"\[Eta]$", "[", 
               RowBox[{"-", "pow"}], "]"}], "[", "2", "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"uu", "[", "2", "]"}], 
              RowBox[{"pow_", "/;", 
               RowBox[{"pow", "<", "0"}]}]], 
             RowBox[{
              RowBox[{"\[Eta]$", "[", "pow2_", "]"}], "[", "2", "]"}]}], 
            "\[RuleDelayed]", 
            RowBox[{
             RowBox[{"\[Eta]$", "[", 
              RowBox[{
               RowBox[{"-", "pow"}], ",", "pow2"}], "]"}], "[", "2", 
             "]"}]}]}], "/.", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"uu", "[", "2", "]"}], 
            RowBox[{"pow_", "/;", 
             RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
           RowBox[{
            RowBox[{"\[Eta]$", "[", 
             RowBox[{"-", "pow"}], "]"}], "[", "2", "]"}]}]}], "/.", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"uu", "[", "1", "]"}], 
            RowBox[{"pow_", "/;", 
             RowBox[{"pow", "<", "0"}]}]], 
           RowBox[{
            RowBox[{"\[Eta]$", "[", "pow2__", "]"}], "[", "2", "]"}]}], 
          "\[RuleDelayed]", 
          RowBox[{
           RowBox[{"\[Eta]$", "[", 
            RowBox[{
             RowBox[{"-", "pow"}], ",", "pow2"}], "]"}], "[", "2", "]"}]}]}], 
        "/.", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"uu", "[", "1", "]"}], 
          RowBox[{"pow_", "/;", 
           RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
         RowBox[{
          RowBox[{"\[Eta]$", "[", 
           RowBox[{"-", "pow"}], "]"}], "[", "2", "]"}]}]}]}], "]"}]}]}], "/.",
    "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[Eta]$bh", "|", "\[Eta]$hb"}], ")"}], "[", "a__", "]"}], "[", 
     RowBox[{"-", "2"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"With", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalLambda]", "=", "16"}], "}"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Expand", "@", 
            RowBox[{"Normal", "@", 
             RowBox[{"Series", "[", 
              RowBox[{
               RowBox[{"Product", "[", 
                RowBox[{
                 RowBox[{"Normal", "@", 
                  RowBox[{"Series", "[", 
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    FractionBox["1", 
                    SuperscriptBox["xh", 
                    RowBox[{
                    RowBox[{"{", "a", "}"}], "[", 
                    RowBox[{"[", "i", "]"}], "]"}]]], "/.", "subJuk"}], "/.", 
                    
                    RowBox[{"u", "\[Rule]", 
                    RowBox[{"uu", "[", "i", "]"}]}]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}],
                    "]"}]}], ",", 
                 RowBox[{"{", 
                  RowBox[{"i", ",", 
                   RowBox[{"Length", "[", 
                    RowBox[{"{", "a", "}"}], "]"}]}], "}"}]}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"g", ",", "0", ",", "\[CapitalLambda]"}], "}"}]}], 
              "]"}]}]}], "/.", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"uu", "[", "3", "]"}], 
             RowBox[{"pow_", "/;", 
              RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
            RowBox[{
             RowBox[{"\[Eta]$b", "[", 
              RowBox[{"-", "pow"}], "]"}], "[", 
             RowBox[{"-", "2"}], "]"}]}]}], "/.", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"uu", "[", "2", "]"}], 
             RowBox[{"pow_", "/;", 
              RowBox[{"pow", "<", "0"}]}]], 
            RowBox[{
             RowBox[{"\[Eta]$b", "[", "pow2_", "]"}], "[", 
             RowBox[{"-", "2"}], "]"}]}], "\[RuleDelayed]", 
           RowBox[{
            RowBox[{"\[Eta]$b", "[", 
             RowBox[{
              RowBox[{"-", "pow"}], ",", "pow2"}], "]"}], "[", 
            RowBox[{"-", "2"}], "]"}]}]}], "/.", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"uu", "[", "2", "]"}], 
           RowBox[{"pow_", "/;", 
            RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
          RowBox[{
           RowBox[{"\[Eta]$b", "[", 
            RowBox[{"-", "pow"}], "]"}], "[", 
           RowBox[{"-", "2"}], "]"}]}]}], "/.", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"uu", "[", "1", "]"}], 
           RowBox[{"pow_", "/;", 
            RowBox[{"pow", "<", "0"}]}]], 
          RowBox[{
           RowBox[{"\[Eta]$b", "[", "pow2__", "]"}], "[", 
           RowBox[{"-", "2"}], "]"}]}], "\[RuleDelayed]", 
         RowBox[{
          RowBox[{"\[Eta]$b", "[", 
           RowBox[{
            RowBox[{"-", "pow"}], ",", "pow2"}], "]"}], "[", 
          RowBox[{"-", "2"}], "]"}]}]}], "/.", "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"uu", "[", "1", "]"}], 
         RowBox[{"pow_", "/;", 
          RowBox[{"pow", "<", "0"}]}]], "\[RuleDelayed]", 
        RowBox[{
         RowBox[{"\[Eta]$b", "[", 
          RowBox[{"-", "pow"}], "]"}], "[", 
         RowBox[{"-", "2"}], "]"}]}]}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{"%", ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SeriesCoefficient", "[", 
    RowBox[{
     RowBox[{
      FractionBox["%", 
       SqrtBox[
        RowBox[{"1", "-", 
         FractionBox[
          RowBox[{"4", 
           SuperscriptBox["g", "2"]}], 
          SuperscriptBox["u", "2"]]}]]], "/.", "subJuk"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "0", ",", "16"}], "}"}]}], "]"}], "//", "Normal"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "\[ImaginaryI]", " ", 
     RowBox[{"UpIntegrate", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         FractionBox["%", 
          RowBox[{
           RowBox[{"-", "2"}], "\[Pi]", " ", "\[ImaginaryI]"}]], "/.", 
         RowBox[{"Shift", "[", 
          RowBox[{
           RowBox[{"-", "2"}], "\[Epsilon]"}], "]"}]}], "/.", 
        RowBox[{"u", "\[Rule]", 
         RowBox[{"u", "-", 
          RowBox[{"\[ImaginaryI]", " ", "\[Epsilon]"}]}]}]}], ",", "u"}], 
      "]"}]}], "//.", "subZetaReduce"}], ",", "canlist"}], "]"}]}], "Input",
 CellChangeTimes->{{3.567269676886179*^9, 3.567270011069875*^9}, {
   3.567270046959412*^9, 3.567270188739163*^9}, {3.567270221648361*^9, 
   3.567270269726235*^9}, {3.567270914203281*^9, 3.567270920206429*^9}, {
   3.567309153990987*^9, 3.567309264237447*^9}, {3.567309300271606*^9, 
   3.56730930665694*^9}, 3.567309366811282*^9, {3.567309407385416*^9, 
   3.567309408227862*^9}, {3.567309446851595*^9, 3.5673094767977753`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"27869184", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "5"]}], "+", 
  RowBox[{
   FractionBox["1316072448", "175"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "4"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{
   FractionBox["461251584", "35"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"21482496", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"40435200", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5", "2"]}], "+", 
  RowBox[{"42488064", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"103348224", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"141087744", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"227836800", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"567333072", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"559872", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "2", ",", "8"}]]}]}]], "Output",
 CellChangeTimes->{
  3.567269947291853*^9, {3.567269987301601*^9, 3.567270011643249*^9}, {
   3.5672700484117537`*^9, 3.567270189211082*^9}, {3.5672702476312027`*^9, 
   3.5672702702794228`*^9}, 3.567270921704664*^9, 3.567309195054737*^9, {
   3.56730924384702*^9, 3.567309268745737*^9}, 3.567309318136202*^9, 
   3.567309369404879*^9, 3.56730941435427*^9, {3.567309448662787*^9, 
   3.56730951275012*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"32198114304", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "1"], " ", 
    SubsuperscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "2", "5"]}], "1925"], "+", 
  RowBox[{
   FractionBox["1091584512", "175"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "4"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  RowBox[{
   FractionBox["269858304", "35"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"20901888", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"29973888", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5", "2"]}], "+", 
  RowBox[{
   FractionBox["83172096", "5"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"83690496", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "-", 
  RowBox[{"129475584", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "+", 
  RowBox[{"65702016", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "9"]}], "-", 
  RowBox[{"188645760", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "11"]}], "+", 
  RowBox[{"1202688", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "6"}]]}], "-", 
  RowBox[{"1016064", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "8"}]]}], "+", 
  RowBox[{"1866240", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "2", ",", "8"}]]}]}]], "Input",
 CellChangeTimes->{{3.5672709417577*^9, 3.5672709432688723`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"1866240", "/", "559872"}]], "Input",
 CellChangeTimes->{{3.567270329719253*^9, 3.567270369123376*^9}, 
   3.567270950392624*^9, 3.5672710400763493`*^9}],

Cell[BoxData[
 FractionBox["10", "3"]], "Output",
 CellChangeTimes->{{3.567270332250846*^9, 3.5672703696034603`*^9}, 
   3.567270951335781*^9, 3.5672710405240717`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"19008", " ", "36", " ", "2"}]], "Input",
 CellChangeTimes->{{3.567270968632719*^9, 3.567270975670203*^9}}],

Cell[BoxData["1368576"], "Output",
 CellChangeTimes->{{3.56727097263723*^9, 3.5672709764112873`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["tmp"], "Input",
 CellChangeTimes->{{3.567270119225613*^9, 3.567270119345149*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"5184", " ", 
     SuperscriptBox["g", "4"], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       OverscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        StyleBox["_",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], "1", 
      RowBox[{"[", 
       RowBox[{"-", "2"}], "]"}]], " ", 
     SubsuperscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["^",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      RowBox[{"2", ",", "1"}], 
      RowBox[{"[", "2", "]"}]]}], 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "4"]]}], "+", 
  FractionBox[
   RowBox[{"10368", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"3", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "3"]], "-", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      OverscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       StyleBox["_",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], "1", 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"4", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]], "+", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"2", ",", "1", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "4"]], "-", 
  FractionBox[
   RowBox[{"10368", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"3", ",", "1", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "3"]], "+", 
  FractionBox[
   RowBox[{"5184", " ", 
    SuperscriptBox["g", "4"], " ", 
    SubsuperscriptBox[
     OverscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], 
      StyleBox["^",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]]], 
     RowBox[{"4", ",", "1", ",", "1"}], 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "2"]]}]], "Output",
 CellChangeTimes->{3.567270119952695*^9}]
}, Open  ]],

Cell["", "Text"],

Cell["", "Text"]
}, Closed]]
},
WindowSize->{1280, 687},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True,
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 2., 2. Inherited],
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3565378871-5147713"->{
  Cell[1091835, 33564, 124, 2, 312, "Print",
   CellTags->"Info3565378871-5147713"],
  Cell[1091962, 33568, 2226, 63, 296, "Print",
   CellTags->"Info3565378871-5147713"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3565378871-5147713", 1580680, 48721}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 136, 3, 52, "Text"],
Cell[696, 25, 177, 3, 54, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[898, 32, 560, 7, 134, "Section"],
Cell[CellGroupData[{
Cell[1483, 43, 92, 1, 136, "Subsection"],
Cell[1578, 46, 586, 16, 172, "Text"],
Cell[2167, 64, 513, 11, 232, "Text"],
Cell[2683, 77, 477, 15, 288, "Input"],
Cell[3163, 94, 163, 3, 104, "Text"],
Cell[3329, 99, 743, 25, 142, "Input"],
Cell[4075, 126, 386, 7, 224, "Text"],
Cell[CellGroupData[{
Cell[4486, 137, 104, 1, 152, "Input"],
Cell[4593, 140, 2442, 78, 318, "Output"]
}, Open  ]],
Cell[7050, 221, 139, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[7214, 226, 6325, 178, 1096, "Input"],
Cell[13542, 406, 3158, 98, 384, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16737, 509, 99, 1, 152, "Input"],
Cell[16839, 512, 2457, 80, 172, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[19345, 598, 108, 1, 96, "Subsection"],
Cell[19456, 601, 178, 2, 104, "Text"],
Cell[CellGroupData[{
Cell[19659, 607, 3168, 97, 336, "Input"],
Cell[22830, 706, 1510, 49, 236, "Output"]
}, Open  ]],
Cell[24355, 758, 229, 4, 164, "Text"],
Cell[CellGroupData[{
Cell[24609, 766, 923, 26, 252, "Input"],
Cell[25535, 794, 215, 4, 152, "Output"],
Cell[25753, 800, 213, 4, 152, "Output"]
}, Open  ]],
Cell[25981, 807, 314, 5, 164, "Text"],
Cell[26298, 814, 649, 21, 200, "Input"],
Cell[CellGroupData[{
Cell[26972, 839, 206, 4, 108, "Input"],
Cell[27181, 845, 10321, 345, 1036, "Output"]
}, Open  ]],
Cell[37517, 1193, 152, 3, 104, "Text"],
Cell[37672, 1198, 151, 2, 104, "Text"],
Cell[CellGroupData[{
Cell[37848, 1204, 173, 4, 108, "Input"],
Cell[38024, 1210, 1211, 45, 188, "Output"]
}, Open  ]],
Cell[39250, 1258, 228, 5, 104, "Text"],
Cell[39481, 1265, 236, 8, 198, "Input"],
Cell[CellGroupData[{
Cell[39742, 1277, 879, 26, 328, "Input"],
Cell[40624, 1305, 1020, 34, 192, "Output"]
}, Open  ]],
Cell[41659, 1342, 521, 18, 140, "Text"],
Cell[42183, 1362, 115, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[42323, 1367, 381, 11, 198, "Input"],
Cell[42707, 1380, 1071, 35, 208, "Output"]
}, Open  ]],
Cell[43793, 1418, 121, 1, 104, "Text"],
Cell[43917, 1421, 179, 4, 104, "Text"],
Cell[CellGroupData[{
Cell[44121, 1429, 127, 2, 108, "Input"],
Cell[44251, 1433, 2682, 81, 208, "Output"]
}, Open  ]],
Cell[46948, 1517, 104, 1, 104, "Text"],
Cell[47055, 1520, 748, 26, 218, "Input"],
Cell[47806, 1548, 101, 1, 104, "Text"],
Cell[47910, 1551, 213, 7, 104, "Text"],
Cell[CellGroupData[{
Cell[48148, 1562, 266, 7, 108, "Input"],
Cell[48417, 1571, 260, 7, 170, "Output"]
}, Open  ]],
Cell[48692, 1581, 188, 6, 104, "Text"],
Cell[48883, 1589, 457, 13, 202, "Text"],
Cell[49343, 1604, 453, 14, 184, "Input"],
Cell[49799, 1620, 235, 7, 112, "Text"],
Cell[50037, 1629, 2131, 70, 388, "Input"],
Cell[52171, 1701, 203, 6, 104, "Text"],
Cell[52377, 1709, 3207, 106, 374, "Input"],
Cell[55587, 1817, 118, 1, 104, "Text"],
Cell[55708, 1820, 412, 14, 288, "Input"],
Cell[56123, 1836, 92, 1, 104, "Text"],
Cell[56218, 1839, 423, 15, 314, "Input"],
Cell[56644, 1856, 399, 12, 186, "Text"],
Cell[57046, 1870, 111, 1, 104, "Text"],
Cell[57160, 1873, 2039, 56, 400, "Input"],
Cell[CellGroupData[{
Cell[59224, 1933, 2627, 71, 430, "Input"],
Cell[61854, 2006, 3321, 93, 330, "Output"],
Cell[65178, 2101, 1396, 22, 230, "Output"]
}, Open  ]],
Cell[66589, 2126, 177, 4, 104, "Text"],
Cell[66769, 2132, 328, 6, 164, "Text"],
Cell[67100, 2140, 2925, 92, 488, "Input"],
Cell[70028, 2234, 742, 18, 212, "Text"],
Cell[70773, 2254, 1079, 33, 194, "Input"],
Cell[71855, 2289, 623, 18, 132, "Text"],
Cell[72481, 2309, 115, 1, 104, "Text"],
Cell[72599, 2312, 624, 19, 178, "Input"],
Cell[73226, 2333, 157, 3, 104, "Text"],
Cell[73386, 2338, 1393, 43, 320, "Input"],
Cell[74782, 2383, 128, 1, 104, "Text"],
Cell[74913, 2386, 5213, 158, 770, "Input"],
Cell[80129, 2546, 621, 18, 184, "Input"],
Cell[CellGroupData[{
Cell[80775, 2568, 1624, 45, 250, "Input"],
Cell[82402, 2615, 507, 9, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82946, 2629, 1385, 39, 360, "Input"],
Cell[84334, 2670, 278, 6, 108, "Output"]
}, Open  ]],
Cell[84627, 2679, 4974, 149, 696, "Input"],
Cell[CellGroupData[{
Cell[89626, 2832, 1700, 47, 250, "Input"],
Cell[91329, 2881, 534, 10, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91900, 2896, 1200, 33, 284, "Input"],
Cell[93103, 2931, 302, 7, 108, "Output"]
}, Open  ]],
Cell[93420, 2941, 250, 5, 104, "Text"],
Cell[93673, 2948, 1313, 42, 246, "Input"],
Cell[CellGroupData[{
Cell[95011, 2994, 4611, 112, 662, "Input"],
Cell[99625, 3108, 2104, 71, 230, "Output"]
}, Open  ]],
Cell[101744, 3182, 194, 4, 104, "Text"],
Cell[CellGroupData[{
Cell[101963, 3190, 3325, 90, 646, "Input"],
Cell[105291, 3282, 1981, 69, 230, "Output"]
}, Open  ]],
Cell[107287, 3354, 133, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[107445, 3359, 2024, 63, 280, "Input"],
Cell[109472, 3424, 859, 27, 206, "Output"]
}, Open  ]],
Cell[110346, 3454, 182, 4, 104, "Text"],
Cell[110531, 3460, 321, 11, 112, "Input"],
Cell[110855, 3473, 99, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[110979, 3478, 1552, 44, 514, "Input"],
Cell[112534, 3524, 429, 12, 118, "Output"]
}, Open  ]],
Cell[112978, 3539, 724, 21, 158, "Text"],
Cell[CellGroupData[{
Cell[113727, 3564, 1335, 40, 432, "Input"],
Cell[115065, 3606, 147, 3, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[115249, 3614, 990, 29, 324, "Input"],
Cell[116242, 3645, 481, 13, 118, "Output"]
}, Open  ]],
Cell[116738, 3661, 153, 3, 104, "Text"],
Cell[116894, 3666, 816, 23, 206, "Input"],
Cell[CellGroupData[{
Cell[117735, 3693, 1057, 31, 300, "Input"],
Cell[118795, 3726, 474, 13, 118, "Output"]
}, Open  ]],
Cell[119284, 3742, 187, 4, 104, "Text"],
Cell[CellGroupData[{
Cell[119496, 3750, 1039, 31, 292, "Input"],
Cell[120538, 3783, 3351, 98, 330, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[123926, 3886, 1173, 35, 292, "Input"],
Cell[125102, 3923, 289, 6, 108, "Output"]
}, Open  ]],
Cell[125406, 3932, 186, 4, 104, "Text"],
Cell[CellGroupData[{
Cell[125617, 3940, 1000, 30, 170, "Input"],
Cell[126620, 3972, 21621, 370, 858, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[148290, 4348, 129, 1, 96, "Subsection"],
Cell[148422, 4351, 107, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[148554, 4356, 202, 5, 108, "Input"],
Cell[148759, 4363, 363, 12, 158, "Output"]
}, Open  ]],
Cell[149137, 4378, 937, 29, 246, "Input"],
Cell[150077, 4409, 1012, 26, 178, "Input"],
Cell[151092, 4437, 97, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[151214, 4442, 1936, 54, 540, "Input"],
Cell[153153, 4498, 195, 4, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[153385, 4507, 815, 26, 176, "Input"],
Cell[154203, 4535, 636, 21, 166, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[154876, 4561, 2346, 65, 674, "Input"],
Cell[157225, 4628, 271, 5, 108, "Output"]
}, Open  ]],
Cell[157511, 4636, 142, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[157678, 4641, 103, 1, 108, "Input"],
Cell[157784, 4644, 1946, 65, 182, "Output"]
}, Open  ]],
Cell[159745, 4712, 604, 20, 178, "Input"],
Cell[160352, 4734, 719, 20, 184, "Input"],
Cell[CellGroupData[{
Cell[161096, 4758, 1882, 53, 536, "Input"],
Cell[162981, 4813, 295, 5, 108, "Output"]
}, Open  ]],
Cell[163291, 4821, 937, 29, 246, "Input"],
Cell[164231, 4852, 830, 27, 246, "Input"],
Cell[CellGroupData[{
Cell[165086, 4883, 2188, 63, 600, "Input"],
Cell[167277, 4948, 195, 4, 108, "Output"]
}, Open  ]],
Cell[167487, 4955, 137, 1, 104, "Text"],
Cell[CellGroupData[{
Cell[167649, 4960, 968, 27, 398, "Input"],
Cell[168620, 4989, 911, 29, 162, "Output"]
}, Open  ]],
Cell[169546, 5021, 142, 1, 104, "Text"],
Cell[169691, 5024, 1859, 60, 344, "Input"],
Cell[171553, 5086, 154, 2, 104, "Text"],
Cell[171710, 5090, 1153, 39, 220, "Input"],
Cell[172866, 5131, 2089, 60, 572, "Input"],
Cell[174958, 5193, 262, 5, 164, "Text"],
Cell[175223, 5200, 1026, 33, 184, "Input"],
Cell[176252, 5235, 838, 25, 184, "Input"],
Cell[177093, 5262, 236, 4, 164, "Text"],
Cell[177332, 5268, 858, 26, 184, "Input"],
Cell[178193, 5296, 151, 3, 104, "Text"],
Cell[178347, 5301, 328, 9, 126, "Input"],
Cell[178678, 5312, 218, 4, 104, "Text"],
Cell[178899, 5318, 714, 22, 178, "Input"],
Cell[179616, 5342, 4927, 150, 1026, "Input"],
Cell[184546, 5494, 138, 1, 104, "Text"],
Cell[184687, 5497, 2339, 75, 504, "Input"],
Cell[187029, 5574, 3330, 90, 854, "Input"],
Cell[190362, 5666, 3330, 91, 854, "Input"],
Cell[193695, 5759, 3502, 88, 1410, "Input"],
Cell[197200, 5849, 3553, 89, 1410, "Input"],
Cell[CellGroupData[{
Cell[200778, 5942, 801, 18, 170, "Input"],
Cell[201582, 5962, 338, 5, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201957, 5972, 564, 15, 170, "Input"],
Cell[202524, 5989, 166, 2, 108, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[202739, 5997, 497, 13, 118, "Subsection"],
Cell[203239, 6012, 379, 10, 172, "Text"],
Cell[CellGroupData[{
Cell[203643, 6026, 4234, 109, 818, "Input"],
Cell[207880, 6137, 14034, 445, 718, "Output"]
}, Open  ]],
Cell[221929, 6585, 480, 8, 224, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[222446, 6598, 115, 1, 96, "Subsection"],
Cell[CellGroupData[{
Cell[222586, 6603, 101, 1, 96, "Subsubsection"],
Cell[222690, 6606, 755, 26, 344, "Text"],
Cell[CellGroupData[{
Cell[223470, 6636, 1100, 29, 696, "Input"],
Cell[224573, 6667, 1196, 35, 448, "Output"],
Cell[225772, 6704, 259, 3, 216, "Output"]
}, Open  ]],
Cell[226046, 6710, 303, 9, 224, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[226386, 6724, 111, 1, 64, "Subsubsection"],
Cell[226500, 6727, 118, 1, 2496, "Text"],
Cell[226621, 6730, 268, 5, 2592, "Input"],
Cell[226892, 6737, 4085, 116, 12624, "Input"],
Cell[230980, 6855, 5281, 156, 16752, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[236298, 7016, 1475, 42, 232, "Subsubsection"],
Cell[237776, 7060, 376, 7, 328, "Text"],
Cell[CellGroupData[{
Cell[238177, 7071, 954, 32, 800, "Input"],
Cell[239134, 7105, 174, 3, 236, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[239345, 7113, 237, 8, 392, "Input"],
Cell[239585, 7123, 421, 13, 360, "Output"]
}, Open  ]],
Cell[240021, 7139, 2138, 71, 1032, "Input"],
Cell[242162, 7212, 185, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[242372, 7220, 1705, 49, 792, "Input"],
Cell[244080, 7271, 1680, 56, 424, "Output"]
}, Open  ]],
Cell[245775, 7330, 135, 1, 208, "Text"],
Cell[245913, 7333, 2759, 94, 496, "Input"],
Cell[248675, 7429, 1865, 59, 496, "Input"],
Cell[250543, 7490, 681, 22, 368, "Input"],
Cell[251227, 7514, 1219, 40, 392, "Input"],
Cell[252449, 7556, 1636, 50, 600, "Input"],
Cell[254088, 7608, 2783, 78, 816, "Input"],
Cell[256874, 7688, 134, 1, 208, "Text"],
Cell[257011, 7691, 838, 27, 380, "Input"],
Cell[257852, 7720, 477, 12, 328, "Text"],
Cell[258332, 7734, 626, 17, 252, "Input"],
Cell[258961, 7753, 380, 10, 328, "Text"],
Cell[CellGroupData[{
Cell[259366, 7767, 2799, 81, 1228, "Input"],
Cell[262168, 7850, 2979, 82, 888, "Output"]
}, Closed]],
Cell[265162, 7935, 217, 4, 172, "Text"],
Cell[CellGroupData[{
Cell[265404, 7943, 4213, 116, 1724, "Input"],
Cell[269620, 8061, 2215, 67, 592, "Output"]
}, Closed]],
Cell[271850, 8131, 222, 4, 172, "Text"],
Cell[272075, 8137, 860, 30, 572, "Input"],
Cell[CellGroupData[{
Cell[272960, 8171, 1931, 55, 1356, "Input"],
Cell[274894, 8228, 126, 2, 160, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[275057, 8235, 647, 18, 344, "Input"],
Cell[275707, 8255, 641, 20, 264, "Output"]
}, Open  ]],
Cell[276363, 8278, 283, 5, 208, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[276683, 8288, 2497, 59, 132, "Subsubsection"],
Cell[279183, 8349, 196, 4, 208, "Text"],
Cell[279382, 8355, 2608, 80, 380, "Input"],
Cell[281993, 8437, 157, 3, 208, "Text"],
Cell[282153, 8442, 1038, 30, 360, "Input"],
Cell[283194, 8474, 1437, 43, 396, "Input"],
Cell[284634, 8519, 1566, 47, 536, "Input"],
Cell[286203, 8568, 179, 2, 208, "Text"],
Cell[286385, 8572, 250, 8, 224, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[286672, 8585, 680, 22, 134, "Subsubsection"],
Cell[CellGroupData[{
Cell[287377, 8611, 1267, 35, 436, "Input"],
Cell[288647, 8648, 1619, 51, 180, "Output"],
Cell[290269, 8701, 2503, 80, 148, "Output"],
Cell[292775, 8783, 1618, 48, 172, "Output"]
}, Open  ]],
Cell[294408, 8834, 144, 1, 104, "Text"],
Cell[294555, 8837, 812, 19, 404, "Input"],
Cell[CellGroupData[{
Cell[295392, 8860, 1993, 66, 192, "Input"],
Cell[297388, 8928, 2024, 67, 176, "Output"]
}, Open  ]],
Cell[299427, 8998, 2528, 74, 584, "Input"],
Cell[CellGroupData[{
Cell[301980, 9076, 1673, 47, 478, "Input"],
Cell[303656, 9125, 1546, 49, 196, "Output"],
Cell[305205, 9176, 292, 4, 108, "Output"]
}, Open  ]],
Cell[305512, 9183, 186, 4, 104, "Text"],
Cell[CellGroupData[{
Cell[305723, 9191, 2401, 78, 152, "Input"],
Cell[308127, 9271, 2286, 77, 148, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[310462, 9354, 750, 26, 156, "Subsubsection"],
Cell[311215, 9382, 115, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[311355, 9387, 354, 11, 396, "Input"],
Cell[311712, 9400, 273, 9, 360, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[312022, 9414, 925, 27, 464, "Input"],
Cell[312950, 9443, 1210, 38, 396, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[314197, 9486, 1863, 58, 736, "Input"],
Cell[316063, 9546, 877, 29, 396, "Output"],
Cell[316943, 9577, 257, 7, 248, "Output"]
}, Open  ]],
Cell[317215, 9587, 1771, 57, 444, "Input"],
Cell[318989, 9646, 681, 22, 368, "Input"],
Cell[319673, 9670, 1219, 40, 392, "Input"],
Cell[320895, 9712, 2183, 63, 376, "Input"],
Cell[323081, 9777, 3809, 115, 1544, "Input"],
Cell[326893, 9894, 2678, 77, 940, "Input"],
Cell[CellGroupData[{
Cell[329596, 9975, 3823, 110, 1568, "Input"],
Cell[333422, 10087, 2312, 76, 1048, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[335771, 10168, 1924, 52, 1144, "Input"],
Cell[337698, 10222, 150, 2, 160, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[337885, 10229, 585, 17, 344, "Input"],
Cell[338473, 10248, 761, 23, 396, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339271, 10276, 139, 2, 216, "Input"],
Cell[339413, 10280, 118, 1, 216, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[339580, 10287, 2208, 56, 162, "Subsubsection"],
Cell[341791, 10345, 163, 3, 208, "Text"],
Cell[341957, 10350, 1593, 51, 380, "Input"],
Cell[343553, 10403, 168, 3, 208, "Text"],
Cell[343724, 10408, 839, 25, 364, "Input"],
Cell[344566, 10435, 941, 29, 380, "Input"],
Cell[345510, 10466, 202, 4, 208, "Text"],
Cell[345715, 10472, 4685, 140, 1488, "Input"],
Cell[350403, 10614, 669, 17, 344, "Text"],
Cell[351075, 10633, 196, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[351296, 10641, 5494, 143, 1512, "Input"],
Cell[356793, 10786, 1078, 32, 360, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[357908, 10823, 1552, 43, 552, "Input"],
Cell[359463, 10868, 1414, 38, 380, "Output"]
}, Open  ]],
Cell[360892, 10909, 143, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[361060, 10914, 2174, 62, 700, "Input"],
Cell[363237, 10978, 2458, 72, 896, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[365732, 11055, 706, 21, 464, "Input"],
Cell[366441, 11078, 1534, 43, 332, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[368012, 11126, 7965, 197, 2712, "Input"],
Cell[375980, 11325, 1410, 42, 332, "Output"]
}, Open  ]],
Cell[377405, 11370, 111, 1, 208, "Text"],
Cell[377519, 11373, 236, 5, 208, "Text"],
Cell[377758, 11380, 266, 8, 216, "Text"],
Cell[CellGroupData[{
Cell[378049, 11392, 1916, 57, 1356, "Input"],
Cell[379968, 11451, 3600, 111, 304, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[383605, 11567, 159, 3, 216, "Input"],
Cell[383767, 11572, 3316, 108, 444, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[387120, 11685, 1128, 33, 592, "Input"],
Cell[388251, 11720, 4281, 143, 344, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[392569, 11868, 6715, 179, 924, "Input"],
Cell[399287, 12049, 1059, 34, 400, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[400383, 12088, 3399, 91, 1224, "Input"],
Cell[403785, 12181, 962, 31, 348, "Output"]
}, Open  ]],
Cell[404762, 12215, 117, 1, 208, "Text"],
Cell[404882, 12218, 169, 2, 208, "Text"],
Cell[405054, 12222, 770, 24, 540, "Input"],
Cell[CellGroupData[{
Cell[405849, 12250, 362, 10, 216, "Input"],
Cell[406214, 12262, 4008, 119, 456, "Output"]
}, Open  ]],
Cell[410237, 12384, 451, 11, 224, "Text"],
Cell[410691, 12397, 333, 9, 248, "Text"],
Cell[CellGroupData[{
Cell[411049, 12410, 1489, 46, 924, "Input"],
Cell[412541, 12458, 5569, 185, 348, "Output"]
}, Open  ]],
Cell[418125, 12646, 182, 2, 208, "Text"],
Cell[CellGroupData[{
Cell[418332, 12652, 2020, 63, 1288, "Input"],
Cell[420355, 12717, 2562, 86, 344, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[422954, 12808, 2636, 74, 1216, "Input"],
Cell[425593, 12884, 2992, 97, 264, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[428622, 12986, 191, 5, 216, "Input"],
Cell[428816, 12993, 213, 3, 216, "Output"]
}, Open  ]],
Cell[429044, 12999, 162, 2, 208, "Text"],
Cell[429209, 13003, 114, 3, 236, "Input"],
Cell[CellGroupData[{
Cell[429348, 13010, 776, 22, 348, "Input"],
Cell[430127, 13034, 4101, 120, 456, "Output"],
Cell[434231, 13156, 5793, 182, 428, "Output"]
}, Open  ]],
Cell[440039, 13341, 432, 13, 376, "Input"],
Cell[440474, 13356, 148, 2, 208, "Text"],
Cell[CellGroupData[{
Cell[440647, 13362, 408, 11, 216, "Input"],
Cell[441058, 13375, 165, 2, 216, "Output"]
}, Open  ]],
Cell[441238, 13380, 106, 1, 208, "Text"],
Cell[441347, 13383, 1783, 54, 668, "Input"],
Cell[443133, 13439, 328, 6, 328, "Text"],
Cell[CellGroupData[{
Cell[443486, 13449, 2108, 62, 636, "Input"],
Cell[445597, 13513, 6824, 202, 1696, "Output"]
}, Open  ]],
Cell[452436, 13718, 2317, 64, 548, "Text"],
Cell[454756, 13784, 8618, 244, 2112, "Input"],
Cell[463377, 14030, 135, 3, 208, "Text"],
Cell[463515, 14035, 223, 4, 208, "Text"],
Cell[463741, 14041, 1641, 47, 392, "Input"],
Cell[CellGroupData[{
Cell[465407, 14092, 529, 15, 364, "Input"],
Cell[465939, 14109, 433, 7, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[466409, 14121, 3748, 102, 1888, "Input"],
Cell[470160, 14225, 479, 7, 216, "Output"]
}, Open  ]],
Cell[470654, 14235, 1310, 39, 400, "Text"],
Cell[CellGroupData[{
Cell[471989, 14278, 2933, 83, 1504, "Input"],
Cell[474925, 14363, 4679, 137, 1308, "Output"],
Cell[479607, 14502, 5194, 144, 1308, "Output"]
}, Open  ]],
Cell[484816, 14649, 104, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[484945, 14654, 1174, 35, 528, "Input"],
Cell[486122, 14691, 285, 5, 216, "Output"]
}, Open  ]],
Cell[486422, 14699, 205, 4, 208, "Text"],
Cell[486630, 14705, 566, 17, 380, "Input"],
Cell[487199, 14724, 168, 3, 208, "Text"],
Cell[487370, 14729, 1405, 44, 380, "Input"],
Cell[CellGroupData[{
Cell[488800, 14777, 30, 0, 216, "Input"],
Cell[488833, 14779, 5628, 180, 428, "Output"]
}, Open  ]],
Cell[494476, 14962, 276, 5, 208, "Text"],
Cell[CellGroupData[{
Cell[494777, 14971, 2446, 76, 1360, "Input"],
Cell[497226, 15049, 2564, 78, 424, "Output"],
Cell[499793, 15129, 1404, 44, 400, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[501234, 15178, 130, 2, 216, "Input"],
Cell[501367, 15182, 139, 2, 216, "Output"]
}, Open  ]],
Cell[501521, 15187, 235, 5, 208, "Text"],
Cell[CellGroupData[{
Cell[501781, 15196, 2146, 67, 1352, "Input"],
Cell[503930, 15265, 1277, 40, 300, "Output"],
Cell[505210, 15307, 1654, 51, 424, "Output"],
Cell[506867, 15360, 1378, 44, 400, "Output"]
}, Open  ]],
Cell[508260, 15407, 134, 1, 208, "Text"],
Cell[508397, 15410, 205, 4, 208, "Text"],
Cell[508605, 15416, 210, 6, 380, "Input"],
Cell[508818, 15424, 101, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[508944, 15429, 356, 10, 216, "Input"],
Cell[509303, 15441, 1702, 55, 304, "Output"]
}, Open  ]],
Cell[511020, 15499, 179, 4, 208, "Text"],
Cell[511202, 15505, 1926, 61, 380, "Input"],
Cell[513131, 15568, 119, 1, 208, "Text"],
Cell[513253, 15571, 5661, 171, 736, "Input"],
Cell[518917, 15744, 183, 4, 208, "Text"],
Cell[519103, 15750, 136, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[519264, 15755, 2501, 76, 396, "Input"],
Cell[521768, 15833, 2800, 91, 392, "Output"]
}, Open  ]],
Cell[524583, 15927, 722, 24, 304, "Text"],
Cell[CellGroupData[{
Cell[525330, 15955, 1882, 58, 416, "Input"],
Cell[527215, 16015, 1553, 49, 400, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[528805, 16069, 4714, 139, 800, "Input"],
Cell[533522, 16210, 3787, 112, 764, "Output"]
}, Open  ]],
Cell[537324, 16325, 98, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[537447, 16330, 1535, 37, 384, "Input"],
Cell[538985, 16369, 692, 12, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[539714, 16386, 3693, 100, 1008, "Input"],
Cell[543410, 16488, 595, 10, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[544042, 16503, 946, 26, 464, "Input"],
Cell[544991, 16531, 1753, 57, 396, "Output"]
}, Open  ]],
Cell[546759, 16591, 180, 2, 208, "Text"],
Cell[546942, 16595, 1886, 59, 300, "Input"],
Cell[548831, 16656, 996, 26, 360, "Input"],
Cell[CellGroupData[{
Cell[549852, 16686, 1717, 48, 712, "Input"],
Cell[551572, 16736, 650, 16, 368, "Output"],
Cell[552225, 16754, 482, 11, 300, "Output"]
}, Open  ]],
Cell[552722, 16768, 5310, 145, 1736, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[558069, 16918, 2298, 55, 280, "Subsubsection"],
Cell[560370, 16975, 159, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[560554, 16982, 98, 1, 216, "Input"],
Cell[560655, 16985, 942, 25, 336, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[561634, 17015, 1338, 39, 652, "Input"],
Cell[562975, 17056, 1837, 55, 380, "Output"]
}, Open  ]],
Cell[564827, 17114, 239, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[565091, 17122, 2575, 63, 1180, "Input"],
Cell[567669, 17187, 884, 25, 264, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[568590, 17217, 1561, 45, 640, "Input"],
Cell[570154, 17264, 479, 13, 300, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[570670, 17282, 251, 6, 216, "Input"],
Cell[570924, 17290, 5092, 164, 304, "Output"]
}, Open  ]],
Cell[576031, 17457, 5681, 148, 1888, "Input"],
Cell[CellGroupData[{
Cell[581737, 17609, 104, 1, 216, "Input"],
Cell[581844, 17612, 3599, 109, 764, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[585480, 17726, 102, 1, 216, "Input"],
Cell[585585, 17729, 7709, 229, 1984, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[593343, 17964, 610, 20, 360, "Subsubsection"],
Cell[593956, 17986, 162, 2, 208, "Text"],
Cell[CellGroupData[{
Cell[594143, 17992, 1443, 39, 972, "Input"],
Cell[595589, 18033, 532, 16, 444, "Output"]
}, Open  ]],
Cell[596136, 18052, 185, 4, 208, "Text"],
Cell[596324, 18058, 339, 9, 264, "Input"],
Cell[596666, 18069, 271, 8, 248, "Text"],
Cell[CellGroupData[{
Cell[596962, 18081, 103, 1, 216, "Input"],
Cell[597068, 18084, 7703, 243, 572, "Output"]
}, Open  ]],
Cell[604786, 18330, 232, 7, 208, "Text"],
Cell[CellGroupData[{
Cell[605043, 18341, 1198, 37, 476, "Input"],
Cell[606244, 18380, 6376, 194, 416, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[612657, 18579, 2047, 51, 956, "Input"],
Cell[614707, 18632, 5235, 151, 712, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[619979, 18788, 154, 3, 216, "Input"],
Cell[620136, 18793, 5436, 162, 720, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[625621, 18961, 1724, 52, 432, "Subsubsection"],
Cell[627348, 19015, 289, 5, 328, "Text"],
Cell[627640, 19022, 159, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[627824, 19029, 543, 16, 344, "Input"],
Cell[628370, 19047, 11954, 406, 1536, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[640361, 19458, 137, 2, 216, "Input"],
Cell[640501, 19462, 146, 2, 216, "Output"]
}, Open  ]],
Cell[640662, 19467, 632, 18, 552, "Input"],
Cell[641297, 19487, 3798, 94, 1564, "Input"],
Cell[CellGroupData[{
Cell[645120, 19585, 912, 25, 576, "Input"],
Cell[646035, 19612, 1004, 17, 548, "Message"],
Cell[647042, 19631, 978, 17, 548, "Message"],
Cell[648023, 19650, 479, 9, 288, "Message"],
Cell[648505, 19661, 479, 9, 288, "Message"],
Cell[648987, 19672, 484, 9, 304, "Message"],
Cell[649474, 19683, 541, 10, 168, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[650052, 19698, 171, 4, 216, "Input"],
Cell[650226, 19704, 1357, 47, 364, "Output"]
}, Open  ]],
Cell[651598, 19754, 137, 1, 208, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[651772, 19760, 310, 10, 152, "Subsubsection"],
Cell[652085, 19772, 732, 23, 264, "Input"],
Cell[652820, 19797, 197, 6, 208, "Text"],
Cell[CellGroupData[{
Cell[653042, 19807, 890, 28, 360, "Input"],
Cell[653935, 19837, 5126, 155, 304, "Output"]
}, Open  ]],
Cell[659076, 19995, 215, 7, 208, "Text"],
Cell[659294, 20004, 106, 1, 208, "Text"],
Cell[659403, 20007, 571, 18, 476, "Input"],
Cell[CellGroupData[{
Cell[659999, 20029, 585, 16, 232, "Input"],
Cell[660587, 20047, 5312, 171, 428, "Output"]
}, Open  ]],
Cell[665914, 20221, 1509, 51, 332, "Input"],
Cell[667426, 20274, 127, 2, 216, "Input"],
Cell[667556, 20278, 321, 6, 328, "Text"],
Cell[667880, 20286, 334, 11, 264, "Input"],
Cell[668217, 20299, 854, 25, 476, "Input"],
Cell[CellGroupData[{
Cell[669096, 20328, 854, 26, 476, "Input"],
Cell[669953, 20356, 818, 25, 268, "Output"]
}, Open  ]],
Cell[670786, 20384, 123, 1, 208, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[670946, 20390, 178, 2, 192, "Subsubsection"],
Cell[671127, 20394, 157, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[671309, 20401, 465, 14, 236, "Input"],
Cell[671777, 20417, 726, 22, 444, "Output"]
}, Open  ]],
Cell[672518, 20442, 188, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[672731, 20450, 3695, 102, 1084, "Input"],
Cell[676429, 20554, 4575, 135, 984, "Output"],
Cell[681007, 20691, 171, 3, 216, "Output"]
}, Open  ]],
Cell[681193, 20697, 199, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[681417, 20705, 1232, 33, 520, "Input"],
Cell[682652, 20740, 642, 16, 352, "Output"]
}, Open  ]],
Cell[683309, 20759, 243, 5, 208, "Text"],
Cell[683555, 20766, 127, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[683707, 20771, 644, 18, 384, "Input"],
Cell[684354, 20791, 184, 2, 216, "Output"]
}, Open  ]],
Cell[684553, 20796, 101, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[684679, 20801, 362, 11, 248, "Input"],
Cell[685044, 20814, 139, 2, 216, "Output"]
}, Open  ]],
Cell[685198, 20819, 127, 1, 208, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[685362, 20825, 110, 1, 192, "Subsubsection"],
Cell[685475, 20828, 209, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[685709, 20836, 1911, 54, 812, "Input"],
Cell[687623, 20892, 4006, 123, 1056, "Output"],
Cell[691632, 21017, 312, 4, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[691981, 21026, 3026, 81, 1224, "Input"],
Cell[695010, 21109, 2237, 61, 896, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[697284, 21175, 154, 3, 216, "Input"],
Cell[697441, 21180, 2635, 78, 720, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[700113, 21263, 1347, 37, 588, "Input"],
Cell[701463, 21302, 14646, 426, 2352, "Output"]
}, Open  ]],
Cell[716124, 21731, 166, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[716315, 21738, 319, 6, 216, "Input"],
Cell[716637, 21746, 219, 4, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[716893, 21755, 313, 9, 216, "Input"],
Cell[717209, 21766, 656, 15, 352, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[717902, 21786, 373, 11, 336, "Input"],
Cell[718278, 21799, 1721, 57, 332, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[720036, 21861, 1585, 40, 816, "Input"],
Cell[721624, 21903, 4034, 121, 940, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[725695, 22029, 273, 7, 216, "Input"],
Cell[725971, 22038, 7756, 215, 1736, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[733764, 22258, 5242, 157, 1896, "Input"],
Cell[739009, 22417, 4620, 144, 1048, "Output"],
Cell[743632, 22563, 363, 6, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[744032, 22574, 2192, 65, 760, "Input"],
Cell[746227, 22641, 241, 4, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[746505, 22650, 1206, 31, 540, "Input"],
Cell[747714, 22683, 3511, 119, 704, "Output"],
Cell[751228, 22804, 240, 3, 216, "Output"]
}, Open  ]],
Cell[751483, 22810, 182, 4, 208, "Text"],
Cell[751668, 22816, 171, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[751864, 22824, 489, 15, 284, "Input"],
Cell[752356, 22841, 392, 10, 364, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752785, 22856, 663, 21, 348, "Input"],
Cell[753451, 22879, 296, 9, 364, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[753784, 22893, 5854, 176, 1760, "Input"],
Cell[759641, 23071, 3655, 107, 1072, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[763333, 23183, 235, 6, 216, "Input"],
Cell[763571, 23191, 458, 12, 352, "Output"]
}, Open  ]],
Cell[764044, 23206, 132, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[764201, 23211, 2160, 67, 1032, "Input"],
Cell[766364, 23280, 1135, 36, 252, "Output"]
}, Open  ]],
Cell[767514, 23319, 107, 1, 208, "Text"],
Cell[767624, 23322, 4706, 136, 1920, "Input"],
Cell[772333, 23460, 974, 30, 276, "Text"],
Cell[773310, 23492, 225, 4, 208, "Text"],
Cell[773538, 23498, 3037, 86, 1080, "Input"],
Cell[CellGroupData[{
Cell[776600, 23588, 5171, 159, 1400, "Input"],
Cell[781774, 23749, 2069, 61, 380, "Output"],
Cell[783846, 23812, 1579, 45, 236, "Output"]
}, Open  ]],
Cell[785440, 23860, 102, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[785567, 23865, 1206, 35, 376, "Input"],
Cell[786776, 23902, 1345, 42, 384, "Output"]
}, Open  ]],
Cell[788136, 23947, 1072, 35, 640, "Input"],
Cell[789211, 23984, 101, 1, 208, "Text"],
Cell[789315, 23987, 2594, 83, 840, "Input"],
Cell[791912, 24072, 1918, 57, 668, "Input"],
Cell[793833, 24131, 234, 5, 208, "Text"],
Cell[CellGroupData[{
Cell[794092, 24140, 1312, 38, 844, "Input"],
Cell[795407, 24180, 1345, 40, 312, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[796789, 24225, 4057, 100, 1592, "Input"],
Cell[800849, 24327, 1149, 37, 312, "Output"]
}, Open  ]],
Cell[802013, 24367, 167, 3, 208, "Text"],
Cell[802183, 24372, 2528, 82, 840, "Input"],
Cell[804714, 24456, 126, 1, 208, "Text"],
Cell[804843, 24459, 1143, 39, 376, "Input"],
Cell[805989, 24500, 530, 11, 392, "Text"],
Cell[806522, 24513, 628, 19, 332, "Input"],
Cell[CellGroupData[{
Cell[807175, 24536, 168, 4, 216, "Input"],
Cell[807346, 24542, 149, 3, 216, "Output"]
}, Open  ]],
Cell[807510, 24548, 91, 1, 208, "Text"],
Cell[807604, 24551, 2594, 88, 720, "Input"],
Cell[810201, 24641, 508, 16, 308, "Text"],
Cell[810712, 24659, 4035, 134, 940, "Input"],
Cell[814750, 24795, 635, 21, 360, "Input"],
Cell[815388, 24818, 177, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[815590, 24826, 3054, 81, 1184, "Input"],
Cell[818647, 24909, 4236, 118, 1196, "Output"],
Cell[822886, 25029, 406, 7, 216, "Output"]
}, Open  ]],
Cell[823307, 25039, 166, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[823498, 25046, 148, 2, 216, "Input"],
Cell[823649, 25050, 2113, 65, 652, "Output"]
}, Open  ]],
Cell[825777, 25118, 3495, 107, 1400, "Input"],
Cell[829275, 25227, 230, 4, 208, "Text"],
Cell[829508, 25233, 232, 5, 208, "Text"],
Cell[CellGroupData[{
Cell[829765, 25242, 4697, 127, 1400, "Input"],
Cell[834465, 25371, 3396, 102, 576, "Output"]
}, Open  ]],
Cell[837876, 25476, 161, 3, 208, "Text"],
Cell[838040, 25481, 276, 8, 224, "Input"],
Cell[838319, 25491, 143, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[838487, 25498, 1239, 42, 512, "Input"],
Cell[839729, 25542, 1644, 47, 432, "Output"]
}, Open  ]],
Cell[841388, 25592, 277, 5, 208, "Text"],
Cell[CellGroupData[{
Cell[841690, 25601, 1620, 52, 640, "Input"],
Cell[843313, 25655, 1736, 58, 448, "Output"],
Cell[845052, 25715, 1227, 38, 264, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[846316, 25758, 1343, 45, 640, "Input"],
Cell[847662, 25805, 801, 19, 236, "Output"]
}, Open  ]],
Cell[848478, 25827, 201, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[848704, 25835, 515, 16, 264, "Input"],
Cell[849222, 25853, 330, 10, 264, "Output"]
}, Open  ]],
Cell[849567, 25866, 631, 17, 344, "Text"],
Cell[850201, 25885, 274, 8, 208, "Text"],
Cell[850478, 25895, 2706, 83, 1100, "Input"],
Cell[853187, 25980, 321, 8, 224, "Text"],
Cell[CellGroupData[{
Cell[853533, 25992, 285, 8, 232, "Input"],
Cell[853821, 26002, 118, 1, 216, "Output"]
}, Open  ]],
Cell[853954, 26006, 191, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[854170, 26014, 1477, 47, 1148, "Input"],
Cell[855650, 26063, 4653, 156, 968, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[860340, 26224, 672, 19, 416, "Input"],
Cell[861015, 26245, 2194, 75, 528, "Output"]
}, Open  ]],
Cell[863224, 26323, 168, 3, 208, "Text"],
Cell[863395, 26328, 137, 3, 208, "Text"],
Cell[863535, 26333, 1530, 46, 352, "Input"],
Cell[865068, 26381, 2906, 89, 1144, "Input"],
Cell[CellGroupData[{
Cell[867999, 26474, 1542, 50, 1168, "Input"],
Cell[869544, 26526, 3606, 122, 748, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[873187, 26653, 667, 19, 416, "Input"],
Cell[873857, 26674, 1584, 55, 312, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[875490, 26735, 126, 1, 192, "Subsubsection"],
Cell[875619, 26738, 127, 1, 208, "Text"],
Cell[875749, 26741, 962, 29, 364, "Input"],
Cell[CellGroupData[{
Cell[876736, 26774, 392, 12, 336, "Input"],
Cell[877131, 26788, 2487, 78, 416, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[879655, 26871, 1169, 33, 608, "Input"],
Cell[880827, 26906, 7439, 237, 1564, "Output"],
Cell[888269, 27145, 273, 8, 216, "Output"]
}, Open  ]],
Cell[888557, 27156, 179, 4, 216, "Input"],
Cell[CellGroupData[{
Cell[888761, 27164, 775, 21, 480, "Input"],
Cell[889539, 27187, 3306, 112, 988, "Output"],
Cell[892848, 27301, 168, 2, 216, "Output"]
}, Open  ]],
Cell[893031, 27306, 1594, 46, 600, "Input"],
Cell[CellGroupData[{
Cell[894650, 27356, 792, 27, 428, "Input"],
Cell[895445, 27385, 1288, 46, 472, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[896770, 27436, 122, 2, 216, "Input"],
Cell[896895, 27440, 1824, 60, 264, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[898756, 27505, 1653, 48, 600, "Input"],
Cell[900412, 27555, 55106, 1844, 6152, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[955555, 29404, 4300, 108, 1776, "Input"],
Cell[959858, 29514, 623, 11, 340, "Message"],
Cell[960484, 29527, 2438, 70, 604, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[962959, 29602, 5058, 143, 1872, "Input"],
Cell[968020, 29747, 623, 11, 340, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[968680, 29763, 235, 6, 216, "Input"],
Cell[968918, 29771, 96, 1, 216, "Output"]
}, Open  ]],
Cell[969029, 29775, 4004, 116, 1852, "Input"],
Cell[CellGroupData[{
Cell[973058, 29895, 665, 21, 400, "Input"],
Cell[973726, 29918, 5237, 174, 1480, "Output"]
}, Open  ]],
Cell[978978, 30095, 5591, 176, 1616, "Input"],
Cell[CellGroupData[{
Cell[984594, 30275, 1366, 44, 496, "Input"],
Cell[985963, 30321, 616, 17, 216, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[986616, 30343, 1988, 62, 632, "Input"],
Cell[988607, 30407, 846, 23, 236, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[989490, 30435, 1176, 39, 620, "Input"],
Cell[990669, 30476, 7874, 246, 896, "Output"]
}, Open  ]],
Cell[998558, 30725, 91, 1, 216, "Input"],
Cell[CellGroupData[{
Cell[998674, 30730, 1633, 50, 620, "Input"],
Cell[1000310, 30782, 41614, 1268, 3520, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1041961, 32055, 173, 4, 224, "Input"],
Cell[1042137, 32061, 1151, 44, 392, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1043325, 32110, 518, 15, 216, "Input"],
Cell[1043846, 32127, 2055, 66, 440, "Output"]
}, Open  ]],
Cell[1045916, 32196, 142, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[1046083, 32201, 2153, 58, 888, "Input"],
Cell[1048239, 32261, 6858, 187, 1564, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1055134, 32453, 381, 11, 400, "Input"],
Cell[1055518, 32466, 1937, 61, 684, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1057492, 32532, 6969, 187, 1804, "Input"],
Cell[1064464, 32721, 4194, 116, 860, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1068695, 32842, 692, 22, 248, "Input"],
Cell[1069390, 32866, 646, 21, 248, "Output"]
}, Open  ]],
Cell[1070051, 32890, 788, 19, 484, "Text"],
Cell[1070842, 32911, 168, 2, 208, "Text"],
Cell[1071013, 32915, 826, 25, 198, "Input"],
Cell[CellGroupData[{
Cell[1071864, 32944, 1730, 56, 446, "Input"],
Cell[1073597, 33002, 363, 7, 316, "Output"]
}, Open  ]],
Cell[1073975, 33012, 4032, 115, 526, "Input"],
Cell[1078010, 33129, 101, 1, 104, "Text"],
Cell[1078114, 33132, 5543, 160, 698, "Input"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1083718, 33299, 171, 4, 70, "Section"],
Cell[1083892, 33305, 546, 15, 344, "Text"],
Cell[CellGroupData[{
Cell[1084463, 33324, 375, 10, 232, "Input"],
Cell[1084841, 33336, 146, 3, 216, "Output"]
}, Open  ]],
Cell[1085002, 33342, 887, 28, 360, "Text"],
Cell[CellGroupData[{
Cell[1085914, 33374, 285, 7, 264, "Input"],
Cell[1086202, 33383, 1543, 51, 368, "Output"]
}, Open  ]],
Cell[1087760, 33437, 393, 9, 344, "Text"],
Cell[1088156, 33448, 106, 1, 272, "Subsection"],
Cell[CellGroupData[{
Cell[1088287, 33453, 829, 16, 256, "Subsection"],
Cell[CellGroupData[{
Cell[1089141, 33473, 95, 1, 192, "Subsubsection"],
Cell[1089239, 33476, 477, 17, 224, "Text"],
Cell[1089719, 33495, 236, 5, 208, "Text"],
Cell[1089958, 33502, 1711, 52, 752, "Input"],
Cell[CellGroupData[{
Cell[1091694, 33558, 116, 2, 216, "Input"],
Cell[CellGroupData[{
Cell[1091835, 33564, 124, 2, 312, "Print",
 CellTags->"Info3565378871-5147713"],
Cell[1091962, 33568, 2226, 63, 296, "Print",
 CellTags->"Info3565378871-5147713"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1094237, 33637, 155, 3, 216, "Input"],
Cell[1094395, 33642, 10197, 342, 1592, "Output"]
}, Open  ]],
Cell[1104607, 33987, 205, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[1104837, 33995, 2966, 82, 1544, "Input"],
Cell[1107806, 34079, 611, 12, 340, "Message"],
Cell[1108420, 34093, 823, 20, 268, "Output"],
Cell[1109246, 34115, 634, 15, 268, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1109917, 34135, 2069, 56, 704, "Input"],
Cell[1111989, 34193, 878, 23, 268, "Output"]
}, Open  ]],
Cell[1112882, 34219, 431, 8, 328, "Text"],
Cell[CellGroupData[{
Cell[1113338, 34231, 594, 17, 432, "Input"],
Cell[1113935, 34250, 171, 3, 216, "Output"]
}, Open  ]],
Cell[1114121, 34256, 154, 2, 208, "Text"],
Cell[1114278, 34260, 1522, 45, 504, "Input"],
Cell[1115803, 34307, 839, 26, 400, "Input"],
Cell[CellGroupData[{
Cell[1116667, 34337, 1207, 37, 512, "Input"],
Cell[1117877, 34376, 1322, 45, 248, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1119236, 34426, 260, 7, 216, "Input"],
Cell[1119499, 34435, 1044, 37, 224, "Output"]
}, Open  ]],
Cell[1120558, 34475, 142, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[1120725, 34482, 1400, 41, 480, "Input"],
Cell[1122128, 34525, 1219, 37, 416, "Output"]
}, Open  ]],
Cell[1123362, 34565, 88, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[1123475, 34570, 1908, 55, 656, "Input"],
Cell[1125386, 34627, 7461, 246, 1064, "Output"],
Cell[1132850, 34875, 388, 5, 216, "Output"]
}, Open  ]],
Cell[1133253, 34883, 250, 5, 208, "Text"],
Cell[1133506, 34890, 845, 26, 428, "Input"],
Cell[1134354, 34918, 379, 10, 224, "Text"],
Cell[CellGroupData[{
Cell[1134758, 34932, 3057, 80, 1500, "Input"],
Cell[1137818, 35014, 5293, 142, 1244, "Output"]
}, Open  ]],
Cell[1143126, 35159, 134, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[1143285, 35166, 478, 15, 412, "Input"],
Cell[1143766, 35183, 387, 14, 368, "Output"]
}, Open  ]],
Cell[1144168, 35200, 110, 1, 208, "Text"],
Cell[1144281, 35203, 266, 6, 232, "Input"],
Cell[1144550, 35211, 239, 8, 224, "Text"],
Cell[CellGroupData[{
Cell[1144814, 35223, 1227, 35, 456, "Input"],
Cell[1146044, 35260, 268, 8, 392, "Output"]
}, Open  ]],
Cell[1146327, 35271, 237, 8, 224, "Text"],
Cell[CellGroupData[{
Cell[1146589, 35283, 3370, 87, 1532, "Input"],
Cell[1149962, 35372, 959, 28, 348, "Output"]
}, Open  ]],
Cell[1150936, 35403, 145, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[1151106, 35410, 2387, 69, 1080, "Input"],
Cell[1153496, 35481, 4627, 131, 1048, "Output"],
Cell[1158126, 35614, 1964, 54, 448, "Output"]
}, Open  ]],
Cell[1160105, 35671, 127, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[1160257, 35678, 763, 23, 264, "Input"],
Cell[1161023, 35703, 1317, 40, 248, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1162377, 35748, 374, 11, 224, "Input"],
Cell[1162754, 35761, 849, 28, 248, "Output"]
}, Open  ]],
Cell[1163618, 35792, 294, 5, 208, "Text"],
Cell[1163915, 35799, 129, 3, 208, "Text"],
Cell[1164047, 35804, 1310, 41, 460, "Input"],
Cell[1165360, 35847, 1170, 39, 400, "Input"],
Cell[1166533, 35888, 617, 16, 344, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1167187, 35909, 579, 13, 312, "Subsubsection"],
Cell[1167769, 35924, 813, 26, 392, "Input"],
Cell[1168585, 35952, 329, 6, 328, "Text"],
Cell[CellGroupData[{
Cell[1168939, 35962, 482, 14, 216, "Input"],
Cell[1169424, 35978, 1513, 47, 300, "Output"]
}, Open  ]],
Cell[1170952, 36028, 126, 3, 208, "Text"],
Cell[1171081, 36033, 1006, 32, 496, "Input"],
Cell[1172090, 36067, 111, 1, 208, "Text"],
Cell[1172204, 36070, 3024, 94, 1128, "Input"],
Cell[1175231, 36166, 220, 5, 208, "Text"],
Cell[1175454, 36173, 15272, 475, 4688, "Input"],
Cell[1190729, 36650, 373, 9, 224, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1191139, 36664, 971, 27, 320, "Subsubsection"],
Cell[1192113, 36693, 2149, 72, 1132, "Input"],
Cell[CellGroupData[{
Cell[1194287, 36769, 1000, 27, 616, "Input"],
Cell[1195290, 36798, 1009, 32, 456, "Output"]
}, Open  ]],
Cell[1196314, 36833, 223, 6, 328, "Text"],
Cell[1196540, 36841, 832, 28, 744, "Input"],
Cell[1197375, 36871, 133, 3, 208, "Text"],
Cell[1197511, 36876, 684, 23, 572, "Input"],
Cell[CellGroupData[{
Cell[1198220, 36903, 735, 22, 520, "Input"],
Cell[1198958, 36927, 2758, 79, 732, "Output"]
}, Open  ]],
Cell[1201731, 37009, 236, 8, 208, "Text"],
Cell[CellGroupData[{
Cell[1201992, 37021, 1363, 35, 600, "Input"],
Cell[1203358, 37058, 2306, 66, 704, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1205713, 37130, 1312, 35, 280, "Subsubsection"],
Cell[1207028, 37167, 172, 4, 208, "Text"],
Cell[1207203, 37173, 1247, 37, 364, "Input"],
Cell[1208453, 37212, 1339, 36, 408, "Input"],
Cell[1209795, 37250, 227, 4, 208, "Text"],
Cell[CellGroupData[{
Cell[1210047, 37258, 289, 8, 216, "Input"],
Cell[1210339, 37268, 388, 12, 344, "Output"]
}, Open  ]],
Cell[1210742, 37283, 1464, 50, 376, "Input"],
Cell[1212209, 37335, 450, 14, 228, "Text"],
Cell[1212662, 37351, 955, 31, 360, "Input"],
Cell[1213620, 37384, 2967, 89, 508, "Input"],
Cell[1216590, 37475, 105, 1, 208, "Text"],
Cell[1216698, 37478, 731, 25, 524, "Input"],
Cell[1217432, 37505, 114, 1, 208, "Text"],
Cell[1217549, 37508, 4386, 138, 1392, "Input"],
Cell[1221938, 37648, 257, 5, 208, "Text"],
Cell[1222198, 37655, 5755, 171, 840, "Input"],
Cell[1227956, 37828, 878, 25, 340, "Text"],
Cell[1228837, 37855, 2249, 69, 376, "Input"],
Cell[1231089, 37926, 274, 5, 328, "Text"],
Cell[1231366, 37933, 25832, 779, 3468, "Input"],
Cell[1257201, 38714, 121, 3, 208, "Text"],
Cell[1257325, 38719, 1716, 56, 444, "Input"],
Cell[1259044, 38777, 130, 3, 208, "Text"],
Cell[1259177, 38782, 3734, 118, 1052, "Input"],
Cell[1262914, 38902, 33581, 976, 9772, "Input"],
Cell[CellGroupData[{
Cell[1296520, 39882, 100, 1, 216, "Input"],
Cell[1296623, 39885, 1969, 66, 376, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1298629, 39956, 555, 16, 392, "Input"],
Cell[1299187, 39974, 406, 12, 236, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1299654, 39993, 137, 3, 196, "Subsection"],
Cell[CellGroupData[{
Cell[1299816, 40000, 161, 3, 216, "Input"],
Cell[1299980, 40005, 237, 7, 256, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1300254, 40017, 1825, 52, 336, "Subsubsection"],
Cell[1302082, 40071, 97, 1, 208, "Text"],
Cell[1302182, 40074, 2101, 63, 668, "Input"],
Cell[1304286, 40139, 2148, 67, 464, "Input"],
Cell[CellGroupData[{
Cell[1306459, 40210, 1044, 29, 536, "Input"],
Cell[1307506, 40241, 1551, 47, 704, "Output"]
}, Open  ]],
Cell[1309072, 40291, 1799, 46, 752, "Input"],
Cell[1310874, 40339, 851, 24, 492, InheritFromParent],
Cell[1311728, 40365, 768, 24, 344, "Output"],
Cell[1312499, 40391, 144, 3, 208, "Text"],
Cell[1312646, 40396, 5213, 163, 1208, "Input"],
Cell[1317862, 40561, 129, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[1318016, 40568, 2687, 74, 1088, "Input"],
Cell[1320706, 40644, 844, 25, 344, "Output"]
}, Open  ]],
Cell[1321565, 40672, 701, 23, 360, "Input"],
Cell[1322269, 40697, 2739, 86, 1144, "Input"],
Cell[CellGroupData[{
Cell[1325033, 40787, 755, 23, 296, "Input"],
Cell[1325791, 40812, 2172, 66, 1004, "Output"]
}, Open  ]],
Cell[1327978, 40881, 98, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[1328101, 40886, 2382, 68, 1096, "Input"],
Cell[1330486, 40956, 768, 24, 344, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1331303, 40986, 1766, 43, 288, "Subsubsection"],
Cell[1333072, 41031, 2654, 78, 560, "Input"],
Cell[1335729, 41111, 106, 1, 208, "Text"],
Cell[CellGroupData[{
Cell[1335860, 41116, 2151, 55, 1272, "Input"],
Cell[1338014, 41173, 2109, 69, 640, "Output"]
}, Open  ]],
Cell[1340138, 41245, 107, 1, 208, "Text"],
Cell[1340248, 41248, 201, 6, 216, "Text"],
Cell[CellGroupData[{
Cell[1340474, 41258, 1313, 35, 752, "Input"],
Cell[1341790, 41295, 1141, 37, 396, "Output"]
}, Open  ]],
Cell[1342946, 41335, 3335, 91, 1360, "Input"],
Cell[1346284, 41428, 3116, 90, 1260, "Input"],
Cell[CellGroupData[{
Cell[1349425, 41522, 1065, 30, 732, "Input"],
Cell[1350493, 41554, 2918, 91, 344, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1353448, 41650, 131, 2, 216, "Input"],
Cell[1353582, 41654, 33043, 1121, 2632, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1386662, 42780, 126, 2, 216, "Input"],
Cell[1386791, 42784, 2945, 97, 268, "Output"]
}, Open  ]],
Cell[1389751, 42884, 5039, 159, 1608, "Input"],
Cell[CellGroupData[{
Cell[1394815, 43047, 801, 25, 384, "Input"],
Cell[1395619, 43074, 625, 21, 248, "Output"]
}, Open  ]],
Cell[1396259, 43098, 3806, 94, 1648, "Input"],
Cell[CellGroupData[{
Cell[1400090, 43196, 1032, 30, 424, "Input"],
Cell[1401125, 43228, 11623, 310, 2600, "Output"]
}, Open  ]],
Cell[1412763, 43541, 412, 10, 208, "Text"],
Cell[CellGroupData[{
Cell[1413200, 43555, 1492, 43, 716, "Input"],
Cell[1414695, 43600, 8846, 278, 2172, "Output"]
}, Open  ]],
Cell[1423556, 43881, 525, 13, 232, "Text"],
Cell[1424084, 43896, 2800, 86, 1040, "Input"],
Cell[CellGroupData[{
Cell[1426909, 43986, 1223, 38, 296, "Input"],
Cell[1428135, 44026, 1462, 47, 344, "Output"]
}, Open  ]],
Cell[1429612, 44076, 2900, 95, 296, "Input"],
Cell[CellGroupData[{
Cell[1432537, 44175, 98, 1, 216, "Input"],
Cell[1432638, 44178, 1957, 66, 640, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1434632, 44249, 3145, 98, 1144, "Input"],
Cell[1437780, 44349, 25330, 751, 5344, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1463183, 45108, 262, 8, 75, "Section"],
Cell[1463448, 45118, 490, 11, 170, "Text"],
Cell[1463941, 45131, 285, 5, 164, "Text"],
Cell[1464229, 45138, 534, 19, 200, "Input"],
Cell[CellGroupData[{
Cell[1464788, 45161, 1269, 35, 548, "Input"],
Cell[1466060, 45198, 1229, 43, 480, "Output"]
}, Open  ]],
Cell[1467304, 45244, 731, 24, 412, "Text"],
Cell[1468038, 45270, 490, 17, 412, "Input"],
Cell[CellGroupData[{
Cell[1468553, 45291, 1919, 53, 1056, "Input"],
Cell[1470475, 45346, 1240, 33, 460, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1471752, 45384, 1416, 40, 712, "Input"],
Cell[1473171, 45426, 3472, 101, 780, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1476680, 45532, 442, 13, 428, "Input"],
Cell[1477125, 45547, 3492, 102, 780, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1480654, 45654, 101, 1, 304, "Input"],
Cell[1480758, 45657, 842, 30, 480, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1481637, 45692, 645, 18, 620, "Input"],
Cell[1482285, 45712, 3058, 103, 540, "Output"]
}, Open  ]],
Cell[1485358, 45818, 820, 24, 668, "Input"],
Cell[CellGroupData[{
Cell[1486203, 45846, 96, 1, 304, "Input"],
Cell[1486302, 45849, 15449, 425, 2232, "Output"]
}, Open  ]],
Cell[1501766, 46277, 69, 1, 216, "Input"],
Cell[CellGroupData[{
Cell[1501860, 46282, 972, 26, 712, "Input"],
Cell[1502835, 46310, 6279, 169, 844, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1509151, 46484, 447, 13, 304, "Input"],
Cell[1509601, 46499, 9411, 278, 2532, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1519049, 46782, 1000, 27, 932, "Input"],
Cell[1520052, 46811, 13747, 353, 2056, "Output"]
}, Open  ]],
Cell[1533814, 47167, 318, 9, 220, "Text"],
Cell[CellGroupData[{
Cell[1534157, 47180, 147, 3, 304, "Input"],
Cell[1534307, 47185, 333, 11, 432, "Output"]
}, Open  ]],
Cell[1534655, 47199, 452, 13, 328, "Text"],
Cell[CellGroupData[{
Cell[1535132, 47216, 315, 11, 492, "Input"],
Cell[1535450, 47229, 703, 24, 468, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1536190, 47258, 33, 0, 304, "Input"],
Cell[1536226, 47260, 2979, 101, 540, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1539242, 47366, 2419, 79, 796, "Input"],
Cell[1541664, 47447, 895, 28, 460, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1542596, 47480, 842, 28, 1120, "Input"],
Cell[1543441, 47510, 196, 3, 324, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1543674, 47518, 103, 1, 304, "Input"],
Cell[1543780, 47521, 1946, 65, 452, "Output"]
}, Open  ]],
Cell[1545741, 47589, 1429, 44, 524, "Input"],
Cell[CellGroupData[{
Cell[1547195, 47637, 3738, 110, 1560, "Input"],
Cell[1550936, 47749, 7770, 264, 1112, "Output"]
}, Open  ]],
Cell[1558721, 48016, 302, 8, 220, "Text"],
Cell[1559026, 48026, 152, 3, 208, "Text"],
Cell[CellGroupData[{
Cell[1559203, 48033, 8953, 244, 4832, "Input"],
Cell[1568159, 48279, 3215, 106, 728, "Output"]
}, Open  ]],
Cell[1571389, 48388, 3473, 122, 948, InheritFromParent],
Cell[CellGroupData[{
Cell[1574887, 48514, 176, 3, 304, "Input"],
Cell[1575066, 48519, 167, 3, 412, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1575270, 48527, 130, 2, 304, "Input"],
Cell[1575403, 48531, 101, 1, 304, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1575541, 48537, 95, 1, 304, "Input"],
Cell[1575639, 48540, 4538, 159, 792, "Output"]
}, Open  ]],
Cell[1580192, 48702, 16, 0, 208, "Text"],
Cell[1580211, 48704, 16, 0, 208, "Text"]
}, Closed]]
}
]
*)

(* End of internal cache information *)
