(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    189505,       6069]
NotebookOptionsPosition[    178089,       5696]
NotebookOutlinePosition[    178462,       5713]
CellTagsIndexPosition[    178419,       5710]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Usage of the packages ", "Title",
 CellChangeTimes->{{3.568692868984467*^9, 3.568692911501943*^9}}],

Cell["\<\
provided with the paper \"Multiple zeta functions and double wrapping in \
planar N = 4 SYM\"\
\>", "Subtitle",
 CellChangeTimes->{{3.56869291288573*^9, 3.568692917246587*^9}, {
  3.569076386552355*^9, 3.569076386775065*^9}}],

Cell[CellGroupData[{

Cell["\<\
initialization cells : Load the files of packages\
\>", "Section",
 CellChangeTimes->{{3.568692934940939*^9, 3.568692944658409*^9}, {
  3.56869304697738*^9, 3.5686930779735727`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "Init", "}"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"Init", "[", "]"}], ":=", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"$Path", "=", 
         RowBox[{"Cases", "[", 
          RowBox[{
           RowBox[{"DeleteDuplicates", "[", 
            RowBox[{"Prepend", "[", 
             RowBox[{"$Path", ",", "PathToFiles"}], "]"}], "]"}], ",", 
           "_String"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"$Copyright", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Get", "/@", 
         RowBox[{"{", 
          RowBox[{
          "\"\<stylepackage.m\>\"", ",", "\"\<zetafunctions.m\>\"", ",", 
           "\"\<FiNLIEKonishi.m\>\"", ",", "\"\<UpWiseIntegrate.m\>\""}], 
          "}"}]}], ";"}], ")"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Check", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PathToFiles", "=", 
         RowBox[{"NotebookDirectory", "[", "]"}]}], ";", 
        RowBox[{"Init", "[", "]"}]}], ",", 
       RowBox[{
        RowBox[{"PathToFiles", "=", 
         RowBox[{"InputString", "[", 
          RowBox[{
          "\"\<Please enter the name of the folder where you saved the \
packages\>\"", ",", "$UserDocumentsDirectory"}], "]"}]}], ";", 
        RowBox[{"Init", "[", "]"}]}]}], "]"}]}]}], "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.569076101040699*^9, 3.569076240705756*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["stylepackage", "Section",
 CellChangeTimes->{{3.5686963458116283`*^9, 3.568696363145094*^9}, 
   3.5690160386151114`*^9, {3.5690167212124443`*^9, 3.569016723537724*^9}}],

Cell[TextData[{
 "The stylepackage was designed to produce a nice output for the FiNLIE \
quantities in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", and to make this output suitable for new input in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellChangeTimes->{{3.569016726417934*^9, 3.5690167315213957`*^9}, {
  3.569016785409567*^9, 3.569016823630515*^9}}],

Cell[CellGroupData[{

Cell["\<\
Internal formatting and standard output\
\>", "Subsection",
 CellChangeTimes->{{3.56901730501108*^9, 3.569017327394164*^9}}],

Cell["\<\
Internally, the FiNLIE quantities and related objects are stored as \
function$specification[indices][shift] or function$specification[shift]. In \
fact, any input of this form will be formatted on the output\
\>", "Text",
 CellChangeTimes->{{3.569016945088696*^9, 3.5690170531730213`*^9}, {
  3.569017345066372*^9, 3.569017347697877*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"FUN$SPEC", "[", 
     RowBox[{"indices1", ",", "indices2"}], "]"}], "[", "shift", "]"}], ",", 
   RowBox[{"FUN$SPEC", "[", "shift", "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.5690169925946283`*^9, 3.569017072391464*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubsuperscriptBox["FUN", 
    RowBox[{"indices1", ",", "indices2"}], 
    RowBox[{"[", "shift", "]"}]], ",", 
   SuperscriptBox["FUN", 
    RowBox[{"[", "shift", "]"}]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.569016993773632*^9, 3.569017025824088*^9, {3.569017064335713*^9, 
   3.569017072899446*^9}}]
}, Open  ]],

Cell["\<\
If shift=1, or shift =-1, the following output is generated\
\>", "Text",
 CellChangeTimes->{{3.56901725187105*^9, 3.5690172721079493`*^9}, {
  3.569017361936936*^9, 3.569017364032586*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"FUN$SPEC", "[", 
     RowBox[{"indices1", ",", "indices2"}], "]"}], "[", "1", "]"}], ",", 
   RowBox[{"FUN$SPEC", "[", 
    RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.569017279160838*^9, 3.569017281334671*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubsuperscriptBox["FUN", 
    RowBox[{"indices1", ",", "indices2"}], "+"], ",", 
   SuperscriptBox["FUN", "-"]}], "}"}]], "Output",
 CellChangeTimes->{3.569017281768784*^9}]
}, Open  ]],

Cell["\<\
Specifications should be a word from letters\
\>", "Text",
 CellChangeTimes->{{3.56901708180457*^9, 3.5690170914913807`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["specorder"], "Input",
 CellChangeTimes->{{3.569017093104394*^9, 3.5690170945085173`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"t\"\>", ",", "\<\"h\"\>", ",", "\<\"b\"\>", ",", "\<\"f\"\>", 
   ",", "\<\"r\"\>", ",", "\<\"u\"\>", ",", "\<\"w\"\>", ",", "\<\"g\"\>"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.56901709511483*^9}]
}, Open  ]],

Cell["\<\
t puts tilde over function, h -puts hat over function, b - puts bar over \
function, f - means that it is a form, r - right gauge, u - upper gauge, w - \
white gauge, g- physical gauge (bold font in the paper \
\[OpenCurlyDoubleQuote]solving the AdS/CFT Y-system\[CloseCurlyDoubleQuote]). \
A combination of specifications can be used\
\>", "Text",
 CellChangeTimes->{{3.569017100354916*^9, 3.569017188119773*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"T$hu", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{"T$tr", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "[", "1", "]"}], ",", 
   RowBox[{"FUN$bg", "[", "2", "]"}], ",", 
   RowBox[{"FUN$bg", "[", "2", "]"}], ",", 
   RowBox[{"N$r", "[", 
    RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.569017189054237*^9, 3.569017245592901*^9}, {
  3.5690174032110977`*^9, 3.569017417474506*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[GothicCapitalT]",
      FontColor->RGBColor[0.54, 0.35, 0.15]], 
     StyleBox["^",
      FontColor->RGBColor[0.54, 0.35, 0.15]]], 
    RowBox[{"2", ",", "1"}], "+"], ",", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["\[ScriptCapitalT]",
      FontColor->RGBColor[1, 0, 0]], 
     StyleBox["~",
      FontColor->RGBColor[1, 0, 0]]], 
    RowBox[{"2", ",", "1"}], "+"], ",", 
   SuperscriptBox[
    OverscriptBox["\:ff26\:ff35\:ff2e", "_"], 
    RowBox[{"[", "2", "]"}]], ",", 
   SuperscriptBox[
    OverscriptBox["\:ff26\:ff35\:ff2e", "_"], 
    RowBox[{"[", "2", "]"}]], ",", 
   SuperscriptBox["\[ScriptCapitalN]", "-"]}], "}"}]], "Output",
 CellChangeTimes->{{3.569017201314883*^9, 3.5690172369676027`*^9}, {
  3.569017411575039*^9, 3.5690174179477673`*^9}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["How one can input expressions", "Subsection",
 CellChangeTimes->{{3.56901730501108*^9, 3.569017327394164*^9}, {
  3.569017381946439*^9, 3.5690173923035097`*^9}}],

Cell[TextData[{
 "\[LineSeparator]Expressions can be obviously entered as \
function$specification[indices][shift], but if the function is declared in \
stylepackage, then user-friendly input can be used. E.g, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " correctly recognizes any of this input:"
}], "Text",
 CellChangeTimes->{{3.569017441749733*^9, 3.569017548219121*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubsuperscriptBox["q", 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   RowBox[{
    RowBox[{"q", "[", 
     RowBox[{"1", ",", "3"}], "]"}], "[", "1", "]"}], ",", 
   RowBox[{
    SubscriptBox["q", 
     RowBox[{"1", ",", "3"}]], "[", "1", "]"}], ",", 
   RowBox[{
    RowBox[{"q$", "[", 
     RowBox[{"1", ",", "3"}], "]"}], "[", "1", "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubsuperscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   SubsuperscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   SubsuperscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   SubsuperscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], 
    RowBox[{"1", ",", "3"}], "+"]}], "}"}]], "Output",
 CellChangeTimes->{3.569017549779995*^9}]
}, Open  ]],

Cell["\<\
Note that we can change appearance by changing gauge\
\>", "Text",
 CellChangeTimes->{{3.569017559976851*^9, 3.5690175701763983`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    SubsuperscriptBox["q", 
     RowBox[{"1", ",", "3"}], "+"], ",", 
    RowBox[{
     RowBox[{"q", "[", 
      RowBox[{"1", ",", "3"}], "]"}], "[", "1", "]"}], ",", 
    RowBox[{
     SubscriptBox["q", 
      RowBox[{"1", ",", "3"}]], "[", "1", "]"}], ",", 
    RowBox[{
     RowBox[{"q$", "[", 
      RowBox[{"1", ",", "3"}], "]"}], "[", "1", "]"}]}], "}"}], "/.", 
  RowBox[{"q$", "\[Rule]", "q$u"}]}]], "Input",
 CellChangeTimes->{{3.569017574901662*^9, 3.56901757710775*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubsuperscriptBox[
    StyleBox["\[GothicQ]",
     FontColor->RGBColor[0.54, 0.35, 0.15]], 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   SubsuperscriptBox[
    StyleBox["\[GothicQ]",
     FontColor->RGBColor[0.54, 0.35, 0.15]], 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   SubsuperscriptBox[
    StyleBox["\[GothicQ]",
     FontColor->RGBColor[0.54, 0.35, 0.15]], 
    RowBox[{"1", ",", "3"}], "+"], ",", 
   SubsuperscriptBox[
    StyleBox["\[GothicQ]",
     FontColor->RGBColor[0.54, 0.35, 0.15]], 
    RowBox[{"1", ",", "3"}], "+"]}], "}"}]], "Output",
 CellChangeTimes->{3.5690175776313963`*^9}]
}, Open  ]],

Cell["\<\
The list of declared functions is\
\>", "Text",
 CellChangeTimes->{{3.569017582994397*^9, 3.5690175877198887`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["FunctionsOfSpectralParameter"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"T\"\>", ",", "\<\"W\"\>", ",", "\<\"Y\"\>", ",", "\<\"Yt\"\>", 
   ",", "\<\"Yo\"\>", ",", "\<\"q\"\>", ",", "\<\"p\"\>", 
   ",", "\<\"\[Eta]\"\>", ",", "\<\"\[Eta]0\"\>", ",", "\<\"\[Zeta]\"\>", 
   ",", "\<\"Q\"\>", ",", "\<\"\[ScriptU]\"\>", ",", "\<\"q\"\>", 
   ",", "\<\"p\"\>", ",", "\<\"\[Omega]\"\>", ",", "\<\"\[Phi]\"\>", 
   ",", "\<\"\[Theta]\"\>", ",", "\<\"H\"\>", 
   ",", "\<\"\[ScriptCapitalF]\"\>", ",", "\<\"\[ScriptCapitalC]\"\>", 
   ",", "\<\"Bp\"\>", ",", "\<\"Bm\"\>", ",", "\<\"Rp\"\>", ",", "\<\"Rm\"\>",
    ",", "\<\"Qb\"\>", ",", "\<\"\[ScriptCapitalX]\"\>", 
   ",", "\<\"\[ScriptCapitalY]\"\>", ",", "\<\"h\"\>", ",", "\<\"f\"\>", 
   ",", "\<\"x\"\>", ",", "\<\"y\"\>", ",", "\<\"U\"\>", ",", "\<\"V\"\>", 
   ",", "\<\"Q\"\>", ",", "\<\"\[Delta]Q\"\>", ",", "\<\"Q0\"\>", 
   ",", "\<\"Th\"\>", ",", "\<\"\[ScriptCapitalF]h\"\>", ",", "\<\"hh\"\>", 
   ",", "\<\"xh\"\>", ",", "\<\"yh\"\>", ",", "\<\"Uh\"\>", 
   ",", "\<\"U0h\"\>", ",", "\<\"Ub\"\>", ",", "\<\"Qt\"\>", ",", "\<\"T\"\>",
    ",", "\<\"W\"\>", ",", "\<\"Y\"\>", ",", "\<\"Yt\"\>", ",", "\<\"Yo\"\>", 
   ",", "\<\"q\"\>", ",", "\<\"p\"\>", ",", "\<\"\[Eta]\"\>", 
   ",", "\<\"\[Eta]0\"\>", ",", "\<\"\[Zeta]\"\>", ",", "\<\"Q\"\>", 
   ",", "\<\"\[ScriptU]\"\>", ",", "\<\"\[DoubleStruckCapitalT]\"\>", 
   ",", "\<\"\[GothicCapitalT]\"\>", ",", "\<\"\[ScriptCapitalT]\"\>", 
   ",", "\<\"\:ff34\"\>", ",", "\<\"\[DoubleStruckQ]\"\>", 
   ",", "\<\"\[GothicQ]\"\>", ",", "\<\"\[ScriptQ]\"\>", 
   ",", "\<\"\:ff51\"\>", ",", "\<\"\[DoubleStruckP]\"\>", 
   ",", "\<\"\[GothicP]\"\>", ",", "\<\"\[ScriptP]\"\>", 
   ",", "\<\"\:ff50\"\>", ",", "\<\"\[Omega]\"\>", ",", "\<\"\[Phi]\"\>", 
   ",", "\<\"\[Theta]\"\>", ",", "\<\"H\"\>", 
   ",", "\<\"\[ScriptCapitalF]\"\>", ",", "\<\"\[ScriptCapitalC]\"\>", 
   ",", "\<\"Bp\"\>", ",", "\<\"Bm\"\>", ",", "\<\"Rp\"\>", ",", "\<\"Rm\"\>",
    ",", "\<\"Qb\"\>", ",", "\<\"\[ScriptCapitalX]\"\>", 
   ",", "\<\"\[ScriptCapitalY]\"\>", ",", "\<\"h\"\>", ",", "\<\"f\"\>", 
   ",", "\<\"x\"\>", ",", "\<\"y\"\>", ",", "\<\"U\"\>", ",", "\<\"V\"\>", 
   ",", "\<\"\[Delta]Q\"\>", ",", "\<\"Q0\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.569017646283319*^9}]
}, Open  ]],

Cell["\<\
In section \[OpenCurlyDoubleQuote]Style Assignments\[CloseCurlyDoubleQuote] \
of the package the reader can add his favorite functions.\
\>", "Text",
 CellChangeTimes->{{3.569017660902617*^9, 3.56901767225257*^9}, {
  3.5690177044037247`*^9, 3.569017712027178*^9}}],

Cell["\<\
The following functions are defined as without indices\
\>", "Text",
 CellChangeTimes->{{3.569017740481916*^9, 3.5690177507854156`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$FunctionsWithoutArguments", "[", 
  RowBox[{"[", 
   RowBox[{"All", ",", "1"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.569017727995081*^9, 3.569017736315888*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"\[Phi]\"\>", ",", "\<\"\[Theta]\"\>", ",", "\<\"H\"\>", 
   ",", "\<\"\[ScriptCapitalF]\"\>", ",", "\<\"\[ScriptCapitalC]\"\>", 
   ",", "\<\"Bp\"\>", ",", "\<\"Bm\"\>", ",", "\<\"Rp\"\>", ",", "\<\"Rm\"\>",
    ",", "\<\"Qb\"\>", ",", "\<\"\[ScriptCapitalX]\"\>", 
   ",", "\<\"\[ScriptCapitalY]\"\>", ",", "\<\"h\"\>", ",", "\<\"f\"\>", 
   ",", "\<\"x\"\>", ",", "\<\"y\"\>", ",", "\<\"U\"\>", ",", "\<\"V\"\>", 
   ",", "\<\"Q\"\>", ",", "\<\"\[Delta]Q\"\>", ",", "\<\"Q0\"\>"}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.5690177289336987`*^9, 3.569017736712503*^9}}]
}, Open  ]],

Cell["\<\
For each function we specify two parameters:\
\>", "Text",
 CellChangeTimes->{{3.569018052709676*^9, 3.569018077204564*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$FunctionsWithoutArguments", "[", 
  RowBox[{"[", "3", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.569018065849986*^9, 3.5690180676477003`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"H\"\>", ",", "False", ",", 
   RowBox[{"{", 
    RowBox[{"FontColor", "\[Rule]", 
     RowBox[{"RGBColor", "[", 
      RowBox[{"0.54`", ",", "0.35`", ",", "0.15`"}], "]"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5690180679614773`*^9}]
}, Open  ]],

Cell["\<\
First entry above is symbol of a function. Next argument - whether it is \
gauge depended (i.e. whether u,r,w,g specs mean something), the last argument \
- what style to use to depict function. In this case we use a particular \
color code which is different for different functions\
\>", "Text",
 CellChangeTimes->{{3.569018091036282*^9, 3.56901818595262*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"H", ",", "q", ",", "p"}], "}"}]], "Input",
 CellChangeTimes->{{3.569018171513921*^9, 3.569018178186719*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   StyleBox["H",
    FontColor->RGBColor[0.54, 0.35, 0.15]], ",", 
   StyleBox["q",
    FontColor->RGBColor[0, 0, 0.8]], ",", 
   StyleBox["p",
    FontColor->RGBColor[0.8, 0, 0]]}], "}"}]], "Output",
 CellChangeTimes->{{3.56901817193906*^9, 3.569018178606435*^9}}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will recognize U as U$[0]:"
}], "Text",
 CellChangeTimes->{{3.569017754525316*^9, 3.5690177881841097`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{"U", "\[IndentingNewLine]", 
 RowBox[{"%", "//", "FullForm"}]}], "Input",
 CellChangeTimes->{{3.5690177607072563`*^9, 3.56901776707556*^9}}],

Cell[BoxData[
 StyleBox["U",
  FontColor->RGBColor[0, 0.6, 0]]], "Output",
 CellChangeTimes->{{3.569017761099059*^9, 3.569017789133132*^9}}],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"U$", "[", "0", "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{{3.569017761099059*^9, 3.5690177891413727`*^9}}]
}, Open  ]],

Cell["and U[2] as U$[2]", "Text",
 CellChangeTimes->{{3.5690177968233757`*^9, 3.56901780231145*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"U", "[", "2", "]"}], "\[IndentingNewLine]", 
 RowBox[{"%", "//", "FullForm"}]}], "Input",
 CellChangeTimes->{{3.569017803419812*^9, 3.569017807768989*^9}}],

Cell[BoxData[
 SuperscriptBox[
  StyleBox["U",
   FontColor->RGBColor[0, 0.6, 0]], 
  RowBox[{"[", "2", "]"}]]], "Output",
 CellChangeTimes->{3.569017808142177*^9}],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"U$", "[", "2", "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.569017808144865*^9}]
}, Open  ]],

Cell["\<\
At the same time, one can override the default and assign some  index to U \
that explains its properties:\
\>", "Text",
 CellChangeTimes->{{3.569017819774762*^9, 3.569017860389325*^9}, {
  3.569018838410975*^9, 3.569018844918973*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"U", "[", "\"\<as\>\"", "]"}], "[", "2", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"%", "//", "FullForm"}], ",", 
   RowBox[{"%", "/.", 
    RowBox[{"nsweak", "[", "1", "]"}]}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.569017827528397*^9, 3.569017869072216*^9}, {
  3.56901881198938*^9, 3.5690188135776463`*^9}}],

Cell[BoxData[
 SubsuperscriptBox[
  StyleBox["U",
   FontColor->RGBColor[0, 0.6, 0]], "\<\"as\"\>", 
  RowBox[{"[", "2", "]"}]]], "Output",
 CellChangeTimes->{{3.569017830848428*^9, 3.569017869472764*^9}, 
   3.569018813978833*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TagBox[
    StyleBox[
     RowBox[{
      RowBox[{"U$", "[", "\"\<as\>\"", "]"}], "[", "2", "]"}],
     ShowSpecialCharacters->False,
     ShowStringCharacters->True,
     NumberMarks->True],
    FullForm], ",", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SqrtBox["2"], " ", 
     SuperscriptBox["g", "2"]}], 
    SuperscriptBox[
     StyleBox["\[ScriptU]",
      FontColor->RGBColor[0, 0, 0.4]], 
     RowBox[{"[", "2", "]"}]]]}], "}"}]], "Output",
 CellChangeTimes->{{3.569017830848428*^9, 3.569017869472764*^9}, 
   3.569018813988867*^9}]
}, Open  ]],

Cell["Functions with indices ", "Text",
 CellChangeTimes->{{3.5690179744328613`*^9, 3.5690180113759203`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"$FunctionsWithArguments", "[", 
   RowBox[{"[", 
    RowBox[{"All", ",", "1"}], "]"}], "]"}], "\[IndentingNewLine]"}]], "Input",\

 CellChangeTimes->{{3.569017998841722*^9, 3.569018012785542*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"T\"\>", ",", "\<\"W\"\>", ",", "\<\"Y\"\>", ",", "\<\"Yt\"\>", 
   ",", "\<\"Yo\"\>", ",", "\<\"q\"\>", ",", "\<\"p\"\>", 
   ",", "\<\"\[Eta]\"\>", ",", "\<\"\[Eta]0\"\>", ",", "\<\"\[Zeta]\"\>", 
   ",", "\<\"Q\"\>", ",", "\<\"\[ScriptU]\"\>"}], "}"}]], "Output",
 CellChangeTimes->{{3.569017999751399*^9, 3.569018014429719*^9}}]
}, Open  ]],

Cell["have three parameters", "Text",
 CellChangeTimes->{{3.569018018645357*^9, 3.569018021750791*^9}, {
  3.569018195215925*^9, 3.56901819939951*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$FunctionsWithArguments", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.569018022994343*^9, 3.5690180294159718`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"T\"\>", ",", "True", ",", "\<\"Default\"\>", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5690180298104258`*^9}]
}, Open  ]],

Cell["\<\
Legend: 1st entry - symbol of function, 2nd entry - whether it is gauge \
dependent, 3rd entry - output style, 4th entry default value of index. \
Default index is not displayed in the output. E.g:\
\>", "Text",
 CellChangeTimes->{{3.5690180384698877`*^9, 3.569018043637864*^9}, {
  3.569018202015607*^9, 3.5690182587976513`*^9}, {3.569018300644589*^9, 
  3.569018323251689*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[ScriptCapitalT]", "[", "1", "]"}], "\[IndentingNewLine]", 
 RowBox[{"%", "//", "FullForm"}]}], "Input",
 CellChangeTimes->{{3.5690182479327517`*^9, 3.5690182962214746`*^9}}],

Cell[BoxData[
 SuperscriptBox[
  StyleBox["\[ScriptCapitalT]",
   FontColor->RGBColor[1, 0, 0]], "+"]], "Output",
 CellChangeTimes->{{3.56901824888017*^9, 3.5690183097192917`*^9}}],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{
    RowBox[{"T$r", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "[", "1", "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{{3.56901824888017*^9, 3.569018309725878*^9}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Shifts of argument and complex conjugation\
\>", "Subsection",
 CellChangeTimes->{{3.56901730501108*^9, 3.569017327394164*^9}, {
  3.569017381946439*^9, 3.5690173923035097`*^9}, {3.569017894550523*^9, 
  3.569017896483879*^9}, {3.569017945244227*^9, 3.569017952281703*^9}}],

Cell["\<\
There are only two operations provided.  The first one is Shift\
\>", "Text",
 CellChangeTimes->{{3.569017954573209*^9, 3.569017964864944*^9}, {
  3.569018332443796*^9, 3.569018361905599*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"q$", "[", 
   RowBox[{"1", ",", "3"}], "]"}], "[", "2", "]"}], "\[IndentingNewLine]", 
 RowBox[{"%", "/.", 
  RowBox[{"Shift", "[", "ee", "]"}]}]}], "Input",
 CellChangeTimes->{{3.569018343670594*^9, 3.569018354196364*^9}}],

Cell[BoxData[
 SubsuperscriptBox[
  StyleBox["q",
   FontColor->RGBColor[0, 0, 0.8]], 
  RowBox[{"1", ",", "3"}], 
  RowBox[{"[", "2", "]"}]]], "Output",
 CellChangeTimes->{3.569018354605959*^9}],

Cell[BoxData[
 SubsuperscriptBox[
  StyleBox["q",
   FontColor->RGBColor[0, 0, 0.8]], 
  RowBox[{"1", ",", "3"}], 
  RowBox[{"[", 
   RowBox[{"2", "+", "ee"}], "]"}]]], "Output",
 CellChangeTimes->{3.569018354610096*^9}]
}, Open  ]],

Cell[TextData[{
 "Shift does not affect \[Zeta], y and \[Eta]0 which are constants by their \
meaning (\[Zeta] is MZV, \[Eta]0 is \[Eta] at ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    SqrtBox["12"]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ", y is Zhukovsky variable at Bethe root):"
}], "Text",
 CellChangeTimes->{{3.569018367683251*^9, 3.5690183887446337`*^9}, {
  3.56901844223094*^9, 3.569018473077957*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "[", "1", "]"}], "[", "0", "]"}], ",", 
   RowBox[{
    RowBox[{"y$h", "[", "1", "]"}], "[", "1", "]"}], ",", 
   RowBox[{"\[Eta]0", "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{"\[Eta]", "[", 
     RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", "3", "]"}]}], 
  "}"}], "\[IndentingNewLine]", 
 RowBox[{"%", "/.", 
  RowBox[{"Shift", "[", "2", "]"}]}]}], "Input",
 CellChangeTimes->{{3.56901838947159*^9, 3.5690184331067467`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], ",", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["y",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "1", "+"], ",", 
   SuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], 
    RowBox[{"[", "2", "]"}]], ",", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"1", ",", "2", ",", "3"}], 
    RowBox[{"[", "3", "]"}]]}], "}"}]], "Output",
 CellChangeTimes->{{3.5690184067006607`*^9, 3.569018433475423*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], ",", 
   SubsuperscriptBox[
    OverscriptBox[
     StyleBox["y",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], "1", "+"], ",", 
   SuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], 
    RowBox[{"[", "2", "]"}]], ",", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"1", ",", "2", ",", "3"}], 
    RowBox[{"[", "5", "]"}]]}], "}"}]], "Output",
 CellChangeTimes->{{3.5690184067006607`*^9, 3.569018433482697*^9}}]
}, Open  ]],

Cell["\<\
The second operation is complex conjugation:\
\>", "Text",
 CellChangeTimes->{{3.569018488166195*^9, 3.569018495972498*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["q", 
    RowBox[{"2", ",", "3"}]], ",", 
   RowBox[{"x$h", "[", "2", "]"}], ",", 
   RowBox[{"x$", "[", "2", "]"}]}], "}"}], "\[IndentingNewLine]", 
 RowBox[{"%", "/.", "mconj"}]}], "Input",
 CellChangeTimes->{{3.569018498041895*^9, 3.569018523813822*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox[
    StyleBox["q",
     FontColor->RGBColor[0, 0, 0.8]], 
    RowBox[{"2", ",", "3"}]], ",", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], 
    RowBox[{"[", "2", "]"}]], ",", 
   SuperscriptBox[
    StyleBox["x",
     FontColor->RGBColor[0, 0.6, 0]], 
    RowBox[{"[", "2", "]"}]]}], "}"}]], "Output",
 CellChangeTimes->{{3.569018505201466*^9, 3.5690185240639477`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox[
    OverscriptBox[
     StyleBox["q",
      FontColor->RGBColor[0, 0, 0.8]], 
     StyleBox["_",
      FontColor->RGBColor[0, 0, 0.8]]], 
    RowBox[{"2", ",", "3"}]], ",", 
   SuperscriptBox[
    OverscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     StyleBox["^",
      FontColor->RGBColor[0, 0.6, 0]]], 
    RowBox[{"[", 
     RowBox[{"-", "2"}], "]"}]], ",", 
   FractionBox["1", 
    SuperscriptBox[
     StyleBox["x",
      FontColor->RGBColor[0, 0.6, 0]], 
     RowBox[{"[", 
      RowBox[{"-", "2"}], "]"}]]]}], "}"}]], "Output",
 CellChangeTimes->{{3.569018505201466*^9, 3.569018524068634*^9}}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[" \[Eta]- and \[Zeta]-functions", "Section",
 CellChangeTimes->{{3.5686963458116283`*^9, 3.568696363145094*^9}, 
   3.5690160386151114`*^9}],

Cell[CellGroupData[{

Cell["\[Eta] functions, notation", "Subsection",
 CellChangeTimes->{{3.568694006449274*^9, 3.5686940086527557`*^9}, {
  3.568696093545659*^9, 3.568696111743641*^9}, {3.569016051469844*^9, 
  3.5690160546598463`*^9}}],

Cell[CellGroupData[{

Cell["\[Eta] - function", "Subsubsection",
 CellChangeTimes->{{3.568694532991096*^9, 3.5686945426017942`*^9}}],

Cell["\<\
The standard definition (10) of \[Eta] functions read\
\>", "Text",
 CellChangeTimes->{{3.568693624993126*^9, 3.5686936389750767`*^9}, {
  3.56869366954272*^9, 3.568693672477797*^9}, {3.5686937472736053`*^9, 
  3.5686937772476997`*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "...", ",", 
    SubscriptBox["a", "k"]}]], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sum]", 
      RowBox[{
       RowBox[{
        RowBox[{"0", "\[LessEqual]", 
         SubscriptBox["n", "1"], "<", 
         SubscriptBox["n", "2"], "<"}], "..."}], "<", "\[Infinity]"}]], 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"I", " ", 
           SubscriptBox["n", "1"]}]}], ")"}], 
        SubscriptBox["a", "1"]]], 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"I", " ", 
           SubscriptBox["n", "2"]}]}], ")"}], 
        SubscriptBox["a", "2"]]]}]}], "..."}], 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"I", " ", 
        SubscriptBox["n", "k"]}]}], ")"}], 
     SubscriptBox["a", "k"]]]}]}]], "DisplayFormula",
 CellChangeTimes->{{3.56869378377785*^9, 3.568693860423718*^9}, {
  3.568694347364859*^9, 3.56869434782889*^9}}],

Cell[TextData[{
 "The \[Eta]-function ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "2", ",", "3"}]]],
  CellChangeTimes->{{3.5686939370961246`*^9, 3.5686939798330393`*^9}}],
 " can be entered in the three equivelent ways below :"
}], "Text",
 CellChangeTimes->{{3.568693896139995*^9, 3.568693944032043*^9}, {
   3.568693989809064*^9, 3.56869399066432*^9}, 3.568694042329105*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Eta]$", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", "0", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eta", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", "0", 
  "]"}], "\[IndentingNewLine]", 
 SubscriptBox[
  StyleBox["\[Eta]",
   FontWeight->Bold,
   FontColor->RGBColor[0.8, 0, 0]], 
  RowBox[{"1", ",", "2", ",", "3"}]]}], "Input",
 CellChangeTimes->{{3.5686939370961246`*^9, 3.5686939798330393`*^9}}],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " internally stores this object as \[Eta]$[1,2,3][0]"
}], "Text",
 CellChangeTimes->{{3.568695247021839*^9, 3.568695269883648*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["shifted \[Eta] - function", "Subsubsection",
 CellChangeTimes->{{3.568694532991096*^9, 3.568694550235756*^9}}],

Cell["\<\
The argument of the \[Eta] function can be shifted, folowing the notation\
\>", "Text",
 CellChangeTimes->{{3.568694170858671*^9, 3.568694174026062*^9}, {
  3.568694206476108*^9, 3.568694239247299*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "...", ",", 
    SubscriptBox["a", "k"]}], 
   RowBox[{"[", "s", "]"}]], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sum]", 
      RowBox[{
       RowBox[{
        RowBox[{"0", "\[LessEqual]", 
         SubscriptBox["n", "1"], "<", 
         SubscriptBox["n", "2"], "<"}], "..."}], "<", "\[Infinity]"}]], 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"I", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["n", "1"], "+", 
             FractionBox["s", "2"]}], ")"}]}]}], ")"}], 
        SubscriptBox["a", "1"]]], 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"I", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["n", "2"], "+", 
             FractionBox["s", "2"]}], ")"}]}]}], ")"}], 
        SubscriptBox["a", "2"]]]}]}], "..."}], 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"I", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "k"], "+", 
          FractionBox["s", "2"]}], ")"}]}]}], ")"}], 
     SubscriptBox["a", "k"]]]}]}]], "DisplayFormula",
 CellChangeTimes->{{3.568696309776065*^9, 3.56869631920601*^9}}],

Cell[TextData[{
 "This shifted \[Eta]-function ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "2", ",", "3"}], 
   RowBox[{"[", "5", "]"}]]],
  CellChangeTimes->{{3.5686939370961246`*^9, 3.5686939798330393`*^9}}],
 " can be entered in the three equivelent ways below :"
}], "Text",
 CellChangeTimes->{{3.568693896139995*^9, 3.568693944032043*^9}, {
   3.568693989809064*^9, 3.56869399066432*^9}, 3.568694042329105*^9, {
   3.568694477077594*^9, 3.5686945023607903`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Eta]$", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", "5", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eta", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", "5", 
  "]"}], "\[IndentingNewLine]", 
 SubsuperscriptBox[
  StyleBox["\[Eta]",
   FontWeight->Bold,
   FontColor->RGBColor[0.8, 0, 0]], 
  RowBox[{"1", ",", "2", ",", "3"}], 
  RowBox[{"[", "5", "]"}]]}], "Input",
 CellChangeTimes->{{3.5686939370961246`*^9, 3.5686939798330393`*^9}, {
  3.5686944918133707`*^9, 3.5686944992161694`*^9}}],

Cell["\<\
Mathematica internally stores this object as \[Eta]$[1, 2, 3][5]\
\>", "Text",
 CellChangeTimes->{{3.568695275352754*^9, 3.56869528210784*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["generalized \[Eta] - function", "Subsubsection",
 CellChangeTimes->{{3.568694532991096*^9, 3.568694557619184*^9}}],

Cell["\<\
Generalized \[Eta]-functions are an intermediate object, used internally for \
some computations, such as integrals.
They are defined by the formula (16), which reads\
\>", "Text",
 CellChangeTimes->{{3.5686946285265293`*^9, 3.568694648684648*^9}, {
  3.5686947082957907`*^9, 3.568694731232882*^9}, {3.56869491561841*^9, 
  3.5686949562091637`*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "...", ",", 
    SubscriptBox["a", "k"]}], 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["v", "1"], ",", 
     SubscriptBox["v", 
      RowBox[{"2", ",", "...", ",", 
       SubscriptBox["v", "k"]}]]}], "}"}]], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sum]", 
      RowBox[{
       RowBox[{
        RowBox[{"0", "\[LessEqual]", 
         SubscriptBox["n", "1"], "<", 
         SubscriptBox["n", "2"], "<"}], "..."}], "<", "\[Infinity]"}]], 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", 
           SubscriptBox["n", "1"]}], "-", 
          SubscriptBox["v", "1"]}], ")"}], 
        SubscriptBox["a", "1"]]], 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", 
           SubscriptBox["n", "2"]}], "-", 
          SubscriptBox["v", "2"]}], ")"}], 
        SubscriptBox["a", "2"]]]}]}], "..."}], 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["n", "k"]}], "-", 
       SubscriptBox["v", "3"]}], ")"}], 
     SubscriptBox["a", "k"]]]}]}]], "DisplayFormula",
 CellChangeTimes->{{3.5686962678033857`*^9, 3.568696278874515*^9}}],

Cell["Internally, this package uses the following notation instead", "Text",
 CellChangeTimes->{{3.56869488634024*^9, 3.56869491033838*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "...", ",", 
    SubscriptBox["a", "k"]}], 
   RowBox[{"[", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["s", "1"], ",", 
      SubscriptBox["s", 
       RowBox[{"2", ",", "...", ",", 
        SubscriptBox["s", "k"]}]]}], "}"}], "]"}]], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sum]", 
      RowBox[{
       RowBox[{
        RowBox[{"0", "\[LessEqual]", 
         SubscriptBox["n", "1"], "<", 
         SubscriptBox["n", "2"], "<"}], "..."}], "<", "\[Infinity]"}]], 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["n", "1"], "+", 
             FractionBox[
              SubscriptBox["s", "1"], "2"]}], ")"}]}]}], ")"}], 
        SubscriptBox["a", "1"]]], 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["n", "2"], "+", 
             FractionBox[
              SubscriptBox["s", "2"], "2"]}], ")"}]}]}], ")"}], 
        SubscriptBox["a", "2"]]]}]}], "..."}], 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "k"], "+", 
          FractionBox[
           SubscriptBox["s", "k"], "2"]}], ")"}]}]}], ")"}], 
     SubscriptBox["a", "k"]]]}]}]], "DisplayFormula",
 CellChangeTimes->{{3.5686962892569103`*^9, 3.568696300280842*^9}}],

Cell[TextData[{
 "The Generalized \[Eta] - function ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "2", ",", "3"}], 
   RowBox[{"[", 
    RowBox[{"{", 
     RowBox[{"0", ",", "2", ",", "0"}], "}"}], "]"}]]],
  CellChangeTimes->{3.568694688393715*^9}],
 "can then be entered in the four ",
 "equivelent ways below :"
}], "Text",
 CellChangeTimes->{{3.568695012304942*^9, 3.568695052437594*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Eta]$", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "2", ",", "0"}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eta", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "2", ",", "0"}], "}"}], "]"}], "\[IndentingNewLine]", 
 SubsuperscriptBox[
  StyleBox["\[Eta]",
   FontWeight->Bold,
   FontColor->RGBColor[0.8, 0, 0]], 
  RowBox[{"1", ",", "2", ",", "3"}], 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"0", ",", "2", ",", "0"}], "}"}], "]"}]], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GeneralizedEta", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"-", "I"}], ",", "0"}], "}"}], "]"}], "[", "]"}]}], "Input",
 CellChangeTimes->{{3.568694681376299*^9, 3.568694687760166*^9}, {
   3.568695058770843*^9, 3.568695137704288*^9}, 3.568695194149653*^9}],

Cell["\<\
Mathematica internally stores this object as \[Eta]$[1, 2, 3][{0,2,0}]\
\>", "Text",
 CellChangeTimes->{{3.5686952885918827`*^9, 3.568695304463874*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["argument of the \[Eta] - function", "Subsubsection",
 CellChangeTimes->{{3.568694532991096*^9, 3.568694569115382*^9}}],

Cell[TextData[{
 "In the above examples the argument of the \[Eta]-functions is implicitely \
equal to \[OpenCurlyDoubleQuote]u\[CloseCurlyDoubleQuote].\nOne can sometimes \
have to specify the argument, for instance when one wishes to specify that \
the generalized \[Eta]-function ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "2", ",", "3"}], 
   RowBox[{"[", 
    RowBox[{"{", 
     RowBox[{"0", ",", "2", ",", "0"}], "}"}], "]"}]]],
  CellChangeTimes->{
   3.568694688393715*^9, {3.5686951265349092`*^9, 3.568695138253285*^9}, {
    3.5686951898092813`*^9, 3.568695194577715*^9}}],
 " should be evaluated at position u=\[ImaginaryI]. To this end, one can set \
the last argument of the function GeneralizedEta.\n\nFor instance the \
function ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "2", ",", "3"}], 
   RowBox[{"{", 
    RowBox[{"0", ",", "v", ",", "0"}], "}"}]]], "Input",
  CellChangeTimes->{{3.5686943014853687`*^9, 3.5686944042180634`*^9}, {
   3.5686944392479267`*^9, 3.568694459536045*^9}, {3.568694749138173*^9, 
   3.568694800996316*^9}, {3.568694833958826*^9, 3.568694875912271*^9}, {
   3.56869547785707*^9, 3.568695479186859*^9}}],
 ", evaluated at point u=I, is entered as"
}], "Text",
 CellChangeTimes->{{3.568695317244562*^9, 3.568695457588315*^9}, {
  3.568695493223123*^9, 3.568695612100019*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GeneralizedEta", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
   RowBox[{"{", 
    RowBox[{"0", ",", "v", ",", "0"}], "}"}], "]"}], "[", "I", "]"}]], "Input",\

 CellChangeTimes->{{3.568695626614295*^9, 3.56869562837249*^9}}],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not display a nicely formated output for this quantity, because this \
intermediate quantity should never appear in the result of a computation, and \
it should always simplify to a more standard quantity."
}], "Text",
 CellChangeTimes->{{3.568695644916297*^9, 3.568695705977344*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
MZV (alias \[Zeta]-functions or Euler-Zagier sums), notation\
\>", "Subsection",
 CellChangeTimes->{{3.568694013621009*^9, 3.5686940168526382`*^9}, {
  3.568694080997748*^9, 3.5686941029484987`*^9}, {3.569016058150051*^9, 
  3.569016060243334*^9}}],

Cell["\<\
The MZV are related to \[Eta] functions through the relation (11) :\
\>", "Text",
 CellChangeTimes->{{3.568693624993126*^9, 3.5686936389750767`*^9}, {
  3.56869366954272*^9, 3.568693672477797*^9}, {3.5686937472736053`*^9, 
  3.5686937772476997`*^9}, {3.56869406223063*^9, 3.5686940645093193`*^9}, {
  3.568694111219677*^9, 3.568694131507586*^9}, {3.568695766740934*^9, 
  3.56869579612471*^9}}],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "...", ",", 
    SubscriptBox["a", "k"]}]], "=", 
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SubscriptBox["a", "1"]}], "-", 
       SubscriptBox["a", "2"], "-"}], "..."}], "-", 
     SubscriptBox["a", "k"]}]], 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{
     SubscriptBox["a", "1"], ",", 
     SubscriptBox["a", "2"], ",", "...", ",", 
     SubscriptBox["a", "k"]}]], 
   RowBox[{"(", "\[ImaginaryI]", ")"}]}]}]], "DisplayFormula"],

Cell[TextData[{
 "For instance the \[Zeta]-function ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"1", ",", "3"}]]],
  CellChangeTimes->{3.568695960645444*^9}],
 " can be entered in the four equivalent ways below :"
}], "Text",
 CellChangeTimes->{{3.568695924682797*^9, 3.568695970867468*^9}, {
   3.568696058298126*^9, 3.5686960616635237`*^9}, 3.5690152534855947`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Zeta]$", "[", 
   RowBox[{"1", ",", "3"}], "]"}], "[", "0", "]"}], "\[IndentingNewLine]", 
 SubscriptBox[
  StyleBox["\[Zeta]",
   FontWeight->Bold,
   FontColor->GrayLevel[0.4]], 
  RowBox[{"1", ",", "3"}]], "\[IndentingNewLine]", 
 RowBox[{"zeta", "[", 
  RowBox[{"1", ",", "3"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "3"}], 
   RowBox[{"[", "2", "]"}]], "/.", "sub\[Eta]\[Zeta]"}]}], "Input",
 CellChangeTimes->{{3.5686959536136703`*^9, 3.568696031451742*^9}}],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " internally stores this object as ",
 "\[Zeta]$[1,3][0]"
}], "Text",
 CellChangeTimes->{{3.568695247021839*^9, 3.568695269883648*^9}, 
   3.568696073484613*^9}],

Cell["In the example", "Text",
 CellChangeTimes->{{3.56869611608403*^9, 3.568696123928814*^9}}],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"1", ",", "3"}], 
   RowBox[{"[", "2", "]"}]], "/.", "sub\[Eta]\[Zeta]"}]], "Input"],

Cell["\<\
the substitution sub\[Eta]\[Zeta] specifies that the implicit argument u of \
this \[Eta]-function is set to zero. Then, this \[Eta]-function becomes a \
\[Zeta]-function\
\>", "Text",
 CellChangeTimes->{{3.568696132344357*^9, 3.568696179261752*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Relations among \[Zeta]-functions", "Subsection",
 CellChangeTimes->{{3.568696367201199*^9, 3.5686963751789722`*^9}}],

Cell[CellGroupData[{

Cell["Shuffle relation", "Subsubsection",
 CellChangeTimes->{{3.5686964546308928`*^9, 3.5686964572299557`*^9}}],

Cell[TextData[{
 "the substitution ",
 StyleBox["subShuffle",
  FontWeight->"Bold"],
 " implements the shuffle relations.\nFor instance, you can run"
}], "Text",
 CellChangeTimes->{{3.568696494766191*^9, 3.5686965740183907`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "2"}]]}], "/.", 
  StyleBox["subShuffle",
   FontWeight->"Bold"]}]], "Input",
 CellChangeTimes->{{3.568696558398024*^9, 3.5686965613035088`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"10", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "2", ",", "6"}]]}], "+", 
  RowBox[{"16", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "3", ",", "5"}]]}], "+", 
  RowBox[{"18", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "4", ",", "4"}]]}], "+", 
  RowBox[{"16", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "5", ",", "3"}]]}], "+", 
  RowBox[{"10", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "6", ",", "2"}]]}], "+", 
  RowBox[{"10", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "1", ",", "6"}]]}], "+", 
  RowBox[{"9", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "2", ",", "5"}]]}], "+", 
  RowBox[{"9", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "3", ",", "4"}]]}], "+", 
  RowBox[{"8", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "4", ",", "3"}]]}], "+", 
  RowBox[{"5", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "5", ",", "2"}]]}], "+", 
  RowBox[{"3", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"3", ",", "2", ",", "4"}]]}], "+", 
  RowBox[{"4", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"3", ",", "3", ",", "3"}]]}], "+", 
  RowBox[{"3", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"3", ",", "4", ",", "2"}]]}], "+", 
  RowBox[{"2", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"4", ",", "2", ",", "3"}]]}], "+", 
  RowBox[{"2", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"4", ",", "3", ",", "2"}]]}], "+", 
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"5", ",", "2", ",", "2"}]]}]], "Output",
 CellChangeTimes->{3.568696610417577*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Stuffle relations", "Subsubsection",
 CellChangeTimes->{{3.5686964610860033`*^9, 3.568696465349577*^9}, 
   3.56950982189832*^9}],

Cell[TextData[{
 "the substitution ",
 StyleBox["subS",
  FontWeight->"Bold"],
 "tuf",
 StyleBox["fle",
  FontWeight->"Bold"],
 " implements the shuffle relations.\nFor instance, you can run"
}], "Text",
 CellChangeTimes->{{3.568696494766191*^9, 3.5686965740183907`*^9}, 
   3.5686966043136683`*^9, 3.569509823868496*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "2"}]]}], "/.", 
  StyleBox["subStuffle",
   FontWeight->"Bold"]}]], "Input",
 CellChangeTimes->{{3.568696558398024*^9, 3.5686965613035088`*^9}, 
   3.5686966055064697`*^9, 3.569509825032526*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"2", ",", "7"}]], "+", 
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"7", ",", "2"}]], "+", 
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"2", ",", "2", ",", "5"}]], "+", 
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"2", ",", "5", ",", "2"}]], "+", 
  SubscriptBox[
   StyleBox["\[Zeta]",
    FontWeight->Bold,
    FontColor->GrayLevel[0.4]], 
   RowBox[{"5", ",", "2", ",", "2"}]]}]], "Output",
 CellChangeTimes->{3.568696608181354*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Simplifications of \[Zeta]-functions", "Subsubsection",
 CellChangeTimes->{{3.5686964686856117`*^9, 3.568696482565147*^9}}],

Cell["\<\
Not all \[Zeta]-functions are independant. The substitution subZetaReduce can \
be used to simplifies combinations of \[Zeta]-functions.

Two examples are provided below :\
\>", "Text",
 CellChangeTimes->{{3.5686963778975353`*^9, 3.5686963938718557`*^9}, {
  3.5686966389737663`*^9, 3.568696639437993*^9}, {3.568696681109943*^9, 
  3.56869674689025*^9}, {3.568696829647133*^9, 3.56869688248234*^9}, {
  3.568697163821829*^9, 3.568697170912198*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "4", ",", "3"}]], "//.", "subZetaReduce"}]}]], "Input",
 CellChangeTimes->{{3.568697135899433*^9, 3.56869714049223*^9}, {
  3.568697241537767*^9, 3.568697260022587*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    FractionBox["8", "15"]}], " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "3"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"]}], "+", 
  FractionBox[
   RowBox[{"2", " ", 
    SubsuperscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3", "3"]}], "3"], "+", 
  RowBox[{"3", " ", 
   SubsuperscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2", "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"]}], "-", 
  RowBox[{"14", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "7"]}], "+", 
  FractionBox[
   RowBox[{"593", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "9"]}], "36"]}]], "Output",
 CellChangeTimes->{
  3.568697140650621*^9, {3.568697254533126*^9, 3.568697260483333*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"10", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"1", ",", "2", ",", "6"}]]}], "+", 
    RowBox[{"16", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"1", ",", "3", ",", "5"}]]}], "+", 
    RowBox[{"18", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"1", ",", "4", ",", "4"}]]}], "+", 
    RowBox[{"16", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"1", ",", "5", ",", "3"}]]}], "+", 
    RowBox[{"10", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"1", ",", "6", ",", "2"}]]}], "+", 
    RowBox[{"10", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"2", ",", "1", ",", "6"}]]}], "+", 
    RowBox[{"9", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"2", ",", "2", ",", "5"}]]}], "+", 
    RowBox[{"9", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"2", ",", "3", ",", "4"}]]}], "+", 
    RowBox[{"8", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"2", ",", "4", ",", "3"}]]}], "+", 
    RowBox[{"5", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"2", ",", "5", ",", "2"}]]}], "+", 
    RowBox[{"3", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"3", ",", "2", ",", "4"}]]}], "+", 
    RowBox[{"4", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"3", ",", "3", ",", "3"}]]}], "+", 
    RowBox[{"3", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"3", ",", "4", ",", "2"}]]}], "+", 
    RowBox[{"2", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"4", ",", "2", ",", "3"}]]}], "+", 
    RowBox[{"2", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"4", ",", "3", ",", "2"}]]}], "+", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"5", ",", "2", ",", "2"}]], "-", 
    RowBox[{
     FractionBox["3", "10"], " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "2"], " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "5"]}]}], "//.", "subZetaReduce"}], "//", 
  "Expand"}]], "Input",
 CellChangeTimes->{{3.568696779772189*^9, 3.5686967957486343`*^9}, {
   3.568696911037603*^9, 3.568696919243247*^9}, {3.568696967880089*^9, 
   3.568696968820444*^9}, {3.568697147711207*^9, 3.5686971575975323`*^9}, 
   3.568697245312592*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.5686969198364267`*^9, 3.5686969691953573`*^9, {3.568697148650021*^9, 
   3.568697157702142*^9}}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Simplification of (Generalized) \[Eta]-function (appendix A.1 and A.2)\
\>", "Subsection",
 CellChangeTimes->{{3.568697297145286*^9, 3.5686973432425213`*^9}, {
  3.568697561134836*^9, 3.568697572396408*^9}}],

Cell[TextData[{
 "The simplifications that are needed on \[Eta]-functions are provided by the \
command ",
 StyleBox["EndSimplify",
  FontWeight->"Bold"]
}], "Text",
 CellChangeTimes->{{3.568697575775895*^9, 3.568697603585178*^9}}],

Cell["\<\
an example is provided below, corresponding to the equation (A.4) :

\
\>", "Text",
 CellChangeTimes->{{3.568697612403098*^9, 3.568697620323186*^9}, {
  3.568699969820003*^9, 3.5687000035710897`*^9}}],

Cell["\<\
the code bellow allows to obtain the relation (A.4) from the mathematica \
package\
\>", "Text",
 CellChangeTimes->{{3.568697875504202*^9, 3.568697897103325*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"GeneralizedEta", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "u"}], ",", "0"}], "}"}], "]"}], "[", "I", "]"}], "/.", 
    "EndSimplify"}], "//.", 
   RowBox[{"shiftmeta", "[", "2", "]"}]}], "//", "FuntoMHZbasis"}]], "Input",
 CellChangeTimes->{{3.5686980776203337`*^9, 3.5686980902322063`*^9}, {
  3.569015381489738*^9, 3.5690153886990643`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], 
   SuperscriptBox["u", "2"]], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "-", 
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"2", ",", "2"}]], "-", 
  RowBox[{"2", " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"3", ",", "1"}]]}]}]], "Output",
 CellChangeTimes->{{3.5686980790517073`*^9, 3.568698091017653*^9}, {
  3.5690153856939173`*^9, 3.569015389248931*^9}}]
}, Open  ]],

Cell["let us detail the functions that appear there :", "Text",
 CellChangeTimes->{{3.568697906519524*^9, 3.5686979159989033`*^9}}],

Cell[TextData[{
 StyleBox["GeneralizedEta[2, 2][{-u, 0}][I]",
  FontWeight->"Bold"],
 " defines the left-hand side of (A.4), up to the notation ",
 StyleBox["u\[Congruent]-v",
  FontWeight->"Bold"],
 ". Then ",
 StyleBox["EndSimplify",
  FontWeight->"Bold"],
 " transformed the expression into the following standardized output"
}], "Text",
 CellChangeTimes->{{3.568697928993659*^9, 3.568698058560625*^9}, {
  3.5686981401910973`*^9, 3.568698209546651*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GeneralizedEta", "[", 
     RowBox[{"2", ",", "2"}], "]"}], "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "u"}], ",", "0"}], "}"}], "]"}], "[", "I", "]"}], "/.", 
  "EndSimplify"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "1"]}], 
    SuperscriptBox["u", "3"]]}], "+", 
  FractionBox[
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], 
   SuperscriptBox["u", "2"]], "-", 
  FractionBox[
   RowBox[{"2", " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "1", 
     RowBox[{"[", "2", "]"}]]}], 
   SuperscriptBox["u", "3"]], "-", 
  FractionBox[
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2", 
    RowBox[{"[", "2", "]"}]], 
   SuperscriptBox["u", "2"]], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubsuperscriptBox[
      StyleBox["\[Eta]",
       FontWeight->Bold,
       FontColor->RGBColor[0.8, 0, 0]], "2", 
      RowBox[{"[", "2", "]"}]], ")"}], "2"]}], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3", 
    RowBox[{"[", "2", "]"}]]}], "-", 
  RowBox[{"2", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "1", 
    RowBox[{"[", "2", "]"}]], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3", 
    RowBox[{"[", "2", "]"}]]}], "+", 
  FractionBox[
   RowBox[{"5", " ", 
    SubsuperscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "4", 
     RowBox[{"[", "2", "]"}]]}], "2"], "+", 
  RowBox[{"2", " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"1", ",", "3"}], 
    RowBox[{"[", "2", "]"}]]}]}]], "Output",
 CellChangeTimes->{3.568698194300167*^9}]
}, Open  ]],

Cell[TextData[{
 "In the present case, the result can be made more compact by using the \
relation (13) to express all the shifted functions ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], "I", 
   RowBox[{"[", "2", "]"}]]],
  CellChangeTimes->{3.568698194300167*^9}],
 " in terms of functions ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], "I"]],
  CellChangeTimes->{3.568698194300167*^9}],
 " without shift. The substitution ",
 StyleBox["shiftmeta[2]",
  FontWeight->"Bold"],
 " (which means \[CloseCurlyDoubleQuote]decrease the shift all \
\[Eta]-functions that have shift 2\[CloseCurlyDoubleQuote]) does this. The \
substitution inverse to  ",
 StyleBox["shiftmeta[2]",
  FontWeight->"Bold"],
 " is ",
 StyleBox["shiftpeta[0]",
  FontWeight->"Bold"],
 ".",
 StyleBox["\n",
  FontWeight->"Bold"],
 "After the substitution ",
 StyleBox["shiftmeta[2]",
  FontWeight->"Bold"],
 ", we obtain:"
}], "Text",
 CellChangeTimes->{{3.568698218302864*^9, 3.56869823262272*^9}, {
  3.568698706068536*^9, 3.568698947790689*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"GeneralizedEta", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "u"}], ",", "0"}], "}"}], "]"}], "[", "I", "]"}], "/.", 
    "EndSimplify"}], "//.", 
   RowBox[{"shiftmeta", "[", "2", "]"}]}], "//", "Expand"}]], "Input",
 CellChangeTimes->{3.5687000265358*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], 
   SuperscriptBox["u", "2"]], "-", 
  FractionBox[
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2", "2"], "2"], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "-", 
  RowBox[{"2", " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "+", 
  FractionBox[
   RowBox[{"5", " ", 
    SubscriptBox[
     StyleBox["\[Eta]",
      FontWeight->Bold,
      FontColor->RGBColor[0.8, 0, 0]], "4"]}], "2"], "+", 
  RowBox[{"2", " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"1", ",", "3"}]]}]}]], "Output",
 CellChangeTimes->{3.568698956668787*^9, 3.5687000277913446`*^9}]
}, Open  ]],

Cell[TextData[{
 "The result can still be made a little more compact by using the stuffle \
relation to express ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}]],
  CellChangeTimes->{3.568698956668787*^9}],
 " as ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "4"], "+", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"1", ",", "3"}]], "+", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"3", ",", "1"}]]}]],
  CellChangeTimes->{3.568698956668787*^9}],
 ". The function ",
 StyleBox["FuntoMHZbasis",
  FontWeight->"Bold"],
 " systematically applies s",
 "tuf",
 "fle relations until no products of \[Eta]-functions remain."
}], "Text",
 CellChangeTimes->{{3.568698970569738*^9, 3.568699171921022*^9}, {
  3.569015481901978*^9, 3.5690155689430027`*^9}, {3.569509826230811*^9, 
  3.569509827964764*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"GeneralizedEta", "[", 
       RowBox[{"2", ",", "2"}], "]"}], "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "u"}], ",", "0"}], "}"}], "]"}], "[", "I", "]"}], "/.", 
    "EndSimplify"}], "//.", 
   RowBox[{"shiftmeta", "[", "2", "]"}]}], "//", "FuntoMHZbasis"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], 
   SuperscriptBox["u", "2"]], "-", 
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "1"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "-", 
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"2", ",", "2"}]], "-", 
  RowBox[{"2", " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], 
    RowBox[{"3", ",", "1"}]]}]}]], "Output",
 CellChangeTimes->{3.568699179778331*^9}]
}, Open  ]],

Cell["\<\
which is exactly the relation (A.4)\
\>", "Text",
 CellChangeTimes->{{3.568699187432705*^9, 3.568699195959258*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Integration", "Subsection",
 CellChangeTimes->{{3.5687000985262737`*^9, 3.568700100357398*^9}}],

Cell["\<\
The function UpIntegrate iteratively computes integrals of the form (14).
It takes two arguments : the integrand and the integration variable. The \
integration contour is always supposed to be the real axis, which is closed \
upwards at infinity.\
\>", "Text",
 CellChangeTimes->{{3.568700106773642*^9, 3.5687002029222307`*^9}}],

Cell[TextData[{
 "we will explain, below, how one asks mathematica to compute the integral \
(14):\n\n\nIn order to compute this integral, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs to know the real part of ",
 StyleBox["v",
  FontWeight->"Bold"],
 ", to know the position of the pole with respect to the integration contour. \
We will chose ",
 StyleBox["Im[v] \[Congruent] \[Epsilon] > 0",
  FontWeight->"Bold"],
 ".  One simple way to do this is by replacing ",
 StyleBox["v",
  FontWeight->"Bold"],
 " with ",
 StyleBox["Re[u]+\[ImaginaryI] \[Epsilon]",
  FontWeight->"Bold"],
 ", (because the package will be more at ease with simplifying that contain \
",
 StyleBox["u",
  FontWeight->"Bold"],
 ", rather thatn ",
 StyleBox["v",
  FontWeight->"Bold"],
 "). We then denote by ",
 StyleBox["w",
  FontWeight->"Bold"],
 " the integration variable:"
}], "Text",
 CellChangeTimes->{{3.5687002706559877`*^9, 3.568700335008955*^9}, {
  3.568700377231502*^9, 3.5687005038550386`*^9}, {3.5687006373233547`*^9, 
  3.568700757227812*^9}, {3.568700797240596*^9, 3.568700841505402*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"UpIntegrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "/", 
      RowBox[{"(", 
       RowBox[{"2", "*", "Pi", "*", "I", "*", 
        RowBox[{"(", 
         RowBox[{"w", "-", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Re", "[", "u", "]"}], "+", 
            RowBox[{"I", " ", "\[Epsilon]"}]}], ")"}]}], ")"}]}], ")"}]}], 
     "*", 
     RowBox[{
      RowBox[{"etab", "[", "3", "]"}], "[", 
      RowBox[{"-", "2"}], "]"}], "*", 
     RowBox[{
      RowBox[{"eta", "[", "2", "]"}], "[", "2", "]"}]}], ",", "w"}], "]"}], "/.",
   "EndSimplify"}]], "Input",
 CellChangeTimes->{{3.5687007916536922`*^9, 3.568700793114395*^9}, {
  3.568700862235277*^9, 3.568700864698989*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], 
   SuperscriptBox["u", "3"]], "-", 
  FractionBox[
   RowBox[{"6", " ", 
    SubsuperscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "2", "2"]}], 
   RowBox[{"5", " ", "u"}]], "+", 
  FractionBox[
   RowBox[{"2", " ", "\[ImaginaryI]", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox["u", "2"]], "+", 
  FractionBox[
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2", 
    RowBox[{"[", "2", "]"}]], 
   SuperscriptBox["u", "3"]], "+", 
  RowBox[{"3", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2", 
    RowBox[{"[", "2", "]"}]]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3", 
    RowBox[{"[", "2", "]"}]]}], "+", 
  RowBox[{
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2", 
    RowBox[{"[", "2", "]"}]], " ", 
   SubsuperscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3", 
    RowBox[{"[", "2", "]"}]]}], "-", 
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], "5", 
   RowBox[{"[", "2", "]"}]], "-", 
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"2", ",", "3"}], 
   RowBox[{"[", "2", "]"}]]}]], "Output",
 CellChangeTimes->{{3.568700785955949*^9, 3.568700793566815*^9}, 
   3.5687008650317373`*^9}]
}, Open  ]],

Cell["\<\
like in previous example, this result can be made more compact by using the \
relation (13), which is contained in the substitution shiftmeta[2]. We obtain\
\
\>", "Text",
 CellChangeTimes->{{3.56870088772254*^9, 3.5687009537033873`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"UpIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "1"}], "/", 
        RowBox[{"(", 
         RowBox[{"2", "*", "Pi", "*", "I", "*", 
          RowBox[{"(", 
           RowBox[{"w", "-", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Re", "[", "u", "]"}], "+", 
              RowBox[{"I", " ", "\[Epsilon]"}]}], ")"}]}], ")"}]}], ")"}]}], 
       "*", 
       RowBox[{
        RowBox[{"etab", "[", "3", "]"}], "[", 
        RowBox[{"-", "2"}], "]"}], "*", 
       RowBox[{
        RowBox[{"eta", "[", "2", "]"}], "[", "2", "]"}]}], ",", "w"}], "]"}], 
    "/.", "EndSimplify"}], "//.", 
   RowBox[{"shiftmeta", "[", "2", "]"}]}], "//", "Expand"}]], "Input",
 CellChangeTimes->{{3.5687009726775093`*^9, 3.568700987010356*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"6", " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "2"]}], 
    RowBox[{"5", " ", "u"}]]}], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox["u", "2"]], "+", 
  RowBox[{"3", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2"]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "-", 
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], "5"], "-", 
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"2", ",", "3"}]]}]], "Output",
 CellChangeTimes->{{3.568700959701107*^9, 3.5687009873160887`*^9}}]
}, Open  ]],

Cell["\<\
finally, we can use the stuffle relation to make this result even more \
compact :\
\>", "Text",
 CellChangeTimes->{{3.568700994213835*^9, 3.5687010138759737`*^9}, 
   3.5690156017115097`*^9, 3.569509829297326*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rltint", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"UpIntegrate", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"-", "1"}], "/", 
         RowBox[{"(", 
          RowBox[{"2", "*", "Pi", "*", "I", "*", 
           RowBox[{"(", 
            RowBox[{"w", "-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Re", "[", "u", "]"}], "+", 
               RowBox[{"I", " ", "epsilon"}]}], ")"}]}], ")"}]}], ")"}]}], 
        "*", 
        RowBox[{
         RowBox[{"etab", "[", "3", "]"}], "[", 
         RowBox[{"-", "2"}], "]"}], "*", 
        RowBox[{
         RowBox[{"eta", "[", "2", "]"}], "[", "2", "]"}]}], ",", "w"}], "]"}],
      "/.", "EndSimplify"}], "//.", 
    RowBox[{"shiftmeta", "[", "2", "]"}]}], "//", 
   "FuntoMHZbasis"}]}]], "Input",
 CellChangeTimes->{{3.5687010241741667`*^9, 3.5687010281431017`*^9}, {
  3.5687011767771063`*^9, 3.5687011771364613`*^9}, {3.568702671072138*^9, 
  3.568702672951502*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"6", " ", 
     SubsuperscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], "2", "2"]}], 
    RowBox[{"5", " ", "u"}]]}], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "3"]}], 
   SuperscriptBox["u", "2"]], "+", 
  RowBox[{"3", " ", "\[ImaginaryI]", " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "3"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "2"]}], "+", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "2"], " ", 
   SubscriptBox[
    StyleBox["\[Eta]",
     FontWeight->Bold,
     FontColor->RGBColor[0.8, 0, 0]], "3"]}], "+", 
  SubscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"3", ",", "2"}]]}]], "Output",
 CellChangeTimes->{
  3.568701029702536*^9, {3.568701164102748*^9, 3.568701178638546*^9}, 
   3.5687026860228577`*^9}]
}, Open  ]],

Cell[TextData[{
 "To express this in the limit ",
 StyleBox["v\[Rule] 0",
  FontWeight->"Bold"],
 ", one can use the following taylor ",
 "expansion",
 "."
}], "Text",
 CellChangeTimes->{{3.5687010780205383`*^9, 3.5687011524487667`*^9}, {
  3.568701307702662*^9, 3.568701329328937*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rltint", "/.", 
   RowBox[{"TaylorOptions", "[", 
    RowBox[{"Order", "\[Rule]", "0"}], "]"}]}], "/.", 
  "EndSimplify"}]], "Input",
 CellChangeTimes->{{3.568701157442823*^9, 3.56870122819213*^9}, {
  3.56870127745674*^9, 3.568701283051734*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], "5"]}], "2"]}]], "Output",
 CellChangeTimes->{{3.568701193084199*^9, 3.568701228345016*^9}, 
   3.568701283580227*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Taylor expansions", "Subsection",
 CellChangeTimes->{{3.5686941806501637`*^9, 3.568694185361277*^9}}],

Cell[TextData[{
 "One often has to perform some taylor expansion of various quantities \
involving \[Eta]-functions.\nThe subsitution ",
 StyleBox["TaylorExpand",
  FontWeight->"Bold"],
 " performs such a taylor expansion. It has various options, which can be \
specified by\nrunning for instance ",
 StyleBox["SetOptions[Taylor,Point\[Rule]0];",
  FontWeight->"Bold"],
 "\nAn alternative way to set options is to use the substitution ",
 StyleBox["TaylorOptions",
  FontWeight->"Bold"],
 " instead of ",
 StyleBox["TaylorExpand",
  FontWeight->"Bold"],
 ", and then some options can be given as arguments.\n\n For instance, the \
command"
}], "Text",
 CellChangeTimes->{{3.568701399930724*^9, 3.568701506642419*^9}, {
   3.568701678198655*^9, 3.568701791371423*^9}, 3.569015637038329*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"3", ",", "2"}], 
   RowBox[{"[", "2", "]"}]], "/.", 
  RowBox[{"TaylorOptions", "[", 
   RowBox[{
    RowBox[{"Point", "\[Rule]", "0"}], ",", 
    RowBox[{"Order", "\[Rule]", "1"}], ",", 
    RowBox[{"Variable", "\[Rule]", "g"}], ",", 
    RowBox[{"\[Eta]Factor", "\[Rule]", 
     RowBox[{"2", " ", "z"}]}], ",", 
    RowBox[{"Series", "\[Rule]", "Keep"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.568701584343841*^9, 3.5687016584347153`*^9}, {
  3.568701852731627*^9, 3.568701856970818*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox[
      StyleBox["\[Zeta]",
       FontWeight->Bold,
       FontColor->GrayLevel[0.4]], 
      RowBox[{"3", ",", "2"}]]}]}], "+", 
   RowBox[{"2", " ", "z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], 
        RowBox[{"3", ",", "3"}]]}], "+", 
      RowBox[{"3", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], 
        RowBox[{"4", ",", "2"}]]}]}], ")"}], " ", "g"}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "2"],
    SeriesData[$CellContext`g, 0, {}, 0, 2, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
   Complex[0, -1] $CellContext`\[Zeta]$[3, 2][0], 
    2 $CellContext`z (2 $CellContext`\[Zeta]$[3, 3][0] + 
     3 $CellContext`\[Zeta]$[4, 2][0])}, 0, 2, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.568701627694304*^9, 3.5687016590454082`*^9}, 
   3.5687018102316113`*^9, 3.5687018578388443`*^9}]
}, Open  ]],

Cell[TextData[{
 "performs a taylor expansion of the function ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"3", ",", "2"}], 
   RowBox[{"[", "2", "]"}]]],
  CellChangeTimes->{{3.568701584343841*^9, 3.5687016584347153`*^9}}],
 ", where the implicit argument of the function is set to ",
 StyleBox["u\[Congruent]2 g z",
  FontWeight->"Bold"],
 " (as specified by ",
 StyleBox["Variable\[Rule]g,\[Eta]Factor\[Rule]2 z",
  FontWeight->"Bold"],
 " ), and the Taylor exansion is performed with respect to the variable ",
 StyleBox["g",
  FontWeight->"Bold"],
 ".\nThis expansion is done to the order one, arround the point ",
 StyleBox["g=0",
  FontWeight->"Bold"],
 ". The last option,",
 StyleBox[" Series\[Rule]Keep",
  FontWeight->"Bold"],
 ", means that mathematica should return a result which contains a symbol ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"+", 
    InterpretationBox[
     SuperscriptBox[
      RowBox[{"O", "[", "g", "]"}], "2"],
     SeriesData[$CellContext`g, 0, {}, 0, 2, 1],
     Editable->False]}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.568701627694304*^9, 3.5687016590454082`*^9}, 
    3.5687018102316113`*^9, 3.5687018578388443`*^9}],
 "."
}], "Text",
 CellChangeTimes->{{3.568701813362934*^9, 3.568702053213324*^9}, {
  3.5690156506771297`*^9, 3.569015651340683*^9}}],

Cell[TextData[{
 "\nThe above example can be very usefull to expand various quantities in the \
double scaling regime.\n\nThe package also provides a function ",
 StyleBox["Taylorv0",
  FontWeight->"Bold"],
 ".  For instance, we can expand in powers of ",
 StyleBox["g",
  FontWeight->"Bold"],
 " the function ",
 Cell[BoxData[
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"3", ",", "2"}], 
   RowBox[{"[", "2", "]"}]]],
  CellChangeTimes->{{3.568702110094342*^9, 3.568702114165358*^9}}],
 " evluated at point ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "=", 
    SubscriptBox["u", "1"]}], TraditionalForm]]],
 " (ie at the Bethe root). This is done below :"
}], "Text",
 CellChangeTimes->{{3.568702058675664*^9, 3.5687020853052187`*^9}, {
  3.56870212085505*^9, 3.568702214949218*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   StyleBox["\[Eta]",
    FontWeight->Bold,
    FontColor->RGBColor[0.8, 0, 0]], 
   RowBox[{"3", ",", "2"}], 
   RowBox[{"[", "2", "]"}]], "/.", 
  RowBox[{"Taylorv0", "[", "2", "]"}]}]], "Input",
 CellChangeTimes->{{3.568702110094342*^9, 3.568702114165358*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   SubsuperscriptBox[
    StyleBox["\[Eta]0",
     FontWeight->Bold,
     FontColor->RGBColor[0, 0.5, 0]], 
    RowBox[{"3", ",", "2"}], 
    RowBox[{"[", "2", "]"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], 
         RowBox[{"3", ",", "3"}], 
         RowBox[{"[", "2", "]"}]]}], "-", 
       RowBox[{"3", " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], 
         RowBox[{"4", ",", "2"}], 
         RowBox[{"[", "2", "]"}]]}]}], ")"}], " ", 
     SuperscriptBox["g", "2"]}], 
    SqrtBox["3"]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "g", "]"}], "3"],
    SeriesData[$CellContext`g, 0, {}, 0, 3, 1],
    Editable->False]}],
  SeriesData[$CellContext`g, 0, {
    $CellContext`\[Eta]0$[3, 2][2], 
    0, (4 3^Rational[-1, 2]) ((-2) $CellContext`\[Eta]0$[3, 3][2] - 
     3 $CellContext`\[Eta]0$[4, 2][2])}, 0, 3, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.568702114804974*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Numerical computation", "Subsection",
 CellChangeTimes->{{3.568694187801199*^9, 3.5686941924810143`*^9}, {
  3.568703166663309*^9, 3.568703170359289*^9}, {3.5690156975784283`*^9, 
  3.569015700745829*^9}}],

Cell[TextData[{
 "The function BoldCompute allows to numerically compute \[Zeta]- and \
\[Eta]-function. It is based on Euler-Maclorin ressumation. For \
single-indexed entries it uses ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " rutines instead. We can use it to numerically check several relations"
}], "Text",
 CellChangeTimes->{{3.5687027794322653`*^9, 3.56870280023074*^9}, {
  3.568702858397092*^9, 3.568702858637294*^9}, {3.568703175431179*^9, 
  3.568703189934416*^9}, {3.5690157172817297`*^9, 3.569015734521225*^9}, {
  3.569015869843955*^9, 3.569015883611732*^9}}],

Cell[CellGroupData[{

Cell["relations among \[Zeta]-functions", "Subsubsection",
 CellChangeTimes->{{3.5687031945585527`*^9, 3.568703201933807*^9}}],

Cell["\<\
for instance, if we want to check numerically the relation \
\>", "Text",
 CellChangeTimes->{{3.568702807991737*^9, 3.5687028231342373`*^9}, {
  3.568702872476667*^9, 3.568702874548375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], "5"], " ", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "2"}]]}], "=", 
  RowBox[{
   RowBox[{"10", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"1", ",", "2", ",", "6"}]]}], "+", 
   RowBox[{"16", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"1", ",", "3", ",", "5"}]]}], "+", 
   RowBox[{"18", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"1", ",", "4", ",", "4"}]]}], "+", 
   RowBox[{"16", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"1", ",", "5", ",", "3"}]]}], "+", 
   RowBox[{"10", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"1", ",", "6", ",", "2"}]]}], "+", 
   RowBox[{"10", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"2", ",", "1", ",", "6"}]]}], "+", 
   RowBox[{"9", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"2", ",", "2", ",", "5"}]]}], "+", 
   RowBox[{"9", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"2", ",", "3", ",", "4"}]]}], "+", 
   RowBox[{"8", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"2", ",", "4", ",", "3"}]]}], "+", 
   RowBox[{"5", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"2", ",", "5", ",", "2"}]]}], "+", 
   RowBox[{"3", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"3", ",", "2", ",", "4"}]]}], "+", 
   RowBox[{"4", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"3", ",", "3", ",", "3"}]]}], "+", 
   RowBox[{"3", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"3", ",", "4", ",", "2"}]]}], "+", 
   RowBox[{"2", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"4", ",", "2", ",", "3"}]]}], "+", 
   RowBox[{"2", " ", 
    SubscriptBox[
     StyleBox["\[Zeta]",
      FontWeight->Bold,
      FontColor->GrayLevel[0.4]], 
     RowBox[{"4", ",", "3", ",", "2"}]]}], "+", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"5", ",", "2", ",", "2"}]]}]}]], "DisplayFormula",
 CellChangeTimes->{{3.568702834301074*^9, 3.568702849451015*^9}}],

Cell[TextData[{
 "up to a ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", 
    RowBox[{"-", "30"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " digits precision, then we can run the command below :"
}], "Text",
 CellChangeTimes->{{3.568702855182377*^9, 3.568702892699604*^9}, {
  3.568703663809403*^9, 3.5687036698316298`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"$RecursionLimit", "=", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"BoldCompute", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Zeta]", "5"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], 
         RowBox[{"2", ",", "2"}]]}], ",", 
       RowBox[{
        RowBox[{"10", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"1", ",", "2", ",", "6"}]]}], "+", 
        RowBox[{"16", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"1", ",", "3", ",", "5"}]]}], "+", 
        RowBox[{"18", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"1", ",", "4", ",", "4"}]]}], "+", 
        RowBox[{"16", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"1", ",", "5", ",", "3"}]]}], "+", 
        RowBox[{"10", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"1", ",", "6", ",", "2"}]]}], "+", 
        RowBox[{"10", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"2", ",", "1", ",", "6"}]]}], "+", 
        RowBox[{"9", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"2", ",", "2", ",", "5"}]]}], "+", 
        RowBox[{"9", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"2", ",", "3", ",", "4"}]]}], "+", 
        RowBox[{"8", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"2", ",", "4", ",", "3"}]]}], "+", 
        RowBox[{"5", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"2", ",", "5", ",", "2"}]]}], "+", 
        RowBox[{"3", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"3", ",", "2", ",", "4"}]]}], "+", 
        RowBox[{"4", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"3", ",", "3", ",", "3"}]]}], "+", 
        RowBox[{"3", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"3", ",", "4", ",", "2"}]]}], "+", 
        RowBox[{"2", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"4", ",", "2", ",", "3"}]]}], "+", 
        RowBox[{"2", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], 
          RowBox[{"4", ",", "3", ",", "2"}]]}], "+", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], 
         RowBox[{"5", ",", "2", ",", "2"}]]}]}], "}"}], ",", "0", ",", "30"}],
     "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.568702897593231*^9, 3.568702992499958*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
   "0.841718250803702584280505684271512041699561118973891698771`29.\
718093760345457", "+", 
    RowBox[{"0``30.241096753103083", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{
   "0.8417182508037025842805056842924799518613610571066048731662`26.\
813757260595366", "+", 
    RowBox[{"0``26.986938548456003", " ", "\[ImaginaryI]"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.568702914711656*^9, 3.568702927520829*^9}, 
   3.568702966472924*^9, 3.568703005512294*^9, 3.568703069628714*^9}]
}, Open  ]],

Cell["\<\
We see that the two expressions are numerically compatible.\
\>", "Text",
 CellChangeTimes->{{3.568703098104458*^9, 3.568703136027594*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "relations between \[Eta] and ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[Eta]", "_"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 "."
}], "Subsubsection",
 CellChangeTimes->{{3.568703316294393*^9, 3.568703331909223*^9}}],

Cell[TextData[{
 "Let us check the relation (A.8) numerically :\nThis equation has a free \
parameter u. For simplicity we will check the equation when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "=", 
    RowBox[{"\[Pi]", "-", 
     SqrtBox["2"]}]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ".\nWe will evaluate the r.h.s. and the l.h.s. up to a ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", 
    RowBox[{"-", "30"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " precision"
}], "Text",
 CellChangeTimes->{{3.568703333168598*^9, 3.5687034035139427`*^9}, {
  3.5687034444606657`*^9, 3.568703499436125*^9}, {3.568703674672942*^9, 
  3.5687036854640713`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"$RecursionLimit", "=", "\[Infinity]"}], "}"}], ",", 
   RowBox[{"BoldCompute", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["\[Eta]", "2", 
           RowBox[{"[", "2", "]"}]], "+", 
          SubscriptBox[
           OverscriptBox["\[Eta]", "_"], "2"]}], ")"}], "2"], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "4"}], " ", 
         RowBox[{
          RowBox[{"\[Zeta]$", "[", "2", "]"}], "[", "0", "]"}], 
         RowBox[{"(", 
          RowBox[{
           SubsuperscriptBox["\[Eta]", "2", 
            RowBox[{"[", "2", "]"}]], "+", 
           SubscriptBox[
            OverscriptBox["\[Eta]", "_"], "2"]}], ")"}]}], "+", 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["\[Eta]", "4", 
           RowBox[{"[", "2", "]"}]], "+", 
          SubscriptBox[
           OverscriptBox["\[Eta]", "_"], "4"]}], ")"}]}]}], "}"}], ",", 
     RowBox[{"\[Pi]", "-", 
      SqrtBox["2"]}], ",", "30"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.568703522243329*^9, 3.568703618037243*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
   "5.828708313720723698188403902825354340186362488887143`26.13021117206375*^-\
7", "+", 
    RowBox[{"0``32.66566884528033", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{
   "5.828708313720723698188403902923752595664462392962601`22.73527952125693*^-\
7", "+", 
    RowBox[{"0``28.96983090730226", " ", "\[ImaginaryI]"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.568703622687368*^9}]
}, Open  ]],

Cell[TextData[{
 "We see that the function ",
 StyleBox["BoldCompute",
  FontWeight->"Bold"],
 " takes up to 3 arguments. The first one is the quantity to be evaluated, \
the second one is the argument of the \[Eta]-functions (here ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "=", 
    RowBox[{"\[Pi]", "-", 
     SqrtBox["2"]}]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 "), the third one is the desired precision on the result."
}], "Text",
 CellChangeTimes->{{3.568703700874097*^9, 3.568703807773682*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Simplifications of generalized \[Eta]-functions\
\>", "Subsubsection",
 CellChangeTimes->{{3.568703851305187*^9, 3.5687038653682423`*^9}}],

Cell[TextData[{
 "As generalized \[Eta]-functions are only intermediate functions, the \
package does not provide an efficient way to evaluate them.\nHowever, a \
substitution ",
 StyleBox["BBold",
  FontWeight->"Bold"],
 " allows to check some relations.\n\n\nFor instance, let us numerically \
check the relation (A.4), at the point ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "=", 
    RowBox[{"\[Pi]", "-", 
     SqrtBox["2"]}]}], TraditionalForm]]],
 " :"
}], "Text",
 CellChangeTimes->{{3.568703868152413*^9, 3.5687040597402277`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox[
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "2"], 
       SuperscriptBox["u", "2"]], "-", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
       SubscriptBox[
        StyleBox["\[Zeta]",
         FontWeight->Bold,
         FontColor->GrayLevel[0.4]], "1"], " ", 
       SubscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "3"]}], "-", 
      SubscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"2", ",", "2"}]], "-", 
      RowBox[{"2", " ", 
       SubscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"3", ",", "1"}]]}]}], "/.", 
     RowBox[{"u", "\[Rule]", 
      RowBox[{"\[Pi]", "-", 
       SqrtBox["2"]}]}]}], "//", 
    RowBox[{
     RowBox[{"BoldCompute", "[", 
      RowBox[{"#", ",", 
       RowBox[{"\[Pi]", "-", 
        SqrtBox["2"]}]}], "]"}], "&"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"GeneralizedEta", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "u"}], "-", "I"}], ",", 
         RowBox[{"-", "I"}]}], "}"}], "]"}], "[", "]"}], "/.", 
     RowBox[{"u", "\[Rule]", 
      RowBox[{"\[Pi]", "-", 
       SqrtBox["2"]}]}]}], "/.", 
    RowBox[{"BBold", "[", "]"}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.568704061830679*^9, 3.568704182031176*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
    "-", "0.03395740111622286869250892459499878877`17.143203962033216"}], "+", 
    RowBox[{
    "0.23501440500827504368966957620755960478`18.00926517028459", " ", 
     "\[ImaginaryI]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "0.033957401130435584`"}], "+", 
    RowBox[{"0.23501440499898069`", " ", "\[ImaginaryI]"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.568704100834776*^9, {3.568704131138254*^9, 3.568704182796379*^9}}]
}, Open  ]],

Cell["\<\
we see that the rhs and the lhs agree for the first 6 digits.\
\>", "Text",
 CellChangeTimes->{{3.5687041960384007`*^9, 3.568704230652169*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["MZVs with 100-digit precision", "Subsubsection",
 CellChangeTimes->{{3.568703851305187*^9, 3.5687038653682423`*^9}, {
  3.569015779688945*^9, 3.569015789942087*^9}, {3.569015972538513*^9, 
  3.5690159809909353`*^9}}],

Cell["\<\
We used EZFace cite to get 100-digit value of all non-reducible MZV up to \
transcendentality 13.\
\>", "Text",
 CellChangeTimes->{{3.569015795910458*^9, 3.569015852300112*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Zeta]", 
   RowBox[{"1", ",", "2", ",", "8"}]], "/.", "Nnnreducibles"}]], "Input",
 CellChangeTimes->{{3.569015824949666*^9, 3.569015830294821*^9}}],

Cell[BoxData["0.\
000046383080725019974152819941576082763217558297854152947135571799236922931906\
2597060880745638843351497`99."], "Output",
 CellChangeTimes->{3.569015831126438*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nnnreducibles", "[", 
  RowBox[{"[", 
   RowBox[{"All", ",", "1"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5690158341687813`*^9, 3.569015840262911*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "6"}]], ",", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "8"}]], ",", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "2", ",", "8"}]], ",", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"2", ",", "10"}]], ",", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "1", ",", "2", ",", "8"}]], ",", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "2", ",", "10"}]], ",", 
   SubscriptBox[
    StyleBox["\[Zeta]",
     FontWeight->Bold,
     FontColor->GrayLevel[0.4]], 
    RowBox[{"1", ",", "3", ",", "9"}]]}], "}"}]], "Output",
 CellChangeTimes->{3.569015840643855*^9}]
}, Open  ]],

Cell[TextData[{
 "Since single-indexed Zetas are implemented in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", one can compute zeta-functions to a 100-digit precision on fly:"
}], "Text",
 CellChangeTimes->{{3.569015856923602*^9, 3.569015910529482*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Zeta]", 
     RowBox[{"2", ",", "4", ",", "3", ",", "2"}]], "//.", "subZetaReduce"}], "/.",
    "Nnnreducibles"}], "/.", "subZetas"}]], "Input",
 CellChangeTimes->{{3.569015911943112*^9, 3.569015939418178*^9}}],

Cell[BoxData["0.\
001207145075688755810997005753765659573808937527644172998611802557745495599847\
871768707518235027901163091749167206320826648672990670572`96.22494886603876"],\
 "Output",
 CellChangeTimes->{{3.569015920078656*^9, 3.569015955715987*^9}}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Integration", "Subsection",
 CellChangeTimes->{{3.568694195128689*^9, 3.5686941980811977`*^9}}],

Cell[TextData[{
 "let us numerically check the integral (14), when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "=", 
    RowBox[{"\[Pi]", "-", 
     SqrtBox["2"], "+", "I"}]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ".\nLet us first numerically evaluate this intagral up to a ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", 
    RowBox[{"-", "15"}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " precision :"
}], "Text",
 CellChangeTimes->{{3.568704392655736*^9, 3.568704403622188*^9}, {
  3.568704440358658*^9, 3.568704470499976*^9}, {3.5687045628497*^9, 
  3.568704579536847*^9}, {3.5687046353447313`*^9, 3.568704643510848*^9}, {
  3.5687050152415543`*^9, 3.568705015241621*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"vVal", ",", "Integrand", ",", 
     RowBox[{"\[Section]Prec", "=", "15"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"vVal", "=", 
     RowBox[{"\[Pi]", "-", 
      SqrtBox["2"], "+", "I"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Integrand", "=", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "1"}], "/", 
       RowBox[{"(", 
        RowBox[{"2", "*", "Pi", "*", "I", "*", 
         RowBox[{"(", 
          RowBox[{"u", "-", "v"}], ")"}]}], ")"}]}], "*", 
      RowBox[{
       RowBox[{"etab", "[", "3", "]"}], "[", 
       RowBox[{"-", "2"}], "]"}], "*", 
      RowBox[{
       RowBox[{"eta", "[", "2", "]"}], "[", "2", "]"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Integrand", "=", 
     RowBox[{
      RowBox[{"Integrand", "/.", 
       RowBox[{"v", "\[Rule]", "vVal"}]}], "/.", "subtoNum"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"NIntegrate", "[", 
     RowBox[{"Integrand", ",", 
      RowBox[{"{", 
       RowBox[{"u", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"Evaluate", "@", 
       RowBox[{"IntPrec", "@", "\[Section]Prec"}]}]}], "]"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.568704491619192*^9, 3.568704627027623*^9}, 
   3.568705008219902*^9}],

Cell[BoxData[
 RowBox[{"0.10962020551440772411792439252017065908`17.0090525052473", "-", 
  RowBox[{
  "0.10504895551691147537004289745279482708`16.990553631951833", " ", 
   "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.568704620125455*^9, 3.568704629286418*^9}, 
   3.568705018030983*^9, 3.568705146707954*^9}]
}, Open  ]],

Cell["\<\
next we compare with the numeric evaluation of the r.h.s.\
\>", "Text",
 CellChangeTimes->{{3.5687046474706087`*^9, 3.5687046663894167`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"vVal", ",", 
     RowBox[{"\[Section]Prec", "=", "15"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"vVal", "=", 
     RowBox[{"\[Pi]", "-", 
      SqrtBox["2"], "+", "I"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"BoldCompute", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"6", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "2", "2"]}], 
          RowBox[{"5", " ", "u"}]]}], "-", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "3"]}], 
         SuperscriptBox["u", "2"]], "+", 
        RowBox[{"3", " ", "\[ImaginaryI]", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"], " ", 
         SubscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], "2"]}], "+", 
        RowBox[{
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "2"], " ", 
         SubscriptBox[
          StyleBox["\[Eta]",
           FontWeight->Bold,
           FontColor->RGBColor[0.8, 0, 0]], "3"]}], "+", 
        SubscriptBox[
         StyleBox["\[Eta]",
          FontWeight->Bold,
          FontColor->RGBColor[0.8, 0, 0]], 
         RowBox[{"3", ",", "2"}]]}], "/.", 
       RowBox[{"u", "\[Rule]", "vVal"}]}], ",", "vVal", ",", 
      "\[Section]Prec"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.568704542976343*^9, 3.568704545371826*^9}, 
   3.568704616970601*^9, {3.568704687665936*^9, 3.568704742255684*^9}, 
   3.568705011947772*^9, 3.568705138337771*^9, {3.568705185880192*^9, 
   3.568705203533943*^9}}],

Cell[BoxData[
 RowBox[{"0.10962020551440793041206028451403009699`13.138008032062226", "-", 
  RowBox[{
  "0.10504895551691146414391962143107918479`13.1583166315557", " ", 
   "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{
  3.5687047432690153`*^9, 3.568705020788072*^9, 3.568705138903235*^9, {
   3.568705186743477*^9, 3.56870520437845*^9}}]
}, Open  ]],

Cell["\<\
We obtain a very good match between these two results.\
\>", "Text",
 CellChangeTimes->{{3.568705225131218*^9, 3.5687052393385677`*^9}}],

Cell[TextData[{
 "\n\nAs we see from this example, the substitution ",
 StyleBox["subtoNum",
  FontWeight->"Bold"],
 " allows to convert an integrand that contains \[Eta]-functions into a \
numeric approximation of it, that can be used by the function ",
 StyleBox["NIntegrate",
  FontWeight->"Bold"],
 "."
}], "Text",
 CellChangeTimes->{{3.5687052499543324`*^9, 3.568705332238207*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Weak coupling expansion. ", "Section",
 CellChangeTimes->{{3.5686963458116283`*^9, 3.568696363145094*^9}, 
   3.5690160386151114`*^9, {3.5690167212124443`*^9, 3.569016723537724*^9}, {
   3.5690185411404047`*^9, 3.569018564433648*^9}}],

Cell["\<\
All the details about weak coupling expansion are present in WeakCoupling.nb. \
 Random examples of weak coupling expansions are:\
\>", "Text",
 CellChangeTimes->{{3.5690185553708897`*^9, 3.569018624504037*^9}, {
  3.5690187612691107`*^9, 3.569018765194213*^9}, {3.5690196981536503`*^9, 
  3.569019712236685*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Gamma]", "0"], "/.", "weak"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[Gamma]", "0"], "/.", 
  RowBox[{"leadweak", "[", "2", "]"}]}]}], "Input",
 CellChangeTimes->{{3.569018592930118*^9, 3.569018604018964*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"12", " ", 
   SuperscriptBox["g", "2"]}], "-", 
  RowBox[{"48", " ", 
   SuperscriptBox["g", "4"]}], "+", 
  RowBox[{"336", " ", 
   SuperscriptBox["g", "6"]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "2820"}], "-", 
     RowBox[{"288", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{"26508", "+", 
     FractionBox[
      RowBox[{"4", " ", "Im1"}], 
      SqrtBox["3"]], "+", 
     RowBox[{"4320", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "+", 
     RowBox[{"2880", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "269148"}], "-", 
     FractionBox[
      RowBox[{"20", " ", "Im1"}], 
      SqrtBox["3"]], "+", 
     FractionBox[
      RowBox[{"4", " ", "Im2"}], 
      SqrtBox["3"]], "-", 
     RowBox[{"55296", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"44064", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "-", 
     RowBox[{"30240", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "14"], " ", 
   RowBox[{"(", 
    RowBox[{"2877240", "+", 
     RowBox[{"56", " ", 
      SqrtBox["3"], " ", "Im1"}], "-", 
     FractionBox[
      RowBox[{"20", " ", "Im2"}], 
      SqrtBox["3"]], "+", 
     FractionBox[
      RowBox[{"4", " ", "Im3"}], 
      SqrtBox["3"]], "+", 
     RowBox[{"697536", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "+", 
     RowBox[{"3456", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
     RowBox[{"579744", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "+", 
     RowBox[{"471744", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "+", 
     RowBox[{"338688", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "16"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "31857132"}], "-", 
     FractionBox[
      RowBox[{"1844", " ", "Im1"}], 
      SqrtBox["3"]], "+", 
     RowBox[{"56", " ", 
      SqrtBox["3"], " ", "Im2"}], "-", 
     FractionBox[
      RowBox[{"20", " ", "Im3"}], 
      SqrtBox["3"]], "+", 
     FractionBox[
      RowBox[{"4", " ", "Im4"}], 
      SqrtBox["3"]], "-", 
     RowBox[{"8912160", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"]}], "-", 
     RowBox[{"79488", " ", 
      SubsuperscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3", "2"]}], "-", 
     RowBox[{"7486848", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "-", 
     RowBox[{"69120", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "3"], " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "5"]}], "-", 
     RowBox[{"6342624", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "7"]}], "-", 
     RowBox[{"5370624", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "9"]}], "-", 
     RowBox[{"3991680", " ", 
      SubscriptBox[
       StyleBox["\[Zeta]",
        FontWeight->Bold,
        FontColor->GrayLevel[0.4]], "11"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.5690185952863293`*^9, 3.5690186048446693`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"12", " ", 
   SuperscriptBox["g", "2"]}], "-", 
  RowBox[{"48", " ", 
   SuperscriptBox["g", "4"]}]}]], "Output",
 CellChangeTimes->{{3.5690185952863293`*^9, 3.569018604849701*^9}}]
}, Open  ]],

Cell[TextData[{
 "Another example is expansion of  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["U", "2"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 "at first five orders (i.e. including wrapping correction). This is a good \
illustration that the answer is expressed through strandard-type expressions. \
nsweak is an upgraded version of leadweak. leadweak contains typically first \
4 orders, while nsweak has 5 orders however for a smaller number of \
quantities."
}], "Text",
 CellChangeTimes->{{3.569019555948636*^9, 3.569019665094769*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox["U", "2"], "/.", 
  RowBox[{"nsweak", "[", "5", "]"}]}]], "Input",
 CellChangeTimes->{{3.5690196135576344`*^9, 3.569019616986866*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox["g", "4"]}], 
    SuperscriptBox[
     StyleBox["\[ScriptU]",
      FontColor->RGBColor[0, 0, 0.4]], "2"]]}], "+", 
  RowBox[{
   SuperscriptBox["g", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["4", 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "4"]]}], "-", 
     FractionBox[
      RowBox[{"12", " ", "\[ImaginaryI]"}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"-", "18"}], "+", 
       RowBox[{"24", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "1", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "8"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["10", 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "6"]]}], "-", 
     FractionBox[
      RowBox[{"36", " ", "\[ImaginaryI]"}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "60"}], " ", "\[ImaginaryI]"}], "+", 
       RowBox[{"144", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"-", "6"}], "+", 
       RowBox[{"48", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
     FractionBox[
      RowBox[{"36", "+", 
       RowBox[{"120", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "-", 
       RowBox[{"144", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
       RowBox[{"72", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"48", " ", "\[ImaginaryI]"}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "4"]]}], "+", 
        FractionBox["144", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "120"}], " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"288", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     FractionBox[
      RowBox[{"144", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "2", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"24", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "3", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"288", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "10"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["28", 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "8"]]}], "-", 
     FractionBox[
      RowBox[{"108", " ", "\[ImaginaryI]"}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "7"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "144"}], " ", "\[ImaginaryI]"}], "+", 
       RowBox[{"432", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
     FractionBox[
      RowBox[{"30", "+", 
       RowBox[{"120", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "6"]], "+", 
     FractionBox[
      RowBox[{"114", "-", 
       RowBox[{"120", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "-", 
       RowBox[{"288", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
       RowBox[{"192", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"312", " ", "\[ImaginaryI]"}], "+", 
       RowBox[{"144", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "-", 
       RowBox[{"864", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
       RowBox[{"432", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"-", "216"}], "-", 
       RowBox[{"624", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "-", 
       RowBox[{"144", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
       RowBox[{"576", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "3"]}], "-", 
       RowBox[{"360", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"864", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"480", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"120", " ", "\[ImaginaryI]"}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "6"]]}], "+", 
        FractionBox["432", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
        FractionBox[
         RowBox[{"144", "-", 
          RowBox[{"1728", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"120", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"576", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"624", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"288", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "-", 
          RowBox[{"1728", " ", "\[ImaginaryI]", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
          RowBox[{"864", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["288", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{"864", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{"144", "-", 
          RowBox[{"1728", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "2", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"48", " ", "\[ImaginaryI]"}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "4"]]}], "+", 
        FractionBox["144", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"456", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"288", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     FractionBox[
      RowBox[{"288", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "4", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"48", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "5", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["576", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{"1728", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{"288", "-", 
          RowBox[{"3456", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"1", ",", "1"}], 
       RowBox[{"[", "2", "]"}]]}], "+", 
     FractionBox[
      RowBox[{"1728", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "2"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"288", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "3"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"1728", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"2", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"288", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"3", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "1", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["g", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["84", 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "10"]]}], "-", 
     FractionBox[
      RowBox[{"336", " ", "\[ImaginaryI]"}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "9"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "360"}], " ", "\[ImaginaryI]"}], "+", 
       RowBox[{"1296", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "7"]], "+", 
     FractionBox[
      RowBox[{"168", "+", 
       RowBox[{"336", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "8"]], "+", 
     FractionBox[
      RowBox[{"222", "-", 
       RowBox[{"840", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "-", 
       RowBox[{"720", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
       RowBox[{"552", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "6"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"1188", " ", "\[ImaginaryI]"}], "-", 
       RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
       RowBox[{"2160", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "2364"}], " ", "\[ImaginaryI]"}], "-", 
       RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "+", 
       RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
       RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "3"]}], "-", 
       RowBox[{"2304", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"5184", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"4320", " ", "\[ImaginaryI]", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"-", "996"}], "-", 
       RowBox[{"312", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"]}], "+", 
       RowBox[{"1872", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
       RowBox[{"1152", " ", 
        SubsuperscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1", "3"]}], "+", 
       RowBox[{"120", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"2304", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "1"], " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "3"]}], "+", 
       RowBox[{"1440", " ", 
        SubscriptBox[
         StyleBox["\[Zeta]",
          FontWeight->Bold,
          FontColor->GrayLevel[0.4]], "5"]}]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "324"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"324", "+", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "54"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}]}], 
      SubsuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], 
       RowBox[{"-", "v0"}], "+"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"972", " ", "\[ImaginaryI]"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "324"}], " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "324"}], "+", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "54"}], " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}]}], 
      SubsuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], 
       RowBox[{"-", "v0"}], "-"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "1944"}], " ", "\[ImaginaryI]"}], "+", 
       RowBox[{"648", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "-", 
       RowBox[{"216", " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "+", 
       RowBox[{"108", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         StyleBox["\[Eta]0",
          FontWeight->Bold,
          FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
       RowBox[{"648", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "-", 
       RowBox[{"216", " ", "\[ImaginaryI]", " ", 
        SqrtBox["3"], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
       RowBox[{"108", " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], 
      StyleBox["\[ScriptU]",
       FontColor->RGBColor[0, 0, 0.4]]], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        StyleBox["\[ScriptU]",
         FontColor->RGBColor[0, 0, 0.4]], "2"]], 
      RowBox[{"(", 
       RowBox[{"2124", "+", 
        RowBox[{"5376", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1"]}], "+", 
        RowBox[{"2592", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
        RowBox[{"1728", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1", "3"]}], "-", 
        RowBox[{"1728", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1", "4"]}], "+", 
        RowBox[{"5904", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "+", 
        RowBox[{"2016", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "-", 
        RowBox[{"5184", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1", "2"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3"]}], "-", 
        RowBox[{"1728", " ", 
         SubsuperscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "3", "2"]}], "+", 
        RowBox[{"3120", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "-", 
        RowBox[{"8640", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "1"], " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "5"]}], "-", 
        RowBox[{"5040", " ", 
         SubscriptBox[
          StyleBox["\[Zeta]",
           FontWeight->Bold,
           FontColor->GrayLevel[0.4]], "7"]}], "-", 
        RowBox[{"216", " ", "\[ImaginaryI]", " ", 
         SqrtBox["3"], " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "2", "+"]}], "+", 
        RowBox[{"216", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "3", "+"]}], "-", 
        RowBox[{"36", " ", "\[ImaginaryI]", " ", 
         SqrtBox["3"], " ", 
         SubsuperscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], "4", "+"]}], "+", 
        RowBox[{"216", " ", "\[ImaginaryI]", " ", 
         SqrtBox["3"], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "-", 
        RowBox[{"216", " ", "\[ImaginaryI]", " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
        RowBox[{"36", " ", "\[ImaginaryI]", " ", 
         SqrtBox["3"], " ", 
         SubsuperscriptBox[
          OverscriptBox[
           StyleBox["\[Eta]0",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]], 
           StyleBox["_",
            FontWeight->Bold,
            FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"972", " ", "\[ImaginaryI]"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "324"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"324", "+", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "54"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], 
      SubsuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "v0", "-"]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "324"}], " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "2", "-"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "324"}], "+", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "3", "-"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "54"}], " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], ")"}], " ", 
        SubsuperscriptBox[
         OverscriptBox[
          StyleBox["\[Eta]0",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]], 
          StyleBox["_",
           FontWeight->Bold,
           FontColor->RGBColor[0, 0.5, 0]]], "4", "-"]}]}], 
      SubsuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "v0", "+"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"336", " ", "\[ImaginaryI]"}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "8"]]}], "+", 
        FractionBox["1296", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "7"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"840", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"1440", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "6"]], "-", 
        FractionBox[
         RowBox[{"5184", " ", 
          SubscriptBox[
           StyleBox["\[Zeta]",
            FontWeight->Bold,
            FontColor->GrayLevel[0.4]], "1"]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"312", " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"3744", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "-", 
          RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
          RowBox[{"2304", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "2592"}], "+", 
          RowBox[{"5184", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "+", 
          RowBox[{"10368", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
          RowBox[{"5184", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "5376"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"5184", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "+", 
          RowBox[{"5184", " ", "\[ImaginaryI]", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
          RowBox[{"6912", " ", "\[ImaginaryI]", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "3"]}], "-", 
          RowBox[{"2016", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "+", 
          RowBox[{"10368", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"], " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}], "+", 
          RowBox[{"8640", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "5"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "1", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["648", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "6"]], "+", 
        FractionBox[
         RowBox[{"2592", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"324", " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], 
          RowBox[{"-", "v0"}], "-"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"324", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], 
          RowBox[{"-", "v0"}], "+"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"324", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "v0", "-"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"324", " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"324", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "v0", "+"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "2304"}], "-", 
          RowBox[{"3456", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "2376"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"10368", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "-", 
          RowBox[{"864", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "1296"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"2592", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "2592"}], "+", 
          RowBox[{"5184", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "+", 
          RowBox[{"10368", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
          RowBox[{"5184", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "2", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"120", " ", "\[ImaginaryI]"}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "6"]]}], "+", 
        FractionBox["576", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
        FractionBox[
         RowBox[{"324", "-", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], 
          RowBox[{"-", "v0"}], "-"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "324"}], "-", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], 
          RowBox[{"-", "v0"}], "+"]], "+", 
        FractionBox[
         RowBox[{"324", "+", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "v0", "-"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "324"}], "+", 
          RowBox[{"108", " ", "\[ImaginaryI]", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "v0", "+"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "3096"}], "-", 
          RowBox[{"1728", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"1272", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"576", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "528"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"6624", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "-", 
          RowBox[{"1728", " ", "\[ImaginaryI]", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "-", 
          RowBox[{"1152", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "3", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["504", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{"1728", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "-", 
        FractionBox[
         RowBox[{"216", " ", "\[ImaginaryI]"}], 
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"54", " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], 
          RowBox[{"-", "v0"}], "-"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"54", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], 
          RowBox[{"-", "v0"}], "+"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"54", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "v0", "-"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"54", " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"54", " ", 
           SqrtBox["3"]}]}], 
         SubsuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "v0", "+"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "1656"}], "-", 
          RowBox[{"3456", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "4", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"96", " ", "\[ImaginaryI]"}], 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "4"]]}], "+", 
        FractionBox["288", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"1488", " ", "\[ImaginaryI]"}], "+", 
          RowBox[{"576", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], "5", 
       RowBox[{"[", "2", "]"}]]}], "+", 
     FractionBox[
      RowBox[{"720", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "6", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"120", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], "7", 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["1440", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "6"]], "+", 
        FractionBox[
         RowBox[{"5184", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "5"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "3744"}], "-", 
          RowBox[{"6912", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "5184"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"20736", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "5184"}], "+", 
          RowBox[{"10368", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}], "+", 
          RowBox[{"20736", " ", 
           SubsuperscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1", "2"]}], "+", 
          RowBox[{"10368", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "3"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"1", ",", "1"}], 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"3456", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
        FractionBox["10368", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "5184"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"20736", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"1", ",", "2"}], 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["576", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{"1728", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "6624"}], "-", 
          RowBox[{"3456", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"1", ",", "3"}], 
       RowBox[{"[", "2", "]"}]]}], "+", 
     FractionBox[
      RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "4"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"576", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "5"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"3456", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
        FractionBox["10368", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "5184"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"20736", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"2", ",", "1"}], 
       RowBox[{"[", "2", "]"}]]}], "-", 
     FractionBox[
      RowBox[{"10800", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"2", ",", "2"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["144", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{"1728", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
        FractionBox["432", 
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"2", ",", "3"}], 
       RowBox[{"[", "2", "]"}]]}], "-", 
     FractionBox[
      RowBox[{"144", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"2", ",", "4"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox["576", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "+", 
        FractionBox[
         RowBox[{"1728", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "6624"}], "-", 
          RowBox[{"3456", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"3", ",", "1"}], 
       RowBox[{"[", "2", "]"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["144", 
          SuperscriptBox[
           StyleBox["\[ScriptU]",
            FontColor->RGBColor[0, 0, 0.4]], "3"]]}], "+", 
        FractionBox[
         RowBox[{"1728", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
        FractionBox["432", 
         StyleBox["\[ScriptU]",
          FontColor->RGBColor[0, 0, 0.4]]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"3", ",", "2"}], 
       RowBox[{"[", "2", "]"}]]}], "+", 
     FractionBox[
      RowBox[{"432", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"3", ",", "3"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"4", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"576", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"5", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"6912", " ", "\[ImaginaryI]"}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "4"]], "-", 
        FractionBox["20736", 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "3"]], "+", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "10368"}], " ", "\[ImaginaryI]"}], "-", 
          RowBox[{"41472", " ", "\[ImaginaryI]", " ", 
           SubscriptBox[
            StyleBox["\[Zeta]",
             FontWeight->Bold,
             FontColor->GrayLevel[0.4]], "1"]}]}], 
         SuperscriptBox[
          StyleBox["\[ScriptU]",
           FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}], " ", 
      SubsuperscriptBox[
       StyleBox["\[Eta]",
        FontWeight->Bold,
        FontColor->RGBColor[0.8, 0, 0]], 
       RowBox[{"1", ",", "1", ",", "1"}], 
       RowBox[{"[", "2", "]"}]]}], "-", 
     FractionBox[
      RowBox[{"20736", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "1", ",", "2"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "1", ",", "3"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"20736", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "2", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "3", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"20736", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"2", ",", "1", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "+", 
     FractionBox[
      RowBox[{"3456", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"3", ",", "1", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]], "-", 
     FractionBox[
      RowBox[{"41472", " ", 
       SubsuperscriptBox[
        StyleBox["\[Eta]",
         FontWeight->Bold,
         FontColor->RGBColor[0.8, 0, 0]], 
        RowBox[{"1", ",", "1", ",", "1", ",", "1"}], 
        RowBox[{"[", "2", "]"}]]}], 
      SuperscriptBox[
       StyleBox["\[ScriptU]",
        FontColor->RGBColor[0, 0, 0.4]], "2"]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.569019617602038*^9}]
}, Open  ]],

Cell["For asymptotic expressions one can use", "Text",
 CellChangeTimes->{{3.569018635878851*^9, 3.569018641894464*^9}, 
   3.569018770569577*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"T$r", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "[", "2", "]"}], "/.", 
  "largevolume"}]], "Input",
 CellChangeTimes->{{3.569018643961844*^9, 3.5690186502330008`*^9}}],

Cell[BoxData[
 RowBox[{"1", "-", 
  FractionBox[
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], "+"]}], "+", 
    FractionBox["1", 
     SuperscriptBox[
      StyleBox["x",
       FontColor->RGBColor[0, 0.6, 0]], 
      RowBox[{"[", "3", "]"}]]]}], 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      OverscriptBox[
       StyleBox["y",
        FontColor->RGBColor[0, 0.6, 0]], 
       StyleBox["^",
        FontColor->RGBColor[0, 0.6, 0]]], "-"]}], "+", 
    SuperscriptBox[
     OverscriptBox[
      StyleBox["y",
       FontColor->RGBColor[0, 0.6, 0]], 
      StyleBox["^",
       FontColor->RGBColor[0, 0.6, 0]]], "+"]}]]}]], "Output",
 CellChangeTimes->{3.5690186504840717`*^9}]
}, Open  ]],

Cell["\<\
Try step by step the following chain of replacements to see what they do:\
\>", "Text",
 CellChangeTimes->{{3.569018679492889*^9, 3.569018693748495*^9}, {
  3.56901972521621*^9, 3.569019733028119*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Y11Y22", "/.", "largevolume"}], "/.", "subFunsBethe"}], "/.", 
     "subJuk"}], "/.", 
    RowBox[{
     SubscriptBox["v", "0"], "\[Rule]", "v00"}]}], "/.", 
   RowBox[{"g", "\[Rule]", 
    RowBox[{"1", "/", "2"}]}]}], "/.", 
  RowBox[{"u", "\[Rule]", 
   RowBox[{"Random", "[", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.569018657496685*^9, 3.569018701855289*^9}}],

Cell[BoxData[
 RowBox[{"27.42395198975152`", "\[VeryThinSpace]", "+", 
  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{{3.569018661991804*^9, 3.569018702609688*^9}, 
   3.569019734971993*^9}]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{1280, 752},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True,
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (July 22, \
2012)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 105, 1, 76, "Title"],
Cell[687, 25, 235, 5, 46, "Subtitle"],
Cell[CellGroupData[{
Cell[947, 34, 192, 4, 67, "Section"],
Cell[1142, 40, 1472, 38, 88, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[2651, 83, 175, 2, 37, "Section"],
Cell[2829, 87, 395, 11, 26, "Text"],
Cell[CellGroupData[{
Cell[3249, 102, 134, 3, 35, "Subsection"],
Cell[3386, 107, 349, 6, 26, "Text"],
Cell[CellGroupData[{
Cell[3760, 117, 290, 7, 27, "Input"],
Cell[4053, 126, 349, 10, 38, "Output"]
}, Open  ]],
Cell[4417, 139, 199, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[4641, 147, 300, 8, 27, "Input"],
Cell[4944, 157, 216, 6, 34, "Output"]
}, Open  ]],
Cell[5175, 166, 135, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[5335, 173, 103, 1, 27, "Input"],
Cell[5441, 176, 245, 5, 27, "Output"]
}, Open  ]],
Cell[5701, 184, 424, 7, 41, "Text"],
Cell[CellGroupData[{
Cell[6150, 195, 514, 14, 27, "Input"],
Cell[6667, 211, 871, 25, 38, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[7587, 242, 167, 2, 26, "Subsection"],
Cell[7757, 246, 383, 8, 41, "Text"],
Cell[CellGroupData[{
Cell[8165, 258, 400, 13, 34, "Input"],
Cell[8568, 273, 575, 19, 34, "Output"]
}, Open  ]],
Cell[9158, 295, 144, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[9327, 302, 533, 16, 34, "Input"],
Cell[9863, 320, 641, 19, 34, "Output"]
}, Open  ]],
Cell[10519, 342, 125, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[10669, 349, 54, 0, 27, "Input"],
Cell[10726, 351, 2250, 34, 43, "Output"]
}, Open  ]],
Cell[12991, 388, 275, 5, 26, "Text"],
Cell[13269, 395, 146, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[13440, 402, 194, 4, 27, "Input"],
Cell[13637, 408, 616, 10, 27, "Output"]
}, Open  ]],
Cell[14268, 421, 134, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[14427, 428, 170, 3, 27, "Input"],
Cell[14600, 433, 292, 8, 27, "Output"]
}, Open  ]],
Cell[14907, 444, 372, 6, 41, "Text"],
Cell[CellGroupData[{
Cell[15304, 454, 148, 3, 27, "Input"],
Cell[15455, 459, 305, 9, 27, "Output"]
}, Open  ]],
Cell[15775, 471, 175, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[15975, 480, 154, 2, 43, "Input"],
Cell[16132, 484, 140, 3, 27, "Output"],
Cell[16275, 489, 247, 8, 39, "Output"]
}, Open  ]],
Cell[16537, 500, 100, 1, 26, "Text"],
Cell[CellGroupData[{
Cell[16662, 505, 180, 3, 43, "Input"],
Cell[16845, 510, 164, 5, 30, "Output"],
Cell[17012, 517, 221, 8, 39, "Output"]
}, Open  ]],
Cell[17248, 528, 245, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[17518, 537, 377, 10, 43, "Input"],
Cell[17898, 549, 231, 6, 32, "Output"],
Cell[18132, 557, 597, 20, 52, "Output"]
}, Open  ]],
Cell[18744, 580, 109, 1, 26, "Text"],
Cell[CellGroupData[{
Cell[18878, 585, 231, 6, 43, "Input"],
Cell[19112, 593, 375, 6, 27, "Output"]
}, Open  ]],
Cell[19502, 602, 151, 2, 26, "Text"],
Cell[CellGroupData[{
Cell[19678, 608, 167, 3, 27, "Input"],
Cell[19848, 613, 211, 5, 27, "Output"]
}, Open  ]],
Cell[20074, 621, 389, 7, 26, "Text"],
Cell[CellGroupData[{
Cell[20488, 632, 200, 3, 43, "Input"],
Cell[20691, 637, 180, 4, 27, "Output"],
Cell[20874, 643, 301, 10, 39, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[21224, 659, 283, 5, 27, "Subsection"],
Cell[21510, 666, 202, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[21737, 674, 259, 6, 43, "Input"],
Cell[21999, 682, 195, 6, 36, "Output"],
Cell[22197, 690, 220, 7, 36, "Output"]
}, Open  ]],
Cell[22432, 700, 435, 11, 39, "Text"],
Cell[CellGroupData[{
Cell[22892, 715, 508, 14, 43, "Input"],
Cell[23403, 731, 730, 24, 38, "Output"],
Cell[24136, 757, 730, 24, 38, "Output"]
}, Open  ]],
Cell[24881, 784, 134, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[25040, 791, 314, 8, 49, "Input"],
Cell[25357, 801, 536, 18, 33, "Output"],
Cell[25896, 821, 684, 24, 47, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[26641, 852, 145, 2, 37, "Section"],
Cell[CellGroupData[{
Cell[26811, 858, 216, 3, 35, "Subsection"],
Cell[CellGroupData[{
Cell[27052, 865, 110, 1, 24, "Subsubsection"],
Cell[27165, 868, 246, 5, 26, "Text"],
Cell[27414, 875, 1261, 42, 42, "DisplayFormula"],
Cell[28678, 919, 470, 12, 29, "Text"],
Cell[29151, 933, 475, 14, 61, "Input"],
Cell[29629, 949, 197, 5, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[29863, 959, 116, 1, 18, "Subsubsection"],
Cell[29982, 962, 212, 4, 26, "Text"],
Cell[30197, 968, 1535, 51, 50, "DisplayFormula"],
Cell[31735, 1021, 565, 14, 35, "Text"],
Cell[32303, 1037, 561, 16, 66, "Input"],
Cell[32867, 1055, 153, 3, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[33057, 1063, 120, 1, 18, "Subsubsection"],
Cell[33180, 1066, 359, 7, 41, "Text"],
Cell[33542, 1075, 1544, 50, 44, "DisplayFormula"],
Cell[35089, 1127, 140, 1, 26, "Text"],
Cell[35232, 1130, 1844, 60, 53, "DisplayFormula"],
Cell[37079, 1192, 488, 15, 35, "Text"],
Cell[37570, 1209, 978, 27, 82, "Input"],
Cell[38551, 1238, 162, 3, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[38750, 1246, 124, 1, 18, "Subsubsection"],
Cell[38877, 1249, 1515, 35, 104, "Text"],
Cell[40395, 1286, 285, 8, 27, "Input"],
Cell[40683, 1296, 357, 7, 26, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[41089, 1309, 258, 5, 27, "Subsection"],
Cell[41350, 1316, 404, 7, 26, "Text"],
Cell[41757, 1325, 775, 27, 26, "DisplayFormula"],
Cell[42535, 1354, 446, 12, 29, "Text"],
Cell[42984, 1368, 629, 18, 85, "Input"],
Cell[43616, 1388, 227, 7, 26, "Text"],
Cell[43846, 1397, 95, 1, 26, "Text"],
Cell[43944, 1400, 222, 7, 36, "Input"],
Cell[44169, 1409, 260, 5, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[44466, 1419, 123, 1, 27, "Subsection"],
Cell[CellGroupData[{
Cell[44614, 1424, 111, 1, 24, "Subsubsection"],
Cell[44728, 1427, 229, 6, 41, "Text"],
Cell[CellGroupData[{
Cell[44982, 1437, 409, 14, 30, "Input"],
Cell[45394, 1453, 2722, 97, 30, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[48165, 1556, 135, 2, 18, "Subsubsection"],
Cell[48303, 1560, 321, 10, 41, "Text"],
Cell[CellGroupData[{
Cell[48649, 1574, 459, 15, 30, "Input"],
Cell[49111, 1591, 747, 27, 30, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[49907, 1624, 129, 1, 18, "Subsubsection"],
Cell[50039, 1627, 458, 9, 56, "Text"],
Cell[CellGroupData[{
Cell[50522, 1640, 329, 9, 30, "Input"],
Cell[50854, 1651, 1237, 44, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52128, 1700, 3475, 113, 65, "Input"],
Cell[55606, 1815, 149, 3, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[55816, 1825, 217, 4, 27, "Subsection"],
Cell[56036, 1831, 231, 6, 26, "Text"],
Cell[56270, 1839, 210, 6, 56, "Text"],
Cell[56483, 1847, 172, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[56680, 1855, 498, 14, 27, "Input"],
Cell[57181, 1871, 865, 29, 47, "Output"]
}, Open  ]],
Cell[58061, 1903, 131, 1, 26, "Text"],
Cell[58195, 1906, 457, 12, 26, "Text"],
Cell[CellGroupData[{
Cell[58677, 1922, 262, 9, 27, "Input"],
Cell[58942, 1933, 2142, 75, 53, "Output"]
}, Open  ]],
Cell[61099, 2011, 1159, 37, 63, "Text"],
Cell[CellGroupData[{
Cell[62283, 2052, 410, 13, 27, "Input"],
Cell[62696, 2067, 1234, 43, 51, "Output"]
}, Open  ]],
Cell[63945, 2113, 1221, 41, 29, "Text"],
Cell[CellGroupData[{
Cell[65191, 2158, 377, 12, 27, "Input"],
Cell[65571, 2172, 788, 28, 47, "Output"]
}, Open  ]],
Cell[66374, 2203, 125, 3, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[66536, 2211, 101, 1, 26, "Subsection"],
Cell[66640, 2214, 339, 6, 41, "Text"],
Cell[66982, 2222, 1097, 32, 86, "Text"],
Cell[CellGroupData[{
Cell[68104, 2258, 763, 23, 27, "Input"],
Cell[68870, 2283, 2061, 72, 53, "Output"]
}, Open  ]],
Cell[70946, 2358, 248, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[71219, 2367, 846, 25, 27, "Input"],
Cell[72068, 2394, 1493, 53, 51, "Output"]
}, Open  ]],
Cell[73576, 2450, 224, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[73825, 2459, 1009, 29, 27, "Input"],
Cell[74837, 2490, 1196, 42, 51, "Output"]
}, Open  ]],
Cell[76048, 2535, 286, 9, 26, "Text"],
Cell[CellGroupData[{
Cell[76359, 2548, 281, 7, 27, "Input"],
Cell[76643, 2557, 297, 9, 46, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[76989, 2572, 107, 1, 26, "Subsection"],
Cell[77099, 2575, 789, 19, 101, "Text"],
Cell[CellGroupData[{
Cell[77913, 2598, 633, 17, 36, "Input"],
Cell[78549, 2617, 1191, 36, 33, "Output"]
}, Open  ]],
Cell[79755, 2656, 1387, 40, 53, "Text"],
Cell[81145, 2698, 856, 25, 80, "Text"],
Cell[CellGroupData[{
Cell[82026, 2727, 308, 9, 36, "Input"],
Cell[82337, 2738, 1266, 40, 60, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[83652, 2784, 211, 3, 26, "Subsection"],
Cell[83866, 2789, 584, 11, 26, "Text"],
Cell[CellGroupData[{
Cell[84475, 2804, 126, 1, 24, "Subsubsection"],
Cell[84604, 2807, 200, 4, 26, "Text"],
Cell[84807, 2813, 3118, 108, 23, "DisplayFormula"],
Cell[87928, 2923, 345, 10, 29, "Text"],
Cell[CellGroupData[{
Cell[88298, 2937, 3792, 113, 53, "Input"],
Cell[92093, 3052, 541, 13, 33, "Output"]
}, Open  ]],
Cell[92649, 3068, 149, 3, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[92835, 3076, 253, 8, 20, "Subsubsection"],
Cell[93091, 3086, 690, 19, 64, "Text"],
Cell[CellGroupData[{
Cell[93806, 3109, 1203, 35, 39, "Input"],
Cell[95012, 3146, 438, 12, 33, "Output"]
}, Open  ]],
Cell[95465, 3161, 523, 14, 31, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[96025, 3180, 148, 3, 18, "Subsubsection"],
Cell[96176, 3185, 541, 15, 91, "Text"],
Cell[CellGroupData[{
Cell[96742, 3204, 1687, 55, 75, "Input"],
Cell[98432, 3261, 499, 14, 27, "Output"]
}, Open  ]],
Cell[98946, 3278, 153, 3, 26, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[99136, 3286, 222, 3, 18, "Subsubsection"],
Cell[99361, 3291, 187, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[99573, 3299, 190, 4, 30, "Input"],
Cell[99766, 3305, 182, 3, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[99985, 3313, 183, 4, 27, "Input"],
Cell[100171, 3319, 1085, 38, 33, "Output"]
}, Open  ]],
Cell[101271, 3360, 262, 6, 26, "Text"],
Cell[CellGroupData[{
Cell[101558, 3370, 277, 7, 30, "Input"],
Cell[101838, 3379, 254, 4, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[102153, 3390, 101, 1, 26, "Subsection"],
Cell[102257, 3393, 713, 19, 50, "Text"],
Cell[CellGroupData[{
Cell[102995, 3416, 1349, 39, 115, "Input"],
Cell[104347, 3457, 320, 6, 27, "Output"]
}, Open  ]],
Cell[104682, 3466, 151, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[104858, 3473, 2014, 58, 99, "Input"],
Cell[106875, 3533, 346, 7, 27, "Output"]
}, Open  ]],
Cell[107236, 3543, 146, 3, 26, "Text"],
Cell[107385, 3548, 386, 10, 56, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[107820, 3564, 240, 3, 37, "Section"],
Cell[108063, 3569, 323, 6, 26, "Text"],
Cell[CellGroupData[{
Cell[108411, 3579, 265, 6, 43, "Input"],
Cell[108679, 3587, 4492, 153, 136, "Output"],
Cell[113174, 3742, 216, 6, 30, "Output"]
}, Open  ]],
Cell[113405, 3751, 559, 12, 44, "Text"],
Cell[CellGroupData[{
Cell[113989, 3767, 175, 4, 30, "Input"],
Cell[114167, 3773, 61801, 1843, 939, "Output"]
}, Open  ]],
Cell[175983, 5619, 146, 2, 26, "Text"],
Cell[CellGroupData[{
Cell[176154, 5625, 215, 6, 27, "Input"],
Cell[176372, 5633, 753, 27, 59, "Output"]
}, Open  ]],
Cell[177140, 5663, 211, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[177376, 5671, 454, 14, 27, "Input"],
Cell[177833, 5687, 216, 4, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
